package com.despark.tuning3d.model.cars
{
	import com.despark.tuning3d.model.challenges.ChallengeData;
	import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.images.PersistentImageProvider;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.model.parts.Material3dtProvider;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartDisk;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImage;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.parts.PartSlotDisk;
import com.despark.tuning3d.model.users.UsersProxy;
	import com.utils.Console;

	import flash.utils.Dictionary;

import org.as3commons.lang.ArrayUtils;
import org.puremvc.as3.patterns.facade.Facade;

	import starling.utils.StringUtil;

	public class Car implements PersistentImageProvider
{
		public static const HD_TYPES:Array = ["hd", "hd2", "hd 2", "moto", "moto hd", "truck", "pickup hd", "american_muscle", "all"];

		public static var FRONT_DISKS_SLOTS:Array=["front_wheel", "disks_front", "disk_front_right", "disk_front_left"];
		public static var REAR_DISKS_SLOTS:Array=["rear_wheel", "disks_rear", "disk_rear_left", "disk_rear_right"];
		public static var DISKS_SLOTS:Array=["disks"].concat(FRONT_DISKS_SLOTS, REAR_DISKS_SLOTS);
		public static var WINDOW_SLOTS: Array = ["windows", "left_windows", "right_windows", "rear_windshield", "front_windshield"];
		public static var CAR_DISKS_SLOTS:Array=[ "disks", "disks_rear", "disks_front","disk_front_right","disk_front_left","disk_rear_left","disk_rear_right"];
		public static var CHOPPER_DISKS_SLOTS:Array=[ "rear_wheel", "front_wheel"];
		public static var LIGHT_SLOTS:Array=[ "rear_lights", "front_lights"];
		public static var TIRES_SLOTS:Array=[ "tire", "tires", "tires_front", "tires_rear","tire_front", "tire_rear","tire_rear_right", "tire_rear_left","tire_front_right", "tire_front_left" ];
		public static var TIRE_VINYL_PARTS:Array=["tire_vinyl_front_left","tire_vinyl_front_right","tire_vinyl_rear_left","tire_vinyl_rear_right"];
		public static var BRAKES_SLOTS:Array=[ "brake","brakes", "brake_rear", "brake_front", "brake_rear_right", "brake_rear_left","brake_front_right", "brake_front_left" ];
		public static var DISKS_SHADOW_SLOTS: Array = [ 'shadow_front_left','shadow_front_right','shadow_rear_left','shadow_rear_right'];
		public static var INTERIOR_SLOTS: Array = ["seats", "interior", "interiors", "seat"];
		public static var MISC_SLOTS: Array = ["mask", "shadow", "black"];
		public static var EXTERIOR_EXCLUDE: Array = DISKS_SLOTS.concat(TIRES_SLOTS, BRAKES_SLOTS, TIRE_VINYL_PARTS, CHOPPER_DISKS_SLOTS, DISKS_SHADOW_SLOTS, INTERIOR_SLOTS, MISC_SLOTS, WINDOW_SLOTS, LIGHT_SLOTS);
		public static var WHEEL_PARTS: Array = DISKS_SLOTS.concat(TIRES_SLOTS, BRAKES_SLOTS, TIRE_VINYL_PARTS, CHOPPER_DISKS_SLOTS, DISKS_SHADOW_SLOTS);
		public static var updateTextureBlocker:Boolean=false;
		public static var BlankCar:Car=null;
		public static var statesMapInvert:Object = {
			'cm1': 'cm1-neon',
			'cm1-of1': 'cm1-neon-of1',
			'cm1-of2': 'cm1-neon-of2',
			'cm1-trr': 'cm1-neon-trr',
			'cm1-trl': 'cm1-neon-trl',
			'cm1-of1-trr': 'cm1-neon-of1-trr',
			'cm1-of2-trr': 'cm1-neon-of2-trr',
			'cm1-of1-trl': 'cm1-neon-of1-trl',
			'cm1-of2-trl': 'cm1-neon-of2-trl',
			
			'cm2': 'cm2-neon',
			'cm2-of1': 'cm2-neon-of1',
			'cm2-of2': 'cm2-neon-of2',
			'cm2-trr': 'cm2-neon-trr',
			'cm2-trl': 'cm2-neon-trl',
			'cm2-of1-trr': 'cm2-neon-of1-trr',
			'cm2-of2-trr': 'cm2-neon-of2-trr',
			'cm2-of1-trl': 'cm2-neon-of1-trl',
			'cm2-of2-trl': 'cm2-neon-of2-trl',
			
			'trr': 'neon-trr',
			'trl': 'neon-trrl',
			'of1': 'neon-of1',
			'of2': 'neon-of2',
			'of1-trr': 'neon-of1-trr',
			'of2-trr': 'neon-of2-trr',
			'of1-trl': 'neon-of1-trl',
			'of2-trl': 'neon-of2-trl',
			
			'neon': 'neon'
		};
		public static var statesMap:Object = {
			'cm1-neon': 'cm1',
			'cm1-neon-of1': 'cm1-of1',
			'cm1-neon-of2': 'cm1-of2',
			'cm1-neon-trr': 'cm1-trr',
			'cm1-neon-trl': 'cm1-trl',
			'cm1-neon-of1-trr': 'cm1-of1-trr',
			'cm1-neon-of2-trr': 'cm1-of2-trr',
			'cm1-neon-of1-trl': 'cm1-of1-trl',
			'cm1-neon-of2-trl': 'cm1-of2-trl',
			
			'cm2-neon': 'cm2',
			'cm2-neon-of1': 'cm2-of1',
			'cm2-neon-of2': 'cm2-of2',
			'cm2-neon-trr': 'cm2-trr',
			'cm2-neon-trl': 'cm2-trl',
			'cm2-neon-of1-trr': 'cm2-of1-trr',
			'cm2-neon-of2-trr': 'cm2-of2-trr',
			'cm2-neon-of1-trl': 'cm2-of1-trl',
			'cm2-neon-of2-trl': 'cm2-of2-trl',
			
			'neon-trr': 'trr',
			'neon-trrl': 'trl',
			'neon-of1': 'of1',
			'neon-of2': 'of2',
			'neon-of1-trr': 'of1-trr',
			'neon-of2-trr': 'of2-trr',
			'neon-of1-trl': 'of1-trl',
			'neon-of2-trl': 'of2-trl',
			
			'neon': 'neon'
		};
		
		
		public static var statesMapForTairsAndBrake:Object = {
			'cm1-neon': 'cm1',
			'cm1-neon-of1': 'cm1-of1',
			'cm1-neon-of2': 'cm1-of2',
			'cm1-neon-trr': 'cm1-trr',
			'cm1-neon-trl': 'cm1-trl',
			'cm1-neon-of1-trr': 'cm1-of1-trr',
			'cm1-neon-of2-trr': 'cm1-of2-trr',
			'cm1-neon-of1-trl': 'cm1-of1-trl',
			'cm1-neon-of2-trl': 'cm1-of2-trl',
			
			'cm2-neon': 'cm2',
			'cm2-neon-of1': 'cm2-of1',
			'cm2-neon-of2': 'cm2-of2',
			'cm2-neon-trr': 'cm2-trr',
			'cm2-neon-trl': 'cm2-trl',
			'cm2-neon-of1-trr': 'cm2-of1-trr',
			'cm2-neon-of2-trr': 'cm2-of2-trr',
			'cm2-neon-of1-trl': 'cm2-of1-trl',
			'cm2-neon-of2-trl': 'cm2-of2-trl',
			
			'neon-trr': 'trr',
			'neon-trrl': 'trl',
			'neon-of1': 'of1',
			'neon-of2': 'of2',
			'neon-of1-trr': 'of1-trr',
			'neon-of2-trr': 'of2-trr',
			'neon-of1-trl': 'of1-trl',
			'neon-of2-trl': 'of2-trl',
			
			'neon': ''
		};
		
		public static var SlotNamesForStatesMap:Array = ["brake_front", "brake_rear", "tires_front", "tires_rear"]; //TODO sync
		
		public var haveIconFolder:Boolean=false;
		public var suvTires : Boolean = false;
		public var tireMasks : Boolean = false;
		public var carType : String="";

		public var stepOfFrameIndex : int = 2;
		public var stepOfFrameIndexUp : int = 10;
		
		public var countFrames : int = 50;//FRAMES : int = 25;//50;
		
		public var camera24Frames:Boolean=false;
		public var camera12Frames:Boolean=false;
		public var countAngels:int=1;
		public var stepFrames:int=2;
		
		
		public var backgrounds:Array = [];
		
		public var haveSound: Boolean = false;
		public var hasSoundRevs: Boolean = false;

		public var partMaterialZip:Boolean = false;
		public var partMaterialEncrypted:Boolean = true;
		
		public var bgIndex:int=0;
		
		//public static var stepOfFrameIndexUp_global: int = 10
		//public static var stepOfFrameIndex_global : int = 2;
		public static var weakDevice : Boolean = false;

		public var defaultBgId:String = null;
		
		public var ultraIcon:Boolean=false;
		public var topCameraIcon:Boolean=false;
		public var offsetIcon:Boolean=false;
		public var stanceIcon:Boolean=false;
		public var car3d:Boolean=false;
		public var carAngelIndexDefault:int=0;
		public var isBigSize:Boolean=false;
		public var fullsize:Boolean = false;
		
		public var defaultBkgMobile:Array=null;
		
		public var defaultBkg:Array=null;
		public var hashTags:String="";
	
		
		public var id : String;
		public var isEditable : Boolean = true;
		public var isChallenge : Boolean = false;
		public var challengeData:ChallengeData;
		public var imageURL : String;
		public var brandImageUrl:String;
		public var name : String;
		public var model : String;
		public var year : int;
		public var startYear : int;
		public var endYear : int;
		public var frontBumperX : int;
		public var type:String;
		public var bodyTypeList:Array;
		public var updated:Number=0;
		public var defaultZoom:Number = 0;
		public var tintDisabled:Boolean = false;
		public var updateText : String;
		public var custom_offset: Object;
		//public var frontBumperY : int;
		public var frontBumperWidth : int;
		public var frontBumperHeight : int;
		public var suspension : int;
		public var defaultSuspension:int;
		public var maxSuspension : int;
		public var minSuspension : int;
		public var isNewMask : Boolean;
		public var carchooserOrder:int;
		public var hdsize: Boolean;
		public var path : String;
		public var selectedBackground : CarBackground;
		public var selectedFillColor: Object;
		public var isFeatured : Boolean;
		public var maxPartSlotLayerIndex : int;
		public var displayName : String;
		public var featuredImageUrl : String;
		public var bodyPartColor : PartColor;
		//public var assetFile : File;
		public var moreCarsURL : String;
		public var isShaving : Boolean = false;
		public var carFeaturesNew:Boolean=false;
		public var carFeaturesFull:Boolean=false;
		public var useZipSplitCam: Boolean;

		public var startFrame:int=0;
		//public var frames:String="";
		
		public var size:Object=null;
		public var limitedOptions : Boolean = false;

		public var feathersCarOrder: int = 0;
		
		//public var price:Number=0;
		//public var discountprice:Number=-1;
		//public var discounttime:Date=null;
		public var body:String=null;
		//public var lock:Boolean=false;
		// public var show:Boolean=false;
		private var _packet:Packet=null;
		
		private var _packages:Array=null;
		
		
		private var _possiblePartSlots : Array;
		private var possiblePartSlotsMap : Dictionary;
		private var possiblePartsMap : Dictionary;
		private var _selectedPartSlots : Array;
		private var _editablePartSlots : Array;
		private var _possiblePartColors : Array;
		private var possiblePartColorsMap : Object;
		private var _possiblePartSlotsSorted : Array;

		public var partSlotsForDownload : Array;
		
		
		private var _masks:Object;
		
		private var _isDefault : Boolean;
		
		public var camera1Frames:Object=null;
		public var camera2Frames:Object=null;
		//public var camera2Full:Boolean = false;
		public var camera1LayerPositionStep:int = 2;
		public var camera2LayerPositionStep:int = 12;
		//public var camera2FramesStep:int=10;
		
		
		public var purchase_car_image:String=null;
		

		public var limiterMaxSuspension:int=0;
		public var limiterMinSuspension:int=0;

		public var disks:Array=[];
		
		public var disksPosition:Object={};
		
		public var componentsDependencies:Array=[];
		
		public var componentsDependenciesWheel:Array=[];
		
		public var frontOffset:Number=0;
		
		public var layersPositions:Array=[];
		
		public var pizdec:Boolean=false;

		public var setWide:String = "2";

		public var isChopper:Boolean=false;
		public var isBeetle: Boolean = false;

		public var carChooserBgName:String = null;

    	private var addListener:Boolean=false;

		public var suspensionForState : Object;
		public var suspensionForComponents : Object;
        public var suspensionComponentsArray : Array;



    	private var _possiblePartSlotsDisks:Array;
    	private var _possiblePartSlotsBrakes:Array;
    	private var _possiblePartSlotsTires:Array;
    	private var _possiblePartSlotsVinyls:Array;

		private var _imagePath : String;


		public var wheelConfigurator: Boolean;
		public var offsetDifference: Boolean;

		public function dispose():void{

			suspensionForState = null;
			suspensionForComponents = null;

			id=null;
			
			size = null;
			
			disksPosition = null;
			disks = null;
			defaultBkg = null;
			
			backgrounds = null;

            imageURL=null;
			brandImageUrl=null;
			
			name=null;
			
			model=null;
			
			path=null;
			
			selectedBackground=null;
			
			bodyPartColor=null;
			
			body = null;
			
			moreCarsURL=null;
			
			_packet=null;
			
			_packages=null;

            defaultBgId = null;
            defaultBkgMobile=null;
            defaultBkg=null;
			hashTags=null;

			if(_possiblePartSlots){
				var partSlot:PartSlot;
				for(var i:int=0; i<_possiblePartSlots.length; i++){
					
					partSlot=_possiblePartSlots[i];
					
					if(partSlot){
						
						partSlot.dispose();
						
						partSlot=null;
					}
				}
			}
			
			

			if(_possiblePartSlots){
				_possiblePartSlots.length=0;
			}

			possiblePartsMap = null;
			
			_possiblePartSlots=null;
			
			possiblePartSlotsMap=null;
			
			_selectedPartSlots=null;
			
			_editablePartSlots=null;
			
			_possiblePartColors=null;
			
			possiblePartColorsMap=null;
			
			_possiblePartSlotsSorted=null;
			
			partSlotsForDownload=null;
			
			_masks=null;
			
			purchase_car_image=null;
			
			if(componentsDependencies){
				
				componentsDependencies.length=0;
			}

            componentsDependencies = null;
			
			layersPositions = null;
			
			componentsDependenciesWheel = null;
			
			camera1Frames = null;
			camera2Frames = null;
			//_possiblePartSlotsSorted = null;
			

		}

		public static function formatBodyType(str:String):Array{
			if(!str){
				return null;
			}
			var typeList:Array = str.split(',');
			var result:Array = [];
			for(var k:int = 0; k < typeList.length; k++){
				var bodyType:String = StringUtil.trim(typeList[k]).toUpperCase();
				if(result.indexOf(bodyType) == -1){
					result.push(bodyType);
				}
			}
			return result;
		}
		
		public function getSiteLinkTuning():String{
			
			var makeStr:String = name.toLowerCase().replace(/\s/g, '.');
			var modelStr:String = model.toLowerCase().replace(/\s/g, '.');
			var typeStr:String = type.toLowerCase().replace(/\s/g, '.');
			var yearStr:String = (year+"").toLowerCase().replace(/\s/g, '.');
			
			return "https://www.3dtuning.com/en-US/tuning/"+makeStr+"/"+modelStr+"/"+typeStr+"."+yearStr;
		}
		
		
		public function getSiteLinkGrage(usernameStr:String, shortId:String):String{
			
			var makeStr:String = name.toLowerCase().replace(/\s/g, '.');
			var modelStr:String = model.toLowerCase().replace(/\s/g, '.');
			var typeStr:String = type.toLowerCase().replace(/\s/g, '.');
			var yearStr:String = (year+"").toLowerCase().replace(/\s/g, '.');
			return "https://www.3dtuning.com/en-US/"+usernameStr+"/"+makeStr+"/"+modelStr+"/"+typeStr+"."+yearStr+"/"+shortId;
		}

		public function getEditorLink(userName:String, shortId:String, index: Number, hdQuality:Boolean) : String {
			var token : String = (Facade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).getUserToken();
			var editorType:String = hdQuality?"editorFullHd": "editor";
			var str : String = getSiteLinkGrage(userName, shortId) + "/" + editorType + "/?index=" + index + '&token='+ token + '&platform=app';
			//str = str.replace("https://www.3dtuning.com/en-US/", "https://ad-debugger.3dtuning.com/en-US/"); // TODO REMOVE
			Console.log(str);
			return str;
		}
		
		public function getDependencyPartIds(dependencySlotName:String):Array{
			
			var result:Array=[];
			for(var i:int=0; i<componentsDependencies.length; i++){
				
				var dependency:Object=componentsDependencies[i];
				
				for (var slotName:String in dependency){
					
					for each (var inSlotName:String in dependency[slotName]){
						
						if(inSlotName==dependencySlotName){
							
							var part:Part=getPossiblePartSlot(slotName).selectedPart;
							
							if(part){
								result.push(part.id);
							}
						}
					}
					
				}
				
			}
				
			return result;
		}
		
		public function get packets():Array{
			if(isChallenge){
				return null;
			}
			return _packages;
		}
		
		public function get packet():Packet{
			//Нужно вернуть пакет если тип компонента component и он не куплен то возвращаем его
			if(isChallenge){
				return null;
			}
			if(_packages){
				var i:int=0;
				var p:Packet;
				//ищем купленный пакет с типом component. Если есть возвращаем его
				for( i=0; i<_packages.length; i++){
					p=_packages[i];
					if(p.type==Packet.TYPE_COMPONENTS && !p.lock){
						return p;
					}
				}
				
				//если нет купленых пакет с типом компонент то возвращаем заблокированный
				for( i=0; i<_packages.length; i++){
					p=_packages[i];
					if(p.type==Packet.TYPE_COMPONENTS && p.lock){    
						return p;    
					}
				}
			}

			
			if(_packages){
                //возвращаем купленный пакет с типом subcomponent
                for(i=0; i<_packages.length; i++){
                    p=_packages[i];
                    if(p.type==Packet.TYPE_SUBCOMPONENT &&  !p.lock){
                        return p;
                    }
                }

				//возвращаем не купленный пакет с типом subcomponent
				for(i=0; i<_packages.length; i++){
					p=_packages[i];
					if(p.type==Packet.TYPE_SUBCOMPONENT &&  p.lock){
						return p;
					}
				}
				

				
				
				
			}
			//}
			
			if(_packet){
				
				return _packet;
			}else{
				
				return null;
			}
			
		}

    public function addPacket(value:Packet):void{
			
			if(!_packages)
				_packages=[];
			
			_packages.push(value);
			
			
			if(_packet==null){
				_packet=value;
			}else{
				
				if(_packet.fee<value.fee){
					
					_packet=value;
				}
			}
			
			
		}
		//public var _isExistsMask
		
		public function set masks(value:Object):void{
			
			_masks=value;
		}
		
		
		public function get masks():Object{
			
			return _masks;
		}
		
		public function get frameIndeX() : int 
		{
			return -1;
		}
		
		public function get isDefault() : Boolean 
		{
			return _isDefault;
		}
		
		public function set isDefault(value : Boolean) : void
		{
			_isDefault = value;
		}
		
		public function Car() 
		{
			super();
			
			_possiblePartSlots = [];
			possiblePartSlotsMap = new Dictionary(true);
			_selectedPartSlots = [];
			_editablePartSlots = [];
			_possiblePartColors = [];
			_possiblePartSlotsSorted = [];
			possiblePartColorsMap = {};
			
			
		}

		public function getPossibleStateOfPartSlots(partSlotsArray : Array) : String
		{
			for each(var partSlot : PartSlot in partSlotsArray)
			{
				if((partSlot.name.indexOf("tires") >= 0 || partSlot.name.indexOf("disks")) >= 0 )
				{
					if( partSlot.selectedPart.selectedState)
					{
                    	return partSlot.selectedPart.selectedState.nameOfState;
					}
					else {
						return "default";
					}
				}
			}
			return null;
		}

		public function getPossibleStateOfPartSlots2(partSlotsArray : Array) : String {
			for each(var partSlot : PartSlot in partSlotsArray) {
				if((partSlot.name.indexOf("tire") >= 0 || partSlot.name.indexOf("disk")) >= 0 ) {
					if( partSlot.selectedPart.selectedState) {
                    	return partSlot.selectedPart.selectedState.nameOfState;
					} else {
						return "default";
					}
				}
			}
			return null;
		}


		/**
		 * Checks if the given array of parts ( ids ) are selected
		 * @param partIds the array to check for
		 * @return array of partslots if is all selected, else return null
		 */
		public function isListOfPartsSelected(partIds : Array) : Array {
			var arr : Array = [];
			var ln:int = partIds.length;
			var suspensionMap:Object = {};
			for(var i:int = 0; i < ln; i++){
				var part:Part = findPart(partIds[i]);
				var parentSlotName:String = null;
				if(!part){
					//Console.log("Part not found", partIds[i]);
					//return null;
					continue;
				}
				if(part.parentSlotName){
					parentSlotName = part.parentSlotName
				}else if(part.parentSlot && part.parentSlot.name){
					parentSlotName = part.parentSlot.name;
				}
				if(parentSlotName !== null){
					suspensionMap[parentSlotName] ||= [];
					if(suspensionMap[parentSlotName].indexOf(part.id) == -1){
						suspensionMap[parentSlotName].push(part.id);
					}
				}
			}

			for(var partType:String in suspensionMap){
				var partSlot:PartSlot = getPossiblePartSlot(partType);
				if(!partSlot){
					continue;
				}
				if(!partSlot.selectedPart || suspensionMap[partType].indexOf(partSlot.selectedPart.id) == -1){
					return null;
				}
				arr.push(partSlot);
			}

			return arr;
		}

		/**
		 * Checks if the given array of parts ( ids ) are selected
		 * @param partIds the array to check for
		 * @return array of partslots if is all selected, else return null
		 */
		public function isListOfPartsSelected_old(partIds : Array) : Array
		{
			var arr : Array = [];
			for each (var id : String in partIds)
			{
				var partSlot : PartSlot = isPartSelected(id);
				if(!partSlot)
				{
					return null; // if at least one of the parts is missing, return false/null
				}
				arr.push(partSlot)

			}

			return arr; // if there is no part that is missing than return true/listofpartslots
		}


		/**
		 * The function checks if the given part ( partID ) is selected on car
		 * @param partId - the part's/component's id to check, if it is selected
		 * @return the selected partslot if selected , null if not
		 */
		public function isPartSelected ( partId : String) : PartSlot
		{
			for each ( var partSlot : PartSlot in _possiblePartSlots)
			{
				if(!partSlot.selectedPart) continue;

				var id : String;
				if(partSlot is PartSlotDisk)
				{
					id = (partSlot.selectedPart as PartDisk).realId;
				}
				else {
					id = partSlot.selectedPart.id;
				}

				if(id == partId)
				{
					return partSlot;
				}

			}

			return null;
		}

		protected function onGlobalEventMemLeakTest(event:*):void{
			
		}

        public function recreateMasks(part : Part, __angels : Array):void {
            var allPartStates:Object = part.states;
            var maskPartSlot:PartSlot = getPossiblePartSlot("masks");
            var newParts : Array = [];
			
			if(!maskPartSlot){
				//maskPartSlot.possibleParts = [];
				maskPartSlot = new PartSlot(this);
				maskPartSlot.parentCar = this;
				
				maskPartSlot.layerIndex = 200;
				maskPartSlot.name = 'masks';
				//usedPartSlotsMap['masks'] = maskPartSlot;
				maskPartSlot.isEditable = false;
				
				maskPartSlot.selectedColor = PartColor.noColor();
				this.addPossiblePartSlot(maskPartSlot);
			}
			
            for each(var p : Part in maskPartSlot.possibleParts)
            {
                if(p.nameOfState =="") newParts.push(p);

            }
            maskPartSlot.possibleParts = newParts;
            for(var stateName:String in allPartStates as Object){
                var newMaskPart:Part = new Part(maskPartSlot);
                newMaskPart.id = this.id;
                newMaskPart.folder = part.folder + '/states/' + stateName + '/' + part.selectedMaterial.name + '/';
                newMaskPart.name = stateName;
                newMaskPart.nameOfState = stateName;
                
				Tracer.print("part.affectedSlots:" + part.affectedSlots, "Car:RecreateMask");
				
				newMaskPart.maskForSlots = ArrayUtils.clone(part.affectedSlots);
                maskPartSlot.addPossiblePart(newMaskPart);
                for(var filesKey : String in allPartStates[stateName].possibleMaterialsMap)
                {
                    if (filesKey == 'mask') {
                        var partMaterial:PartMaterial = new PartMaterial(newMaskPart);
                        partMaterial.name = filesKey;
                        partMaterial.isPaintable = filesKey != "metal";
                        var imageURLPrefix:String = part.parentSlot.name+'/'+ part.folder + '/states/' + stateName +  '/';
                        imageURLPrefix += partMaterial.name;
                        imageURLPrefix += "/";
                        partMaterial.componentPath = imageURLPrefix;
						
						//newMaskPart.imagesUrlBytes.writeUTF(partMaterial.name);
						var oldPartMaterial:PartMaterial = allPartStates[stateName].possibleMaterialsMap[filesKey];

						for each (var cameraIndex:int in __angels){
							for(var frame:int=0; frame<this.countFrames; frame++){
								var imageURL:PartMaterialImage = oldPartMaterial.getImage(frame,cameraIndex);
								if(imageURL){
									//http://cdn1.3dtuning.com/cars/Tesla%20Model%20S%2012-12%20HD%205-door%20liftback%202017%20decripted/disks_front/default R21/states/cm2-of2-trr/mask/a0004_624_434.png
									var url:String = imageURL.url;
									var mas:Array = url.split("/");
									
									if(mas && mas.length>0) url = mas[mas.length-1];
									
									partMaterial.addImage(url, this.stepOfFrameIndex);
									
								}
								
							}
							
						}
						
						/*for each (var camera : Array in allPartStates[stateName].possibleMaterialsMap[filesKey].imagesMap)
                        {
                            for each(var imageURL:PartMaterialImage in camera) {
                                partMaterial.addImage(imageURL.url, this.stepOfFrameIndex);
                            }
                        }*/

                        newMaskPart.addPossibleMaterial(partMaterial);

                        if (!newMaskPart.selectedMaterial)
                            newMaskPart.selectedMaterial = partMaterial;
                    }
                }

            }

        }

	private function arrayInArray(in1:Array, subject:Array):Boolean {
		var index:int = 0;
		while(subject.indexOf(in1[index])>-1 && index<in1.length){
			index++;
		}
		return index>=in1.length;
	}

	public function getDependentPart(part:Part):Part{
		if(!part || !part.dependence || part.dependence.length == 0){
			return null;
		}
		var result:Part = null;
		var parentSlot:PartSlot = part.parentSlot;
		mainLoop: for(var i:int = 0; i < possiblePartSlots.length; i++){
			var ps:PartSlot = possiblePartSlots[i];
			if(ps.selectedPart) {
                for (var j:int = 0; j < parentSlot.possibleParts.length; j++) {
                    var pt:Part = parentSlot.possibleParts[j];
                    if (pt.dependence.indexOf(ps.selectedPart.id) != -1 && pt.name == part.name/* && pt.id != part.id*/) {
                        result = pt;
						break mainLoop;
                    }
                }
            }
		}
		return result;
	}

    public function getDependence(findPart:Part/*findDependendId:String*/):Array {
        //const allPossibleComponents:Array = car.selectedPartSlots;
        var arrOfPartSlotForUpdate:Array = [];
        var cmpType:PartSlot;
        var cmpTypeName:String;
        var cmp:Part;
        var newDepSlot:PartSlot;
        var newDepSlotPart:Part;
        var possibleParts:Array;
        var partMaterial:PartMaterial;

        var allPossibleComponents:Array = findPart.depsMap.length > 0 ? findPart.depsMap : [];

        var isRadiusChange:Boolean = false;
        var currentRadius:String = null;

        var rearWheelPartSlot:PartSlotDisk = getPossiblePartSlot("rear_wheel") as PartSlotDisk;
        var materialName:String = null;

        if (rearWheelPartSlot && rearWheelPartSlot.selectedPart) {
            isRadiusChange = true;
            currentRadius = (rearWheelPartSlot.selectedPart as PartDisk).selectedDiskSize + "";

        }

		var allPossiblePartsId: Array = [];

		for each (var carComponentType: PartSlot in possiblePartSlots) {
			if (carComponentType.selectedPart) {
				allPossiblePartsId.push(carComponentType.selectedPart.realId);
			}
		}

        for each(cmpTypeName in allPossibleComponents) {
            cmpType = getPossiblePartSlot(cmpTypeName);
            if (cmpType.isDependence && cmpType.selectedPart != null) {
                possibleParts = cmpType.possibleParts;//tuningScreen.wideFilter(cmpType.possibleParts, cmpType);
                for each(cmp in possibleParts) {
                    if (cmp.isDependence && arrayInArray(cmp.dependence, allPossiblePartsId) && cmp.parentSlot.name != "rear_wheel") {
                        newDepSlot = getPossiblePartSlot(cmp.parentSlot.name);
                        if ((!cmp.wide && newDepSlot.selectedPart.name == cmp.name) || (cmp.wide && currentRadius && cmp.wide == currentRadius && newDepSlot.selectedPart.name == cmp.name)) {
							newDepSlotPart = newDepSlot.getPartById(cmp.id);
							if (newDepSlot.selectedPart != newDepSlotPart) {
								materialName = newDepSlot.selectedPart.selectedMaterial ? newDepSlot.selectedPart.selectedMaterial.name : null;
								newDepSlot.selectedPart = newDepSlotPart;
								partMaterial = newDepSlot.selectedPart.getPossibleMaterial(materialName);
								if (partMaterial) newDepSlot.selectedPart.selectedMaterial = partMaterial;
								arrOfPartSlotForUpdate.push(newDepSlot);
								arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(getDependence(newDepSlotPart));
							}
							break;
                        }
                    }else {
						if (isRadiusChange && cmp.wide && currentRadius && cmp.parentSlot.name != "rear_wheel") {
							newDepSlot = getPossiblePartSlot(cmp.parentSlot.name);
							if (newDepSlot.selectedPart.name == cmp.name && cmp.wide == currentRadius && cmp.dependence && inDeps(cmp.dependence)) {
								newDepSlotPart = newDepSlot.getPartById(cmp.realId);
								if (newDepSlot.selectedPart != newDepSlotPart) {
									//newDepSlot.selectedPart = newDepSlotPart;
									materialName = newDepSlot.selectedPart.selectedMaterial ? newDepSlot.selectedPart.selectedMaterial.name : null;
									newDepSlot.selectedPart = newDepSlotPart;
									partMaterial = newDepSlot.selectedPart.getPossibleMaterial(materialName);
									if (partMaterial) newDepSlot.selectedPart.selectedMaterial = partMaterial;
									arrOfPartSlotForUpdate.push(newDepSlot);
									arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(getDependence(newDepSlotPart));
								}
								break;
							}
						} else {
							if (isRadiusChange && cmp.wide && currentRadius && cmp.parentSlot.name == "rear_wheel") {
								newDepSlot = getPossiblePartSlot(cmp.parentSlot.name);
								(cmp as PartDisk).selectedDiskSize = parseInt(currentRadius);
								if (newDepSlot.selectedPart.name == cmp.name && cmp.dependence && inDeps(cmp.dependence)) {
									newDepSlotPart = newDepSlot.getPartById(cmp.id);
									if (newDepSlot.selectedPart != newDepSlotPart) {
										//newDepSlot.selectedPart = newDepSlotPart;
										materialName = newDepSlot.selectedPart.selectedMaterial ? newDepSlot.selectedPart.selectedMaterial.name : null;
										newDepSlot.selectedPart = newDepSlotPart;
										(newDepSlot.selectedPart as PartDisk).selectedDiskSize = parseInt(currentRadius);
										partMaterial = newDepSlot.selectedPart.getPossibleMaterial(materialName);
										if (partMaterial) newDepSlot.selectedPart.selectedMaterial = partMaterial;
										arrOfPartSlotForUpdate.push(newDepSlot);
										arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(getDependence(newDepSlotPart));
									}
									break;
								}
							}
						}
					}
                }
            }
        }
        return arrOfPartSlotForUpdate;
    }

    private function inDeps(dep:*):Boolean {

        const allPossibleComponents:Array = possiblePartSlots;

        var cmpType:PartSlot;
        var deps:Array;

        if (dep as String) {
            deps = [dep];
        } else {
            deps = dep;
        }

        if (!deps) return false;

        if (deps.length <= 0) return false;

        for each(cmpType in allPossibleComponents) {

            if (cmpType && cmpType.selectedPart) {

                if (deps.indexOf(cmpType.selectedPart.realId) > -1) return true;
            }

        }


        return false;
    }

        public function addWhiteColor() : void
		{
            var whitePartColor : PartColor = new PartColor();
            whitePartColor.value = 0xfeffff;
            whitePartColor.name = "white";
            _possiblePartColors.insertAt(2, whitePartColor);
            possiblePartColorsMap[whitePartColor.value] = whitePartColor;
		}
		
		public function addPossiblePartColor(partColor : PartColor) : void
		{
			_possiblePartColors.push(partColor);
			possiblePartColorsMap[partColor.value] = partColor;

           partColor = null;
		}
		
		public function get possiblePartColors() : Array
		{
			var newArr : Array = [];
			for each (var partColor : PartColor in _possiblePartColors){
				newArr.push(partColor.copy());
			}
			return newArr;
		}
		
		public function getPossiblePartColor(value : uint) : PartColor  
		{
			var partColor : PartColor = possiblePartColorsMap[value];
			if(partColor){
				partColor = partColor.copy();
			}
			return partColor;
		}
		

		public function addPossiblePartSlot(partSlot : PartSlot) : void {
			if(possiblePartSlotsMap[partSlot.name]){
				trace("ERROR: Part slot with this name already exists!!!!!!!!!!!!!!!!!!", partSlot.name);
				return;
			}
			if(!addListener){
				//Tuning3D.memLeakTest.addEventListener(MemLeakTest.GLOBAL_EVENT, onGlobalEventMemLeakTest,false, 0, true);
				addListener=true;
			}


            possiblePartSlotsMap[partSlot.name] = partSlot;
			//_possiblePartSlots[partSlot.layerIndex] = partSlot;
            _possiblePartSlots.push(partSlot);


            if(partSlot.isEditable)
            {
                editablePartSlots.push(partSlot);
            }

            _possiblePartSlotsSorted.push(partSlot);

			possiblePartsMap ||= new Dictionary();
			for(var i:int = 0; i < partSlot.possibleParts.length; i++){
				addPossiblePart(partSlot.possibleParts[i]);
			}

            partSlot = null;
		}

		public function addPossiblePart(part:Part):void{
			possiblePartsMap ||= new Dictionary();
			possiblePartsMap[part.id] = part;
		}
		
		public function test() : void
		{
			
		}
		
		public function getPossiblePartSlot(name : String) : PartSlot
		{
			if(!possiblePartSlotsMap) return null;
			return possiblePartSlotsMap[name];
		}

		public function getLightsOnComponent():PartSlot{
			for each(var partSlot:PartSlot in selectedPartSlots){
				if(partSlot && partSlot.selectedPart && partSlot.selectedPart.states && partSlot.selectedPart.states.hasOwnProperty('lights')){
					return partSlot;
				}
			}
			return null;
		}

		public function get possiblePartSlotsTires() : Array {
			if(!_possiblePartSlotsTires) {
                _possiblePartSlotsTires = [];
				for(var i : int = 0; i < _possiblePartSlots.length; i++) {
					var ps : PartSlot = _possiblePartSlots[i];
					if (Car.TIRES_SLOTS.indexOf(ps.name) >= 0) {
                        _possiblePartSlotsTires.push(ps);
					}
				}
			}
			return _possiblePartSlotsTires;
		}

		public function get possiblePartSlotsVinyls() : Array {
			if(!_possiblePartSlotsVinyls) {
                _possiblePartSlotsVinyls = [];
				for(var i : int = 0; i < _possiblePartSlots.length; i++) {
					var ps : PartSlot = _possiblePartSlots[i];
					if (Car.TIRE_VINYL_PARTS.indexOf(ps.name) >= 0) {
                        _possiblePartSlotsVinyls.push(ps);
					}
				}
			}
			return _possiblePartSlotsVinyls;
		}


    	public function get possiblePartSlotsDisks() : Array
		{
			if(!_possiblePartSlotsDisks)
			{
				_possiblePartSlotsDisks = [];
				for(var i : int = 0; i < _possiblePartSlots.length; i++)
				{
					var ps : PartSlot = _possiblePartSlots[i];
					if (Car.DISKS_SLOTS.indexOf(ps.name) >= 0)
					{
						_possiblePartSlotsDisks.push(ps);
					}
				}
			}

			return _possiblePartSlotsDisks;

		}

		public function get possiblePartSlotsBrakes() : Array
		{
			if(!_possiblePartSlotsBrakes)
			{
                _possiblePartSlotsBrakes = [];
				for(var i : int = 0; i < _possiblePartSlots.length; i++)
				{
					var ps : PartSlot = _possiblePartSlots[i];
					if (Car.BRAKES_SLOTS.indexOf(ps.name) >= 0)
					{
                        _possiblePartSlotsBrakes.push(ps);
					}
				}
			}

			return _possiblePartSlotsBrakes;

		}

		public function get possiblePartSlots() : Array
		{
			return _possiblePartSlots;
		}
		
		public function get possiblePartSlotsSorted() : Array 
		{
			return _possiblePartSlotsSorted;
		}
		
		public function get editablePartSlots() : Array
		{
			return _editablePartSlots;
		}
		
		public function addSelectedPartSlot(partSlot : PartSlot) : void {
			_selectedPartSlots[partSlot.layerIndex] = partSlot;
		}
		
		public function get selectedPartSlots() : Array 
		{
			return _selectedPartSlots;
		}
		
		public function arrangePartSlots() : void
		{
			
			var partSlotsOrder : Array = [
				"Wheel",
				"Front bumper",
				"Rear bumper",
				"Sides",
				"Front lights",
				"Rear lights",
				"Top spoiler",
				"Spoiler",
				"Hood",
				"Wings",
				"Mirrors",
				"Radiator",
				"Top air intake",
				"Front air intake",				
				"Packs",
				"Vinilis",
				"Neon",
				"Fox eyes",
				"Pipe",
				"Protector",
				"Wings air intake",
				"Tinted glass",
				"Body",
			];
			
			//			var partSlotsOrder : Array = [
			//				"Body",+
			//				"Wheel",+
			//				"Front bumper",+
			//				"Rear bumper",+
			//				"Sides",+
			//				"Front lights",+
			//				"Rear lights",+
			//				"Fox eyes",+
			//				"Top spoiler",+
			//				"Spoiler",+
			//				"Hood",+
			//				"Wings",+
			//				"Mirrors",+
			//				"Radiator",+
			//				"Pipe",+
			//				"Protector",+
			//				"Top air intake",+
			//				"Front air intake",+
			//				"Wings air intake",
			//				"Tinted glass",
			//				"Packs",
			//				"Vinilis",+
			//				"Neon"
			//			];
			
			var displayNameMap : Object = {};
			var partSlot : PartSlot;
			

			for each(partSlot in possiblePartSlots)
			{

				if (partSlot.parentCar.isBeetle && partSlot.name.toLowerCase() == 'body') {

				} else {
					if(partSlot.displayName && partSlot.isEditable){
						displayNameMap[partSlot.displayName.toLowerCase()] = partSlot;
					}else{
						var name_str : String = partSlot.name.toLowerCase();
						if((name_str != "shadow") && (name_str != "black")){
							displayNameMap[partSlot.name.toLowerCase()] = partSlot;
						}


					}
				}

				
			}
			
			var result : Array = [];
			var resultWithOneDetailFromMainLIst : Array = [];
			var resultWithOneDetailFromAddLIst : Array = [];
			var displayName : String;
			var partSlot2 :PartSlot;
			
			for each(displayName in partSlotsOrder)
			{	
				partSlot = displayNameMap[displayName.toLowerCase()];
				if(partSlot)
				{
					if((partSlot.possibleParts.length<=1) && (partSlot.name.toLowerCase() != "windows" )){
						resultWithOneDetailFromMainLIst.push(partSlot);
					}else{
						result.push(partSlot);
					}
				}
				
				delete displayNameMap[displayName.toLowerCase()];
			}
			
			//добавим остальное
			for each(partSlot2  in displayNameMap)
			{//не основной список
				if((partSlot2.possibleParts.length<=1) && (partSlot2.name.toLowerCase() != "windows" )){
					resultWithOneDetailFromAddLIst.push(partSlot2);
				}else{
					result.push(partSlot2);
				}
			}
			
			//переместим теперь
			for each(partSlot2 in resultWithOneDetailFromMainLIst)
			{
				result.push(partSlot2);
			}
			
			for each(partSlot2 in resultWithOneDetailFromAddLIst)
			{
				result.push(partSlot2);
			}
			
			var newResult:Array=[];
			
			for each(partSlot2 in result)
			{

				if(partSlot2.name!="chassi")
				{
					newResult.push(partSlot2);
                }
				else if (suvTires)
				{
					newResult.push(partSlot2);
					partSlot2.isEditable = true;
				}
			}

			_editablePartSlots = newResult;

            partSlotsOrder = null;
            displayNameMap = null;
            partSlot = null;
            partSlot2 = null;
            result=null;
			newResult = null;
		}
		
		
		
		public function get localFile() : String
		{
			if(imageURL){
				var logoParts : Array = imageURL.split("/");
				var localImagePath : String = "Cars/Logos/"+ this.id + logoParts[logoParts.length - 1];
				return localImagePath;
			}else{
				return "";
			}
		}


        public function set remoteUrl(value:String):void{
			
			imageURL=value;
		}
		public function get remoteUrl() : String
		{
			return imageURL;
		}
		
		public function get isEncrypted() : Boolean 
		{
			return false;
		}
		
		public function get ignoresTransparentPixelsOnTouch() : Boolean 
		{
			return false;
		}
		
		public function get touchable() : Boolean 
		{
			return true;
		}
		
		public function get important() : Boolean
		{
			return false;
		}
		
		public function get brightness() : Number 
		{
			return 1.0;
		}
		
		public function set brightness(value : Number) : void
		{
			
		}
		
		public function get material3dtProvider() : Material3dtProvider
		{
			return null;
		}

    public function get imagePath():String {
        return _imagePath;
    }

    public function set imagePath(value:String):void {
        _imagePath = value;
    }

	public static function jsonToCar(obj:Object):Car{
		var car:Car = new Car();
		car.id = obj._id;
		car.defaultBkg = obj.defaultBkg;
		car.defaultBkgMobile = obj.defaultBkgMobile;
		car.custom_offset = obj.custom_offset;
		return car;
	}

	public function findPart(value:String):Part {
		if(possiblePartsMap[value]){
			return possiblePartsMap[value];
		}
		var result:Part;
		for(var i:int = 0; i < possiblePartSlots.length; i++){
			var ps:PartSlot = possiblePartSlots[i] as PartSlot;
			if(ps){
				var p:Part = ps.getPartById(value);
				if(p){
					result = p;
					break;
				}
			}
		}
		return result;
	}

	public function get exteriorSelectedPartSlots():Array{
		var result:Array = selectedPartSlots.concat();
		for(var i:int = result.length - 1; i >= 0; i--){
			if(!result[i] || EXTERIOR_EXCLUDE.indexOf(PartSlot(result[i]).name) != -1 || PartSlot(result[i]).isBackground()){
				result.removeAt(i);
			}
		}
		return result;
	}

	public function get interiorSelectedPartSlots():Array{
		var result:Array = selectedPartSlots.concat();
		var windowsPartSlot:PartSlot = getPossiblePartSlot("windows");
		for(var i:int = result.length - 1; i >= 0; i--){
			if(!result[i] || INTERIOR_SLOTS.indexOf(PartSlot(result[i]).name) == -1){
				result.removeAt(i);
			}
		}
		if(result.length == 0){
			result.push(windowsPartSlot);
		}
		return result;
	}
}
}

