package com.despark.tuning3d.model.cars
{
	import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.brands.BrandsParser;
import com.despark.tuning3d.model.core.Parser;
	import com.despark.tuning3d.model.core.Tracer;
	import com.despark.tuning3d.model.core.http.HttpClient;
	import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
	import com.despark.tuning3d.model.parts.Part;
	import com.despark.tuning3d.model.parts.PartDisk;
	import com.despark.tuning3d.model.parts.PartMaterial;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.core.DASLog;
	import com.utils.Console;
	import com.utils.JSONUtils;
	import com.utils.Logger;
	import com.utils.MemoryTracker;

import feathers.utils.display.getDisplayObjectDepthFromStage;

import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.system.System;
	import flash.utils.getTimer;
	import flash.utils.setTimeout;
	
	import by.blooddy.crypto.serialization.JSONer;
	
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import starling.utils.AssetManager;
	
	public class CarsProxy extends Proxy
	{
		public static const NAME : String = "CarsProxy";
		private static var _IS_FULL : Boolean = false;
		public static const CARS_FULL_PREFIX:String = "/full";
		public static var COMPONENTS_PREFIX:String = "/components";

		public var httpClient : HttpClient;
		public var assetManager : AssetManager;
		
		private var carsParser :Parser;
		//private var carsParserOld : CarsParserOld;
		private var _loadedCar : Car;
		private var tunningJSON : Object;
		private var tunning : Tuning;
		private var tuningSelectedBackground : CarBackground;
		private var carJSON : Object;
		private var componentsJSON : Object;
		private var componentsSizesJSON : Object;
		
		private var _bonusCars:Array=null;
		
		private var failedLoadingCar: Boolean;
		
		private var countComponentTypes:int=0;
		private var loadedComponentTypes:int=0;
		private var allComponents:Array;

		public static function get IS_FULL() : Boolean
		{
			return _IS_FULL;
		}


		public static function set IS_FULL(value : Boolean) : void
		{
			_IS_FULL = value;
            if(_IS_FULL)
            {
                COMPONENTS_PREFIX = "/components" + CARS_FULL_PREFIX;
            }
            else {
                COMPONENTS_PREFIX = "/components";
            }
		}


		public function CarsProxy()
		{
			super(NAME);


            if(IS_FULL)
            {
                COMPONENTS_PREFIX = "/components" + CARS_FULL_PREFIX;
            }
            else {
                COMPONENTS_PREFIX = "/components";
            }


			//carsParser = new CarsParser();
			
			//carsParserOld = new CarsParserOld();
		}
		
		public function get loadedCar() : Car
		{
			return _loadedCar;  
		}
		
		
		
		public function get bonusCars():Array{
			
			if(!_bonusCars){
				
				_bonusCars=getBonusCars();
			}
			
			return _bonusCars;
		}
		
		public function getBonusCars():Array{
			
			var file:File=File.applicationStorageDirectory.resolvePath("bonus_cars.dat");
			
			var result:String='[]';
			
			if(file.exists){
				var fs:FileStream=new FileStream(); 
				fs.open(file, FileMode.READ);
				result=fs.readUTF()
				fs.close();
			
			}

			_bonusCars = JSONUtils.parse(JSONer, result) as Array;
			return _bonusCars;
		}
		
		public function bonusCarExists(car_id:String):Boolean{
			
			var cars:Array=bonusCars;
			var carExists:Boolean=false;
			
			for(var i:int=0; i<cars.length; i++){
				
				if(cars[i]==car_id){
					
					carExists=true;
					break;
				}
			}
			
			return carExists;
		}
		
		public function addBonusCars(car_id:String):void{
			
			var cars:Array=getBonusCars();
			var carExists:Boolean=false;
			for(var i:int=0; i<cars.length; i++){
				
				if(cars[i]==car_id){
					
					carExists=true;
					break;
				}
			}
			
			if(!carExists){
				
				cars.push(car_id);
			}
			
			
			var file:File=File.applicationStorageDirectory.resolvePath("bonus_cars.dat");
			
			_bonusCars=cars;
			var result:String=JSONer.stringify(cars);
			
			
			var fs:FileStream=new FileStream(); 
			fs.open(file, FileMode.WRITE);
			fs.writeUTF(result);
			fs.close();
			
		}
		
		public function loadCarForTunning(tuning : Tuning,  aTuningJSON : Object = null) : void
		{
            var backgrounds : Array = CarBackgroundsProxy.CAR_ALL_BGS;
			if(aTuningJSON)
			{
				tunningJSON = aTuningJSON;
				loadCarFromTuningJSON(tuning, aTuningJSON, backgrounds);
			}
			else
			{
				httpClient.getPath("tunings/" + tuning.id, null, function(success : Boolean, tunningDetailsJSONString : String) : void
				{
					if(success)
					{
						tunningJSON = JSONUtils.parse(JSONer, tunningDetailsJSONString);
						loadCarFromTuningJSON(tuning, tunningJSON, backgrounds);
					}
					else 
					{
						sendNotification(Notifications.CAR_TUNNING_LOADING_FAILED, {isChallenge:tuning.car.isChallenge});
					}
				});
			}
		}

		public function sendZipErrorData(zipUrl :String) : void
		{
			var params : Object = {};
			params['url'] = zipUrl;

            Tracer.print(zipUrl, "ImageLoadWithStates", "zipNotFound", true);

			httpClient.postPath("",params, function (success : Boolean, result : Object) : void {
				if(success)
				{
					Tracer.print("zip error data sent ");
				}
            })
		}
		
		private function loadCarFromTuningJSON(aTuning : Tuning, tuningJSON : Object, carBackgrounds : Array) : void
		{
			tunning = aTuning;
            if(tuningJSON.background)
            {
                if(tuningJSON.background is String){
                    tunning.backgroundId = tuningJSON.background;
                }else{
                    tunning.backgroundId= tuningJSON.background._id;
                }
            }


            var car : Car ;
			if(tunningJSON.car is String){
                car = new Car();
				car.id = tunningJSON.car;
			}else if(tunningJSON.car is Object){
                car = BrandsParser.allCarsList[tuningJSON.car.id];
				if(!car)
				{
					car = new Car();
					car.id = tuningJSON.car.id;
				}

			}
			
			if(tunning.backgroundId)
                for each(var carBackground : CarBackground in carBackgrounds)
                {
                    if(carBackground.id == tunning.backgroundId)
                    {
                        tuningSelectedBackground = carBackground.copyWithShowMainUrl();
                        break;
                    }
                }

            loadCar(car);
		}
		
		public function hasCachedCar(car : Car) : Boolean  
		{
			try {
                return httpClient.hasCache("cars/full" + car.id) && httpClient.hasCache("cars/" + car.id + COMPONENTS_PREFIX);
            }catch (e : Error)
			{return false}
		}
		
		private var startTime:int;
		
		private var _callbackCompleteLoadComponent:Function = null;
		private var _loadedComponents:int = 0;
		private var _loadingComponents:int = 0;
		private var _resultComponents:Array;
		
		public function loadComponent(partSlots:Array, callbackCompleteLoadComponent:Function):void
		{
			
			_callbackCompleteLoadComponent = callbackCompleteLoadComponent;
			var partSlot:PartSlot
			
			_loadedComponents = 0;
			_loadingComponents = 0;
			_resultComponents = [];
			var parts:Array = [];
			var part:Part;
			
			for each(partSlot in partSlots)
			{
				if(!partSlot.selectedPart) continue;
				
				if(!partSlot.selectedPart.selectedMaterial) continue;
				
				if(partSlot.selectedPart.selectedMaterial.isPartMaterialImagesLoaded) {
					continue;
				}

				if(partSlot.selectedParts && partSlot.selectedParts.length>0)
				{
					for(var i:int=0; i<partSlot.selectedParts.length; i++)
					{
						if(!partSlot.selectedParts[i].selectedMaterial) continue;
						if(partSlot.selectedParts[i].selectedMaterial.isPartMaterialImagesLoaded) continue;
				
						_loadingComponents++;
						parts.push(partSlot.selectedParts[i]);
					}
				}
				else
				{
					_loadingComponents++;
					parts.push(partSlot.selectedPart);
				}
			}

			if(_loadedComponents>=_loadingComponents)
			{
				if(_callbackCompleteLoadComponent){
					_callbackCompleteLoadComponent(_resultComponents);
					_callbackCompleteLoadComponent=null;
				}
				_resultComponents = null;
				
			} else {
				for each (part in parts){
					if(part){
						loadOneComponent(part, completeLoadOneComponent);
					}
				}
			}
		}
		
		
		private function completeLoadOneComponent(part:Part, component:Object):void{
			if(part && part.parentSlotName == "tire_front_left"){
				Console.log();
			}

			if(component){
				_loadedComponents++;
				Part.LOADED_PARTS[component._id] = true;
				part.parseFiles(component);
				part.parseSubparts(component);
				part.parseStates(component);
                _resultComponents ||= [];
				_resultComponents.push(component);
			}else if(part && part.id){
				loadOneComponent(part, completeLoadOneComponent);
			}
			
			if(_loadedComponents>=_loadingComponents){
				
				if(_callbackCompleteLoadComponent){
					_callbackCompleteLoadComponent(_resultComponents);
					_callbackCompleteLoadComponent=null;
					
					//trace("_callbackCompleteLoadComponent");
				}
				_resultComponents = null;
			}
		}


		public function loadFastModeData(carId:String, resultHandler:Function):void{
			httpClient.getPath("https://api.3dtuning.com/api/v1/fastModeSpecific/" + carId, null, function (success:Boolean, result:String):void{
				if(success) {
					var resultObj:Object = JSONUtils.parse(JSONer, result);
					resultHandler(resultObj);
				}else{
					resultHandler({});
				}
			});
		}
		
		private function loadOneComponent(part:Part, completeLoadOneComponent:Function):void{
			var id:String = part.id;
			var partDisk:PartDisk ;
			if(part is PartDisk){
				partDisk = part as PartDisk;
				id = partDisk.getSelectedRadiusId();
			}
			if(id == null){
				if(completeLoadOneComponent){
					completeLoadOneComponent(null, null);
				}
				return;
			}
			
			httpClient.getPath("components/" + id , {isOptimized:true}, function(success : Boolean, componentsResult : Object) : void {
				if(success) {
					//Logger.d("components:"+componentsResult, this);
					//var startTime:int = getTimer();
					var errorData:Object = {
						"function": "loadOneComponent",
						"carId": (part && part.parentSlot && part.parentSlot.parentCar)? part.parentSlot.parentCar.id: "",
						"carName": (part && part.parentSlot && part.parentSlot.parentCar)? part.parentSlot.parentCar.displayName: "",
						"partSlotName": (part && part.parentSlot)? part.parentSlot.name: "",
						"partId": id,
						"partName": part? part.name: ""
					};
					componentsJSON = JSONUtils.parse(JSONer, componentsResult as String, null, errorData);
					CarsParser.componentJSONRestoreOptimizedMaterials(componentsJSON);
					
					if(completeLoadOneComponent){
						completeLoadOneComponent(part, componentsJSON);
					}
					//trace("parse component time:", getTimer()-startTime)
					//componentsResult=null;
					//checkIfAllCarFilesLoaded();
					
				}else{
					
					if(completeLoadOneComponent){
						completeLoadOneComponent(part, null);
					}
					//httpClient.cancelAllRequests();
					//notifyForFailedCarLoading();
				}
			});
			
		}

		private function loadNonChopperComponents(car : Car ) : void
		{
			var params : Object;
			if(car.carType == 'pickup hd' || car.id == "5dbae7da9101e13b9472a138") {
				params = {};
				params["not_organic_sorting"] = true;
			}
			params ||= {};
			params['isMobile'] = true;
			params['isOptimized'] = true;
			// params['type'] = 'simple';
            httpClient.getPath("cars/"+ car.id + COMPONENTS_PREFIX , params, function(success : Boolean, componentsResult : Object) : void
            {


                if(success && !failedLoadingCar)
                {
                    //Logger.d("components:"+componentsResult, this);
                    var startTime:int = getTimer();
                    componentsJSON = JSONUtils.parse(JSONer, componentsResult as String);
                    trace("parse component time:", getTimer()-startTime);
                    Tracer.print("CAR COMPONENTS LOADED", getProxyName());
                    componentsResult=null;
                    checkIfAllCarFilesLoaded();

                }
                else
                {
                    httpClient.cancelAllRequests();
                    notifyForFailedCarLoading();
                }
            });
		}

		private function loadNonChopper (car : Car) : void {
			var isChallenge:Boolean = car.isChallenge;
            httpClient.getPath("cars/" + car.id  , null, function(success : Boolean, carResult : Object) : void
            {
                if(success)
                {
                    var startTime:int = getTimer();
                    carJSON = JSONUtils.parse(JSONer, carResult as String);
                    Tracer.print("CAR LOADED", getProxyName());
                    trace("parse car time:", getTimer()-startTime);

                    carResult=null;

                    if(!carJSON.components || (carJSON.components.length == 0)){
                        //Logger.d("Error!!!!! no components", this);
                        failedLoadingCar = true;
                        carJSON = null;
                        httpClient.cancelAllRequests();
                        notifyForFailedCarLoading();
                    }else{
                        checkIfAllCarFilesLoaded();
                    }

                }
                else
                {
                    Tracer.print("CAR FAILED", getProxyName());
                    httpClient.cancelAllRequests();
                    notifyForFailedCarLoading(isChallenge);
                }
            });
		}

		private function loadChopper(car : Car) : void
		{
            httpClient.getPath("cars/" + car.id  , null, function(success : Boolean, carResult : Object) : void
            {
                if(success)
                {

                    carJSON = JSONUtils.parse(JSONer, carResult as String);

                    carResult=null;

                    if(!carJSON.components || (carJSON.components.length == 0)){
                        //Logger.d("Error!!!!! no components", this);
                        failedLoadingCar = true;
                        carJSON = null;
                        httpClient.cancelAllRequests();
                        notifyForFailedCarLoading();
                    }else{
                        //checkIfAllCarFilesLoaded();
                        allComponents=[];
                        countComponentTypes=0;
                        componentsJSON=null;
                        loadedComponentTypes=0;
                        var partSlotName:String;
                        for each (partSlotName in carJSON.layersPositions[0]){
                            if(partSlotName=="") continue;
                            countComponentTypes++;
                        }

                        for each (partSlotName in carJSON.layersPositions[0]){
                            if(partSlotName=="") continue;

                            httpClient.getPath("cars/" + car.id + COMPONENTS_PREFIX  , {type:partSlotName, isOptimized:true}, function(success : Boolean, componentsResult : Object) : void
                            {

                                if(success && !failedLoadingCar)
                                {

                                    componentsJSON = JSONUtils.parse(JSONer, componentsResult as String);

                                    allComponents=allComponents.concat(componentsJSON);
                                    loadedComponentTypes++;

                                    componentsJSON=null;
                                    componentsResult=null;

                                    if(loadedComponentTypes==countComponentTypes){

                                        componentsJSON=allComponents;
                                        allComponents=null;

                                        checkIfAllCarFilesLoaded();
                                    }
                                }
                                else
                                {
                                    httpClient.cancelAllRequests();
                                    notifyForFailedCarLoading();
                                }
                                // &type=typeName
                            });


                        }
                    }

                }
                else
                {
					httpClient.cancelAllRequests();
					notifyForFailedCarLoading();
                }
            });
		}


		public function loadCar(car : Car) : void
		{
			startTime = getTimer();
			trace("start loadCar");
			httpClient.httpCache.clean();
			
			carJSON = componentsJSON = componentsSizesJSON = null;
			//+ "?forMobile=true"
			failedLoadingCar = false;

			if (CarsParser.CHOPPER_ID_LIST.indexOf(car.id) != -1) {
				car.isChopper = true;
			}else if (CarsParser.BEETLE_ID_LIST.indexOf(car.id) != -1) {
				car.isBeetle = true;
			}
			Tracer.print("START CAR LOAD", getProxyName());

			if(car.isChopper){
				loadChopper(car);
			}else{


				loadNonChopper(car);
				loadNonChopperComponents(car);

			}
		}
		
		private function notifyForFailedCarLoading(isChallenge:Boolean = false) : void
		{
			sendNotification(tunningJSON ? Notifications.CAR_TUNNING_LOADING_FAILED : Notifications.CAR_LOADING_FAILED, {isChallenge:isChallenge});
		}
		
		private function checkIfAllCarFilesLoaded() : void
		{


			trace("checkIfAllCarFilesLoaded", getTimer()-startTime);
			startTime = getTimer();
			var partMaterialZip:Boolean=false;
			
			if(carJSON && !componentsJSON){
				
				
				
				if(carJSON.hasOwnProperty("partMaterialZip")){
					partMaterialZip=carJSON.partMaterialZip;
				}
				
				if(partMaterialZip)
					sendNotification(Notifications.CAR_LOADED_STEP, {progress:0.01});
			}
			
			if(carJSON && componentsJSON)
			{
				if(carJSON.hasOwnProperty("partMaterialZip")){
					partMaterialZip=carJSON.partMaterialZip;
				}
				


				if(partMaterialZip){
					sendNotification(Notifications.CAR_LOADED_STEP, {progress:0.02});
					setTimeout(handleCarLoaded, 5);
				
				}else{
					
					handleCarLoaded();
				}
				
			}
		}
		
		
		
		private function handleCarLoaded() : void
		{
			trace("handleCarLoaded:", getTimer() - startTime );
			startTime = getTimer();
			
			var partMaterialZip:Boolean=false;
			
			try{
				
				if(carJSON.hasOwnProperty("partMaterialZip")){
					partMaterialZip=carJSON.partMaterialZip;
				}
				
				if(partMaterialZip){
					
					carsParser = new CarsParser();
					(carsParser as CarsParser).parseObjectAsync(carJSON, null, componentsJSON, componentsSizesJSON, tunningJSON, parserProgressCallback, parserCompleteCallback);
				
				}else{
					
					carsParser = new CarsParserOld();
					var car:Car=carsParser.parseObject(carJSON, null, componentsJSON, componentsSizesJSON, tunningJSON) as Car;
					
					parserCompleteCallback(car);
				}
				
			}catch(e:Error){
				
				Tracer.print("handleCarLoaded error:"+e.getStackTrace()/*+" stack:"+e.getStackTrace()*/);
			}
		}
		
		
		
		private function parserProgressCallback(progress:Number):void{
			
			sendNotification(Notifications.CAR_LOADED_STEP, {progress:0.02+(progress*0.38)});
		}
		
		
		private function parserCompleteCallback(car:Car):void{
			
			_loadedCar = car;
			
			//когда путь уже посчитан...
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.changeCar(_loadedCar);
			
			if(tunningJSON)
			{
				_loadedCar.selectedBackground = tuningSelectedBackground;
				if(tunning)
				{
					tunning.car = _loadedCar;
				}
				sendNotification(Notifications.CAR_TUNNING_LOADED, loadedCar);
			}
			else
			{
				sendNotification(Notifications.CAR_LOADED, loadedCar);
			}

			if (carJSON) {
				carJSON.length = 0;
			}
			if (componentsJSON) {
				componentsJSON.length = 0;
			}


			if(componentsSizesJSON)
				componentsSizesJSON.length = 0;
			
			if(tunningJSON)
				tunningJSON.length = 0;
			
			if(allComponents){
				
				allComponents.length = 0;
			}
			
			carJSON = null;
			componentsJSON = null;
			componentsSizesJSON = null;
			tunningJSON = null;
			
			allComponents = null;
			
			
			//tuningSelectedBackground = null;
			//allComponents = null;
			
			_bonusCars = null;
			
			_loadedCar=null;
			
			
			trace("handleCarLoaded:", getTimer() - startTime );
			//System.gc();
			//System.gc();
		}
		
		
		public function updatePartSlotsSizes(car : Car, partSlots : Array) : void
		{
			/*var carLocalFile : File = File.cacheDirectory.resolvePath(car.path);
			carLocalFile.preventBackup = true;
			for each(var partSlot : PartSlot in partSlots) 
			{
				var partSlotItemsCount : int = 0;
				var downloadedImagesCount : int = 0;
				
				for each(var part : Part in partSlot.possibleParts)
				{	
					for each(var partMaterial : PartMaterial in part.possibleMaterials)	
					{
						//partSlotItemsCount += partMaterial.countImages;
						
						var imagesFile : File = carLocalFile.resolvePath(partSlot.name + "/" + part.folder + "/" + partMaterial.name);
						imagesFile.preventBackup = true;
						if(imagesFile.exists)
						{
							downloadedImagesCount += imagesFile.getDirectoryListing().length;
						}						
					}
				}
				
				if(partSlotItemsCount != 0)
				{
					partSlot.percentLoaded = downloadedImagesCount / partSlotItemsCount;
				}
				else
				{
					partSlot.percentLoaded = 0;
				}
			}*/
		}
	}
}

