package com.despark.tuning3d.model.challenges {

    import by.blooddy.crypto.serialization.JSONer;

    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.http.HttpClient;
    import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.friend.FriendProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.modals.ChallengeErrorCode;
    import com.utils.Console;
    import com.utils.DateParser;
    import com.utils.FileUtil;

    import flash.filesystem.File;
    import flash.net.URLRequestMethod;

    import org.puremvc.as3.patterns.proxy.Proxy;

    public class ChallengesProxy extends Proxy {
    public static const NAME : String = "ChallengesProxy";
    public var httpClient:HttpClient;
    private var showCountError:Boolean = false;
    private var challengeNotes:File;
    private var myChallengeVotesFile : File;
    public var myChallengeVotes: Object;


    public function ChallengesProxy() {
        super(NAME);
        loadChallengeVotesFromFile();
    }

    private function loadChallengeVotesFromFile(): void {
        myChallengeVotesFile = File.applicationStorageDirectory.resolvePath("myChallengeVotes.json");
        if(myChallengeVotesFile.exists){
            var votesStr:String = FileUtil.getFileContent(myChallengeVotesFile);
            if(votesStr && votesStr.length){
                try{
                    myChallengeVotes = JSONer.parse(votesStr);
                }catch (e:Error) {
                    myChallengeVotes = {};
                }
            }else{
                myChallengeVotes = {};
            }
        }else{
            myChallengeVotes = {};
        }
    }


    public function loadUserChallenges(userId:String, selectTuning:Boolean) : void {
        var params:Object = {
            user: userId,
            limit: 0,
            status: "accepted"
        };
        var url : String = HttpClient.CHALLENGE_ROOT_URL;
        httpClient.getPath(url, params, function(success : Boolean, result : Object) : void {
            if (success) {
                sendNotification(Notifications.CHALLENGES_LOADED, {user:userId, challenges:JSONer.parse(result), selectTuning:selectTuning});
            } else {
                trace("Failed to load challenges", JSON.stringify(result));
            }
        });
    }

    private function createChallengeCallback(success : Boolean, result : Object, headers:Array = null) : void{
        if(success){
            sendNotification(Notifications.CHALLENGE_CREATE_SUCCESS);
        } else {
            var message: String;
            if(showCountError){
                if(result.hasOwnProperty("title") && result.hasOwnProperty("body")){
                    message = result.title + result.body;
                }else if(result.hasOwnProperty("message")){
                    message = result.message;
                }
                showCountError = false;
                Console.log("createChallengeCallback", JSON.stringify(result));
                var errorCode:int = 0;
                if(result.hasOwnProperty("errorType")){
                    errorCode = ChallengeErrorCode.getStatusCode(result.errorType);
                }
                sendNotification(Notifications.CHALLENGE_CREATE_ERROR, {statusCode: errorCode, message: message});
            }
        }
    }

    public function getNotifications(skip:int = 0, limit:int = 7, completeCallback:Function = null):void{
        var url:String = "http://mailer.3dtuning.com/api/v1/notifications?mobile=true&limit="+limit.toString()+"&skip="+skip.toString()+"&typeGroup=new_challenge_all&populatebackground=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
                function (success : Boolean, result : Object) : void {
                    if(completeCallback!=null) {
                        if (success) {
                            completeCallback.apply(null, [JSON.parse(result as String)]);
                        }else{
                            completeCallback.apply(null, [{}]);
                        }
                    }
                }, url);
    }

    public function getRequests(skip:int = 0, limit:int = 20, completeCallback:Function = null):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        var url:String = HttpClient.CHALLENGE_ROOT_URL + "?mobile=true&skip="+skip+"&limit="+limit+"&populate=owner.tuning,owner.user&receiver.user="+user.id+"&smart=true&sort=-updated&status=pending";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
            function (success : Boolean, result : Object) : void {
                if(completeCallback!=null) {
                    if (success) {
                        completeCallback.apply(null, [JSON.parse(result as String)]);
                    }else{
                        completeCallback.apply(null, [{}]);
                    }
                }
            }, url);
    }

    /**
     * new challenges
     */
    public function getChallengeList(completeCallback: Function, skip:int = 0, limit:int = 9, finishSkip:int = 0): void {
        var url: String = HttpClient.CHALLENGES_LIST_URL;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.currentUser && usersProxy.currentUser.isSuperAdmin){
            url += "?full=true";
        }
        httpClient.getPath(url, {limit:limit, skip:skip, finish_skip:finishSkip}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeCallback(challengeData);
            } else {
                completeCallback(null, false);
            }
        })
    }

     public function getChallengeVoteList(completeCallback: Function): void {
        var url: String = HttpClient.CHALLENGES_VOTE_LIST;
        httpClient.getPath(url, {}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeCallback(challengeData);
            } else {
                completeCallback(null, false);
            }
        })
    }

    public function getLeaderboardData(completeCallback: Function): void {
        var url: String = HttpClient.CHALLENGES_LEADERBOARD_URL;
        httpClient.getPath(url, {}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeCallback(challengeData);
            } else {
                completeCallback(null, false);
            }
        })
    }

    public function getChallengeLeaderboardData(challengeId:String, completeCallback: Function, limit:int = 12, skip:int = 0): void {
        var url: String = HttpClient.CHALLENGES_LEADERBOARD_URL + "/" + challengeId;
        var params:Object = {limit:limit, skip:skip};
        if(skip == 0){
            params["userBuild"] = true;
        }
        httpClient.getPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                if(challengeData is Array){
                    completeCallback(challengeData);
                }else if(challengeData.hasOwnProperty("user") && challengeData.hasOwnProperty("all")){
                    if(challengeData.all.length > limit - 2){
                        challengeData.all = challengeData.all.slice(0, limit - 2);
                    }
                    completeCallback(challengeData.all, true, challengeData["user"]);
                }
            } else {
                completeCallback(null, false);
            }
        })
    }

    public function getUserChallengeTunings(userId:String, completeCallback: Function, limit:int = 12, skip:int = 0, bodyType:String = "all"): void {
        var url: String = HttpClient.CHALLENGES_USER_TUNINGS + "/" + userId;
        var params:Object = {limit:limit, skip:skip};
        if(bodyType && bodyType != "all"){
            params.type = bodyType;
        }
        httpClient.getPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeCallback(challengeData);
            } else {
                completeCallback(null, false);
            }
        })
    }

    public function getChallengeVotePair(challengeId:String, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_RANDOM_PAIR + "/" + challengeId;
        httpClient.getPath(url, {}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeHandler(challengeData);
            } else {
                completeHandler(null, false);
            }
        })
    }


    public function voteForChallenge(data:Object, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_VOTE_LIST;
        httpClient.postPath(url, JSON.stringify(data), function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                completeHandler(success, challengeData);
            } else {
                completeHandler(success, null);
            }
        })
    }

    public function addKeyForChallenge(userId:String, keys:int, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_ADD_KEY;
       var params:Object = JSON.stringify({user: userId, keys: keys});
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var userData: Object = JSON.parse(result);
                saveUserJSONData(userData);
                completeHandler(userData);
                sendNotification(Notifications.USER_DATA_UPDATED, userData);
            } else {
                completeHandler(null);
            }
        })
    }

    public function addDiamondForChallenge(userId:String, diamonds:int, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_ADD_DIAMOND;
       var params:Object = JSON.stringify({user: userId, diamonds: diamonds});
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var userData: Object = JSON.parse(result);
                saveUserJSONData(userData);
                completeHandler(userData);
                sendNotification(Notifications.USER_DATA_UPDATED, userData);
            } else {
                completeHandler(null);
            }
        })
    }

    public function addBothForChallenge(userId:String, diamonds:int, keys:int):void {
        var url: String = HttpClient.CHALLENGES_ADD_BOTH;
       var params:Object = JSON.stringify({user: userId, diamonds: diamonds, keys: keys});
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var userData: Object = JSON.parse(result);
                saveUserJSONData(userData);
                sendNotification(Notifications.USER_DATA_UPDATED, userData);
            } else {

            }
        })
    }

    public function addVoteKeyForChallenge(challengeId:String, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_ADD_VOTE_KEY;
        var params:Object = JSON.stringify({challenge: challengeId});
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);
                saveUserJSONData(challengeData);
                completeHandler(challengeData);
            } else {
                completeHandler(null);
            }
        })
    }

    public function getGuestDeviceData(deviceId:String, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_GUEST_DAILY_WELCOME + "/" + deviceId;
        httpClient.getPath(url, null, function (success : Boolean, result : String): void {
            if (success) {
                if(result){
                    var resultData: Object = JSON.parse(result);
                    completeHandler(resultData);
                }else{
                    completeHandler({firstVisit:true});
                }
            } else {
                completeHandler(null);
            }
        })
    }

    private function saveUserJSONData(userData:Object):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.saveUserJSONData(userData);
    }

    public function addUserDailyRewardData(userId:String, rewardData:Object):void {
        var params:Object = JSON.stringify({date: DateParser.toW3CDTF(new Date())});
        var url: String = HttpClient.CHALLENGES_USER_DAILY_REWARDS + "/" + userId;
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if(success){
                addBothForChallenge(userId, rewardData.diamonds, rewardData.keys);
            }
        });
    }

    public function updateRewardData(userId:String, rewardData:Object):void {
        var params:Object = {rewardCount:rewardData.rewardCount};
        var url: String = HttpClient.CHALLENGES_USER_DAILY_REWARDS + "/" + userId;
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
            if(success){
                Console.log();
            }else{
                Console.log();
            }
        });
    }

    public function getUserRewardData(userId:String, completeHandler:Function):void {
        var url: String = HttpClient.CHALLENGES_USER_DAILY_REWARDS + "/" + userId;
        httpClient.getPath(url, null, function (success : Boolean, result : String): void {
            if (success) {
                if(result){
                    var resultData: Object = JSON.parse(result);
                    completeHandler(resultData);
                }else{
                    completeHandler({firstVisit:true});
                }
            } else {
                completeHandler(null);
            }
        })
    }

    public function updateGuestDeviceData(deviceId:String):void {
        var params:Object = JSON.stringify({date: DateParser.toW3CDTF(new Date())});
        var url: String = HttpClient.CHALLENGES_GUEST_DAILY_WELCOME + "/" + deviceId;
        httpClient.postPath(url, params, function (success : Boolean, result : String): void {
        });
    }

    public function getNewNotifications(skip:int = 0, limit:int = 7, completeCallback:Function = null):void{
        var url:String = "http://mailer.3dtuning.com/api/v1/notifications?mobile=true&limit="+limit.toString()+"&skip="+skip.toString()+"&typeGroup=challengeNew&populatebackground=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
                function (success : Boolean, result : Object) : void {
                    if(completeCallback!=null) {
                        if (success) {
                            completeCallback.apply(null, [JSON.parse(result as String)]);
                        }else{
                            completeCallback.apply(null, [{}]);
                        }
                    }
                }, url);
    }


        /**
     * END OF NEW CHALLENGES
     */

    public function markAllUnreadNotifications(completeCallback:Function = null):void{
        challengeNotes = File.applicationStorageDirectory.resolvePath("challenge-notes.json");
        var challengeNotesContent:Object;
        if(challengeNotes.exists){
            challengeNotesContent = JSON.parse(FriendProxy.fileGetContent(challengeNotes));
        }
        challengeNotesContent ||= {};
        getRequestIdList(function (idList:Object):void {
            if(idList && idList.data){
                var arr:Array = idList.data;
                for(var i:int = 0; i < arr.length; i++){
                    if(!challengeNotesContent.hasOwnProperty(arr[i])){
                        challengeNotesContent[arr[i]] = 1;
                    }
                }
            }
            FriendProxy.filePutContent(challengeNotes, JSON.stringify(challengeNotesContent));
            if(completeCallback != null){
                completeCallback.call();
            }
        });
    }

    public function markUnreadNotifications() : void{
        httpClient.loadPath('', URLRequestMethod.PUT, null, HttpCachePolicy.NONE,false, function (success:Boolean, result:Object):void{
            if(success){

            }
        }, 'http://mailer.3dtuning.com/api/v1/notifications?type=challengeNew');
    }

    public function claimRewards(challengeId:String, rewardCount:int) : void{
        var params:Object = {'claimed': true};
        httpClient.putPath('', params, function (success:Boolean, result:Object):void{
            if(success){
                addDiamondForChallenge(UsersProxy.loggedInUser().id, rewardCount, function(userData:Object):void{

                });
            }else{
                Console.log();
            }
        }, 'http://mailer.3dtuning.com/api/v1/notifications/claim/' + challengeId);
    }

    public function getRequestIdList(completeCallback:Function = null):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        if(user){
            var url:String = HttpClient.CHALLENGE_ROOT_URL + "?limit=50000&populate=owner.tuning,owner.user&receiver.user="+user.id+"&smart=true&sort=-updated&status=pending&count=true";
            httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
                    function (success : Boolean, result : Object) : void {
                        if(completeCallback!=null) {
                            if (success) {
                                completeCallback.apply(null, [JSON.parse(result as String)]);
                            }
                        }
                    }, url);
        }
    }

    public function getUnreadNotificationCount(completeCallback:Function = null):void{
        var url:String = "http://mailer.3dtuning.com/api/v1/notifications?typeGroup=challengeNew&count=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
            function (success : Boolean, result : Object) : void {
                if(completeCallback!=null) {
                    if (success) {
                        completeCallback.apply(null, [JSON.parse(result as String)]);
                    }
                }
            }, url);
    }


        public function resetRewardsData(userId:String):Object {
            var defaultData:Object = {userId:userId, rewardCount: 0, date: DateParser.getDateTime(new Date())};
            updateRewardData(userId, defaultData);
            return defaultData;
        }
    }
}
