
package com.despark.tuning3d.model.core.images
{
import com.despark.tuning3d.model.core.Tracer;

import flame.crypto.MD5;

import flash.display.BitmapData;
import flash.filesystem.File;
import flash.filesystem.FileMode;
import flash.filesystem.FileStream;
import flash.geom.Matrix;
import flash.utils.ByteArray;

public class BitmapDataUtil
	{
		public static var globalWritingErrorHandler : Function;
		public static var writingBitmapDataErrorOccured : Boolean;
		
		private static var fileStream : FileStream = new FileStream();
		
		
		public static function clearWritingError() : void
		{
			writingBitmapDataErrorOccured = false;  
		}
		
		public static function saveImageBytes(byteArray : ByteArray, file : File, errorHandler : Function = null) : void 
		{
			if(writingBitmapDataErrorOccured)
			{
				return;
			}
			
			try
			{			
				fileStream.open(file, FileMode.WRITE);
				fileStream.writeBytes(byteArray, 0, byteArray.length);
				fileStream.close();

			}
			catch(error : Error) {
			
				Tracer.print("saveImageBytes error:"  + error.message)

				if(!writingBitmapDataErrorOccured)
				{
					if(errorHandler)
					{
						errorHandler();
					}
					else 
					{
						globalWritingErrorHandler();
					}
				}
				
				try{
					fileStream.close();
				}catch(e:Error){}
				
				writingBitmapDataErrorOccured = true;		
			}
		}
		


		private static const md5Hash:MD5 = new MD5();
		private static const IVArray:Array = [166, 209, 127, 210, 125, 48, 68, 238, 228, 156, 81, 126, 64, 67, 190, 9, 86, 87, 23, 91, 196, 85, 224, 58, 136, 62, 104, 19, 250, 199, 103, 202];
		private static const ba : ByteArray = new ByteArray();
		
		public static function decryptImageBytes(byteArray : ByteArray, url : String, __filename : String = null) : void
		{
			
			var fileName:String = null;
			if(__filename){
				fileName = __filename;//.toString();//stver0016_-2_277.png
				//fileName = "stver0016_-2_277.png"
			}else{
				var index:int = url.lastIndexOf("/")+ 1;
				fileName = url;
				if (index > 0)
					fileName = url.substr(index);
			}
			
			
			if(fileName){
				
				fileName=fileName.replace("a10", "a00");
				fileName=fileName.replace("a20", "a00");
				fileName=fileName.replace("a30", "a00");
			}
			
			
			ba.clear();
			ba.writeUTFBytes("Oqw#90" + fileName + "9oJ#Tl");
			var keyHash:ByteArray = md5Hash.computeHash(ba);
			
			var keyArray:Array= [40, 157, 109, 167, 208, 203, 150, 8, 136, 161, 220, 157, 42, 154, 243, 102];
			
			for (var i:int = 0; i < 16; i++) {
				keyArray.push(keyHash[i]);
			}
			
			for (i = 0; i < byteArray.length; i++) {
				byteArray[i] ^= keyArray[i%32];
				byteArray[i] ^= IVArray[i%32];
			}
		}
		

		public static function scaledBitmapData(bitmapData : BitmapData, factor : Number) : BitmapData
		{
			var matrix : Matrix = new Matrix();
			matrix.scale(factor, factor);
			
			var resultBitmapData : BitmapData = new BitmapData(bitmapData.width * factor, bitmapData.height * factor);
			resultBitmapData.draw(bitmapData, matrix, null, null, null, true);
			
			return resultBitmapData;
		}
		
	}
}
