package com.despark.tuning3d.model.friend
{
import by.blooddy.crypto.serialization.JSONer;

import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.utils.DevHelper;

import flash.filesystem.File;
import flash.filesystem.FileMode;
import flash.filesystem.FileStream;
import flash.net.URLRequestMethod;

import org.puremvc.as3.patterns.proxy.Proxy;

public class FriendProxy extends Proxy {
    public static const NAME:String = "FriendProxy";
    public var httpClient:HttpClient;
    private var youFriends:Object = {};
    private var youFriendsFile:File;
    private var viewedFriendRequests:File;
    private var userProxy:UsersProxy;
    private static const TESTING : Boolean = false;
    public function FriendProxy(){
        super(NAME);
        Friend.PROXY = this;
        youFriendsFile = File.applicationStorageDirectory.resolvePath("youFriends.json");
        viewedFriendRequests = File.applicationStorageDirectory.resolvePath("viewedRequests.json");
        loadFriendsFromFile();
        userProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
    }

    public static function filePutContent(file:File, content:String):void {
        var fileStream:FileStream = new FileStream();
        fileStream.open(file, FileMode.WRITE);
        fileStream.writeUTFBytes(content);
        fileStream.close();
    }

    public static function fileGetContent(file:File):String {
        var fileStream:FileStream = new FileStream();
        fileStream.open(file, FileMode.READ);
        var yourJSONdata:String = fileStream.readUTFBytes(fileStream.bytesAvailable);
        fileStream.close();
        return yourJSONdata;
    }

    private function loadFriendsFromFile():void {
        if (youFriendsFile.exists) {
            youFriends = JSONer.parse(fileGetContent(youFriendsFile));
        } else {
            youFriends = {};
        }
    }


    public function add_friend(friend:Friend):void
    {
        DevHelper._log("add_friend(" + friend.user.id + ")");
        if (TESTING) {
            Constants.CURRENT_SENT_FRIENDS.push(friend);
            sendNotification(Notifications.FRIENDS_REQUEST_SEND , {friend : friend});
            friend.notifyHolder(Friend.STATE_REQUEST_SENT);

            return;
        }

        var url:String = "http://friends.3dtuning.com/api/v1/requests/" + friend.user.id;
        httpClient.loadPath("", URLRequestMethod.POST, {type: "friendship"}, HttpCachePolicy.NONE, true, function (success:Boolean, result:Object):void {
            if (success) {
                Constants.CURRENT_SENT_FRIENDS.push(friend);
                friend.notifyHolder(Friend.STATE_REQUEST_SENT);
                sendNotification(Notifications.FRIENDS_REQUEST_SEND, {friend : friend});
            }
            else {
                sendNotification(Notifications.FRIENDS_REQUEST_SEND_FAILED);
            }
        }, url);
    }

    public function undo_friend(friend:Friend):void
    {
        sendNotification(Notifications.UNDO_FRIENDS_REQUEST);
        friend.notifyHolder(Friend.STATE_NO_FRIEND);
        /*httpClient.loadPath("",URLRequestMethod.POST,{type:"friendship"},HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void {
            success ? sendNotification(Notifications.UNDO_FRIENDS_REQUEST,{item:item}) : sendNotification(Notifications.UNDO_FRIENDS_REQUEST_FAILED);},
                "http://friends.3dtuning.com/api/v1/requests/"+ id);*/
    }

    public function approveRequest(friend:Friend):void
    {
        DevHelper._log("approveRequest(" + friend.user.id + ")");
        if (TESTING) {
            friend.notifyHolder(Friend.STATE_FRIEND);
            sendNotification(Notifications.CONFIRM_FRIENDS_REQUEST,{friend : friend});
            return;
        }

        var url:String = "http://friends.3dtuning.com/api/v1/requests/" + friend.requestId;
        httpClient.loadPath("", URLRequestMethod.PUT, {
            status: "approved",
            type: 'friendship'
        }, HttpCachePolicy.NONE, true, function (success:Boolean, result:Object):void {
            if (success) {
                friend.notifyHolder(Friend.STATE_FRIEND);
                sendNotification(Notifications.CONFIRM_FRIENDS_REQUEST,{friend : friend});
            }
            else {
                sendNotification(Notifications.CONFIRM_FRIENDS_REQUEST_FAILED);
            }
        }, url);
    }

    public function setFriendRequestRead(friends:Array):void{
        var content:String;
        if(viewedFriendRequests.exists) {
            content = fileGetContent(viewedFriendRequests);
        }
        var friendIdList:Object = content?JSON.parse(content):{};
        var changed:Boolean = false;
        for(var i:int = 0; i < friends.length; i++){
            if(!friendIdList[Friend(friends[i]).user.id]) {
                friendIdList[Friend(friends[i]).user.id] = true;
                changed = true;
            }
        }
        if(changed) {
            var jsonData:String = JSON.stringify(friendIdList);
            filePutContent(viewedFriendRequests, jsonData);
        }
        sendNotification(Notifications.PENDING_FRIENDS_LOADED, {unread:false});
    }

    public function unreadFriendRequestExists(friends:Array):Boolean{
        var content:String;
        if(viewedFriendRequests.exists) {
            content = fileGetContent(viewedFriendRequests);
        }else if(friends.length){
            return true;
        }
        if(!content){
            return false;
        }
        var requestList:Object = JSON.parse(content);
        for(var i:int = 0; i < friends.length; i++){
            if(!requestList.hasOwnProperty(Friend(friends[i]).user.id)){
                return true;
            }
        }
        return false;
    }



    public function rejectRequest(friend: Friend):void
    {
        DevHelper._log("rejectRequest(" + friend.requestId + ")");
        if (TESTING) {
            sendNotification(Notifications.FRIENDS_REQUEST_DENIED,{friend : friend});
            friend.notifyHolder(Friend.STATE_NO_FRIEND);
            return;
        }

        var url:String = "http://friends.3dtuning.com/api/v1/requests/" + friend.requestId;
        httpClient.loadPath("", URLRequestMethod.PUT, {
            status: "rejected",
            type: 'friendship'
        }, HttpCachePolicy.NONE, true, function (success:Boolean, result:Object):void {
            if (success) {
                sendNotification(Notifications.FRIENDS_REQUEST_DENIED,{friend : friend});
                friend.notifyHolder(Friend.STATE_NO_FRIEND);
            }
            else {
                sendNotification(Notifications.FRIENDS_REQUEST_DENIED_FAILED);
            }
        }, url);
    }

    public function delete_friend(friend: Friend):void
    {
        DevHelper._log("delete_friend(" + friend.user.id + ")");
        if (TESTING) {
            sendNotification(Notifications.DELETE_FRIENDS_REQUEST_SEND, {friend : friend});
            friend.notifyHolder(Friend.STATE_NO_FRIEND);
            return;
        }

        var url:String = "http://friends.3dtuning.com/api/v1/friends/" + friend.user.id;
        httpClient.loadPath("", URLRequestMethod.DELETE, null, HttpCachePolicy.NONE, true, function (success:Boolean, result:Object):void {
            if (success) {
                friend.notifyHolder(Friend.STATE_NO_FRIEND);
                sendNotification(Notifications.DELETE_FRIENDS_REQUEST_SEND, {friend : friend});
            }
            else {
                sendNotification(Notifications.DELETE_FRIENDS_REQUEST_SEND_FAILED);
            }
        }, url);
    }

    public static function isUserExistInArray(array : Array, id : String) : Boolean
    {
        for(var i : int = 0; i < array.length; i++) {
            if(array[i].user.id == id) {
                return true;
            }
        }
        return false;
    }

    /**
     *
     */
    public function getUserFriendStatus(friend:Friend):void
    {
        var url : String = "http://friends.3dtuning.com/api/v1/requests/userStatus/" + friend.user.id;


        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void {
            if (success) {
                var json : Object = JSONer.parse(result);
                if(json.friend){
                    friend.notifyHolder(Friend.STATE_FRIEND);
                }
                else if(json.requestSend){
                    friend.notifyHolder(Friend.STATE_REQUEST_SENT);
                }
                else if(json.pending){
                    friend.notifyHolder(Friend.STATE_PENDING);
                }
                else {
                    friend.notifyHolder(Friend.STATE_NO_FRIEND);
                }
            }
        }, url);
    }

    /**
     * Loads current user friends
     */
    public function loadFriends(user:User, skip:int = 0, limit:int = 10):void
    {
        var url : String = "http://friends.3dtuning.com/api/v1/friends";

        var params : Object = {};
        params.limit = limit;
        params.skip  = skip;
        params.user  = user.id;

        httpClient.loadPath("", URLRequestMethod.GET, params, HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void
        {
            if(success)
            {
                var friends : Array = [];
                var friendsJSON : Object = JSONer.parse(String(result));

                var jsonCollection : Object;
                jsonCollection = friendsJSON.friends;

                var friend  : Friend;
                for each(var jsn : Object in jsonCollection)
                {
                    friend = new Friend(Friend.STATE_FRIEND, jsn);
                    friends.push(friend);
                }

                Constants.CURRENT_USER_FRIENDS = friends;
                sendNotification(Notifications.FRIENDS_LOADED, {friends: friends, total:friendsJSON.total});
            }
            else
            {
                sendNotification(Notifications.FRIENDS_LOADING_FAILED);
            }
        },url);
    }

//    public function loadSentFriends(user : User) : void
//    {
//        var url : String = "http://friends.3dtuning.com/api/v1/requests/" + user.id;
//        var params : Object = {};
//        params.type = "friendship";
//
//        httpClient.loadPath("",URLRequestMethod.GET,params,HttpCachePolicy.NONE,true,function(success:Boolean,result:Object) : void
//        {
//            if(success)
//            {
//                var friend  : Friend;
//                var jsn : Object = JSONer.parse(String(result));
//
//                if(jsn.status == "none")
//                {
//                    sendNotification(Notifications.CHECK_SENT_FRIENDS, {friends: null});
//                    return;
//                }
//                friend = new Friend(Friend.STATE_REQUEST_SENT, jsn);
//                friend.state = Friend.STATE_REQUEST_SENT;
//                friend.requestId = jsn._id;
//
//                Constants.CURRENT_SENT_FRIENDS.push(friend);
//
//                sendNotification(Notifications.CHECK_SENT_FRIENDS, {friends: friend});
//            }
//        },url);
//    }

    public function loadPendings() : void
    {
        var url : String = "http://friends.3dtuning.com/api/v1/requests";
        var params : Object = {};
        params.type = "friendship";

        httpClient.loadPath("",URLRequestMethod.GET,params,HttpCachePolicy.NONE,true,function(success:Boolean,result:Object) : void
        {
            if(success){
                var friends : Array = [];
                var friend  : Friend;

                var parsedObj : Object = JSONer.parse(String(result));

                for each(var jsn : Object in parsedObj){
                    friend = new Friend(Friend.STATE_PENDING, jsn);
                    friend.requestId = jsn._id;
                    friends.push(friend);
                }
                Constants.CURRENT_PENDING_FRIENDS = friends;
                sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: friends, unread:unreadFriendRequestExists(friends)});
            }else{
                sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
            }
        },url);
    }



    public function search(key : String, skip:int, sent : Array, pending : Array, completFunction:Function, limit:int = 15) : void{
        var url : String = HttpClient.API_DOMAIN + "/api/v1/users/public";

        var params:Object = {
            limit:limit,
            q:encodeURI(key),
            skip:skip
        };

        httpClient.loadPath("", URLRequestMethod.GET,params, HttpCachePolicy.NONE, true,
                function (success:Boolean, result:String) : void {
            if(success){
                var items : Array = [];
                var friend  : Friend;
                var resultObj : Object = JSONer.parse(result);
                var state : String;


                for each (var js : Object in resultObj.users){

                    state = Friend.STATE_NO_FRIEND;

                    if(js.hasOwnProperty('isFriend') && js['isFriend']){
                        state = Friend.STATE_FRIEND;
                    }

                    if(sent && isUserExistInArray(sent,js._id)){
                        state = Friend.STATE_REQUEST_SENT;
                    }

                    if(pending && isUserExistInArray(pending,js._id)){
                        state = Friend.STATE_PENDING;
                    }

                    friend = new Friend(state, js);
                    items.push(friend);
                }
                completFunction(items);
            } else {
                trace(success);

            }
        }, url);


    }
}
}