package com.despark.tuning3d.model.parts
{
	public class PartMaterial extends Object
	{
		
		public static var partMaterialImageMem:Array = [];
		private static var partMaterialImagePos:int = -1;
		private static var partMaterialImageStatic:PartMaterialImage;
		private static const LABEL_LIST:Object = {
			'mate':'Matte',
            'metal':'Factory'
		};

		public static const FIXED_MATERIAL_TYPES:Array = ['metal', 'glossy', 'white', 'mate', 'lines', 'space', 'chip', 'snake', 'skulls', 'rust', 'redblack', 'gold', 'silver', 'chameleon', 'chrome'];
		public static const UNPAINTABLE_MATERIAL_TYPES:Array = ['metal', 'gold', 'white', 'silver', 'camo', "camo2", "carbon", "carbon2", "digital", "digital2", "snow", "snow2", "snake", "rust"];
		public static const COLORABLE_MATERIAL_TYPES:Array = ["glossy", "mate", "redblack", "chameleon", "chrome", "lines", "space", "chip", "skulls"];
        public static function InitPartMaterialImageMem(count:int):void{
			var i:int=0;
			for(i=0; i<count; i++){
				partMaterialImageMem.push(PartMaterialImagePool.getInstance(null,null,null,null));
			}
			partMaterialImagePos=0;
		}

		public function get label():String{
			if(name){
				if(LABEL_LIST.hasOwnProperty(name)) {
                    return LABEL_LIST[name];
                }
				return name.charAt(0).toUpperCase() + name.substr(1);
			}
			return null;
		}

		public static function isColorable(str:String):Boolean{
			return COLORABLE_MATERIAL_TYPES.indexOf(str) != -1;
		}
		/*
		public static function isColorable(str:String):Boolean{
			return COLORABLE_MATERIAL_TYPES.indexOf(str) != -1;
		}*/
		
		public static function ResetPartMaterialImageMem():void{
			partMaterialImagePos = 0;
		}
		
		public static function GetPartMaterialImage():PartMaterialImage{
			
			if(partMaterialImagePos<partMaterialImageMem.length) {
				partMaterialImageStatic = partMaterialImageMem[partMaterialImagePos];
				partMaterialImagePos++;
				return partMaterialImageStatic;
			}
			else {
				partMaterialImageStatic = PartMaterialImagePool.getInstance(null,null,null,null);
				partMaterialImageMem.push(partMaterialImageStatic);
				partMaterialImagePos++;
				return partMaterialImageStatic;
			}
		}
		
		
		public function PartMaterial(part:Part)
		{
			super();
			
			parentPart=part;
			_imagesMap = {};
		}



		public var _name : String;
		public var isPaintable : Boolean;
		public var parentPart : Part;
		private var _imagesMap :Object;
		public var componentPath:String;
		
		public var isPartMaterialImagesLoaded:Boolean = false;
		
		public function dispose():void{
			var cameraIndex:int;
			var frameIndex:int;
			var partMaterialImage:PartMaterialImage;
			for (cameraIndex in _imagesMap){
				
				for (frameIndex in _imagesMap[cameraIndex]){
					
					partMaterialImage=_imagesMap[cameraIndex][frameIndex];
					if(partMaterialImage){
						partMaterialImage.dispose();
						partMaterialImage=null;
					}
					delete _imagesMap[cameraIndex][frameIndex];
				}
				delete _imagesMap[cameraIndex];
			}
			_imagesMap = null;
			componentPath = null;
			parentPart=null;
			_name=null;
		}
		
		public function set name(__value: String) : void {
			_name = __value;
            isPaintable = PartMaterial.isColorable(_name);
		}
		
		public function get name() : String
		{
			return _name;
		}
		
		public static function valueInObject(value:int, object:Object):Boolean{
			var item_key:*;
			for(item_key in object){
				if(object[item_key]==value){
					return true;
				}
			}
			return false;
		}
		
		private static var filePattern:RegExp = /^a(\d{1})0(\d{2})_(-?\d{1,4})_(-?\d{1,4})\.png$/;
		private static var filePatternResult:Array;
		
		private static var angleIndex:int;
		private static var frameIndex:int;
		private var partMaterialImage:PartMaterialImage;
		
		public function addImage(url : String, stepOfFrameIndex : int, shadowOffset:Number=0, partSlot:PartSlot = null) :Boolean
		{

			filePatternResult = filePattern.exec(url);
			angleIndex = parseInt(filePatternResult[1]);
			frameIndex =  parseInt(filePatternResult[2]);
			
			
			if(angleIndex==0 /*&& frameIndex%stepOfFrameIndex != 0*/){
				if( parentPart.parentSlot.parentCar.camera1Frames){
					if( !valueInObject(frameIndex,  parentPart.parentSlot.parentCar.camera1Frames))
						return false;
				}else if(frameIndex%stepOfFrameIndex != 0) return false;
				
			}else {
				if(parentPart.parentSlot.parentCar.camera2Frames){
					
					if( !valueInObject(frameIndex,  parentPart.parentSlot.parentCar.camera2Frames))
						return false;
					
				}else if( frameIndex%parentPart.parentSlot.parentCar.stepOfFrameIndexUp != 0) return false;
				
				
				if(angleIndex+1>parentPart.parentSlot.parentCar.countAngels) parentPart.parentSlot.parentCar.countAngels=angleIndex+1;
			}
			
			
			
			partMaterialImage=PartMaterialImagePool.getInstance(null,null, null, this);
			//partMaterialImage.isEncrypted=isEncrypted;
			partMaterialImage.parentMaterial = this;
			partMaterialImage.x=parseInt(filePatternResult[3])+shadowOffset;
			partMaterialImage.y=parseInt(filePatternResult[4]);
			partMaterialImage.frameIndex = frameIndex;
			partMaterialImage.angleIndex = angleIndex;
			
			if(!_imagesMap[angleIndex]) _imagesMap[angleIndex]={};
				
			_imagesMap[angleIndex][frameIndex] = partMaterialImage;
			return true;
		}
		public function getImage( frameIndex : int, angleIndex:int=0) : PartMaterialImage
		{
			if(_imagesMap[angleIndex] && _imagesMap[angleIndex][frameIndex]) return _imagesMap[angleIndex][frameIndex];
			else return null;

		}

        public function get imagesMap():Object {
            return _imagesMap;
        }

        public function set imagesMap(value:Object):void {
			if (value == {}) {
				trace('SetEmpty');
			}
            _imagesMap = value;
        }
	}
}


