package com.despark.tuning3d.model.sounds
{
	import com.despark.tuning3d.model.settings.Settings;

	import flash.filesystem.File;
	import flash.media.AudioPlaybackMode;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	import flash.net.URLRequest;

	public class SoundManager {
		public static const DISABLED_PART:String = "disable_part.mp3";
		public static const MENU:String = "menu.mp3";
		public static const MENU_CLOSE:String = "menu_close.mp3";
		public static const PART_MENU:String = "parts_menu.mp3";
		public static const PART_MENU_CLOSE:String = "parts_menu_close.mp3";
		public static const LOADING:String = "loading.mp3";
		public static const PAINTING:String = "painting.mp3";
		public static const SAVE_AND_NOTIFICATION:String = "save_and_notification.mp3";
		public static const SELECT_PART:String = "select_parts.mp3";
		public static const SELECT_PART_BACK:String = "select_parts_back.mp3";
		public static const THEME:String = "theme.mp3";
		//
		private static const soundList:Object = {};
		private static const soundChannelList:Object = {};
		public static var isMute:Boolean = false;
		public function SoundManager() {

		}

		public static function init():void{
			SoundMixer.audioPlaybackMode = AudioPlaybackMode.AMBIENT;
			var soundOn:Boolean = Settings.soundOn;
			isMute = !soundOn;
		}

		private static function playSoundFile(soundId:String, loops:int = 0, position:Number = 0, volume:Number = 1):SoundChannel{
			if(isMute){
				return null;
			}
			var soundTransform:SoundTransform;
			if(volume != 1){
				soundTransform = new SoundTransform(volume);
			}
			if(soundList[soundId]){
				return Sound(soundList[soundId]).play(position, loops, soundTransform);
			}
			var sound:Sound = new Sound();
			soundList[soundId] = sound;
			var file:File = getSoundFile(soundId);
			sound.load(new URLRequest(file.url));
			return sound.play(position, loops, soundTransform);
		}

		public static function playSound(soundId:String, loops:int = 0, saveChannel:Boolean = false, volume:Number = 1):SoundChannel{
			if(saveChannel){
				var soundChannel:SoundChannel = playSoundFile(soundId, loops, 0, volume);
				if(soundChannel){
					var channelData:SoundChannelData = soundChannelList[soundId] as SoundChannelData||new SoundChannelData();
					channelData.loops = loops;
					channelData.volume = volume;
					channelData.clear();
					channelData.soundChannel = soundChannel;
					soundChannelList[soundId] = channelData;
				}
				return soundChannel;
			}
			return playSoundFile(soundId, loops, 0, volume);
		}

		public static function pauseChannel(soundId:String):void{
			var channelData:SoundChannelData = soundChannelList[soundId] as SoundChannelData;
			if(channelData){
				channelData.pause();
			}
		}

		public static function resumeChannel(soundId:String):void{
			var channelData:SoundChannelData = soundChannelList[soundId] as SoundChannelData;
			if(channelData){
				channelData.clear();
				channelData.soundChannel = playSoundFile(soundId, 0, channelData.position, channelData.volume);
				if(channelData.loops && channelData.position){
					channelData.loops--;
					channelData.position = 0;
					channelData.completeCallback = function():void{
						channelData.completeCallback = null;
						channelData.soundChannel = playSoundFile(soundId, channelData.loops, 0, channelData.volume);
					}
				}
			}
		}

		private static function getSoundFile(soundName : String) : File {
			var appDirectory : File = File.applicationDirectory;
			var path : File;
            path = appDirectory.resolvePath("assets/sounds/" + soundName);
			path.preventBackup = true;
			return path;
		}

		public static function soundMute(value:Boolean):void {
			if(value){
				SoundMixer.stopAll();
			}
			// AdsManager.$.turnOffSound(value);
			isMute = value;
			Settings.soundOn = !value;
		}
	}
}

import flash.events.Event;
import flash.media.SoundChannel;

class SoundChannelData {
	public var position:Number = 0;
	public var volume:Number = 1;
	public var loops:int = 0;
	private var _soundChannel:SoundChannel;
	private var _completeCallback:Function;
	public function SoundChannelData(soundChannel:SoundChannel = null, loops:int = 0){
		this.loops = loops;
		this.soundChannel = soundChannel;
	}

	private function onSoundComplete(event:Event):void {
		clear();
		if(_completeCallback != null){
			_completeCallback();
		}
	}

	public function clear():void{
		if(_soundChannel){
			_soundChannel.removeEventListener(Event.SOUND_COMPLETE, onSoundComplete);
			_soundChannel.stop();
			_soundChannel = null;
		}
	}

	public function pause():void {
		if(_soundChannel){
			position = _soundChannel.position;
			_soundChannel.stop();
		}
	}

	public function set soundChannel(value:SoundChannel):void {
		clear();
		_soundChannel = value;
		if(_soundChannel && loops != 0){
			_soundChannel.addEventListener(Event.SOUND_COMPLETE, onSoundComplete);
		}
	}

	public function set completeCallback(value:Function):void {
		_completeCallback = value;
	}
}