/**
 * Created by Render-2 on 8/8/2017.
 */
package com.despark.tuning3d.model.timeline {

import by.blooddy.crypto.serialization.JSONer;

import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.feed.ChallengeNotificationData;
    import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.NotificationView.NotificationFeedData;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.utils.Console;
    import com.utils.ObjectUtils;

import flash.net.URLRequestMethod;

import org.puremvc.as3.patterns.proxy.Proxy;

public class TimelineProxy extends Proxy{

    public static const NAME : String = "TimelineProxy";

    public static const ITEMS_TO_EXCLUDED : Array = [/*FeedData.TYPE_CHALLENGE_ANSWER, */FeedData.TYPE_CHALLENGE_INVITE/*, FeedData.TYPE_CHALLENGE_WIN*/];
    public static const NOTIFICATIONS_TYPES : Array = [FeedData.TYPE_NEW_FRIEND, FeedData.TYPE_TUNING_LIKE, FeedData.TYPE_TUNING_COMMENT, NotificationFeedData.TYPE_POST_COMMENT, NotificationFeedData.TYPE_POST_LIKE];

    public var lastUpdatedDate : Date;
    public var httpClient : HttpClient;
    private var _loadedFeed : int = 0;
    private var _loadedNotificationsCount : int = 0;
    private var _tracer : Tracer;
    public static var timelineFilters:Object = {};
    public function TimelineProxy() {
        super(NAME);
        FeedData.Proxy = this;
        _tracer = new Tracer("TimelineProxy");

    }

    public function deleteCommentFromPost(commentId : String) : void
    {
        var url : String = "http://feed-staging.api.3dtuning.com/api/v1/feed/comments/"+commentId;
        httpClient.deletePath("", null, function (success : Boolean, result : Object) : void {
            if(success) {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_DELETED, json)
            }

        }, url)
    }

    public function deletCommentFromTuning(tuningId : String, commentId: String) : void {
        var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+ tuningId + "/comments/"+commentId;
        httpClient.deletePath("", null, function (success : Boolean, result : Object) : void {
            if(success)
            {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_DELETED, json)
            }

        }, url)
    }

    public function editTuningComment(tuningId : String, text : String, commentId : String ) : void
    {
        var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+ tuningId + "/comments/"+ commentId;
        httpClient.putPath("", {comment:text}, function (success : Boolean, result : Object) : void {
            if(success)
            {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_EDITED, json)
            }

        }, url)
    }

    public function addCommentToTuning(tuningId : String, text: String) : void
    {

        var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+ tuningId + "/comments?activation=true";
        httpClient.postPath("", {comment:text}, function (success : Boolean, result : Object) : void {
            if(success) {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_ADDED, json)
            }else{
                trace('comment not added');
                sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'comment', stopPropagation:true});
            }

        }, url)


    }

    public function editPostComment(postId : String, text : String, commentId : String) : void
    {
        httpClient.putPath("feed/" + postId+ "/comments/"+commentId, {comment:text}, function (success : Boolean, result : Object) : void {
            if(success)
            {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_EDITED, json)
            }
        })
    }
    public function addCommentToPost(postId : String, text: String) : void
    {
        httpClient.postPath("feed/" + postId+ "/comments", {comment:text}, function (success : Boolean, result : Object) : void {
            if(success)
            {
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_COMMENT_ADDED, json)
            }
        });
    }

    public function getMoreCommentsTuning(tuningId : String, skip : int, limit : int) : void {
        var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+ tuningId + "/comments";
        httpClient.getPath(url,
                {
                    skip:skip,
                    limit:limit,
                    order:1
                }, function(success : Boolean, result : Object) : void {

                    if (success) {
                        sendNotification(Notifications.TIMELINE_MORE_COMMENTS, {isTuning: true, data: JSON.parse(result as String)});
                    }
                    else {
                        //_tracer.warn("failed to load comments: " + httpClient.lastErrorStatusCode);
                    }
                })

    }

    public function getCountryApi(successCallback:Function) : void {
        var url : String = "http://feed-staging.api.3dtuning.com/api/v1/usergeo";
        httpClient.getPath(url, null, function(success : Boolean, result : Object) : void {
                    if (success) {
                        result = JSON.parse(result as String);
                        successCallback(result);
                    } else {

                    }
                })

    }

    public function getMoreCommentsPost(postId : String, skip : int, limit : int) : void
    {
        var url : String = "feed/" + postId+ "/comments";
        httpClient.getPath(url,
                {
                    skip:skip,
                    limit:limit,
                    order:1
                }, function(success : Boolean, result : Object) : void {

                    if (success) {
                        sendNotification(Notifications.TIMELINE_MORE_COMMENTS, {isTuning: false, data: JSON.parse(result as String)});
                    }
                    else {
                        //_tracer.warn("failed to load comments: " + httpClient.lastErrorStatusCode);
                    }
                })
    }

    public function updateRefreshDate() : void
    {
        lastUpdatedDate = new Date();
    }

    public function refresh(user : User, time: String = null, firstPost:String = null, currentTimeline:Boolean = true) : void {
        var data:Object = {
            to:time,
            limit: 100,
            skip: 0
        };
        ObjectUtils.mergeInto(TimelineScreen.feedFilter, data);
        if(firstPost){
            data.firstPost = firstPost;
        }
        if(!currentTimeline){
            data.withFriends = data.country = data.fullWorld = data.withOfficial = false;
            data.withMy = true;
        }else if(data.onlyTop){
            data.withMy = false;
        }else if(data.withOfficial){
            data.withFriends = data.country = data.fullWorld = data.withMy = false;
        }
        httpClient.getPath(
                "feed/" + user.id,
                data,
                function(success : Boolean, result : Object) : void {
                    if (success) {
                        var timelineJSON : Object = JSONer.parse(String(result));
                        var feed : Array=[];
                        var pendingFeed:Object = {};
                        var i:int;
                        var loadAsyncTunings:Boolean = false;
                        var failed:Array = [];
                        var tuningReadyHandler:Function = function (tuning:Tuning, json:Object = null, id:String = null):void {
                            if(json && pendingFeed[json.id]){
                                pendingFeed[json.id].tuning = json;
                                delete pendingFeed[json.id];
                            }else if(id && pendingFeed[id]){
                                failed.push(id);
                                delete pendingFeed[id];
                            }
                            if(ObjectUtils.isEmpty(pendingFeed)){
                                pendingFeed = null;
                                for(i = 0 ; i  < timelineJSON.length; i++){
                                    var loaded:Boolean = (!timelineJSON[i].tuning || (timelineJSON[i].tuning && timelineJSON[i].tuning.car));
                                    if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 && loaded){
                                        var item:FeedData = new FeedData(timelineJSON[i]);
                                        feed.push(item);
                                    }
                                    _loadedFeed++;
                                }
                                sendNotification(Notifications.TIMELINE_DATA_REFRESHED, {feed: feed});
                            }
                        };
                        for(i = 0 ; i  < timelineJSON.length; i++){
                            if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                if(timelineJSON[i].hasOwnProperty("tuning") && !timelineJSON[i].tuning.hasOwnProperty("car")){
                                    loadAsyncTunings = true;
                                    break;
                                }
                            }
                        }
                        if(loadAsyncTunings){
                            for(i = 0 ; i  < timelineJSON.length; i++){
                                if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                    if(timelineJSON[i].hasOwnProperty("tuning") && !timelineJSON[i].tuning.hasOwnProperty("car")){
                                        pendingFeed[timelineJSON[i].tuning._id] = timelineJSON[i];
                                        (ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy).getTuning(timelineJSON[i].tuning._id, tuningReadyHandler);
                                    }
                                }
                            }
                            return;
                        }
                        for(i = 0 ; i  < timelineJSON.length; i++){
                            if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                var item : FeedData = new FeedData(timelineJSON[i]);
                                feed.push(item);
                            }
                            _loadedFeed++;
                        }
                        if(ObjectUtils.isEmpty(pendingFeed)){
                            pendingFeed = null;
                        }
                        if(!pendingFeed) {
                            sendNotification(Notifications.TIMELINE_DATA_REFRESHED, {feed: feed});
                        }
                    }
                    else {
                        sendNotification(Notifications.TIMELINE_DATA_LOADING_FAILED, httpClient.lastErrorStatusCode);
                    }
                }
        );
    }

    public function getComments(isTuning : Boolean, id : String, skip : int, limit : int, callBackFunction : Function) : void{
        var url : String;

        if(isTuning){
            url = "http://ratings.3dtuning.com/api/v1/tunings/" + id + "/comments?";
        }else {
            url = "feed/" + id + "/comments?";
        }

//        if(skip == 0){
            url += "widthcount=true&";
//        }

        url += "skip=" + skip +"&limit=" + limit;

        httpClient.getPath(
                url,
                null,
                function (status : Boolean, result : String) : void{
                    if(status){
                        var obj : Object = JSONer.parse(result);
                        callBackFunction(obj);
                    }else {
                        callBackFunction(null);
                    }
                }
        )
    }

    public function getLikes(isTuning : Boolean, id : String, callBackFunction : Function) : void{
        var url : String = "" + ((isTuning) ? "feedtuning/" : "feed/") + id + "/likes";

        httpClient.getPath(
            url,
            null,
            function (status : Boolean, result : String) : void{
                if(status){
                    var obj : Object = JSONer.parse(result);
                    callBackFunction(obj);
                }else {
                    callBackFunction(null);
                }
            }
        )
    }

    public function likeFeed(id : String, isTuning : Boolean, iLiked : Boolean) : void
    {
        var url : String;
        var likeType : String = iLiked ? "/unlike" : "/like";

        if(isTuning){
            url = "http://ratings.3dtuning.com/api/v1/tunings/" + id + "/ratings?activation=true";
        }else {
            url = "feed/"+ id + likeType;
        }

        var completeFunction : Function = function(success : Boolean, result : Object) : void {
            if(success) {
                sendNotification(Notifications.TIMELINE_LIKE_SUCCESS);
            }else {
                sendNotification(Notifications.TIMELINE_LIKE_FAILED);
                sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'like', stopPropagation:true});
            }
        };

        if(!iLiked)
        {
            httpClient.postPath(url,null, completeFunction);
        }
        else {
            httpClient.deletePath(url,null, completeFunction);
        }
    }

    public function makeFullNotificationsRead() : void{
        httpClient.loadPath('', URLRequestMethod.PUT, null, HttpCachePolicy.NONE, false, function (success:Boolean, result:Object):void{
            if(success){
                Constants.ALL_UNREAD_NOTIFICATION_NUM = 0;
                sendNotification(Notifications.UPDATE_NOTIFICATION_COUNTS);
            }
        }, 'http://mailer.3dtuning.com/api/v1/notifications/?type=full');
    }

    public function loadFullUnreadNotificationCount(completeFunction:Function) : void{
        httpClient.loadPath('', URLRequestMethod.GET, null, HttpCachePolicy.NONE,false, function (success:Boolean, result:Object):void{
            if(success){
                var obj : Object = JSONer.parse(result);

                Constants.ALL_UNREAD_NOTIFICATION_NUM = obj.count;
                completeFunction(obj);
            }else{
                completeFunction(null);
            }
        }, 'http://mailer.3dtuning.com/api/v1/notifications?typeGroup=full&count=true');
    }

    public function loadFeed(user : User, skip : int = 0, limit : int = 10, fromDate: String = null, firstPost:String = null, reload:Boolean = false, requestId:String = null, currentTimeline:Boolean = true) : void {
        _loadedFeed = skip;
        var params : Object = {
            limit: limit,
            skip: skip
        };
        ObjectUtils.mergeInto(TimelineScreen.feedFilter, params);
        if (fromDate) {
            params.from = fromDate;
        }
        if(firstPost){
            params.firstPost = firstPost;
        }
        if(!currentTimeline){
            params.withFriends = params.country = params.fullWorld = params.withOfficial = false;
            params.withMy = true;
        }else if(params.onlyTop){
            params.withMy = false;
        }else if(params.withOfficial){
            params.withFriends = params.country = params.fullWorld = params.withMy = false;
        }
        httpClient.getPath(
                "feed/" + user.id,
                params,
                function(success : Boolean, result : Object) : void {
                    if (success) {
                        var timelineJSON : Object = JSONer.parse(String(result));

                        var feed : Array=[];
                        var pendingFeed:Object = {};
                        var i:int;
                        var loadAsyncTunings:Boolean = false;
                        var failed:Array = [];
                        var tuningReadyHandler:Function = function (tuning:Tuning, json:Object = null, id:String = null):void {
                            if(json && pendingFeed[json.id]){
                                pendingFeed[json.id].tuning = json;
                                delete pendingFeed[json.id];
                            }else if(id && pendingFeed[id]){
                                failed.push(id);
                                delete pendingFeed[id];
                            }
                            if(ObjectUtils.isEmpty(pendingFeed)){
                                pendingFeed = null;
                                for(i = 0 ; i  < timelineJSON.length; i++){
                                    var loaded:Boolean = (!timelineJSON[i].tuning || (timelineJSON[i].tuning && timelineJSON[i].tuning.hasOwnProperty("car")));
                                    if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 && loaded){
                                        var item : FeedData = new FeedData(timelineJSON[i]);
                                        feed.push(item);
                                    }
                                    _loadedFeed++;
                                }
                                sendNotification(Notifications.TIMELINE_DATA_LOADED, {feed: feed, reload:reload, requestId:requestId});
                            }
                        };

                        for(i = 0 ; i  < timelineJSON.length; i++){
                            if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                if(timelineJSON[i].hasOwnProperty("tuning") && !timelineJSON[i].tuning.hasOwnProperty("car")){
                                    loadAsyncTunings = true;
                                    break;
                                }
                            }
                        }
                        if(loadAsyncTunings){
                            for(i = 0 ; i  < timelineJSON.length; i++){
                                if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                    if(timelineJSON[i].hasOwnProperty("tuning") && !timelineJSON[i].tuning.hasOwnProperty("car") && timelineJSON[i].tuning.hasOwnProperty("_id")){
                                        pendingFeed[timelineJSON[i].tuning._id] = timelineJSON[i];
                                        (ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy).getTuning(timelineJSON[i].tuning._id, tuningReadyHandler);
                                    }
                                }
                            }
                            return;
                        }
                        for(i = 0 ; i  < timelineJSON.length; i++){
                            if(ITEMS_TO_EXCLUDED.indexOf(timelineJSON[i].type) < 0 ){
                                var item : FeedData = new FeedData(timelineJSON[i]);
                                feed.push(item);
                            }
                            _loadedFeed++;
                        }
                        if(ObjectUtils.isEmpty(pendingFeed)){
                            pendingFeed = null;
                        }
                        if(!pendingFeed) {
//                            if (feed.length == 0) {
////                                loadFeed(user, limit, limit, fromDate);
//                            } else {
//                                sendNotification(Notifications.TIMELINE_DATA_LOADED, {feed: feed});
//                            }
                            sendNotification(Notifications.TIMELINE_DATA_LOADED, {feed: feed, reload:reload, requestId:requestId});
                        }
                    }
                    else {
                        sendNotification(Notifications.TIMELINE_DATA_LOADING_FAILED, httpClient.lastErrorStatusCode);
                    }
                }
        );
    }



    public function postToTimeline(tuningId:String):void {
        var params : Object = {tuningid: tuningId};
        httpClient.postPath("feed/tuning/", JSONer.stringify(params), function (success : Boolean, result : Object) : void {
            if(success){
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_POST_ADDED, json)
            }else{
                trace("couldn't add new post");
            }
        })
    }

    public function newPost(text:String, currentUser:User, images:Array = null):void {
        var params : Object = {id:currentUser.id, text:text, type:"text"};

        if(images){
            var urls : Array = [];
            for(var i : int = 0; i < images.length; i ++){
                var imageUrlToPost : String = images[i].url;
                if(imageUrlToPost){
                    urls.push(imageUrlToPost);
                }

            }

            if(urls.length){
                if(!params.metadata){
                    params.metadata = [];
                }

                params.metadata.push({
                    type: 'images',
                    photos: urls
                });
            }

        }


        httpClient.postPath("feed/"+currentUser.id, JSONer.stringify(params), function (success : Boolean, result : Object) : void {
            if(success){
                var json : Object = JSONer.parse(String(result));
                sendNotification(Notifications.TIMELINE_POST_ADDED, json)
            }else{
                trace("couldn't add new post");
            }
        })
    }
}
}
