package com.despark.tuning3d.model.users {
	//
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.utils.ObjectUtils;

import org.puremvc.as3.patterns.proxy.Proxy;

	public class UserProfileProxy extends Proxy {
        public var httpClient:HttpClient;
        public static const NAME : String = "UserProfileProxy";
		public function UserProfileProxy() {
			super(NAME);
		}

        public function loadUserProfile() : void {
            var url : String = HttpClient.PROFILE_ROOT;
            httpClient.getPath(url, null, function(success : Boolean, result : Object) : void {
                if (success) {
                    sendNotification(Notifications.PROFILE_LOADED, {result: JSON.parse(result as String)});
                } else {
                    trace("Failed to load profile data", JSON.stringify(result));
                }
            });
        }

        public function sendVerificationEmail() : void {
            var url : String = HttpClient.PROFILE_ROOT + '/mail/resend-verification';
            httpClient.postPath(url, null, function(success : Boolean, result : Object) : void {
                if (success) {
                    sendNotification(Notifications.VERIFICATION_EMAIL_RESENT, {success:true, result: result});
                } else {
                    sendNotification(Notifications.VERIFICATION_EMAIL_RESENT, {success:false, result: result});
                }
            });
        }

        public function deleteAccount(userId:String) : void {
            var url : String = HttpClient.DELETE_ACCOUNT_URL + userId;
            httpClient.putPath(url, null, function(success : Boolean, result : Object) : void {
                if (success) {
                    sendNotification(Notifications.DELETE_ACCOUNT, {success:true, result: result});
                } else {
                    sendNotification(Notifications.DELETE_ACCOUNT, {success:false, result: result});
                }
            });
        }

        public function saveProfileChanges(fullName:String = null, email:String = null, password:Object = null) : void {
            var url : String = HttpClient.PROFILE_ROOT;
            var data:Object = {};
            if(fullName){
                data.name = fullName;
            }
            if(email){
                data.email = email;
            }
            if(!ObjectUtils.isEmpty(data)) {
                httpClient.putPath(url, data, function (success:Boolean, result:Object):void {
                    if (success) {
                        sendNotification(Notifications.PROFILE_CHANGES_SAVED, {success: true, result: result});
                    } else {
                        if (result is String) {
                            sendNotification(Notifications.PROFILE_CHANGES_SAVED, {success: false, result: result});
                        } else if (result is Array) {
                            sendNotification(Notifications.PROFILE_CHANGES_SAVED, {success: false, result: result[0]});
                        } else if(result.hasOwnProperty("errorText")){
                            try{
                                result.errorText = JSON.parse(result.errorText as String);
                            }catch (e:Error) {

                            }
                            sendNotification(Notifications.PROFILE_CHANGES_SAVED, {success: false, result: result.errorText});
                        }
                    }
                });
            }


            if(password){
                httpClient.putPath(url + '/password', password, function(success : Boolean, result : Object) : void {
                    if (success) {
                        sendNotification(Notifications.PASSWORD_CHANGED, {success:true, result: result});
                    } else {
                        sendNotification(Notifications.PASSWORD_CHANGED, {success:false, result: result});
                    }
                });
            }
        }
	}
}