package com.despark.tuning3d.model.users
{
	//
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.core.Tracer;
	import com.despark.tuning3d.model.core.http.HttpClient;
	import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
	import com.despark.tuning3d.model.packages.PackagesProxy;
	import com.despark.tuning3d.model.packages.Packet;
	import com.despark.tuning3d.model.settings.Settings;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.distriqt.FacebookManager;
	import com.distriqt.GameService;
	import com.distriqt.PushNotification;
	import com.distriqt.extension.pushnotifications.PushNotifications;
	import com.utils.FileUtil;
	import com.utils.Logger;

	import flash.data.EncryptedLocalStore;
	import flash.desktop.NativeApplication;
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.globalization.LocaleID;
	import flash.globalization.StringTools;
	import flash.net.SharedObject;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.net.URLRequestHeader;
	import flash.net.URLRequestMethod;
	import flash.net.navigateToURL;
	import flash.system.Capabilities;
	import flash.utils.ByteArray;
	import flash.utils.setTimeout;

	import org.puremvc.as3.patterns.proxy.Proxy;

	import starling.core.Starling;

	public class UsersProxy extends Proxy {

		public static const NAME : String = "UsersProxy";

		public static const typeOfSN_VK: String = "typeOfSN_VK";
		public static const typeOfSN_FB : String = "typeOfSN_FB";
		public static const typeOfSN_TW : String = "typeOfSN_TW";
		public static const typeOfSN_none : String = "typeOfSN_none";
		public static const typeOfSN_shareInVK : String = "typeOfSN_shareInVK";
		public static const typeOfSN_shareInFB : String = "typeOfSN_shareInFB";
		public static const typeOfSN_shareInOK : String = "typeOfSN_shareInOK";
		public static const typeOfSN_shareInPin : String = "typeOfSN_shareInPin";

		public static const typeOfAds_main : String = "typeOfAds_main";
		public static const typeOfAds_tuning : String = "typeOfAds_tuning";

		public var httpClient : HttpClient;

		private var currentUserSharedObject : SharedObject;
		private var _currentUser : User;
		public var tokenId : String;

		private var urlreq:URLRequest;
		private var urlLoad:URLLoader = new URLLoader();

		private var _deviceId:String="";

		private var _tokenCooke:String="";


		private var _tokenCookeGuest:String="";
		private var _purchaseResult:Array=null;
		private var _purchaseResultMap:Object=null;
		private var _purchasesLoaded:Boolean = false;

		public function UsersProxy() {
			super(NAME);
			currentUserSharedObject = SharedObject.getLocal("currentUserSharedObject");
		}

		public static function loggedInUser():User{
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
			return usersProxy.currentUser;
		}

		public static function loggedInUserJSON():Object{
			var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
			if(usersProxy.currentUser){
				return usersProxy.currentUserJSON;
			}
            return null;
		}

		public function logoutCurrentUser() : void {
			httpClient.isUserSigned = false;
			httpClient.userToken = "";
			_tokenCooke = "";
			_purchasesLoaded = false;

            if(currentUserSharedObject.data.loginThroughSN == typeOfSN_FB) {
                try {
                    FacebookManager.$.logOut();
                }catch (e:Error) {

                }
            }
            _currentUser = null;
			saveUserJSONData(null);
			currentUserSharedObject.clear();

			if(EncryptedLocalStore.isSupported) {
				try {
                    EncryptedLocalStore.removeItem("username");
                    EncryptedLocalStore.removeItem("password");
                    EncryptedLocalStore.removeItem("params");
                    EncryptedLocalStore.removeItem("path");
                }catch (e:Error) {

                }
			}
			httpClient.httpCache.clean();
            var file:File = File.applicationStorageDirectory.resolvePath("timeline_settings.json");
            if(file.exists){
                file.deleteFile();
            }

			sendNotification(Notifications.USER_LOGGED_OUT);
		}

		public function loadCurrentUser(loadHandler:Function) : void {
			if(!currentUser){
				loadHandler(null);
				return;
			}
			httpClient.getPath(
				"accounts/my",
					{t:new Date().time},
				function(success : Boolean, result : Object) : void {
					if(success){
						loadHandler(result);
						saveUserJSONData(JSON.parse(result as String));
					}else{
						loadHandler(null);
					}
				});
		}

		public function checkUserData(loadHandler:Function) : void {
			if(!currentUser){
				loadHandler(false);
				return;
			}
			httpClient.getPath(
				"accounts/my",
					null,
				function(success : Boolean, result : Object) : void {
					if(success){
						if(result is String){
							result = JSON.parse(result as String);
						}
						if(result._id == currentUser.id){
							loadHandler(true);
						}else{
							loadHandler(false);
						}
					}else{
						loadHandler(false);
					}
				});
		}



		public function login(username : String, password : String, params : Object = null, __path : String = null) : void
		{

			trace("UserProxy login");
			var object : Object = null;
			var path : String = null;
			if(username && password){
				object = {
					username: username,
					password: password
				};

				path = "accounts/signin";

			}else{
				object = params;
				if(__path == null){
					sendNotification(Notifications.USER_LOGIN_FAILED);
					return;
				}
				path = __path;
			}

			httpClient.postPath(path,
				object,
				function(success : Boolean, loginResult : Object, responseHeader:Array=null) : void
				{
					if(success)
					{
						if(responseHeader){

							for(var i:int=0; i<responseHeader.length; i++){

								if(responseHeader[i].name=="Set-Cookie"){

									var value:String=responseHeader[i].value;
									if (value.indexOf('token') >= 0) {
										_tokenCooke=value.replace("token=", "");
										_tokenCooke=_tokenCooke.substr(0, _tokenCooke.indexOf(";"));
									}
								}
							}
						}

						//_tokenCooke
						httpClient.isUserSigned = true;
						httpClient.userToken = getUserToken();

						if(EncryptedLocalStore.isSupported) {
							saveUserCredentials(username, password, params, __path);
						}

						saveUserJSONString(loginResult as String);


						AUTH_TYPE=Notifications.USER_LOGGED_IN;

						loginSendedNotifyFlag=false;
						getPurchases();
						sendTokenId();
                        sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN);
					}
					else
					{
						sendNotification(Notifications.USER_LOGIN_FAILED);
					}
				});
		}

		private var AUTH_TYPE:String="";


		private function saveUserCredentials(username : String, password : String, __params : Object = null, __path : String = null) : void
		{

			if(username){
				var usernameBytes : ByteArray = new ByteArray();
				usernameBytes.writeUTF(username);
                try{
                    EncryptedLocalStore.setItem("username", usernameBytes);
				}catch (e:Error) {

                }

            }

			if(password){
				var passwordBytes : ByteArray = new ByteArray();
				passwordBytes.writeUTF(password);
				try{
                    EncryptedLocalStore.setItem("password", passwordBytes);
				}catch (e:Error) {

                }
			}

			if(__params){
				var paramsBytes : ByteArray = new ByteArray();
				var paramsStr : String = stringFromObject(__params);
				paramsBytes.writeUTF(paramsStr);

                try{
                    EncryptedLocalStore.setItem("params", paramsBytes);
                }catch (e:Error) {

                }
			}

			if(__path){
				var pathBytes : ByteArray = new ByteArray();
				pathBytes.writeUTF(__path);
				EncryptedLocalStore.setItem("path", pathBytes);
                try{
                    EncryptedLocalStore.setItem("path", pathBytes);
                }catch (e:Error) {

                }
			}

		}

		public static function stringFromObject(__object : Object) : String
		{
			var str : String = "";
			for(var key:String in __object){
				str += key + "=" + __object[key] + "&";
			}

			if(str.length > 0){
				str = str.slice(0,str.length-1);
			}
			return str;
		}

		public static function objectFromString(__str : String) : Object{

			var arrOfParams : Array = __str.split("&");
			var object : Object = new Object();

			for each(var paramItem : String in arrOfParams){
				var pair : Array = paramItem.split("=");
				if(pair.length != 2)continue;
				var key : String = pair[0];
				var value : String = pair[1];
				object[key] = value;
			}

			return object;
		}

		private function saveUserJSONString(jsonString : String) : void {
			var loginJSON : Object;
			try{
				loginJSON = JSON.parse(jsonString);
			}catch (e:Error) {
				return;
			}

			saveUserJSONData(loginJSON);

			if (loginJSON.hasOwnProperty('coins')) {
				GameService.$.addReward(loginJSON.coins as Number);
                useCoin();
			}
			var user_id : String = loginJSON._id;
			sendTokenId(user_id);
		}

		public function saveUserJSONData(userJSON:Object):void{
			if(userJSON){
				try{
					currentUserSharedObject.data.currentUserJSON = userJSON;
					currentUserSharedObject.flush();
				}catch (e:Error) {

				}
			}else{
				try{
					currentUserSharedObject.clear();
				}catch (e:Error) {

				}
			}

		}

		public function extendSession() : void
		{
			if(!EncryptedLocalStore.isSupported)
			{
				sendNotification(Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE);
				return;
			}

			try{
                var usernameBytes : ByteArray = EncryptedLocalStore.getItem("username");
                var passwordBytes : ByteArray = EncryptedLocalStore.getItem("password");
                var paramsBytes : ByteArray = EncryptedLocalStore.getItem("params");
                var pathBytes : ByteArray = EncryptedLocalStore.getItem("path");
			}catch (e:Error) {

            }


			if ((!usernameBytes || !passwordBytes) && (!paramsBytes || !pathBytes)){
				sendNotification(Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE);
				return;
			}

			var object : Object;
			var path : String;
			if(usernameBytes && passwordBytes){
				var username : String = usernameBytes.readUTF();
				var password : String = passwordBytes.readUTF();

				object = {
					username: username,
					password: password
				};

				path = "accounts/signin";

			}else{
				var paramsStr : String = paramsBytes.readUTF();
				path = pathBytes.readUTF();

				object = objectFromString(paramsStr);

			}

			httpClient.loadPath(
				path,
				URLRequestMethod.POST,
				object,
				HttpCachePolicy.NONE,
				false,
				function(success : Boolean, loginResult : Object,  responseHeader:Array=null) : void
				{
					if(success)
					{

						if(responseHeader){

							for(var i:int=0; i<responseHeader.length; i++){

								if(responseHeader[i].name=="Set-Cookie"){

									var value:String=responseHeader[i].value;
                                    if (value.indexOf('token') >= 0) {
                                        _tokenCooke = value.replace("token=", "");
                                        _tokenCooke = _tokenCooke.substr(0, _tokenCooke.indexOf(";"));
                                        httpClient.userToken = _tokenCooke;
                                        httpClient.isUserSigned = true;
                                    }
								}
							}
						}


						sendNotification(Notifications.USER_SESSION_EXTENDED);
					}
					else
					{
						if(httpClient.lastErrorStatusCode == 400)
						{
							sendNotification(Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE);
						}
						else
						{
							sendNotification(Notifications.USER_SESSION_EXTENDING_FAILED);
						}
					}
				});
		}

		public function registerUser(email : String, username : String, password : String, subscribe : Boolean) : void
		{
			httpClient.postPath(
				"accounts",
				{
					email: email,
					username: username,
					password: password,
					newsLetter: subscribe
				},
				function(success : Boolean, registerResult : Object, responseHeader:Array=null) : void
				{
					if(success)
					{

						trace("reg success");
						if(responseHeader){

							for(var i:int=0; i<responseHeader.length; i++){

								if(responseHeader[i].name=="Set-Cookie"){

									var value:String=responseHeader[i].value;
									_tokenCooke=value.replace("token=", "");
									_tokenCooke=_tokenCooke.substr(0, _tokenCooke.indexOf(";"));

								}
							}
						}



						httpClient.isUserSigned = true;
                        if (httpClient.isUserSigned) {
                            httpClient.userToken = getUserToken();
                        }
						if(EncryptedLocalStore.isSupported)
						{
							saveUserCredentials(username, password);
						}

						saveUserJSONString(registerResult as String);


						AUTH_TYPE=Notifications.USER_REGISTERED;

						loginSendedNotifyFlag=false;


						sendTokenId();
					}
					else
					{
						trace("reg failed");
						if(registerResult is String && registerResult == "Invalid username or password"){
                            registerResult = [{
								location:"body",
								msg: "The username already exists. Please use another username",
								param: "username",
								value: username
							}];
						}
						sendNotification(Notifications.USER_REGISTRATION_FAILED, registerResult);
					}
				});
		}

		private var loginSendedNotifyFlag:Boolean=false;

		public function sendForgottenPassword(email : String) : void
		{
			httpClient.postPath(
				"accounts/resetPassword",
				{
					email: email
				},
				function(success : Boolean, result : Object) : void
				{
					if(success)
					{
						sendNotification(Notifications.PASSWORD_RESET);
					}
					else
					{
						sendNotification(Notifications.PASSWORD_RESETTING_FAILED);
					}
				});
		}

		public function getUserToken() : String {
			return _tokenCooke;
		}

		public function useCoin(): void {
            var urlll : String = Constants.API_URL_api+"accounts/usecoins";
            var urlLoad1:URLLoader = new URLLoader();
            var urlreq1:URLRequest = new URLRequest(urlll);
            urlreq1.useCache=false;
            urlreq1.requestHeaders = [];
            urlreq1.requestHeaders = [new URLRequestHeader("Cookie", "token="+_tokenCooke)];
            urlreq1.contentType="application/json";

            urlreq1.manageCookies=true;
            urlreq1.method = URLRequestMethod.GET;


			var completeFunction: Function = function(e: Event): void {
                urlLoad1.removeEventListener(Event.COMPLETE, completeFunction);
			};

            var errorFunction: Function = function(e: Event): void {
                urlLoad1.removeEventListener(IOErrorEvent.IO_ERROR, errorFunction);
            };


            urlLoad1.addEventListener(IOErrorEvent.IO_ERROR, errorFunction);

            urlLoad1.addEventListener(Event.COMPLETE, completeFunction);
            urlLoad1.load(urlreq1);
		}

		public function isUserDataValid():Boolean{
			var userJSON : Object = (currentUserSharedObject && currentUserSharedObject.data)?currentUserSharedObject.data.currentUserJSON: null;
			return !(!userJSON || !_tokenCooke || _tokenCooke == "" || !httpClient.isUserSigned || httpClient.userToken == "" || !httpClient.userToken);
		}

		public function get currentUser() : User {
			var userJSON : Object = currentUserSharedObject.data.currentUserJSON;
			if(userJSON) {
				if(!_currentUser) {
					var user : User = new User();
					user.username = userJSON.username;
					user.id = userJSON._id;
					user.name = userJSON.name;

					user.emailVerified = userJSON.emailVerified;
					user.challengeCoins = userJSON.challengeCoins;

					if(userJSON.hasOwnProperty("roles")){
						user.roles = userJSON.roles;
						trace("user.roles:", user.roles)
					}
					if(userJSON.avatarPath && userJSON.avatarPath.length){
						user.avatarPath = userJSON.avatarPath;
					}else if(userJSON.avatar && userJSON.avatar is Array){
						for(var i:int = 0; i < userJSON.avatar.length; i++){
							if(userJSON.avatar[i].current){
								user.avatarPath = userJSON.avatar[i].path;
								userJSON.avatarPath = user.avatarPath;
								break;
							}
						}
						if(user.avatarPath){
							saveUserJSONData(userJSON);
						}else{
							user.avatarPath = "";
						}
					}else{
						user.avatarPath = "";
					}
					_currentUser = user;
				}
				return _currentUser;
			}
			return null;
		}

		public function get currentUserJSON() : Object {
			var userJSON : Object = currentUserSharedObject.data.currentUserJSON;
			if(userJSON) {
				return userJSON;
			}
			return null;
		}

		//работа с памятью
		public function setArrOfCustDownloads(ArrOfCustDownloads: Array, idOfCar: String): void{
			var dicOfArrOfCustDownloads: Object = currentUserSharedObject.data.dicOfArrOfCustDownloads;
			if(!dicOfArrOfCustDownloads){
				dicOfArrOfCustDownloads = {};
				currentUserSharedObject.data.dicOfArrOfCustDownloads = dicOfArrOfCustDownloads;
			}

			var arrOfCustDownloads:Array = dicOfArrOfCustDownloads[idOfCar];
			if(!arrOfCustDownloads){
				arrOfCustDownloads = [];
			}
			arrOfCustDownloads = arrOfCustDownloads.concat(ArrOfCustDownloads);

			dicOfArrOfCustDownloads[idOfCar] = arrOfCustDownloads;

			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function existFolderInArrOfCustDownloads(PathTofolder: String, idOfCar: String): Boolean{
			var dicOfArrOfCustDownloads: Object = currentUserSharedObject.data.dicOfArrOfCustDownloads;
			if(!dicOfArrOfCustDownloads){
				return false;
			}

			var arrOfCustDownloads:Array = dicOfArrOfCustDownloads[idOfCar];
			if(!arrOfCustDownloads){
				return false;
			}

			var index:int = arrOfCustDownloads.indexOf(PathTofolder);

			return (index>=0);
		}


		public function deleteArrOfCustDownload(partSlotDirectory : File, idOfCar: String): void
		{




			var folders:Array = partSlotDirectory.getDirectoryListing();

			var ArrOfCustDownloadsForDelete: Array = [];

			for(var j:int = 0; j< folders.length; j++){
				var folder:File = folders[j]

				if((folder.name != "default") && (folder.isDirectory)){
					ArrOfCustDownloadsForDelete.push(folder.nativePath);
				}
			}

			var dicOfArrOfCustDownloads: Object = currentUserSharedObject.data.dicOfArrOfCustDownloads;
			if(!dicOfArrOfCustDownloads){
				dicOfArrOfCustDownloads = new Object();
				currentUserSharedObject.data.dicOfArrOfCustDownloads = dicOfArrOfCustDownloads;
			}

			var arrOfCustDownloads:Array = dicOfArrOfCustDownloads[idOfCar];
			if(!arrOfCustDownloads){
				return;
			}

			for(var i2:int = 0; i2<ArrOfCustDownloadsForDelete.length; i2++){
				var objectForDelete:Object = ArrOfCustDownloadsForDelete[i2];
				if(objectForDelete){
					var index:int = arrOfCustDownloads.indexOf(objectForDelete);
					while(index>=0){
						arrOfCustDownloads.splice(index, 1);
						index = arrOfCustDownloads.indexOf(objectForDelete);
					}
				}
			}

			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function setSaveDefCar(SaveDefCar: Boolean, idOfCar: String): void{
			var dicOfSaveDefCar: Object = currentUserSharedObject.data.dicOfSaveDefCar;
			if(!dicOfSaveDefCar){
				dicOfSaveDefCar = {};
				currentUserSharedObject.data.dicOfSaveDefCar = dicOfSaveDefCar;
			}
			dicOfSaveDefCar[idOfCar] = SaveDefCar;

			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function saveDefCar(idOfCar: String): Boolean{

			var dicOfSaveDefCar: Object = currentUserSharedObject.data.dicOfSaveDefCar;
			if(!dicOfSaveDefCar || (dicOfSaveDefCar[idOfCar]==null)){
				return false;
			}

			return dicOfSaveDefCar[idOfCar];
		}


		public function setSizeObject(SizeObject: Object, idOfCar: String): void{
			var dicOfSizeObject: Object = currentUserSharedObject.data.dicOfSizeObject;
			if(!dicOfSizeObject){
				dicOfSizeObject = {};
				currentUserSharedObject.data.dicOfSizeObject = dicOfSizeObject;
			}

			dicOfSizeObject[idOfCar] = SizeObject;

			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function dicOfSizeObjects(): Object{
			var dicOfSizeObject: Object = currentUserSharedObject.data.dicOfSizeObject;
			return dicOfSizeObject;
		}

		public function deleteCacheForSizeObject(SizeObject: Object): Number{
			//удаляем дефолтные папки и папки которые не выбраны в CustomDownloads

			if(!SizeObject.pathOfCarLocalFile){
				return 0;
			}

			//получим путь к папке этой машины (базовая папка)
			var carLocalFile : File = File.cacheDirectory.resolvePath(SizeObject.pathOfCarLocalFile);;//File(SizeObject.pathOfCarLocalFile);
			carLocalFile.preventBackup = true;
			//пробегаемся по папкам в базовой папке (папка слота)
			if(!carLocalFile.exists){
				return 0;
			}

			var bSaveDefCar:Boolean = saveDefCar(SizeObject.idOfCar);
			if(bSaveDefCar){
				return 0;
			}
			var foldersOfSlots:Array = carLocalFile.getDirectoryListing();

			for each(var folderOfSlot:File in foldersOfSlots){
				if(!folderOfSlot.isDirectory)continue;

				//пробегаемся по содержимому в слотовой папке
				var contentOfSlotFolder:Array = folderOfSlot.getDirectoryListing();
				for each(var folder:File in contentOfSlotFolder){

					if(!folder.isDirectory)continue;
					var thisIsDefault : Boolean = (folder.name == "default");
					//если это папка default то удаляем эту папку
					if(thisIsDefault){
						folder.deleteDirectory(true);

						continue;
					}
				}
			}

			SizeObject.size = 0;
			//сохраняем результат
			setSizeObject(SizeObject,SizeObject.idOfCar);

			return 0;
		}

		public function deleteAllCache(): void
		{
			AlertView.show("Сache deleted", 3);//The cache is cleaned.

			var dicOfSizeObject: Object = currentUserSharedObject.data.dicOfSizeObject;
			if(!dicOfSizeObject){
				return;
			}

			var previousCarObject: Object = currentUserSharedObject.data.previousCarObject;
			var currentCarId: String = previousCarObject.previousCarId;

			for each(var SizeObject: Object in dicOfSizeObject){

				if(SizeObject.idOfCar == currentCarId){
					continue;
				}

				deleteCacheForSizeObject(SizeObject);
			}
		}

		public function changeCar(newCar: Car):void{
			var previousCarObject: Object = currentUserSharedObject.data.previousCarObject;
			if(!previousCarObject){
				previousCarObject = {};
				currentUserSharedObject.data.previousCarObject = previousCarObject;
			}

			var previousCarId: String = previousCarObject.previousCarId;
			var previousCarPath: String = previousCarObject.pathOfCar;
			previousCarObject.previousCarId = newCar.id;
			previousCarObject.pathOfCar = newCar.path;
			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
			previousCarObject = null;
			if(!previousCarId){
				return;
			}
			if(previousCarId == newCar.id){
				return;
			}

			//задача: при смене машины сделать проверку по папкам defaults и массиву custom download, а также по переменной save car и удалить не входящие в список файлы

			var bSaveDefCar:Boolean = saveDefCar(previousCarId);

			//получим путь к папке этой машины (базовая папка)
			var carLocalFile : File = File.cacheDirectory.resolvePath(previousCarPath);
			carLocalFile.preventBackup = true;
			//пробегаемся по папкам в базовой папке (папка слота)
			if(!carLocalFile.exists){
				return;
			}

			var foldersOfSlots:Array = carLocalFile.getDirectoryListing();

			var TotalSizeInteger:Number = new Number();

			for each(var folderOfSlot:File in foldersOfSlots){
				if(!folderOfSlot.isDirectory)continue;

				//пробегаемся по содержимому в слотовой папке
				var contentOfSlotFolder:Array = folderOfSlot.getDirectoryListing();
				for each(var folder:File in contentOfSlotFolder){

					if(!folder.isDirectory)continue;
					var thisIsDefault : Boolean = (folder.name == "default");


					if(thisIsDefault){
						continue;
					}

					//иначе если это !default папка и ее нет в массиве dicOfArrOfCustDownloads то удаляем эту папку
					if(!existFolderInArrOfCustDownloads(folder.nativePath,previousCarId)){
						folder.deleteDirectory(true);
						continue;
					}
				}
			}


			var sizeObject : Object = {};
			sizeObject.size = TotalSizeInteger;
			var currentDate:Date = new Date();
			sizeObject.dateAndTime = currentDate.time;
			sizeObject.pathOfCarLocalFile = carLocalFile.nativePath;
			sizeObject.idOfCar = previousCarId;

			setSizeObject(sizeObject,previousCarId);

			//сделать проверку размеров по словарю объектов размеров и если предел превышен то удалить самое старое
			var dicOfSizeObject: Object = dicOfSizeObjects();

			var arrOfValues : Array = [];
			for each(var SizeObject :Object in dicOfSizeObject){
				arrOfValues.push(SizeObject);
			}

			arrOfValues.sortOn(['dateAndTime'], [Array.DESCENDING])

			var commonSize: Number = new Number();
			var maxSize:Number = 650 * 1024 * 1024;
			for each(var SizeObject2 :Object in arrOfValues){

				commonSize += SizeObject2.size;
			}
			if(commonSize > maxSize){

				var megabytesForDelete : Number = commonSize - maxSize;

				for(var i : int = arrOfValues.length-1 ; i >=0; i--){
					var SizeObject3 :Object = arrOfValues[i];

					var deletedMegabytes:Number = deleteCacheForSizeObject(SizeObject3);
					megabytesForDelete -= deletedMegabytes;

					if(megabytesForDelete<=0){
						break;
					}
				}
			}
		}


		public function GetFolderSize(Source:Array):Number
		{
			var TotalSizeInteger:Number = new Number();
			for(var i:int = 0;i<Source.length;i++){
				if(Source[i].isDirectory){
					TotalSizeInteger +=   GetFolderSize(Source[i].getDirectoryListing());
				}
				else{
					TotalSizeInteger += Source[i].size;
				}
			}
			return TotalSizeInteger;
		}

		//работа с comment
		public function ShowRateBoxIfNeed():void
		{
			var RateMe: String = currentUserSharedObject.data.RateMe;

			if((RateMe == "rated")||(RateMe == "no")){
				return;
			}

			var NumOfCall: Number = currentUserSharedObject.data.NumOfCall;
			if(!NumOfCall){
				NumOfCall = 0;
			}
			NumOfCall ++;
			currentUserSharedObject.data.NumOfCall = NumOfCall;
			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
			if((NumOfCall == 3) || (NumOfCall == 10)|| (NumOfCall == 30)|| (NumOfCall == 130)){
				ShowRateBox();
			}
		}

		public function ShowRateBox():void
		{

			var funRate:Function = function (event:ChallengeAlertEvent):void {
				var url : String = null;
				if(Constants.isIOS){
					url = AlertView.language.translate("mob_settings", "appleStoreUrl");
				}else{
					url = AlertView.language.translate("mob_settings", "googlePlayUrl");
				}

				if(url){
					navigateToURL(new URLRequest(url));//вот это открывает приложение в apple store где его можно оценить iOS7
				}

				currentUserSharedObject.data.RateMe = "rated";
				try{
					currentUserSharedObject.flush();
				}catch (e:Error) {

				}
			};
			var funLater:Function = function (event:ChallengeAlertEvent):void {
				currentUserSharedObject.data.RateMe = "later";
				try{
					currentUserSharedObject.flush();
				}catch (e:Error) {

				}
			};
			var funNo:Function = function (event:ChallengeAlertEvent):void {
				currentUserSharedObject.data.RateMe = "no";
				try{
					currentUserSharedObject.flush();
				}catch (e:Error) {

				}
			};
/*

			var textOfMessage:String = AlertView.language.translate("feedback", "text");
			var comment:String = AlertView.language.translate("feedback", "comment");

			if(!textOfMessage){
				 textOfMessage= "If you enjoy using Tuning3D, would you mind taking a moment to rate it? It won't take more than a minute. Thanks for your support!";
			}

			if(!comment){
				comment = "Leave a comment";
			}
*/

			//CustomAlertView.show(textOfMessage, comment, "Remind me later", "No, Thanks",funRate, funLater, funNo);
			var rateBoxAlert:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.RATE_BOX_POPUP, null, Starling.current.stage);
			rateBoxAlert.addEventListener(ChallengeAlertEvent.COMMENT_TRIGGERED, funRate);
			rateBoxAlert.addEventListener(ChallengeAlertEvent.REMIND_TRIGGERED, funLater);
			rateBoxAlert.addEventListener(ChallengeAlertEvent.CLOSE_TRIGGERED, funNo);
			rateBoxAlert.open();

			// CustomAlertView.show(textOfMessage, {label: comment, callback: funRate}, {label: "Remind me later", callback: funLater}, {label: "No, Thanks", isDefault:true, callback: funNo});

		}


		// работа с translate
		public function needCheckTranslate():Boolean
		{
			if(!currentUserSharedObject.data.dateAndTimeOfLastCheckTranslate){
				return true;
			}
			var dateAndTimeOfLastCheckTranslate : Number = currentUserSharedObject.data.dateAndTimeOfLastCheckTranslate;
			var currentDate:Date = new Date();
			var currentDateAndTime : Number = currentDate.time;
			var numberOfHours : Number = (currentDateAndTime - dateAndTimeOfLastCheckTranslate)/1000/60/60;

			if(numberOfHours > 4){
				return true;
			}

			return false;
		}

		public function checkedTranslate():void
		{
			var currentDate:Date = new Date();
			//Date(
			var dateAndTime : Number = currentDate.time;

			currentUserSharedObject.data.dateAndTimeOfLastCheckTranslate = dateAndTime;
			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		// работа с связкой имени картинки и пути к ней для 3dt файлов
		public function saveNameOfImage(__NameOfImage : String, __nativePath : String) : void
		{
			var dicOfNameOfImageFrom3dt: Object = currentUserSharedObject.data.dicOfNameOfImageFrom3dt;
			if(!dicOfNameOfImageFrom3dt){
				dicOfNameOfImageFrom3dt = {};
				currentUserSharedObject.data.dicOfNameOfImageFrom3dt = dicOfNameOfImageFrom3dt;
			}

			dicOfNameOfImageFrom3dt[__nativePath] = __NameOfImage;
			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function nameOfImageByNativePath(__nativePath : String) : String
		{

			var dicOfNameOfImageFrom3dt: Object = currentUserSharedObject.data.dicOfNameOfImageFrom3dt;
			if(!dicOfNameOfImageFrom3dt){
				return null;
			}
			var nameOfImage : String = dicOfNameOfImageFrom3dt[__nativePath];

			return nameOfImage;
		}

		///работа с логированием
		public function set loginThroughSN(__typeOfSN : String) : void
		{
			currentUserSharedObject.data.loginThroughSN = __typeOfSN;
			try{
				currentUserSharedObject.flush();
			}catch (e:Error) {

			}
		}

		public function get loginThroughSN() : String
		{
			var loginThroughSN_s: String = currentUserSharedObject.data.loginThroughSN;
			if(!loginThroughSN_s){
				return typeOfSN_none;
			}

			return loginThroughSN_s;
		}

		//работа с TokenId
		public function sendTokenId(user_id : String = null):void
		{
			if(user_id==null && this.currentUser){

				user_id=this.currentUser.id;
			}

			var urlll : String = Constants.API_URL_api + "devices";

			var os:String="android";

			if(Device.isIos()){
				os="ios";
			}


			urlreq = new URLRequest(urlll);

			var jsonRequest:Object={};


			if(user_id){
				jsonRequest["uid"]=user_id;
			}
			jsonRequest["type"] = os;

			var locale:String=new StringTools(LocaleID.DEFAULT).actualLocaleIDName;
			jsonRequest["osVersion"]=Capabilities.os;

			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var settings : Settings = settingsProxy.currentSettings;


			jsonRequest["lang"]=settings.selectedLanguageCode;
			jsonRequest["country"] =  locale.substr(3);
			jsonRequest["appVersion"] = Logger.getInstance().getAppVersion().replace(".", "").replace(".","");

            try{
				if(PushNotification.isInitialized){
					jsonRequest["token"] = PushNotifications.service.getDeviceToken();
				}else{
					jsonRequest["token"] = null;
				}
			}catch(e:Error){
                jsonRequest["token"] = null;
            }

			urlreq.contentType="application/json";
			urlreq.data = JSON.stringify(jsonRequest);//variables;
			urlreq.method = URLRequestMethod.POST;

			urlLoad.addEventListener(IOErrorEvent.IO_ERROR,iohandler);
			urlLoad.addEventListener(Event.COMPLETE,compHandler);
			urlLoad.addEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandler);
			urlLoad.load(urlreq);
		}

		private function iohandler(e:IOErrorEvent):void
		{
			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,iohandler);
			urlLoad.removeEventListener(Event.COMPLETE,compHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandler);

			var manualDevicesId:String="555caf7e55f225030092815a";
			onRecieveDivicesId(manualDevicesId);
		}

		public function get deviceId():String{

			return _deviceId;
		}

		private function compHandler(e:Event):void{
			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,iohandler);
			urlLoad.removeEventListener(Event.COMPLETE,compHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandler);
			try{
				var result:Object=JSON.parse(URLLoader(e.target).data);
				onRecieveDivicesId(result._id);
			}catch(e:Error){

			}
		}

		private function onRecieveDivicesId(device_id:String):void
		{
			Tracer.print("onRecieveDivicesId:"+ device_id);
			_deviceId=device_id;

			try{
				if(Constants.IN_APP_PURCHASE_ENABLED){
                    if(purchasesCallback==null){
						trace('*********onRecieveDivicesId*******')
						getPurchases();
					}
				}
                else
                {
                    if(this.currentUser){

                        if(!loginSendedNotifyFlag){
                            loginSendedNotifyFlag=true;
                            sendNotification(AUTH_TYPE);
                            sendNotification(Notifications.USER_AUTHENTICATED, {wasAlreadyAuthenticated: false});
                        }
                    }
				}
			}
            catch(e:Error){
                Tracer.print("Error:"+e.message);
            }

		}

		private function httpHandler(e:HTTPStatusEvent):void{

		}


		private function createTestProduct(_deviceId:String, product:Object):void
		{
			urlLoad=new URLLoader();

			var urlll : String = Constants.API_URL_device + "products";
			//urlString : String;

			urlreq = new URLRequest(urlll);
			urlreq.requestHeaders = [];
			urlreq.requestHeaders.push(new URLRequestHeader("Authorization", Constants.API_DEVICE_AUTH_KEY));
			urlreq.contentType="application/json";
			urlreq.data = JSON.stringify(product);
			urlreq.manageCookies=true;
			urlreq.method = URLRequestMethod.POST;
			urlLoad.addEventListener(IOErrorEvent.IO_ERROR,ioErrorCreateTestProductHandler);
			urlLoad.addEventListener(Event.COMPLETE,completeCreateTestProductHandler);
			urlLoad.addEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusCreateTestProductHandler);
			urlLoad.load(urlreq);
		}

		private function ioErrorCreateTestProductHandler(e:IOErrorEvent):void
		{

			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorCreateTestProductHandler);
			urlLoad.removeEventListener(Event.COMPLETE,completeCreateTestProductHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusCreateTestProductHandler);
		}

		private function completeCreateTestProductHandler(e:Event):void
		{
			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorCreateTestProductHandler);
			urlLoad.removeEventListener(Event.COMPLETE,completeCreateTestProductHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusCreateTestProductHandler);

		}

		private function httpStatusCreateTestProductHandler(e:HTTPStatusEvent):void
		{

			Tracer.print("httpStatusCreateTestProductHandler Status:" + e.status);
		}

		public function deleteProduct(productId:String):void{
			var urlll : String = Constants.API_URL_device+"products/"+productId;
			urlreq = new URLRequest(urlll);
			urlreq.requestHeaders = [];
			urlreq.requestHeaders.push(new URLRequestHeader("Authorization", Constants.API_DEVICE_AUTH_KEY));
			urlreq.contentType="application/json";
			urlreq.manageCookies=true;
			urlreq.method = URLRequestMethod.DELETE;

			urlLoad=new URLLoader();
			urlLoad.load(urlreq);
			urlLoad.addEventListener(IOErrorEvent.IO_ERROR,ioErrorDeleteProductHandler);
			urlLoad.addEventListener(Event.COMPLETE,completeDeleteProductHandler);
			urlLoad.addEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusDeleteProductHandler);
		}

		private function ioErrorDeleteProductHandler(e:IOErrorEvent):void
		{
			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorDeleteProductHandler);
			urlLoad.removeEventListener(Event.COMPLETE,completeDeleteProductHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusDeleteProductHandler);
		}

		private function completeDeleteProductHandler(e:Event):void
		{
			urlLoad.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorDeleteProductHandler);
			urlLoad.removeEventListener(Event.COMPLETE,completeDeleteProductHandler);
			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusDeleteProductHandler);

		}

		private function httpStatusDeleteProductHandler(e:HTTPStatusEvent):void
		{
			Tracer.print("httpStatusDeleteProductHandler Status:" + e.status);
            Tracer.print("httpHandler:"+JSON.stringify(e.responseHeaders));
		}

		//работа с банерами
		private var adsUrlLoader:URLLoader=null;
		public function ads(__typeOfAds : String,__brend : String,__model : String, __function : Function) : void
		{
			var params : String = "?";
			params += "width="+ Capabilities.screenResolutionX +"&";
			params += "height="+ Capabilities.screenResolutionY +"&";

			if(__typeOfAds == typeOfAds_tuning){
				params += "placement=tuning&";
			}
			else if (__typeOfAds == typeOfAds_main){
				params += "placement=main&";
			}

			if(httpClient.language){
				params += "region="+ httpClient.language.code.substr(0,2).toUpperCase() +"&";
			}

			params += "tags=[";
			if(Device.isIos()){
				params += "ios";
			}
			else {
				params += "android";
			}

			if(__brend){
				params += "," + __brend + "," + __model;
			}

			params += "]&r="+(Math.random()*10000);

			adsUrlLoader=new URLLoader();
			var adsUrlRequest:URLRequest=new URLRequest(Constants.API_URL_ads + params);
			adsUrlLoader.addEventListener(Event.COMPLETE, function(event:Event) : void
			{
				var result:String=URLLoader(event.target).data;
				try{
					var JSON_object : Object = JSON.parse(String(result));
					__function(JSON_object);
				}
                                catch(error:Error){
					__function(null);
				}
			});

			adsUrlLoader.addEventListener(IOErrorEvent.IO_ERROR, function(event:Event) : void{
				__function(null);
			}); adsUrlLoader.load(adsUrlRequest);
		}



		public static function parseJSONObject(value:String):Object{

			var res:Array;
			var pattern:RegExp = /"(.*?)".*?=.*?\"(.*?)"\;/gsm;

			var result:Object={};
			res=pattern.exec(value);
			while (res){
				result[res[1]]=res[2];
				res=pattern.exec(value);
			}

			return result;
		}

		private var purchasesCallback:Function=null;
		public function getPurchases(callback:Function=null, resetPackages:Boolean = true):void{
			Tracer.print("*******UserProxy.getPurchases()********");
			purchasesCallback=callback;
			if(!Device.isIos()){
				if(currentUser) {
					var urlll:String = Constants.API_URL_api + "new-purchases";
					var urlLoad1:URLLoader = new URLLoader();
					var urlreq1:URLRequest = new URLRequest(urlll);
					urlreq1.useCache = false;
					urlreq1.requestHeaders = [new URLRequestHeader("Cookie", "token=" + _tokenCooke)];
					urlreq1.contentType = "application/json";

					urlreq1.manageCookies = true;
					urlreq1.method = URLRequestMethod.GET;

					urlLoad1.addEventListener(IOErrorEvent.IO_ERROR, ioErrorGetPurchasesHandler);
					urlLoad1.addEventListener(Event.COMPLETE, completeGetPurchasesHandler);
					urlLoad1.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusGetPurchasesHandler);
					urlLoad1.load(urlreq1);
				}
				return;
			}

			setTimeout(function():void{
				_purchaseResultMap={};
				var json:String=readPurchases();
				var result:Array=JSON.parse(json) as Array;
				_purchaseResult = result;

				if(purchasesCallback){
					purchasesCallback(_purchaseResult);
					purchasesCallback=null;
					_purchaseResult=null;
				}

				sendNotification(Notifications.PURCHASES_LOADED, {purchases: result, resetPackages: resetPackages});

				if(!loginSendedNotifyFlag){
					loginSendedNotifyFlag=true;

					if(currentUser){
						sendNotification(AUTH_TYPE);
						sendNotification(Notifications.USER_AUTHENTICATED, {wasAlreadyAuthenticated: false});

					}
				}
			}, 1000);
		}

		private function ioErrorGetPurchasesHandler(e:IOErrorEvent):void
		{
			purchasesCallback=null;

			var loader:URLLoader=e.target as URLLoader;
			loader.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorGetPurchasesHandler);
			loader.removeEventListener(Event.COMPLETE,completeGetPurchasesHandler);
			loader.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusGetPurchasesHandler);

			sendNotification(Notifications.PURCHASES_LOADING_FAILED);
		}

		public function isExistPacket(packet_id:String):Boolean {
			return packet_id && _purchaseResultMap && _purchaseResultMap[packet_id];
		}

		public function setPurchasesData(json:String):void
                {
			_purchaseResultMap={};

			var result:Array=JSON.parse(json) as Array;
			_purchaseResult=result;

			var packagesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;

			for(var i:int=0; i<_purchaseResult.length;  i++){

				var packet:Packet=packagesProxy.findPacketById(_purchaseResult[i].product);

				if(packet){
					_purchaseResultMap[packet.productId]=packet;//_purchaseResult[i].product;
				}
			}

		}

		private function completeGetPurchasesHandler(e:Event=null, data:String=null):void{

			_purchaseResultMap={};

			var loader:URLLoader=null;

			if(e){
				loader=e.target as URLLoader;
				loader.removeEventListener(IOErrorEvent.IO_ERROR,ioErrorGetPurchasesHandler);
				loader.removeEventListener(Event.COMPLETE,completeGetPurchasesHandler);
			}

			var json:String="";

			if(loader && loader.data){

				json=loader.data as String;
			}else if(data){
				json=data;

			}else{

				sendNotification(Notifications.PURCHASES_LOADING_FAILED);

				if(!loginSendedNotifyFlag){
					loginSendedNotifyFlag=true;
					sendNotification(AUTH_TYPE);
					sendNotification(Notifications.USER_AUTHENTICATED, {wasAlreadyAuthenticated: false});
				}
				return;
			}


			try{
                var i : int;
				var jsonFirst:String=readPurchases();
				//
				var resultFirst:Array=JSON.parse(jsonFirst) as Array;
				var result:Array=JSON.parse(json) as Array;
				//
				setPurchasesData(json);
				_purchaseResult=result;
				// removing repeated objects
				var purchaseList:Object = {};
				for(i = 0; i < _purchaseResult.length; i++){
					var obj:Object = _purchaseResult[i];
					if(obj.hasOwnProperty("product")){
						if(!purchaseList.hasOwnProperty(obj.product) || (!purchaseList[obj.product]["productId"] && obj.productId)){
							purchaseList[obj.product] = obj;
						}
					}
				}
				//
				_purchasesLoaded = true;
				_purchaseResult = [];
				for each(var item:Object in purchaseList){
					_purchaseResult.push(item);
				}
				//

				for(i=0; i<resultFirst.length; i++){
					if(resultFirst[i].product && !packetExistsInArray(_purchaseResult,resultFirst[i].product)){
						_purchaseResult.push(resultFirst[i]);
					}
				}

				var jsonString:String=JSON.stringify(_purchaseResult);
				savePurchases(jsonString);

				var xml : XML = NativeApplication.nativeApplication.applicationDescriptor;
				var ns : Namespace = xml.namespace();
				var version : String = xml.ns::versionNumber;
				var file:File=File.applicationStorageDirectory.resolvePath(version+"_first_login.dat");
				var fs:FileStream=new FileStream();

				fs.open(file, FileMode.WRITE);
				fs.writeByte(255);
				fs.close();

				if(purchasesCallback){
					purchasesCallback(_purchaseResult);
					purchasesCallback=null;
					_purchaseResult=null;
				}

				setTimeout(function():void{
					sendNotification(Notifications.PURCHASES_LOADED, {purchases: _purchaseResult});
				}, 2000);

				if(!loginSendedNotifyFlag){
					loginSendedNotifyFlag=true;
					sendNotification(AUTH_TYPE);
					sendNotification(Notifications.USER_AUTHENTICATED, {wasAlreadyAuthenticated: false});
				}
			}catch(error:Error){
				purchasesCallback=null;
				sendNotification(Notifications.PURCHASES_LOADING_FAILED);
			}
		}

		private function sendIapVerify(receiptJSONParam:Object, packetParam:Packet ):void{

			if(iapVerifyCallback==null){
				iapVerify({receipt:receiptJSONParam}, packetParam, function(verify:Boolean, msg:String) : void{
				});
			}else{
				setTimeout(sendIapVerify, 100, receiptJSONParam, packetParam);
			}
		}

		private function httpStatusGetPurchasesHandler(e:HTTPStatusEvent):void{

			urlLoad.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpStatusGetPurchasesHandler);
			Tracer.print("purchase STATUS : " + e.status,"httpStatusGetPurchasesHandler")
		}

		private var iapVerifyCallback:Function=null;
		private var iapVerifyPacket:Packet=null;
		private var _receipt:String=null;
		public function iapVerify(receiptData:Object, packet:Packet, callback:Function) : void
		{
			trace('iapVerify');
			if(!packet)
				return;

			iapVerifyPacket=packet;

			iapVerifyCallback=callback;
			var user:User=this.currentUser;
			//user.
			var platform:String="android";
			if(Device.isIos()){

				platform="ios";
			}

			var params:Object;
			var receiptStr:String;
			var deviceId:String=_deviceId;//productInfo['unique-identifier'];//Tuning3D.udid;
			if(deviceId==null || deviceId==""){
				_deviceId="555caf7e55f225030092815a"
				deviceId=_deviceId;
			}

			var packageName:String="Test";
			var transactionId:String;
			var productId:String;
			var accessToken:String=tokenId ;

			var feeUSD:Number=parseFloat((packet.fee/60).toFixed(2));
			var fee:Number=packet.fee;
			var feeCurrency:String=packet.feeCurrency;
			var sandBox:int=Constants.IN_APP_PURCHASE_SANDBOX;

			if(platform=="ios"){
				transactionId=receiptData.transactionId;//productInfo['transaction-id'];

				productId=receiptData.productId;
				var receipt:String=receiptData.receipt;
				var bid:String="com.3dtuning.Tuning3D";//productInfo['bid'];//Tuning3D.udid;


				params={

					device:deviceId,
					platform:platform,
					transactionId:transactionId,
					product:packet.id,
					productId:productId,
					receipt:receipt,
					bid:bid,
					feeUSD:feeUSD,
					fee:fee,
					feeCurrency:feeCurrency,
					sandBox:sandBox
				};

				if(user)
					params['uid']=user.id;
				else
					params['uid']=deviceId;

			}else{
				transactionId=receiptData.transactionId;
				packageName=receiptData.packageName;
				accessToken=Constants.IN_APP_GOOGLE_PLAY_PUBLIC_KEY; //productInfo['purchaseToken'];

				params={
					platform:platform,
					transactionId:transactionId,
					device:deviceId,
					product:packet.id,
					receipt:receiptData.receipt,
					packageName:packageName,
					accessToken:accessToken,
					feeUSD:feeUSD,
					fee:fee,
					feeCurrency:feeCurrency,
					sandBox:sandBox
				};

				if(user)
					params['uid']=user.id;
				else
					params['uid']=deviceId;
			}

			var urlPath:String="iap/verify";
			if(receiptData.hasOwnProperty("restoreUrl") && receiptData.restoreUrl){
				urlPath="iap/recovery";
			}

			var urlLoad1:URLLoader=new URLLoader();
			var urlreq1:URLRequest = new URLRequest(Constants.API_URL_api+urlPath);

			urlreq1.idleTimeout= Constants.IMAGE_REQUESTS_TIMEOUT*10;
			urlreq1.requestHeaders = new Array(new URLRequestHeader("Cookie",
				"token="+_tokenCooke));

			trace('JSON.stringify(params)', JSON.stringify(params));

			urlreq1.contentType="application/json";
			urlreq1.data = JSON.stringify(params);

			urlreq1.manageCookies=true;
			urlreq1.method = URLRequestMethod.POST;

			urlLoad1.addEventListener(IOErrorEvent.IO_ERROR,iohandlerVerify);
			urlLoad1.addEventListener(Event.COMPLETE,compHandlerVerify);
			urlLoad1.addEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandlerVerify);

			urlLoad1.load(urlreq1);
		}


		private function iohandlerVerify(e:IOErrorEvent):void
		{
			iapVerifyPacket=null;
			if (iapVerifyCallback) {
				iapVerifyCallback(false, "IOError");
			}
			iapVerifyCallback=null;

			var loader:URLLoader=e.target as URLLoader;

			loader.removeEventListener(IOErrorEvent.IO_ERROR,iohandlerVerify);
			loader.removeEventListener(Event.COMPLETE,compHandlerVerify);
			loader.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandlerVerify);

		}

		private function packetExistsInArray(purchases:Array, packet_id:String):Boolean{

			for(var index : int in purchases){

				if(purchases[index].product==packet_id){
					return true;
				}
			}

			return false;
		}

		private function compHandlerVerify(e:Event):void{
			var loader:URLLoader=e.target as URLLoader;
			loader.removeEventListener(IOErrorEvent.IO_ERROR,iohandlerVerify);
			loader.removeEventListener(Event.COMPLETE,compHandlerVerify);
			loader.removeEventListener(HTTPStatusEvent.HTTP_STATUS,httpHandlerVerify);

			try{
				var result:Object=JSON.parse(URLLoader(e.target).data);

				var verify:Boolean=false;

				if(result && result.hasOwnProperty('verify')){

					verify=result.verify;
				}

				var json:String=readPurchases();
				var resultPurchases:Array=JSON.parse(json) as Array;
				for(var index : int in resultPurchases){

					if(!packetExistsInArray(resultPurchases, iapVerifyPacket.id)){

						resultPurchases.push({product:iapVerifyPacket.id, productId:iapVerifyPacket.productId});
						savePurchases(JSON.stringify(resultPurchases));
					}
				}


				if(verify && iapVerifyCallback){
					iapVerifyCallback(true, "");
					iapVerifyCallback=null;
					iapVerifyPacket=null;

				}else{
					if (iapVerifyCallback) {
						iapVerifyCallback(false, "error");
					}
					iapVerifyCallback=null;
					iapVerifyPacket=null;
				}




			}catch(e:Error){

			}
		}

		private function sendGetPurchases(iapVerifyCallbackParam:Function, iapVerifyPacketParam:Packet):void{

			if(purchasesCallback==null){

				getPurchases(function(result:Object):void{

					for each(var purchase:Object in result){

						var product_id:String=purchase.product;

						if(product_id==iapVerifyPacketParam.id){
							iapVerifyCallbackParam(true, "");
							iapVerifyCallback=null;
							iapVerifyPacket=null;

							return;
						}
					}

					iapVerifyCallbackParam(false, "Error");
					iapVerifyCallback=null;
					iapVerifyPacket=null;
					_receipt=null;

				});

			}else{
				setTimeout(sendGetPurchases, 100, iapVerifyCallbackParam, iapVerifyPacketParam);
			}

		}

		private function httpHandlerVerify(e:HTTPStatusEvent):void{

			var responseHeader:Array=e.responseHeaders;

			if(responseHeader){

				for(var i:int=0; i<responseHeader.length; i++){

					if(responseHeader[i].name=="Set-Cookie"){

						var value:String=responseHeader[i].value;
						_tokenCookeGuest=value.replace("token=", "");
						_tokenCookeGuest=_tokenCookeGuest.substr(0, _tokenCooke.indexOf(";"));

					}
				}
			}
		}

		public function savePurchases(packages:String):void{
			var file_nameStr:String = "purchases_new";
			var file_name:String=file_nameStr + ".json";
			if(!Device.isIos()){
				if(currentUser ){
					file_name=file_nameStr + currentUser.id+".json";
				}
			}
			packages = FileUtil.fixJSONData(packages);
			var file1: File = File.cacheDirectory.resolvePath(file_name);
			if(!FileUtil.putFileContent(file1, packages)){
				file1 = File.applicationStorageDirectory.resolvePath(file_name);
				if(!FileUtil.putFileContent(file1, packages)){
					file1 = File.applicationDirectory.resolvePath(file_name);
					FileUtil.putFileContent(file1, packages);
				}
			}
		}

		public function readPurchases():String{
			try{
				var file_nameStr:String = "purchases_new";
				var file_name:String=file_nameStr + ".json";
				if(!Device.isIos()){
					if(currentUser ){
						file_name = file_nameStr+currentUser.id+".json";
					}
				}
				var file1: File = File.cacheDirectory.resolvePath(file_name);
				var jsonStr:String;
				if(file1.exists){
					jsonStr = FileUtil.getFileContent(file1);
					jsonStr = FileUtil.fixJSONData(jsonStr);
					return jsonStr;
				}
				file1 = File.applicationStorageDirectory.resolvePath(file_name);
				if(file1.exists){
					jsonStr = FileUtil.getFileContent(file1);
					jsonStr = FileUtil.fixJSONData(jsonStr);
					return jsonStr;
				}
				file1 = File.applicationDirectory.resolvePath(file_name);
				if(file1.exists){
					jsonStr = FileUtil.getFileContent(file1);
					jsonStr = FileUtil.fixJSONData(jsonStr);
					return jsonStr;
				}
				return "[]";
			}catch(e:Error){
				return "[]";
			}
		}

		public function get purchasesLoaded():Boolean {
			return _purchasesLoaded;
		}
	}
}