package com.despark.tuning3d.model.votes
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.http.HttpClient;
	import com.despark.tuning3d.model.core.images.Base64;
	import com.despark.tuning3d.model.packages.PackagesParser;
	import com.despark.tuning3d.model.packages.Packet;
	import com.despark.tuning3d.model.parts.Part;
	import com.despark.tuning3d.model.parts.PartDisk;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.sharing.SharingProxy;
	import com.despark.tuning3d.model.users.User;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.hurlant.math.bi_internal;
	import com.utils.Logger;
	
	import flash.display.BitmapData;
	import flash.display.PNGEncoderOptions;
	import flash.events.DataEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.geom.Matrix;
	import flash.net.URLVariables;
	import flash.system.Capabilities;
	import flash.utils.ByteArray;
	
	import mx.utils.StringUtil;
	
	import org.puremvc.as3.patterns.proxy.Proxy;
	import by.blooddy.crypto.serialization.JSONer;
	
		
	public class VotesProxy extends Proxy
	{
		public static const NAME : String = "VotesProxy";
		
		public var httpClient : HttpClient;
		
		private var youVotes:Object={};
		
		private var youVotesFile : File;           
		
		public function VotesProxy()
		{
			super(NAME);
			
			
			youVotesFile = File.applicationStorageDirectory.resolvePath("youVotes.json");
			
			loadVotesFromFile();
			//.resolvePath("pendingTuning.json");
			//pendingTuningFile = File.cacheDirectory.resolvePath("pendingTuning.json")
			//pendingTuningFile.preventBackup = true;
		}
		
		private function filePutContent(file:File, content:String):void{
			
			var fileStream:FileStream = new FileStream();
			fileStream.open(file, FileMode.WRITE);
			
			fileStream.writeUTFBytes(content);
			
			fileStream.close();
		}
		
		private function fileGetContent(file:File):String{
			
			var fileStream:FileStream = new FileStream();
			
			fileStream.open(file, FileMode.READ);
			
			var yourJSONdata:String = fileStream.readUTFBytes(fileStream.bytesAvailable);
			
			fileStream.close();
			
			
			return yourJSONdata;
		}
		
		private function loadVotesFromFile():void{
			
			if(youVotesFile.exists){
				
				youVotes = JSONer.parse(fileGetContent(youVotesFile));
				
				
			}else
			
				youVotes={};
		}
		
		private function saveVotesToFile():void{
			
			var json:String=JSONer.stringify(youVotes);
			
			filePutContent(youVotesFile, json);
		}
		
		private function getCurrentTime():Number{
			
			var date:Date=new Date();
			
			return date.getTime();
		}
		
		public function vote(id:String):Boolean{
			
			
			var currentTime:Number=getCurrentTime();
			
			if(youVotes[id] && currentTime-youVotes[id].time<86400000)
				return false;
			
			
			youVotes[id]={time:currentTime};
			
			
			httpClient.getPath(
				"voting/"+id, null, 
				function(success : Boolean, result : Object) : void
				{
					
				
					
					if(success)
					{
						Tracer.print("успешный голос")
						//sendNotification(Notifications.VOTES_LOADED, {votes: votes});
					}
					else
					{

                        Tracer.print("не удалось сохранить голос");
						//sendNotification(Notifications.VOTES_LOADING_FAILED);
					}
				});
			
			saveVotesToFile();
			
			return true;
			
		}
		
		public function delete_vote(id):Boolean{
			
			var currentTime:Number=getCurrentTime();
			
			if(!youVotes[id] )
				return false;
				
			httpClient.putPath(
				"voting/"+id, null, 
				function(success : Boolean, result : Object) : void
				{	
					if(success)
					{
                        Tracer.print("успешно удалили голос")
						//sendNotification(Notifications.VOTES_LOADED, {votes: votes});
					}
					else
					{
                        Tracer.print("не удалось удалить  голос")
						//sendNotification(Notifications.VOTES_LOADING_FAILED);
					}
				});
			
			saveVotesToFile();
			
			delete youVotes[id];
			
			return true;
		}
		
		public function load(search:String="", votingType:String = "all_cars", sortBy:String = null, sortByOrder:String = null) : void {
			var currentTime:Number=getCurrentTime();
			var searchUpCase:String=search.toUpperCase();
			searchUpCase=StringUtil.trim(searchUpCase);
			var params:Object = {votingType: votingType, sortingType:sortBy, sortingOrder: sortByOrder};
			// http://api.3dtuning.com/api/v1/voting?forSite=true&limit=20&skip=0&sortingOrder=desc&sortingType=name&votingType=american_truck
			httpClient.getPath(
				"voting", params,
				function(success : Boolean, result : Object) : void {
				if(success) {
					var votesJSON : Object = JSONer.parse(String(result));
					var votes : Array = [];
					var newVotesJSON:Array=[];
					for (var voteJSONIndex : int in votesJSON) {
						newVotesJSON.push(votesJSON[voteJSONIndex]);
					}
					var vote : Vote;
					var order:int=1;
					for each(var voteJSON : Object in newVotesJSON) {
						var name:String=voteJSON.make+" "+voteJSON.model;
						name = name.toUpperCase();
						if (searchUpCase=="" || (searchUpCase!="" &&  name.indexOf(searchUpCase) >= 0)) {
							vote = new Vote();						
							vote.id = voteJSON._id;
							vote.make = voteJSON.make;
							vote.model = voteJSON.model;
							vote.name = voteJSON.make+" "+voteJSON.model;
							vote.image = voteJSON.image;
							vote.showVotingCount = voteJSON.showVotingCount;
							vote.voting = voteJSON.voting;
							vote.v = voteJSON['__v'];
							vote.order = order;
							vote.youHaveLiked = youVotes[vote.id] && currentTime - youVotes[vote.id].time < 86400000;
							votes.push(vote);			
						}
						order++;
					}
					sendNotification(Notifications.VOTES_LOADED, {votes: votes});  
				} else {
					sendNotification(Notifications.VOTES_LOADING_FAILED);
				}
			});
		}
		public function load_old(search:String="", votingType:String = "all_cars", sortBy:String = null) : void {
			var currentTime:Number=getCurrentTime();
			var searchUpCase:String=search.toUpperCase();

			searchUpCase=StringUtil.trim(searchUpCase);
			var params:Object = {votingType: votingType, sortingType:sortBy};
			// http://api.3dtuning.com/api/v1/voting?forSite=true&limit=20&skip=0&sortingOrder=desc&sortingType=name&votingType=american_truck
			httpClient.getPath(
				"voting", params,
				function(success : Boolean, result : Object) : void
			{
					//Времменая штука пока апи не работают нормально
				if(!success){
					var json:String='[{"_id":"583e9d91373ea60440558618","make":"BMW","model":"Z3 Coupe Coupe 1999","image":"//data.3dtuning.com/news/BM__Kaojz.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9d9a373ea6044055861a","make":"BMW","model":"5 series Sedan 2003","image":"//data.3dtuning.com/news/BwgmYq8PP.jpg","showVotingCount":true,"voting":6,"__v":0},{"_id":"583e9daf373ea6044055861d","make":"BMW","model":"1 series 5 Door Hatchback 2005","image":"//data.3dtuning.com/news/lM45sWUPz.jpg","showVotingCount":true,"voting":4,"__v":0},{"_id":"583e9dab373ea6044055861b","make":"BMW","model":"3 series Sedan 2012","image":"//data.3dtuning.com/news/RMI5YaEzj.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9dad373ea6044055861c","make":"BMW","model":"1 Series 3 Door Hatchback 2009","image":"//data.3dtuning.com/news/R0E5IGojP.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9db1373ea6044055861e","make":"Austin-Healey","model":"3000 Convertible 1959","image":"//data.3dtuning.com/news/RwMRIaozz.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9db3373ea6044055861f","make":"Audi","model":"Q7 SUV 2016","image":"//data.3dtuning.com/news/RwX4cG8zP.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9db5373ea60440558620","make":"Audi","model":"A6 Allroad Quattro Wagon 2013","image":"//data.3dtuning.com/news/B0D4IGoj3.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9db8373ea60440558621","make":"Audi","model":"S5 Sportback Sedan 2011","image":"//data.3dtuning.com/news/R01QIaEPz.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9db9373ea60440558622","make":"Audi","model":"TT Roadster 2007","image":"//data.3dtuning.com/news/5gJPIGEzj.jpg","showVotingCount":true,"voting":23,"__v":0},{"_id":"583e9dbc373ea60440558623","make":"Audi","model":"A4 Sedan 2005","image":"//data.3dtuning.com/news/Rw83Iao3j.jpg","showVotingCount":true,"voting":6,"__v":0},{"_id":"583e9dbe373ea60440558624","make":"Aston Martin","model":"V12 Zagato Coupe 2012","image":"//data.3dtuning.com/news/R0jPIaozj.jpg","showVotingCount":true,"voting":16,"__v":0},{"_id":"583e9dc0373ea60440558625","make":"Aston Martin","model":"V8 Vantage Coupe 1993","image":"//data.3dtuning.com/news/5gKOsq83z.jpg","showVotingCount":true,"voting":22,"__v":0},{"_id":"583e9dc2373ea60440558626","make":"Aston Martin","model":"Rapide sedan 2010","image":"//data.3dtuning.com/news/B0FycqUP3.jpg","showVotingCount":true,"voting":24,"__v":0},{"_id":"583e9dc5373ea60440558627","make":"Aston Martin","model":"DBS Volante Convertible 2010","image":"//data.3dtuning.com/news/R0k2cGUjz.jpg","showVotingCount":true,"voting":5,"__v":0},{"_id":"583e9dc7373ea60440558628","make":"Aston Martin","model":"Cygnet 3 Door 2012","image":"//data.3dtuning.com/news/B00ycG8zj.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9dca373ea60440558629","make":"Ascari","model":"KZ1R Coupe 2005","image":"//data.3dtuning.com/news/BMshsaUzj.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9dcb373ea6044055862a","make":"Alfa Romeo","model":"Brera 3 Door 2005","image":"//data.3dtuning.com/news/RwDNsGozz.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9dcd373ea6044055862b","make":"Alfa Romeo","model":"159 4 Door Saloon 2006","image":"//data.3dtuning.com/news/lM1hYq8PP.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9dcf373ea6044055862c","make":"Alfa Romeo","model":"159 Sportwagon 2005","image":"//data.3dtuning.com/news/lMZ0cqoPz.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9dd1373ea6044055862d","make":"Alfa Romeo","model":"156 4 Door Saloon 1997","image":"//data.3dtuning.com/news/50U0sqUjz.jpg","showVotingCount":true,"voting":5,"__v":0},{"_id":"583e9dd3373ea6044055862e","make":"Alfa Romeo","model":"147 3 Door Hatchback 2010","image":"//data.3dtuning.com/news/BgjgIWoP3.jpg","showVotingCount":true,"voting":2,"__v":0},{"_id":"583e9dd5373ea6044055862f","make":"Acura","model":"TSX sedan 2009","image":"//data.3dtuning.com/news/RgLvXqEjP.jpg","showVotingCount":true,"voting":10,"__v":0},{"_id":"583e9dd7373ea60440558630","make":"Acura","model":"RSX Coupe 2005","image":"//data.3dtuning.com/news/5ga-rW8Pz.jpg","showVotingCount":true,"voting":3,"__v":0},{"_id":"583e9dd8373ea60440558631","make":"Acura","model":"NSX Coupe 1997","image":"//data.3dtuning.com/news/50kfbWU3P.jpg","showVotingCount":true,"voting":5,"__v":0},{"_id":"583e9dda373ea60440558632","make":"Acura","model":"Acura NSX Coupe 2005","image":"//data.3dtuning.com/news/BgeubaojP.jpg","showVotingCount":true,"voting":13,"__v":0},{"_id":"583e9ddc373ea60440558633","make":"Acura","model":"Integra Type-R Coupe 2001","image":"//data.3dtuning.com/news/5wGKraUP3.jpg","showVotingCount":true,"voting":9,"__v":0},{"_id":"583e9dde373ea60440558634","make":"AMC","model":"Pacer X 3 Door Hatchback 1975","image":"//data.3dtuning.com/news/lM5ubWEzz.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9de0373ea60440558635","make":"AMC","model":"Javelin-AMX Coupe 1971","image":"//data.3dtuning.com/news/5gwKrGUzz.jpg","showVotingCount":true,"voting":1,"__v":0},{"_id":"583e9de2373ea60440558636","make":"AMC","model":"Gremlin X 3 Door Hatchback 1970","image":"//data.3dtuning.com/news/5M8GqaE3z.jpg","showVotingCount":true,"voting":3,"__v":0}]';
					result=json;
					success=true;
				}

				if(success)
				{
					var votesJSON : Object = JSONer.parse(String(result));
					var votes : Array = [];

					function orderSortKey(a,b):int
					{
						var a_i : int = int(a.voting);
						var b_i : int = int(b.voting);

						if(a_i<b_i)return 1;
						if(a_i>b_i)return -1;

						if(a_i==b_i){
							a.name = a.make+" "+a.model;
							b.name = b.make+" "+b.model;
							var list:Array=[a, b];
							list.sortOn("name");

							return list.indexOf(a)==0 ? -1:1;
						}
						return 0;
					}

					var newVotesJSON:Array=[];
					for (var voteJSONIndex : int in votesJSON) {
						//votesJSON[voteJSONIndex].name=(votesJSON.make+" "+votesJSON.model).toLowerCase();
						newVotesJSON.push(votesJSON[voteJSONIndex]);
					}

					//newVotesJSON.sortOn("make", "model");
					newVotesJSON.sort(orderSortKey);

					var vote : Vote = new Vote();
					var order:int=1;
					for each(var voteJSON : Object in newVotesJSON)
					{
						var name:String=voteJSON.make+" "+voteJSON.model;
						name=name.toUpperCase();

						if (searchUpCase=="" || (searchUpCase!="" &&  name.indexOf(searchUpCase) >= 0))
						{
							vote = new Vote();
							vote.id=voteJSON._id;
							vote.make=voteJSON.make;
							vote.model=voteJSON.model;
							vote.name=voteJSON.make+" "+voteJSON.model;
							vote.image=voteJSON.image;
							vote.showVotingCount=voteJSON.showVotingCount;
							vote.voting=voteJSON.voting;
							vote.v=voteJSON['__v'];
							vote.order=order;
							vote.youHaveLiked=youVotes[vote.id] && currentTime-youVotes[vote.id].time<86400000;
							votes.push(vote);
						}

						order++;
					}

					sendNotification(Notifications.VOTES_LOADED, {votes: votes});
				}
				else
				{
					sendNotification(Notifications.VOTES_LOADING_FAILED);
				}
			});
		}
	}
}