package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesScreen;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.utils.ArrayUtil;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class ChallengesScreenMediator  extends StackedBannerScreenMediator {
        public function ChallengesScreenMediator(stackedScreen : StackedScreen) {
            super(stackedScreen);
        }
        override public function onRegister() : void {
            super.onRegister();
            challengesScreen.addEventListener(StackedScreen.HOME_SELECTED, challengeScreenHomeSelectedSelectedHandler); //+
            challengesScreen.addEventListener(StackedScreen.BACK_BUTTON_SELECTED, backButtonSelectedSelectedHandler); //+
            updateNotifications();
            //
            SoundManager.playSound(SoundManager.SELECT_PART);
        }

        override public function onRemove() : void {
            super.onRemove();
            // ***************************
            challengesScreen.removeEventListener(StackedScreen.HOME_SELECTED, challengeScreenHomeSelectedSelectedHandler); //+
            challengesScreen.removeEventListener(StackedScreen.BACK_BUTTON_SELECTED, backButtonSelectedSelectedHandler); //+
        }


        override public function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_AUTHENTICATED,
                Notifications.REWARD_ARS_LOAD,
                Notifications.REWARD_DATA_CHANGE,
                Notifications.DISPLAY_CHALLENGE_LOADER,
                Notifications.HIDE_CHALLENGE_LOADER,
                Notifications.USER_LOGIN_CANCELLED
            ]);
        }

        private function updateNotifications():void{
            var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            if(UsersProxy.loggedInUser()){
                challengesProxy.getUnreadNotificationCount(function (result:Object):void {
                    if(result && result.hasOwnProperty("count")){
                        Constants.UNREAD_CHALLENGE_NOTIFICATIONS = result.count;
                    }else{
                        Constants.UNREAD_CHALLENGE_NOTIFICATIONS = 0;
                    }
                    challengesScreen.updateNotifications();
                });
            }else{
                Constants.UNREAD_CHALLENGE_NOTIFICATIONS = 0;
                challengesScreen.updateNotifications();
            }
        }

        override protected function homeSelectedHandler(event:Event):void {

        }

        private function userAuthenticate(pendingForAction : String):void {
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

            if (!usersProxy.currentUser) _garage_delay = 1.5;
            else _garage_delay = 0.2;

            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

            pendingForAuthAction = pendingForAction;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.DISPLAY_CHALLENGE_LOADER:
                    challengesScreen.showLoadBar();
                    break;
                case Notifications.HIDE_CHALLENGE_LOADER:
                    challengesScreen.hideLoadBar();
                    break;
                case Notifications.USER_LOGIN_CANCELLED:
                    challengesScreen.hideLoadBar();
                    challengesScreen.restoreScreen();
                    break;
            }
            super.handleNotification(note);
        }

        public function get challengesScreen():ChallengesScreen {
            return viewComponent as ChallengesScreen;
        }
    }
}
