package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.components.challenges.ChallengesTuningsListScreen;
    import com.despark.tuning3d.view.components.challenges.modals.ChallengeTuningViewPopup;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;
    import com.utils.Console;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class ChallengesTuningsListScreenMediator extends ChallengesScreenMediator {
        public function ChallengesTuningsListScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            Constants.selectedUser = null;
            challengesListScreen.showLoadBar();
            challengesListScreen.addEventListener(ChallengeEvent.TUNING_SELECTED, tuningSelectedHandler);
            challengesListScreen.addEventListener(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST, loadChallengeList);
        }

        override public function onRemove():void {
            super.onRemove();
            challengesListScreen.dispose();
        }

        public override function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.REMOVE_POP_UP
            ]);
        }

        private function loadChallengeList(event:Event):void{
            if(challengesListScreen.isLoading || challengesListScreen.skip % challengesListScreen.limit != 0){
                return;
            }
            challengesListScreen.isLoading = true;
            if(challengesListScreen.skip == 0){
                challengesListScreen.showLoadBar();
            }
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getChallengeLeaderboardData(challengesListScreen.challengeId, function(result:Object, success:Boolean = true, userBuild:Object = null):void{
                if(success){
                    if(userBuild){
                        challengesListScreen.loadUserBuild(userBuild);
                    }
                    challengesListScreen.loadTunings(result as Array);
                    challengesListScreen.hideLoadBar();
                }
                challengesListScreen.isLoading = false;
            }, challengesListScreen.limit, challengesListScreen.skip);
        }

        protected function tuningSelectedHandler(event:Event):void{
            AdsManager.$.bannerHide();
            sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, event.data);
        }

        override protected function challengeScreenHomeSelectedSelectedHandler(event:Event):void{
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_SCREEN, {"tabMenu": true});
        }

        public function get challengesListScreen():ChallengesTuningsListScreen {
            return viewComponent as ChallengesTuningsListScreen;
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.REMOVE_POP_UP:
                    if(note.getBody() is ChallengeTuningViewPopup){
                        BannerStackedScreen.showAdBanner();
                    }
                    break;
            }
            super.handleNotification(note);
        }
    }
}
