package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.components.challenges.ChallengesUserTuningsScreen;
    import com.despark.tuning3d.view.components.challenges.modals.ChallengeTuningViewPopup;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

    public class ChallengesUserTuningsScreenMediator extends ChallengesScreenMediator {
        public function ChallengesUserTuningsScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            Constants.selectedUser = null;
            challengesListScreen.addEventListener(ChallengeEvent.TUNING_SELECTED, tuningSelectedHandler);
            challengesListScreen.addEventListener(StackedScreen.BACK_BUTTON_SELECTED, backButtonTriggeredHandler);
            challengesListScreen.addEventListener(ChallengeEvent.REQUEST_CHALLENGE_TUNING_LIST, loadChallengeList);
        }

        override public function onRemove():void {
            super.onRemove();
            challengesListScreen.removeEventListener(ChallengeEvent.TUNING_SELECTED, tuningSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.BACK_BUTTON_SELECTED, backButtonTriggeredHandler);
            challengesListScreen.dispose();
        }

        public override function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.REMOVE_POP_UP
            ]);
        }

        private function tuningSelectedHandler(event:Event):void {
            AdsManager.$.bannerHide();
            sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, event.data);
        }

        private function loadChallengeList(event:Event):void{
            if(challengesListScreen.isLoading || challengesListScreen.skip % challengesListScreen.limit != 0){
                return;
            }
            challengesListScreen.isLoading = true;
            if(challengesListScreen.skip == 0){
                challengesListScreen.showLoadBar();
            }
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getUserChallengeTunings(challengesListScreen.garageOwner.id, function(result:Object, success:Boolean = true):void{
                if(success){
                    challengesListScreen.loadTunings(result as Array);
                    challengesListScreen.hideLoadBar();
                }
                challengesListScreen.isLoading = false;
            }, challengesListScreen.limit, challengesListScreen.skip, event.data.bodyType);
        }

        protected function backButtonTriggeredHandler(event:Event):void{
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_SCREEN);
        }

        override protected function challengeScreenHomeSelectedSelectedHandler(event:Event):void{
            if(challengesListScreen.userGarage){
                sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
            }else{
                sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_SCREEN, {"tabMenu": true});
            }
        }

        public function get challengesListScreen():ChallengesUserTuningsScreen {
            return viewComponent as ChallengesUserTuningsScreen;
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.REMOVE_POP_UP:
                    if(note.getBody() is ChallengeTuningViewPopup){
                        BannerStackedScreen.showAdBanner();
                    }
                    break;
            }
            super.handleNotification(note);
        }
    }
}
