package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.BrandsScreenMediator;
    import com.despark.tuning3d.view.components.challenges.ChallengesUserTuningsScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesVotingScreen;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.utils.ArrayUtil;
    import com.utils.Console;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

    public class ChallengesVotingScreenMediator extends ChallengesScreenMediator {
        private var challengeProxy : ChallengesProxy;
        private var eventData:Object;
        public function ChallengesVotingScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            challengeProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            Constants.selectedUser = null;
            challengesVotingScreen.showLoadBar();
            challengesVotingScreen.addEventListener(ChallengeEvent.LOAD_CHALLENGE_PAIR, loadChallengePairHandler);
            challengesVotingScreen.addEventListener(ChallengeEvent.VOTE_FOR_CHALLENGE, voteForChallengeHandler);
            challengesVotingScreen.addEventListener(ChallengeEvent.ADD_CHALLENGE_KEY, completeChallengeVoting);
            loadChallenge();
        }

        private function voteForChallengeHandler(event:Event):void {
            eventData = event.data;
            challengeProxy.voteForChallenge(event.data, function (success:Boolean, result:Object):void {
                if(success){
                    challengesVotingScreen.showVoteResult(result);
                }else{
                    challengesVotingScreen.challengeVotingFailedHandler(addKeysForChallenge);
                }
            });
        }

        private function completeChallengeVoting(event:Event):void {
            addKeysForChallenge();

            var challengeId:String;
            if(event.data && event.data._id){
                challengeId = event.data._id;
            }
            if(challengeId){
                challengeProxy.addVoteKeyForChallenge(challengeId, function (result:Object):void {
                    if(result){
                        trace();
                    }else{
                        trace();
                    }
                });
            }
        }

        private function addKeysForChallenge():void{
            var keys:int = Math.min(3, Constants.MAX_KEY_AMOUNT - challengesVotingScreen.user.keys);
            challengeProxy.addKeyForChallenge(UsersProxy.loggedInUser().id, keys, function (result:Object):void {
                if(result){
                    challengesVotingScreen.updateUserData(result);
                    var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.KEYS_RECEIVED, {keys: keys, totalKeys:challengesVotingScreen.user.keys, maxKeys: Constants.MAX_KEY_AMOUNT}, challengesVotingScreen);
                    statsDialog.open();
                }else{
                    trace();
                }
            });
        }

        override public function onRemove():void {
            super.onRemove();
            challengesVotingScreen.removeEventListener(ChallengeEvent.LOAD_CHALLENGE_PAIR, loadChallengePairHandler);
            challengesVotingScreen.removeEventListener(ChallengeEvent.VOTE_FOR_CHALLENGE, voteForChallengeHandler);
            challengesVotingScreen.dispose();
            challengeProxy = null;
        }

        override public function listNotificationInterests() : Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_LOGGED_OUT,
                Notifications.USER_AUTHENTICATED
            ]);
        }

        override protected function handleUserAuthenticated():void{
            super .handleUserAuthenticated();
            switch(pendingForAuthAction){
                case StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_VOTE:
                    if(stackedScreen is ChallengesVotingScreen){
                        loadChallenge();
                    }
                    break;
            }

        }

        private function loadChallengePairHandler(event:Event):void {
            eventData = event.data;
            if(UsersProxy.loggedInUser()){
                if(eventData){
                    challengeProxy.getChallengeVotePair(eventData._id, function(result:Object, success:Boolean = true):void{
                        if(success && result && !result.hasOwnProperty("msg")){
                            challengesVotingScreen.displayChallengePair(result);
                        }else{
                            if(challengesVotingScreen.votedTuningCount){
                                challengesVotingScreen.challengeVotingFailedHandler(addKeysForChallenge);
                            }else{
                                challengesVotingScreen.challengeVotingFailedHandler();
                            }
                        }
                        challengesVotingScreen.hideLoadBar();
                    });
                }
            }else{
                challengesVotingScreen.showLoadBar();
                userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_VOTE);
            }

        }

        private function loadChallenge():void{
            challengesVotingScreen.showLoadBar();
            challengeProxy.getChallengeVoteList(function(result:Object, success:Boolean = true):void{
                if(success){
                    challengesVotingScreen.loadChallenges(result);
                    challengesVotingScreen.hideLoadBar();
                }else{
                    challengesVotingScreen.loadChallenges(null);
                    challengesVotingScreen.hideLoadBar();
                }
            });
        }

        override protected function challengeScreenHomeSelectedSelectedHandler(event:Event):void{
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        public function get challengesVotingScreen():ChallengesVotingScreen {
            return viewComponent as ChallengesVotingScreen;
        }

        override public function handleNotification(note:INotification):void {
            super.handleNotification(note);
            switch (note.getName()) {

            }
        }
    }
}
