package com.despark.tuning3d.view
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarsProxy;
	import com.despark.tuning3d.model.core.images.BitmapDataUtil;
	import com.despark.tuning3d.model.parts.Part;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.parts.PartSlotsProxy;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.PopUp;
	import com.despark.tuning3d.view.components.downloads.DownloadsDefCarSlotView;
	import com.despark.tuning3d.view.components.downloads.DownloadsPartSlotView;
	import com.despark.tuning3d.view.components.downloads.DownloadsPopUp;
	import com.despark.tuning3d.view.components.downloads.DownloadsProgressView;
	import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;
import com.utils.Logger;
	
	import flash.desktop.NativeApplication;
	import flash.desktop.SystemIdleMode;
	
	import org.puremvc.as3.interfaces.INotification;
	
	import starling.core.Starling;
	import starling.events.Event;

	public class DownloadsPopUpMediator extends PopUpMediator
	{
		private var sizesSum : Number = 0;
		private var selectedPartSlots : Array;
		private var partSlotsProxy : PartSlotsProxy;
		private var carsProxy : CarsProxy;
		private var downloadsProgressView : DownloadsProgressView;
		private var expandedPartSlots : Array;
		private var SaveDefCar: Boolean;
		
		public function DownloadsPopUpMediator(popUp : PopUp)
		{
			
			//Logger.d("DownloadsPopUpMediator", this);	
			
			super(popUp);
			
			selectedPartSlots = [];
			
			partSlotsProxy = facade.retrieveProxy(PartSlotsProxy.NAME) as PartSlotsProxy;
			carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
		}
		
		override public function listNotificationInterests():Array 
		{
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.CAR_PARTS_DOWNLOADED,
				Notifications.CAR_PARTS_DOWNLOAD_FAILED,
				Notifications.CAR_PARTS_DOWNLOAD_CACHING_FAILED
			])
		}
		
		override public function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case Notifications.CAR_PARTS_DOWNLOADED:
					handlePartsDownloaded();
					break;
				
				case Notifications.CAR_PARTS_DOWNLOAD_FAILED:
					handlePartsFailedDownload();
					break;
				
				case Notifications.CAR_PARTS_DOWNLOAD_CACHING_FAILED:
					handlePartsDownloadCachingFailed();
					break;
			}
			
			super.handleNotification(note);
		}
		
		public function get downloadsPopUp() : DownloadsPopUp
		{
			return DownloadsPopUp(popUp);
		}
		
		override public function onRegister() : void
		{
			super.onRegister();
			
			downloadsPopUp.addEventListener(Event.ADDED_TO_STAGE, downloadsPopUpAddedToStageHandler);
			downloadsPopUp.addEventListener(DownloadsPopUp.DELETE_SELECTED, downloadsPopUpDeleteSelectedHandler);
			downloadsPopUp.addEventListener(DownloadsPopUp.DOWNLOAD_SELECTED, downloadsPopUpDownloadSelectedHandler);
			downloadsPopUp.addEventListener(DownloadsPartSlotView.SELECTION_STATE_CHANAGED, downloadsPartSlotViewSelectionStateChangedHandler);
			downloadsPopUp.addEventListener(DownloadsDefCarSlotView.SELECTION_DEF_CAR_STATE_CHANAGED, downloadsDefCarSlotViewSelectionStateChangedHandler);
			
			downloadsPopUp.addEventListener(DownloadsPopUp.CLEAN_CACHE_SELECTED, downloadsPopUpDownloadCleanCacheHandler);
			//
		}
		
		override public function onRemove() : void
		{
			super.onRemove();
			
			
			downloadsPopUp.removeEventListener(Event.ADDED_TO_STAGE, downloadsPopUpAddedToStageHandler);
			downloadsPopUp.removeEventListener(DownloadsPopUp.DELETE_SELECTED, downloadsPopUpDeleteSelectedHandler);
			downloadsPopUp.removeEventListener(DownloadsPopUp.DOWNLOAD_SELECTED, downloadsPopUpDownloadSelectedHandler);
			downloadsPopUp.removeEventListener(DownloadsPartSlotView.SELECTION_STATE_CHANAGED, downloadsPartSlotViewSelectionStateChangedHandler);
			downloadsPopUp.removeEventListener(DownloadsDefCarSlotView.SELECTION_DEF_CAR_STATE_CHANAGED, downloadsDefCarSlotViewSelectionStateChangedHandler);
			
			downloadsPopUp.removeEventListener(DownloadsPopUp.CLEAN_CACHE_SELECTED, downloadsPopUpDownloadCleanCacheHandler);
			
						
			//Logger.d("onRemove", this);
		}
		
		private function downloadsPopUpAddedToStageHandler(event : Event) : void
		{
			downloadsPopUp.setDownloadButtonEnabled(false);
			downloadsPopUp.setDeleteButtonEnabled(false);
			downloadsPopUp.totalValueLabel.text = "0 MB";
			
			var car : Car = downloadsPopUp.car;
			
			downloadsPopUp.enterIntializationState();
			
			Starling.juggler.delayCall(function() : void
			{
				expandedPartSlots = getExpandedPartSlots("disks", 3);
				
				carsProxy.updatePartSlotsSizes(car, expandedPartSlots);
				downloadsPopUp.partSlotsForDownload = expandedPartSlots;
				
				downloadsPopUp.enterSelectionState();
			}, 0.5);
		}
		
		private function getExpandedPartSlots(partSlotToExpandName : String, groups : int) : Array
		{
			var partSlotsForDownload : Array = [];
			
			for each(var partSlot : PartSlot in downloadsPopUp.car.possiblePartSlotsSorted)
			{	
				if(partSlot.name!="masks"){
					if(partSlot.name != partSlotToExpandName)
					{
						partSlotsForDownload.push(partSlot);   
					}
					else
					{
						expandPartSlot(partSlot, partSlotsForDownload, groups);
					}
				}
			}
			
			return partSlotsForDownload;
		}
		
		private function expandPartSlot(partSlot : PartSlot, partSlotsForDownload : Array, groups : int) : void
		{
			var parts : Array = partSlot.possibleParts;
			
			if(parts.length < groups)
			{
				partSlotsForDownload.push(partSlot);
				return;
			}
			
			var groupSize : int = int(parts.length / groups);
			var newPartSlot : PartSlot;
			var addedPartSlots : Array = [];
			
			for(var i : int; i < parts.length; i++)
			{
				if(i % groupSize == 0 && parts.length - i >= groupSize)
				{
					newPartSlot = new PartSlot(partSlot.parentCar);
					addedPartSlots.push(newPartSlot);
					partSlotsForDownload.push(newPartSlot);
					
					newPartSlot.displayName = partSlot.displayName +  " " + addedPartSlots.length;
					newPartSlot.nameSuffix = " " + addedPartSlots.length;
					newPartSlot.name = partSlot.name;
				}
				
				newPartSlot.addPossiblePart(parts[i]);
				parts[i].parentSlot = partSlot;
			}
			
			for each(var addedPartSlot : PartSlot in addedPartSlots)
			{
				var percentSize : Number = addedPartSlot.possibleParts.length / parts.length;
				addedPartSlot.sizeInMegabytes = percentSize * partSlot.sizeInMegabytes;	
			}
		}
		
		private function downloadsPartSlotViewSelectionStateChangedHandler(event : Event) : void
		{
			var partSlotView : DownloadsPartSlotView = DownloadsPartSlotView(event.target);
			var partSlot : PartSlot = partSlotView.partSlot;
							
			if(partSlotView.isSelectedForDownload)
			{
				selectedPartSlots.push(partSlot);
			}
			else
			{
				selectedPartSlots.splice(selectedPartSlots.indexOf(partSlot), 1);
			}
		
			updateButtons();
		}
		
		private function downloadsDefCarSlotViewSelectionStateChangedHandler(event : Event) : void
		{
			
			var DefCarSlotView : DownloadsDefCarSlotView = DownloadsDefCarSlotView(event.target);
			/*
			var partSlot : PartSlot = partSlotView.partSlot;
			
			if(partSlotView.isSelectedForDownload)
			{
			selectedPartSlots.push(partSlot);
			}
			else
			{
			selectedPartSlots.splice(selectedPartSlots.indexOf(partSlot), 1);
			}
			*/
			SaveDefCar = DefCarSlotView.isSelectedForDownload;
			
			var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
			var car: Car = carsProxy.loadedCar;
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			
			usersProxy.setSaveDefCar(SaveDefCar,car.id);
			
			updateButtons();
			
		}
		
		private function updateButtons() : void
		{
			var atLeastOneCachedPart : Boolean = false;
			var sizesSum : Number = 0;
			
			for each(var partSlot : PartSlot in selectedPartSlots)
			{				
				if(partSlot.percentLoaded > 0)
				{
					atLeastOneCachedPart = true;
				}
				var mb:Number= partSlot.sizeInMegabytes;
				if( partSlot.sizeInMegabytes>1024)
					mb=mb/1024/1024;
				
				
				var leftMegabytesForDownload : Number = mb * (1 - partSlot.percentLoaded);
				sizesSum += leftMegabytesForDownload;
			}
			
			downloadsPopUp.setDeleteButtonEnabled(atLeastOneCachedPart);
			//downloadsPopUp.setDownloadButtonEnabled(sizesSum > 0);
			downloadsPopUp.setDownloadButtonEnabled(selectedPartSlots.length>0);
			downloadsPopUp.totalValueLabel.text = Math.abs(sizesSum).toFixed(2) + " MB";
		}
		
		private function progressHandler(value : Number) : void
		{
			downloadsProgressView.progress = value;
		}
		
		//
		private function downloadsPopUpDownloadCleanCacheHandler(event : Event) : void
		{
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.deleteAllCache();
		
		}
		
		private function downloadsPopUpDownloadSelectedHandler(event : Event) : void
		{
			BitmapDataUtil.clearWritingError();
			NativeApplication.nativeApplication.systemIdleMode = SystemIdleMode.KEEP_AWAKE;
			downloadsProgressView = new DownloadsProgressView();
			downloadsProgressView.language = currentLanguage;
			downloadsProgressView.addEventListener(DownloadsProgressView.CANCEL_SELECTED, downloadsProgressViewCancelSelectedHandler);
			downloadsPopUp.enterPreparingState();
			Starling.juggler.delayCall(function() : void
			{
				try
				{
			
					downloadsPopUp.enterDownloadingState(downloadsProgressView);				
					partSlotsProxy.prepareForDownload(downloadsPopUp.car);
					partSlotsProxy.progressHandler = progressHandler;
					
					for each(var partSlot : PartSlot in selectedPartSlots)
					{
						partSlotsProxy.addPartSlotForDownload(partSlot);
					}
					
					var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
					var car: Car = carsProxy.loadedCar;
					
					partSlotsProxy.startDownload(car);
				
				}catch(e:Error){
					
					Logger.e("downloadsPopUpDownloadSelectedHandler:"+e.message, this);
				}
				
			}, 0.5);
		}
		
		private function downloadsProgressViewCancelSelectedHandler(event : Event) : void
		{	
			NativeApplication.nativeApplication.systemIdleMode = SystemIdleMode.NORMAL;
			downloadsPopUp.enterIntializationState();
			
			Starling.juggler.delayCall(function() : void
			{
				partSlotsProxy.cancelDownload();
				
				carsProxy.updatePartSlotsSizes(downloadsPopUp.car, expandedPartSlots);
				updateButtons();
			
				downloadsPopUp.enterSelectionState();
			
				downloadsProgressView = null;
			}, 0.5);
			
			//AlertView.show("Stopped", 2);
		}
		
		private function handlePartsDownloaded() : void
		{
			NativeApplication.nativeApplication.systemIdleMode = SystemIdleMode.NORMAL;
		
			Starling.juggler.delayCall(function() : void
			{
				downloadsPopUp.enterIntializationState();
				
				Starling.juggler.delayCall(function() : void
				{
					carsProxy.updatePartSlotsSizes(downloadsPopUp.car, expandedPartSlots);
					updateButtons();
			
					downloadsPopUp.enterSelectionState();
			
					downloadsProgressView = null;
				}, 0.5);
			
				//AlertView.show("Done", 1);
			}, 0.3); // let some time for the user to see the '100%' text
		}
		
		private function handlePartsFailedDownload() : void
		{
			Logger.e("handlePartsFailedDownload", this);
			
			
			NativeApplication.nativeApplication.systemIdleMode = SystemIdleMode.NORMAL;
			downloadsPopUp.enterIntializationState();
			
			Starling.juggler.delayCall(function() : void
			{
				carsProxy.updatePartSlotsSizes(downloadsPopUp.car, expandedPartSlots);
				updateButtons();
				
				downloadsPopUp.enterSelectionState();
			
				downloadsProgressView = null;
			
				AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "noInternetConnection"), -1, false);
			}, 0.5);
		}
		
		private function handlePartsDownloadCachingFailed() : void
		{
			//Logger.e("handlePartsDownloadCachingFailed", this);
			
			NativeApplication.nativeApplication.systemIdleMode = SystemIdleMode.NORMAL;
			downloadsPopUp.enterIntializationState();
			
			Starling.juggler.delayCall(function() : void
			{
				carsProxy.updatePartSlotsSizes(downloadsPopUp.car, expandedPartSlots);
				updateButtons();
			
				downloadsPopUp.enterSelectionState();
			
				downloadsProgressView = null;
			
				AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "noEnoughDiskSpace"), -1, false);
			}, 0.5);
		}
		
		private function downloadsPopUpDeleteSelectedHandler(event : Event) : void
		{
			
			//Logger.d("downloadsPopUpDeleteSelectedHandler", this);
			
			downloadsPopUp.enterDeletionState();
			
			Starling.juggler.delayCall(function() : void 
			{
				partSlotsProxy.deleteDownloadedPartSlots(downloadsPopUp.car, selectedPartSlots);
				updateButtons();
			
				downloadsPopUp.enterSelectionState();
			
				//AlertView.show("Deleted", 1);
			}, 0.5);
		}
	}
}