package com.despark.tuning3d.view {
    //
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.parts.Part;
    import com.despark.tuning3d.model.parts.PartMaterial;
    import com.despark.tuning3d.model.parts.PartMaterialImageTest;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.ChallengeUser;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.challenges.modals.EnterChallengePopup;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.core.PopUpMediator;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;
    import com.utils.RandomSplashManager;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;
    import starling.utils.ScaleMode;

    public class EnterChallengePopupMediator extends PopUpMediator{
        public function EnterChallengePopupMediator(popUp:PopUp) {
            super(popUp);
        }

        override public function onRegister():void {
            super.onRegister();

            enterChallengePopup.addEventListener(EnterChallengePopup.ENTER_CHALLENGE, enterChallengeHandler);
            enterChallengePopup.addEventListener(EnterChallengePopup.ENTER_CHALLENGE_GARAGE, enterChallengeGarageHandler);
            enterChallengePopup.addEventListener(EnterChallengePopup.ENTER_CHALLENGE_VOTING, enterChallengeVotingHandler);
            enterChallengePopup.addEventListener(EnterChallengePopup.ENTER_CHALLENGE_GALLERY, enterChallengeGalleryHandler);
        }

        override public function onRemove():void {
            super.onRemove();

            enterChallengePopup.removeEventListener(EnterChallengePopup.ENTER_CHALLENGE, enterChallengeHandler);
            enterChallengePopup.removeEventListener(EnterChallengePopup.ENTER_CHALLENGE_GARAGE, enterChallengeGarageHandler);
            enterChallengePopup.removeEventListener(EnterChallengePopup.ENTER_CHALLENGE_VOTING, enterChallengeVotingHandler);
            enterChallengePopup.removeEventListener(EnterChallengePopup.ENTER_CHALLENGE_GALLERY, enterChallengeGalleryHandler);
        }

        protected function get enterChallengePopup():EnterChallengePopup{
            return EnterChallengePopup(popUp);
        }

        override public function listNotificationInterests() : Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [

            ]);
        }

        override public function handleNotification(note:INotification):void {
            super.handleNotification(note);
        }

        private function enterChallengeHandler(event: Event): void {
            var data:Object = event.data;
            var isGarage:Boolean = false;
            if(data["isGarage"]){
                isGarage = true;
                data = data.challengeData;
            }
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser){
                ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
                popUpCloseButtonSelectedHandler(null);
                sendNotification(Notifications.AUTHENTICATE_USER);
            }else if(!usersProxy.isUserDataValid()){
                usersProxy.extendSession();
            }else if(enterChallengePopup.user.submittedChallenges.indexOf(enterChallengePopup.challengeData.id) != -1){
                AlertView.show("Start challenge denied", -1, false);
            }else if(enterChallengePopup.user.keys >= parseInt(enterChallengePopup.challengeData.cost)){
                if(isGarage){
                    enterChallengeGarageHandler(data);
                }else if(AdsManager.ENABLED){
                    sendNotification(Notifications.DISPLAY_CHALLENGE_LOADER);
                    AdsManager.$.requestInterstitial(function(success:Boolean):void{
                        sendNotification(Notifications.HIDE_CHALLENGE_LOADER);
                        showCar({car: data.car, backLevel: 0, carView: null, tuning: null, challengeData: data});
                    });
                }else{
                    showCar({car: data.car, backLevel: 0, carView: null, tuning: null, challengeData: data});
                }
                popUpCloseButtonSelectedHandler(null);
            }else{
                var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.KEYS_NOT_ENOUGH, null, enterChallengePopup);
                statsDialog.addEventListener(ChallengeAlertEvent.VOTING_TRIGGERED, enterChallengeVotingHandler);
                statsDialog.open();
            }
        }

        private function enterChallengeGarageHandler(data:Object): void {
            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, data);
            popUpCloseButtonSelectedHandler(null);
        }

        private function enterChallengeVotingHandler(event: Event): void {
            sendNotification(Notifications.DISPLAY_CHALLENGES_VOTING_SCREEN);
            popUpCloseButtonSelectedHandler(null);
        }

        private function enterChallengeGalleryHandler(event: Event): void {
            var data:Object = event.data;
            sendNotification(Notifications.DISPLAY_LEADERBOARD_CHALLENGE_TUNINGS, {challengeId:data._id});
            popUpCloseButtonSelectedHandler(null);
        }

        private function showCar(data:Object):void{
            if(data && data.car){
                var tmpCar:Car = data.car as Car;
                if(tmpCar && tmpCar.updated > 0){
                    Tuning3D.CAR_UPDATE_DATE = tmpCar.updated;
                }
            }

            var partSlot:PartSlot = new PartSlot(data.car as Car);
            var partCarSplash:Part = new Part(partSlot);
            var partMaterial:PartMaterial = new PartMaterial(partCarSplash);

            var randomSplashUrl:String = RandomSplashManager.getRandomUrl();
            var carSplashImageProvider:PartMaterialImageTest = new PartMaterialImageTest(randomSplashUrl,"0",null,partMaterial);
            carSplashImageProvider.frameIndex = 0;
            carSplashImageProvider.angleIndex = 0;
            carSplashImageProvider.isEncrypted = false;


            var carSplashImage:ImageLoaderWithStates = new ImageLoaderWithStates();
            carSplashImage.overrideDispose = false;
            carSplashImage.snapToPixels = false;
            if(Constants.TOTAL_MEMORY<1024 && Constants.SCALE_FACTOR<2){
                carSplashImage.bitmapScale = 0.5;
            }

            carSplashImage.visible = false;
            carSplashImage.completeHandler = function(image:ImageLoaderWithStates):void{
                data.carSplashImage = carSplashImage;
                sendNotification(Notifications.DISPLAY_TUNING_SCREEN, data);
            };

            carSplashImage.scaleMode =	ScaleMode.NONE;
            carSplashImage.persistentImageProvider = carSplashImageProvider;
            Starling.current.stage.addChild(carSplashImage);
        }

    }
}
