package com.despark.tuning3d.view
{
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.friend.FriendProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
    import com.despark.tuning3d.view.core.PopUpMediator;
    import com.utils.ArrayUtil;
    import com.utils.Tracker;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;

    public class FriendsListPopUpMediator extends PopUpMediator
{
    private var _proxy : FriendProxy;
    private var _isLoading : Boolean = false;

    public function FriendsListPopUpMediator(stackedScreen : PopUp)
    {
        super(stackedScreen);
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        Friend.CURRENT_USER = usersProxy.currentUser;
    }

    override public function onRegister() : void
    {
        super.onRegister();

        _proxy = (facade.retrieveProxy(FriendProxy.NAME) as FriendProxy);
        _proxy.loadFriends(Friend.CURRENT_USER, 0);
        _proxy.loadPendings();
        _isLoading = true;

        friendsListScreen.addEventListener(FriendsListPopUp.SEARCH_FRIENDS, friendsSearchHandler);
        friendsListScreen.addEventListener(FriendsListPopUp.LOAD_MORE_FRIENDS, friendsMoreHandler);
        friendsListScreen.addEventListener(FriendsListPopUp.PENDING_FRIENDS_READ, handlePendingFriendsReadHandler);

        friendsListScreen.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);
        friendsListScreen.addEventListener(StackedScreen.GARAGE_SELECTED, friendsSelectedHandler);
        friendsListScreen.addEventListener(StackedScreen.TIMELINE_SELECTED, timelineSelectedHandler);
        friendsListScreen.addEventListener(StackedScreen.MAILS_SELECTED, mailsSelectedHandler);
        Tracker.trackEvent("Friends Popup", "open");
    }




    override public function onRemove() : void {
        super.onRemove();

        friendsListScreen.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);
        friendsListScreen.removeEventListener(FriendsListPopUp.SEARCH_FRIENDS, friendsSearchHandler);
        friendsListScreen.removeEventListener(FriendsListPopUp.LOAD_MORE_FRIENDS, friendsMoreHandler);
        friendsListScreen.removeEventListener(FriendsListPopUp.PENDING_FRIENDS_READ, handlePendingFriendsReadHandler);

        friendsListScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, friendsSelectedHandler);
        friendsListScreen.removeEventListener(StackedScreen.TIMELINE_SELECTED, timelineSelectedHandler);
        friendsListScreen.removeEventListener(StackedScreen.MAILS_SELECTED, mailsSelectedHandler);
        BannerStackedScreen.hideAdBanner(true);
    }

    private function handleChallengeRequest(event:Event):void {
        BannerStackedScreen.hideAdBanner(false);
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function friendsSelectedHandler(e : Event) : void{
        BannerStackedScreen.hideAdBanner(false);
        friendsListScreen.close();
        var user:User = e.data.user;
        Starling.juggler.delayCall(function () :void {
            Constants.selectedUser = user;
            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : false});
        }, 0.2)
    }

    private function timelineSelectedHandler(e : Event) : void{
        BannerStackedScreen.hideAdBanner(false);
        friendsListScreen.close();
        var user:User = e.data.user;
        Constants.selectedUser = user;
        Starling.juggler.delayCall(function () :void {
            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, {user : Constants.selectedUser, deleteOtherScreens : false});
        }, 0.2)

    }

    private function mailsSelectedHandler(e : Event) : void{
        BannerStackedScreen.hideAdBanner(false);
        var user:User = e.data.user;
        Constants.selectedUser = user;
        Starling.juggler.delayCall(function () :void {
            sendNotification(Notifications.DISPLAY_INSTANT_MAIL_POPUP, {toUser: Constants.selectedUser});
        }, 0.2)

    }

    private function friendsSearchHandler(event:Event):void {
        if(_isLoading) return;
        _isLoading = true;
        var text : String = event.data.text;
        var skip : int = event.data.skip;
        _proxy.search(text,skip, FriendsListPopUp.sentData, FriendsListPopUp.pendingData, function (items:Array) : void{
            _isLoading = false;
            if(skip == 0){
                friendsListScreen.setSearchResult(items);
            }
            else {
                friendsListScreen.appendSearchResult(items);
            }
        })
    }

    private function friendsMoreHandler(event:Event):void {
        if(_isLoading)  return;

        _isLoading = true;
        var skip :int = friendsListScreen.friends.length;

        _proxy.loadFriends(Friend.CURRENT_USER, skip);
    }


    public function get friendsListScreen() : FriendsListPopUp {
        return viewComponent as  FriendsListPopUp;
    }

    override public function listNotificationInterests() : Array
    {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.PENDING_FRIENDS_LOADED,
            Notifications.FRIENDS_LOADED,
            Notifications.FRIENDS_LOADING_FAILED,
            Notifications.FRIENDS_REQUEST_SEND,
            Notifications.FRIENDS_REQUEST_SEND_FAILED,
            Notifications.UNDO_FRIENDS_REQUEST,
            Notifications.UNDO_FRIENDS_REQUEST_FAILED,
            Notifications.FRIENDS_REQUEST_DENIED,
            Notifications.FRIENDS_REQUEST_DENIED_FAILED,
            Notifications.CONFIRM_FRIENDS_REQUEST,
            Notifications.CONFIRM_FRIENDS_REQUEST_FAILED,
            Notifications.DELETE_FRIENDS_REQUEST_SEND,
            Notifications.DELETE_FRIENDS_REQUEST_SEND_FAILED
        ]);
    }

    override public function handleNotification(note : INotification) : void
    {
        switch(note.getName())
        {
                ////////////////////////////////////////////
            case Notifications.PENDING_FRIENDS_LOADED:
                handlePendingFriendsLoaded(note.getBody());
                break;
            case Notifications.FRIENDS_LOADED:
                handleFriendsLoaded(note.getBody());
                break;
            case Notifications.FRIENDS_LOADING_FAILED:
                handleFriendsLoadingFailed();
                break;
            case Notifications.FRIENDS_REQUEST_SEND:
                handleFriendRequestSend();
                break;
            case Notifications.FRIENDS_REQUEST_SEND_FAILED:
                handleFriendRequestSendFailure();
                break;
            case Notifications.UNDO_FRIENDS_REQUEST:
                handleFriendRequestUndo();
                break;
            case Notifications.UNDO_FRIENDS_REQUEST_FAILED:
                handleFriendRequestUndoFailure();
                break;
            case Notifications.FRIENDS_REQUEST_DENIED:
                handleFriendRequestDeniedSend(note.getBody());
                break;
            case Notifications.FRIENDS_REQUEST_DENIED_FAILED:
                handleFriendRequestDeniedSendFailure();
                break;
            case Notifications.CONFIRM_FRIENDS_REQUEST:
                handleFriendRequestConfirmSend(note.getBody());
                break;
            case Notifications.CONFIRM_FRIENDS_REQUEST_FAILED:
                handleFriendRequestConfirmSendFailure();
                break;
            case Notifications.DELETE_FRIENDS_REQUEST_SEND:
                handleFriendRequestDeleteSend(note.getBody());
                break;
            case Notifications.DELETE_FRIENDS_REQUEST_SEND_FAILED:
                handleFriendRequestDeleteSendFailure();
                break;
        }

        super.handleNotification(note);
    }

    private function handlePendingFriendsLoaded(friendsInfo:Object) : void {
        friendsListScreen.unreadRequests = friendsInfo.unread;
        if(friendsInfo.friends) {
            friendsListScreen.pendings = friendsInfo.friends;
        }
    }

    private function handlePendingFriendsReadHandler(event:Event) : void {
        _proxy.setFriendRequestRead(event.data.friendList);
    }

    private function handleFriendsLoaded(friendsInfo:Object) : void {
        _isLoading = false;
        friendsListScreen.friends = friendsInfo.friends;
        friendsListScreen.total = friendsInfo.total;
    }

    private function handleFriendsLoadingFailed() : void
    {
        _isLoading = false;
    }

    private function handleFriendRequestSend() : void{
    }
    private function handleFriendRequestSendFailure() : void {
    }
    private function handleFriendRequestUndo() : void{
    }
    private function handleFriendRequestUndoFailure() : void{
    }

    private function handleFriendRequestDeniedSend(info:Object) : void{
//        tuningVideoPopup.removePending(info.friend);
    }
    private function handleFriendRequestDeniedSendFailure() : void {
    }

    private function handleFriendRequestConfirmSend(info:Object) : void{
//        tuningVideoPopup.removePending(info.friend);
        friendsListScreen.addFriend(info.friend);

    }
    private function handleFriendRequestConfirmSendFailure() : void {
    }

    private function handleFriendRequestDeleteSend(info:Object) : void{
        friendsListScreen.removeFriend(info.friend);
    }

    private function handleFriendRequestDeleteSendFailure() : void {
    }
}
}