package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.cars.CarsProxy;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImageTest;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.sharing.SharingInstagramProxy;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.gallery.GalleryTuningPopup;
    import com.despark.tuning3d.view.components.mailbox.MailsScreen;
    import com.despark.tuning3d.view.components.tunings.TuningsListMenuView;
import com.despark.tuning3d.view.components.tunings.editing.TuningScreen;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;
import com.utils.RandomSplashManager;

import flash.display.Loader;
import flash.events.Event;

//import flash.events.Event;
import flash.net.URLRequest;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;
import starling.events.Event;
import starling.utils.ScaleMode;

public class GalleryTuningPopupMediator extends PopUpMediator {
    private var _tuningPopup:GalleryTuningPopup;

    public function GalleryTuningPopupMediator(stackedScreen:PopUp) {
        super(stackedScreen);
        _tuningPopup = popUp as GalleryTuningPopup;
    }

    private function initialize():void{
        loadSelectedUserTunings();
    }

    override public function onRegister():void {
        super.onRegister();
        tuningPopup.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        tuningPopup.addEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        tuningPopup.addEventListener(TuningScreen.SHARE_INSTAGRAM_SELECTED, shareTuningInstagramHandler);
        tuningPopup.addEventListener(MailsScreen.SEND_INSTANT_MAIL, sendInstantMailHandler);
        tuningPopup.addEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);

        tuningPopup.addEventListener(TuningsListMenuView.LIKE_SELECTED, tuningLikeSelectedHandler);
        tuningPopup.addEventListener(TuningsListMenuView.COMMENT_SELECTED, tuningCommentSelectedHandler);
        tuningPopup.addEventListener(TuningsListMenuView.CHALLENGE_SELECTED, tuningChallengeSelectedHandler);
        tuningPopup.addEventListener(TuningsListMenuView.TUNING_DELETE, tuningDeleteHandler);
        Starling.juggler.delayCall(initialize, 1);
    }

    override public function onRemove():void {
        super.onRemove();
        tuningPopup.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        tuningPopup.removeEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        tuningPopup.removeEventListener(TuningScreen.SHARE_INSTAGRAM_SELECTED, shareTuningInstagramHandler);
        tuningPopup.removeEventListener(MailsScreen.SEND_INSTANT_MAIL, sendInstantMailHandler);

        tuningPopup.removeEventListener(TuningsListMenuView.LIKE_SELECTED, tuningLikeSelectedHandler);
        tuningPopup.removeEventListener(TuningsListMenuView.COMMENT_SELECTED, tuningCommentSelectedHandler);
        tuningPopup.removeEventListener(TuningsListMenuView.CHALLENGE_SELECTED, tuningChallengeSelectedHandler);
        tuningPopup.removeEventListener(TuningsListMenuView.TUNING_DELETE, tuningDeleteHandler);
    }

    protected function tuningDeleteHandler(event:starling.events.Event):void{
        if(alertView){
            alertView.hide();
        }
        alertView = AlertView.show(AlertView.language.translate("mob_downloads", "deleting"), -1);
        var tuningsProxy : TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
        if(event.data && event.data.deleteFrame) {
            var index:int = tuningPopup.selectedTuningFrameIndex;
            tuningsProxy.deleteTuningFrame2(tuningPopup.selectedTuning, index);
        }else{
            tuningsProxy.deleteTuning(tuningPopup.selectedTuning);
            tuningPopup.close();
        }
    }

    private function tuningCommentSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function tuningLikeSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function tuningChallengeSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function challengeRequestHandler(event:starling.events.Event):void{
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function sendInstantMailHandler(event:starling.events.Event):void {
        sendNotification(Notifications.DISPLAY_INSTANT_MAIL_POPUP, event.data);
    }

    private function userDetailsSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    public function loadSelectedUserTunings():void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.currentUser) {
            loadUserTunings(usersProxy.currentUser.id);
        }
        //
        /*if(tuningPopup.selectedTuning && tuningPopup.selectedTuning.car && tuningPopup.selectedTuning.car.isEditable){
            var tuningsProxy : TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            tuningsProxy.getTuning(tuningPopup.selectedTuning.shortId);
        }*/
    }

    private function loadUserTunings(userId:String):void{
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        if(userId) {
            challengesProxy.loadUserChallenges(userId, false);
        }
    }

    private function tuningSelectedHandler(event : starling.events.Event) : void {

        var selectedTuning : Tuning = tuningPopup.selectedTuning;
        var info : Object = {
            car : selectedTuning.car
        };
        var userProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(event.data && event.data.isEditing && userProxy.currentUser) {
            if(selectedTuning.userObj && selectedTuning.userObj is User){
                selectedTuning.tuningUserID = User(selectedTuning.userObj).id;
            }
            if (selectedTuning.tuningUserID == userProxy.currentUser.id || (selectedTuning.userObj && selectedTuning.userObj.hasOwnProperty("_id") && selectedTuning.userObj._id == userProxy.currentUser.id) || selectedTuning.userObj == userProxy.currentUser.id) {
                info["tuning"] = selectedTuning;
            }
        }

        var partSlot:PartSlot = new PartSlot(selectedTuning.car);
        var partCarSplash:Part = new Part(partSlot);
        var partMaterial:PartMaterial = new PartMaterial(partCarSplash);
        var randomSplashUrl:String = RandomSplashManager.getRandomUrl();

        trace("randomSplashUrl:", randomSplashUrl);

        var carSplashImageProvider:PartMaterialImageTest = new PartMaterialImageTest(randomSplashUrl,"0",null,partMaterial);
        carSplashImageProvider.frameIndex = 0;
        carSplashImageProvider.angleIndex = 0;
        carSplashImageProvider.isEncrypted = false;


        var carSplashImage:ImageLoaderWithStates = new ImageLoaderWithStates();
        carSplashImage.overrideDispose = true;
        carSplashImage.snapToPixels = false;
        if(Constants.TOTAL_MEMORY<1024 && Constants.SCALE_FACTOR<2){
            carSplashImage.bitmapScale = 0.5;
        }
        carSplashImage.visible = false;
        tuningPopup.stage.addChild(carSplashImage);
        carSplashImage.completeHandler = function(image:ImageLoaderWithStates):void{
            info.carSplashImage=carSplashImage;
            sendNotification(Notifications.DISPLAY_TUNING_SCREEN, info);
        };
        carSplashImage.scaleMode =	ScaleMode.NONE;
        carSplashImage.persistentImageProvider = carSplashImageProvider;
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.TUNING_COMMENTS_LOADED,
            Notifications.SHARING_READY,
            Notifications.TUNING_SHARING_MESSAGE_ENTERED,
            Notifications.TUNING_FRAME_DELETED,
            Notifications.TUNING_DELETION_FAILED,
            Notifications.TUNING_LOADED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.TUNING_COMMENTS_LOADED:
                handleCommentDataLoaded(note.getBody());
                break;
            case Notifications.SHARING_READY:
                handleSharingReady(note.getBody() as String);
                break;
            case Notifications.TUNING_SHARING_MESSAGE_ENTERED:
                handleSharingTextEnter();
                break;
            case Notifications.TUNING_FRAME_DELETED:
                handleTuningFrameDeleted();
                break;
            case Notifications.TUNING_DELETION_FAILED:
                handleTuningFrameDeleteFailed();
                break;
            case Notifications.TUNING_LOADED:
                tuningLoadedHandler(note.getBody() as Tuning);
                break;

        }
        super.handleNotification(note);
    }

    private function handleSharingTextEnter(): void {
        var sharingObject: Object = {};
        if (tuningPopup.selectedTuning) {
            sharingObject.imageUrl = 'http:' + tuningPopup.selectedTuning.snapshots[0].replace(/https?:/i, '');

            var url:URLRequest = new URLRequest(sharingObject.imageUrl);
            var img:Loader = new Loader();


            img.load(url);
            img.contentLoaderInfo.addEventListener(flash.events.Event.COMPLETE, function (e: flash.events.Event): void {
                sharingObject.bitmapData = e.target.content.bitmapData;
                tuningPopup.dispatchEvent(new starling.events.Event(TuningScreen.SHARE_INSTAGRAM_SELECTED, false, sharingObject));
            });

        }
    }

    private function shareTuningInstagramHandler(event:starling.events.Event):void {
        var sharingInstagramProxy:SharingInstagramProxy = facade.retrieveProxy(SharingInstagramProxy.NAME) as SharingInstagramProxy;
        sharingInstagramProxy.sharePhoto('', event.data.bitmapData, function ():void {
            event.data.bitmapData.dispose();
        }, event.data.url);

    }

    private function handleCommentDataLoaded(info:Object):void {
        tuningPopup.comments = info.comments;
        if(alertView) {
            alertView.hide();
        }
    }

    private function tuningLoadedHandler(tuning:Tuning):void {
        if(tuningPopup.selectedTuning){
            tuningPopup.selectedTuning.car = tuning.car;
        }
    }

    private function handleTuningFrameDeleted() : void {
        if(alertView) {
            alertView.hide();
        }
        tuningPopup.updateTuning();
        Starling.juggler.delayCall(tuningPopup.updatePageIndicatorPosition, .01);
    }

    private function handleTuningFrameDeleteFailed() : void {
        if(alertView) {
            alertView.hide();
        }
    }

    private function tuningScreenSocialSelectedHandler(event:starling.events.Event):void {
        if(tuningPopup.selectedTuning) {
            (facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy).loadComments(tuningPopup.selectedTuning.id);
        }
    }

    private function handleSharingReady(_title : String) : void {
        var tuning:Tuning = tuningPopup.selectedTuning;
        var usernameStr:String = tuning.userObj.username;
        var shortId:String = tuning.shortId;
        var link:String="";
        try{
            link = tuning.car.getSiteLinkGrage(usernameStr, shortId);
        }catch(e:Error){
            trace("error:", e.getStackTrace());
        }
        sendNotification(Notifications.DISPLAY_TUNING_SHARING_POP_UP,{title : _title, siteLink:link});
    }

    public function get tuningPopup():GalleryTuningPopup {
        return _tuningPopup;
    }
}
}
