package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.modals.*;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.PopUpMediator;
    import com.utils.ArrayUtil;
    import com.utils.Tracker;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;

    public class InAppNotificationsPopupMediator extends PopUpMediator {
    private var _proxy:ChallengesProxy;
    private var _notificationPopUp:InAppNotificationsPopup;
    public function InAppNotificationsPopupMediator(popUp:PopUp) {
        super(popUp);
        _notificationPopUp = popUp as InAppNotificationsPopup;
    }

    override public function onRegister():void {
        super.onRegister();
        _proxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;

        _notificationPopUp.addEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);


        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_REQUEST_TUNING, tuningSelectedHandler);

        _notificationPopUp.addEventListener(StackedScreen.TUNING_SELECTED, userTuningSelectedHandler);

        _notificationPopUp.addEventListener(ChallengeEvent.TUNING_SELECTED, challengeTuningSelectedHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.CHALLENGE_CLAIM_REWARD, claimRewardHandler);


        _proxy.markAllUnreadNotifications(function ():void {
            sendNotification(Notifications.CHALLENGE_NOTIFICATIONS_VIEWED);
        });

        var timelineProxy:TimelineProxy = facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
        timelineProxy.makeFullNotificationsRead();
        //
        Starling.juggler.delayCall(function():void{
            BannerStackedScreen.hideAdBanner(false);
        }, 0.2);
        Tracker.trackEvent("VS Notifications Popup", "open");
    }

    override public function onRemove():void {
        super.onRemove();
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_REQUEST_TUNING, tuningSelectedHandler);
        _notificationPopUp.removeEventListener(StackedScreen.TUNING_SELECTED, userTuningSelectedHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.TUNING_SELECTED, challengeTuningSelectedHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.CHALLENGE_CLAIM_REWARD, claimRewardHandler);
    }

    private function tuningSelectedHandler(event:Event):void {
        _notificationPopUp.close();
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.closeHandler =  closeHandler;
        sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_TUNING_POPUP, data);
    }

    private function viewUserTuningsHandler(event:Event):void {
        Constants.selectedUser = event.data.user;
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.CHALLENGE_CREATE_ERROR
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.CHALLENGE_CREATE_ERROR:

                break;
        }
        super.handleNotification(note);
    }

    private function getRequestsHandler(event:Event):void {
        _proxy.getRequests(event.data.skip, event.data.limit, _notificationPopUp.loadInvitations);
    }

    private function getNotificationsHandler(event:Event):void {
        _proxy.getNotifications(event.data.skip, event.data.limit, _notificationPopUp.loadNotifications);
    }

    private function handleChallengeRequest(event:Event):void {
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function claimRewardHandler(event:Event):void {
        var data:Object = event.data;
        var challengeProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        challengeProxy.claimRewards(data._id, data.diamond);
    }

    private function challengeTuningSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, event.data);
    }

    private function userTuningSelectedHandler(event:Event):void {
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.eventDispatcher = _notificationPopUp;

        var selectedUser:User = data.user as User;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var currentUser:User = usersProxy.currentUser;
        if((currentUser && selectedUser && selectedUser.id != currentUser.id) || !currentUser){
            Constants.selectedUser = selectedUser;
        }else{
            Constants.selectedUser = null;
        }
        sendNotification(Notifications.DISPLAY_GALLERY_TUNING_POPUP, data);
    }
}
}
