package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.view.components.auth.UserDetailsPopup;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.mailbox.MailBoxPopup;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.core.PopUpMediator;
    import com.utils.ArrayUtil;

    import flash.events.StageOrientationEvent;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;

    public class MailBoxPopupMediator extends PopUpMediator {
    private var _mailBoxPopup:MailBoxPopup;

    public function MailBoxPopupMediator(stackedScreen:PopUp) {
        super(stackedScreen);
        _mailBoxPopup = popUp as MailBoxPopup;
    }

    private function initialize():void{

    }

    override public function onRegister():void {
        super.onRegister();
        _mailBoxPopup.addEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
        _mailBoxPopup.addEventListener(PopUp.CLOSE_BUTTON_SELECTED, popupCloseHandler);
        Starling.juggler.delayCall(initialize, .1);
    }

    override public function onRemove():void {
        super.onRemove();
        _mailBoxPopup.removeEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
        _mailBoxPopup.removeEventListener(PopUp.CLOSE_BUTTON_SELECTED, popupCloseHandler);
    }

    private function garageSelectedHandler(event : Event): void {
        if((Constants.selectedUser && FeedData.CURRENT_USER && Constants.selectedUser.id != FeedData.CURRENT_USER.id) || (event.data && event.data.enableClosing)){
            Starling.juggler.delayCall(function () :void {
                sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, {user:Constants.selectedUser, deleteOtherScreens : false, options: [UserDetailsPopup.TIMELINE, UserDetailsPopup.GARAGE, UserDetailsPopup.CHALLENGE]});
                //sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : true});
            }, 0.1);
        }else {
            /*Starling.juggler.delayCall(function () : void{
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, null);
            }, 0.1);*/
        }
    }

    private function popupCloseHandler(event : Event): void {
        sendNotification(Notifications.MAIL_BOX_POPUP_CLOSES);
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.USER_MAILS_LOADED,
            Notifications.MAIL_SEND_SUCCESS
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.USER_MAILS_LOADED:
                handleUserMailsLoaded(note.getBody().messages, note.getBody().isPushing);
                break;
            case Notifications.USER_MAILS_LOAD_FAILED:
                handleUserMailsLoadFailed();
                break;
            case Notifications.MAIL_SEND_SUCCESS:
                handleMailSendSuccessful(note.getBody());
                break;
            case Notifications.MAIL_SEND_FAILED:
                handleMailSendSuccessful(note.getBody());
                break;

        }
        super.handleNotification(note);
    }

        private function handleMailSendSuccessful(message:Object):void {
            mailBoxPopup.addNewMessage(message);
            mailBoxPopup.cleanInputs();
        }

    private function handleUserMailsLoaded(mails:Array, isPushing:Boolean = true):void {
        /*if(!mailBoxPopup || !mailBoxPopup.stage){
            facade.removeMediator(mediatorName);
            return;
        }*/
        mailBoxPopup.addMessages(mails, isPushing);
    }

    private function handleUserMailsLoadFailed() : void {

    }

    public function get mailBoxPopup():MailBoxPopup {
        return _mailBoxPopup;
    }
}
}
