package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.users.UserProfileProxy;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.auth.ProfilePopup;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.core.*;
import com.utils.ArrayUtil;
    import com.utils.Console;
    import com.utils.Tracker;

    import org.puremvc.as3.interfaces.INotification;

import starling.events.Event;

public class ProfilePopupMediator extends PopUpMediator {
    public function ProfilePopupMediator(popUp:PopUp) {
        super(popUp);
    }

    public function get profilePopup() : ProfilePopup {
        return ProfilePopup(popUp);
    }

    override public function listNotificationInterests():Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.PROFILE_LOADED,
            Notifications.PROFILE_CHANGES_SAVED,
            Notifications.VERIFICATION_EMAIL_RESENT,
            Notifications.DELETE_ACCOUNT,
            Notifications.PASSWORD_CHANGED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.PROFILE_LOADED:
                profilePopup.setProfileData(note.getBody().result);
                break;
            case Notifications.PROFILE_CHANGES_SAVED:
                profilePopup.setChangeResult(note.getBody());
                if(note.getBody().success){
                    var profileProxy:UserProfileProxy = facade.retrieveProxy(UserProfileProxy.NAME) as UserProfileProxy;
                    profileProxy.loadUserProfile();
                }
                break;
            case Notifications.PASSWORD_CHANGED:
                profilePopup.setChangeResult(note.getBody());
                break;
            case Notifications.VERIFICATION_EMAIL_RESENT:
                break;
            case Notifications.DELETE_ACCOUNT:
                    if(note.getBody().success){
                        var userProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
                        userProxy.logoutCurrentUser();
                        profilePopup.dispatchEventWith(PopUp.CLOSE_BUTTON_SELECTED);
                    }else{
                        AlertView.show("Failed to delete your account", 0, false, null, true);
                    }
                break;
        }
        super.handleNotification(note);
    }

    override public function onRegister():void {
        super.onRegister();

        var userProxy:UsersProxy = new UsersProxy();
        if(userProxy.currentUser){
            var profileProxy:UserProfileProxy = facade.retrieveProxy(UserProfileProxy.NAME) as UserProfileProxy;
            profileProxy.loadUserProfile();
        }

        profilePopup.addEventListener(ProfilePopup.SAVE_CHANGES, saveChangesHandler);
        profilePopup.addEventListener(ProfilePopup.RESEND_EMAIL, resendEmailHandler);
        profilePopup.addEventListener(ProfilePopup.DELETE_ACCOUNT, deleteAccountHandler);

        Tracker.trackEvent("User Profile Popup", "open");
    }

    private function saveChangesHandler(event:Event):void {
        var profileProxy:UserProfileProxy = facade.retrieveProxy(UserProfileProxy.NAME) as UserProfileProxy;
        profileProxy.saveProfileChanges(event.data.name, event.data.email, event.data.password);
    }

    private function resendEmailHandler(event:Event):void {
        var profileProxy:UserProfileProxy = facade.retrieveProxy(UserProfileProxy.NAME) as UserProfileProxy;
        profileProxy.sendVerificationEmail();
    }

    private function deleteAccountHandler(event:Event):void {
        var userProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(userProxy.currentUser){
            var profileProxy:UserProfileProxy = facade.retrieveProxy(UserProfileProxy.NAME) as UserProfileProxy;
            profileProxy.deleteAccount(userProxy.currentUser.id);
        }
    }

    override public function onRemove():void {
        super.onRemove();
    }

}
}
