package com.despark.tuning3d.view
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.auth.RegisterScreen;
	import com.despark.tuning3d.view.components.auth.UserAgreementView;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.despark.tuning3d.view.components.core.StackedScreenNavigator;
	import com.despark.tuning3d.view.components.core.Window;
	import com.despark.tuning3d.view.core.StackedScreenMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;
	
	import starling.events.Event;

	public class RegisterScreenMediator extends LoginScreenMediator
	{
		public function RegisterScreenMediator(screen : StackedScreen)
		{
			super(screen);
			//var t:StackedScreenNavigator
			StackedScreenNavigator.POP_TO_ROOT_FROM_REGISTER = true;
		}
		
		public function get registerScreen() : RegisterScreen
		{
			return stackedScreen as RegisterScreen;
		}
		
		override public function onRegister() : void
		{
			super.onRegister();
			
			registerScreen.addEventListener(Event.ADDED_TO_STAGE, registerScreenAddedToStageHandler);
			registerScreen.addEventListener(RegisterScreen.AGREE_CHECK_CHANGED, registerScreenAgreeCheckChangedHandler);
			registerScreen.addEventListener(RegisterScreen.EMAIL_CHANGED, registerScreenEmailChangedHandler);
			registerScreen.addEventListener(RegisterScreen.USERNAME_CHANGED, registerScreenUsernameChangedHandler);
			registerScreen.addEventListener(RegisterScreen.PASSWORD_CHANGED, registerScreenPasswordChangedHandler);
			registerScreen.addEventListener(RegisterScreen.AGREEMENT_SELECTED, registerScreenAgreementSelectedHandler);
			registerScreen.addEventListener(RegisterScreen.REGISTER_SELECTED, registerScreenRegisterSelectedHandler);
		}
		
		override public function onRemove() : void
		{
			super.onRemove();
			
			registerScreen.removeEventListener(Event.ADDED_TO_STAGE, registerScreenAddedToStageHandler);
			registerScreen.removeEventListener(RegisterScreen.AGREE_CHECK_CHANGED, registerScreenAgreeCheckChangedHandler);
			registerScreen.removeEventListener(RegisterScreen.EMAIL_CHANGED, registerScreenEmailChangedHandler);
			registerScreen.removeEventListener(RegisterScreen.USERNAME_CHANGED, registerScreenUsernameChangedHandler);
			registerScreen.removeEventListener(RegisterScreen.PASSWORD_CHANGED, registerScreenPasswordChangedHandler);
			registerScreen.removeEventListener(RegisterScreen.AGREEMENT_SELECTED, registerScreenAgreementSelectedHandler);
			registerScreen.removeEventListener(RegisterScreen.REGISTER_SELECTED, registerScreenRegisterSelectedHandler);
		}
		
		public override function listNotificationInterests() : Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.USER_REGISTERED,
				Notifications.USER_REGISTRATION_FAILED,
				Notifications.APPLICATION_BECAME_ACTIVE
			]);
		}
		
		private function handleApplicationBecameActive() : void {
			if(Device.isIos()){
                return;
            }
			if(this.registerScreen){
				registerScreen.invalidate();
			}
		}   
		
		public override function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case Notifications.USER_REGISTERED:
					handleUserRegistered();
					break;
				
				case Notifications.USER_REGISTRATION_FAILED:
					handleUserRegistrationFailed(note.getBody());
					break;
				
				case Notifications.APPLICATION_BECAME_ACTIVE:{
					
					handleApplicationBecameActive();
				}
					break;
			}
			
			super.handleNotification(note);
		}
		
		private function registerScreenAgreementSelectedHandler(event : Event) : void
		{
			UserAgreementView.show(AlertView.language.translate("userAgreement", "main"));
		}
		
		private function registerScreenAddedToStageHandler(event : Event) : void
		{
			registerScreen.setRegisterButtonEnabled(false);
		}
		
		private function registerScreenAgreeCheckChangedHandler(event : Event) : void
		{
			updateRegisterButton();
		}
		
		private function registerScreenEmailChangedHandler(event : Event) : void
		{
			updateRegisterButton();
		}
		
		private function registerScreenUsernameChangedHandler(event : Event) : void
		{
			updateRegisterButton();
		}
		
		private function registerScreenPasswordChangedHandler(event : Event) : void
		{
			updateRegisterButton();
		}
		
		private function updateRegisterButton() : void
		{
			var enabled : Boolean =  registerScreen.username.length > 0 && registerScreen.password.length > 0 && registerScreen.agree;
			registerScreen.setRegisterButtonEnabled(enabled);
		}
		
		private function handleUserRegistered() : void
		{ 
			if(alertView)
			{
				alertView.hide();
			}
			
			trace("reg success 2")
			
			
			
			var windowMediator : WindowMediator = facade.retrieveMediator(WindowMediator.NAME) as WindowMediator;
			windowMediator.popToRootScreen(windowMediator.stackSize - 3);
			
			StackedScreenNavigator.POP_TO_ROOT_FROM_REGISTER = false;
			
		}
		
		private function handleUserRegistrationFailed(errorInfo : Object) : void {
			if(alertView){
				alertView.hide();
			}
			registerScreen.setRegisterButtonEnabled(true);
			if(!(errorInfo is String)) {
				registerScreen.setErrorTexts(errorInfo);
			}
		}
		
		private function registerScreenRegisterSelectedHandler(event : Event) : void
		{
			alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1);
			
			registerScreen.setRegisterButtonEnabled(false);
			registerScreen.clearErrorLabels();
			
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			
			var email:String=registerScreen.email;
			
			if(email.length<=0){
				var time:int=(new Date().valueOf());  
				email=time+""+(Math.random()*10000000000).toFixed(0)+"@3dtuning.com";
			}
			
			usersProxy.registerUser(email, registerScreen.username, registerScreen.password, registerScreen.shouldSubscribe);
		}
	}
}