package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesLeaderboardScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesListScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesVotingScreen;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.despark.tuning3d.view.core.StackedScreenMediator;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;
    import com.utils.Console;
    import com.utils.ObjectUtils;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class StackedBannerScreenMediator  extends StackedScreenExtendedMediator {
        protected var _garage_delay:Number = 0.5;
        public function StackedBannerScreenMediator(stackedScreen : StackedScreen) {
            super(stackedScreen);
        }
        override public function onRegister() : void {
            super.onRegister();
            // *********** Challenge screen tab change handlers ***********
            stackedBannerScreen.addEventListener(StackedBannerScreen.CHALLENGES_LEADERBOARD_SELECTED, challengesLeaderBoardSelectHandler);
            stackedBannerScreen.addEventListener(StackedBannerScreen.CHALLENGES_RATE_SELECTED, challengesRateSelectHandler);
            stackedBannerScreen.addEventListener(StackedBannerScreen.CHALLENGES_GARAGE_SELECTED, challengesGarageSelectHandler);
            stackedBannerScreen.addEventListener(StackedBannerScreen.CHALLENGES_NOTIFICATIONS_SELECTED, challengesNotesSelectHandler);
            stackedBannerScreen.addEventListener(StackedBannerScreen.CHALLENGES_HELP_SELECTED, challengesHelpSelectHandler);
            //
            stackedBannerScreen.addEventListener(StackedScreen.DISPLAY_REWARD_INFO_POPUP, displayRewardInfoPopUp);
            stackedBannerScreen.addEventListener(StackedBannerScreen.UPDATE_USER_DATA, userDataUpdatedHandler);
            //
            updateUserData();
        }

        override public function onRemove() : void {
            super.onRemove();
            // *********** Challenge screen tab change handlers ***********
            stackedBannerScreen.removeEventListener(StackedBannerScreen.CHALLENGES_LEADERBOARD_SELECTED, challengesLeaderBoardSelectHandler);
            stackedBannerScreen.removeEventListener(StackedBannerScreen.CHALLENGES_RATE_SELECTED, challengesRateSelectHandler);
            stackedBannerScreen.removeEventListener(StackedBannerScreen.CHALLENGES_GARAGE_SELECTED, challengesGarageSelectHandler);
            stackedBannerScreen.removeEventListener(StackedBannerScreen.CHALLENGES_NOTIFICATIONS_SELECTED, challengesNotesSelectHandler);
            stackedBannerScreen.removeEventListener(StackedBannerScreen.CHALLENGES_HELP_SELECTED, challengesHelpSelectHandler);
            //
            stackedBannerScreen.removeEventListener(StackedScreen.DISPLAY_REWARD_INFO_POPUP, displayRewardInfoPopUp);
            stackedBannerScreen.removeEventListener(StackedBannerScreen.UPDATE_USER_DATA, userDataUpdatedHandler);
            //

        }


        override public function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_AUTHENTICATED,
                Notifications.APPLICATION_BECAME_ACTIVE,
                Notifications.REWARD_ARS_LOAD,
                Notifications.REWARD_ARS_OPENED,
                Notifications.REWARD_DATA_CHANGE,
                Notifications.DISPLAY_CHALLENGE_LOADER,
                Notifications.HIDE_CHALLENGE_LOADER,
                Notifications.USER_LOGIN_CANCELLED,
                Notifications.USER_DATA_UPDATED
            ]);
        }

        protected function updateUserData():void{
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(UsersProxy.loggedInUser()){
                usersProxy.loadCurrentUser(function (result:Object):void {
                    if(result){
                        stackedBannerScreen.updateUserData(result);
                    }else{
                        clearUserData();
                    }
                });
            }else{
                clearUserData();
            }
        }

        private function clearUserData():void{
            stackedBannerScreen.updateUserData(null);
        }

        protected function challengeScreenHomeSelectedSelectedHandler(event: Event):void {
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        protected function backButtonSelectedSelectedHandler(event: Event):void {
            sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN, {backLevel: stackedScreen.backLevel});
        }

        private function displayRewardInfoPopUp(event: Event): void {
            userAuthenticate(StackedScreenExtendedMediator.AUTH_REWARD_INFO_POPUP);
        }

        private function userAuthenticate(pendingForAction : String, pendingForData : Object = null):void {
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

            if (!usersProxy.currentUser) _garage_delay = 1.5;
            else _garage_delay = 0.2;

            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

            pendingForAuthAction = pendingForAction;
            pendingForAuthData = pendingForData;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.APPLICATION_BECAME_ACTIVE:
                    handleApplicationBecameActive();
                    break;
                case Notifications.USER_AUTHENTICATED:
                    handleUserAuthenticated();
                    break;
                case Notifications.REWARD_ARS_LOAD:
                    stackedBannerScreen.updateRewardsAdsStatus();
                    break;
                case Notifications.REWARD_ARS_OPENED:
                    stackedBannerScreen.updateRewardsAdsStatus();
                    break;
                case Notifications.REWARD_DATA_CHANGE:
                    updateUserData();
                    break;
                case Notifications.DISPLAY_CHALLENGE_LOADER:
                    stackedBannerScreen.showLoadBar();
                    break;
                case Notifications.HIDE_CHALLENGE_LOADER:
                    stackedBannerScreen.hideLoadBar();
                    break;
                case Notifications.USER_LOGIN_CANCELLED:
                    stackedBannerScreen.hideLoadBar();
                    break;
                case Notifications.USER_DATA_UPDATED:
                    handleUserDataUpdate(note.getBody());
                    break;
            }
            super.handleNotification(note);
        }

        private function handleUserDataUpdate(userData:Object):void {
            if(userData){
                stackedBannerScreen.updateUserData(userData);
            }
        }

        private function handleApplicationBecameActive() : void {
            if(Device.isIos()) {
                return;
            }

            if(stackedBannerScreen) {
                stackedBannerScreen.invalidate();
            }
        }

        protected function handleUserAuthenticated() : void {
            switch(pendingForAuthAction){
                case StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_GARAGE:
                    sendNotification(Notifications.DISPLAY_USER_CHALLENGE_TUNINGS, ObjectUtils.merge({userGarage: true}, pendingForAuthData));
                    break;
                case StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS:
                    sendNotification(Notifications.DISPLAY_CHALLENGES_NOTIFICATIONS_SCREEN, pendingForAuthData);
                    break;
                case StackedScreenExtendedMediator.AUTH_REWARD_INFO_POPUP:
                    AdsManager.$.showBannerAfterComplete = false;
                    AdsManager.$.bannerHide(false);
                    sendNotification(Notifications.DISPLAY_REWARD_INFO_POPUP, pendingForAuthData);
                    break;
            }
        }

        // ***** Challenge tabs change handler
        private function challengesLeaderBoardSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_SCREEN, event.data);
        }

        private function challengesRateSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGES_VOTING_SCREEN, event.data);
        }

        private function challengesGarageSelectHandler(event : Event) : void {
            userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_GARAGE, event.data);
        }

        private function challengesNotesSelectHandler(event : Event) : void {
            userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS, event.data);
        }

        private function challengesHelpSelectHandler(event : Event) : void {

        }

        protected function userDataUpdatedHandler(event:Event):void {

        }

        public function get stackedBannerScreen():StackedBannerScreen {
            return viewComponent as StackedBannerScreen;
        }
    }
}
