package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.timeline.TimelineProxy;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentData;
import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineFeedViewPopup;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;

import starling.events.Event;

public class TimelineFeedViewPopupMediator extends PopUpMediator {
    private var _addCommentModel : FeedData;
    private var _timelineFeedPreview:TimelineFeedViewPopup;
    public function TimelineFeedViewPopupMediator(popUp:PopUp) {
        super(popUp);
        _timelineFeedPreview = popUp as TimelineFeedViewPopup;
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(),
                [Notifications.TIMELINE_COMMENT_ADDED]
        )
    }

    override public function onRegister():void {
        super.onRegister();
        _timelineFeedPreview.addEventListener(FeedData.COMMENT_SEND, sendCommentHandler);
        _timelineFeedPreview.addEventListener(FeedData.COMMENT_EDIT, editCommentHandler);
    }

    override public function onRemove():void {
        super.onRemove();
        _timelineFeedPreview.removeEventListener(FeedData.COMMENT_SEND, sendCommentHandler);
        _timelineFeedPreview.removeEventListener(FeedData.COMMENT_EDIT, editCommentHandler);
    }

    private function sendCommentHandler(event:Event):void {
        var timelineProxy : TimelineProxy = facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
        var model : FeedData = event.data.data;
        var text : String = event.data.text;
        if(!model.isTuningUse) {
            timelineProxy.addCommentToPost(model.id, text);
        } else {
            timelineProxy.addCommentToTuning(model.tuning._id, text);
        }
        _addCommentModel = model;
    }

    override public function handleNotification(note:INotification):void {
        super.handleNotification(note);
        switch(note.getName()) {
            case Notifications.TIMELINE_COMMENT_ADDED:
                handleTimelineCommentAdded(note.getBody());
                break;
        }
    }

    private function handleTimelineCommentAdded(data : Object) : void {
        var comment : CommentData = new CommentData(data.comment);
        _addCommentModel.commentsPocket.updateComment(comment);
        _addCommentModel = null;
    }

    private function editCommentHandler(event:Event):void {
        var timelineProxy : TimelineProxy = facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
        var model : FeedData = event.data.feedData;
        var commendData : CommentData= event.data.comment;
        if(!model.isTuningUse) {
            timelineProxy.editPostComment(model.id, commendData.comment, commendData.id);
        } else {
            timelineProxy.editTuningComment(model.tuning._id, commendData.comment, commendData.id);
        }
        _addCommentModel = model;
    }

}
}
