package com.despark.tuning3d.view
{
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.cars.CarBackground;
import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.friend.FriendProxy;
import com.despark.tuning3d.model.languages.LanguagesProxy;
import com.despark.tuning3d.model.settings.Settings;
import com.despark.tuning3d.model.settings.SettingsProxy;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.tunings.editing.TuningSettingsPopUp;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.flexnroses.mobile.utils.hardware.ExifUtils;
import com.utils.ArrayUtil;
import com.utils.Logger;

import feathers.core.PopUpManager;

import flash.display.Bitmap;
import flash.display.BitmapData;
import flash.display.Loader;
import flash.events.Event;
import flash.events.MediaEvent;
import flash.events.StatusEvent;
import flash.geom.Rectangle;
import flash.media.Camera;
import flash.media.CameraRoll;
import flash.media.MediaPromise;
import flash.net.URLRequest;

import jp.shichiseki.exif.ExifInfo;
import jp.shichiseki.exif.ExifLoader;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;
import starling.events.Event;

import uk.soulwire.utils.display.DisplayUtils;

public class TuningSettingsPopUpMediator extends PopUpMediator
	{		
		public static const AUTH_ACTION_UPLOAD_BACKGROUND : String = "uploadBackground";
		public static const AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA : String = "AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA";
		
		private var cameraRoll : CameraRoll;
		private var imageLoader : Loader;
		private var pendingForAuthAction : String;
		private var exifLoader : ExifLoader;
		private var mediaPromise : MediaPromise;
		
		public function TuningSettingsPopUpMediator(popUp : PopUp)
		{
			//Logger.d("TuningSettingsPopUpMediator", this);
			super(popUp);
		}
		
		public function get tuningSettingsPopUp() : TuningSettingsPopUp
		{
			return TuningSettingsPopUp(popUp);
		}
		
		override public function onRegister() : void
		{
			super.onRegister(); 
			
			tuningSettingsPopUp.addEventListener(starling.events.Event.ADDED_TO_STAGE, tuningSettingsPopUpAddedToStageHandler);
			tuningSettingsPopUp.addEventListener(TuningSettingsPopUp.BACKGROUND_SELECTED, tuningSettingsPopUpBackgroundSelectedHandler);
			tuningSettingsPopUp.addEventListener(TuningSettingsPopUp.LANGUAGE_CHANGED, tuningSettingsPopUpLanguageChangedHandler);
			tuningSettingsPopUp.addEventListener(TuningSettingsPopUp.UPLOAD_BACKGROUND_SELECTED, tuningSettingsPopUpUploadBackgroundSelectedHandler);
			tuningSettingsPopUp.addEventListener(TuningSettingsPopUp.DELETE_BACKGROUND_SELECTED, tuningSettingsDeleteBackgroundSelectedHandler);
			//
			tuningSettingsPopUp.addEventListener(TuningSettingsPopUp.SNAPSHOT_CAMERA_SELECTED, tuningSettingsSnapshotCameraSelectedHandler);
			
			//
		}
		
		override public function onRemove() : void
		{
			super.onRemove();
			
			var carBackgroundsProxy : CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
			carBackgroundsProxy.cancelUploading();
			
			tuningSettingsPopUp.removeEventListener(starling.events.Event.ADDED_TO_STAGE, tuningSettingsPopUpAddedToStageHandler);
			tuningSettingsPopUp.removeEventListener(TuningSettingsPopUp.BACKGROUND_SELECTED, tuningSettingsPopUpBackgroundSelectedHandler);
			tuningSettingsPopUp.removeEventListener(TuningSettingsPopUp.LANGUAGE_CHANGED, tuningSettingsPopUpLanguageChangedHandler);
			tuningSettingsPopUp.removeEventListener(TuningSettingsPopUp.UPLOAD_BACKGROUND_SELECTED, tuningSettingsPopUpUploadBackgroundSelectedHandler);
			tuningSettingsPopUp.removeEventListener(TuningSettingsPopUp.DELETE_BACKGROUND_SELECTED, tuningSettingsDeleteBackgroundSelectedHandler);
			tuningSettingsPopUp.removeEventListener(TuningSettingsPopUp.SNAPSHOT_CAMERA_SELECTED, tuningSettingsSnapshotCameraSelectedHandler);
			//tuningSettingsPopUp.removeEventListener(CarView.SPNAPSHOT_FROM_CAMERA_CREATED, handleSnapShotFromCameraCreated);
			
			//Logger.d("onRemove", this);
		}
		
		public override function listNotificationInterests() : Array 
		{
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.CAR_BACKGROUNDS_LOADED,
				Notifications.CAR_BACKGROUNDS_LOADING_FAILED,
				Notifications.CAR_BACKGROUND_UPLOADED,
				Notifications.CAR_BACKGROUND_UPLOADING_FAILED,
				Notifications.USER_AUTHENTICATED,
				Notifications.USER_LOGIN_CANCELLED,
				Notifications.CAR_BACKGROUND_DELETED,
				Notifications.CAR_BACKGROUND_DELETION_FAILED,
				Notifications.LANGUAGE_LOADED,
				Notifications.LANGUAGE_LOADING_FAILED
			]);
		}
		
		public override function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case Notifications.CAR_BACKGROUNDS_LOADED:
					handleCarBackgroundsLoaded(note.getBody() as Array);
					break;
				
				case Notifications.CAR_BACKGROUNDS_LOADING_FAILED:
					handleCarBackgroundsLoadingFailed();
					break;
				
				case Notifications.CAR_BACKGROUND_UPLOADED:
					handleCarBackgroundUploaded();
					break;
				
				case Notifications.CAR_BACKGROUND_UPLOADING_FAILED:
					handleCarBackgroundUploadedFailed(note.getBody() as String);
					break;
				
				case Notifications.USER_AUTHENTICATED:
					handleUserAuthenticated(note.getBody().wasAlreadyAuthenticated);
					break;
				
				case Notifications.USER_LOGIN_CANCELLED:
					handleUserLoginCancelled();
					break;
				
				case Notifications.CAR_BACKGROUND_DELETED:
					handleCarBackgroundDeleted();
					break;
				
				case Notifications.CAR_BACKGROUND_DELETION_FAILED:
					handleCarBackgroundDeletionFailed();
					break;
				
				case Notifications.LANGUAGE_LOADED:
					handleLanguageLoaded();
					break;
				
				case Notifications.LANGUAGE_LOADING_FAILED:
					handleLanguageLoadingFailed();
					break;
			}
			
			super.handleNotification(note);
		}
		
		//
		
		private function tuningSettingsPopUpAddedToStageHandler(event : starling.events.Event) : void
		{
			Starling.juggler.delayCall(refreshSettings, 0.2);
		}
		
		
		
		private function upload_bitmapdataFromCameraIfNeed() : void
		{
			
			if(tuningSettingsPopUp.bitmapdataFromCamera){
				
				//увеличем размер картинки до минимальных размеров если необходимо
				
				uploadBitmapData(tuningSettingsPopUp.bitmapdataFromCamera);
				tuningSettingsPopUp.bitmapdataFromCamera.dispose();
				tuningSettingsPopUp.bitmapdataFromCamera = null;
				
			}
		}
		
		
		
		private function handleCarBackgroundsLoaded(carBackgrounds : Array) : void
		{
			//Logger.d("handleCarBackgroundsLoaded", this);
			
			
			
			alertView.hide();
			tuningSettingsPopUp.carBackgrounds = carBackgrounds;
		}
		
		private function handleCarBackgroundsLoadingFailed() : void
		{
			
			Logger.e("handleCarBackgroundsLoadingFailed", this);
			
		
			alertView.hide();
		}
		
		private function tuningSettingsPopUpBackgroundSelectedHandler(event : starling.events.Event) : void
		{
			tuningSettingsPopUp.touchable = false;
			Starling.juggler.delayCall(function() : void
			{
				try{
				
					
					var copyOfCarBackground : CarBackground = null;
					if(tuningSettingsPopUp.selectedBackground){
						copyOfCarBackground = tuningSettingsPopUp.selectedBackground.copyWithShowMainUrl();
					}
					
					sendNotification(Notifications.CAR_BACKGROUND_SELECTED, copyOfCarBackground);
					sendNotification(Notifications.REMOVE_POP_UP, tuningSettingsPopUp);
					
				}catch(e:Error){
					
					Logger.e("tuningSettingsPopUpBackgroundSelectedHandler error:"+e.messages, this);
				}
				
				
				
			}, 0.2);
		}
		
		private function updateSettings() : void
		{
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var settings : Settings = settingsProxy.currentSettings;
			
			settings.autoscrollEnabled = tuningSettingsPopUp.autoscrollEnabled;
			
			if(tuningSettingsPopUp.selectedLanguage)
			{
				settings.selectedLanguageCode = tuningSettingsPopUp.selectedLanguage.code;
			}
			
			settingsProxy.update(settings);
		}
		
		protected override function popUpCloseButtonSelectedHandler(event : starling.events.Event) : void
		{
			updateSettings();
			
			super.popUpCloseButtonSelectedHandler(event);
		}
		
		private function tuningSettingsPopUpLanguageChangedHandler(event : starling.events.Event) : void
		{
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			languagesProxy.loadLanguage(tuningSettingsPopUp.selectedLanguage);
			
			if(alertView)
			{
				alertView.hide();
			}
			
			alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1, true);
		}
		
		private function handleLanguageLoaded() : void
		{
			if(alertView)
			{
				alertView.hide();
			}
			
			tuningSettingsPopUp.closeLanguagePickerList();
			updateSettings();
			
			sendNotification(Notifications.LANGUAGE_CHANGED);
		}
		
		private function handleLanguageLoadingFailed() : void
		{

			if(alertView)
			{
				alertView.hide();
			}
			
			tuningSettingsPopUp.selectedLanguage = null;
			tuningSettingsPopUp.closeLanguagePickerList();
		}
		
		//
		
		
		//
		private function tuningSettingsSnapshotCameraSelectedHandler(event : starling.events.Event) : void
		{
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			
			if(!usersProxy.currentUser)
			{
				
				ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
				
				//tuningSettingsPopUp
				PopUpManager.addPopUp(tuningSettingsPopUp, false); //change by maxb showPopUp to addPopUp
				
				pendingForAuthAction = AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA;
				
				sendNotification(Notifications.AUTHENTICATE_USER);
				
				return;
			}
			//sendNotification(Notifications.CAMERA_SNAPSHOT,{param1: tuningSettingsPopUp.testInputView1.text,param2: tuningSettingsPopUp.testInputView2.text,param3: tuningSettingsPopUp.testInputView3.text});
			
			//camera=Camera.getCamera();
			
			
			//if(camera){
			
				//camera.addEventListener(StatusEvent.STATUS, cameraStatusHandler);
				
				
				sendNotification(Notifications.CAMERA_SNAPSHOT);
				sendNotification(Notifications.REMOVE_POP_UP, tuningSettingsPopUp);
				
			/*}else{
				
				AlertView.show( currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"),-1,  false) ;
			}*/
		}
		
		private var camera:Camera;
		
		private function cameraStatusHandler(event:StatusEvent):void{
			
			if(!camera)
				return;
			
			if (event.code == "Camera.Unmuted") {
				//connectCamera(); 
				camera.removeEventListener(StatusEvent.STATUS, cameraStatusHandler);
				sendNotification(Notifications.CAMERA_SNAPSHOT);
				sendNotification(Notifications.REMOVE_POP_UP, tuningSettingsPopUp);
			
			}else{
				
				
				
				AlertView.show( currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"),-1,  false) ;
			}
		}
		
		private function tuningSettingsPopUpUploadBackgroundSelectedHandler(event : starling.events.Event) : void
		{
			selectImageFromCameraRoll();
		}
		
		private function selectImageFromCameraRoll() : void
		{
			pendingForAuthAction = AUTH_ACTION_UPLOAD_BACKGROUND;
			ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
		
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			
			if(!usersProxy.currentUser)
			{
				//PopUpManager.
				PopUpManager.addPopUp(tuningSettingsPopUp, false); //change by maxb showPopUp to addPopUp
			}
			
			sendNotification(Notifications.AUTHENTICATE_USER);
		}
		
		private function uploadBitmapData(bitmapData : BitmapData) : void
		{
			var processedBitmapData : BitmapData = processedBitmapDataForUpload(bitmapData);
			
			bitmapData.dispose();
			
			tuningSettingsPopUp.enterUploadingState();
			
			var carBackgroundsProxy : CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
			carBackgroundsProxy.upload(processedBitmapData);
		}
		
		private function processedBitmapDataForUpload(bitmapData : BitmapData) : BitmapData
		{
			HttpClient.saveTestFile("bitmap1.png",bitmapData);
			var bitmap : Bitmap = DisplayUtils.createThumb(bitmapData, Constants.MIN_UPLOAD_IMAGE_WIDTH, Constants.MIN_UPLOAD_IMAGE_HEIGHT);
			HttpClient.saveTestFile("bitmap2.png",bitmap.bitmapData);
			
			return bitmap.bitmapData;
		}
		
		private function handleCarBackgroundUploaded() : void
		{			
			AlertView.show(AlertView.language.translate("mob_saving", "uploaded"));
			tuningSettingsPopUp.enterNormalState();
			
			if(alertView)alertView.hide();
			alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1, true);
			Starling.juggler.delayCall(refreshSettings, 8.0);
		}
		
		private function refreshSettings() : void 
		{
			
			
			
			if(alertView)alertView.hide();
			alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1, true);
			
			var carBackgroundsProxy : CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
			carBackgroundsProxy.loadBackgrounds(getNeededUser(), true);
			
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			var settings : Settings = settingsProxy.currentSettings;
			
			if(!settings.selectedLanguageCode)
			{
				settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
			}
			
			tuningSettingsPopUp.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
			
			tuningSettingsPopUp.settings = settings;
			tuningSettingsPopUp.languages = languagesProxy.languages;
			
			
			
			
			upload_bitmapdataFromCameraIfNeed();
			
		}

        private function getNeededUser() : String
        {
            var userID : String = null;

            if (Constants.selectedUser){
                userID = Constants.selectedUser.id;
            }
            else if(facade.retrieveProxy(UsersProxy.NAME) as UsersProxy){
                userID = (facade.retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser.id;
            }

            return userID;
        }

		private function handleCarBackgroundUploadedFailed(error : String = null) : void
		{
			tuningSettingsPopUp.enterNormalState();
			
			AlertView.show(AlertView.language.translate("mob_saving", "uploadingFailed"), -1, false);
		}
		
		private function handleUserAuthenticated(wasAlreadyAuthenticated : Boolean) : void
		{
			if(!tuningSettingsPopUp.stage)
			{
				return;
			}
			
			ImageLoaderWithStates.disposeWhenRemovedFromStage = true;
			
			if(pendingForAuthAction == AUTH_ACTION_UPLOAD_BACKGROUND)
			{
				
				//PopUpManager.addPopUp(tuningSettingsPopUp, true); //change by maxb showPopUp to addPopUp
				
				if(!wasAlreadyAuthenticated)
				{
					refreshSettings();
				}
				
				if(Device.isIos()) 
				{
					browseForImage_iOS();
				}
				else
				{
					browseForImage_Android();
				}
			}else if(pendingForAuthAction == AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA)
			{
				
				tuningSettingsSnapshotCameraSelectedHandler(null);
				
				
			}
			
			
			//
            var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
            friendProxy.loadPendings();
			pendingForAuthAction = null;
		}
		
		private function scaleRectangle(rectangle : Rectangle, factor : Number) :void
		{
			rectangle.x *= factor;
			rectangle.y *= factor;
			rectangle.width *= factor;
			rectangle.height *= factor;
		}
		
		private function browseForImage_iOS() : void
		{
//			if (AirImagePicker.getInstance().isImagePickerAvailable())
//			{
//				var anchor : Rectangle = tuningSettingsPopUp.uploadBackgroundButton.getBounds(tuningSettingsPopUp.stage);
//				scaleRectangle(anchor, tuningSettingsPopUp.assetManager.scaleFactor);
//
//				AirImagePicker.getInstance().displayImagePicker(
//					function(status : String, ...mediaArgs) : void
//					{
//						if(status == AirImagePicker.STATUS_OK)
//						{
//							var bitmapData : BitmapData = mediaArgs[0];
//
//							processAndUploadBitmapData(bitmapData);
//						}
//					},
//					0,
//					0,
//					anchor);
//			}
//			else
//			{
//				AlertView.show("Browsing in camera roll is not supported.", -1, false);
//			}
		}
		
		
		
		private function processAndUploadBitmapData(bitmapData : BitmapData) : void
		{
			if(bitmapData.width >= Constants.MIN_UPLOAD_IMAGE_WIDTH && 
				bitmapData.height >= Constants.MIN_UPLOAD_IMAGE_HEIGHT)
			{
				uploadBitmapData(bitmapData);
			}
			else 
			{
				AlertView.show(AlertView.language.translate("mob_saving", "imageSizeValidation") + " " + Constants.MIN_UPLOAD_IMAGE_WIDTH + "x" + Constants.MIN_UPLOAD_IMAGE_HEIGHT + ".", -1, false);
			}
		}
		
		private function browseForImage_Android() : void
		{
			if(CameraRoll.supportsBrowseForImage )
			{
				cameraRoll = new CameraRoll(); 
				cameraRoll.addEventListener(MediaEvent.SELECT, cameraRollSelectHandler);
				cameraRoll.browseForImage();  
			}
			else
			{
				AlertView.show("Browsing in camera roll is not supported.", -1, false);
			}
		}
		
		private function cameraRollSelectHandler(event : MediaEvent) : void
		{     
			mediaPromise = event.data;
			
			exifLoader = new ExifLoader();
			exifLoader.addEventListener(flash.events.Event.COMPLETE, exifLoaderCompleteHandler );
			exifLoader.load( new URLRequest( mediaPromise.file.url ) );
		}
		
		private function exifLoaderCompleteHandler(event : flash.events.Event) : void
		{
			if(imageLoader)
			{
				imageLoader.unload();
			}
			
			imageLoader = new Loader();
			imageLoader.contentLoaderInfo.addEventListener(flash.events.Event.COMPLETE, imageLoaderCompleteHandler);
			imageLoader.loadFilePromise(mediaPromise);
		}
		
		private function imageLoaderCompleteHandler(event : flash.events.Event) : void
		{
			var exif:ExifInfo = this.exifLoader.exif;
			var rotation:int = ExifUtils.getEyeOrientedAngle( exif.ifds );
			
			var bitmap : Bitmap = ExifUtils.getEyeOrientedBitmap(Bitmap(event.currentTarget.content), exif.ifds);
			
			processAndUploadBitmapData(bitmap.bitmapData);
			
		}
		
		private function imageLoaderIoErrorHandler(event : flash.events.Event) : void
		{
			AlertView.show("Could not load selected image.", -1, false);
		}
		
		
		private function handleUserLoginCancelled() : void
		{
			if(pendingForAuthAction == AUTH_ACTION_UPLOAD_BACKGROUND)
			{
				PopUpManager.addPopUp(tuningSettingsPopUp, true); //change by maxb showPopUp to addPopUp
			}else if(pendingForAuthAction == AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA)
			{
				PopUpManager.addPopUp(tuningSettingsPopUp, true); //change by maxb showPopUp to addPopUp
			}
			
			pendingForAuthAction = null;
		}
		
		private function tuningSettingsDeleteBackgroundSelectedHandler(event : starling.events.Event) : void
		{
			AlertView.show(AlertView.language.translate("mob_saving", "areYouSure"), -1, false, function(yesSelected : Boolean) : void
			{
				if(yesSelected)
				{
					tuningSettingsPopUp.enterDeletingState();
					deleteSelectedBackground();
				}
			});
		}
		
		private function deleteSelectedBackground() : void
		{
			var carBackgroundsProxy : CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
			carBackgroundsProxy.deleteBackground(tuningSettingsPopUp.selectedBackground);
		}
		
		private function handleCarBackgroundDeleted() : void
		{
			tuningSettingsPopUp.enterNormalState();
			refreshSettings();
		}
		
		private function handleCarBackgroundDeletionFailed() : void
		{
			tuningSettingsPopUp.enterNormalState();
		}
	}
}