/**
 * Created by Render-2 on 8/23/2017.
 */
package com.despark.tuning3d.view.components.NotificationView {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.DateParser;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;

    import flash.display.BitmapData;
    import flash.geom.Rectangle;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;

    public class NotificationPanelViewItemRenderer extends ListItemRenderer{
    private var _gap:int;
    private static const _vGap:Number = 2.5;
    private var _bgQuad:Quad;
    private var _nameLabel:MultilingualLabel;
    private var _descriptionLabel:MultilingualLabel;
    private var _bottomLineQuad:Quad;

    private var _carNameLabel:MultilingualLabel;
    private var _avatarXMargin:Number;
    private var _avatarSize:Number;
    private var _avatar : UserCircularAvatar;

    private var _eventDispatcher : *;
    private var _labelContainer:LayoutGroup;
    private var _dateContainer:LayoutGroup;
    private var _dateLabel:Label;

    private var _postName:Button;

    private var _iconImage:Image;
    private var _iconName:String;
    private var _iconTailImage:Quad;
    private var _borderImage:Image;
    private var _dateImage:Image;
    private static var _borderTexture:Texture;

    public function set eventDispatcher(value:*) : void{
        _eventDispatcher = value;
    }
    public function get eventDispatcher() : *{
        return _eventDispatcher;
    }

    public function NotificationPanelViewItemRenderer() {
    }

    override protected function initialize():void
    {
        super.initialize();

        _gap = 10;
        _avatarXMargin = 10;
        _avatarSize = 40;

        _bottomLineQuad = new Quad(1, 1, 0x9a9899);
        addChild(_bottomLineQuad);

        _iconTailImage = new Quad(1, 1, 0xe0e0e0);
        _iconTailImage.width = 15;
        _iconTailImage.height = 0.5;

        _bgQuad = new Quad(1,1,0xffffff);
        addChild(_bgQuad);

        var labelLayout : HorizontalLayout = new HorizontalLayout();
        labelLayout.horizontalAlign = HorizontalAlign.LEFT;
        labelLayout.verticalAlign = VerticalAlign.MIDDLE;
        labelLayout.gap = 3;
        _labelContainer = new LayoutGroup();
        _labelContainer.layout = labelLayout;
        addChild(_labelContainer);
        _labelContainer.x = leftMargin + _avatarSize + _avatarXMargin + _gap;

        _nameLabel = new MultilingualLabel();
        _nameLabel.textRendererFactory = NotificationPanelLabelFactory.nameLabelRenderer;
        _nameLabel.addEventListener(TouchEvent.TOUCH,userDetailsSelectHandler);
        _nameLabel.y = _gap;
        _labelContainer.addChild(_nameLabel);

        _descriptionLabel = new MultilingualLabel();
        _descriptionLabel.textRendererFactory = NotificationPanelLabelFactory.textLabelRenderer;
        _descriptionLabel.x = _gap;

        _labelContainer.addChild(_descriptionLabel);

        _carNameLabel= new MultilingualLabel();
        _carNameLabel.textRendererFactory = NotificationPanelLabelFactory.carNameLabelRenderer;
        _carNameLabel.addEventListener(TouchEvent.TOUCH,onCarSelectHandler);
        _labelContainer.addChild(_carNameLabel);

        _postName = new Button();
        _postName.labelFactory = NotificationPanelLabelFactory.postNameRenderer;
        _labelContainer.addChild(_postName);


        _dateContainer = new LayoutGroup();
        var dateLayout:HorizontalLayout = new HorizontalLayout();
        dateLayout.verticalAlign = VerticalAlign.MIDDLE;
        _dateContainer.layout = dateLayout;

        addChild(_dateContainer);
        _dateImage = new Image(ButtonFactory.assetManager.getTexture("dateIcon"));
        _dateImage.scale = ButtonFactory.assetManager.scaleFactor/2;
        _dateContainer.addChild(_dateImage);

        _dateLabel = new Label();
        _dateLabel.textRendererFactory = NotificationPanelLabelFactory.postDateLabelRenderer;
        _dateContainer.addChild(_dateLabel);
        _dateLabel.x = _dateImage.width;
        _dateImage.y = (_dateLabel.height-_dateImage.height)/2;

        _labelContainer.x = leftMargin + _avatarSize + _avatarXMargin + _gap;
        _labelContainer.y = (actualHeight-getInfoHeight())/2;
    }


    public function get feedData():NotificationFeedData {
        return _data as NotificationFeedData;
    }

    override protected function commitData():void {
        super.commitData();
        update();
    }


    override protected function commitSelection():void {
    }

    /**
     * "new_friend",
     * "tuning_like",
     * "tuning_comment",
     * "post_comment",
     */
    private function update():void
    {
        if(!feedData) return;
        _carNameLabel.text = "";
        _carNameLabel.visible = false;

        _postName.label = "";
        _postName.visible = false;
        switch(feedData.feedType) {
            case FeedData.TYPE_NEW_FRIEND:
            case NotificationFeedData.TYPE_POST_COMMENT:
            case NotificationFeedData.TYPE_POST_LIKE:
                _nameLabel.text = feedData.creator.nameToShow;
                _nameLabel.visible = true;
                _nameLabel.validate();

                _descriptionLabel.text = feedData.descriptionLabel;
                _descriptionLabel.visible = true;
                _descriptionLabel.validate();

                if(feedData.feed){
                    if(feedData.feed.text && feedData.feed.text.length) {
                        var postNameText:String = feedData.feed.text.substr(0, Math.min(feedData.feed.text.length - 1, 19)) +(feedData.feed.text.length>20?'...':'');
                        var rex:RegExp = /[\s\r\n]+/gim;
                        postNameText = postNameText.replace(rex,' ');
                        _postName.label = '"' + postNameText + '"';
                        _postName.visible = true;
                    }else if(feedData.feed.date){
                        _postName.label = feedData.feed.date.toDateString();
                        _descriptionLabel.text = _descriptionLabel.text + " from";
                        _postName.visible = true;
                    }
                }
                break;
            case FeedData.TYPE_TUNING_LIKE:
            case FeedData.TYPE_TUNING_COMMENT:
                _nameLabel.text = feedData.creator.nameToShow;
                _nameLabel.visible = true;
                _nameLabel.validate();

                _descriptionLabel.text = feedData.descriptionLabel;
                _descriptionLabel.visible = true;
                _descriptionLabel.validate();


                _carNameLabel.text = feedData.getCarFullName();
                _carNameLabel.visible = true;
                _carNameLabel.validate();

                break;
        }

        _dateLabel.text = DateParser.getDateAppropriateText(feedData.availableDate);
        _dateLabel.validate();
        if(_dateImage){
            _dateImage.y = (_dateLabel.height - _dateImage.height)/2;
        }
        _dateContainer.y = _labelContainer.y + _labelContainer.height + 2; //_borderImage?(_borderImage.y+_borderImage.height-_dateContainer.height):(actualHeight - _dateLabel.height);
        _dateContainer.x = _labelContainer.x; //(actualWidth - _dateLabel.width)/2;

        updateAvatar(feedData.creator);
        updateCarThumbnail();
        updateLabelPosition();
        updateIcon();
    }

    private function updateIcon():void {
        if(feedData.iconTextureName && feedData.iconTextureName != _iconName){
            _iconName = feedData.iconTextureName;
            if(_iconImage){
                _iconImage.removeFromParent(true);
            }
            _iconImage = new Image(ButtonFactory.assetManager.getTexture(_iconName));
            _iconImage.scale = .75 * ButtonFactory.assetManager.scaleFactor/2;
            addChild(_iconImage);
            if(!_iconTailImage.parent){
                addChild(_iconTailImage);
            }

            _borderTexture ||= function ():Texture {
                var bd:BitmapData = new BitmapData(20, 20, true, 0xFFe0e0e0);
                bd.fillRect(new Rectangle( 1, 1, 18, 18), 0xFFFFFFFF);
                var txt:Texture = Texture.fromBitmapData(bd);
                Image.bindScale9GridToTexture(txt, new Rectangle(5,5,10,10));
                return txt;
            }();

            _borderImage ||= new Image(_borderTexture);
            _borderImage.alpha = 0.6;
            _borderImage.width = actualWidth - (leftMargin + rightMargin);
            _borderImage.height = _bgQuad.height - 2 * _gap;
            if(!_borderImage.parent){
                addChildAt(_borderImage, getChildIndex(_bgQuad)+1);
            }
            invalidate();
        }
    }

    private function get leftMargin():Number{
        return Device.isIphoneXVariant()?90:((_iconImage?_iconImage.width:0) + (_iconTailImage?_iconTailImage.width:0));
    }

    private function get rightMargin():Number{
        return Device.isIphoneXVariant()?45:_gap;
    }

    private function updateLabelPosition():void {
        var diff:Number = (_avatar?(_avatarSize + 2 * _avatarXMargin):_avatarXMargin) + imageLoader?imageLoader.width:0;
        _labelContainer.width = (_borderImage?_borderImage.width:actualWidth) - diff;
        _labelContainer.x = leftMargin + _avatarSize + _avatarXMargin + _gap;
        _labelContainer.y = _avatar?(_avatar.y+(_avatar.height-(_labelContainer.height+_dateContainer.height))/2):(actualHeight-_carNameLabel.height-_gap)/2;
        _labelContainer.validate();
    }

    private function updateCarThumbnail():void {
        if(feedData.tuning) {

            addChild(imageLoader);
            imageLoader.textureScale = 0.25;
            imageLoader.visible = true;
            var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
            simpleImageProvider.urlForDownloading = feedData.thumbURL;
            simpleImageProvider.sourceUrl = 'http://data.3dtuning.com/tun/';
            simpleImageProvider.destinationPath = "cars/thumb/";


            if(!imageLoader.recieveImageSize){
                imageLoader.recieveImageSize=completeLoader;
            }

            if(!imageLoader.errorHandler){
                imageLoader.errorHandler=function ()  : void{
                    trace("error download");
                };
            }

            if (imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != simpleImageProvider.remoteUrl)
            {
                imageLoader.persistentImageProvider = simpleImageProvider;
            }
            else if (!imageLoader.persistentImageProvider)
            {
                imageLoader.persistentImageProvider = simpleImageProvider;
            }
        }
        else {
            imageLoader.visible = false;
        }
    }

    private function completeLoader(w:Number, h:Number):void {
        updateThumbnailPositionAndSize();
        imageLoader.addEventListener(TouchEvent.TOUCH, onCarSelectHandler);
    }


    private function updateThumbnailPositionAndSize() : void{
        if(imageLoader){
            if(imageLoader.image && imageLoader.image.texture){
                var aspect : Number = imageLoader.image.texture.nativeWidth / imageLoader.image.texture.nativeHeight;
                imageLoader.height = _borderImage.height - 2;
                imageLoader.width = imageLoader.height * aspect;

                imageLoader.x = actualWidth - (rightMargin + imageLoader.width);
                imageLoader.y = (actualHeight - imageLoader.height ) / 2;
            }
        }

    }

    private function updateAvatar(userIconToShow : User) : void
    {
        if(!_avatar )
        {
            _avatar = new UserCircularAvatar(userIconToShow, _avatarSize);
            _avatar.x = leftMargin + _avatarXMargin;
            _avatar.y = (actualHeight - _avatar.height)/2;
            _avatar.addEventListener(TouchEvent.TOUCH, userDetailsSelectHandler);
            addChild(_avatar);
            updateLabelPosition();
        }else {
            _avatar.user = userIconToShow;
        }
    }

    private function getInfoHeight():Number{
        return _labelContainer.height + _dateContainer.height + 2;
    }

    override protected function draw():void {
        super.draw();

        _bgQuad.width = actualWidth;
        _bgQuad.height = actualHeight - 2*_vGap;
        _bgQuad.y = _vGap;

        _bottomLineQuad.y = _bgQuad.y + _bgQuad.height;
        _bottomLineQuad.width = actualWidth;
        _bottomLineQuad.height = 0.5;
        _dateLabel.validate();
        _dateContainer.validate();
        _dateContainer.y = _labelContainer.y + _labelContainer.height + 2;//_borderImage?(_borderImage.y+_borderImage.height-_dateContainer.height-1):(actualHeight - _dateLabel.height);
        _dateContainer.x = _labelContainer.x;
        if(_iconImage){
            _iconImage.x = leftMargin - (_iconImage.width + _iconTailImage.width);
            _iconImage.y = (actualHeight-_iconImage.height)/2;
            if(_iconTailImage){
                _iconTailImage.width = 15;
                _iconTailImage.height = 0.5;
                _iconTailImage.x = _iconImage.x + _iconImage.width;
                _iconTailImage.y = _iconImage.y + (_iconImage.height-_iconTailImage.height)/2;
                if(_iconTailImage.parent && getChildIndex(_iconTailImage) < getChildIndex(_bgQuad)){
                    setChildIndex(_iconTailImage, getChildIndex(_bgQuad)+1);
                }
            }
        }
        if(_borderImage){
            _borderImage.width = actualWidth - (leftMargin + rightMargin);
            _borderImage.height = _bgQuad.height - 2*_gap;
            _borderImage.x = leftMargin; //(_borderImage.width + _gap);
            _borderImage.y = _bgQuad.y + _gap;
        }

        if(_avatar){
            _avatar.x = leftMargin + _avatarXMargin;
            _avatar.y = (actualHeight - _avatar.height)/2;
        }
        if(_labelContainer){
            _labelContainer.validate();
            _labelContainer.x = leftMargin + _avatarSize + _avatarXMargin + _gap;
            _labelContainer.y = (actualHeight-getInfoHeight())/2;
        }
        updateThumbnailPositionAndSize();
        updateLabelPosition();
    }


    private function userDetailsSelectHandler(event:TouchEvent) : void{
        if(owner.isScrolling){
            return;
        }
        var touch1 : Touch = event.getTouch(_nameLabel, TouchPhase.ENDED);
        var touch2 : Touch = event.getTouch(_avatar, TouchPhase.ENDED);
        if (touch1 || touch2) {
            Constants.selectedUser =  feedData.creator;
            eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED, false, {user: Constants.selectedUser});
        }
    }

    private function onCarSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !this.owner.isScrolling){
            if(feedData && feedData.tuning) {
                SoundManager.playSound(SoundManager.PART_MENU);
                eventDispatcher.dispatchEvent(new Event(StackedScreen.TUNING_SELECTED, false, {tuning:TuningsProxy.parseTuning(feedData.tuning), user:feedData.owner}));
            }
        }
    }

    override public function dispose():void {
        super.dispose();
        if (_nameLabel) {
            _nameLabel.removeEventListener(TouchEvent.TOUCH, userDetailsSelectHandler);
        }
        if(_avatar) {
            _avatar.removeFromParent(true);
            _avatar = null;
        }
        if(_postName){
            _postName.removeFromParent(true);
            _postName = null;
        }
        if(imageLoader){
            imageLoader.removeFromParent(true);
            imageLoader = null;
        }
        if(_iconImage){
            _iconImage.removeFromParent(true);
            _iconImage = null;
        }
        if(_iconTailImage){
            _iconTailImage.removeFromParent(true);
            _iconTailImage = null;
        }
        if(_borderImage){
            _borderImage.removeFromParent(true);
            _borderImage = null;
        }
        if(_dateImage){
            _dateImage.removeFromParent(true);
            _dateImage = null;
        }
    }
}
}
