package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.utils.Tracker;

	import starling.events.Event;

	public class ForgottenPasswordScreen extends AuthScreen {
		public static const EMAIL_CHANGED : String = "EmailChanged";
		public static const SEND_SELECTED : String = "SendSelected";
		
		private var emailInputView : ProfilePageLabel;
		private var sendButton : MultilingualButton;
		
		public function ForgottenPasswordScreen() {
			super();
		}

		override public function updateWithLanguage(language : Language) : void {
			super .updateWithLanguage(language);
			inputTitle.text = language.translate("mob_signup", "resetPassword").toUpperCase();
			emailInputView.placeholderText = language.translate("mob_signup", "email").toUpperCase();
		}
		
		override protected function initialize() : void {
			super.initialize();

			inputTitle.paddingTop = 5;
			inputTitle.paddingBottom = 5;
			
			emailInputView = new ProfilePageLabel();
			emailInputView.addEventListener(Event.CHANGE, emailInputViewChangeHandler);
			emailInputView.width = inputWidth;
			emailInputView.height = 30;
			_inputContainer.addChild(emailInputView);
			
			sendButton = ButtonFactory.defaultButton("SEND EMAIL", 0xC90809);
			sendButton.addEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
			sendButton.labelFactory = LabelFactory.profileButtonLabel;
			sendButton.setSize(140, 30);
			_inputContainer.addChild(sendButton);
			
			emailInputView.onEnterPress = function() : void {
				sendButtonTriggeredHandler(null);
			}

			scrollContainer.addChild(_inputContainer);
			super.initializeBackButton();
		}

		override public function ShowDone():void{
			Tracker.trackEvent("Reset Password Screen", "open");
		}
		
		override protected function draw() : void {
			super.draw();

			inputTitle.text = "<b>FORGOT PASSWORD</b><br><font size='11'>Enter your email address to reset password</font>";
			inputTitle.height = NaN;
			inputTitle.validate();

			_inputContainer.x = (actualWidth + LEFT_MARGIN - _inputContainer.width) / 2 - variantRightMargin;
			_inputContainer.y = _inputContainer.y = _socialContainer.y + 90;
		}
		
		public function get email() : String {
			return emailInputView.text;
		}
		
		private function emailInputViewChangeHandler(event : Event) : void {
			dispatchEventWith(EMAIL_CHANGED);
		}

		private function sendButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(SEND_SELECTED);
		}
		
		public function setSendButtonEnabled(value : Boolean) : void {
			sendButton.alpha = value ? 1.0 : 0.5;
			sendButton.touchable = value;
		}
	}
}