package com.despark.tuning3d.view.components.auth {
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.utils.FontSelector;

import feathers.controls.Label;
import feathers.controls.TextInput;
    import feathers.controls.text.StageTextTextEditor;
    import feathers.controls.text.TextFieldTextEditor;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextEditor;
import feathers.core.ITextRenderer;
    import feathers.events.FeathersEventType;

    import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class ProfilePageLabel extends TextInput {
    private var _placeHolder:Label;
    private var _placeholderText:String;
    private var _onEnterPress:Function;
    private var _focusInCallback:Function;
    private var _focusOutCallback:Function;
    public function ProfilePageLabel(placeholder:String = '') {
        super();
        _placeholderText = placeholder;
    }


    override protected function initialize():void {
        super.initialize();

        paddingLeft = paddingRight = 10;
        textEditorFactory = function():ITextEditor {
            /*var editor : TextFieldTextEditor = new TextFieldTextEditor();
            editor.textFormat = new TextFormat("_sans", 11, 0x242424);
            editor.textFormat.letterSpacing = .3;*/
            
            var editor : StageTextTextEditor = new StageTextTextEditor();
            editor.fontSize = 12;
            editor.fontFamily = "_sans";
            editor.color = 0x242424;
            editor.multiline = false;
            return editor;
        };

        var bgTexture:Texture = ButtonFactory.assetManager.getTexture("profile_field_bg");
        var bgImage:Image = new Image(bgTexture);
        bgImage.scale9Grid = new Rectangle(bgTexture.frameWidth * 30/65, 0/*bgTexture.frameHeight / 2*/, bgTexture.frameWidth * 5/65, bgTexture.frameHeight);
        backgroundSkin = bgImage;

        _placeHolder = new Label();
        _placeHolder.textRendererFactory = placeHolderFactory;
        _placeHolder.text = _placeholderText;
        _placeHolder.height = 15;
        addChild(_placeHolder);

        addEventListener(FeathersEventType.ENTER, enterPress);
    }

    override protected function textEditor_focusInHandler(event:Event):void{
        super .textEditor_focusInHandler(event);
        if(_focusInCallback != null){
            _focusInCallback();
        }
        _placeHolder.visible = false;
    }

    override protected function textEditor_focusOutHandler(event:Event):void{
        super .textEditor_focusOutHandler(event);
        if(_focusOutCallback != null){
            _focusOutCallback();
        }
        _placeHolder.visible = (text.length==0);
    }

    public function setText(value:String):void {
        text = value;
        _placeHolder.visible = value.length == 0;
    }

    override protected function focusInHandler(event:Event):void {
        super.focusInHandler(event);
        _placeHolder.visible = false;
    }

    override protected function focusOutHandler(event:Event):void {
        super.focusOutHandler(event);
        _placeHolder.visible = (text.length==0);
    }

    override protected function draw():void {
        super.draw();
        if(_placeHolder){
            _placeHolder.x = 10;
            _placeHolder.y = (height - _placeHolder.height) / 2;
        }
    }

    override public function set isEnabled(value:Boolean):void{
        super .isEnabled = value;
        alpha = (isEnabled?1:0.8);
    }

    override public function dispose():void {
        super.dispose();
        if(_placeHolder){
            _placeHolder.removeFromParent(true);
            _placeHolder = null;
        }

        if(_onEnterPress){
            _onEnterPress = null;
        }
        if(_focusInCallback){
            _focusInCallback = null;
        }
        if(_focusOutCallback){
            _focusOutCallback = null;
        }
    }

    private static function placeHolderFactory() : ITextRenderer {
        var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
        var bold:Boolean = false;
        var italic:Boolean = false;
        var color:uint = 0x58595b;
        var size:int = 11;
        var align:String=TextFormatAlign.LEFT;
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), size, color);
        textRenderer.textFormat.align = align;
        textRenderer.textFormat.bold = bold;
        textRenderer.textFormat.italic =italic;
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.wordWrap = false;
        textRenderer.isHTML = true;
        return textRenderer;
    }

    public function set placeholderText(value:String):void {
        _placeholderText = value;
        if(_placeHolder){
            _placeHolder.text = _placeholderText;
        }
    }

    private function enterPress(event:Event) : void {
        if (onEnterPress)	{
            onEnterPress();
        }
    }

    public function get onEnterPress():Function {
        return _onEnterPress;
    }

    public function set onEnterPress(value:Function):void {
        _onEnterPress = value;
    }

    public function set focusInCallback(value:Function):void {
        _focusInCallback = value;
    }

    public function set focusOutCallback(value:Function):void {
        _focusOutCallback = value;
    }
}
}
