package com.despark.tuning3d.view.components.cars
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarsProxy;
	import com.despark.tuning3d.model.core.AppMode;
	import com.despark.tuning3d.model.core.images.PersistentImageProvider;
	import com.despark.tuning3d.model.packages.Packet;
	import com.despark.tuning3d.view.components.brands.CarsList;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.CarListItemRenderer;
	import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.utils.FontSelector;

	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.ITextRenderer;

	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.setTimeout;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.textures.Texture;

	public class CarItemRenderer extends CarListItemRenderer
	{
		protected var carUnlock:Image;
		
		private var carLock:Image;
		private var quad:Quad;

		private var completeLoad:Boolean=false;
		
		private var _underlined : Boolean = true;
		private var _horizontalOffset : Number;
		private var underline : Quad;
		private var leftBorder : Quad;
		private var rightBorder : Quad;
		private var _verticalOffset : Number = 5;
		
		public  var _underOffset : Number = 0;
		public  var _carOffset : Number = 8;
		private var _overlay : Quad;
		
		public function CarItemRenderer(){
			super();
		}
		
		public function set horizontalOffset(value:Number):void{
			_horizontalOffset = value;
		}
		
		public function get horizontalOffset():Number{
			return _horizontalOffset;
		}
	
		public function set underlined(value:Boolean):void{
			_underlined = value;
		}

		public function get underlined():Boolean{
			return _underlined;
		}
		
		public function get car() : Car{
			return data as Car;       
		}

		protected override function initialize():void
		{
			quad = new Quad(actualWidth, actualHeight,0xECECEC);
			quad.alpha = 0;
			addChild(quad);  
			
			_overlay = new Quad(actualWidth, actualHeight,0xECECEC);
			_overlay.alpha = 0;
			
			underline = new Quad(1, 1, 0x4b4b4b);
			underline.visible = true;
			addChild(underline);

			leftBorder = new Quad(1, 1, 0x4b4b4b);
			leftBorder.visible = true;
			addChild(leftBorder);

			rightBorder = new Quad(1, 1, 0x4b4b4b);
			addChild(rightBorder);

			super.initialize();
			
			var carLockTexture : Texture = AlertView.assetManager.getTexture("challenge_rewards_icon");
			carLock=new Image(carLockTexture);
			carLock.color = 0xFF0000;
			carLock.width = 18;
			carLock.scale = carLock.scaleX;
			
			var carUnlockTexture:Texture = AlertView.assetManager.getTexture("buy_check_icon");	
			carUnlock=new Image(carUnlockTexture);
			carUnlock.color = 0xFF0000;

			addChild(carLock);
			addChild(carUnlock);

			label.textRendererFactory = getTextRenderer;
			tlabel.textRendererFactory = getTextRenderer;
			ylabel.textRendererFactory = getTextRenderer;
		}

		private function getTextRenderer():ITextRenderer {
			var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
			textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 9, 0xFFFFFF);
			textRenderer.textFormat.align = TextFormatAlign.LEFT;
			textRenderer.textFormat.leftMargin = 0;
			textRenderer.textFormat.bold = true;
			textRenderer.embedFonts = true;
			return textRenderer;
		}
		
		protected override function layout():void
		{
			label.visible = false;
			
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
			labelHeight = 13;
			pading = 5;
			tlabel.height = ylabel.height = labelHeight;
			tlabel.x = pading;
			tlabel.y = pading - 2;

			ylabel.x = pading;
			ylabel.y = tlabel.y + tlabel.height;

			tlabel.width = actualWidth - 2 * pading;
			ylabel.width = actualWidth  - 2 * pading;

			if(car && car.displayName == "more cars?"){
				tlabel.width = NaN;
				tlabel.validate();
				tlabel.x = (actualWidth - tlabel.width) / 2;
				tlabel.y = pading;
			}

			if (imageLoader){
				imageLoader.x = _carOffset;
				imageLoader.y = tlabel.y + tlabel.height + _carOffset;
				imageLoader.width  = actualWidth  - 2 * _carOffset;
				imageLoader.height = actualHeight - (tlabel.y + tlabel.height + 1.5* pading);
			}

			var posx : Number = actualWidth - carLock.width - 3;

			carLock.x = carUnlock.x = posx;
			carLock.y = carUnlock.y = 3;

			if( car && car.packet && car.packet.lock){
				carLock.visible = car.packet.type != Packet.TYPE_SUBCOMPONENT ;
			}
			else {
				carLock.visible=false;
			}

			if (car && car.carFeaturesNew && car.carFeaturesFull)
			{
				carLock.x = carUnlock.x = posx;

				if (car && car.packet && car.packet.lock)
				{
					if (car.packet.type != Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
						carUnlock.visible=false;
				}
			}
			else if(car && car.carFeaturesFull)
			{
				carLock.x = carUnlock.x = posx;
				if (car && car.packet && car.packet.lock)
				{
					if (car.packet.type != Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else if(car && car.carFeaturesNew)
			{
				carLock.x = carUnlock.x = posx;

				if(car && car.packet && car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else
			{
				carLock.x = carUnlock.x = posx;
				if(car && car.packet && car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
						carUnlock.visible=false;
				}
			}


			//////////////////////////////////////////////////////////
			quad.x = 0;
			quad.width  = actualWidth;
			quad.height = actualHeight;

			if (_underlined)
			{
				underline.visible = true;
				underline.y = actualHeight - underline.height;
				underline.width = actualWidth - _underOffset;
				underline.x = _underOffset;

			}
			else {
				underline.visible = false;
			}
		  
			
			if(leftBorder){
				leftBorder.height = actualHeight;
			}

			if(rightBorder){
				rightBorder.height = actualHeight;
				rightBorder.x = actualWidth;
			}
		}
		
		private var imageWidth:Number=-1;
		private var imageHeight:Number=-1;
		protected function completeLoader(image:ImageLoaderWithStates=null):void{
			setTimeout(completeLoaderImage, 50);
		}
		
		protected function errorLoader():void{
			completeLoad = false;
			if(car){
				if(car.imageURL && car.imageURL.indexOf("side_2.png") != -1){
					car.imageURL = car.imageURL.replace("side_2.png", "side.png");
					imageLoader.persistentImageProvider = car;
				}else if(car.imageURL != Car.BlankCar.imageURL){
					car.imageURL = Car.BlankCar.imageURL;
					imageLoader.persistentImageProvider = car;
				}
			}else{

			}
			completeLoad = true;
			//invalidate();
		}
		
		protected function completeLoaderImage():void
		{
			if (imageLoader && imageLoader.image)
			{
				completeLoad = true;
				imageLoader.image.color = 0xFF0000;
				imageWidth=imageLoader.image.width;
				imageHeight=imageLoader.image.height;
				
				underline.visible = _underlined;
				invalidate();
			}
			else
			{
				setTimeout(completeLoaderImage, 10);
			}		
		} 
			
		protected override function commitData() : void 
		{
			super.commitData();
			
			if (car)
			{
				if(car.imageURL && car.imageURL.indexOf("side.png") != -1){
					car.imageURL = car.imageURL.replace("side.png", "side_2.png");
				}


				var newPersistentImageProvider:PersistentImageProvider=car;
							
				if(!imageLoader.completeHandler){			
					imageLoader.completeHandler=completeLoader;
				}
				
				if(!imageLoader.errorHandler){			
					imageLoader.errorHandler=errorLoader;
				}
				
				if (imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != newPersistentImageProvider.remoteUrl)
				{
					completeLoad=false;
					imageLoader.persistentImageProvider = car;
				}
				else if (!imageLoader.persistentImageProvider)
				{
					completeLoad=false;
					imageLoader.persistentImageProvider = car;
				}


				var str : String = car.model;
				if(car.body && car.body.length>0)
				{
					str += " " + car.body;
				}

				label.text  = str;
				if(car.displayName == "more cars?"){
					tlabel.text = car.displayName;
				}else{
					if(carList.showBrandName){
						tlabel.text = (car.name?car.name + " ":"") + (car.model?car.model:"");
					}else{
						tlabel.text = (car.model?car.model:"");
					}
				}

				if(car.displayName == "more cars?"){
					ylabel.text = "";
				}else{
					ylabel.text = yearText;
				}
			}
			var itemIndex:int = owner.dataProvider.getItemIndex(car);
			rightBorder.visible = (itemIndex % 3 == 2 || itemIndex == owner.dataProvider.length - 1);
		}

		private function get yearText():String{
			if(car){
				if(car.startYear && car.endYear && car.startYear <= car.endYear){
					return car.startYear.toString() + "-" + car.endYear.toString();
				}
				return car.year.toString();
			}
			return "";
		}


		public function get carList() : CarsList{
			return owner as CarsList;
		}
		
		protected override function   draw() : void
		{      
			super.draw();


			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;

			if(car && car.packet && car.packet.lock && !car.packet.show){
				carLock.visible = car.packet.type != Packet.TYPE_SUBCOMPONENT ;
			}
			else
			{
				if(car && !carsProxy.bonusCarExists(car.id) && car.packet && car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=false;
				}
				else {
					carLock.visible=false;
				}
			}

			if(car && car.packet && car.packet.lock)
			{
				if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=true;
					carUnlock.visible=false;
				}
				else{
					carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else if(car && car.packet && !car.packet.lock)
			{
				if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=false;
					carUnlock.visible=true;
				}
				else{
					carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else
			{
					carLock.visible=false;
					carUnlock.visible=false;
			}


			layout();
		}

		
		protected override function autoSizeIfNeeded() : Boolean
		{
			return true;
		}
	}
}