package com.despark.tuning3d.view.components.cars
{
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.view.BrandsScreenMediator;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.utils.Console;

    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.Loader;
    import flash.events.Event;
    import flash.events.HTTPStatusEvent;
    import flash.events.IEventDispatcher;
    import flash.events.IOErrorEvent;
    import flash.events.MouseEvent;
    import flash.events.ProgressEvent;
    import flash.filesystem.File;
    import flash.geom.Rectangle;
    import flash.net.URLRequest;
    import flash.system.ApplicationDomain;
    import flash.system.LoaderContext;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.textures.Texture;
    import starling.utils.AssetManager;
    import starling.utils.ScaleMode;

    public class FeaturedCarItemRenderer extends ListItemRenderer {
    private var quad:Quad;
    private var carLockIcon:Image;
    private var carUnlockIcon:Image;
    private static var embbededBgTexture : Texture;
    private var loader:Loader;
    private var _clipRectQuad:Quad;

    public static var assetManager : AssetManager;
    private var _carLastId : String = "";
    private static var _cachedTextures:Object;
    public var isLoaded: Boolean = false;

    public function FeaturedCarItemRenderer()
    {
        super();

        if(!embbededBgTexture)
        {
            embbededBgTexture = Texture.fromColor(600, 600/1.7);
        }
        _cachedTextures ||= {};
        _hitArea=new Rectangle(0,0,0,0);
        this._isQuickHitAreaEnabled=true;
    }

    public function get car() : Car{
        return data as Car;
    }

    override protected  function autoSizeIfNeeded():Boolean{
        return true;
    }

    public function get isCurrentImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex);
    }

    public function get isPrevImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex + 1);
    }

    public function get isNextImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex - 1);
    }

    public function get isAllowedImage():Boolean{
        return isCurrentImage || isPrevImage || isNextImage;
    }

    public function loading(): void {
        if (!imageLoader) {
            initialize();
        }
        if (!imageLoader.completeHandler) {
            imageLoader.completeHandler = completeLoadImage;
        }
        if(car.featuredImageUrl){
            car.featuredImageUrl = car.featuredImageUrl.replace("default.png", "car_chooser_app.jpg");
        }
        var url:String = car.featuredImageUrl;
        var newCar:Car;

        var isTruckImage:Boolean = AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR;
        var isMuscleImage:Boolean = AppMode.self.mode == AppMode.MODE_AMERICAN_MUSCLE;

        if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url || ((isTruckImage || isMuscleImage) && !imageLoader.currentTexture && isAllowedImage)) {
            newCar = new Car();
            newCar.id = car.id;
            newCar.imageURL = url;

            imageLoader.completeHandler = completeLoadImage;
            imageLoader.errorHandler2 = errorLoadImage;
            imageLoader.persistentImageProvider = newCar;
        }

        if (car && car.packet && car.packet.lock) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = true;
                carUnlockIcon.visible = false;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else if (car && car.packet) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = false;
                carUnlockIcon.visible = true;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else {
            carLockIcon.visible = false;
            carUnlockIcon.visible = false;
        }
    }

    protected override function initialize():void {
        _clipRectQuad = new Quad(1,1);

        quad = new Quad(this.actualWidth, this.actualHeight, 0x008800);
        quad.alpha = 0;
        addChild(quad);

        label = new MultilingualLabel();
        addChild(label);

        imageLoader = new ImageLoaderWithStates();
        imageLoader.overrideDispose = true;
        imageLoader.scaleMode = ScaleMode.NONE;
        addChild(imageLoader);

        var carLockTexture:Texture=ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon");
        carLockIcon=new Image(carLockTexture);
        carLockIcon.visible=false;
        //addChild(carLockIcon);

        var carUnlockTexture:Texture=ButtonFactory.assetManager.getTexture("car_unlock_big_icon");
        carUnlockIcon=new Image(carUnlockTexture);
        carUnlockIcon.color = 0x1fb207;
        carUnlockIcon.visible=false;
        //addChild(carUnlockIcon);
    }

    public function specialDispose():void{
        if(car){
            _carLastId = car.id;
        }else{
            _carLastId = "";
        }
        disposeLoader();
        if(imageLoader ) {
            imageLoader.sepcialDispose();
        }
        isLoaded = false;
    }

    override public function dispose():void{
        super.dispose();
    }

    protected override function layout():void {
        if(_clipRectQuad){
            _clipRectQuad.width = actualWidth;
            _clipRectQuad.height = actualHeight;
            this.mask = _clipRectQuad;
        }

        label.visible=false;
        resampleCar(imageLoader);

        quad.width=this.actualWidth;
        quad.height=this.actualHeight;
        layoutLockIcons();

        carLockIcon.visible = false;
        carUnlockIcon.visible = false;


        _hitArea.width=actualWidth;
        _hitArea.height=actualHeight;
    }

    private function layoutLockIcons():void{
        carLockIcon.x = actualWidth / 2 + 70;
        carUnlockIcon.x = actualWidth / 2 + 70;
        if(imageLoader && imageLoader.currentTexture){
            carLockIcon.y = imageLoader.y + (imageLoader.height - carLockIcon.height) / 2 - 20;
            carUnlockIcon.y = imageLoader.y + (imageLoader.height - carUnlockIcon.height) / 2 - 20;
        }else{
            carLockIcon.y = (actualHeight - carLockIcon.height) / 2 - 20;
            carUnlockIcon.y = (actualHeight - carUnlockIcon.height) / 2 - 20;
        }
    }

    private function resampleCar(target:ImageLoaderWithStates): void {
        if(!target){
            return;
        }
        var tmpWidth:Number;
        var tmpHeight:Number;
        var y_offset:Number = 47;
        if(target.currentTexture){
            if(AppMode.self.mode == AppMode.MODE_TUNING || AppMode.self.mode == AppMode.MODE_AMERICAN_MUSCLE){
                tmpWidth = target.currentTexture.frameWidth * 1.5 / 3.75;
                tmpHeight = target.currentTexture.frameHeight * 1.5 / 3.75;
            }else{
                tmpWidth = target.currentTexture.frameWidth * 1.5 / 4;
                tmpHeight = target.currentTexture.frameHeight * 1.5 / 4;
            }
        }else{
            isLoaded = false;
        }

        target.width = tmpWidth;
        target.height = tmpHeight;

        target.x = (actualWidth - target.width) / 2;
        target.y = (actualHeight - target.height) / 2;
        target.validate();
        layoutLockIcons();
    }

    protected override function commitData() : void {
        super.commitData();
        Starling.juggler.delayCall(completeLoadImage, 5, null); // call complete to hide menu, in case internet is to slow, and
        if(car.id == _carLastId) {
            completeLoadImage(null);
            return;
        }
        _carLastId = car.id;
        specialDispose();
        if (car) {
            if(car.featuredImageUrl){
                car.featuredImageUrl = car.featuredImageUrl.replace("default.png", "car_chooser_app.jpg");
            }
            var url:String = car.featuredImageUrl;
            var newCar:Car;
            if(AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR || AppMode.self.mode == AppMode.MODE_AMERICAN_MUSCLE) {
                completeLoadImage(null);
                if(isAllowedImage && imageLoader.persistentImageProvider is Car && Car(imageLoader.persistentImageProvider).id != car.id) {
                    newCar = new Car();
                    newCar.id = car.id;
                    newCar.imageURL = url;

                    imageLoader.sepcialDispose();

                    imageLoader.completeHandler = completeLoadImage;
                    imageLoader.errorHandler2 = errorLoadImage;
                    imageLoader.persistentImageProvider = newCar;
                }
            } else {

                if (!imageLoader.completeHandler) {
                    imageLoader.completeHandler = completeLoadImage;
                }


                if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url) {
                    newCar = new Car();
                    newCar.id = car.id;
                    newCar.imageURL = url;

                    imageLoader.completeHandler = completeLoadImage;
                    imageLoader.errorHandler2 = errorLoadImage;
                    imageLoader.persistentImageProvider = newCar;
                }

                if (car && car.packet && car.packet.lock) {
                    if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                        carLockIcon.visible = true;
                        carUnlockIcon.visible = false;
                    } else {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = false;
                    }
                } else if (car && car.packet) {
                    if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = true;
                    } else {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = false;
                    }
                } else {
                    carLockIcon.visible = false;
                    carUnlockIcon.visible = false;
                }

                //loadBg();
            }
        }
    }

    private function disposeLoader() : void
    {
        if(loader)
        {
            if(loader.contentLoaderInfo && loader.contentLoaderInfo.hasEventListener(Event.COMPLETE))
            {
                deleteListeners(loader.contentLoaderInfo);
            }
            try
            {
                loader.unloadAndStop(true);

            }
            catch (e:Error) {trace(e.getStackTrace())}

            loader=null;
        }
    }


    private function configureListeners(dispatcher:IEventDispatcher):void {
        dispatcher.addEventListener(Event.COMPLETE, completeHandler);
        dispatcher.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.addEventListener(Event.INIT, initHandler);
        dispatcher.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.addEventListener(Event.OPEN, openHandler);
        dispatcher.addEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.addEventListener(Event.UNLOAD, unLoadHandler);
    }

    private function deleteListeners(dispatcher:IEventDispatcher):void {
        dispatcher.removeEventListener(Event.COMPLETE, completeHandler);
        dispatcher.removeEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.removeEventListener(Event.INIT, initHandler);
        dispatcher.removeEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.removeEventListener(Event.OPEN, openHandler);
        dispatcher.removeEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.removeEventListener(Event.UNLOAD, unLoadHandler);
    }

    private function completeHandler(event:Event):void {
        isLoaded = true;
        var bitmap:Bitmap;
        try{
            bitmap = loader.content as Bitmap;
        }catch (e:Error) {

        }
        if(!bitmap) {
            return;
        }

        var bitmapData:BitmapData = bitmap.bitmapData;
        try{
            bitmapData.width;
            if(!bitmapData) {
                return;
            }
        }catch (e:Error) {
            return;
        }
        _cachedTextures ||= {};
        if(_cachedTextures[loader.contentLoaderInfo.url]){
            Texture(_cachedTextures[loader.contentLoaderInfo.url]).dispose();
            delete _cachedTextures[loader.contentLoaderInfo.url];
        }
        var texture:Texture = Texture.fromBitmapData(bitmapData,false,false,1);
        _cachedTextures[loader.contentLoaderInfo.url] = texture;
        resetBackground(texture);

        bitmapData.dispose();
    }

    private function destroyCachedTexture():void{
        if(!_cachedTextures)
            return;
        for(var key:String in _cachedTextures){
            Texture(_cachedTextures[key]).dispose();
        }
        _cachedTextures = {};
    }

    private function httpStatusHandler(event:HTTPStatusEvent):void {
        Tracer.print(event.toString());
    }

    private function initHandler(event:Event):void {
    }

    private function ioErrorHandler(event:IOErrorEvent):void {
        Tracer.print(event.toString());
        resetBackground(embbededBgTexture);
    }

    private function resetBackground(texture:Texture):void{
        if(loader) {
            deleteListeners(loader.contentLoaderInfo);
        }
    }

    private function openHandler(event:Event):void {
    }

    private function progressHandler(event:ProgressEvent):void {
    }

    private function unLoadHandler(event:Event):void {
    }

    private function errorLoadImage(image:ImageLoaderWithStates):void{
        completeLoadImage(null)
    }

    private function completeLoadImage(image:ImageLoaderWithStates):void{
        BrandsScreenMediator.FIRST_LOAD_COMPLETE = true;
        BrandsScreenMediator.SECOND_LOAD_COMPLETE = true;
        isLoaded = true;
        resampleCar(imageLoader);
        if(this.owner)
            this.owner.dispatchEventWith("featuredCompleteLoad");
    }

    protected override function draw():void {
        super.draw();
        if (car && car.featuredImageUrl) {
            var url:String=car.featuredImageUrl;
            var newCar:Car;
            var deleteFile:Boolean=false;

            if (imageLoader.persistentImageProvider) {
                var localFileURL:String = car.featuredImageUrl.replace("http://cdn1.3dtuning.com/", "");

                var localFile: File = File.cacheDirectory.resolvePath(localFileURL);

                if (localFile.exists) {
                    var creationDate:Date=new Date();
                    creationDate.setTime(Date.parse(localFile.creationDate));
                    var nowDate:Date=new Date();
                    var interval:int=(nowDate.getTime()/1000)-(creationDate.getTime()/1000);

                    if(interval>60*60*3){
                        localFile.deleteFile();
                        deleteFile=true;
                    }
                }
            }


            var b:Boolean = (AppMode.self.mode == AppMode.MODE_TUNING || (isAllowedImage && AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR) || (isAllowedImage && AppMode.self.mode == AppMode.MODE_AMERICAN_MUSCLE));
            url = url.replace("default.png", "car_chooser_app.jpg");
            if ((imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != url && b) || deleteFile) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;
                imageLoader.persistentImageProvider = newCar;
            } else if (!imageLoader.persistentImageProvider && b) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;
                imageLoader.persistentImageProvider = newCar;
            }

            if (car && car.packet && car.packet.lock) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=true;
                    carUnlockIcon.visible=false;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else if(car && car.packet) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=true;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else{
                carLockIcon.visible=false;
                carUnlockIcon.visible=false;
            }
        }
        label.visible=false;
        if(imageLoader){
            resampleCar(imageLoader);
        }
    }
}
}