package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandScreenCarChooser;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeLeaderboardItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeLeaderboardUserRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeScreenItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeUserAvatar;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedLabel;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.utils.FontSelector;
    import com.utils.ObjectUtils;
    import com.utils.Tracker;

    import feathers.controls.ButtonState;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.ScrollContainer;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.FeathersControl;
    import feathers.core.ITextRenderer;
    import feathers.core.ToggleGroup;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalLayout;

    import flash.geom.Rectangle;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;

    public class ChallengesLeaderboardScreen extends ChallengesScreen {
        private var _challengesList:ExtendedList;
        private var _usersList:List;
        private var _topUsersLabel:ExtendedLabel;
        private var _userLabelGroup:LayoutGroup;
        private var _userAvatar:ChallengeUserAvatar;
        private var _userRankLabel:Label;
        private var _userRankValue:String = "";
        private var _leaderboardData:Object;
        private var _leaderboardDataCollections:Object;
        //
        private var _categoryGroup:LayoutGroup;
        private var _categoryToggleGroup:ToggleGroup;
        //
        public static const CHALLENGE_SELECTED:String = "ChallengesLeaderboardScreen::challengeSelected";
        public static const USER_SELECTED:String = "ChallengesLeaderboardScreen::userSelected";
        private static const ITEM_WIDTH:Number = 56.5;
        private static var LAST_SELECTED_SECTION:String = "all";
        public function ChallengesLeaderboardScreen(name:String = 'ChallengesListScreen') {
            super(name);
        }

        override protected function initialize():void {
            super.initialize();
            _toggleGroup.selectedItem = _leaderboardBtn;


            _challengesList = new ExtendedList();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 1;

            _challengesList.height = contentHeight / 2;
            _challengesList.paddingTop = 0;
            _challengesList.layout = listLayout;
            _challengesList.verticalScrollPolicy = ScrollPolicy.OFF;
            _challengesList.horizontalScrollPolicy = ScrollPolicy.ON;
            _challengesList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, challengeSelectedHandler);
            addChild(_challengesList);
            _challengesList.itemRendererFactory = function():IListItemRenderer{
                var renderer:ChallengeLeaderboardItemRenderer = new ChallengeLeaderboardItemRenderer();
                renderer.isWideScreen = Constants.isWideScreen;
                renderer.isUltraWideScreen = Constants.isUltraWideScreen;
                renderer.owner = _challengesList;
                renderer.setSize((contentWidth - 3) / 4, _challengesList.height);
                return renderer;
            }


            createUsersList();

            _topUsersLabel = new ExtendedLabel();
            _topUsersLabel.paddingLeft = _usersList.paddingLeft;
            _topUsersLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitle2Label;
            _topUsersLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
            _topUsersLabel.defaultIcon.scale = 0.7 * ButtonFactory.assetManager.scaleFactor / 2;
            _topUsersLabel.labelOffsetX = 3;
            _topUsersLabel.labelOffsetY = 2;
            _topUsersLabel.text = "TOP TUNERS";
            addChild(_topUsersLabel);
            var colorFilter:ColorMatrixFilter = new ColorMatrixFilter();
            colorFilter.tint(0x535353, 1);
            _topUsersLabel.defaultIcon.filter = colorFilter;
            _topUsersLabel.validate();

            if(UsersProxy.loggedInUser() && user){
                createUserLabel();
            }

            //
            _categoryGroup = new LayoutGroup();
            var groupLayout:HorizontalLayout = new HorizontalLayout();
            groupLayout.gap = 5;
            _categoryGroup.layout = groupLayout;
            _categoryToggleGroup = new ToggleGroup();

            createToggleButton("ALL", "all");
            for(var i:int = 0; i < BrandScreenCarChooser.BODY_TYPE_LABELS.length; i++){
                _categoryGroup.addChild(createSeparator());
                createToggleButton(BrandScreenCarChooser.BODY_TYPE_LABELS[i].name, BrandScreenCarChooser.BODY_TYPE_LABELS[i].value);
            }
            addChild(_categoryGroup);
            _categoryToggleGroup.isSelectionRequired = true;
            _categoryToggleGroup.addEventListener(Event.CHANGE, categoryChangeHandler);
            //
        }

        private static function createSeparator():Quad{
            var quad:Quad = new Quad(1,1, 0xcccccc);
            quad.width = 0.5;
            quad.height = 16;
            return quad;
        }

        private function createToggleButton(label:String, value:String):void{
            var toggleButton:ToggleButton = new ToggleButton();
            toggleButton.labelFactory = toggleLabelRenderer;
            toggleButton.label = label;
            toggleButton.name = value;
            toggleButton.paddingLeft = toggleButton.paddingRight = 1;
            toggleButton.paddingTop = 1;
            toggleButton.paddingBottom = 2.5;
            toggleButton.height = 16;
            toggleButton.defaultSkin = new Quad(1,1,0xFFFFFF);
            var selectedSkin:Image = new Image(ButtonFactory.assetManager.getTexture("button_underline"));
            selectedSkin.color = 0xfba500;
            toggleButton.defaultSelectedSkin = selectedSkin;

            _categoryToggleGroup.addItem(toggleButton);
            _categoryGroup.addChild(toggleButton);

            if(value == LAST_SELECTED_SECTION){
                toggleButton.isSelected = true;
            }
        }

        private static function toggleLabelRenderer():ITextRenderer{
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            var textFormat:TextFormat = new TextFormat(FontSelector.getMontserratRegular(), 10, 0x424242);
            var selectedTextFormat:TextFormat = new TextFormat(FontSelector.getMontserratBold(), 10, 0x424242);
            textRenderer.textFormat = textFormat;
            textRenderer.embedFonts = true;
            textRenderer.wordWrap = false;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.isHTML = false;
            textRenderer.setTextFormatForState(ButtonState.HOVER_AND_SELECTED, selectedTextFormat);
            textRenderer.setTextFormatForState(ButtonState.DOWN_AND_SELECTED, selectedTextFormat);
            textRenderer.setTextFormatForState(ButtonState.UP_AND_SELECTED, selectedTextFormat);
            textRenderer.setTextFormatForState(ButtonState.FOCUSED_AND_SELECTED, selectedTextFormat);
            textRenderer.setTextFormatForState(ButtonState.DISABLED_AND_SELECTED, selectedTextFormat);
            return textRenderer;
        }

        private function categoryChangeHandler(event:Event):void {
            setLeaderboardCategoryData(_categoryToggleGroup.selectedItem.name);
        }

        /*override protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {
            super .nativeStage_orientationChangeHandler(event);
            updateOrientation();
        }*/

        override public function updateUserData(value:Object):void{
            super .updateUserData(value);
            if(UsersProxy.loggedInUser() && user && !_userLabelGroup){
                createUserLabel();
                invalidate(FeathersControl.INVALIDATION_FLAG_SIZE);
            }
        }

        private function createUserLabel():void{
            _userLabelGroup = new LayoutGroup();
            _userLabelGroup.width = 65;
            _userAvatar ||= new ChallengeUserAvatar(user, 51);
            _userAvatar.x = (_userLabelGroup.width - _userAvatar.width) / 2;
            _userLabelGroup.addChild(_userAvatar);
            var groupLayout:VerticalLayout = new VerticalLayout();
            groupLayout.horizontalAlign = HorizontalAlign.CENTER;
            groupLayout.paddingTop = 9;
            _userLabelGroup.layout = groupLayout;
            addChild(_userLabelGroup);


            var frameTxt:Texture = ButtonFactory.assetManager.getTexture("border_line");
            var frameImg:LayoutImage = new LayoutImage(frameTxt);
            frameImg.scale9Grid = new Rectangle(frameTxt.width/2 - 0.5, frameTxt.height / 2 - 0.5, 1, 1);
            frameImg.color = 0xfba500;
            frameImg.includeInLayout = false;
            frameImg.width = _userLabelGroup.width;
            frameImg.height = 83;
            _userLabelGroup.addChild(frameImg);

            var _yourRankLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 9, 0x535353, TextFormatAlign.CENTER, false, false);
            _yourRankLabel.text = "YOUR RANK";
            _yourRankLabel.paddingTop = 0;
            _userLabelGroup.addChild(_yourRankLabel);

            _userRankLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratBold(), 7.5, 0x535353, TextFormatAlign.CENTER, false, false);
            _userRankLabel.paddingTop = -1;
            _userLabelGroup.addChild(_userRankLabel);
            _userRankLabel.text = _userRankValue;
        }

        private function updateOrientation():void{
            if(!stage || !_challengesList){
                return;
            }
            _challengesList.orientation = stage.starling.nativeStage.orientation;
        }

        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'LEADERBOARD';
        }

        private function challengeSelectedHandler(event:Event):void {
            dispatchEvent(new Event(CHALLENGE_SELECTED, false, event.data));
        }

        private function userSelectedHandler(event:Event):void {
            if(_categoryToggleGroup.selectedItem.name == "all"){
                eventData = event.data;
            }else{
                var eventData:Object = ObjectUtils.merge(event.data, {bodyType: _categoryToggleGroup.selectedItem.name});
            }
            dispatchEvent(new Event(USER_SELECTED, false, eventData));
        }

        override protected function draw():void {
            super.draw();
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid) {
                _challengesList.width = actualWidth;
                _challengesList.x = (actualWidth - _challengesList.width) / 2;
                _challengesList.y = marginTop;

                var avatarWidth:Number = _userLabelGroup?_userLabelGroup.width:0;
                var userListWidth:Number;
                var num:int;
                if(Device.isIphoneXVariant()){
                    _topUsersLabel.x = _challengesList.x + _sideMargin;
                    userListWidth = _challengesList.width - 2 * _sideMargin - avatarWidth + 10 - (_usersList.paddingLeft + _usersList.paddingRight);
                    num = int(userListWidth/(ITEM_WIDTH + 10));
                    _usersList.width = num * ITEM_WIDTH + (num - 1) * 10 + (_usersList.paddingLeft + _usersList.paddingRight);
                }else{
                    userListWidth = _challengesList.width - avatarWidth + 10 - (_usersList.paddingLeft + _usersList.paddingRight);
                    _topUsersLabel.x = _challengesList.x;
                    num = int(userListWidth/(ITEM_WIDTH + 10));
                    _usersList.width = num * ITEM_WIDTH + (num - 1) * 10 + (_usersList.paddingLeft + _usersList.paddingRight);
                }

                _topUsersLabel.y = _challengesList.y + _challengesList.height + 8;
                _usersList.x = (actualWidth - _usersList.width - avatarWidth) / 2;
                _usersList.y = _topUsersLabel.y + _topUsersLabel.height + 7;

                if(_userLabelGroup){
                    _userLabelGroup.x = _usersList.x + _usersList.width - _usersList.paddingRight;
                    _userLabelGroup.y = _usersList.y;
                }

                _categoryGroup.x = _topUsersLabel.x + _topUsersLabel.width + 10;
                _categoryGroup.y = _topUsersLabel.y;
            }
        }

        public function loadLeaderboard(object:Object):void{
            if(object){
                _leaderboardData = ObjectUtils.clone(object);
                if(_challengesList){
                    _challengesList.dataProvider = new ListCollection(object.lastChallengesTop);
                }
                var ratings:Array = object.leaderboard.ratings.concat();
                if(object.leaderboard.activeUserPosition){
                    _userRankValue = object.leaderboard.activeUserPosition.toString();
                    if(_userRankLabel){
                        _userRankLabel.text = _userRankValue;
                        _userRankLabel.validate();
                    }
                }
                _leaderboardDataCollections = {};
                _leaderboardDataCollections["all"] = new ListCollection(ratings);
                setLeaderboardCategoryData(LAST_SELECTED_SECTION);

                if(onComplete != null){
                    onComplete();
                    onComplete = null;
                }
            }
        }
        
        private function setLeaderboardCategoryData(bodyTypeValue:String):void{
            var ratings:Array;
            var i:int;
            var collection:ListCollection;
            var activeUserPosition:String = "";
            var activeUserRating:String;

            if(_leaderboardDataCollections[bodyTypeValue]){
                collection = _leaderboardDataCollections[bodyTypeValue];
            }else if(_leaderboardData.leaderboardByType[bodyTypeValue]){
                ratings = _leaderboardData.leaderboardByType[bodyTypeValue]["ratings"].concat();
                _leaderboardDataCollections[bodyTypeValue] = new ListCollection(ratings);
                collection = _leaderboardDataCollections[bodyTypeValue];

            }
            if(bodyTypeValue == "all"){
                activeUserPosition = _leaderboardData.leaderboard.activeUserPosition;
                activeUserRating = _leaderboardData.leaderboard.activeUserRating;
                for(i = 0; i < collection.length; i++){
                    if(collection.getItemAt(i).totalRating){
                        collection.getItemAt(i).challengeNewSystemRatings = collection.getItemAt(i).totalRating;
                    }
                }
            }else{
                activeUserPosition = _leaderboardData.leaderboardByType[bodyTypeValue].activeUserPosition;
                activeUserRating = _leaderboardData.leaderboardByType[bodyTypeValue].activeUserRating;
                for(i = 0; i < collection.length; i++){
                    collection.getItemAt(i).totalRating ||= collection.getItemAt(i).challengeNewSystemRatings;
                    collection.getItemAt(i).challengeNewSystemRatings = collection.getItemAt(i).challengeNewRatings[bodyTypeValue].rating;
                }
            }
            if(_usersList && collection){
                try{
                    _usersList.removeEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, userSelectedHandler);
                    _usersList.dataProvider = collection;
                    _usersList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, userSelectedHandler);
                }catch (e:Error) {
                    _usersList.removeFromParent(true);
                    createUsersList();
                    _usersList.dataProvider = collection;
                }

            }
            if(_userRankLabel){
                _userRankLabel.text = activeUserPosition;
            }
            if(_userAvatar){
                _userAvatar.setRatingLabel(activeUserRating);
            }
            LAST_SELECTED_SECTION = bodyTypeValue;
        }

        private function createUsersList():void{
            _usersList = new List();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 10;

            _usersList.height = 90;
            _usersList.paddingLeft = _usersList.paddingRight = 15;
            _usersList.paddingTop = 10;
            _usersList.layout = listLayout;
            _usersList.verticalScrollPolicy = ScrollPolicy.OFF;
            _usersList.horizontalScrollPolicy = ScrollPolicy.ON;
            _usersList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, userSelectedHandler);
            _usersList.snapToPages = false;
            addChild(_usersList);
            _usersList.itemRendererFactory = function():IListItemRenderer{
                var renderer:ChallengeLeaderboardUserRenderer = new ChallengeLeaderboardUserRenderer();
                renderer.owner = _usersList;
                renderer.setSize(ITEM_WIDTH, _usersList.height);
                return renderer;
            }
        }

        override public function ShowDone():void {
            if(onComplete == null){
                BannerStackedScreen.showAdBanner();
            }
            Tracker.trackEvent("Challenges Screen", "open");
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        override public function dispose():void {
            super.dispose();
            if (_challengesList) {
                _challengesList.removeFromParent(true);
                _challengesList = null;
            }
            if (_usersList) {
                _usersList.removeFromParent(true);
                _usersList = null;
            }
            if (_topUsersLabel) {
                _topUsersLabel.removeFromParent(true);
                _topUsersLabel = null;
            }
        }
    }
}
