package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeListItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeScreenItemRenderer;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.utils.Console;
    import com.utils.DateParser;
    import com.utils.Tracker;

    import feathers.controls.Button;
    import feathers.controls.List;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalLayout;

    import flash.display.StageOrientation;

    import flash.events.StageOrientationEvent;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.events.Event;

    public class ChallengesListScreen extends ChallengesScreen {
        private var _challengesList:ExtendedList;
        private var _leftButton:Button;
        private var _rightButton:Button;
        private var _limit:int = 9;
        private var _skip:int;
        private var _finishSkip:int;
        private var _isLoading:Boolean = false;
        private var _scrollToNextPageOnLoad:Boolean = false;
        private var _challengeListIsScrolling:Boolean = false;
        //
        public static const ENTER_CHALLENGE_SELECTED:String = "enterChallengeSelected";
        public static const REQUEST_CHALLENGE_LIST:String = "requestChallengeList";
        public function ChallengesListScreen(name:String = 'ChallengesListScreen') {
            super(name);
        }

        override protected function initialize():void {
            super.initialize();
            _toggleGroup.selectedItem = _homeBtn;

            _challengesList = new ExtendedList();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 0;
            _challengesList.layout = listLayout;
            _challengesList.snapToPages = true;
            _challengesList.verticalScrollPolicy = ScrollPolicy.OFF;
            _challengesList.horizontalScrollPolicy = ScrollPolicy.ON;
            _challengesList.addEventListener(ChallengeScreenItemRenderer.ITEM_SELECTED, challengeSelectedHandler);
            _challengesList.addEventListener(Event.SCROLL, challengesListScrollHandler);

            addChild(_challengesList);
            _challengesList.itemRendererFactory = function():IListItemRenderer{
                var renderer:ChallengeListItemRenderer = new ChallengeListItemRenderer();
                renderer.titlePadding = _sideMargin;
                renderer.owner = _challengesList;
                renderer.setSize(_challengesList.width / 3, _challengesList.height);
                return renderer;
            }

            _leftButton = ButtonFactory.simpleButtonByTextureName("slide_lt_arr", true, 0xFF0000, false);
            addChild(_leftButton);

            _rightButton = ButtonFactory.simpleButtonByTextureName("slide_lt_arr", true, 0xFF0000, false, true);
            addChild(_rightButton);

            _leftButton.addEventListener(Event.TRIGGERED, leftButtonTriggeredHandler);
            _rightButton.addEventListener(Event.TRIGGERED, rightButtonTriggeredHandler);

            if(Device.isIphoneXVariant()){
                updateOrientation();
            }
        }

        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'CHALLENGES';
        }

        override public function updateUserData(result:Object):void{
            super .updateUserData(result);
            Starling.juggler.delayCall(function():void{
                requestChallengeListData();
            }, 0.1);
        }

        public function clearChallengeData():void{
            if(_challengesList && _challengesList.dataProvider){
                _challengesList.dataProvider.removeAll();
            }
            _skip = 0;
            _finishSkip = 0;
        }

        private function requestChallengeListData():void{
            if(!isLoading){
                dispatchEventWith(ChallengesListScreen.REQUEST_CHALLENGE_LIST);
            }
        }

        private function leftButtonTriggeredHandler(event:Event):void {
            if(!_challengeListIsScrolling){
                _challengeListIsScrolling = true;
                SoundManager.playSound(SoundManager.SELECT_PART);
            }

            if(_challengesList.horizontalPageIndex){
                _challengesList.scrollToPageIndex(Math.max(0, _challengesList.horizontalPageIndex - 1), 0, 0.5);
            }
        }

        private function rightButtonTriggeredHandler(event:Event):void {
            if(!_challengeListIsScrolling){
                _challengeListIsScrolling = true;
                SoundManager.playSound(SoundManager.SELECT_PART);
            }

            if(_challengesList.horizontalPageIndex < _challengesList.horizontalPageCount - 1){
                _challengesList.scrollToPageIndex(Math.min(_challengesList.horizontalPageCount - 1, _challengesList.horizontalPageIndex + 1), 0, 0.5);
            }else if(isLoading){
                attemptNextPageLoading();
            }
        }

        private function attemptNextPageLoading():void{
            if(!_scrollToNextPageOnLoad){
                _scrollToNextPageOnLoad = true;
                showLoadBar();
            }
        }

        private function challengeSelectedHandler(event:Event):void {
            dispatchEvent(new Event(ENTER_CHALLENGE_SELECTED, false, event.data));
            /*var itemRenderer:ChallengeListItemRenderer =  _challengesList.itemToItemRenderer(event.data) as ChallengeListItemRenderer;
            if(itemRenderer){
                itemRenderer.openInfo(event.data);
            }*/
        }

        private function challengesListScrollHandler(event:Event):void {
            if(_challengesList.isScrolling && !_challengeListIsScrolling){
                _challengeListIsScrolling = true;
                SoundManager.playSound(SoundManager.SELECT_PART);
            }
            if(!_challengesList.isScrolling){
                _challengeListIsScrolling = false;
            }

            if(Device.isIphoneXVariant() && _challengesList.dataProvider){
                _challengesList.orientationUpdate = true;
                _challengesList.dataProvider.updateAll();
                if(!_challengesList.isScrolling){
                    _challengesList.orientationUpdate = false;
                }
            }
        }

        private function challengeListScrollHandler(event:Event):void {
            if(!isLoading && _challengesList.horizontalPageIndex == _challengesList.horizontalPageCount - 1){
                requestChallengeListData();
            }
            if(!_scrollToNextPageOnLoad && isLoading && _challengesList.horizontalScrollPosition > _challengesList.maxHorizontalScrollPosition){
                attemptNextPageLoading();
            }
        }

        override protected function draw():void {
            super.draw();
            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            if (sizeInvalid) {
                _challengesList.width = contentWidth + 0.5;
                _challengesList.height = contentHeight;
                _challengesList.x = -0.25;
                _challengesList.y = marginTop;
                if(Device.isIphoneXVariant()){
                    updateOrientation();
                }else{
                    _leftButton.x = 0;
                    _leftButton.y = marginTop + (contentHeight - _rightButton.height) / 2;

                    _rightButton.x = contentWidth - _rightButton.width;
                    _rightButton.y = marginTop + (contentHeight - _rightButton.height) / 2;
                }
            }
        }

        override public function ShowDone():void {
            if(onComplete == null){
                BannerStackedScreen.showAdBanner();
            }
            Tracker.trackEvent("Challenges Screen", "open");
            while(stage && stage.numChildren && stage.getChildAt(stage.numChildren - 1) is Image){
                stage.getChildAt(stage.numChildren - 1).removeFromParent(true);
            }
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        override public function dispose():void {
            super.dispose();
            if (_challengesList) {
                _challengesList.dispose();
                _challengesList = null;
            }
        }

        override protected function nativeStage_orientationChangeHandler(event:StageOrientationEvent):void {
            super .nativeStage_orientationChangeHandler(event);
            updateOrientation();
        }

        private function updateOrientation():void{
            if(!stage || !_challengesList){
                return;
            }
            _challengesList.orientation = stage.starling.nativeStage.orientation;

            var marginSize:Number = 24;
            if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_RIGHT){
                _leftButton.x = marginSize;
                _rightButton.x = contentWidth - _rightButton.width;
            }else if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_LEFT){
                _leftButton.x = 0;
                _rightButton.x = actualWidth - (_rightButton.width + marginSize);
            }else{
                _leftButton.x = 0;
                _rightButton.x = contentWidth - _rightButton.width;
            }

            _leftButton.y = marginTop + (contentHeight - _rightButton.height) / 2;
            _rightButton.y = marginTop + (contentHeight - _rightButton.height) / 2;
        }

        public function loadChallenges(obj:Object):void {
            var result:Array = obj as Array;
            var i:int;
            for(i = result.length - 1; i >= 0; i--){
                result[i].carName = getCarName(result[i].car);
                var challengeData:ChallengeData = new ChallengeData(result[i]);
                if(challengeData.status == ChallengeData.IN_CHALLENGE && user && user.submittedChallenges.indexOf(challengeData.id) != -1){
                    var finishDate:Date = new Date();
                    finishDate.time -= 1;
                    result[i].finishDate = result[i].votingStartDate = DateParser.toW3CDTF(finishDate);
                    challengeData.status = ChallengeData.IN_VOTING;
                }
                if(challengeData.status == ChallengeData.FINISHED){
                    _finishSkip++;
                }
            }

            if(_challengesList){
                if(!_challengesList.dataProvider){
                    _challengesList.dataProvider = new ListCollection(result);
                }else{
                    _challengesList.dataProvider.addAll(new ListCollection(result));
                }
                _skip = _challengesList.dataProvider.length;
                if(!_challengesList.hasEventListener(Event.SCROLL, challengeListScrollHandler)){
                    _challengesList.addEventListener(Event.SCROLL, challengeListScrollHandler);
                }
                if(_scrollToNextPageOnLoad){
                    _scrollToNextPageOnLoad = false;
                    _challengesList.validate();
                    rightButtonTriggeredHandler(null);
                }
            }
            if(onComplete != null){
                onComplete();
                onComplete = null;
            }
        }

        public static function getCarName(carObj:Object):String{
            if(!carObj){
                return "";
            }
            var startYear:String;
            var endYear:String;
            if(carObj is Car){
                var car:Car = carObj as Car;
                startYear = car.startYear?car.startYear.toString().substr(2):"";
                endYear = car.endYear?car.endYear.toString().substr(2):"";
                return startYear + ((startYear != "" && endYear != "")?"-":"") + endYear + " " + car.name + " " + car.model;
            }
            startYear = carObj.start_year?carObj.start_year.toString().substr(2):"";
            endYear = carObj.end_year?carObj.end_year.toString().substr(2):"";
            return startYear + ((startYear != "" && endYear != "")?"-":"") + endYear + " " + carObj.make + " " + carObj.model;
        }

        public function get limit():int {
            return _limit;
        }

        public function get skip():int {
            return _skip;
        }

        public function get finishSkip():int {
            return _finishSkip;
        }

        public function get isLoading():Boolean {
            return _isLoading;
        }

        public function set isLoading(value:Boolean):void {
            _isLoading = value;
        }
    }
}
