package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.feed.ChallengeNotificationData;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.NotificationView.NotificationFeedData;
    import com.despark.tuning3d.view.components.NotificationView.NotificationPanelViewItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotificationItemRenderer;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineTuningView;
    import com.distriqt.AdsManager;

    import feathers.controls.Label;
    import feathers.controls.List;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import starling.core.Starling;
    import starling.display.Quad;
    import starling.events.Event;

    public class ChallengesNotificationsScreen extends ChallengesScreen {

        private var _notificationList:List;
        private var _notifications : Array;
        private static const MAX_COUNT:int = 20;
        private static const MAX_LIST_COUNT:int = 80;
        private var rendererWidth : Number;
        //
        private var isLoading:Boolean = false;
        private var notesLoaded:Boolean = false;

        private var loadedData:Array;
        private var loadedNotes:Array;
        private var loadedRequests:Array;

        private var _messageLabel : Label;
        public function ChallengesNotificationsScreen() {
            super("ChallengesNotificationsScreen");
            FeedData.CURRENT_USER = UsersProxy.loggedInUser();
            AdsManager.$.showBannerAfterComplete = false;
        }

        override protected function initialize():void {
            super.initialize();

            _toggleGroup.selectedItem = _notificationsBtn;

            loadedData = [];
            loadedNotes = [];
            loadedRequests = [];
            addEventListener(ChallengeEvent.VIEW_TUNING, viewTuningHandler);

            var layout : VerticalLayout = new VerticalLayout();
            layout.gap = 0;
            layout.horizontalAlign = HorizontalAlign.LEFT;
            layout.verticalAlign = VerticalAlign.TOP;
            layout.useVirtualLayout = true;

            _notificationList = new List();
            _notificationList.dataProvider = new ListCollection();
            _notificationList.layout = layout;
            _notificationList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
            _notificationList.horizontalScrollPolicy = ScrollPolicy.OFF;
            _notificationList.verticalScrollPolicy = ScrollPolicy.ON;
            _notificationList.backgroundSkin = new Quad(1,1,0xe8e8e8);
            _notificationList.addEventListener(Event.SCROLL, listScrollHandler);

            _notificationList.width = contentWidth;
            _notificationList.height = contentHeight;
            _notificationList.x = 0;
            _notificationList.y = marginTop;
            rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;
            _notificationList.itemRendererFactory = notificationItemRendererFactory;

            addChild(_notificationList);

            _messageLabel = new MultilingualLabel();
            _messageLabel.includeInLayout = false;
            _messageLabel.visible = false;
            _messageLabel.height = 14;
            _messageLabel.textRendererFactory = LabelFactory.notificationsMessageField;
            _messageLabel.text = "You don't have any notifications at this moment";
            _messageLabel.width = actualWidth;
            addChild(_messageLabel);
            commitNotificationLoad();
        }


        override protected function initTopMenuContainer():void{
            super .initTopMenuContainer();
            title.text = 'NOTIFICATIONS';
        }

        public function commitNotificationLoad():void{
            if(!isLoading){
                dispatchEvent(new Event(ChallengeEvent.GET_NOTIFICATIONS, false, {skip:loadedNotes.length, limit: MAX_COUNT}));
                isLoading = true;
                showLoadBar();
            }
        }

        public function loadNotifications(data:Object):void {
            var arr:Array = convertDataToVO(data);
            loadedData = loadedData.concat(arr);
            loadedData.sortOn("time", Array.NUMERIC);
            loadedNotes = loadedNotes.concat(arr);
            if(arr.length<MAX_COUNT){
                notesLoaded = true;
            }
            isLoading = false;
            addLoadedNotesToList();
            //
            if(onComplete != null){
                onComplete();
                onComplete = null;
            }
        }

        private function addLoadedNotesToList():void{
            if(!isLoading){
                if(loadedData.length) {
                    var count:int = Math.min(loadedData.length, MAX_COUNT);
                    notifications = loadedData.slice(0, count);
                    loadedData.splice(0, count);
                }
                if(_messageLabel){
                    _messageLabel.visible = (!_notifications || _notifications.length == 0);
                }
                hideLoadBar();
            }
        }

        public function set notifications (value : Array) : void {
            if(_notifications && _notifications.length > 0) {
                var newListCollection : ListCollection = new ListCollection(value);
                _notificationList.dataProvider.addAll(newListCollection);
                if(_notificationList.verticalScrollPosition>0) {
                    _notificationList.scrollToPosition(0, _notificationList.verticalScrollPosition + _notificationList.height / 2, 1);
                }
            } else {
                _notifications = value;
                if(_notificationList) {
                    _notificationList.dataProvider = new ListCollection(value);
                    _notificationList.invalidate(INVALIDATION_FLAG_DATA);
                }
            }
            Starling.juggler.delayCall(cleanupList, 1);
        }

        private function cleanupList():void{
            if(_notificationList && _notificationList.isScrolling){
                Starling.juggler.delayCall(cleanupList, 1);
                return;
            }
            if(_notificationList && _notificationList.dataProvider.length>MAX_LIST_COUNT){
                _notificationList.validate();
                var itemSize:Number = ((_notificationList.maxVerticalScrollPosition+_notificationList.height)/_notificationList.dataProvider.length);
                var currentIndex:int = int(_notificationList.verticalScrollPosition/itemSize);
                var skipStart:int = Math.max(int(currentIndex - MAX_LIST_COUNT/2), 0);
                var skipEnd:int = Math.min(int(currentIndex + MAX_LIST_COUNT/2)+1, _notificationList.dataProvider.length);
                var i:int;
                for(i = 1; i < skipStart; i++){
                    if(_notifications[i] is ChallengeNotificationData){
                        ChallengeNotificationData(_notifications[i]).dispose();
                    }
                }
                for(i = skipEnd; i < _notifications.length; i++){
                    if(_notifications[i] is ChallengeNotificationData){
                        ChallengeNotificationData(_notifications[i]).dispose();
                    }
                }
                trace("itemSize",itemSize, _notificationList.dataProvider.length, "currentIndex", currentIndex, "skipStart", skipStart, "skipEnd", skipEnd);
            }
        }

        override protected function draw():void {
            super.draw();
            _notificationList.width = contentWidth;
            _notificationList.height = contentHeight;
            _notificationList.x = 0;
            _notificationList.y = marginTop;
            rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;

            if(_messageLabel){
                _messageLabel.x = (actualWidth - _messageLabel.width) / 2;
                _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
            }
        }

        private function listScrollHandler(event:Event):void {
            if(!_notificationList || !_notifications || _notifications.length == 0 || _notificationList.dataProvider.length == 0) {
                return;
            }
            if(_notificationList.verticalScrollPosition >= _notificationList.maxVerticalScrollPosition-30 && !isLoading) {
                commitNotificationLoad();
            }else if(_notificationList.verticalScrollPosition <= Math.max(_notificationList.maxVerticalScrollPosition - 2 * stage.stageHeight, 0)){
                Starling.juggler.removeDelayedCalls(cleanupList);
                Starling.juggler.delayCall(cleanupList, 0.2);
            }
        }

        private function notificationItemRendererFactory() : IListItemRenderer {
            var renderer : ChallengeNotificationItemRenderer = new ChallengeNotificationItemRenderer();
            renderer.isUltraWide = Constants.isUltraWideScreen;
            renderer.isWide = Constants.isWideScreen;
            renderer.setSize(rendererWidth, 80);
            renderer.eventDispatcher = this;
            return renderer;
        }


        override public function dispose():void {
            super.dispose();
            Starling.juggler.removeDelayedCalls(cleanupList);
            if(_notifications) {
                for (var i:int = _notifications.length - 1; i >= 0; i--) {
                    if(_notifications[i] is ChallengeNotificationData){
                        ChallengeNotificationData(_notifications[i]).destroy();
                    }else if(_notifications[i] is NotificationFeedData){
                        NotificationFeedData(_notifications[i]).destroy();
                    }
                }
                _notifications = null;
            }
            if(_notificationList){
                _notificationList.removeEventListener(Event.SCROLL, listScrollHandler);
                _notificationList.dataProvider = null;
                _notificationList.removeFromParent(true);
                _notificationList = null;
            }
            if(_messageLabel){
                _messageLabel.removeFromParent(true);
                _messageLabel = null;
            }
            loadedData = null;
            loadedNotes = null;
            loadedRequests = null;
        }

        private function viewTuningHandler(event:Event):void {
            var tuning:Tuning = event.data.tuning;
            var tuningParams : Object = {
                tuning: tuning,
                user: null,
                data: null,
                likes: 0,
                images: event.data.tuning.fullHdSnapshots,
                iLiked: false
            };
            var timelineTuningView:TimelineTuningView = new TimelineTuningView(tuningParams);
            addChild(timelineTuningView);
            timelineTuningView.layoutData = null;
            timelineTuningView.setSize(Starling.current.stage.width, Starling.current.stage.height);
        }

        public function removeItem(item:ChallengeNotificationData):void{
            var index:int = _notifications.indexOf(item);
            if(index != -1){
                _notifications.removeAt(index);
                _notificationList.invalidate(FeathersControl.INVALIDATION_FLAG_DATA);
                item.dispose();
            }
        }

        private static function convertDataToVO(data:Object):Array{
            var result:Array = [];
            for(var i:int = 0; i < data.length; i++){
                /*if(!data[i].tuning){
                    continue;
                }*/
                var item : ChallengeNotificationData = ChallengeNotificationData.createInstance(data[i]);
                result.push(item);
            }
            return result;
        }
    }
}
