package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.core.FeathersControl;

    import flash.geom.Rectangle;
    import flash.text.TextFormatAlign;

    import starling.animation.Transitions;
    import starling.core.Starling;
    import starling.display.DisplayObjectContainer;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class ChallengeAlertView extends FeathersControl{
        //
        public static const DIAMONDS:String = "diamonds";
        public static const KEYS:String = "keys";
        public static const KEYS_NOT_ENOUGH:String = "keysNotEnough";
        public static const DIAMONDS_NOT_ENOUGH:String = "diamondsNotEnough";
        public static const DIAMONDS_NOT_ENOUGH_OPTIONS:String = "diamondsNotEnoughOptions";
        public static const STARS:String = "stars";
        public static const CHALLENGE_SUBMITTED:String = "challengeSubmitted";
        public static const SUBMIT_CHALLENGE:String = "submitChallenge";
        public static const HOW_TO_VOTE:String = "howToVote";
        public static const KEYS_RECEIVED:String = "keysReceived";
        public static const DAILY_REWARDS_LIMIT_REACHED:String = "dailyRewardsLimitReached";
        public static const DAILY_REWARDS:String = "dailyRewards";
        public static const DAILY_GUEST_WELCOME:String = "dailyGuestRewards";
        public static const REWARDED_ADS_POPUP:String = "rewardedAdsPopup";
        public static const REWARDED_ADS_NOT_AVAILABLE_POPUP:String = "rewardedAdsNotAvailablePopup";
        public static const WELCOME_GUEST_POPUP:String = "welcomeGuestPopup";
        public static const WELCOME_USER_POPUP:String = "welcomeUserPopup";
        public static const UNLOCK_PACKAGE:String = "unlockPackagePopup";
        public static const UNLOCK_PACKAGE_ONE_TIME:String = "unlockOneTimePackagePopup";
        public static const UNLOCK_MODEL_ONE_TIME:String = "unlockOneTimeModelPopup";
        public static const CHALLENGE_REQUIREMENTS:String = "challengeRequirementsPopup";
        public static const CHALLENGE_NOT_AVAILABLE:String = "challengeNotAvailable";
        public static const RATE_BOX_POPUP:String = "moreCarsPopup";
        public static const CHALLENGE_TIME_LEFT:String = "challengeTimeLeft";
        public static const CHALLENGE_EXPIRED:String = "challengeExpired";
        //
        private var _contentWidth:Number;
        private var _data:*;
        private var _contentHeight:Number;
        private var _closeButton:Button;
        private var _background:Quad;

        protected var _titleLabel:Label;
        protected var _headerBorder:Quad;
        protected var _contentBackground:Image;
        protected var _closeHandler:Function;
        private static var isOpenView:ChallengeAlertView;
        public function ChallengeAlertView(contentWidth:Number = 320, contentHeight:Number = 207) {
            _contentWidth = contentWidth;
            _contentHeight = contentHeight;
    }

    public static function show(dialogType:String, data:* = null, target:DisplayObjectContainer = null):ChallengeAlertView{
        if(isOpenView && isOpenView.stage){
            isOpenView.close();
        }
        var dialogue:ChallengeAlertView;
        if(dialogType == DIAMONDS){
            dialogue = new DiamondStatsDialog();
        }else if(dialogType == DIAMONDS_NOT_ENOUGH){
            dialogue = new DiamondNotEnoughStatsDialog();
        }else  if(dialogType == DIAMONDS_NOT_ENOUGH_OPTIONS){
            dialogue = new DiamondNotEnoughOptionsDialog();
        }else if(dialogType == KEYS){
            dialogue = new KeysStatsDialog();
        }else if(dialogType == KEYS_NOT_ENOUGH){
            dialogue = new KeysNotEnoughStatsDialog();
        }else if(dialogType == STARS){
            dialogue = new StarsStatsDialog();
        }else if(dialogType == SUBMIT_CHALLENGE){
            dialogue = new SubmitChallengePopup();
        }else if(dialogType == CHALLENGE_SUBMITTED){
            dialogue = new ChallengeSubmittedDialog();
        }else if(dialogType == HOW_TO_VOTE){
            dialogue = new VoteHelpDialog();
        }else if(dialogType == KEYS_RECEIVED){
            dialogue = new KeysReceivedDialog();
        }else if(dialogType == DAILY_REWARDS_LIMIT_REACHED){
            dialogue = new RewardedDailyLimit();
        }else if(dialogType == DAILY_REWARDS){
            dialogue = new DailyRewards();
        }else if(dialogType == DAILY_GUEST_WELCOME){
            dialogue = new DailyGuestPopup();
        }else if(dialogType == REWARDED_ADS_POPUP){
            dialogue = new RewardedAdsPopup();
        }else  if(dialogType == UNLOCK_PACKAGE){
            dialogue = new UnlockPackagesPopup();
        }else  if(dialogType == UNLOCK_PACKAGE_ONE_TIME){
            dialogue = new UnlockOneTimePackagesPopup();
        }else if(dialogType == UNLOCK_MODEL_ONE_TIME){
            dialogue = new UnlockOneTimeModelPopup();
        }else if(dialogType == REWARDED_ADS_NOT_AVAILABLE_POPUP){
            dialogue = new RewardedAdsNotAvailablePopup();
        }else if(dialogType == WELCOME_GUEST_POPUP){
            dialogue = new WelcomeGuestPopup();
        }else if(dialogType == WELCOME_USER_POPUP){
            dialogue = new WelcomeUserPopup();
        }else if(dialogType == RATE_BOX_POPUP){
            dialogue = new RateBoxPopup();
        }else if(dialogType == CHALLENGE_REQUIREMENTS){
            dialogue = new ChallengeRequirementsNotMet();
        }else if(dialogType == CHALLENGE_NOT_AVAILABLE){
            dialogue = new ChallengeNotAvailable();
        }else if(dialogType == CHALLENGE_TIME_LEFT){
            dialogue = new ChallengeTimeLeft();
        }else if(dialogType == CHALLENGE_EXPIRED){
            dialogue = new ChallengeExpired();
        }
        if(dialogue){
            dialogue.data = data;
        }
        if(target){
            target.addChild(dialogue);
        }
        isOpenView = dialogue;
        return dialogue;
    }

    public function open():void{
        alpha = 0;
        Starling.current.stage.addChild(this);
        Starling.juggler.tween(this, 0.1, {alpha: 1, transition: Transitions.LINEAR});
        SoundManager.playSound(SoundManager.SELECT_PART);
    }

        protected function get self():ChallengeAlertView{
            return this;
        }

    public function close():void{
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        Starling.juggler.tween(this, 0.25,
                {
                    alpha: 0,
                    transition: Transitions.LINEAR,
                    onComplete:function ():void {
                        if(_closeHandler != null){
                            _closeHandler.call();
                        }
                        removeFromParent(true);
                        if(isOpenView == this){
                            isOpenView = null;
                        }
                    }
                });
    }

    protected override function initialize():void{
        super.initialize();
        _background = new Quad(1,1,0x000000);
        _background.alpha = 0.75;
        addChild(_background);

        _contentBackground = new Image(ButtonFactory.assetManager.getTexture("popUpContentBackground"));
        _contentBackground.scale9Grid = new Rectangle( 13, 13, 10, 6);
        _contentBackground.width = _contentWidth;
        _contentBackground.height = _contentHeight;
        addChild(_contentBackground);


        _closeButton = new Button();
        var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture("details_close"));
        closeIcon.scale = 0.75 * ButtonFactory.assetManager.scaleFactor/2;
        var closeSkin:Quad = new Quad(1,1,0xFFFFFF);
        closeSkin.width = closeSkin.height = 30;
        closeSkin.alpha = 0;
        _closeButton.width = closeIcon.width;
        _closeButton.height = closeIcon.height;
        _closeButton.defaultIcon = closeIcon;
        _closeButton.defaultSkin = closeSkin;
        addChild(_closeButton);
        _closeButton.validate();
        _closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);

        addEventListener(TouchEvent.TOUCH, foregroundTouchHandler);

        _titleLabel = LabelFactory.createSimpleLabel(FontSelector.AlumniSansHtml, 21, 0x4a4a4a, TextFormatAlign.LEFT, false, true);
        addChild(_titleLabel);

        _headerBorder = new Quad(1,1,0x4a4a4a);
        addChild(_headerBorder);
    }

        private function foregroundTouchHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(_background, TouchPhase.ENDED);
            var noTouch:Touch = event.getTouch(_contentBackground, TouchPhase.ENDED);
            if(touch && !noTouch){
                close();
            }
        }

        private function closeButtonTriggeredHandler(event:Event):void {
            close();
        }

    override protected function draw():void {
        super.draw();
        if(stage){
            _background.width = stage.stageWidth;
            _background.height = stage.stageHeight;

            _contentBackground.x = (stage.stageWidth - _contentBackground.width) / 2;
            _contentBackground.y = (stage.stageHeight - _contentBackground.height) / 2;

            _headerBorder.width = _contentBackground.width - 50;
            _headerBorder.height = 0.5;
            _headerBorder.x = _contentBackground.x + (_contentBackground.width - _headerBorder.width) / 2;
            _headerBorder.y = _contentBackground.y + 40;

            _closeButton.x = _contentBackground.x + _contentBackground.width - _closeButton.width - 5;
            _closeButton.y = _contentBackground.y + 2;

            _titleLabel.y = _contentBackground.y + 8;
        }
    }

    override public function dispose():void {
        super.dispose();

        if(_closeButton){
            _closeButton.removeFromParent(true);
            _closeButton = null;
        }

        if(_background){
            _background.removeFromParent(true);
            _background = null;
        }

        if(_contentBackground){
            _contentBackground.removeFromParent(true);
            _contentBackground = null;
        }

        if(_titleLabel){
            _titleLabel.removeFromParent(true);
            _titleLabel = null;
        }

        if(_headerBorder){
            _headerBorder.removeFromParent(true);
            _headerBorder = null;
        }
    }

        public function get data():* {
            return _data;
        }

        public function set data(value:*):void {
            _data = value;
        }
    }
}

import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
import com.despark.tuning3d.model.challenges.ChallengeData;
import com.despark.tuning3d.model.core.UIStrings;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ExtendedLabel;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.core.LayoutImage;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.utils.DateParser;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Label;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;
import starling.utils.StringUtil;

class InfoDialog extends ChallengeAlertView{
    protected var _messageLabel:Label;
    public function InfoDialog(popupWidth:Number = 320, popupHeight:Number = 170){
        super (popupWidth, popupHeight);
    }

    override protected function initialize():void{
        super .initialize();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.1;
        textFormat.leading = -1.5;
        _messageLabel = LabelFactory.createSimpleLabel(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        addChild(_messageLabel);
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _messageLabel.width = _headerBorder.width;
            _messageLabel.x = _headerBorder.x;
            _messageLabel.y = _headerBorder.y + 15;
        }
    }

    override public function dispose():void{
        super .dispose();
        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }
    }
}

class DiamondStatsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _challengeBtn:MultilingualButton;
    private var _rewardsBtn:MultilingualButton;
    public function DiamondStatsDialog(){
        super (320, 175);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "DIAMONDS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
        iconImage.scale = 0.8 * ButtonFactory.assetManager.scaleFactor / 2;

        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "Use Diamonds to unlock <b>“FEATURED”</b> Packages." + "\n\nEarn Diamonds by getting high scores in Tuning\n" +
                "Challenges or by watching Rewarded Ads.*";
        addChild(_messageLabel);
        _messageLabel.validate();

        _challengeBtn = ButtonFactory.defaultButton("GO TO CHALLENGES", 0x187499);
        _challengeBtn.width = 125;
        _challengeBtn.height = 23;
        _challengeBtn.paddingLeft = _challengeBtn.paddingRight = 10;
        _challengeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_challengeBtn);
        _challengeBtn.validate();
        _challengeBtn.addEventListener(Event.TRIGGERED, challengesSelectedHandler);

        _rewardsBtn = ButtonFactory.defaultButton("REWARDED ADS", 0xff0000, "button_gradient");
        _rewardsBtn.width = 125;
        _rewardsBtn.height = 23;
        _rewardsBtn.paddingLeft = _rewardsBtn.paddingRight = 5;
        _rewardsBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _rewardsBtn.addEventListener(Event.TRIGGERED, rewardsSelectedHandler);
        addChild(_rewardsBtn);
        _rewardsBtn.validate();

        var rewardsIcon:Image = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_btn_icon"));
        rewardsIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        rewardsIcon.color = 0xFFFFFE;

        _rewardsBtn.defaultIcon = rewardsIcon;
        _rewardsBtn.iconOffsetX = -5;
        _rewardsBtn.labelOffsetX = -1.5;
    }

    private function rewardsSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.REWARDS_TRIGGERED);
        }
        close();
    }

    private function challengesSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2 + 1;

            _challengeBtn.x = _contentBackground.x + _contentBackground.width /2 - (_challengeBtn.width + 10);
            _challengeBtn.y = _contentBackground.y + _contentBackground.height - (_challengeBtn.height / 2 + 30);

            _rewardsBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _rewardsBtn.y = _contentBackground.y + _contentBackground.height - (_rewardsBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
        if(iconImage){
            iconImage.removeFromParent(true);
            iconImage = null;
        }
    }
}

class DiamondNotEnoughOptionsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _challengeBtn:MultilingualButton;
    private var _rewardsBtn:MultilingualButton;
    private var _optionsBtn:Button;
    public function DiamondNotEnoughOptionsDialog(){
        super (380, 192);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "NOT ENOUGH DIAMONDS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
        iconImage.scale = 0.8 * ButtonFactory.assetManager.scaleFactor / 2;

        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "You don't have enough "+UIStrings.getDiamondChar()+" <b>Diamonds</b> to open\n" +
                "this package.\n<br>" +
                "Earn Diamonds by getting high scores in Tuning\n"+
                "Challenges or by watching Rewarded Ads.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _challengeBtn = ButtonFactory.defaultButton("GO TO CHALLENGES", 0x187499);
        _challengeBtn.width = 112;
        _challengeBtn.height = 23;
        _challengeBtn.paddingLeft = _challengeBtn.paddingRight = 10;
        _challengeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_challengeBtn);
        _challengeBtn.validate();
        _challengeBtn.addEventListener(Event.TRIGGERED, challengesSelectedHandler);

        _rewardsBtn = ButtonFactory.defaultButton("REWARDED ADS", 0xff0000, "button_gradient");
        _rewardsBtn.width = 112;
        _rewardsBtn.height = 23;
        _rewardsBtn.paddingLeft = _rewardsBtn.paddingRight = 5;
        _rewardsBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _rewardsBtn.addEventListener(Event.TRIGGERED, rewardsSelectedHandler);
        addChild(_rewardsBtn);
        _rewardsBtn.validate();

        _optionsBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _optionsBtn.defaultSkin = closeImg;
        _optionsBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        _optionsBtn.label = "MORE OPTIONS";
        _optionsBtn.width = 97;
        _optionsBtn.height = 23;
        addChild(_optionsBtn);
        _optionsBtn.validate();
        _optionsBtn.addEventListener(Event.TRIGGERED, moreOptionsSelectedHandler);


        var rewardsIcon:Image = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_btn_icon"));
        rewardsIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        rewardsIcon.color = 0xFFFFFE;

        _rewardsBtn.defaultIcon = rewardsIcon;
        _rewardsBtn.iconOffsetX = -5;
        _rewardsBtn.labelOffsetX = -1.5;

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        }
    }

    private function rewardsSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.REWARDS_TRIGGERED);
        }
        close();
    }

    private function challengesSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    private function moreOptionsSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.OPTIONS_TRIGGERED);
        }
        close();
    }

    override public function set data(value:*):void {
        super .data = value;
        if(value && value.hasOwnProperty("enabled") && _rewardsBtn){
            _rewardsBtn.isEnabled = value["enabled"];
            _rewardsBtn.alpha = (_rewardsBtn.isEnabled? 1: 0.75);
        }
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2 + 1;

            var startX:Number = _contentBackground.x + (_contentBackground.width - (_challengeBtn.width + _rewardsBtn.width + _optionsBtn.width + 20)) / 2;

            _challengeBtn.x = startX;
            _challengeBtn.y = _contentBackground.y + _contentBackground.height - (_challengeBtn.height / 2 + 30);

            _rewardsBtn.x = _challengeBtn.width + _challengeBtn.x + 10;
            _rewardsBtn.y = _contentBackground.y + _contentBackground.height - (_rewardsBtn.height / 2 + 30);

            _optionsBtn.x = _rewardsBtn.width + _rewardsBtn.x + 10;
            _optionsBtn.y = _contentBackground.y + _contentBackground.height - (_rewardsBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
        if(iconImage){
            iconImage.removeFromParent(true);
            iconImage = null;
        }
        if(_challengeBtn){
            _challengeBtn.removeFromParent(true);
            _challengeBtn = null;
        }
    }
}


class DiamondNotEnoughStatsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _challengeBtn:MultilingualButton;
    private var _rewardsBtn:MultilingualButton;
    public function DiamondNotEnoughStatsDialog(){
        super (320, 192);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "NOT ENOUGH DIAMONDS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
        iconImage.scale = 0.8 * ButtonFactory.assetManager.scaleFactor / 2;

        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "You don't have enough "+UIStrings.getDiamondChar()+" <b>Diamonds</b> to open\n" +
                "this package.\n<br>" +
                "Earn Diamonds by getting high scores in Tuning\n"+
                "Challenges or by watching Rewarded Ads.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _challengeBtn = ButtonFactory.defaultButton("GO TO CHALLENGES", 0x187499);
        _challengeBtn.width = 125;
        _challengeBtn.height = 23;
        _challengeBtn.paddingLeft = _challengeBtn.paddingRight = 10;
        _challengeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_challengeBtn);
        _challengeBtn.validate();
        _challengeBtn.addEventListener(Event.TRIGGERED, challengesSelectedHandler);

        _rewardsBtn = ButtonFactory.defaultButton("REWARDED ADS", 0xff0000, "button_gradient");
        _rewardsBtn.width = 125;
        _rewardsBtn.height = 23;
        _rewardsBtn.paddingLeft = _rewardsBtn.paddingRight = 5;
        _rewardsBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _rewardsBtn.addEventListener(Event.TRIGGERED, rewardsSelectedHandler);
        addChild(_rewardsBtn);
        _rewardsBtn.validate();

        var rewardsIcon:Image = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_btn_icon"));
        rewardsIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        rewardsIcon.color = 0xFFFFFE;

        _rewardsBtn.defaultIcon = rewardsIcon;
        _rewardsBtn.iconOffsetX = -5;
        _rewardsBtn.labelOffsetX = -1.5;

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        }
    }

    private function rewardsSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.REWARDS_TRIGGERED);
        }
        close();
    }

    private function challengesSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    override public function set data(value:*):void {
        super .data = value;
        if(value && value.hasOwnProperty("enabled") && _rewardsBtn){
            _rewardsBtn.isEnabled = value["enabled"];
            _rewardsBtn.alpha = (_rewardsBtn.isEnabled? 1: 0.75);
        }
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2 + 1;

            _challengeBtn.x = _contentBackground.x + _contentBackground.width /2 - (_challengeBtn.width + 10);
            _challengeBtn.y = _contentBackground.y + _contentBackground.height - (_challengeBtn.height / 2 + 30);

            _rewardsBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _rewardsBtn.y = _contentBackground.y + _contentBackground.height - (_rewardsBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
        if(iconImage){
            iconImage.removeFromParent(true);
            iconImage = null;
        }
    }
}

class KeysStatsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _votingBtn:MultilingualButton;
    private var _closeBtn:Button;
    public function KeysStatsDialog(){
        super (320, 176);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "KEYS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_keys_icon"));
        iconImage.scale = 0.95 * ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "Keys are used to enter Tuning Challenges." +
                "\n\nTo earn more Keys, vote on other Tuning\n " + "Challenges entries in the <b>Voting Section.</b>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _votingBtn = ButtonFactory.defaultButton("VOTING", 0x267596);
        _votingBtn.width = 96;
        _votingBtn.height = 23;
        _votingBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_votingBtn);
        _votingBtn.validate();
        _votingBtn.addEventListener(Event.TRIGGERED, votingSelectedHandler);


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function votingSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.VOTING_TRIGGERED);
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;

            _votingBtn.x = _contentBackground.x + _contentBackground.width /2 - (_votingBtn.width + 10);
            _votingBtn.y = _contentBackground.y + _contentBackground.height - (_votingBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class KeysNotEnoughStatsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _votingBtn:MultilingualButton;
    private var _closeBtn:Button;
    public function KeysNotEnoughStatsDialog(){
        super (320, 176);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "NOT ENOUGH KEYS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_keys_icon"));
        iconImage.scale = 0.95 * ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "You don't have enough Keys to apply to this\n" + "challenge." + "\n\nTo earn more Keys, vote on other Tuning\n " + "Challenges entries in the <b>Voting Section.</b>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _votingBtn = ButtonFactory.defaultButton("VOTING", 0x267596);
        _votingBtn.width = 96;
        _votingBtn.height = 23;
        _votingBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_votingBtn);
        _votingBtn.validate();
        _votingBtn.addEventListener(Event.TRIGGERED, votingSelectedHandler);


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function votingSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.VOTING_TRIGGERED);
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;

            _votingBtn.x = _contentBackground.x + _contentBackground.width /2 - (_votingBtn.width + 10);
            _votingBtn.y = _contentBackground.y + _contentBackground.height - (_votingBtn.height / 2 + 35);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 35);
        }
    }

    override public function dispose():void{
        super .dispose();
        if(_votingBtn){
            _votingBtn.removeFromParent(true);
            _votingBtn = null;
        }
        if(iconImage){
            iconImage.removeFromParent(true);
            iconImage = null;
        }
        if(_closeBtn){
            _closeBtn.removeFromParent(true);
            _closeBtn = null;
        }
    }
}

class StarsStatsDialog extends InfoDialog{
    private var iconImage:Image;

    private var _challengesBtn:MultilingualButton;
    private var _closeBtn:Button;
    public function StarsStatsDialog(){
        super (320, 191);
    }

    override protected function initialize():void{
        super .initialize();
        _titleLabel.text = "STARS";
        _titleLabel.validate();


        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "Your Star Score is the average score of all Tuning<br>"
                + "Challenges that you have participated.<br><br>"
                + "Please note that your standings in the leaderboard will  appear after you have submitted 3 challenges.";
        addChild(_messageLabel);
        _messageLabel.validate();


        _challengesBtn = ButtonFactory.defaultButton("CHALLENGES", 0x267596);
        _challengesBtn.width = 96;
        _challengesBtn.height = 23;
        _challengesBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_challengesBtn);
        _challengesBtn.validate();
        _challengesBtn.addEventListener(Event.TRIGGERED, challengesTriggeredHandler);


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 90;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);

    }

    private function challengesTriggeredHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            _messageLabel.y = _headerBorder.y + 10;

            iconImage.x = _titleLabel.x + _titleLabel.width + 5;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;

            _challengesBtn.x = _contentBackground.x + _contentBackground.width /2 - (_challengesBtn.width + 10);
            _challengesBtn.y = _contentBackground.y + _contentBackground.height - (_challengesBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();

    }
}

class ChallengeSubmittedDialog extends InfoDialog{
    private var _continueBtn:Button;
    private var iconImage:Image;
    public function ChallengeSubmittedDialog(){
        super (241, 154);
    }

    override protected function initialize():void{
        super .initialize();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_submitted_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _titleLabel.text = "BUILD SUBMITTED";
        _titleLabel.validate();

        var votingFinishDate:Date = DateParser.parseDate(data.votingFinishDate);
        var timeLeft:String = DateParser.getInChallengeLabel(votingFinishDate.time - new Date().time, {sec:["second", "seconds"], min:["minute","minutes"], hr:["hour","hours"], day:["day", "days"]});

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = 0;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Results will be ready in " + timeLeft + ".\nThank you for submitting your\nbuild for voting.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _continueBtn = ButtonFactory.defaultButton("CONTINUE", 0x187499);
        _continueBtn.width = 91;
        _continueBtn.height = 23;
        _continueBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _continueBtn.addEventListener(Event.TRIGGERED, continueTriggeredHandler);
        addChild(_continueBtn);
        _continueBtn.validate();
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
    }

    private function continueTriggeredHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 15;

            _continueBtn.x = _contentBackground.x + (_contentBackground.width - _continueBtn.width) / 2;
            _continueBtn.y = _contentBackground.y + _contentBackground.height - (_continueBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }

}

class VoteHelpDialog extends InfoDialog{
    public function VoteHelpDialog(){
        super(320, 191);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "HOW VOTING WORKS";
        _titleLabel.validate();

        _messageLabel.text = "Your votes determine the Star scores & ratings in\n"+
                "Tuning Challenges - the more votes an entry\n"+
                "receives, the higher its Star score & rating gets!\n<br>"+
                "Voting earns you "+UIStrings.getKeyChar()+" <b>Keys</b> so you can enter more\n"+
                "Tuning Challenges - each set of five votes earns\n"+
                "you "+UIStrings.getKeyChar()+" <b>Keys!</b>\n<br>"+
                "<b>DON'T FORGET TO VOTE!</b>";
        addChild(_messageLabel);
        _messageLabel.validate();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 8;
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class KeysReceivedDialog extends ChallengeAlertView{
    private var _keysLabel:ExtendedLabel;
    private var _continueBtn:MultilingualButton;
    private var _totalKeysLabel:Label;

    public function KeysReceivedDialog(){
        super(201, 136.5);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "KEYS RECEIVED";
        _titleLabel.validate();

        _totalKeysLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 8, 0x4a4a4a, TextFormatAlign.CENTER, false);
        _totalKeysLabel.width = 140;
        _totalKeysLabel.text = "keys " + data.totalKeys + "/" + data.maxKeys;
        addChild(_totalKeysLabel);

        _keysLabel = new ExtendedLabel();
        _keysLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 21, 0x4a4a4a, TextFormatAlign.LEFT, false);
        _keysLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_keys_icon"));

        _keysLabel.defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _keysLabel.labelOffsetX = 5;
        var keys:String = (data.keys <= 0?"0":data.keys.toString());
        _keysLabel.text = (keys == "0"?"":"+") + keys;
        addChild(_keysLabel);
        _keysLabel.validate();


        _continueBtn = ButtonFactory.defaultButton("CONTINUE VOTING", 0x187499);
        _continueBtn.width = 121;
        _continueBtn.height = 23;
        _continueBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_continueBtn);
        _continueBtn.validate();
        _continueBtn.addEventListener(Event.TRIGGERED, continueTriggeredHandler);
    }

    private function continueTriggeredHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = _titleLabel.width;
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;

            _totalKeysLabel.x = _headerBorder.x + (_headerBorder.width - _totalKeysLabel.width) / 2;
            _totalKeysLabel.y = _headerBorder.y + 5;

            _keysLabel.x = _headerBorder.x + (_headerBorder.width - _keysLabel.width) / 2;
            _keysLabel.y = _headerBorder.y + 21;

            _continueBtn.x = _headerBorder.x + (_headerBorder.width - _continueBtn.width) / 2;
            _continueBtn.y = _keysLabel.y + _keysLabel.height + 8;
        }
    }

    override public function dispose():void{
        super .dispose();
        if(_titleLabel){
            _titleLabel.removeFromParent(true);
            _titleLabel = null;
        }
        if(_keysLabel){
            _keysLabel.removeFromParent(true);
            _keysLabel = null;
        }
        if(_totalKeysLabel){
            _totalKeysLabel.removeFromParent(true);
            _totalKeysLabel = null;
        }
        if(_continueBtn){
            _continueBtn.removeFromParent(true);
            _continueBtn = null;
        }

        if(_headerBorder){
            _headerBorder.removeFromParent(true);
            _headerBorder = null;
        }
    }
}

class RewardedDailyLimit extends InfoDialog{
    private var iconImage:Image;

    private var _challengesBtn:MultilingualButton;
    private var _closeBtn:Button;
    public function RewardedDailyLimit(){
        super (320, 195);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "DAILY LIMIT OF REWARDS ADS";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "You have reached the limit of 6 daily Rewarded\n" +
                "Ads. Tomorrow the limit will be reset.\n\n"+
        "<font size='10'>Please note that you can also earn "+UIStrings.getDiamondChar()+" </font><b>Diamonds</b>\nby participating in Tuning Challenges.</font>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _challengesBtn = ButtonFactory.defaultButton("CHALLENGES", 0x187499);
        _challengesBtn.width = 96;
        _challengesBtn.height = 23;
        _challengesBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        addChild(_challengesBtn);
        _challengesBtn.validate();
        _challengesBtn.addEventListener(Event.TRIGGERED, challengesSelectedHandler);


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function challengesSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 20;

            _challengesBtn.x = _contentBackground.x + _contentBackground.width /2 - (_challengesBtn.width + 10);
            _challengesBtn.y = _contentBackground.y + _contentBackground.height - (_challengesBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class ChallengeRequirementsNotMet extends InfoDialog{
    private var iconImage:Image;
    private var reqImage:Image;

    private var messageData:Object = function():Object{
        var result:Object = {};
        result[ChallengeData.TYPE_WIDE] = {
            message:"Wide body kits or front & rear fender flares \n" +
                    "must be applied.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Parts Menu  2. Choose & apply required body parts</font>",
            imageId: "req_wide_body"
        };
        result[ChallengeData.TYPE_STANCE] = {
            message:"Wheel camber and/or wheel off-set must be \n" +
                    "applied to the front and rear wheels.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Stance Menu  2. Apply Desired Stance Settings</font>",
            imageId: "req_stance"
        };
        result[ChallengeData.TYPE_BACKGROUND] = {
            message:"Please choose any background from\n the list or upload a new one.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Car settings  2. Select a background from background list</font>",
            imageId: "req_background"
        };


        result[ChallengeData.TYPE_CAMBER] = {
            message:"Wheel camber must be applied to the \n" +
                    "front and/or rear wheels.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Stance Menu  2. Apply Desired Stance Settings</font>",
            imageId: "req_camber"
        };
        result[ChallengeData.TYPE_OFFSET] = {
            message:"Wheel off-set must be applied to the  \n" +
                    "front and/or rear wheels.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Stance Menu  2. Apply Desired Stance Settings</font>",
            imageId: "req_offset"
        };
        result[ChallengeData.TYPE_STRIPES] = {
            message:"Racing Stripe Kits must be applied \n" +
                    "from Vinyls category.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Parts Menu  2. Choose & apply required body parts</font>",
            imageId: "req_racing_strip"
        };
        result[ChallengeData.TYPE_VINYLS] = {
            message:"All Decals must be applied \n" +
                    "from Vinyls category.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Parts Menu  2. Choose & apply required body parts</font>",
            imageId: "req_decals"
        };
        result[ChallengeData.TYPE_TWO_TONE] = {
            message:"Two Tone Vinyl Kits \n" +
                    "must be applied.\n<br>" +
                    "<font size='8'>" + UIStrings.getLightBulbChar() + " <b>Help tips:</b>\n" +
                    "1. Open Parts Menu  2. Choose & apply required body parts</font>",
            imageId: "req_two_tone_vinyl"
        };
        return result;
    }();

    public function ChallengeRequirementsNotMet(){
        super (321, 191);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "CHALLENGE REQUIREMENTS NOT MET!";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_requirement_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        if(data){
            var dataArr:Array = StringUtil.trim(data).split(',');
            var msgData:Object;
            if(dataArr.indexOf(ChallengeData.TYPE_WIDE) != -1){
                msgData = messageData[ChallengeData.TYPE_WIDE];

            }else if(dataArr.indexOf(ChallengeData.TYPE_STANCE) != -1){
                msgData = messageData[ChallengeData.TYPE_STANCE];

            }else if(dataArr.indexOf(ChallengeData.TYPE_BACKGROUND) != -1){
                msgData = messageData[ChallengeData.TYPE_BACKGROUND];

            }else if(dataArr.indexOf(ChallengeData.TYPE_CAMBER) != -1){
                msgData = messageData[ChallengeData.TYPE_CAMBER];

            }else if(dataArr.indexOf(ChallengeData.TYPE_OFFSET) != -1){
                msgData = messageData[ChallengeData.TYPE_OFFSET];

            }else if(dataArr.indexOf(ChallengeData.TYPE_STRIPES) != -1){
                msgData = messageData[ChallengeData.TYPE_STRIPES];

            }else if(dataArr.indexOf(ChallengeData.TYPE_VINYLS) != -1){
                msgData = messageData[ChallengeData.TYPE_VINYLS];

            }else if(dataArr.indexOf(ChallengeData.TYPE_TWO_TONE) != -1){
                msgData = messageData[ChallengeData.TYPE_TWO_TONE];
            }

            if(msgData){
                _messageLabel.text = msgData.message;
                reqImage = new Image(ButtonFactory.assetManager.getTexture(msgData.imageId));
            }
        }

        addChild(_messageLabel);
        _messageLabel.validate();
        if(reqImage){
            reqImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
            reqImage.color = 0xFF0000;
            addChild(reqImage);
        }

    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 10;
            if(reqImage){
                reqImage.width = _headerBorder.width;
                reqImage.scale = reqImage.scaleX;
                reqImage.x = _contentBackground.x + (_contentBackground.width - reqImage.width) / 2;
                reqImage.y = _messageLabel.y + _messageLabel.height + 2;
            }
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class DailyRewards extends InfoDialog{
    private var iconImage:Image;

    private var _closeBtn:Button;
    public function DailyRewards(){
        super (332, 195);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "DAILY REWARD!";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        var andText:String;
        var keysText:String;
        var keyNum:int = parseInt(data.keys);
        if(keyNum){
            if(keyNum == 1){
                keysText = UIStrings.getKeyChar() + " <b>1 Key</b>";
            }else{
                keysText = UIStrings.getKeyChar() + " <b>" + data.keys + " Keys</b>";
            }
        }else{
            keysText = "";
        }
        var diamondText:String;
        var diamondNum:int = parseInt(data.diamonds);
        if(diamondNum){
            if(diamondNum == 1){
                diamondText = UIStrings.getDiamondChar() + " <b>1 Diamond</b>";
            }else{
                diamondText = UIStrings.getDiamondChar() + " <b>" + data.diamonds + " Diamonds</b>";
            }
        }else{
            diamondText = "";
        }
        andText = (diamondNum && keyNum)?" and ":"";

        _messageLabel.text = "Welcome back to 3DTuning!\n" +
                "We have added " + keysText + andText + diamondText + "\nto your account!\n\n"+
        "Please don't forget to come back tomorrow\nfor more "+UIStrings.getKeyChar()+" <b>Keys</b> and "+UIStrings.getDiamondChar()+" <b>Diamonds!</b>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 15;

            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class ChallengeNotAvailable extends InfoDialog{
    private var _closeBtn:Button;
    public function ChallengeNotAvailable(){
        super (240, 160);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "VOTING HAS FINISHED";
        _titleLabel.validate();
        _messageLabel.text = "This challenge is not available\n" +
                "for voting anymore!\n\n"+
        "Please vote for the next challenge.</b>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
        _closeHandler = function():void{
            dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        };
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 10;
            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class ChallengeTimeLeft extends InfoDialog{
    private var _closeBtn:Button;
    public function ChallengeTimeLeft(){
        super (240, 156);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = UIStrings.getTimeLimitChar(18) + " TIME IS ALMOST UP!";
        _titleLabel.validate();
        _messageLabel.text = "There is "+ data +" left for\n" +
                "submitting your build to this\n"+
        "challenge. Please hurry up!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
        _closeHandler = function():void{
            dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        };
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 15;
            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class ChallengeExpired extends InfoDialog{
    private var _closeBtn:Button;
    public function ChallengeExpired(){
        super (380, 216);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = UIStrings.getFinishedChar(16) + "  SUBMISSIONS TO THIS CHALLENGE ARE OVER!";
        _titleLabel.validate();
        _messageLabel.text = "<font size='11'>Unfortunately, the time has run out for submitting\n"
                +"builds to this challenge.\n\n" +
                "Your spent " + UIStrings.getKeyChar()+ " keys for this challenge has been added\n"+
                "back to you account.</font>\n\n"+
        "Please choose another active challenge.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
        _closeHandler = function():void{
            dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        };
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 20;
            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class RewardedAdsPopup extends InfoDialog{
    private var iconImage:Image;

    private var _closeBtn:Button;
    private var _watchBtn:Button;
    public function RewardedAdsPopup(){
        super (380, 215);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "REWARDED ADS IN 3DTUNING";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _messageLabel.text = "<p align='left'>By watching Rewarded Ads you collect "+UIStrings.getDiamondChar()+" <b>Diamonds,</b> that\n" +
                "can be used to open <b>“FEATURED”</b> packages.\n<font size='6'><br></font>"+
        "Every watched Rewarded Ad earns you <b>1 (one) "+UIStrings.getDiamondChar()+" Diamond.</b>\n<font size='6'><br></font>"+
        "Subject to availability of Rewarded Ads, there is a limit of 6 Ads available for each user per day.</p>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _watchBtn = ButtonFactory.defaultButton("WATCH NOW", 0xff0000, "button_gradient");
        _watchBtn.width = 100;
        _watchBtn.height = 23;
        _watchBtn.paddingLeft = _watchBtn.paddingRight = 5;
        _watchBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _watchBtn.addEventListener(Event.TRIGGERED, watchSelectedHandler);
        addChild(_watchBtn);
        _watchBtn.validate();

        var rewardsIcon:Image = new Image(ButtonFactory.assetManager.getTexture("rewards_watch_btn_icon"));
        rewardsIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        rewardsIcon.color = 0xFFFFFE;

        _watchBtn.defaultIcon = rewardsIcon;
        _watchBtn.iconOffsetX = -5;
        _watchBtn.labelOffsetX = -1.5;

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "NOT NOW";
        _closeBtn.width = 96;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
    }

    private function watchSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.WATCH_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 7;

            _watchBtn.x = _contentBackground.x + _contentBackground.width /2 - (_watchBtn.width + 10);
            _watchBtn.y = _contentBackground.y + _contentBackground.height - (_watchBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}


class UnlockPackagesPopup extends InfoDialog{

    private var _closeBtn:Button;
    private var _confirmBtn:Button;
    public function UnlockPackagesPopup(){
        super (321, 176);
    }

    override protected function initialize():void{
        super .initialize();

        var pack:Packet = data as Packet;
        _titleLabel.text = "UNLOCK “" + pack.title.toUpperCase() + "” PACKAGE";
        _titleLabel.validate();

        _messageLabel.text = "Using <b>" + pack.coin3dFee.toString() + "</b> " + UIStrings.getDiamondChar()+" <b>Diamonds</b> will grant you unlimited\n" +
        "access to all of the content available in\n" +
        "<b>“" + pack.title.toUpperCase() + "”</b> Package for 7 days!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _confirmBtn = ButtonFactory.defaultButton("CONFIRM", 0xff0000, "button_gradient");
        _confirmBtn.width = 92;
        _confirmBtn.height = 23;
        _confirmBtn.paddingLeft = _confirmBtn.paddingRight = 5;
        _confirmBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _confirmBtn.addEventListener(Event.TRIGGERED, confirmSelectedHandler);
        addChild(_confirmBtn);
        _confirmBtn.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 91;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
    }

    private function confirmSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CONFIRM_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 25;

            _confirmBtn.x = _contentBackground.x + _contentBackground.width /2 - (_confirmBtn.width + 10);
            _confirmBtn.y = _contentBackground.y + _contentBackground.height - (_confirmBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class UnlockOneTimePackagesPopup extends InfoDialog{

    private var _closeBtn:Button;
    private var _confirmBtn:Button;
    public function UnlockOneTimePackagesPopup(){
        super (321, 176);
    }

    override protected function initialize():void{
        super .initialize();

        var pack:Packet = data as Packet;
        _titleLabel.text = "UNLOCK ONE “" + pack.title.toUpperCase() + "” MODEL";
        _titleLabel.validate();

        _messageLabel.text = "Using <b>" + Packet.ONE_TIME_PACKET_FEE.toString() + "</b> " + UIStrings.getDiamondChar()+" <b>Diamonds</b> will grant you ONE-TIME\n" +
        "access to any one of the models available in\n" +
        "<b>“" + pack.title.toUpperCase() + "”</b> package.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _confirmBtn = ButtonFactory.defaultButton("CONFIRM", 0xff0000, "button_gradient");
        _confirmBtn.width = 92;
        _confirmBtn.height = 23;
        _confirmBtn.paddingLeft = _confirmBtn.paddingRight = 5;
        _confirmBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _confirmBtn.addEventListener(Event.TRIGGERED, confirmSelectedHandler);
        addChild(_confirmBtn);
        _confirmBtn.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 91;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
    }

    private function confirmSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CONFIRM_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 25;

            _confirmBtn.x = _contentBackground.x + _contentBackground.width /2 - (_confirmBtn.width + 10);
            _confirmBtn.y = _contentBackground.y + _contentBackground.height - (_confirmBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class UnlockOneTimeModelPopup extends InfoDialog{

    private var _optionsBtn:Button;
    private var _confirmBtn:Button;
    public function UnlockOneTimeModelPopup(){
        super (321, 160);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "UNLOCK THIS MODEL";
        _titleLabel.validate();

        _messageLabel.text = "Using <b>" + Packet.ONE_TIME_PACKET_FEE.toString() + "</b> " + UIStrings.getDiamondChar()+" <b>Diamonds</b> will grant you ONE-TIME\n" +
        "access to this model.";
        addChild(_messageLabel);
        _messageLabel.validate();

        _confirmBtn = ButtonFactory.defaultButton("CONFIRM", 0xff0000, "button_gradient");
        _confirmBtn.width = 112;
        _confirmBtn.height = 23;
        _confirmBtn.paddingLeft = _confirmBtn.paddingRight = 5;
        _confirmBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _confirmBtn.addEventListener(Event.TRIGGERED, confirmSelectedHandler);
        addChild(_confirmBtn);
        _confirmBtn.validate();

        _optionsBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _optionsBtn.defaultSkin = closeImg;
        _optionsBtn.label = "MORE OPTIONS";
        _optionsBtn.width = 112;
        _optionsBtn.height = 23;
        _optionsBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_optionsBtn);
        _optionsBtn.validate();
        _optionsBtn.addEventListener(Event.TRIGGERED, moreOptionsSelectedHandler);

        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
            _closeHandler = null;
        };
    }

    private function confirmSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CONFIRM_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    private function moreOptionsSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.OPTIONS_TRIGGERED);
            _closeHandler = null;
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;
            _messageLabel.y = _headerBorder.y + 15;

            _confirmBtn.x = _contentBackground.x + _contentBackground.width /2 - (_confirmBtn.width + 10);
            _confirmBtn.y = _contentBackground.y + _contentBackground.height - (_confirmBtn.height / 2 + 30);

            _optionsBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _optionsBtn.y = _contentBackground.y + _contentBackground.height - (_optionsBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class RewardedAdsNotAvailablePopup extends InfoDialog{
    private var iconImage:Image;

    private var _closeBtn:Button;
    private var _challengesBtn:Button;
    public function RewardedAdsNotAvailablePopup(){
        super (380, 215);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "REWARDED ADS IN 3DTUNING";
        _titleLabel.validate();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.58;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Unfortunately, there are no Reward Ads available at this moment.\n" +
                "Please come back a little bit later for more Reward Ads.\n\n"+
        "Please note that you can also earn "+UIStrings.getDiamondChar()+" <b>Diamonds</b> by\n"+
        "participating in <b>Tuning Challenges</b>\n<font size='8'><br></font>"+
        "<font size='9'>We will also notify you once there are available Reward Ads with a\nnotification sign on the Diamonds icon on the home screen</font>";
        addChild(_messageLabel);
        _messageLabel.validate();

        _challengesBtn = ButtonFactory.defaultButton("CHALLENGES", 0x187499);
        _challengesBtn.width = 96;
        _challengesBtn.height = 23;
        _challengesBtn.paddingLeft = _challengesBtn.paddingRight = 5;
        _challengesBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _challengesBtn.addEventListener(Event.TRIGGERED, challengesSelectedHandler);
        addChild(_challengesBtn);
        _challengesBtn.validate();


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 90;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function challengesSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CHALLENGES_TRIGGERED);
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.CLOSE_TRIGGERED);
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 18;

            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);

            _challengesBtn.x = _contentBackground.x + _contentBackground.width /2 - (_challengesBtn.width + 10);
            _challengesBtn.y = _contentBackground.y + _contentBackground.height - (_challengesBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class WelcomeGuestPopup extends InfoDialog{
    private var _closeBtn:Button;
    private var _loginBtn:Button;
    public function WelcomeGuestPopup(){
        super (380, 215);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "WELCOME TO THE UPDATED 3DTUNING!";
        _titleLabel.validate();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.2;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Brand new \"Tuning Challenges\" and major updates to the\n"+
                "\"Tuning Configurator\" are available in this update!\n\n" +
                "Please log in to your account to claim "+UIStrings.getKeyChar()+" <b>Keys</b> & "+UIStrings.getDiamondChar()+" <b>Diamonds</b>\n"+
        "so you can start your  tuning journey straight away!\n\n"+
        "Hope you will enjoy this update!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _loginBtn = ButtonFactory.defaultButton("LOG IN", 0x187499);
        _loginBtn.width = 96;
        _loginBtn.height = 23;
        _loginBtn.paddingLeft = _loginBtn.paddingRight = 5;
        _loginBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _loginBtn.addEventListener(Event.TRIGGERED, loginSelectedHandler);
        addChild(_loginBtn);
        _loginBtn.validate();


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "NOT NOW";
        _closeBtn.width = 90;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function loginSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.LOGIN_TRIGGERED);
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;

            _messageLabel.y = _headerBorder.y + 15;

            _loginBtn.x = _contentBackground.x + _contentBackground.width /2 - (_loginBtn.width + 10);
            _loginBtn.y = _contentBackground.y + _contentBackground.height - (_loginBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class RateBoxPopup extends InfoDialog{
    private var _closeBtn:Button;
    private var _commentBtn:Button;
    public function RateBoxPopup(){
        super (321, 176);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "WELCOME TO 3DTUNING!";
        _titleLabel.validate();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = 0;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "We are constantly adding new cars, parts and\n" +
                "tuning option. Your feedback is very important\n" +
                "for us to stay on the right track!\n<font size='9'><br></font>" +
                "Please give us your feedback!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _commentBtn = ButtonFactory.defaultButton("LEAVE A COMMENT", 0x187499);
        _commentBtn.width = 121;
        _commentBtn.height = 23;
        _commentBtn.paddingLeft = _commentBtn.paddingRight = 5;
        _commentBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _commentBtn.addEventListener(Event.TRIGGERED, loginSelectedHandler);
        addChild(_commentBtn);
        _commentBtn.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "REMIND ME LATER";
        _closeBtn.width = 121;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);

        _closeHandler = function():void{
            self.dispatchEvent(new ChallengeAlertEvent(ChallengeAlertEvent.CLOSE_TRIGGERED));
        };
    }

    private function loginSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            if(data && data.car){
                self.dispatchEvent(new ChallengeAlertEvent(ChallengeAlertEvent.COMMENT_TRIGGERED, {car: data.car}));
            }else{
                self.dispatchEvent(new ChallengeAlertEvent(ChallengeAlertEvent.COMMENT_TRIGGERED));
            }
        };
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEvent(new ChallengeAlertEvent(ChallengeAlertEvent.REMIND_TRIGGERED));
        };
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;

            _messageLabel.y = _headerBorder.y + 12;

            _commentBtn.x = _contentBackground.x + _contentBackground.width /2 - (_commentBtn.width + 10);
            _commentBtn.y = _contentBackground.y + _contentBackground.height - (_commentBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class DailyGuestPopup extends InfoDialog{
    private var _closeBtn:Button;
    private var _loginBtn:Button;
    private var iconImage:Image;
    public function DailyGuestPopup(){
        super (241, 154);
    }

    override protected function initialize():void{
        super .initialize();

        iconImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_popup_icon"));
        iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
        iconImage.color = 0xFF0000;
        addChild(iconImage);

        _titleLabel.text = "DAILY REWARD!";
        _titleLabel.validate();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.2;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Welcome back to <b>3DT</b>uning\nPlease log in to claim your daily\nrewards!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _loginBtn = ButtonFactory.defaultButton("LOG IN", 0x187499);
        _loginBtn.width = 71;
        _loginBtn.height = 23;
        _loginBtn.paddingLeft = _loginBtn.paddingRight = 5;
        _loginBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _loginBtn.addEventListener(Event.TRIGGERED, loginSelectedHandler);
        addChild(_loginBtn);
        _loginBtn.validate();


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "LATER";
        _closeBtn.width = 71;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function loginSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEventWith(ChallengeAlertEvent.LOGIN_TRIGGERED);
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            var titleWidth:Number = iconImage.width + _titleLabel.width + 5;
            iconImage.x = _contentBackground.x + (_contentBackground.width - titleWidth) / 2;
            iconImage.y = _titleLabel.y + (_titleLabel.height - iconImage.height) / 2;
            _titleLabel.x = iconImage.x + iconImage.width + 5;

            _messageLabel.y = _headerBorder.y + 15;

            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);

            _loginBtn.x = _contentBackground.x + _contentBackground.width /2 - (_loginBtn.width + 10);
            _loginBtn.y = _contentBackground.y + _contentBackground.height - (_loginBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 10;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class SubmitChallengePopup extends InfoDialog{
    private var _closeBtn:Button;
    private var _submitBtn:Button;
    public function SubmitChallengePopup(){
        super (241, 139);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "SUBMIT YOUR BUILD";
        _titleLabel.validate();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.2;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Do you want to submit your\nbuild for voting?";
        addChild(_messageLabel);
        _messageLabel.validate();

        var submitLabel:String = "SUBMIT";
        var btnWidth:Number = 71;
        if(data && data.indexOf("filter") != -1){
            submitLabel = "EDIT & SUBMIT";
            btnWidth = 100;
        }

        _submitBtn = ButtonFactory.defaultButton(submitLabel, 0x187499);
        _submitBtn.width = btnWidth;
        _submitBtn.height = 23;
        _submitBtn.paddingLeft = _submitBtn.paddingRight = 5;
        _submitBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
        _submitBtn.addEventListener(Event.TRIGGERED, submitSelectedHandler);
        addChild(_submitBtn);
        _submitBtn.validate();


        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CANCEL";
        _closeBtn.width = btnWidth;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function submitSelectedHandler(event:Event):void {
        _closeHandler = function():void{
            self.dispatchEvent(new ChallengeAlertEvent(ChallengeAlertEvent.SUBMIT_TRIGGERED));
        }
        close();
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;

            _messageLabel.y = _headerBorder.y + 10;

            _submitBtn.x = _contentBackground.x + _contentBackground.width /2 - (_submitBtn.width + 6.5);
            _submitBtn.y = _contentBackground.y + _contentBackground.height - (_submitBtn.height / 2 + 30);

            _closeBtn.x = _contentBackground.x + _contentBackground.width /2 + 6.5;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}

class WelcomeUserPopup extends InfoDialog{
    private var _closeBtn:Button;
    public function WelcomeUserPopup(){
        super (380, 215);
    }

    override protected function initialize():void{
        super .initialize();

        _titleLabel.text = "WELCOME TO THE UPDATED 3DTUNING!";
        _titleLabel.validate();

        var textFormat:TextFormat = new TextFormat();
        textFormat.letterSpacing = -0.2;
        textFormat.leading = -1.5;
        _messageLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.MontserratHtml, 11, 0x535353, TextFormatAlign.CENTER, true, true, textFormat);
        _messageLabel.text = "Brand new \"Tuning Challenges\" and major updates to the\n"+
                "\"Tuning Configurator\" are available in this update!\n\n" +
                "Also we have added "+UIStrings.getKeyChar()+" <b> 30 Keys</b> and "+UIStrings.getDiamondChar()+" <b> 12 Diamonds</b> to\n"+
        "your account, so you can start your tuning journey\nstraight away!\n\n"+
        "Hope you will enjoy this update!";
        addChild(_messageLabel);
        _messageLabel.validate();

        _closeBtn = new Button();
        var closeImg:Image = new Image(Texture.fromTexture(ButtonFactory.assetManager.getTexture("button_border"), null, null, false, 2.5));
        closeImg.color = 0x4a4a4a;
        closeImg.scale9Grid = new Rectangle(12.5,0,closeImg.texture.frameWidth - 25, closeImg.texture.frameHeight);
        _closeBtn.defaultSkin = closeImg;
        _closeBtn.label = "CLOSE";
        _closeBtn.width = 90;
        _closeBtn.height = 23;
        _closeBtn.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0x424242, TextFormatAlign.CENTER, false);
        addChild(_closeBtn);
        _closeBtn.validate();
        _closeBtn.addEventListener(Event.TRIGGERED, closeSelectedHandler);
    }

    private function closeSelectedHandler(event:Event):void {
        close();
    }

    override protected function draw():void{
        super .draw();
        if(stage){
            _titleLabel.x = _contentBackground.x + (_contentBackground.width - _titleLabel.width) / 2;

            _messageLabel.y = _headerBorder.y + 15;

            _closeBtn.x = _contentBackground.x + (_contentBackground.width - _closeBtn.width) / 2;
            _closeBtn.y = _contentBackground.y + _contentBackground.height - (_closeBtn.height / 2 + 30);
        }
    }

    override public function dispose():void{
        super .dispose();
    }
}