package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LayoutImage;
    import com.utils.FontSelector;

    import feathers.controls.Label;

    import flash.display.StageOrientation;

    import flash.geom.Rectangle;

    import flash.text.TextFormatAlign;

    import starling.display.Image;

    import starling.display.Quad;
    import starling.textures.Texture;

    public class ChallengeLeaderboardItemRenderer extends ChallengeItemRenderer {
    private var _topDesignLabel:Label;
    public var isWideScreen:Boolean;
    public var isUltraWideScreen:Boolean;
    public function ChallengeLeaderboardItemRenderer() {
        super();
    }

    override protected function draw():void {
        super.draw();
        var dataInvalid:Boolean = isInvalid(INVALIDATION_FLAG_DATA);
        if(dataInvalid) {
            commitData();
            if(_topDesignLabel){
                _topDesignLabel.y = 6;
                _titleLabel.y = _topDesignLabel.y + _topDesignLabel.height;
            }

            _titleLabel.validate();
            if(actionLabel){
                actionLabel.validate();
                actionLabel.x = actualWidth - (actionLabel.width + 15);
                actionLabel.y = actualHeight - (actionLabel.height + 5);
            }

            updateOrientationDependantView();
        }
        autoSizeIfNeeded();
    }

    override public function dispose():void{
        super .dispose();
        if(_topDesignLabel){
            _topDesignLabel.removeFromParent(true);
            _topDesignLabel = null;
        }
    }

    private function updateOrientationDependantView():void{
        if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isFirstVisibleItem()){
            _topDesignLabel.paddingLeft = 36;
        }else{
            _topDesignLabel.paddingLeft = 12;
        }
        _titleLabel.paddingLeft = _topDesignLabel.paddingLeft;
    }

    override protected function commitData():void{
        super .commitData();
        if(_data){
            _titleLabel.height = Number.NaN;
            _titleLabel.text = _data.name;
            _titleLabel.validate();
        }
    }

    override protected function initialize():void{
        super .initialize();
        _topDesignLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 6.5, 0xFFFFFF, TextFormatAlign.LEFT, false);
        _topDesignLabel.paddingLeft = 12;
        _topDesignLabel.paddingRight = 5;
        _topDesignLabel.paddingTop = 1.5;
        _topDesignLabel.height = 11.25;
        _topDesignLabel.text = "TOP DESIGNS";
        var topBg:Quad = new Quad(1,1, 0xfba500);
        _topDesignLabel.backgroundSkin = topBg;
        addChild(_topDesignLabel);
        _topDesignLabel.validate();


        _titleLabel.paddingTop = 0;
        _titleLabel.paddingBottom = 0;
        if(isWideScreen){
            _topDesignLabel.paddingLeft = 8;
            _titleLabel.width = actualWidth * 9 / 10;
        }else{
            _topDesignLabel.paddingLeft = 12;
            _titleLabel.width = actualWidth * 5 / 6;
        }
        _topDesignLabel.paddingRight = 5;
        _titleLabel.paddingLeft = _topDesignLabel.paddingLeft;
        _titleLabel.paddingRight = _topDesignLabel.paddingRight;
        _titleLabel.wordWrap = true;
        _titleLabel.includeInLayout = false;
        _titleLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitleLabel;
        _titleLabel.backgroundSkin = new Quad(1, 1, 0x000000);
        _titleLabel.backgroundSkin.alpha = 0.75;

        var badgeTxt:Texture = ButtonFactory.assetManager.getTexture("button_border");
        Image.bindScale9GridToTexture(badgeTxt, new Rectangle(badgeTxt.width/3, 0, badgeTxt.width/3, badgeTxt.height));
        var badgeBg:Image = new Image(badgeTxt);
        badgeBg.color = 0xFFFFFE;
        actionLabel.backgroundSkin = badgeBg;
        actionLabel.includeInLayout = false;
        _labelContainer.addChild(actionLabel);

        actionLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansSemiBold(), 13, 0xFFFFFF, TextFormatAlign.CENTER, false);
        actionLabel.includeInLayout = false;
        actionLabel.width = 40;
        actionLabel.height = 15;
        actionLabel.paddingTop = -1;
        _labelContainer.addChild(actionLabel);
        actionLabel.text = "VIEW";
    }
}
}