package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;

import feathers.core.FeathersControl;

import starling.display.Canvas;
import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.filters.DropShadowFilter;

public class ChallengeTuningAvatar extends FeathersControl {
    private var _tuning:Tuning;
    private var _userAvatar:UserCircularAvatar;
    private var _vsTuningLoader:VsImageLoader;
    private var _vsTuningImage:Image;

    private var circleBorder:Canvas;
    private var maskImage:Canvas;
    private static const SIZE:int = 100;
    private static const BORDER_SIZE:int = 2;
    private static const GAP:int = 5;
    private var _loadComplete:Function;
    //
    public function ChallengeTuningAvatar() {
        super();
        width = height = SIZE;
        addEventListener(TouchEvent.TOUCH, touchHandler);
    }

    private function touchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        if(touch){
            dispatchEvent(new Event(Event.TRIGGERED));
        }
    }

    public function get tuning():Tuning {
        return _tuning;
    }

    override protected function initialize():void {
        super.initialize();
        circleBorder = new Canvas();
        circleBorder.beginFill(0xFFFFFF, 1);
        circleBorder.drawCircle(SIZE/2, SIZE/2, SIZE/2);
        circleBorder.endFill();
        circleBorder.filter = new DropShadowFilter(0, 0);

        maskImage = new Canvas();
        maskImage.beginFill();
        maskImage.drawCircle(SIZE/2, SIZE/2, SIZE/2-BORDER_SIZE);
        maskImage.endFill();
    }

    public function set tuning(value:Tuning):void {
        _tuning = value;
        if(_vsTuningLoader){
            _vsTuningLoader.removeFromParent(true);
        }
        if(_vsTuningImage){
            _vsTuningImage.mask = null;
            _vsTuningImage.removeFromParent();
            _vsTuningImage.texture.dispose();
            _vsTuningImage.dispose();
            _vsTuningImage = null;
        }
        var url:String = getTuningSnapshots(_tuning)[0];
        if(url) {
            if (url.indexOf("http:") != 0) {
                url = "http:" + url;
            }
            _vsTuningLoader = new VsImageLoader();
            _vsTuningLoader.source = url;
            _vsTuningLoader.addEventListener(Event.COMPLETE, vsImageCompleteHandler);
            addChild(_vsTuningLoader);
            _userAvatar = new UserCircularAvatar(new User(_tuning.userObj), 30, true, 1.5);
        }
    }

    override protected function draw():void {
        super.draw();
        if(_userAvatar){
            _userAvatar.x = actualWidth - (_userAvatar.width + GAP);
            _userAvatar.y = GAP;
        }

    }

    private function vsImageCompleteHandler(event:Event):void {
        _vsTuningLoader.removeFromParent();
        _vsTuningImage = new Image(_vsTuningLoader.texture);
        _vsTuningImage.scale = 2 * SIZE/_vsTuningImage.width;
        _vsTuningImage.x = (circleBorder.width-_vsTuningImage.width)/2;
        _vsTuningImage.y = (circleBorder.height-_vsTuningImage.height)/2;

        addChild(circleBorder);
        addChild(_vsTuningImage);
        addChild(maskImage);
        addChild(_userAvatar);
        _vsTuningImage.mask = maskImage;
        invalidate(INVALIDATION_FLAG_ALL);
        if(_loadComplete!=null){
            _loadComplete.call();
        }
    }

    private static function getTuningSnapshots(tuning:Tuning):Array{
        return (tuning.fullHdSnapshots && tuning.fullHdSnapshots.length)?tuning.fullHdSnapshots:tuning.snapshots;
    }


    override public function dispose():void {
        super.dispose();
        _tuning = null;
        if(_userAvatar){
            _userAvatar.removeFromParent(true);
            _userAvatar = null;
        }
        if(_vsTuningLoader){
            _vsTuningLoader.removeFromParent(true);
            _vsTuningLoader = null;
        }
        if(_vsTuningImage){
            _vsTuningImage.removeFromParent(true);
            _vsTuningImage = null;
        }
        if(circleBorder){
            circleBorder.removeFromParent(true);
            circleBorder = null;
        }
        if(maskImage){
            maskImage.removeFromParent(true);
            maskImage = null;
        }
    }

    public function set loadComplete(value:Function):void {
        _loadComplete = value;
    }
}
}

import feathers.controls.ImageLoader;

import starling.textures.Texture;

class VsImageLoader extends ImageLoader{
    public function VsImageLoader(){
        super ();
    }

    public function get texture():Texture{
        return _texture;
    }
}