package com.despark.tuning3d.view.components.challenges.controls
{
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.ChallengeUser;
    import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
    import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
    import com.utils.ObjectUtils;

    import feathers.core.FeathersControl;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;

    import flash.geom.Point;

    import starling.core.Starling;

    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class ChallengeTuningView extends FeathersControl implements ITuningListMenuVIew {
        private var _snapshots : Array;
        private var _bg:Quad;
        private var _params : Object;

        private var _tuning:Tuning;
        private var _user:ChallengeUser;
        private var _profileLabel:ChallengeUserLabel;
        private var _tuningView:TuningItemRenderer;
        private var _contentScale:Number = 1;
        public static const INDEX_CHANGED:String = "indexChanged";

        public function ChallengeTuningView(params:Object){
            super();
            if (params){
                if(params.tuning.car is Car){
                    params.tuning.car.isChallenge = true;
                }else{
                    params.tuning.car.challengeNewSystemCar = true;
                }
                _params = params;
                if(_params.tuning is Tuning){
                    _tuning = _params.tuning;
                }else if(_params.tuning is Object){
                    _tuning = TuningsProxy.parseTuning(_params.tuning);
                }

                if(_params.user is ChallengeUser){
                    _user = _params.user as ChallengeUser;
                }else if(_tuning && _tuning.userObj && !(_tuning.userObj is String)){
                    _user = new ChallengeUser(_tuning.userObj);
                }else if(_params.user && !(_params.user is String)){
                    _user = new ChallengeUser(_params.user);
                }
                _snapshots = getTuningSnapshots(_tuning);
            }
        }

        public function set page(value:int):void{
            if(_tuningView && _tuningView.pageIndicator && _tuningView.pageIndicator.pageCount > value){
                _tuningView.pageIndicator.selectedIndex = value;
            }
        }

        public function set params(value:Object):void{
            _params = value;

            _snapshots = getTuningSnapshots(_params.senderTuning);
            invalidate(INVALIDATION_FLAG_ALL);
        }

        override protected function initialize():void {
            super.initialize();
            _bg  = new Quad(1,1,0xffffff);
            addChild(_bg);
            _bg.alpha = 1;

            var layout : HorizontalLayout = new HorizontalLayout();
            layout.gap = 0;
            layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.verticalAlign = VerticalAlign.MIDDLE;

            _tuningView = new TuningItemRenderer();
            _tuningView.addEventListener(Event.CHANGE, tuningViewChangeHandler);
            _tuningView.pageChangeDelay = 0;
            _tuningView.scale = _contentScale;
            addChild(_tuningView);
            _tuningView.pageIndicator.visible = false;
            _tuningView.data = _tuning;
            var contentOffset:Number = 50;
            if(ObjectUtils.getValue(_params, "featuresRules.customFrame", false) || ObjectUtils.getValue(_params, "stats.challengeHostId.featuresRules.customFrame", false)){
                contentOffset = 0;
            }
            _tuningView.leftOffset = contentOffset;
            _tuningView.setSize(stage.stageWidth / _contentScale, stage.stageHeight / _contentScale);
            _tuningView.y = 0;


            _profileLabel = new ChallengeUserLabel();
            _profileLabel.user = _user;
            if(!_params || !_params.hasOwnProperty("type") || _params["type"] != "challenge_new_diamond"){
                addChild(_profileLabel);
            }
            _tuningView.addEventListener(TuningItemRenderer.ZOOM_MODE_START, zoomModeStartHandler);
            _tuningView.addEventListener(TuningItemRenderer.ZOOM_MODE_END, zoomModeEndHandler);
            addEventListener(TouchEvent.TOUCH, tuningItemTouchHandler);
        }

        private function zoomModeEndHandler(event:Event):void {

        }

        private function zoomModeStartHandler(event:Event):void {

        }

        private function tuningItemTouchHandler(event:TouchEvent):void {
            if(event.touches && event.touches.length > 1){
                var touchA:Touch = event.touches[0];
                var touchB:Touch = event.touches[1];

                var currentPosA:Point = touchA.getLocation(parent);
                var previousPosA:Point = touchA.getPreviousLocation(parent);
                var currentPosB:Point = touchB.getLocation(parent);
                var previousPosB:Point = touchB.getPreviousLocation(parent);

                var currentVector:Point = currentPosA.subtract(currentPosB);
                var previousVector:Point = previousPosA.subtract(previousPosB);
                var sizeDiff:Number = currentVector.length / previousVector.length;
                //
                if(sizeDiff > 1){
                    _tuningView.startZoomMode();
                    _tuningView.zoom(event.touches);
                }
            }else{
                var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
                if(touch && touch.tapCount > 1){
                    _tuningView.startZoomMode();
                }
            }
        }

        private function tuningViewChangeHandler(event:Event):void {
            dispatchEventWith(ChallengeTuningView.INDEX_CHANGED, false, {page:_tuningView.pageIndicator.selectedIndex});
        }


    private static function getTuningSnapshots(tuning:Tuning):Array{
        return (tuning.fullHdSnapshots && tuning.fullHdSnapshots.length)?tuning.fullHdSnapshots:tuning.snapshots;
    }

    override protected function draw():void {
        super.draw();
        if(isInvalid(INVALIDATION_FLAG_SIZE)) {
            _bg.width = actualWidth;
            _bg.height = actualHeight;
            _tuningView.y = 0;
            if(_profileLabel){
                _profileLabel.x = 10;
                _profileLabel.y = 10;
            }
        }
    }

    public function getVisibleTuningURL():String {
        return "";
    }

    override public function dispose():void {
        super.dispose();
        if(_tuningView){
            _tuningView.removeFromParent(true);
            _tuningView = null;
        }
        if(_profileLabel){
            _profileLabel.removeFromParent(true);
            _profileLabel = null;
        }
        if(_bg){
            _bg.removeFromParent(true);
            _bg = null;
        }
    }

        public function set contentScale(value:Number):void {
            _contentScale = value;
            if(_tuningView){
                _tuningView.scale = _contentScale;
                _tuningView.setSize((stage.stageWidth + 50) / _contentScale, stage.stageHeight / _contentScale);
            }
        }
    }
}