package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.users.ChallengeUser;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.FontSelector;
    import com.utils.NumberUtils;

    import feathers.controls.Label;

    import feathers.controls.LayoutGroup;

    import flash.text.TextFormatAlign;

    import starling.core.Starling;
    import starling.display.Image;

    import starling.display.Quad;

    public class ChallengeVoteResultView extends LayoutGroup {
        private var _avatar1:UserCircularAvatar;
        private var _avatar2:UserCircularAvatar;
        private var _user1Label:Label;
        private var _user2Label:Label;
        private var _rating1Label:Label;
        private var _rating2Label:Label;
        private var ratingBar1:RatingBar;
        private var ratingBar2:RatingBar;
        private var _lastVotedObject:Object;

        private var _votedForCallout:Image;
        private var _voteBackground:Quad;
        private var _voteBorder:Quad;
        private var _voteBorderMask:Quad;
        public function ChallengeVoteResultView() {
            super();
            backgroundSkin = new Quad(1,1,0x000000);
            backgroundSkin.alpha = 0.65;
        }

        public function loadData(value:Object, lastVotedObject:Object):void{
            _lastVotedObject = lastVotedObject;
            alpha = 0;
            if(_avatar1){
                _avatar1.removeFromParent(true);
            }
            if(_avatar2){
                _avatar2.removeFromParent(true);
            }
            _voteBackground ||= new Quad(1, 1, 0x000000);
            _voteBackground.alpha = 0.15;
            _voteBackground.width = 170;
            _voteBackground.height = 192;

            _voteBorder ||= new Quad(1, 1, 0xFFFFFF);
            _voteBorder.width = _voteBackground.width;
            _voteBorder.height = _voteBackground.height;

            _voteBorderMask ||= new Quad(1, 1, 0xFFFFFF);
            _voteBorderMask.width = _voteBorder.width - 2;
            _voteBorderMask.height = _voteBorder.height - 2;

            var user1:User = new User(value.tuning_one.user);
            var user2:User = new User(value.tuning_two.user);
            _avatar1 = new UserCircularAvatar(user1, 51, true, 1);
            _avatar2 = new UserCircularAvatar(user2, 51, true, 1);
            _user1Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 13, 0xFFFFFF, TextFormatAlign.CENTER, false);
            _user2Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 13, 0xFFFFFF, TextFormatAlign.CENTER, false);
            _user1Label.text = user1.nameToShow;
            _user2Label.text = user2.nameToShow;

            _rating1Label = LabelFactory.createSimpleLabel(FontSelector.getGoldBold(), 21, 0xFFFFFF, TextFormatAlign.CENTER, false);
            _rating2Label = LabelFactory.createSimpleLabel(FontSelector.getGoldBold(), 21, 0xFFFFFF, TextFormatAlign.CENTER, false);

            _rating1Label.text = NumberUtils.toFixed(parseFloat(value.tuning_one.rating), 2);
            _rating2Label.text = NumberUtils.toFixed(parseFloat(value.tuning_two.rating), 2);

            ratingBar1 = new RatingBar();
            ratingBar1.voteValue = value.tuning_one.rating;

            ratingBar2 = new RatingBar();
            ratingBar2.voteValue = value.tuning_two.rating;

            addChild(_voteBackground);
            addChild(_voteBorder);
            addChild(_voteBorderMask);

            addChild(_avatar1);
            addChild(_user1Label);
            addChild(ratingBar1);
            addChild(_rating1Label);

            addChild(_avatar2);
            addChild(_user2Label);
            addChild(ratingBar2);
            addChild(_rating2Label);
            invalidate(INVALIDATION_FLAG_SIZE);
            Starling.juggler.tween(this, 0.25, {alpha: 1});


            _votedForCallout = new Image(ButtonFactory.assetManager.getTexture("you_voted_for_callout"));
            _votedForCallout.scale = ButtonFactory.assetManager.scaleFactor / 2;
            _votedForCallout.visible = false;
            _votedForCallout.color = 0xFF0000;
            addChild(_votedForCallout);

            ratingBar1.animate();
            ratingBar2.animate();
        }

        override protected function draw():void{
            super .draw();

            _voteBackground.y = (actualHeight - _voteBackground.height) * 2 / 5;
            _voteBorder.y = _voteBackground.y;

            _voteBorderMask.y = _voteBorder.y + 1;
            _voteBorder.mask = _voteBorderMask;
            _voteBorder.maskInverted = true;

            _votedForCallout.y = _voteBorder.y + 17;

            if(_avatar1 && _avatar2){
                _avatar1.validate();
                _avatar2.validate();

                _avatar1.x = (actualWidth / 2 - _avatar1.width) / 2;
                _avatar1.y = _voteBorder.y + 50;

                _avatar2.x = actualWidth / 2 + (actualWidth / 2 - _avatar2.width) / 2;
                _avatar2.y = _voteBorder.y + 50;

                if(_user1Label){
                    _user1Label.validate();
                    _user1Label.x = _avatar1.x + (_avatar1.width - _user1Label.width) / 2;
                    _user1Label.y = _avatar1.y + _avatar1.height + 3;

                    ratingBar1.validate();
                    ratingBar1.x = _avatar1.x + (_avatar1.width - ratingBar1.width) / 2;
                    ratingBar1.y = _user1Label.y + _user1Label.height + 4;

                    _rating1Label.validate();
                    _rating1Label.x = ratingBar1.x + (ratingBar1.width - _rating1Label.width) / 2;
                    _rating1Label.y = ratingBar1.y + ratingBar1.height + 2;
                }
                if(_user2Label){
                    _user2Label.validate();
                    _user2Label.x = _avatar2.x + (_avatar2.width - _user2Label.width) / 2;
                    _user2Label.y = _avatar2.y + _avatar2.height + 3;

                    ratingBar2.validate();
                    ratingBar2.x = _avatar2.x + (_avatar2.width - ratingBar2.width) / 2;
                    ratingBar2.y = _user2Label.y + _user2Label.height + 4;

                    _rating2Label.validate();
                    _rating2Label.x = ratingBar2.x + (ratingBar2.width - _rating2Label.width) / 2;
                    _rating2Label.y = ratingBar2.y + ratingBar2.height + 2;
                }
                _votedForCallout.visible = true;
                if(_lastVotedObject.tuning_one.voted && _lastVotedObject.tuning_two.voted){
                    _votedForCallout.visible = false;
                    _voteBackground.visible = false;
                    _voteBorder.visible = false;
                }else if(_lastVotedObject.tuning_one.voted){
                    _votedForCallout.x = _avatar1.x + (_avatar1.width - _votedForCallout.width) / 2;
                    _voteBackground.x = _avatar1.x + (_avatar1.width - _voteBackground.width) / 2;
                }else{
                    _votedForCallout.x = _avatar2.x + (_avatar2.width - _votedForCallout.width) / 2;
                    _voteBackground.x = _avatar2.x + (_avatar2.width - _voteBackground.width) / 2;
                }
            }

            _voteBorder.x = _voteBackground.x;
            _voteBorderMask.x = _voteBorder.x + 1;
        }
    }
}

import com.despark.tuning3d.view.components.core.ButtonFactory;

import feathers.controls.LayoutGroup;

import flash.utils.setTimeout;

import starling.core.Starling;

import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;

class RatingBar extends LayoutGroup{
    private var _start:Vector.<Image> = new Vector.<Image>(5);
    private var _starBorders:Vector.<Image> = new Vector.<Image>(5);
    private var _voteValue:Number;
    private var lastMask:Quad;
    public function RatingBar(){
        var startTxt:Texture = ButtonFactory.assetManager.getTexture("challenge_stars_icon");
        var startBorderTxt:Texture = ButtonFactory.assetManager.getTexture("vote_start_borders");
        for(var i:int = 0; i < _start.length; i++){
            _starBorders[i] = new Image(startBorderTxt);
            _starBorders[i].color = 0xFF0000;
            _starBorders[i].width = 14;
            _starBorders[i].height = 13;
            _starBorders[i].alpha = 0;
            addChild(_starBorders[i]);

            _start[i] = new Image(startTxt);
            _start[i].color = 0xFF0000;
            _start[i].width = 14;
            _start[i].height = 12;
            _start[i].alpha = 0;
            addChild(_start[i]);
        }
    }

    public function set voteValue(value:Number):void{
        _voteValue = value;
    }

    public function animate(star:Image = null, border:Image = null):void{
        var index:int;
        if(star && border){
            index = _start.indexOf(star);
            if(index != -1 && index + 1 < _start.length){
                index++;
            }else{
                return;
            }
        }else{
            index = 0;
        }
        star = _start[index];
        border = _starBorders[index];
        Starling.juggler.tween(star, 0.2, {alpha: 1});
        Starling.juggler.tween(border, 0.2, {alpha: 1});
        setTimeout(animate, 75, star, border);
    }

    override protected function draw():void{
        super.draw();
        var newWidth:Number = 0;
        var visibleNum:int = int(_voteValue) - 1;
        for(var i:int = 1; i < _start.length; i++){
            _starBorders[i].x = _starBorders[i - 1].x + _starBorders[i - 1].width + 4;
            _start[i].x = _start[i - 1].x + _start[i - 1].width + 4;
            newWidth = _start[i].x + _start[i].width;
            _start[i].visible = (i <= visibleNum);
            if(i == visibleNum + 1 && _voteValue > visibleNum){
                _start[i].visible = true;
                lastMask ||= new Quad(1,1,0xFF0000);
                lastMask.width = _start[i].width * (_voteValue % 1);
                lastMask.height = _start[i].height;
                lastMask.x = _start[i].x;
                lastMask.y = _start[i].y;
                if(!contains(lastMask)){
                    addChild(lastMask);
                    _start[i].mask = lastMask;
                }
            }
        }
        if(width != newWidth){
            setSize(newWidth, height);
        }
    }
}