package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.utils.Console;

    import feathers.controls.Button;

    import feathers.controls.Label;
    import feathers.core.FeathersControl;

    import flash.events.TextEvent;

    import flash.geom.Rectangle;

    import starling.animation.Transitions;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.events.EventDispatcher;

    public class ChallengesAlert extends FeathersControl{
    public static const LIMIT:String = "limit";
    public static const REWARDS:String = "rewards";
    public static const DEFAULT:String = "default";
    public static const MINE_IN_CHALLENGE:String = "mineInChallenge";
    public static const OPPONENT_IN_CHALLENGE:String = "opponentInChallenge";
    private static var _instance:ChallengesAlert;
    private static var _propertyObjects:Object;

    private var _alertType:String;
    private var _borders:Image;
    private var _borderContainer:Sprite;
    private var _iconImage:Image;
    private var _titleLabel:Label;
    private var _bodyLabel:Label;
    private var _closeButton:Button;
    private var _challengesButton:Button;
    private var _eventDispatcher:EventDispatcher;
    private var _message:String;
    private var _closeHandler:Function;
    public function ChallengesAlert(alertType:String, message:String = null) {
        width = 184;
        height = 120;
        this._alertType = alertType;
        _propertyObjects ||= getPropertyObject();
        _message = message;
    }

    public static function show(alertType:String, eventDispatcher:EventDispatcher = null, message:String = null, closeHandler:Function = null):void{
        if(_instance){
            _instance.close();
        }
        _instance = new ChallengesAlert(alertType, message);
        _instance._eventDispatcher = eventDispatcher;
        _instance._closeHandler = closeHandler;
        _instance.open();
    }

    public function open():void{
        alpha = 0;
        Starling.current.stage.addChild(this);
        x = (Starling.current.stage.stageWidth - width) / 2;
        y = (Starling.current.stage.stageHeight - height) / 2;
        Starling.juggler.tween(this, 0.1, {alpha: 1, transition: Transitions.LINEAR});
    }

    public function close():void{
        Starling.juggler.tween(this, 0.25,
                {
                    alpha: 0,
                    transition: Transitions.LINEAR,
                    onComplete:function ():void {
                removeFromParent(true);
                        if(_closeHandler != null){
                            _closeHandler();
                            _closeHandler = null;
                        }
                    }
                });
    }

    protected override function initialize():void{
        super.initialize();
        _borderContainer = new Sprite();
        _borders = new Image(ButtonFactory.assetManager.getTexture("challenge_alert_background"));
        _borders.scale9Grid = new Rectangle(_borders.texture.frameWidth / 2 - 0.5, _borders.texture.frameHeight / 2 - 0.5, 1, 1);
        _borderContainer.addChild(_borders);
        addChild(_borderContainer);

        var iconId:String = _propertyObjects[_alertType].icon;
        if(iconId){
            _iconImage = new Image(ButtonFactory.assetManager.getTexture(iconId));
            _iconImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
            addChild(_iconImage);
        }

        _titleLabel = new Label();
        _titleLabel.textRendererFactory = LabelFactory.challengesAlertTitleLabel;
        _titleLabel.text = _propertyObjects[_alertType].title;
        addChild(_titleLabel);
        _titleLabel.validate();

        _bodyLabel = new Label();
        _bodyLabel.wordWrap = true;
        _bodyLabel.textRendererFactory = LabelFactory.challengesAlertBodyLabel;
        _bodyLabel.text = _message?_message:_propertyObjects[_alertType].body;
        addChild(_bodyLabel);
        _bodyLabel.validate();
        _bodyLabel.addEventListener(Event.TRIGGERED, linkEventHandler);

        if(_alertType == REWARDS){
            _challengesButton = new Button();
            var challengeSkin:Quad = new Quad(1, 1, 0xC10000);
            challengeSkin.alpha = 0;
            _challengesButton.defaultSkin = challengeSkin;
            _challengesButton.width = 65;
            _challengesButton.height = 25;
            addChild(_challengesButton);
            _challengesButton.addEventListener(Event.TRIGGERED, challengesTriggeredHandler);
        }

        _closeButton = new Button();
        var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture("details_close"));
        closeIcon.scale = 0.75 * ButtonFactory.assetManager.scaleFactor/2;
        var closeSkin:Quad = new Quad(1,1,0xFFFFFF);
        closeSkin.width = closeSkin.height = 30;
        closeSkin.alpha = 0;
        _closeButton.width = closeIcon.width;
        _closeButton.height = closeIcon.height;
        _closeButton.defaultIcon = closeIcon;
        _closeButton.defaultSkin = closeSkin;
        addChild(_closeButton);
        _closeButton.validate();
        _closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);
    }

        private function challengesTriggeredHandler(event:Event):void {
            if(_eventDispatcher){
                _eventDispatcher.dispatchEventWith(StackedScreen.CHALLENGES_SELECTED);
            }
            close();
        }

        private function linkEventHandler(event:Event):void {
            Console.log(event.data);
        }

        private function closeButtonTriggeredHandler(event:Event):void {
            close();
        }

    override protected function draw():void {
        super.draw();
        var scaleFactor:Number = ButtonFactory.assetManager.scaleFactor / 2;
        _borders.width = actualWidth / scaleFactor;
        _borders.height = actualHeight / scaleFactor;
        _borderContainer.scale = scaleFactor;

        _iconImage.x = (actualWidth - _iconImage.width) / 2;
        _iconImage.y = 7;

        _titleLabel.x = (actualWidth - _titleLabel.width) / 2;
        _titleLabel.y = _iconImage.y + _iconImage.height;

        _bodyLabel.width = actualWidth - 20;
        _bodyLabel.validate();
        _bodyLabel.x = (actualWidth - _bodyLabel.width) / 2;
        _bodyLabel.y = _titleLabel.y + _titleLabel.height + 7;

        _closeButton.x = actualWidth - _closeButton.width - 5;
        _closeButton.y = 5;

        if(_challengesButton){
            _challengesButton.x = _bodyLabel.x + 45;
            _challengesButton.y = _bodyLabel.y + 35;
        }
    }

    override public function dispose():void {
        super.dispose();
    }

    private static function getPropertyObject():Object{
        var result:Object = {};
        result[LIMIT] = {
            icon: "challenge_alert_limit_Icon",
            title: "Limit Alert!",
            body: 'You have reached the maximum allowed amount of active <strong>challenges <font color="#b81919">10!</font></strong><br/><br/>Please wait for at least 1 of your \n' +
                    'active challenges to finish.'
        };

        result[REWARDS] = {
            icon: "challenge_alert_rewards_Icon",
            title: "Not enough Rewards Votes",
            body: 'You don\'t have enough Rewards Votes to start/accept new challenge<br/><br/>Go to <u><a href="event:challenges"><b>CHALLENGES</b></a></u> for vote'
        };

        result[MINE_IN_CHALLENGE] = {
            icon: "challenge_alert_default_Icon",
            title: "Alert!",
            body: 'Your vehicle is already<br/>' +
                    'in Challenge!<br/>' +
                    '<br/>' +
                    'Please wait for that challenge to<br/>' +
                    'finish or choose another vehicle'
        };

        result[OPPONENT_IN_CHALLENGE] = {
            icon: "challenge_alert_default_Icon",
            title: "Alert!",
            body: "Opponent\'s vehicle is already<br/>" +
                    "in Challenge!<br/>" +
                    "<br/>" +
                    "Please wait for that challenge to<br/>" +
                    "finish or choose another vehicle"
        };
        return result;
    }

        public static function get instance():ChallengesAlert {
            return _instance;
        }

        public function get eventDispatcher():EventDispatcher {
            return _eventDispatcher;
        }
    }
}
