package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.view.components.core.ExtendedList;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.DateParser;
    import com.utils.NumberUtils;
    import com.utils.ObjectUtils;

    import feathers.controls.Label;

    import flash.display.StageOrientation;

    import starling.display.Quad;

    public class ChallengesUserTuningItemRenderer extends ChallengesTuningItemRenderer {
    private var _titleLabel:Label;
    private var _statusLabel:ChallengeStatusInVoting;
    private var _challengeData:ChallengeData;
    private var _userGarage:Boolean;
        private var _background:Quad;
    public function ChallengesUserTuningItemRenderer() {
        super ();
    }

    override protected function initialize():void{
        super .initialize();
        if(nameBg){
            nameBg.removeFromParent(true);
            nameBg = null;
        }
        _titleLabel = new Label();
        _titleLabel.minWidth = actualWidth / 2;
        _titleLabel.paddingLeft = _titleLabel.paddingRight = 12;
        _titleLabel.wordWrap = false;
        _titleLabel.includeInLayout = false;
        _titleLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitleLabel;
        _titleLabel.backgroundSkin = new Quad(1, 1, 0x000000);
        _titleLabel.backgroundSkin.alpha = 0.75;
        addChild(_titleLabel);

        _statusLabel = new ChallengeStatusInVoting();
        addChild(_statusLabel);
    }

    override protected function commitData():void{
        super .commitData();
        if(_data){
            _challengeData = new ChallengeData(ObjectUtils.getProperty(_data,"stats.challengeHostId"));
            if(_data.user){
                if(_avatar){
                    _avatar.user = _data.user;
                }else{
                    _avatar = new UserCircularAvatar(_data.user, 26, true, 1);
                }
                _usernameLabel.text = _avatar.user.nameToShow;
                _usernameLabel.validate();
            }else if(_avatar){
                _avatar.removeFromParent(true);
                _avatar = null;
                if(_usernameLabel){
                    _usernameLabel.removeFromParent(true);
                    _usernameLabel = null;
                }
            }
            if(_ratingLabel){
                _ratingLabel.text = NumberUtils.toFixed(parseFloat(_data.stats.rating),2);
            }
            if(_titleLabel){
                _titleLabel.text = tuning.name;
            }
            if(_statusLabel){
                _statusLabel.text = DateParser.getInChallengeLabel(_challengeData.votingFinishDate.time - new Date().time).toString();
            }
            if(!_userGarage && _challengeData.status != ChallengeData.FINISHED){
                imageLoader.bitmapScale = 1;
                imageLoader.scale = 1 / 0.4;
                imageLoader.persistentImageProvider = Car.BlankCar;
                imageLoader.completeHandler = defaultImageLoadComplete;
                if(!_background){
                    _background = new Quad(1,1,0x252525);
                    _background.width = actualWidth;
                    _background.height = actualHeight;
                    addChildAt(_background, 0);
                }
            }else if(_background){
                _background.removeFromParent(true);
                _background = null;
            }
        }
    }

    private function defaultImageLoadComplete(loaderWithStates:ImageLoaderWithStates):void{
        loaderWithStates.width = actualWidth - 20;
        loaderWithStates.height = actualHeight - 20;
        loaderWithStates.x = (actualWidth - loaderWithStates.width) / 2;
        loaderWithStates.y = (actualHeight - loaderWithStates.height) / 2 + 20;
        validate();
    }

    override protected function draw():void{
        super .draw();
        var gap:Number = 10;
        if(_titleLabel){
            _titleLabel.textRendererFactory = LabelFactory.challengeLeaderboardTitleLabel;
            _titleLabel.validate();
            _titleLabel.x = 0;
        }
        if(_statusLabel){
            _statusLabel.validate();
            _statusLabel.x = 0;
        }

        if(_titleLabel){
            _titleLabel.y = gap;
        }

        if(_ratingLabel){
            _ratingLabel.y = _titleLabel.height + _titleLabel.y - 3;
            _ratingLabel.visible = (_challengeData.status == ChallengeData.FINISHED);
        }

        if(_statusLabel){
            _statusLabel.y = _titleLabel.y + _titleLabel.height;
            _statusLabel.visible = (_challengeData.status != ChallengeData.FINISHED);
        }

    }

        override public function updateOrientationDependantView():void{
            super .updateOrientationDependantView();
            if(owner is ExtendedList && ExtendedList(owner).orientation == StageOrientation.ROTATED_RIGHT && isLeftSideItem()){
                var notchSize:Number = 20;
                if(_statusLabel){
                    _statusLabel.paddingLeft = _statusLabel.defaultPaddingLeft + notchSize;
                }
                if(_titleLabel && rowIndex == 1){
                    _titleLabel.paddingLeft = 12;
                }else{
                    _titleLabel.paddingLeft = 12 + notchSize;
                }
            }else if(_statusLabel && _titleLabel){
                _statusLabel.paddingLeft = _statusLabel.defaultPaddingLeft;
                _titleLabel.paddingLeft = 12;
            }
        }

        public function set userGarage(userGarage:Boolean):void {
            _userGarage = userGarage;
        }
    }
}