package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.Label;
import feathers.core.FeathersControl;

    import flash.text.TextFormatAlign;

    import starling.core.Starling;
import starling.display.Image;

public class DateLabel extends FeathersControl {
    private static const SECOND:Number = 1;
    private static const MINUTE:Number = 60*SECOND;
    private static const HOUR:Number = 60*MINUTE;
    private static const DEFAULT_DATE:Date = function ():Date {
        var date:Date = new Date();
        date.setTime(0);
        return date;
    }();
    private var _date:Date;
    private var _label:Label;
    private var _icon:Image;
    public function DateLabel() {
        super();
    }

    public function set time(val:Number):void{
        _date ||= new Date();
        _date.setTime(val);
        Starling.juggler.removeDelayedCalls(updateTime);
        var tm:Number = _date.fullYear-DEFAULT_DATE.fullYear;
        if(tm>0){
            _label.text = tm.toString() + "y ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.month-DEFAULT_DATE.month>0){
            tm = _date.month-DEFAULT_DATE.month;
            _label.text = tm.toString() + "m ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.date-DEFAULT_DATE.date>0){
            tm = _date.date-DEFAULT_DATE.date;
            _label.text = tm.toString() + "d ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.hours-DEFAULT_DATE.hours>0){
            tm = _date.hours-DEFAULT_DATE.hours;
            _label.text = tm.toString() + "h ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.minutes-DEFAULT_DATE.minutes>0){
            tm = _date.minutes-DEFAULT_DATE.minutes;
            _label.text = tm.toString() + "min. ago";
            Starling.juggler.delayCall(updateTime, MINUTE, val+1000*MINUTE);
        }else if(_date.seconds-DEFAULT_DATE.seconds>0){
            tm = _date.seconds-DEFAULT_DATE.seconds;
            _label.text = tm.toString() + "sec. ago";
            Starling.juggler.delayCall(updateTime, SECOND, val+1000*SECOND);
        }
        _label.validate();
        validate();
    }
/*

    public function set time(val:Number):void{
        _date ||= new Date();
        _date.setTime(val);
        Starling.juggler.removeDelayedCalls(updateTime);
        var tm:Number = _date.fullYear-DEFAULT_DATE.fullYear;
        if(tm>0){
            _label.text = tm.toString() + " year"+(tm>1?"s":"")+" ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.month-DEFAULT_DATE.month>0){
            tm = _date.month-DEFAULT_DATE.month;
            _label.text = tm.toString() + " month"+(tm>1?"s":"")+" ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.date-DEFAULT_DATE.date>0){
            tm = _date.date-DEFAULT_DATE.date;
            _label.text = tm.toString() + " day"+(tm>1?"s":"")+" ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.hours-DEFAULT_DATE.hours>0){
            tm = _date.hours-DEFAULT_DATE.hours;
            _label.text = tm.toString() + " hour"+(tm>1?"s":"")+" ago";
            Starling.juggler.delayCall(updateTime, HOUR, val+1000*HOUR);
        }else if(_date.minutes-DEFAULT_DATE.minutes>0){
            tm = _date.minutes-DEFAULT_DATE.minutes;
            _label.text = tm.toString() + " min. ago";
            Starling.juggler.delayCall(updateTime, MINUTE, val+1000*MINUTE);
        }else if(_date.seconds-DEFAULT_DATE.seconds>0){
            tm = _date.seconds-DEFAULT_DATE.seconds;
            _label.text = tm.toString() + " sec. ago";
            Starling.juggler.delayCall(updateTime, SECOND, val+1000*SECOND);
        }
        _label.validate();
        validate();
    }
*/

    private function updateTime(val:Number):void{
        time = val;
    }

    override protected function initialize():void {
        super.initialize();
        _icon = new Image(ButtonFactory.assetManager.getTexture("dateIcon"));
        _icon.scale = _icon.texture.scale/2;
        addChild(_icon);
        _label = LabelFactory.createSimpleLabel(FontSelector.getMontserratRegular(), 7.5, 0x58595b, TextFormatAlign.LEFT, false);
        addChild(_label);
    }

    override protected function draw():void {
        super.draw();
        if(_icon && _label){
            _label.validate();
            _label.x = _icon.x + _icon.width + 3;
            width = _label.x + _label.width;
            height = Math.max(_icon.height, _label.height);
            _icon.y = (height - _icon.height) / 2;
            _label.y = (height - _label.height) / 2;
        }
    }

    override public function dispose():void {
        super.dispose();
        Starling.juggler.removeDelayedCalls(updateTime);
        if(_label){
            _label.removeFromParent(true);
            _label = null;
        }
        if(_icon){
            _icon.removeFromParent(true);
            _icon = null;
        }
    }
}
}
