package com.despark.tuning3d.view.components.challenges.modals {
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.challenges.feed.ChallengeNotificationData;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.NotificationView.NotificationFeedData;
    import com.despark.tuning3d.view.components.NotificationView.NotificationPanelViewItemRenderer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotificationItemRenderer;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineTuningView;
    import com.distriqt.AdsManager;

    import feathers.controls.Label;
    import feathers.controls.List;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.TouchEvent;

    public class InAppNotificationsPopup extends PopUp {
    private var _notificationList:List;
    private var _notifications : Array;
    private static const MAX_COUNT:int = 20;
    private static const MAX_LIST_COUNT:int = 80;
    private var rendererWidth : Number;
    private static var _instance:InAppNotificationsPopup;
    //
    private var isLoading:Boolean = false;
    private var requestsLoaded:Boolean = false;
    private var notesLoaded:Boolean = false;
    private var alertView:AlertView;

    private var loadedData:Array;
    private var loadedNotes:Array;
    private var loadedRequests:Array;
    private var requestedTypes:Object;
    private var _showBannerAfterExit:Boolean;
    private var _bannerScreen:String;

    private var _messageLabel : Label;
    private var _onComplete:Function;
    public function InAppNotificationsPopup(data:Object = null) {
        super();
        FeedData.CURRENT_USER = UsersProxy.loggedInUser();
        _showBannerAfterExit = (data && data.hasOwnProperty("showBannerAfterExit") && data.showBannerAfterExit);
        _bannerScreen = (data && data.hasOwnProperty("bannerScreen"))?data["bannerScreen"]:BannerScreen.BANNER_SCREEN;
        AdsManager.$.showBannerAfterComplete = false;
    }

    override protected function initialize():void {
        super.initialize();
        loadedData = [];
        loadedNotes = [];
        loadedRequests = [];
        requestedTypes = {};
        addEventListener(ChallengeEvent.VIEW_TUNING, viewTuningHandler);

        setSizeInternal(this.stage.stageWidth*1, this.stage.stageHeight,false);
        background.color = 0x555555;
        background.alpha = 0.8;
        contentBackgroundImage.visible = false;

        var layout : VerticalLayout = new VerticalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.LEFT;
        layout.verticalAlign = VerticalAlign.TOP;
        layout.useVirtualLayout = true;

        _notificationList = new List();
        _notificationList.dataProvider = new ListCollection();
        _notificationList.layout = layout;
        _notificationList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        _notificationList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _notificationList.verticalScrollPolicy = ScrollPolicy.ON;
        _notificationList.backgroundSkin = new Quad(1,1,0xe8e8e8);
        _notificationList.addEventListener(Event.SCROLL, listScrollHandler);

        _notificationList.width = actualWidth;
        _notificationList.height = actualHeight;
        _notificationList.x = 0;
        _notificationList.y = 0;
        rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;
        _notificationList.setItemRendererFactoryWithID("regular-notes", notificationItemRendererFactory);
        _notificationList.setItemRendererFactoryWithID("challenge-notes", challengeNotificationItemRendererFactory);
        _notificationList.factoryIDFunction = function(item:Object, index:int):String {
            if(item is ChallengeNotificationData){
                return "challenge-notes";
            }
            return "regular-notes";
        };

        addChild(_notificationList);
        addChild(closeButton);
        closeButton.defaultSkin = new Image( assetManager.getTexture('model_close'));
        closeButton.defaultSkin.alpha = 0.6;
        closeButton.setSize(36, 36);

        _messageLabel = new MultilingualLabel();
        _messageLabel.includeInLayout = false;
        _messageLabel.visible = false;
        _messageLabel.height = 14;
        _messageLabel.textRendererFactory = LabelFactory.notificationsMessageField;
        _messageLabel.text = "You don't have any notifications at this moment";
        _messageLabel.width = actualWidth;
        addChild(_messageLabel);

        commitNotificationLoad();
    }

    override protected function closeButtonTriggeredHandler(event:Event):void{
        super .closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        Starling.juggler.delayCall(function():void{
            if(_showBannerAfterExit){
                AdsManager.$.showBannerAfterComplete = true;
                if(_bannerScreen == BannerScreen.BANNER_SCREEN){
                    BannerStackedScreen.showAdBanner();
                }else if(_bannerScreen == BannerScreen.HOME_SCREEN){
                    AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
                }
            }
        }, 0.5);
    }

    public function commitNotificationLoad():void{
        if(!isLoading){
            var count:int = Math.min(MAX_COUNT + 1, loadedData.length);
            if(count == 0){
                requestedTypes[ChallengeEvent.GET_INVITATIONS] = requestedTypes[ChallengeEvent.GET_NOTIFICATIONS] = true;
            }else {
                for (var i:int = 0; i < count; i++) {
                    if (!notesLoaded) {
                        requestedTypes[ChallengeEvent.GET_NOTIFICATIONS] = true;
                    }
                    if (requestedTypes[ChallengeEvent.GET_INVITATIONS] && requestedTypes[ChallengeEvent.GET_NOTIFICATIONS]) {
                        break;
                    }
                }
            }

            if(requestedTypes[ChallengeEvent.GET_INVITATIONS]){
                dispatchEvent(new Event(ChallengeEvent.GET_INVITATIONS, false, {skip: loadedRequests.length, limit: MAX_COUNT}));
            }else if(requestedTypes[ChallengeEvent.GET_NOTIFICATIONS]){
                dispatchEvent(new Event(ChallengeEvent.GET_NOTIFICATIONS, false, {skip:loadedNotes.length, limit: MAX_COUNT}));
            }
            if(requestedTypes[ChallengeEvent.GET_INVITATIONS] || requestedTypes[ChallengeEvent.GET_NOTIFICATIONS]){
                isLoading = true;
                if(_onComplete == null){
                    alertView = AlertView.show("Loading...", 0);
                }
            }
        }
    }

    public function loadInvitations(data:Object):void {
        var arr:Array = convertDataToVO(data);
        loadedData = loadedData.concat(arr);
        loadedData.sortOn("time", Array.NUMERIC);
        loadedRequests = loadedRequests.concat(arr);
        requestedTypes[ChallengeEvent.GET_INVITATIONS] = false;
        if(arr.length<MAX_COUNT){
            requestsLoaded = true;
        }
        isLoading = requestedTypes[ChallengeEvent.GET_NOTIFICATIONS];
        if(isLoading){
            dispatchEvent(new Event(ChallengeEvent.GET_NOTIFICATIONS, false, {skip:loadedNotes.length, limit: MAX_COUNT}));
        }else{
            addLoadedNotesToList();
        }
    }

    public function loadNotifications(data:Object):void {
        var arr:Array = convertDataToVO(data);
        loadedData = loadedData.concat(arr);
        loadedData.sortOn("time", Array.NUMERIC);
        loadedNotes = loadedNotes.concat(arr);
        requestedTypes[ChallengeEvent.GET_NOTIFICATIONS] = false;
        if(arr.length<MAX_COUNT){
            notesLoaded = true;
        }
        isLoading = requestedTypes[ChallengeEvent.GET_INVITATIONS];
        addLoadedNotesToList();
        if(_onComplete != null){
            _onComplete();
            _onComplete = null;
        }
    }

    private function addLoadedNotesToList():void{
        if(!isLoading){
            if(loadedData.length) {
                var count:int = Math.min(loadedData.length, MAX_COUNT);
                notifications = loadedData.slice(0, count);
                loadedData.splice(0, count);
            }
            _messageLabel.visible = (!_notifications || _notifications.length == 0);
            if(alertView) {
                alertView.hide();
            }
        }
    }

    public function set notifications (value : Array) : void {
        if(_notifications && _notifications.length > 0) {
            var newListCollection : ListCollection = new ListCollection(value);
            _notificationList.dataProvider.addAll(newListCollection);
            if(_notificationList.verticalScrollPosition>0) {
                _notificationList.scrollToPosition(0, _notificationList.verticalScrollPosition + _notificationList.height / 2, 1);
            }
        } else {
            _notifications = value;
            if(_notificationList) {
                _notificationList.dataProvider = new ListCollection(value);
                _notificationList.invalidate(INVALIDATION_FLAG_DATA);
            }
        }
        Starling.juggler.delayCall(cleanupList, 1);
    }

    private function cleanupList():void{
        if(_notificationList && _notificationList.isScrolling){
            Starling.juggler.delayCall(cleanupList, 1);
            return;
        }
        if(_notificationList && _notificationList.dataProvider.length>MAX_LIST_COUNT){
            _notificationList.validate();
            var itemSize:Number = ((_notificationList.maxVerticalScrollPosition+_notificationList.height)/_notificationList.dataProvider.length);
            var currentIndex:int = int(_notificationList.verticalScrollPosition/itemSize);
            var skipStart:int = Math.max(int(currentIndex - MAX_LIST_COUNT/2), 0);
            var skipEnd:int = Math.min(int(currentIndex + MAX_LIST_COUNT/2)+1, _notificationList.dataProvider.length);
            var i:int;
            for(i = 1; i < skipStart; i++){
                if(_notifications[i] is ChallengeNotificationData){
                    ChallengeNotificationData(_notifications[i]).dispose();
                }else if(_notifications[i] is NotificationFeedData){
                    NotificationFeedData(_notifications[i]).dispose();
                }
            }
            for(i = skipEnd; i < _notifications.length; i++){
                if(_notifications[i] is ChallengeNotificationData){
                    ChallengeNotificationData(_notifications[i]).dispose();
                }else if(_notifications[i] is NotificationFeedData){
                    NotificationFeedData(_notifications[i]).dispose();
                }
            }
            trace("itemSize",itemSize, _notificationList.dataProvider.length, "currentIndex", currentIndex, "skipStart", skipStart, "skipEnd", skipEnd);
        }
    }

    override protected function draw():void {
        super.draw();
        _notificationList.width = actualWidth;
        _notificationList.height = actualHeight;
        _notificationList.x = 0;
        _notificationList.y = 0;
        rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;
        if(closeButton){
            closeButton.x = actualWidth-10-closeButton.width;
            closeButton.y = 10;
        }

        if(_messageLabel){
            _messageLabel.x = (actualWidth - _messageLabel.width) / 2;
            _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
        }
    }

    private function listScrollHandler(event:Event):void {
        if(!_notificationList || !_notifications || _notifications.length == 0 || _notificationList.dataProvider.length == 0) {
            return;
        }
        if(_notificationList.verticalScrollPosition >= _notificationList.maxVerticalScrollPosition-30 && !isLoading) {
            commitNotificationLoad();
        }else if(_notificationList.verticalScrollPosition <= Math.max(_notificationList.maxVerticalScrollPosition - 2 * stage.stageHeight, 0)){
            Starling.juggler.removeDelayedCalls(cleanupList);
            Starling.juggler.delayCall(cleanupList, 0.2);
        }
    }

    private function notificationItemRendererFactory() : IListItemRenderer {
        var renderer : NotificationPanelViewItemRenderer = new NotificationPanelViewItemRenderer();
        renderer.setSize(rendererWidth, 80);
        renderer.eventDispatcher = this;
        return renderer;
    }

    private function challengeNotificationItemRendererFactory() : IListItemRenderer {
        var renderer : ChallengeNotificationItemRenderer = new ChallengeNotificationItemRenderer();
        renderer.isUltraWide = Constants.isUltraWideScreen;
        renderer.isWide = Constants.isWideScreen;
        renderer.setSize(rendererWidth, 80);
        renderer.eventDispatcher = this;
        return renderer;
    }


    override public function dispose():void {
        super.dispose();
        Starling.juggler.removeDelayedCalls(cleanupList);
        if(_notifications) {
            for (var i:int = _notifications.length - 1; i >= 0; i--) {
                if(_notifications[i] is ChallengeNotificationData){
                    ChallengeNotificationData(_notifications[i]).destroy();
                }else if(_notifications[i] is NotificationFeedData){
                    NotificationFeedData(_notifications[i]).destroy();
                }
            }
            _notifications = null;
        }
        if(_notificationList){
            _notificationList.removeEventListener(Event.SCROLL, listScrollHandler);
            _notificationList.dataProvider = null;
            _notificationList.removeFromParent(true);
            _notificationList = null;
        }
        if(alertView){
            alertView.hide();
            alertView = null;
        }

        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }
    }

    private function viewTuningHandler(event:Event):void {
        var tuning:Tuning = event.data.tuning;
        var tuningParams : Object = {
            tuning: tuning,
            user: null/*event.data.owner*/,
            data: null,
            likes: 0,
            images: event.data.tuning.fullHdSnapshots,
            iLiked: false
        };
        var timelineTuningView:TimelineTuningView = new TimelineTuningView(tuningParams);
        addChild(timelineTuningView);
        timelineTuningView.layoutData = null;
        timelineTuningView.setSize(Starling.current.stage.width, Starling.current.stage.height);
    }

    public function removeItem(item:ChallengeNotificationData):void{
        var index:int = _notifications.indexOf(item);
        if(index != -1){
            _notifications.removeAt(index);
            _notificationList.invalidate(FeathersControl.INVALIDATION_FLAG_DATA);
            item.dispose();
        }
    }

    public function close():void {
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

    private static function convertDataToVO(data:Object):Array{
        var result:Array = [];
        for(var i:int = 0; i < data.length; i++){
            if(TimelineProxy.NOTIFICATIONS_TYPES.indexOf(data[i].type ) >= 0 ) {
                var item : FeedData = new NotificationFeedData(data[i]);
                result.push(item);
            }else if(ChallengeNotificationData.NOTIFICATION_TYPES.indexOf(data[i].type ) >= 0 ) {
                var challengeItem : ChallengeNotificationData = ChallengeNotificationData.createInstance(data[i]);
                result.push(challengeItem);
            }
        }
        return result;
    }

    public static function getInstance(data:Object = null):InAppNotificationsPopup{
        if(_instance){
            _instance.dispose();
        }
        _instance = new InAppNotificationsPopup(data);
        return _instance;
    }

        public function set onComplete(value:Function):void {
            _onComplete = value;
        }
    }
}
