package com.despark.tuning3d.view.components.core
{
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.utils.FontSelector;

import feathers.core.FeathersControl;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.animation.Transitions;
import starling.animation.Tween;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.Texture;
import starling.utils.AssetManager;

public class AlertViewHL extends FeathersControl implements Multilingual
	{
		public static const MIN_HEIGHT : int = 100;
		public static var assetManager : AssetManager;
		public static var language : Language;
		
		public var simpleAlert : Boolean;
		public var resultHandler : Function;
		
		private var backgroundImage:Image;
		private var label : HyperlinkTextFieldTextRenderer;
		private var labelTitle : HyperlinkTextFieldTextRenderer;
		private var _text : String;
		private var _textOfTitle : String;
		private var closeButton : MultilingualButton;
		private var yesButton : MultilingualButton;
		private var dismissingView : Quad;
		
		public var leftText : String;
		public var rightText : String;
		
		public var yesButtonColor:uint=0xED3F25;
		
		public function get textOfTitle():String
		{
			return _textOfTitle;
		}
		
		public function set textOfTitle(value:String):void
		{
			_textOfTitle = value;
			invalidate("textOfTitle");
		}
		
		public function AlertViewHL()
		{
			super();
			simpleAlert = true;
		}
		
		public function get text():String
		{
			return _text;
		}

		public function set text(value:String):void
		{
			_text = value;
			invalidate("text");
		}
		
		public function updateWithLanguage(language :Language) : void
		{
			//label.updateWithLanguage(language);
		}

		protected override function initialize() : void
		{
			super.initialize();
			
			
			
			var backgroundTexture : Texture = assetManager.getTexture("popUpBackground");
			//var backgroundTextures : Scale9Textures = new Scale9Textures(backgroundTexture, new Rectangle(24, 22, 11, 7));
			
			backgroundImage = new Image(backgroundTexture);
			backgroundImage.scale9Grid = new Rectangle(24, 22, 11, 7);
			backgroundImage.alpha = simpleAlert ? 0.8 : 1.0;
			addChild(backgroundImage);	
			
			
			labelTitle = new HyperlinkTextFieldTextRenderer();
			labelTitle.textFormat = new TextFormat(FontSelector.getFont(true), simpleAlert ? 17 : 12, 0xFFFFFF );
			labelTitle.embedFonts=Language.supportsEmbeddedFont;
			labelTitle.textFormat.bold=true;
			labelTitle.textFormat.align = TextFormatAlign.CENTER;
			labelTitle.wordWrap = true;
			addChild(labelTitle);
			
			label = new HyperlinkTextFieldTextRenderer();
			label.textFormat = new TextFormat(FontSelector.getFont(), simpleAlert ? 15 : 10, 0xFFFFFF );
			label.embedFonts=FontSelector.embedFonts();;
			label.textFormat.align = TextFormatAlign.CENTER;
			//label.textFormat.
			label.wordWrap = true;
			addChild(label);
			
			if(!simpleAlert)
			{
				closeButton = ButtonFactory.defaultButton(rightText.toUpperCase(), 0xB2BDC1);
				closeButton.updateWithLanguage(AlertViewHL.language);
				closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);
				addChild(closeButton);
				
				if(resultHandler)
				{
					yesButton = ButtonFactory.defaultButton(leftText.toUpperCase(), yesButtonColor);
					yesButton.updateWithLanguage(AlertViewHL.language);
					yesButton.addEventListener(Event.TRIGGERED, yesButtonTriggeredHandler);
					addChild(yesButton);
				}
				
				dismissingView = new Quad(stage.stageWidth, stage.stageHeight, 0xFF0000);
				dismissingView.alpha = 0.0;
				dismissingView.addEventListener(TouchEvent.TOUCH, dismissingViewTouchHandler);
				stage.addChildAt(dismissingView, stage.getChildIndex(this));
			}
		}
		
		private function dismissingViewTouchHandler(event : TouchEvent) : void   
		{
			var touch : Touch = event.getTouch(dismissingView);
			
			if(touch && touch.phase == TouchPhase.ENDED)      
			{
				hide();
			}
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid("text") ||isInvalid("textOfTitle"))
			{
				if(AlertViewHL.language)
				{
					//label.updateWithLanguage(AlertViewHL.language);
				}
				else 
				{
					var l:Language = Language.createInstance("en-US");
					//label.updateWithLanguage(l);
				}
				
				label.text = text;
				labelTitle.text = textOfTitle;
			}
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			
			if(autoSizeNeeded || isInvalid(INVALIDATION_FLAG_SIZE))
			{
				backgroundImage.width = actualWidth;
				backgroundImage.height = actualHeight;
				
				//////////////////////////////////
				
				labelTitle.x = 10;
				labelTitle.width = actualWidth - 20;
				labelTitle.y = 10;
				
				if(actualHeight == MIN_HEIGHT && simpleAlert)
				{
					labelTitle.y = (actualHeight - label.height - 10 - labelTitle.height) / 2;
				}
				
				// UGLY FIX FOR AIR TEXT MEASURING PROBLEMS
				//if(AlertViewHL.language && labelTitle.text == AlertViewHL.language.translate("mob_alertsAndOverlays", "savedToCameraRoll"))
				//{
				//	label.y = 20;
				//}
				//////////////////////////////////
				
				label.x = 10;
				label.width = actualWidth - 20;
				label.y = labelTitle.y + labelTitle.height + 10;
				label.height +=10;
				
				//if(actualHeight == MIN_HEIGHT && simpleAlert)
				//{
				//	label.y = (actualHeight - label.height) / 2;
				//}
				
				// UGLY FIX FOR AIR TEXT MEASURING PROBLEMS
				//if(AlertViewHL.language && label.text == AlertViewHL.language.translate("mob_alertsAndOverlays", "savedToCameraRoll"))
				//{
				//	label.y = 20;
				//}
				
				if(closeButton)
				{
					closeButton.y = actualHeight - closeButton.height - 10;
					
					if(resultHandler)
					{
						yesButton.y = closeButton.y;
						yesButton.x = 10;
						yesButton.width = (actualWidth - 20) / 2 - 5;
						
						closeButton.width = yesButton.width;
						closeButton.x = yesButton.x + yesButton.width + 10;
						
						
					}
					else
					{
						closeButton.x = 10;
						
						closeButton.width = actualWidth - 20;
					}
				}
				
				x = (stage.stageWidth - actualWidth) / 2;
				y = (stage.stageHeight - actualHeight) / 2;
			}
		}
		
		protected function autoSizeIfNeeded() : Boolean
		{
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			if(!needsHeight)
			{
				return false;
			}
			
			labelTitle.width = explicitWidth - 20;
			labelTitle.height = NaN;
			labelTitle.validate();
			
			label.width = explicitWidth - 20;
			label.height = NaN;
			label.validate();
			
			//var newHeight : Number = explicitHeight;
			
			//if(needsHeight)
			//{
			var newHeight : Number  = Math.max(simpleAlert ? MIN_HEIGHT : 0, 20 +  Math.ceil(labelTitle.height) + 10 +  Math.ceil(label.height) + 10 + (closeButton ? closeButton.height + 20 : 0));
			//}
			
						
			return setSizeInternal(explicitWidth, newHeight, false);
		}
		
		
		public function hide() : void
		{
			
			AlertViewHL.hideAlertView(this);
		}
		
		public static function show(textOfTitle : String,text : String,leftText : String,rightText : String, hideTimeout : Number = 1.0, simple : Boolean = true, resultHandler : Function = null, yesButtonColor:uint=0xED3F25) : AlertViewHL
		{
			var alertView : AlertViewHL = new AlertViewHL();
			
			alertView.leftText = leftText;
			alertView.rightText = rightText;
			alertView.text = text;
			alertView.textOfTitle = textOfTitle;
			alertView.width = simple ? 100 : 200;
			alertView.alpha = 0.0;
			alertView.simpleAlert = simple;
			alertView.resultHandler = resultHandler;
			alertView.touchable = !simple;
			alertView.yesButtonColor=yesButtonColor;
			
			Starling.current.stage.addChild(alertView);
			
			var tween : Tween = new Tween(alertView, 0.35, Transitions.EASE_IN_OUT);
			tween.fadeTo(1.0);
			Starling.juggler.add(tween);
			
			if(hideTimeout > 0)
			{
				Starling.juggler.delayCall(function() : void
				{
					hideAlertView(alertView);
				}, hideTimeout);
			}
			

			
			return alertView;
		}
		
		public static function hideAlertView(alertView : AlertViewHL) : void
		{
			if(alertView.closeButton)
			{
				alertView.closeButton.visible = false;
			}
			
			if(alertView.yesButton)
			{
				alertView.yesButton.visible = false;
			}
			
			var tween : Tween = new Tween(alertView, 0.35, Transitions.EASE_IN_OUT);
			tween.fadeTo(0.0);
			tween.onComplete = function() : void
			{
				Starling.current.stage.removeChild(alertView);
				Starling.current.stage.removeChild(alertView.dismissingView);
			}
			Starling.juggler.add(tween);
		}
		
		private function closeButtonTriggeredHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			hide();
			
			if(resultHandler)
			{
				resultHandler(false);
			}
		}
		
		private function yesButtonTriggeredHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			hide();
			
			if(resultHandler)
			{
				resultHandler(true);
			}
		}
	}
}