package com.despark.tuning3d.view.components.core {
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.view.components.cars.Background3D;
	import com.utils.BitmapDataUtils;

	import feathers.core.FeathersControl;

	import flash.display.BitmapData;
	import flash.events.PermissionEvent;
	import flash.geom.Matrix;
	import flash.media.Camera;
	import flash.permissions.PermissionStatus;

	import starling.core.Starling;

	import starling.display.Image;
	import starling.textures.Texture;

	public class BackgroundContainer extends FeathersControl {
		
		public static const EVENT_CAMERA_ERROR:String = "EVENT_CAMERA_ERROR";
		
		private var _background:Image;
		private var _backgroundFromCamera:Image = null;
		private var _scale:Number = 1;
		
		private var _handleCameraTexture:Texture = null;
		private var _camera:Camera;
		private var _textureList:Vector.<Texture>;
		private var _moderateSize:Boolean = true;
		private var _defaultTexture:Texture;
		private var _background3D:Background3D;
		private var _ARBackgroundData:BitmapData;
		public function BackgroundContainer() {
			super();
            _textureList = new Vector.<Texture>();
		}

		private function onPermissionStatusChange(event:PermissionEvent):void{
			if(event.status == PermissionStatus.GRANTED){
				if(!_camera){
					startCamera();
				}else{
					setCameraMode();
				}
			} else {
				dispatchEventWith(EVENT_CAMERA_ERROR, false);
				_camera = null;
			}
		}
		
		private function setCameraMode():Boolean{
			if(_camera){
				_camera.setMode(1920, 1080, 30);
				_camera.setQuality(0, 100);
				_handleCameraTexture = Texture.fromCamera(_camera,1, function(texture:Texture):void{
					_backgroundFromCamera = new Image(texture)
					addChild(_backgroundFromCamera);
					invalidate();
				});
				return true;
			}
			return false;
		}
		
		
		public function startCamera():Boolean{
			_camera = Camera.getCamera();
			if(Camera.permissionStatus==PermissionStatus.UNKNOWN || Camera.permissionStatus==PermissionStatus.DENIED){
				_camera.addEventListener(PermissionEvent.PERMISSION_STATUS, onPermissionStatusChange);
				Camera.permissionManager.requestPermission();
				//_camera.requestPermission();
				return false;
			}
			
			return setCameraMode();
		}
		
		public function stopCamera(getBitmapData:Boolean):BitmapData{
			
			if(!_camera) return null;
			
			var bitmapData:BitmapData = null;
			if(getBitmapData){
				
				bitmapData = new BitmapData(_camera.width, _camera.height, false);
				_camera.drawToBitmapData(bitmapData);
				
				if(_camera.width!=1920 || _camera.height!=1080){
					var scaleX:Number = 1920/_camera.width;
					var scaleY:Number = 1080/_camera.height;
					bitmapData = BitmapDataUtils.scaleBitmapDataXY(bitmapData,scaleX,scaleY, true);
				}
			
				var texture:Texture = Texture.fromBitmapData(bitmapData);
				updateBackground(texture);
				
			}
			
			
			if(_handleCameraTexture){
				
				_handleCameraTexture.root.attachCamera(null);
				_handleCameraTexture.dispose();
				_handleCameraTexture = null;
			}
			
			if(_backgroundFromCamera){
				
				_backgroundFromCamera.removeFromParent(true);
				_backgroundFromCamera = null;
			}
			
			
			
			_camera = null;
			
			return bitmapData;
			
		}

		public function uploadCustomBackground(bitmapData:BitmapData):BitmapData{
			if(bitmapData.width!=CarBackground.DEFAULT_WIDTH || bitmapData.height!=CarBackground.DEFAULT_HEIGHT){
				var scaledBitmapData:BitmapData = new BitmapData(CarBackground.DEFAULT_WIDTH, CarBackground.DEFAULT_HEIGHT, false);
				var m:Matrix = new Matrix();
				var sc:Number;
				var tx:Number = 0;
				var ty:Number = 0;
				if(bitmapData.width/bitmapData.height >= CarBackground.DEFAULT_WIDTH/CarBackground.DEFAULT_HEIGHT){
					sc = CarBackground.DEFAULT_HEIGHT / bitmapData.height;
					tx = (CarBackground.DEFAULT_WIDTH - bitmapData.width * sc)/ 2;
				}else{
					sc = CarBackground.DEFAULT_WIDTH / bitmapData.width;
					ty = (CarBackground.DEFAULT_HEIGHT - bitmapData.height * sc) / 2;
				}
				m.scale(sc, sc);
				m.tx = tx;
				m.ty = ty;
				scaledBitmapData.draw(bitmapData, m);
				bitmapData.dispose();
				bitmapData = scaledBitmapData;
			}
			var texture:Texture = Texture.fromBitmapData(bitmapData);
			updateBackground(texture);
			return bitmapData;
		}
		
		
		override protected function initialize():void{
			super.initialize();
			if(_defaultTexture == null){
				_defaultTexture = Texture.fromBitmapData(new BitmapData(1920, 1080,false, 0xd4d2de));
			}
			_background ||= new Image(_defaultTexture);
			addChild(_background);
		}
		
		override public function dispose():void{
			super.dispose();
			if(_background){
				if(_background.texture){
                    _background.texture.dispose();
                    _background.texture = null;
				}
                _background.removeFromParent(true);
                _background = null;
			}
			if(_textureList){
				for(var i:int = 0; i < _textureList.length; i++){
                    _textureList[i].dispose();
                    _textureList[i] = null;
				}
                _textureList = null;
			}

			if(_backgroundFromCamera){
				_backgroundFromCamera.removeFromParent(true);
				_backgroundFromCamera = null;
			}

			if(_handleCameraTexture){
				_handleCameraTexture.dispose();
				_handleCameraTexture = null;
			}

			if(_defaultTexture){
				_defaultTexture.dispose();
				_defaultTexture = null;
			}

			if(_background3D){
				_background3D.removeFromParent(true);
				_background3D = null;
			}
			if(_ARBackgroundData){
				_ARBackgroundData.dispose();
				_ARBackgroundData = null;
			}

		}
		
		public function updateBackground(texture:Texture, disposeOld:Boolean = true):void{
			if(_background){
				if(_background.texture!=texture){
					if(disposeOld && _background.texture){
						_background.texture.dispose();
					}else if(_background.texture){
                        _textureList.push(_background.texture);
					}
					_background.texture = texture;
					_background.readjustSize();

					if(_defaultTexture){
						_defaultTexture.dispose();
						_defaultTexture = null;
					}

					invalidate(INVALIDATION_FLAG_ALL);
				}
			}
		}
		
		public function get texture():Texture{
			if(_background && _background.texture) return _background.texture;
			else return null;
		}
		
		public function set myScale(value:Number):void{
			_scale = value;
			this.invalidate(INVALIDATION_FLAG_SIZE);
		}

		override protected function draw():void{
			super.draw();
			if(this.isInvalid(INVALIDATION_FLAG_SIZE)){
				if(moderateSize){
					_background.width = actualWidth * _scale;
					_background.height = actualHeight * _scale;
				}

				_background.x = (this.actualWidth - _background.width) / 2;
				_background.y = ( this.actualHeight-_background.height) / 2;

				if(_backgroundFromCamera){
					_backgroundFromCamera.width = _background.width;
					_backgroundFromCamera.height = _background.height;
				
					_backgroundFromCamera.x = _background.x;
					_backgroundFromCamera.y = _background.y;
				}
			}
		}

		public function get background():Image {
			return _background;
		}

		public function get moderateSize():Boolean {
			return _moderateSize;
		}

		public function set moderateSize(value:Boolean):void {
			_moderateSize = value;
		}

		public function get textureScale():Number{
			if(background && background.texture && background.texture.width){
				return background.width / background.texture.nativeWidth;
			}
			return 1;
		}

		public function set background3D(value:Background3D):void {
			if(_background3D && _background3D != value){
				_background3D.removeFromParent(true);
			}
			_background3D = value;
			_background3D.includeInLayout = false;
			addChild(_background3D);
		}

		public function get background3D():Background3D {
			return _background3D;
		}

		public function getARBackgroundData():BitmapData {
			return _ARBackgroundData.clone();
		}

		public function set ARBackgroundData(value:BitmapData):void {
			_ARBackgroundData = value;
		}
	}
}