package com.despark.tuning3d.view.components.core {
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.view.ShareMenuButton;
import com.despark.tuning3d.view.components.challenges.modals.StartChallengePopup;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Radio;
import feathers.controls.ToggleButton;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.ITextRenderer;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalAlign;
import feathers.layout.RelativePosition;
import feathers.layout.VerticalAlign;

    import flash.geom.Matrix;

    import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

    import starling.display.Image;

    import starling.display.Image;

import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;
    import starling.filters.ColorMatrixFilter;
    import starling.textures.Texture;
    import starling.textures.Texture;
    import starling.utils.Align;
    import starling.utils.AssetManager;

public class ButtonFactory {
    public static var assetManager:AssetManager;

    public static function defaultButton2(title:String, color:uint):MultilingualButton {
		var buttonTexture:Texture = assetManager.getTexture("button2");
		//buttonTexture.forDelete = false;
		//var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		buttonImage.scale9Grid =  new Rectangle( 14, 0, 15, buttonTexture.frameHeight );
        buttonImage.color = color;

        var button:MultilingualButton = new MultilingualButton();
        button.textFormat = new TextFormat(FontSelector.getFont(true), 9, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;
        button.label = title;
        button.width = 75;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;

        return button;
    }

    public static function tuningSettingsButton(textureId:String, title:String, scaleFactor:Boolean = true):Button {
		var buttonTexture:Texture = assetManager.getTexture(textureId);
		var buttonImage:Image = new Image(buttonTexture);
		var downImage:Image = new Image(buttonTexture);
        downImage.alpha = 0.6;
        var scale:Number = scaleFactor?(assetManager.scaleFactor / 2.75): 1.5/2.75;
        buttonImage.scale = scale;
        downImage.scale = scale;

        buttonImage.color = 0x0f86ff;
        downImage.color = 0xfefefe;

        var button:ExtentedButton = new ExtentedButton();
        button.labelFactory = LabelFactory.tuningSettingLabel;
        button.label = title;
        button.iconPosition = RelativePosition.BOTTOM;
        button.labelOffsetY = -3.5;
        button.iconOffsetY = 1;
        button.defaultIcon = buttonImage;
        button.downIcon = downImage;
        button.disabledIcon = downImage;
        return button;
    }

    public static function newButton(texture:String, title:String, color:uint, padding:int = 0):MultilingualButton {
		var buttonTexture:Texture = assetManager.getTexture(texture);
		//buttonTexture.forDelete = false;
		//var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		buttonImage.scale9Grid = new Rectangle( 14, 0, 15, buttonTexture.frameHeight );

        var button:MultilingualButton = new MultilingualButton();
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 9, color);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };


        button.label = title;
        button.paddingLeft = padding;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage;

        return button;
    }

    public static function menuBigButton(textureName:String = null, color:uint = 0x0055aa) : MainMenuButton{
        var button:MainMenuButton = new MainMenuButton(assetManager.getTexture(textureName||"a_burger_button"), assetManager.scaleFactor, color);
        button.hitAreaPadding = 10;
        return button;

    }

    public static function friendsRadioButton(width: Number, height: Number,activeColor:uint, passiveColor:uint, iconId:String = null):Radio{
        var button : Radio = new Radio();
        var selectedSkin: Quad = new Quad(1,1,activeColor);
        var defaultSkin : Quad = new Quad(1,1,passiveColor);


        if(iconId){
            var iconTxt:Texture = assetManager.getTexture(iconId);
            var defaultIconImage:Image = new Image(iconTxt);
            var activeIconImage:Image = new Image(iconTxt);
            defaultIconImage.color = 0x808080;
            activeIconImage.color = passiveColor;
            defaultIconImage.scale = 0.7;
            activeIconImage.scale = 0.7;
            button.iconOffsetX = -5;
            button.defaultIcon = defaultIconImage;
            button.defaultSelectedIcon = activeIconImage;
        }

        button.defaultSelectedSkin = selectedSkin;
        button.defaultSkin = defaultSkin;
        button.width = width;
        button.height = height;

        button.addEventListener(Event.CHANGE, function (e:Event):void {
            if(button.isSelected)
            {
                button.labelFactory = function ():ITextRenderer {
                    var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                    textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, passiveColor);
                    textRenderer.textFormat.letterSpacing = -.35;
                    textRenderer.textFormat.italic = false;
                    textRenderer.textFormat.bold = false;
                    textRenderer.embedFonts = true;
                    textRenderer.textFormat.align = TextFormatAlign.LEFT;
                    return textRenderer;
                };
            }else {
                button.labelFactory = function ():ITextRenderer {
                    var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                    textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, 0x808080);
                    textRenderer.textFormat.letterSpacing = -.35;
                    textRenderer.textFormat.italic = false;
                    textRenderer.textFormat.bold = false;
                    textRenderer.embedFonts = true;
                    textRenderer.textFormat.align = TextFormatAlign.LEFT;
                    return textRenderer;
                };
            }
        });
        button.labelFactory = function (color:uint=0x454545):ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, activeColor);
            textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        };

        return button;
    }


    public static function scheduleRadioButton(width: Number, height: Number, activeColor:uint, passiveColor:uint):Radio{
        var button : Radio = new Radio();
        var selectedSkin: Quad = new Quad(1,1, activeColor);
        var defaultSkin : Quad = new Quad(1,1, passiveColor);

        button.defaultSelectedSkin = selectedSkin;
        button.defaultSkin = defaultSkin;
        button.width = width;
        button.height = height;

        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, UIColors.UI_COMPONENT_GRAY);
            textRenderer.embedFonts = FontSelector.embedFonts();
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;

            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, 0xFFFFFF);
            return textRenderer;
        };
        return button;
    }


    public static function simpleButtonByTextureName(textureName : String, scale:Boolean = false, textureColor:int = -1, isExtended:Boolean = false, flipped:Boolean = false) : Button {
        var button : Button = isExtended?new ExtentedButton(): new Button();
        button.defaultSkin = imageByTexture(textureName, flipped);
        button.downSkin = imageByTexture(textureName, flipped);
        button.downSkin.alpha = .6;
        if(scale){
            button.defaultSkin.scale = assetManager.scaleFactor / 2;
            button.downSkin.scale = assetManager.scaleFactor / 2;
        }
        if(textureColor != -1){
            Image(button.defaultSkin).color = textureColor;
            Image(button.downSkin).color = textureColor;
        }
        return button;
    }

    public static function simpleButtonByTextureNameIcon(textureName : String, scale:Boolean = false) : Button {
        var button : Button = new Button();
        button.defaultIcon= imageByTexture(textureName);
        button.downIcon= imageByTexture(textureName);
        button.downIcon.alpha = .6;
        button.disabledIcon = button.downIcon;
        var bg:Quad = new Quad(1,1,0xFFFFFF);
        bg.alpha = 0;
        button.defaultSkin = bg;
        if(scale){
            button.defaultIcon.scale = assetManager.scaleFactor / 2;
            button.downIcon.scale = assetManager.scaleFactor / 2;
        }
        return button;
    }

    public static function timelineLikeButtonNew(textureName : String) : ToggleButton {
        var button : ToggleButton = timelineSocialButtonNew(textureName, "garage_liked");
        return button;
    }

    public static function timelineCommentButtonNew(textureName : String) : ToggleButton {
        var button : ToggleButton = timelineSocialButtonNew(textureName, null);
        return button;
    }

    private static function timelineSocialButtonNew(textureName : String, selectedTextureName:String) : ToggleButton {
        var button : ToggleButton = new ToggleButton();
        var defaultSkin:Image = imageByTexture(textureName);
        defaultSkin.scale9Grid = new Rectangle(defaultSkin.texture.frameHeight, 0, defaultSkin.texture.frameWidth - defaultSkin.texture.frameHeight - defaultSkin.texture.frameHeight/5, defaultSkin.texture.frameHeight);
        button.defaultSkin = defaultSkin;

        var downSkin:Image = imageByTexture(textureName);
        downSkin.scale9Grid = new Rectangle(downSkin.texture.frameHeight, 0, downSkin.texture.frameWidth - downSkin.texture.frameHeight - 5, downSkin.texture.frameHeight);

        button.downSkin = downSkin;
        button.downSkin.alpha = .6;

        if(selectedTextureName) {
            var selectedSkin:Image = imageByTexture(selectedTextureName);
            selectedSkin.scale9Grid = new Rectangle(selectedSkin.texture.frameHeight, 0, selectedSkin.texture.frameWidth - selectedSkin.texture.frameHeight - 5, selectedSkin.texture.frameHeight);
            button.defaultSelectedSkin = selectedSkin;
            button.defaultSelectedSkin.alpha = 0.6;
        }

        button.iconOffsetY = -1;
        button.labelOffsetX = 12;
        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), TimelineLabelFactory.SIZE_SOCIAL_SMALL, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };

        return button;
    }

    public static function challengeLiteButton(textureName : String, selectedTextureName: String, direction: String) : ToggleButton {
        var button : ToggleButton = new ToggleButton();
        button.defaultSkin= imageByTexture(textureName);
        button.downSkin= imageByTexture(selectedTextureName);
        button.downSkin.alpha = .6;

        button.defaultSelectedSkin = imageByTexture(selectedTextureName);
        button.defaultSelectedSkin.alpha = 0.8;
        button.iconOffsetY = -1;
        if (direction == 'left') {
            button.labelOffsetX = 12;
        }
        button.labelOffsetY = -1;
        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            if (direction != 'left') {
                textRenderer.textFormat.rightMargin = 24
            }

            return textRenderer;
        };

        return button;
    }

    public static function challengeTopMenuToggle(label:String, textureId:String = null): ToggleButton {
        var defaultSkin: Quad = new Quad(1, 1, UIColors.WHITE);
        defaultSkin.width = 70;
        defaultSkin.height = 23;
        defaultSkin.alpha = 1;

        var disabledSkin: Quad = new Quad(1, 1, UIColors.UI_COMPONENT_GRAY_DISABLED);
        var selectedSkin: Quad = new Quad(1, 1, UIColors.UI_COMPONENT_GRAY);

        var button : ToggleButton = new ToggleButton();
        button.defaultSkin = defaultSkin;
        button.iconOffsetY = -1;
        button.disabledSkin = disabledSkin;
        button.defaultSelectedSkin = selectedSkin;
        button.label = label;
        //
        if(textureId){
            var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
            var selectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
            defaultIcon.color = UIColors.UI_COMPONENT_GRAY;
            selectedIcon.color = UIColors.WHITE;
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;

            button.defaultIcon = defaultIcon;
            button.defaultSelectedIcon = selectedIcon;
            button.iconOffsetY = 1;
            button.gap = 10;
        }
        //

        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, UIColors.UI_COMPONENT_GRAY);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;

            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, UIColors.WHITE);
            return textRenderer;
        };
        button.width = defaultSkin.width;
        button.height = defaultSkin.height;

        return button;
    }

    public static function challengeTopMenuButton(label:String, textureId:String = null): Button {
        var defaultSkin: Quad = new Quad(1, 1, UIColors.WHITE);
        defaultSkin.width = 70;
        defaultSkin.height = 23;
        defaultSkin.alpha = 1;

        var disabledSkin: Quad = new Quad(1, 1, UIColors.UI_COMPONENT_GRAY_DISABLED);
        var button : Button = new Button();
        button.defaultSkin = defaultSkin;
        button.iconOffsetY = -1;
        button.downSkin = disabledSkin;
        button.disabledSkin = disabledSkin;
        button.label = label;
        //
        if(textureId){
            var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
            defaultIcon.width = 24;
            defaultIcon.scale = defaultIcon.scaleX;

            button.defaultIcon = defaultIcon;
            button.iconOffsetY = 1;
            button.gap = 10;
        }
        //

        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 12, UIColors.UI_COMPONENT_GRAY);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.width = defaultSkin.width;
        button.height = defaultSkin.height;

        return button;
    }

    public static function challengeRadioButton(label:String = '', align:String = null, defaultId:String = null, selectedId:String = null): ToggleButton {
        if(align == null){
            align = Align.LEFT;
        }
        defaultId ||= "start_challenge_radio_default";
        selectedId ||= "start_challenge_radio_selected";
        var radioDefault:Image = new Image(assetManager.getTexture(defaultId));
        var radioSelected:Image = new Image(assetManager.getTexture(selectedId));
        radioDefault.scale = assetManager.scaleFactor / 2;
        radioSelected.scale = assetManager.scaleFactor / 2;
        /*radioDefault.scale9Grid = new Rectangle(16, 0);
        radioSelected.scale9Grid = new Rectangle(16, 0);*/
        var button:ToggleButton = new ToggleButton();
        button.defaultSkin = radioDefault;
        button.defaultSelectedSkin = radioSelected;
        button.label = label;
        button.validate();
        button.iconOffsetY = 2;
        button.height = 20;
        //
        if(align == Align.RIGHT){
            var m:Matrix = new Matrix();
            m.scale(-1, 1);
            radioDefault.pivotX = radioDefault.texture.frameWidth;
            radioDefault.pivotY = radioDefault.texture.frameHeight;

            radioSelected.pivotX = radioSelected.texture.frameWidth;
            radioSelected.pivotY = radioSelected.texture.frameHeight;

            radioDefault.rotation = Math.PI;
            radioSelected.rotation = Math.PI;
            button.validate();
        }
        //
        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 9, 0x313131);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 5;
            textRenderer.embedFonts = true;

            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 9, 0xFFFFFF);
            return textRenderer;
        };
        return button;
    }

    public static function timelineFilterButton(): ToggleButton {
        var image: Image = imageByTexture('timeline_filter_default');
        image.scale = assetManager.scaleFactor/2;
        image.alpha = 1;

        var disabled: Image = imageByTexture('timeline_filter_default');
        disabled.alpha = 0.6;
        disabled.scale = assetManager.scaleFactor/2;

        var down: Image = imageByTexture('timeline_filter_selected');
        down.scale = assetManager.scaleFactor/2;

        var button : ToggleButton = new ToggleButton();
        button.defaultIcon= image;
        button.labelOffsetX = 5;
        button.labelOffsetY = -1;
        button.disabledIcon = disabled;
        button.defaultSelectedIcon = down;

        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0x808080);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.height = image.height;
        return button;
    }

    public static function timelineFilterButton2(textureId:String, colorizeIcon:Boolean = true, iconHeight:Number = 0): ToggleButton {
        var image: Image = imageByTexture(textureId);
        image.scale = assetManager.scaleFactor/2;
        if(colorizeIcon){
            image.color = 0x8C8C8C;
        }else{
            image.color = 0xFF0000;
        }
        image.alpha = 1;
        //
        var ratio:Number = 0;
        if(iconHeight > 0){
            ratio = iconHeight / image.height;
            image.height = iconHeight;
            image.width *= ratio;
        }
        //

        var disabledIcon: Image = imageByTexture(textureId);
        if(colorizeIcon){
            disabledIcon.color = 0xCCCCCC;
        }else{
            disabledIcon.color = 0xFF0000;
        }
        disabledIcon.scale = assetManager.scaleFactor/2;
        if(iconHeight > 0){
            disabledIcon.height = iconHeight;
            disabledIcon.width *= ratio;
        }
        var button : ToggleButton = new ToggleButton();
        var selectedSkin:Quad = new Quad(1, 1, 0x8C8C8C);
        var selectedDisabledSkin:Quad = new Quad(1, 1, 0xBCBCBC);
        if(colorizeIcon){
            var selectedIcon: Image = imageByTexture(textureId);
            selectedIcon.color = 0xFCFFFF;
            selectedIcon.scale = assetManager.scaleFactor/2;
            button.defaultSelectedIcon = selectedIcon;
            if(iconHeight > 0){
                selectedIcon.height = iconHeight;
                selectedIcon.width *= ratio;
            }
        }
        button.defaultIcon = image;
        button.disabledIcon = disabledIcon;
        button.defaultSelectedSkin = selectedSkin;
        button.selectedDisabledSkin = selectedDisabledSkin;

        button.labelOffsetX = 5;
        button.labelOffsetY = -1;
        button.paddingLeft = 8;
        button.paddingRight = 13;
        button.height = 21;

        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0x808080);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xFFFFFF);
            textRenderer.disabledTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xDDDDDD);
            return textRenderer;
        };
        return button;
    }

    public static function timelineTopButton(label:String): ToggleButton {
        var button : ToggleButton = new ToggleButton();
        var texture:Texture = assetManager.getTexture('timeline_rate_filter_bg');
        var defaultSkin:Image = new Image(texture);
        defaultSkin.alpha = 0;
        var selectedSkin:Image = new Image(texture);
        selectedSkin.scale = assetManager.scaleFactor/2;
        selectedSkin.scale9Grid = new Rectangle(texture.frameWidth/3, texture.frameHeight/3, texture.frameWidth/3, texture.frameHeight/3);
        button.defaultSkin = defaultSkin;
        button.defaultSelectedSkin = selectedSkin;
        button.label = label;
        button.paddingLeft = button.paddingRight = 5;
        button.height = selectedSkin.height;
        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };

        return button;
    }

    public static function challengeWinCount() : Button
    {
        var image: Image = imageByTexture('garage-winner');
        image.width = 30;
        image.height = 30;

        var button : Button = new Button();
        button.defaultSkin= image;
        button.iconOffsetY = -1;
        button.labelOffsetX = image.width + 5;


        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.height = image.height;
        return button;
    }

    public static function timelineComentSmallIcon() : Button {
        var textureName:String = "commentIcon";
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var buttonImage:Image = new Image(buttonTexture);

        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.alpha = 0.4;

        var buttonImage3:Image = new Image(buttonTexture);
        buttonImage3.color = 0x0000ff;
        buttonImage3.alpha = .8;

        var buttonImage4:Image = new Image(buttonTexture);
        buttonImage4.color = 0x0000ff;
        buttonImage4.alpha = 0.4;


        buttonImage.width = buttonImage.height = 18;
        buttonImage2.width = buttonImage2.height = 18;
        buttonImage3.width = buttonImage3.height = 18;
        buttonImage4.width = buttonImage4.height = 18;

        var button:ToggleButton = new ToggleButton();
//        button.width = 45;
        button.height = 18;
        button.labelFactory = TimelineLabelFactory.likeCountLabelRenderer;
        button.labelOffsetX = 4;
        button.downIcon = buttonImage2;
        button.defaultSelectedIcon = buttonImage;
        button.defaultIcon = buttonImage3;
        button.selectedDownIcon = buttonImage4;

        return button;
    }

    public static function timelineLikeButton():ToggleButton {
        var textureName:String = "likeIcon";
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var buttonImage:Image = new Image(buttonTexture);

        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.alpha = 0.4;

        var buttonImage3:Image = new Image(buttonTexture);
        buttonImage3.color = 0x0000ff;
        buttonImage3.alpha = .8;

        var buttonImage4:Image = new Image(buttonTexture);
        buttonImage4.color = 0x0000ff;
        buttonImage4.alpha = 0.4;


        buttonImage.width = buttonImage.height = 18;
        buttonImage2.width = buttonImage2.height = 18;
        buttonImage3.width = buttonImage3.height = 18;
        buttonImage4.width = buttonImage4.height = 18;

        var button:ToggleButton = new ToggleButton();
//        button.width = 45;
        button.height = 18;
        button.labelFactory = TimelineLabelFactory.likeCountLabelRenderer;
        button.labelOffsetX = 4;
        button.downIcon = buttonImage2;
        button.defaultSelectedIcon = buttonImage;
        button.defaultIcon = buttonImage3;
        button.selectedDownIcon = buttonImage4;

        return button;
    }

    public static function simpleRoundButton(label:String, color:int, textureId:String = null) : MultilingualButton {
        var scale:Number = AlertView.assetManager.scaleFactor / 2;
        var _startButton : MultilingualButton = new MultilingualButton();
        _startButton.textFormat = new TextFormat(FontSelector.getMontserratLight(false), 12, 0xffffff);
        _startButton.textFormat.bold = false;
        _startButton.label = label;
        textureId ||= "a_bg_red_button";

        var buttonTexture:Texture = ButtonFactory.assetManager.getTexture(textureId);
        var defaultSkin:Image = new Image(buttonTexture);
        defaultSkin.scale9Grid = new Rectangle(25/scale,0,25/scale, buttonTexture.frameHeight);
        if(color >= 0){
            defaultSkin.color = color;
        }

        var downSkin:Image = new Image(buttonTexture);
        downSkin.scale9Grid = new Rectangle(25/scale,0,25/scale,buttonTexture.frameHeight);
        if(color >= 0){
            downSkin.color = color;
        }

        _startButton.defaultSkin = defaultSkin;
        _startButton.downSkin = downSkin;
        _startButton.downSkin.alpha = 0.6;

        _startButton.updateWithLanguage(AlertView.language);
        return _startButton;
    }

    public static function simpleRadioButton(label:String, color:int = 0xFEFFFF) : Radio {
        var scale:Number = AlertView.assetManager.scaleFactor / 2;
        var radio : Radio = new Radio();
        radio.label = label;
        var defaultTexture:Texture = ButtonFactory.assetManager.getTexture("simple_radio_default");
        var defaultIcon:Image = new Image(defaultTexture);
        defaultIcon.color = color;
        defaultIcon.scale = scale;
        radio.defaultIcon = defaultIcon;

        var selectedTexture:Texture = ButtonFactory.assetManager.getTexture("simple_radio_selected");
        var selectedIcon:Image = new Image(selectedTexture);
        selectedIcon.color = color;
        selectedIcon.scale = scale;
        radio.defaultSelectedIcon = selectedIcon;

        return radio;
    }

    public static function simpleToggleButton(label:String, color:int, defaultSkin:String = null, selectedSkin:String = null) : ToggleButton {
        var scale:Number = AlertView.assetManager.scaleFactor / 2;
        var button : ToggleButton = new ToggleButton();
        button.label = label;
        var defaultTexture:Texture = ButtonFactory.assetManager.getTexture(defaultSkin);
        var defaultIcon:Image = new Image(defaultTexture);
        defaultIcon.color = color;
        defaultIcon.scale = scale;
        button.defaultIcon = defaultIcon;

        var selectedTexture:Texture = ButtonFactory.assetManager.getTexture(selectedSkin);
        var selectedIcon:Image = new Image(selectedTexture);
        selectedIcon.color = color;
        selectedIcon.scale = scale;
        button.defaultSelectedIcon = selectedIcon;

        return button;
    }

    public static function timelineSocialButton():MultilingualButton {


        var button:MultilingualButton = new MultilingualButton();
        button.labelFactory = TimelineLabelFactory.socialLabelsTextRenderer1;

        button.width = 200;
        button.height = 18;
        button.horizontalAlign = HorizontalAlign.LEFT;

        return button;
    }

    public static function timelineCommentSend():MultilingualButton {
        var buttonTexture:Texture = assetManager.getTexture("sendIcon");
        var buttonImage:Image = new Image(buttonTexture);

        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();
        button.width = 22;
        button.height = 22;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;

        return button;
    }

    public static function timelineCommentButtons(title:String):MultilingualButton {
        var button:MultilingualButton = new MultilingualButton();
        button.textFormat = new TextFormat(FontSelector.getFont(true), 8, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.RIGHT;
        button.textFormat.bold = false;
        button.label = title;
        button.width = 60;
        return button;
    }

    public static function garageLikeButton() : ToggleButton
    {
        var textureName:String = "garage_like";
        var textureName2:String = "garage_liked";
        var buttonTexture1:Texture = assetManager.getTexture(textureName);
        var buttonTexture2:Texture = assetManager.getTexture(textureName2);
        var defaultImage:Image = new Image(buttonTexture1);

        var defaultDown:Image = new Image(buttonTexture1);
        defaultDown.alpha = 0.4;

        var selected:Image = new Image(buttonTexture2);
        selected.alpha = .8;

        var selectedDown:Image = new Image(buttonTexture2);
        selectedDown.alpha = 0.4;

        var button:ToggleButton = new ToggleButton();
        button.defaultSkin = defaultImage;
        button.downSkin = defaultDown;
        button.defaultSelectedSkin = selected;
        button.selectedDownSkin = selectedDown;

        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.labelOffsetX = 12;
        button.width = 60;
        button.height = 30;

        return button;
    }

    public static function garageChallengeButton() : Button{
        var textureName:String = "request_challenge_btn";
        var buttonTexture1:Texture = assetManager.getTexture(textureName);
        var defaultImage:Image = new Image(buttonTexture1);
        defaultImage.color = 0xFF0000;

        var defaultDown:Image = new Image(buttonTexture1);
        defaultDown.alpha = 0.4;
        defaultDown.color = 0xFF0000;

        var button:Button = new Button();
        button.defaultSkin = defaultImage;
        button.downSkin = defaultDown;
        button.width = 28;
        button.height = defaultImage.height * 28/defaultImage.width;
        return button;
    }

    public static function challengeWinCountButton() : Button{
        var button : ToggleButton = new ToggleButton();
        button.defaultSkin= imageByTexture('garage-winner');
        button.iconOffsetY = -1;
        button.labelOffsetX = 12;
        button.labelFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.width = 60;
        button.height = 30;
        return button;
    }

    public static function rectangleButton(title:String):MultilingualButton {
        var buttonTexture:Texture = assetManager.getTexture("timeline_edit_rect");
        var buttonImage:Image = new Image(buttonTexture);
        buttonImage.scale9Grid = new Rectangle(20,10,buttonTexture.width -40 , buttonTexture.frameHeight - 20);

        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.scale9Grid = new Rectangle(20,10,buttonTexture.width -40 , buttonTexture.frameHeight - 20);
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();

        button.label = title;

        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;

        return button;
    }

    public static function featuredButton(title:String):MultilingualButton {
        //Scale3Textures = new Scale3Textures(buttonTexture, 20, 20, Scale3Textures.DIRECTION_HORIZONTAL);
        var buttonTexture:Texture = assetManager.getTexture("featured");
        var buttonImage:Image = new Image(buttonTexture);
        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.alpha = 0.8;

        var button:MultilingualButton = new MultilingualButton();
        button.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xffffff);
        button.textFormat.align = TextFormatAlign.LEFT;
        button.textFormat.bold = true;
        button.wordWrap = true;

        button.labelOffsetX = 50;
        //button.paddingBottom = 2;
        button.label = title;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;
        button.width = 124;
        button.height = 43;

        return button;
    }


    public static function loginButton(title:String, color:uint):MultilingualButton {

		var buttonTexture:Texture = assetManager.getTexture("button");
		//buttonTexture.forDelete = false;
		//var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		buttonImage.scale9Grid = new Rectangle(14,0,15,buttonTexture.frameHeight);
        buttonImage.color = color;

		var buttonImage2:Image = new Image(buttonTexture);
		buttonImage2.scale9Grid = new Rectangle(14,0,15,buttonTexture.frameHeight);
        buttonImage2.color = color;
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 15, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        }

        button.label = title;
        button.width = 136;
        button.height = 44;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;

        return button;
    }


    public static function orangeButton(title:String):MultilingualButton {
		var buttonTexture:Texture = assetManager.getTexture("btn_buy_now");
		//var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 60, 10, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		
		buttonImage.scale9Grid = new Rectangle(60,0,10, buttonTexture.frameHeight);
		
		var buttonImage2:Image = new Image(buttonTexture);
		buttonImage2.scale9Grid = new Rectangle(60,0,10, buttonTexture.frameHeight);
		buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();

        button.textFormat = new TextFormat(FontSelector.getFont(true), 11, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;

        button.label = title;
        button.width = 74;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;
        button.verticalAlign = VerticalAlign.MIDDLE;
        return button;
    }

    public static function buy3dCoinButton():MultilingualButton {
		var buttonTexture:Texture = assetManager.getTexture("btn_buy_coin");
		var buttonImage:Image = new Image(buttonTexture);

		buttonImage.scale = assetManager.scaleFactor/2;

		var buttonImage2:Image = new Image(buttonTexture);
		buttonImage2.scale = assetManager.scaleFactor/2;
		buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();

        button.width = buttonImage.width;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;
        button.verticalAlign = VerticalAlign.MIDDLE;
        return button;
    }

    public static function homeScreenButton(buttonType:String, label:String, labelBackgroundColor:uint, isNarrowScreen:Boolean = false):HomeScreenButton {
        var mainImageTexture : Texture;
        if(buttonType == "tuning") {
            mainImageTexture = assetManager.getTexture("configurator");
        } else if(buttonType == "challenge"){
            mainImageTexture = assetManager.getTexture("challenge");
        }
        var button : HomeScreenButton = new HomeScreenButton(mainImageTexture, label, labelBackgroundColor);
        button.isNarrowScreen = isNarrowScreen;
        return button;

    }

    public static function defaultButton(title:String, color:uint, textureId:String = null):MultilingualButton {
        textureId ||= "button";
		var buttonTexture:Texture = assetManager.getTexture(textureId);
		//var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		buttonImage.scale9Grid = new Rectangle(14,0,15, buttonTexture.frameHeight);
		buttonImage.color = color;
		
		var buttonImage2:Image = new Image(buttonTexture);
		buttonImage2.scale9Grid = new Rectangle(14,0,15, buttonTexture.frameHeight);
		buttonImage2.color = color;
		buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();

        button.textFormat = new TextFormat(FontSelector.getMontserratMedium(true), 9, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;

        button.label = title;
        button.width = 75;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;

        return button;
    }



    public static function cleanCacheButton(title:String, color:uint):MultilingualButton {
		var buttonTexture:Texture = assetManager.getTexture("clean_cache");
		// var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
		var buttonImage:Image = new Image(buttonTexture);
		buttonImage.scale9Grid = new Rectangle(14,0,15,buttonTexture.frameHeight);
		buttonImage.color = color;
		
		var buttonImage2:Image = new Image(buttonTexture);
		buttonImage2.scale9Grid = new Rectangle(14,0,15,buttonTexture.frameHeight);
		buttonImage2.color = color;
		buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();
        button.textFormat = new TextFormat(FontSelector.getFont(true), 6, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.RIGHT;
        button.textFormat.bold = true;
        button.label = title;
        button.width = 60;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;

        return button;
    }

    public static function imageByTexture(textureName:String, flipped:Boolean = false):Image {
        var buttonTexture:Texture;
        buttonTexture = assetManager.getTexture(textureName);
        var image:Image = new Image(buttonTexture);
        if(flipped){
            image.pivotX = image.width;
            image.pivotY = image.height;
            image.rotation = Math.PI;
        }
        return image;
    }

    public static function standTexture(textureName:String):Image {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var image:Image = new Image(buttonTexture);

        image.scaleX = Constants.isSGS4 ? .78 : .82;
        image.scaleY = .79;
        if (Constants.SCALE_FACTOR == 2 && Constants.isIOS || Constants.isSGS4) {
            image.scaleY = .7;
        }

        return image;
    }

    public static function arrowButton(textureName:String, downAlpha:Number = 0.6):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var button:SimpleExtendedButton = new SimpleExtendedButton(30);
//        button.hitAreaPadding = 5;

        var defaultImage : Image = new Image(buttonTexture);
        defaultImage.pivotX = defaultImage.width/2;
        defaultImage.pivotY = defaultImage.height/2;
        button.defaultSkin = defaultImage;

        var downImage:Image = new Image(buttonTexture);
        downImage.pivotX = downImage.width/2;
        downImage.pivotY = downImage.height/2;
        downImage.alpha = downAlpha;
        button.downSkin = downImage;

        /*button.scaleX = .9;
        button.scaleY = .9;*/

        return button;
    }

    public static function tabBarButton(textureName:String, downAlpha:Number = 0.6):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var buttonImage:Image = new Image(buttonTexture);
        buttonImage.alpha = 0.75;
        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.letterSpacing = -.3;
            textRenderer.embedFonts = "NotoSans";
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.defaultSkin = buttonImage;
        button.paddingBottom = 6;
        button.downSkin = buttonImage2;

        button.width = Device.isIos() || assetManager.scaleFactor >= 3.375 ? 180 : 210;
        button.height = Device.isIos() || assetManager.scaleFactor >= 3.375 ? 26 : 28;

        return button;
    }

    public static function dropMenuButton(texture:String, downAlpha:Number = 0.6):Button {
        var buttonTexture:Texture = assetManager.getTexture(texture);
        var buttonImage:Image = new Image(buttonTexture);
        buttonImage.scale9Grid = new Rectangle(buttonTexture.frameWidth/3, 0, buttonTexture.frameWidth/3, buttonTexture.frameHeight);
        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.scale9Grid = new Rectangle(buttonTexture.frameWidth/3, 0, buttonTexture.frameWidth/3, buttonTexture.frameHeight);
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(true), 13, 0x535353);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            // textRenderer.embedFonts = "NotoSans";
            textRenderer.textFormat.italic = true;
            textRenderer.textFormat.bold = true;//true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;
        button.height = 25; //Device.isIos() || assetManager.scaleFactor <= 3.375 ? 29  : 24;
        return button;
    }

    public static function rectButton(textureName:String, textureName1:String, downAlpha:Number = 0.6):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var button:ExtentedButton = new ExtentedButton();
        button.defaultSkin = new Image(buttonTexture);
        button.touchable = false;
        return button;
    }

    public static function detailsMenuButton(textures:Object, prop:Object):Radio
    {
        var _wt:Number = 60;
        var _ht:Number = 30;
        var _tx:String = "";

        if (prop) {
            _wt = prop.w;
            _ht = prop.h;
            _tx = prop.t;
        }

        var textureSelected : Texture =  assetManager.getTexture(textures.sel);

        var button : Radio = new Radio();

        var selectedImage : Image = new Image(textureSelected);
        var idleImage : Image = new Image(textureSelected);
        var disabled : Image = new Image(textureSelected);
        var aspect : Number = selectedImage.width/selectedImage.height;
        disabled.width = selectedImage.width = idleImage.width = _wt*0.32;
        disabled.height = selectedImage.height =idleImage.height = selectedImage.width / aspect;
//        disabled.height = selectedImage.height =idleImage.height = _wt*0.3;

        button.defaultSelectedSkin = new Quad(1,1,0x1053A1);
        button.defaultSelectedIcon= selectedImage;
        selectedImage.color = 0xf7f7f7;

        button.defaultSkin = new Quad(1,1,0xF7F7F7);
        button.defaultIcon = idleImage;
        idleImage.color = 0x454545;

        button.disabledSkin = new Quad(1,1,0x999999);
        button.disabledIcon = disabled;
        disabled.color = 0x454545;

        button.gap = 7;
        button.paddingTop = 7;
        button.iconPosition = RelativePosition.TOP;
        button.horizontalAlign = HorizontalAlign.CENTER;
        button.verticalAlign = VerticalAlign.TOP;


        button.addEventListener(Event.CHANGE, function (e:Event):void {
            if(button.isSelected)
            {
                button.labelFactory = function ():ITextRenderer {
                    var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                    textRenderer.textFormat = new TextFormat(FontSelector.getFont(), (!Constants.isNarrowScreen && !Constants.isIOS ? 10.5 : 10), 0xfdfdfd);
                    textRenderer.textFormat.align = TextFormatAlign.LEFT;
                    textRenderer.textFormat.leftMargin = 0;
                    textRenderer.textFormat.bold = false;
                    textRenderer.embedFonts = true;
                    return textRenderer;
                };
            }else {
                button.labelFactory = function ():ITextRenderer {
                    var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                    textRenderer.textFormat = new TextFormat(FontSelector.getFont(), (!Constants.isNarrowScreen && !Constants.isIOS ? 10.5 : 10), 0x454545);
                    textRenderer.textFormat.align = TextFormatAlign.LEFT;
                    textRenderer.textFormat.leftMargin = 0;
                    textRenderer.textFormat.bold = false;
                    textRenderer.embedFonts = true;
                    return textRenderer;
                };
            }
        });
        button.labelFactory = function (color:uint=0x454545):ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), (!Constants.isNarrowScreen && !Constants.isIOS ? 10.5 : 10), color);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            return textRenderer;
        };

        button.width = _wt;
        button.height = _ht;
        button.label = _tx;

        return button;
    }

    public static function mainMenuButton(textureName:String, prop:Object = null, iconColor:int = -1):Button {
        var _wt:Number = 60;
        var _ht:Number = 20;
        var _tx:String = "";

        if (prop) {
            _tx = prop.t;
        }

        var controller:Button = new Button();
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var buttonImage:Image = new Image(buttonTexture);
        if(iconColor != -1){
            buttonImage.color = iconColor;
        }

        var defaultSkin:Quad = new Quad(_wt, _ht, 0x333437);
        controller.defaultSkin = defaultSkin;

        var downSkin:Quad = new Quad(_wt, _ht, 0x434447);
        controller.downSkin = downSkin;

        var iconButton:Button = new Button();
        iconButton.name = "icon";
        iconButton.width = iconButton.height = _ht;
        iconButton.defaultSkin = buttonImage;
        iconButton.y = 3;
        iconButton.touchable = false;
        controller.addChild(iconButton);

        var labelButton:MultilingualButton = new MultilingualButton();
        labelButton.name = "label";
        labelButton.label = _tx;
        labelButton.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            var fontSize: Number = 10;
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratBold(), fontSize, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.textFormat.letterSpacing = -0.1;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        labelButton.y = 0;
        labelButton.height = _ht + 6;
        labelButton.y = 0;
        labelButton.x = iconButton.x + iconButton.width;
		if(Device.isIphoneXVariant()){
			labelButton.x-= 9;
		}
        controller.addChild(labelButton);
        iconButton.touchable = labelButton.touchable = false;
        controller.setSize(_wt, _ht);
        return controller;
    }

    public static function searchButton(textureName:String, text:String = null):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var button:ExtentedButton = new ExtentedButton();
        button.defaultSkin = new Image(buttonTexture);
        button.downSkin = new Image(buttonTexture);

        if (text) {
            button.labelFactory = function ():ITextRenderer {
                var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 16, 0xA9AAAE);
                textRenderer.textFormat.align = TextFormatAlign.CENTER;
                textRenderer.textFormat.italic = true;
                textRenderer.textFormat.bold = false;
                textRenderer.embedFonts = true;
                return textRenderer;
            }
        }

        return button;
    }

    public static function pickerButton(textureName:String, tx:String = ""):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);

        var button:ExtentedButton = new ExtentedButton();
        button.hitAreaPadding = 5;
        button.defaultSkin = new Image(buttonTexture);

        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = .6;

        button.downSkin = downImage;
        button.textFormat = new TextFormat(FontSelector.getFont(), 18, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;

        return button;
    }

    public static function popupCloseButton(w : int, h : int):Button {
        var buttonTexture:Texture = assetManager.getTexture("ebay-close");
        var button:Button = new SimpleExtendedButton(10);

        var defaultImage:Image =  new Image(buttonTexture);
        button.defaultSkin = defaultImage;

        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = 0.6;

        button.downSkin = downImage;

        button.width = w;
        button.height = h;

        return button;
    }

    public static function screenCloseButton():Button {
        var buttonTexture:Texture = assetManager.getTexture("garage_close");
        var button:Button = new SimpleExtendedButton(10);
        var defaultImage:Image =  new Image(buttonTexture);
        button.defaultSkin = defaultImage;
        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = 0.6;
        button.downSkin = downImage;

        button.width = 24;
        button.height = 24;
        return button;
    }

    public static function maxMinButton(size : Number) : ToggleButton
    {
        var maxTexture:Texture = assetManager.getTexture("ebay-full");
        var minTexture:Texture = assetManager.getTexture("ebay-full-min");
        var maxImage:Image = new Image(maxTexture);
        var minImage:Image = new Image(minTexture);
        var disabled:Image = new Image(maxTexture);

        maxImage.width = maxImage.height= size;
        minImage.width = minImage.height= size;
        disabled.width = disabled.height= size;
        disabled.alpha = 0.6;



        var button:ToggleButton = new ExtendedToggleButton(10);
        button.width = button.height = size;

        button.defaultSelectedSkin= minImage;
        button.defaultSkin = maxImage;
        button.disabledSkin= disabled;

        return button;
    }



    public static function disckSettingButton(labelValue:String = ""):ToggleButton {
        var button:ToggleButton = new ToggleButton();
        button.defaultSkin = imageByTexture("a_disk_set_default");
        button.defaultSelectedSkin = imageByTexture("a_disk_set_select");
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 16, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getFont(), 16, 0x000000);
            return textRenderer;
        };
        button.label = labelValue;
		var w:Number;
		var h:Number;
        w = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
        h = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
		button.width = w;
		button.height = h;
        return button;
    }

    public static function soundButtonGaz(textureName: String): Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        var button:ExtentedButton = new ExtentedButton();
        button.hitAreaPadding = 5;
        var defaultImage:Image =  new Image(buttonTexture);
        button.defaultSkin = defaultImage;
        button.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xc0c2c2);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;
        button.touchable = true;
        button.label = "";
        return button;
    }

    public static function soundButtonEngine(textureName : String, textureInactive: String) : ToggleButton
    {
        var button : ToggleButton = new ToggleButton();
        var defaultTexture: Texture = assetManager.getTexture(textureInactive);
        var downTexture: Texture = assetManager.getTexture(textureName);
        var defaultImage: Image = new Image(defaultTexture);
        var downImage: Image = new Image(downTexture);
        button.defaultSkin = defaultImage;
        button.defaultSelectedSkin = downImage;
        button.alpha = 1;
        return button;
    }

    public static function circleButton(textureName:String, downAlpha:Number = 0.6, button:ExtentedButton = null, hitAreaPadding:int = 5):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        button ||= new ExtentedButton();
        button.hitAreaPadding = hitAreaPadding;

		var defaultImage:Image =  new Image(buttonTexture);
        if(button.defaultSkin){
            button.defaultSkin.dispose();
        }
        button.defaultSkin = defaultImage;

        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = downAlpha;
        if(button.downSkin){
            button.downSkin.dispose();
        }
        button.downSkin = downImage;
        button.disabledSkin = downImage;
        button.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xc0c2c2);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;
        button.label = "";

		var w:Number;
		var h:Number;
        if (textureName == "a_burger_button") {
			w =  Device.isIos() || assetManager.scaleFactor < 3.375 ? 34 : 32;
			h =  Device.isIos() || assetManager.scaleFactor < 3.375 ? 34 : 32;
            if(textureName == "a_burger_button") {
                button.color = 0x0055aa;
            }

        }
        else {
            w = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
            h = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
        }

		button.width = w;
		button.height = h;

        return button;
    }

    public static function getMainMenuProfileButtonSize():Number{
        return Device.isIos() || assetManager.scaleFactor <= 3.375 ? 36 : 30;
    }

    public static function circleDefaultButton(iconTextureName:String, downAlpha:Number = 0.6, backgroundColor:uint = 0xFFFFFF, hitAreaPadding:int = 5, iconColor:int = -1, size:Number = 0):Button {
        var buttonTexture:Texture = assetManager.getTexture("circle_button_bg");
        var iconTexture:Texture = assetManager.getTexture(iconTextureName);

        var button:ExtentedButton = new ExtentedButton();
        button.hitAreaPadding = hitAreaPadding;

		var defaultImage:Image =  new Image(buttonTexture);
        button.defaultSkin = defaultImage;

        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = downAlpha;
        button.downSkin = downImage;

        button.color = backgroundColor;

        var defaultIcon:Image =  new Image(iconTexture);
        if(iconColor != -1){
            defaultIcon.color = iconColor;
        }
        button.defaultIcon = defaultIcon;
        if(size){
            button.width = size;
            button.height = size;
        }else{
            button.width = defaultImage.width;
            button.height = defaultImage.height;
        }
        return button;
    }

    public static function circleDefaultButton2(iconTextureName:String, downAlpha:Number = 0.6, backgroundColor:uint = 0xFFFFFF, hitAreaPadding:int = 5, iconColor:int = -1, size:Number = 0):Button {
        var buttonTexture:Texture = assetManager.getTexture("a_circle_button_bg");
        var iconTexture:Texture = assetManager.getTexture(iconTextureName);

        var button:ExtentedButton = new ExtentedButton();
        button.hitAreaPadding = hitAreaPadding;

		var defaultImage:Image =  new Image(buttonTexture);
        defaultImage.scale = assetManager.scaleFactor / 2;
        button.defaultSkin = defaultImage;

        var downImage:Image = new Image(buttonTexture);
        downImage.scale = assetManager.scaleFactor / 2;
        downImage.alpha = downAlpha;
        button.downSkin = downImage;

        button.color = backgroundColor;

        var defaultIcon:Image =  new Image(iconTexture);
        defaultIcon.scale = assetManager.scaleFactor / 2;
        if(iconColor != -1){
            defaultIcon.color = iconColor;
        }
        button.defaultIcon = defaultIcon;
        return button;
    }

    public static function carChooserTopButton(label:String, name:String = ""):ToggleButton{
        var button:ToggleButton = new ToggleButton();
        var defaultSkin:Quad = new Quad(1,1,0x252525);
        var selectedSkin:Quad = new Quad(1,1,0xFFFFFF);
        button.name = name;
        button.height = 20;
        button.paddingLeft = button.paddingRight = 5.5;
        button.defaultSkin = defaultSkin;
        button.defaultSelectedSkin = selectedSkin;
        button.label = label;
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 9, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.embedFonts = true;
            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratSemiBold(), 9, 0x3d3d3d);
            return textRenderer;
        }
        return button;
    }


    // ***************** TUNING POPUP BUTTONS ***********************

    public static function tuningPopupCircleButton(textureId:String, noColor:Boolean = false): Button {
        var scale: Number = AlertView.assetManager.scaleFactor / 2;
        var button:Button = new MultilingualButton();
        var buttonTexture:Texture = ButtonFactory.assetManager.getTexture(textureId);

        var buttonSkin:Image = new Image(buttonTexture);
        buttonSkin.scale = scale;
        var buttonSelectedSkin:Image = new Image(buttonTexture);
        buttonSelectedSkin.scale = scale;
        buttonSelectedSkin.alpha = 0.6;
        button.defaultSkin = buttonSkin;
        button.downSkin = buttonSelectedSkin;
        button.disabledSkin = buttonSelectedSkin;
        button.validate();
        //
        if(noColor){
            buttonSkin.color = buttonSelectedSkin.color = 0x000000;
        }
        //
        return button;
    }

    public static function deleteButton(): Button {
        return tuningPopupCircleButton("a_delete_bg_button");
    }

    public static function editButton(): Button {
        return tuningPopupCircleButton("tuning_edit_btn", true);
    }

    public static function newTuningButton(): Button {
        return tuningPopupCircleButton("tuning_new_btn", true);
    }

    public static function startButton(): Button {
        return tuningPopupCircleButton("tuning_start_btn");
    }

    public static function filterButton(): Button {
        return tuningPopupCircleButton("tuning_filter_btn");
    }

    public static function postToTimelineButton(): Button {
        return tuningPopupCircleButton("tuning_post_btn");
    }

    public static function startChallengeButton() : Button{
        return tuningPopupCircleButton("request_challenge_btn");
    }
	
	
	public static function transparentButton():Button {
		var buttonTexture:Texture = Texture.fromColor(32,32,0xffffff,0);
		//buttonTexture.forDelete = false;
		/* if (buttonTexture is SubTexture) {
		(buttonTexture as SubTexture).parent.forDelete = false;
		}*/
		var button:ExtentedButton = new ExtentedButton();
		button.hitAreaPadding = 5;
		
		var defaultImage:Image =  new Image(buttonTexture);
		//defaultImage.
		button.defaultSkin = defaultImage;
		
		var downImage:Image = new Image(buttonTexture);
		//downImage.alpha = downAlpha;
		
		button.downSkin = downImage;
		button.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xc0c2c2);
		button.textFormat.align = TextFormatAlign.CENTER;
		button.textFormat.bold = true;
		button.label = "";
		
		
		var w:Number;
		var h:Number;
		
		
		button.width = w;
		button.height = h;
		
		//downImage.width = w;
		//downImage.height = h;
		
		//defaultImage.width = w;
		//defaultImage.height = h;
		
		
		
		
		return button;
	}


    public static function avatarButton(textureName:String, lb:String):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureName);
        //buttonTexture.forDelete = false;
        var button:ExtentedButton = new ExtentedButton();
        button.touchable = false;
        button.hitAreaPadding = 5;
        button.defaultSkin = new Image(buttonTexture);
        button.downSkin = new Image(buttonTexture);
        button.textFormat = new TextFormat(FontSelector.getFont(), 20, 0xFFFFFF);
        button.textFormat.align = TextFormatAlign.CENTER;
        button.textFormat.bold = true;
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 18, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        }
        button.label = lb;
        //button.width = button.height = 42;
        button.width = Device.isIos() || assetManager.scaleFactor < 3.375 ? 38 : 32;
        button.height = Device.isIos() || assetManager.scaleFactor < 3.375 ? 38 : 32;

        return button;
    }


    public static function menuOfShareButton(textureName:String):ShareMenuButton {
        var buttonTexture:Texture = assetManager.getTexture(textureName);

        var button:ShareMenuButton = new ShareMenuButton();
        button.hitAreaPadding = 5;
        button.defaultSkin = new Image(buttonTexture);

        var downImage:Image = new Image(buttonTexture);
        downImage.alpha = 0.6;

        button.downSkin = downImage;
        button.width = buttonTexture.width;
        button.height = buttonTexture.height;

        //if(DeviceCapabilities.isTablet(Starling.current.nativeStage))
        //{
        button.width = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
        button.height = Device.isIos() || assetManager.scaleFactor <= 3.375 ? 30 : 25;
        //}

        return button;
    }


    public static function rewardsButton(rewardCount: Number): Button {
        var buttonTexture: Texture = assetManager.getTexture('rewards_ads_icon');
        var buttonImageDefault: Image = new Image(buttonTexture);

        var button: ExtentedButton = new ExtentedButton();
        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getGoldBold(), 14, 0x0eb4ec);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };
        buttonImageDefault.height = 35;
        buttonImageDefault.scale = buttonImageDefault.scaleY;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDefault;
        button.width = buttonImageDefault.width;
        button.height = buttonImageDefault.height;
        button.label = rewardCount.toString();
        button.horizontalAlign = HorizontalAlign.CENTER;
        button.paddingRight = buttonImageDefault.width * .45;
        button.hitAreaPadding = 10;
        return button;
    }

    public static function buyPacketByReward(price: Number): MultilingualButton {

        var buttonTexture:Texture = assetManager.getTexture("btn_buy_now");
        //var buttonTextures:Scale3Textures = new Scale3Textures(buttonTexture, 60, 10, Scale3Textures.DIRECTION_HORIZONTAL);
        var buttonImage:Image = new Image(buttonTexture);

        buttonImage.scale9Grid = new Rectangle(60,0,10, buttonTexture.frameHeight);

        var buttonImage2:Image = new Image(buttonTexture);
        buttonImage2.scale9Grid = new Rectangle(60,0,10, buttonTexture.frameHeight);
        buttonImage2.alpha = 0.6;

        var button:MultilingualButton = new MultilingualButton();

        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(true), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };

        // button.label = 'USE 1 3DT COIN';
        button.label = 'USE           COINS';
        button.width = 104;
        button.height = buttonImage.height;
        button.defaultSkin = buttonImage;
        button.downSkin = buttonImage2;
        button.verticalAlign = VerticalAlign.MIDDLE;
        //
        var img:Image = new Image(assetManager.getTexture("one_coin_img"));
        // img.height = button.height - 10;
        img.scale = 0.85 * ButtonFactory.assetManager.scaleFactor / 2; // img.scaleY;
        img.x = 37;
        img.y = (button.height - img.height) / 2 - 0.5;
        button.addChild(img);
        return button;

    }



    public static function rewardWatchButton(koef: Number = 1): MultilingualButton {
        if (koef > 1) {
            koef = 1;
        }
        var buttonTexture: Texture = assetManager.getTexture('watch');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width * koef;
        button.height = buttonImageDefault.height * koef;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }

    public static function rewardLaterButton(koef: Number = 1): MultilingualButton {
        if (koef > 1) {
            koef = 1;
        }
        var buttonTexture: Texture = assetManager.getTexture('later');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width * koef;
        button.height = buttonImageDefault.height * koef;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }


    public static function rewardNoButton(koef: Number = 1): MultilingualButton {
        if (koef > 1) {
            koef = 1;
        }
        var buttonTexture: Texture = assetManager.getTexture('no');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width * koef;
        button.height = buttonImageDefault.height * koef;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }



    public static function rewardConfirmButton(): MultilingualButton {
        var buttonTexture: Texture = assetManager.getTexture('confirm');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width;
        button.height = buttonImageDefault.height;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }



    public static function rewardUseCoinButton(count: Number): MultilingualButton {
        var buttonTexture: Texture = assetManager.getTexture('btn_buy_now');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();

        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 9, 0x000000);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };

        button.width = buttonImageDefault.width;
        button.height = buttonImageDefault.height;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;

        button.label = 'USE ' + count + ' 3DT COIN';
        return button;

    }



    public static function rewardCancelButton(): MultilingualButton {
        var buttonTexture: Texture = assetManager.getTexture('cancel');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width;
        button.height = buttonImageDefault.height;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }

    public static function rewardOkButton(): MultilingualButton {
        var buttonTexture: Texture = assetManager.getTexture('ok');
        var buttonImageDefault: Image = new Image(buttonTexture);
        var buttonImageDown: Image = new Image(buttonTexture);
        buttonImageDown.alpha = .8;
        var button: MultilingualButton = new MultilingualButton();
        button.width = buttonImageDefault.width;
        button.height = buttonImageDefault.height;
        button.defaultSkin = buttonImageDefault;
        button.downSkin = buttonImageDown;
        return button;
    }

    public static function rewardsPopUpTitle(): MultilingualButton {
        var button: MultilingualButton = new MultilingualButton();
        var iconTexture: Texture = assetManager.getTexture('rewards_icon');
        var iconImage: Image = new Image(iconTexture);

        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratLight(), 18, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };

        button.label = "REWARDED ADS in 3DT";
        button.height = iconTexture.height;

        button.defaultIcon = iconImage;
        button.gap = 10;
        return button;
    }

    public static function challengeAcceptButton(label:String, textureId:String = null,  textColor:uint = 0xFFFFFF):Button {
        var buttonTexture:Texture = assetManager.getTexture(textureId||"accept_btn_bg");
        var buttonImage:Image = new Image(buttonTexture);
        var button:MultilingualButton = new MultilingualButton();
        buttonImage.scale = buttonImage.texture.scale/2;
        buttonImage.color = 0xFF0000;
        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, textColor);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };
        button.label = label;
        button.defaultSkin = buttonImage;
        return button;
    }

    public static function challengeDeclineButton(label:String):Button {
        var buttonTexture:Texture = assetManager.getTexture("decline_btn_bg");
        var buttonImage:Image = new Image(buttonTexture);
        var button:MultilingualButton = new MultilingualButton();
        buttonImage.scale = buttonImage.texture.scale/2;
        buttonImage.color = 0xFF0000;
        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };
        button.label = label;
        button.defaultSkin = buttonImage;
        return button;
    }


    public static function tuningDeleteButton(label:String, iconId:String):ToggleButton {
        var scale:Number = assetManager.scaleFactor / 2;
        var buttonTexture:Texture = assetManager.getTexture("round_btn_border");
        Image.bindScale9GridToTexture(buttonTexture, new Rectangle(buttonTexture.frameWidth * 30/65, buttonTexture.frameHeight / 2, buttonTexture.frameWidth * 5/65, 0));
        var buttonImage:Image = new Image(buttonTexture);

        var buttonTextureSel:Texture = assetManager.getTexture("round_btn_border_sel");
        Image.bindScale9GridToTexture(buttonTextureSel, new Rectangle(buttonTextureSel.frameWidth * 30/65, buttonTextureSel.frameHeight / 2, buttonTextureSel.frameWidth * 5/65, 0));
        var buttonSelImage:Image = new Image(buttonTextureSel);

        var button:ToggleButton = new ToggleButton();

        var iconImage:Image = new Image(assetManager.getTexture(iconId));
        iconImage.scale = (buttonTexture.frameHeight / 30) * scale;
        //
        button.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), int(11 / scale), 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };
        button.labelOffsetX = 2;
        button.iconOffsetX = -3;
        button.label = label;
        button.defaultSkin = buttonImage;
        button.defaultSelectedSkin = buttonSelImage;
        button.defaultIcon = iconImage;
        button.height = buttonTexture.frameHeight;
        button.scale = 30 / button.height;
        return button;
    }

    public static function createTuningMenuHideButton():ToggleButton{
        var _showBanner:ToggleButton = new ToggleButton();

        var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_open_menu"));
        defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _showBanner.defaultIcon = defaultIcon;

        defaultIcon = new Image(ButtonFactory.assetManager.getTexture("a_open_menu"));
        defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        defaultIcon.alpha = 0.6;
        _showBanner.downIcon = defaultIcon;

        var selectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_close_menu"));
        selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _showBanner.defaultSelectedIcon = selectedIcon;

        selectedIcon = new Image(ButtonFactory.assetManager.getTexture("a_close_menu"));
        selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        selectedIcon.alpha = 0.6;
        _showBanner.selectedDownIcon = selectedIcon;
        _showBanner.width = selectedIcon.width;
        _showBanner.height = selectedIcon.height + 10;
        _showBanner.iconOffsetY = -5;
        return _showBanner;
    }

    public static function createLeaderBoardRegionButton(texture:String, name:String, iconHeight:Number = 0, colorizeIcon:Boolean = true, padding:Boolean = true):ToggleButton{
        var toggleButton:ToggleButton;
        var defaultIcon:Image;
        var selectedIcon:Image;
        var defaultSkin:Quad;
        var selectedSkin:Quad;


        toggleButton = new ToggleButton();
        if(padding){
            toggleButton.paddingLeft = 5;
            toggleButton.paddingRight = 10;
            toggleButton.labelOffsetX = 5;
        }

        toggleButton.name = name;
        defaultIcon = new Image(ButtonFactory.assetManager.getTexture(texture));
        defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
        if(colorizeIcon){
            defaultIcon.color = 0x999999;
        }

        var ratio:Number = iconHeight / defaultIcon.height;
        if(ratio > 0){
            defaultIcon.height = iconHeight;
            defaultIcon.width *= ratio;
        }

        toggleButton.defaultIcon = defaultIcon;
        defaultSkin = new Quad(1,1, 0xFFFFFF);
        defaultSkin.width = 35;
        defaultSkin.height = 22;
        if(ratio){
            defaultSkin.width = defaultIcon.width + 10;
        }
        toggleButton.defaultSkin = defaultSkin;

        selectedIcon = new Image(ButtonFactory.assetManager.getTexture(texture));
        if(colorizeIcon){
            selectedIcon.color = 0xFEFFFF;
        }
        selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
        if(ratio > 0){
            selectedIcon.height = iconHeight;
            selectedIcon.width *= ratio;
        }
        toggleButton.defaultSelectedIcon = selectedIcon;
        selectedSkin = new Quad(1, 1, 0x026edc);
        selectedSkin.width = defaultSkin.width;
        selectedSkin.height = defaultSkin.height;
        toggleButton.defaultSelectedSkin = selectedSkin;
        toggleButton.horizontalAlign = HorizontalAlign.LEFT;
        toggleButton.labelFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0x999999);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            textRenderer.selectedTextFormat = new TextFormat(FontSelector.getMontserratMedium(), 11, 0xFFFFFF);
            return textRenderer;
        };

        return toggleButton;
    }
}
}