package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.settings.SettingsProxy;
	import com.utils.FontSelector;

	import feathers.controls.List;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.core.FeathersControl;

	import flash.geom.Point;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.display.Quad;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;

	public class ListItemRenderer extends FeathersControl implements IListItemRenderer {
		public var fontSize : int;
		protected var label:MultilingualLabel;
		protected var imageLoader : ImageLoaderWithStates;
		protected var _index : int = -1;
		protected var _owner:List;
		protected var _data : Object;
		protected var _isSelected:Boolean;
		protected var useBitmapFont : Boolean;
		private var touchPointID : int;
		private var selectedByTouching : Boolean;
		private static const HELPER_POINT : Point = new Point();
		protected var _textColor : uint;
		protected var _bold : Boolean;
		private var  settingsProxy : SettingsProxy;
		private var lastLang:Language;
		protected var overflowMultiLang:Boolean=false;
		private var _clipRectQuad:Quad;
		public var itemSelectedAgainBlocker:Boolean = false;
		private var _autoResize:Boolean = true;
		
		public function ListItemRenderer() {
			super();
			
			fontSize = 7;
			textColor = 0xFFFFFF;

			addEventListener(TouchEvent.TOUCH, touchHandler);   
		}
		
		protected var _factoryID:String;
		public function get factoryID():String{
			return _factoryID;
		}
		
		
		public function set factoryID(value:String):void{
			_factoryID = value;
		}
	
		
		public function set textColor(value : uint) : void {
			_textColor = value;
			if(label) {
				label.textFormat.color=value;
				label.invalidate();
			}
		}
		
		public function set bold(value : Boolean) : void {
			_bold = value;
			if(label) {
				label.textFormat.font=FontSelector.getFont(_bold);     
				label.textFormat.bold=_bold;
				label.invalidate();
			}
		}
		
		public function get textColor() : uint {
			return _textColor;
		}
		
		public override function dispose() : void {
			removeEventListener(TouchEvent.TOUCH, touchHandler);
			if(imageLoader){
				imageLoader.removeFromParent(true);
				imageLoader = null;
			}
			if(label){
				label.removeFromParent(true);
				label = null;
			}
			if(_clipRectQuad){
				_clipRectQuad.removeFromParent(true);
				_clipRectQuad = null;
			}
			if(_owner && _owner is ExtendedList){
				ExtendedList(_owner).removeItemRenderer(this);
			}
			settingsProxy = null;
			lastLang = null;
			super.dispose();
			_owner = null;
			_data = null;
		}
		
		public function get index():int {
			return _index;
		}
		
		public function set index(value : int) : void {
			if(_index == value) {
				return;
			}
			_index = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get owner() : List {
			return List(_owner);
		}
		
		public function set owner(value : List) : void {
			if(_owner == value) {
				return;
			}
			_owner = value;
			if(_owner is ExtendedList){
				ExtendedList(_owner).addItemRenderer(this);
			}
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		public function get data() : Object {
			return _data;
		}
		
		public function set data(value : Object) : void {
			if(_data == value) {
				return;
			}
			
			_data = value;
			invalidate(INVALIDATION_FLAG_DATA);
		}
		
		
		public function get isSelected() : Boolean {
			return _isSelected;
		}
		
		public function set isSelected(value : Boolean) : void {
			if(_isSelected == value) {
				if(_isSelected && selectedByTouching) {
					if(itemSelectedAgainBlocker) {
						itemSelectedAgainBlocker = false;
						return;
					}
					dispatchEventWith("itemSelectedAgain", true);
				}
				return;
			}
			
			itemSelectedAgainBlocker = false;
			_isSelected = value;
			invalidate(INVALIDATION_FLAG_SELECTED);
			dispatchEventWith(Event.CHANGE);
		}
		
		override protected function initialize() : void {
			super.initialize();
			_clipRectQuad = new Quad(1,1);
			this.lastLang=AlertView.language;
			label = new MultilingualLabel();
			if(!overflowMultiLang){
				label.textFormat = new TextFormat(FontSelector.getFont(_bold), fontSize, textColor);
			}else{
				label.textFormat = new TextFormat("NotoSans", fontSize, textColor);
			}
			label.textFormat.align = TextFormatAlign.CENTER;
			label.textFormat.bold=_bold;
			addChild(label);
			if(!overflowMultiLang)
			label.updateWithLanguage(AlertView.language);
			imageLoader = new ImageLoaderWithStates();
			imageLoader.overrideDispose = false;
			addChild(imageLoader);  
		}
		
		override protected function draw():void {
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var selectionInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SELECTED);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(lastLang!=AlertView.language && !overflowMultiLang){
				lastLang=AlertView.language;
				if(label)
				label.updateWithLanguage(AlertView.language);
				sizeInvalid=true;  
			}

			if(dataInvalid) {
				commitData();
			}
			
			if(selectionInvalid) {
				commitSelection();
			}
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			if(autoSizeNeeded || sizeInvalid) {
				layout();
			}
		}
		
		public function get useEmbeddedFont() : Boolean {
			return Language.supportsEmbeddedFont;
		}
		
		protected function autoSizeIfNeeded():Boolean {
			return false;
		}
		
		protected function commitData() : void {
			
		}
		
		protected function commitSelection() :void {
			if(imageLoader){
				imageLoader.selected = isSelected;
			}
		}
		
		protected function layout():void {
			if(_clipRectQuad){
				_clipRectQuad.width = actualWidth;
				_clipRectQuad.height = actualHeight;
				this.mask = _clipRectQuad;
			}
			if(label){
				label.width = actualWidth;
				label.height=NaN;
				label.validate();
				label.y = actualHeight - label.height;
			}

			if(imageLoader && _autoResize){
				imageLoader.width = actualWidth;
				imageLoader.height = actualHeight;
                imageLoader.height = actualHeight + 5;
			}
		}
		
		protected function touchHandler(event : TouchEvent):void {
			try{
				const touches : Vector.<Touch> = event.getTouches(this);
				for each(var touch : Touch in touches) {
					if(touch.phase == TouchPhase.ENDED) {
						touch.getLocation(stage, HELPER_POINT);
						const isInBounds : Boolean = contains(stage.hitTest(HELPER_POINT));
						if(isInBounds) {
							selectedByTouching = true;
							isSelected = true;
							selectedByTouching = false;         
						}
						return;
					}					
				}
			}catch(e:Error){
				trace("touchHandler error:"+e.message+" stack:"+e.getStackTrace());
			}
		}

		public function get autoResize():Boolean {
			return _autoResize;
		}

		public function set autoResize(value:Boolean):void {
			_autoResize = value;
		}

		public function updateOrientationDependantView():void{

		}
	}
}