package com.despark.tuning3d.view.components.core {
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;

    import flash.text.TextFormatAlign;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.textures.Texture;

    public class MainMenuItem extends Button {
    private var _noteCountGroup:LayoutGroup;
    private var _noteCountLabel:Label;
    public function MainMenuItem(textureName:String, prop:Object = null, countable:Boolean = false, iconColor:int = -1, size:Number = 20) {
        super();
        height = 30;
        paddingRight = 10;
        paddingLeft = 4;
        paddingBottom = 13;
        labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 14, 0x707070, TextFormatAlign.LEFT, false);

        defaultSkin = new Quad(1,1);
        defaultSkin.alpha = 0;

        var buttonTexture:Texture = ButtonFactory.assetManager.getTexture(textureName);
        var btnDefaultIcon:Image = new Image(buttonTexture);
        var btnDownIcon:Image = new Image(buttonTexture);
        btnDownIcon.alpha = 0.6;

        btnDefaultIcon.width = btnDefaultIcon.height = size;
        btnDownIcon.width = btnDownIcon.height = size;
        if(iconColor != -1){
            btnDefaultIcon.color = iconColor;
        }
        defaultIcon = btnDefaultIcon;
        downIcon = btnDownIcon;
        labelOffsetX = 5;
        if(countable){
            _noteCountGroup = MainMenuButton.generateNotificationIcon();
            _noteCountGroup.visible = false;
            _noteCountGroup.x = defaultIcon.x + defaultIcon.width - _noteCountGroup.width;
            _noteCountGroup.y = defaultIcon.y;
            _noteCountGroup.includeInLayout = false;
            for(var i:int = 0; i < _noteCountGroup.numChildren; i++){
                if(_noteCountGroup.getChildAt(i) is Label){
                    _noteCountLabel = _noteCountGroup.getChildAt(i) as Label;
                    break;
                }
            }
        }
    }

    override protected function initialize():void{
        super .initialize();
        if(_noteCountGroup){
            addChild(_noteCountGroup);
        }
    }

    override protected function draw():void{
        super .draw();
        if(_noteCountGroup && contains(_noteCountGroup) && getChildIndex(_noteCountGroup) < numChildren - 1){
            setChildIndex(_noteCountGroup, numChildren - 1);
        }
    }

    public function set noteCount(val:int):void{
        if(_noteCountGroup) {
            _noteCountGroup.visible = (val > 0);
            if(_noteCountLabel)  {
                _noteCountLabel.text = FeedData.getNumberString(val);
                _noteCountLabel.validate();
                _noteCountGroup.width = Math.max(_noteCountLabel.width + 8, _noteCountGroup.height);
                _noteCountGroup.x = defaultIcon.x + defaultIcon.width - _noteCountGroup.width / 2;
                _noteCountGroup.y = defaultIcon.y - _noteCountGroup.height / 2;
            }
        }
    }

    public function get noteCount():int{
        if(_noteCountLabel)  {
            return parseInt(_noteCountLabel.text);
        }
        return 0;
    }
}
}
