package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.packages.Packet;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;
	import com.distriqt.AdsManager;
	import com.distriqt.GameService;
	import com.utils.DateParser;
	import com.utils.FontSelector;
	import com.utils.Tracker;

	import feathers.controls.Button;
	import feathers.controls.Label;
	import feathers.controls.ScrollBar;
	import feathers.controls.ScrollContainer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.layout.Direction;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.VerticalLayout;

	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import mx.utils.StringUtil;

	import starling.animation.Transitions;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.utils.AssetManager;

	public class PacketView extends FeathersControl implements Multilingual
	{

		public static const BUY_NOW_PACKET : String = "buyNowPacket";
		public static const VIEW_CAR_360 : String = "VIEW_CAR_360";
		public static const SHOW_NO_COIN_POP_UP: String = 'SHOW_NO_COIN_POP_UP';
		public static const SHOW_CONFIRM_POP_UP: String = 'SHOW_CONFIRM_POP_UP';

		public static var assetManager : AssetManager;
		public static var language : Language;


		private var _titleDisplay:TextFieldTextRenderer = null;

		private var _background:Quad = null;

		private var _buyNowButton:MultilingualButton = null;

		private var _unlockButton: MultilingualButton = null;
		private var _unlockOneButton: MultilingualButton = null;

		private var _or2Label:TextFieldTextRenderer;
		private var _or1Label:TextFieldTextRenderer;

		private var _diamondLabel:ExtendedLabel;
		private var _diamondOneLabel:ExtendedLabel;

		private var _timeLeftLabel:Label;
		private var _modelCountLabel:Label;
		private var _purchasedLabel:Label;
		private var _purchasedImage:Image;



		private var _descriptionLabel:TextFieldTextRenderer;
		private var _packet:Packet=null;

		private var _hideX:Number=0;
		private var _showX:Number=0;

		private var container : ScrollContainer;
		private var quadDown:Quad;
		private var _priceSize:int = 11;
		private var _titleFontSize:int = 21;
		private var _priceDisplay:Label;
		private var _packageImage:ImageLoaderWithStates;
		//
		private static const FOOTER_HEIGHT:Number = 65;
		public function PacketView() {
			super();
			if(Constants.SCALE_FACTOR==4){
				_priceSize=8;
				_titleFontSize=18;
			}
		}

		public function get showX():Number{
			return _showX;
		}

		public function set showX(value:Number):void{
			_showX=value;
		}

		public function get hideX():Number{
			return _hideX;
		}

		public function set hideX(value:Number):void{
			_hideX=value;
		}

		public function set packet(value:Packet):void{
			_packet = value;
			_priceSize = 11;
			_titleFontSize = 21;
			if(Constants.SCALE_FACTOR==4){
				_priceSize = 8;
				_titleFontSize = 18;
			}
			invalidate("packet");
		}

		public function get packet():Packet{
			return _packet;
		}

		public function updateWithLanguage(language:Language):void {
			if(_buyNowButton){
				_buyNowButton.label=AlertView.language.translate("mob_home", "buynow").toLocaleUpperCase();
			}
		}

		protected override function initialize() : void {
			super.initialize();
			_background=new Quad(1, 1,0xFFFFFF);
			addChild(_background);

			container = new ScrollContainer();
			var containerLayout:VerticalLayout = new VerticalLayout();
			containerLayout.horizontalAlign = HorizontalAlign.CENTER;
			containerLayout.paddingTop = 5;
			containerLayout.gap = 0;
			container.layout = containerLayout;
			container.verticalScrollBarFactory = function ():ScrollBar {
				var scrollbar:ScrollBar = new ScrollBar();
				scrollbar.direction = Direction.VERTICAL;
				scrollbar.thumbFactory = function ():Button {
					var button:Button = new Button();
					var sprite:Sprite=new Sprite();
					var back:Quad = new Quad(10, 25, 0xffffff);
					back.alpha=0;
					var quad:Quad = new Quad(3, 25, 0xc1c3c3);
					sprite.addChild(back);
					sprite.addChild(quad);
					button.defaultSkin = sprite;
					return button;
				}
				return scrollbar;
			}
			addChild( container );


			_titleDisplay=new TextFieldTextRenderer();
			var titleFont:TextFormat = new TextFormat(FontSelector.getAlumniSansMedium(), _titleFontSize, 0x454545);
			titleFont.align = TextFormatAlign.CENTER;
			_titleDisplay.textFormat = titleFont;
			_titleDisplay.wordWrap = true;
			_titleDisplay.isHTML = true;
			_titleDisplay.embedFonts = true;
			_titleDisplay.width = actualWidth;
			container.addChild(_titleDisplay);

			_modelCountLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 13, 0x4a4a4a, TextFormatAlign.CENTER, false);
			_modelCountLabel.width = actualWidth;
			_modelCountLabel.paddingTop = -5;
			container.addChild(_modelCountLabel);


			_buyNowButton = ButtonFactory.defaultButton("BUY NOW", 0xff8400);
			_buyNowButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
			_buyNowButton.addEventListener(Event.TRIGGERED, buyNowButtonTriggeredHandler);
			_buyNowButton.width = 100;
			_buyNowButton.height = 23;
			addChild(_buyNowButton);
			_buyNowButton.validate();

			var lifeTimeLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 8, 0x535353, TextFormatAlign.CENTER, false);
			lifeTimeLabel.text = 'Lifetime';
			lifeTimeLabel.includeInLayout = false;
			lifeTimeLabel.y = _buyNowButton.height;
			lifeTimeLabel.width = _buyNowButton.width;
			_buyNowButton.addChild(lifeTimeLabel);

			_priceDisplay = LabelFactory.createSimpleLabel(FontSelector.getMontserratBold(), _priceSize, 0x454545, TextFormatAlign.CENTER, false);
			_priceDisplay.includeInLayout = false;
			_buyNowButton.addChild(_priceDisplay);


			_unlockOneButton = ButtonFactory.defaultButton("UNLOCK 1 CAR", 0xff0000, "button_gradient");
			_unlockOneButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
			_unlockOneButton.addEventListener(Event.TRIGGERED, buyCoin3DSingleButtonTriggeredHandler);
			_unlockOneButton.width = 100;
			_unlockOneButton.height = 23;
			addChild(_unlockOneButton);
			_unlockOneButton.validate();

			var unlockOneTimeLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 8, 0x535353, TextFormatAlign.CENTER, false);
			unlockOneTimeLabel.text = "One Time Access";
			unlockOneTimeLabel.includeInLayout = false;
			unlockOneTimeLabel.y = _unlockOneButton.height;
			unlockOneTimeLabel.width = _unlockOneButton.width;
			_unlockOneButton.addChild(unlockOneTimeLabel);

			_diamondOneLabel = new ExtendedLabel();
			_diamondOneLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getGoldBold(), 14, 0x4a4a4a, TextFormatAlign.RIGHT, false);
			_diamondOneLabel.labelOffsetY = -2;
			_diamondOneLabel.paddingTop = 1;
			_diamondOneLabel.includeInLayout = false;
			_diamondOneLabel.text = "2";
			_unlockOneButton.addChild(_diamondOneLabel);
			_diamondOneLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
			_diamondOneLabel.defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 3;
			_diamondOneLabel.labelOffsetX = -4;

			_or1Label = new TextFieldTextRenderer();
			_or1Label.textFormat = new TextFormat(FontSelector.getMontserratRegular(), _priceSize, 0x535353);
			_or1Label.wordWrap = false;
			_or1Label.textFormat.align = TextFormatAlign.CENTER;
			_or1Label.embedFonts = true;
			_or1Label.text = 'or';
			_or1Label.validate();
			_or1Label.includeInLayout = false;
			addChild(_or1Label);


			_unlockButton = ButtonFactory.defaultButton("UNLOCK ALL", 0xff0000, "button_gradient");
			_unlockButton.labelFactory = LabelFactory.createTextRendererFactory(FontSelector.getAlumniSansMedium(), 18, 0xFFFFFF, TextFormatAlign.CENTER, false);
			_unlockButton.addEventListener(Event.TRIGGERED, buyCoin3DButtonTriggeredHandler);
			_unlockButton.width = 100;
			_unlockButton.height = 23;
			addChild(_unlockButton);
			_unlockButton.validate();

			var unlockTimeLabel:Label = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 8, 0x535353, TextFormatAlign.CENTER, false);
			unlockTimeLabel.text = 'For ' + DateParser.getInChallengeLabel(GameService.REWARDED_PACKAGE_ACTIVE_DURATION, {sec:["second", "seconds"], min:["minute","minutes"], hr:["hour","hours"], day:["day", "days"]});
			unlockTimeLabel.includeInLayout = false;
			unlockTimeLabel.y = _unlockButton.height;
			unlockTimeLabel.width = _unlockButton.width;
			_unlockButton.addChild(unlockTimeLabel);

			_diamondLabel = new ExtendedLabel();
			_diamondLabel.textRendererFactory = LabelFactory.createTextRendererFactory(FontSelector.getGoldBold(), 14, 0x4a4a4a, TextFormatAlign.RIGHT, false);
			_diamondLabel.labelOffsetY = -2;
			_diamondLabel.paddingTop = 1;
			_diamondLabel.includeInLayout = false;
			_diamondLabel.text = "0";
			_unlockButton.addChild(_diamondLabel);
			_diamondLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
			_diamondLabel.defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 3;
			_diamondLabel.labelOffsetX = -4;

			_or2Label = new TextFieldTextRenderer();
			_or2Label.textFormat = new TextFormat(FontSelector.getMontserratRegular(), _priceSize, 0x535353);
			_or2Label.wordWrap = false;
			_or2Label.textFormat.align = TextFormatAlign.CENTER;
			_or2Label.embedFonts = true;
			_or2Label.text = 'or';
			_or2Label.validate();
			_or2Label.includeInLayout = false;
			addChild(_or2Label);


			_descriptionLabel = new TextFieldTextRenderer();
			var descFormat:TextFormat = new TextFormat(FontSelector.MontserratHtml, _priceSize, 0x535353);
			descFormat.align = TextFormatAlign.LEFT;
			descFormat.blockIndent = 0;
			descFormat.indent = 0;
			descFormat.leftMargin = 0;
			_descriptionLabel.textFormat = descFormat;
			_descriptionLabel.wordWrap = true;
			_descriptionLabel.isHTML = true;
			_descriptionLabel.embedFonts = true;
			container.addChild(_descriptionLabel);

			quadDown=new Quad(1,10,0xffffff);
			quadDown.alpha=0;
			container.addChild(quadDown);

			_purchasedLabel = LabelFactory.createSimpleLabel(FontSelector.getAlumniSansMedium(), 18, 0x4a4a4a, TextFormatAlign.CENTER, false);
			_purchasedLabel.includeInLayout = false;
			addChild(_purchasedLabel);

			_timeLeftLabel = LabelFactory.createSimpleLabel(FontSelector.getMontserratMedium(), 8, 0x4a4a4a, TextFormatAlign.CENTER, false);
			_timeLeftLabel.includeInLayout = false;
			addChild(_timeLeftLabel);
		}

		public static var PACKET_VIEW_WIDTH:Number;

		private function viewCar360(event:Event):void{
			var car:Car = event.data.car;
			dispatchEventWith(VIEW_CAR_360, false, {car:car});
		}

		private function buyNowPacket(event:Event):void{
			dispatchEventWith(BUY_NOW_PACKET, false, {packet:packet});
		}

		private function buyCoin3DButtonTriggeredHandler(event: Event = null): void {
			GameService.$.getRewardsData(function (rewards: int): void {
				Tracker.getInstance().trackEvent("click_buy_width_reward", 'click_buy_width_reward', 'click_buy_width_reward');
				if (rewards >= packet.coin3dFee) {
					dispatchEventWith(SHOW_CONFIRM_POP_UP);
				} else {
					Starling.juggler.delayCall(function ():void {
						AdsManager.$.rewardsCallback = buyCoin3DButtonTriggeredHandler;
						dispatchEventWith(SHOW_NO_COIN_POP_UP);
					}, 0.25);
				}
			})
		}

		private function buyCoin3DSingleButtonTriggeredHandler(event: Event = null): void {
			GameService.$.getRewardsData(function (rewards: int): void {
				Tracker.getInstance().trackEvent("click_buy_width_reward", 'click_buy_width_reward', 'click_buy_width_reward');
				if (rewards >= Packet.ONE_TIME_PACKET_FEE) {
					dispatchEventWith(SHOW_CONFIRM_POP_UP, false, {coin3dFee: Packet.ONE_TIME_PACKET_FEE, oneTimeUnlocked:true});
				} else {
					Starling.juggler.delayCall(function ():void {
						AdsManager.$.rewardsCallback = buyCoin3DSingleButtonTriggeredHandler;
						dispatchEventWith(SHOW_NO_COIN_POP_UP);
					}, 0.25);
				}
			})
		}

		private function buyNowButtonTriggeredHandler(event : Event) : void {
			dispatchEventWith(BUY_NOW_PACKET, false, {packet:packet});
		}

		public function hideRight(animation:Boolean=false):void{
			if(animation){
				var tween : Tween = new Tween(this, 0.65, Transitions.EASE_IN_OUT);
				tween.moveTo(this.showX+this.actualWidth, 0);//fadeTo(1.0);
				tween.onComplete=completeHide;
				Starling.juggler.add(tween);
			}else{
				completeHide();
			}
		}

		private var _completeHideCallback:Function=null
		public function hide(animation:Boolean=false, complete:Function=null):void{
			_completeHideCallback=complete;
			if(animation){
				SoundManager.playSound(SoundManager.SELECT_PART_BACK);
				var tween : Tween = new Tween(this, 0.35, Transitions.EASE_IN_OUT);
				tween.moveTo(_hideX, 0);//fadeTo(1.0);
				tween.onComplete=completeHide;
				Starling.juggler.add(tween);
			}else{
				completeHide();
			}
		}

		override public function setSize(width:Number, height:Number):void{
			super.setSize(width, height);
		}

		private var _completeShowCallback:Function=null;

		public function show(animation:Boolean=false, complete:Function=null):void{

			_completeShowCallback=complete;
			if(this.container)
				this.container.verticalScrollPosition=0;

			if(animation){

				this.x=_hideX;
			}else{

				this.x=_showX;
			}

			this.visible=true;

			if(animation){
				var tween : Tween = new Tween(this, 0.65, Transitions.EASE_IN_OUT);
				tween.moveTo(_showX, 0);

				tween.onComplete=completeShow;
				Starling.juggler.add(tween);
			}else{

				completeShow();
			}
		}


		private function completeHide():void{
			visible=false;
			if(_completeHideCallback){
				_completeHideCallback();
				_completeHideCallback=null;
			}

			dispatchEventWith(Event.CLOSE);

		}

		public function updateLists():void{
			if(_titleDisplay){
				invalidate();
			}
		}

		private function completeShow():void{

			if(_completeShowCallback){

				_completeShowCallback();
				_completeShowCallback=null;
			}

			if(this.container)
				this.container.verticalScrollPosition=0;

			this.dispatchEventWith(Event.OPEN);
		}

		private function layoutBottomButtons():void{
			quadDown.x = 0;
			quadDown.y = 0;
			container.readjustLayout();

			if(_unlockButton.visible){
				_buyNowButton.x = _or2Label.x + (_or2Label.width + 10);
			}else{
				_buyNowButton.x = (actualWidth - _buyNowButton.width) / 2 + 8;
			}
			_buyNowButton.y = actualHeight - (FOOTER_HEIGHT + _buyNowButton.height) / 2;
		}

		private function layoutTopButtons():void{
			var fieldWidth:Number = actualWidth - 80;
			_titleDisplay.width = fieldWidth;
			_modelCountLabel.width = fieldWidth;

			_titleDisplay.y = 5;
			_titleDisplay.validate();
			_modelCountLabel.x = _titleDisplay.x + (_titleDisplay.width - _modelCountLabel.width) / 2;

			_buyNowButton.validate();
			_or2Label.validate();
			_unlockButton.validate();

			var itemGap:Number = 10;
			var buttonGroupWidth:Number = _buyNowButton.width + _or1Label.width + _unlockOneButton.width + _or2Label.width + _unlockButton.width + 4 * itemGap;
			_unlockOneButton.x = (actualWidth - buttonGroupWidth) / 2 + 10;
			_unlockOneButton.y = actualHeight - (FOOTER_HEIGHT + _unlockOneButton.height) / 2;

			_or1Label.x = _unlockOneButton.x + (_unlockOneButton.width + itemGap);
			_or1Label.y = _unlockOneButton.y + (_unlockOneButton.height - _or1Label.height) / 2;

			_unlockButton.x = _or1Label.x + (_or1Label.width + itemGap);
			_unlockButton.y = actualHeight - (FOOTER_HEIGHT + _unlockButton.height) / 2;

			_or2Label.x = _unlockButton.x + (_unlockButton.width + itemGap);
			_or2Label.y = _unlockButton.y + (_unlockButton.height - _or2Label.height) / 2;

			container.width = actualWidth;
			container.height = actualHeight - FOOTER_HEIGHT;
			container.x = 8;
			container.y = 0;

			_background.x = 0;
			_background.y = 0;
			_background.width = actualWidth;
			_background.height = actualHeight;

			_descriptionLabel.width = fieldWidth;

			_purchasedLabel.width = fieldWidth;
			_purchasedLabel.x = (actualWidth - _purchasedLabel.width)/2;
			_purchasedLabel.y = actualHeight - (FOOTER_HEIGHT + _purchasedLabel.height) / 2;

			if(_purchasedImage){
				_purchasedImage.x = (actualWidth - _purchasedImage.width) / 2;
				_purchasedImage.y = _purchasedLabel.y - _purchasedLabel.height;
			}

			_timeLeftLabel.width = fieldWidth;
			_timeLeftLabel.x = (actualWidth - _timeLeftLabel.width) / 2;
			_timeLeftLabel.y = _purchasedLabel.y + _purchasedLabel.height + 1;
		}

		private function updateRewardPacketTimeLeft():void{
			GameService.$.getRewardedPacketUnlockTime(packet.id, function (time:Number):void {
				if(time != 0){
					if(packet.oneTimeUnlocked){
						_timeLeftLabel.text = "ONE CAR";
					}else{
						_timeLeftLabel.text = DateParser.getInChallengeLabel(GameService.REWARDED_PACKAGE_ACTIVE_DURATION - (new Date().time - time), {sec:["second", "seconds"], min:["minute","minutes"], hr:["hour","hours"], day:["day", "days"]}) + " left";
					}
					_timeLeftLabel.validate();
					_timeLeftLabel.x = (actualWidth - _timeLeftLabel.width) / 2;
					_timeLeftLabel.y = _purchasedLabel.y + _purchasedLabel.height + 1;
				}
			});
		}

		private function setRewardsButtonVisibility(value:Boolean):void{
			_unlockOneButton.visible = _or1Label.visible = value;
			_unlockButton.visible = _or2Label.visible = value;
		}

		override protected function draw() : void {
			super.draw();
			if(isInvalid("packet")){
				if(_packet && _titleDisplay){
					var title:String;
					if(AlertView.language) {
						title = AlertView.language.forceTranslate(packet.id, "title_long");
						title ||= AlertView.language.forceTranslate(packet.id, "title");
					}
					title ||= packet.title;

					_titleDisplay.text = title.toUpperCase();
					if(_packet.cars && _packet.cars.length && _packet.projectType){
						_modelCountLabel.text = _packet.cars.length.toString() + " MODEL"+(_packet.cars.length>1?"S":"")+" AVAILABLE";
					}else{
						_modelCountLabel.text = AlertView.language.forceTranslate(packet.id, "caption");
					}
					layoutTopButtons();

					var currency:String="";
					if(packet.feeCurrency){
						currency = packet.feeCurrency.replace("₽", "RUB");
					}
					currency=currency.replace("$", "USD");
					_priceDisplay.text = packet.fee.toFixed(2) +" "+StringUtil.trim(currency);
					_priceDisplay.width = _buyNowButton.width;
					_priceDisplay.validate();
					_priceDisplay.x = (_buyNowButton.width - _priceDisplay.width) / 2;
					_priceDisplay.y = -3 -_priceDisplay.height;
				}

				if(_packet && _descriptionLabel){
					var body:String=_packet.body;
					if(AlertView.language) {
						body = AlertView.language.translate(packet.id, "body");
					}

					_descriptionLabel.text = body.replace(/\s*<br>\s*/mi, '<br><font size="6"><br></font>'); //'<textformat rightmargin="3" >'+body+'</textformat>';
					_descriptionLabel.validate();
				}
				if(_packet){
					if(_packageImage){
						_packageImage.removeFromParent(true);
						_packageImage = null;
					}
					var imageUrl:String = AlertView.language.translate(packet.id, "image");
					if(imageUrl && imageUrl != "image"){
						_packageImage = new ImageLoaderWithStates();
						_packageImage.paddingTop = _packageImage.paddingBottom = 15;
						var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
						simpleImageProvider.urlForDownloading = imageUrl;
						_packageImage.persistentImageProvider = simpleImageProvider;
						_packageImage.completeHandler = function(image:ImageLoaderWithStates=null):void{
							image.width = actualWidth - 100;
						}
						container.addChildAt(_packageImage, container.getChildIndex(_descriptionLabel));
					}
					if(_purchasedImage){
						_purchasedImage.removeFromParent(true);
					}
					if(!packet.lock){
						if(packet.tempOpened){
							_purchasedImage = new Image(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
							_purchasedImage.width = 24;
							_purchasedImage.scale = _purchasedImage.scaleX;
							_purchasedLabel.text = "YOU HAVE UNLOCKED THIS PACKAGE";
							updateRewardPacketTimeLeft();
						}else{
							_purchasedImage = new Image(ButtonFactory.assetManager.getTexture("car_unlock_big_icon"));
							_purchasedImage.color = 0xff8c11;
							_purchasedImage.width = 25;
							_purchasedImage.scale = _purchasedImage.scaleX;
							_purchasedLabel.text = "YOU HAVE PURCHASED THIS PACKAGE";
							_timeLeftLabel.text = "Life time";
							_timeLeftLabel.validate();
						}
						_purchasedLabel.validate();
						_purchasedLabel.x = (actualWidth - _purchasedLabel.width)/2;
						_purchasedLabel.y = actualHeight - (FOOTER_HEIGHT + _purchasedLabel.height) / 2;
						//
						if(_purchasedImage){
							addChild(_purchasedImage);
							_purchasedImage.x = (actualWidth - _purchasedImage.width) / 2;
							_purchasedImage.y = _purchasedLabel.y - _purchasedLabel.height;
						}

						_timeLeftLabel.x = (actualWidth - _timeLeftLabel.width) / 2;
						_timeLeftLabel.y = _purchasedLabel.y + _purchasedLabel.height + 1;
					}

				}
			}

			if(_packet && _buyNowButton){
				if(_packet.lock){
					_purchasedLabel.visible = _timeLeftLabel.visible = false;
					_buyNowButton.visible = true;
					GameService.$.rewardsIsSupport(function (status: Boolean): void {
						setRewardsButtonVisibility(status);
						if (!_packet.coin3dFee || _packet.coin3dFee === 0) {
							setRewardsButtonVisibility(false);
						}
						layoutBottomButtons();
					});
				}else{
					setRewardsButtonVisibility(false);
					_buyNowButton.visible = false;
					_purchasedLabel.visible = _timeLeftLabel.visible = true;
				}

				if (!_packet.coin3dFee || _packet.coin3dFee === 0) {
					setRewardsButtonVisibility(false);
				}else{
					_diamondLabel.text = _packet.coin3dFee.toString();
					_diamondLabel.validate();

					_diamondLabel.x = (_unlockButton.width - _diamondLabel.width) / 2 + 2;
					_diamondLabel.y = -_diamondLabel.height - 3;

					_diamondOneLabel.x = (_unlockOneButton.width - _diamondOneLabel.width) / 2 + 2;
					_diamondOneLabel.y = -_diamondOneLabel.height - 3;
				}
			}

			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				layoutTopButtons();
				layoutBottomButtons();
			}

		}
	}
}