package com.despark.tuning3d.view.components.core
{
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.view.components.brands.BrandsScreen;

import feathers.controls.Button;
import feathers.core.FeathersControl;

import starling.display.Quad;
import starling.events.Event;
import starling.utils.AssetManager;

//import com.despark.tuning3d.view.components.tunings.editing.AdsButton1;

public class SearchView extends FeathersControl implements Multilingual
	{
		public static var assetManager : AssetManager;
		public static var language : Language;
		
		private var image:Quad;
		
		public  var minimizedState:ExtentedButton;
		private var maximizedState:ExtentedButton;
		private var searchIcon:Button;
		
		private var _text : String = "";
		private var _language:Language = null;
		
		private var textInput : TextInputExtended;
		private var _stateMaximized : Boolean;
		private var _owner : BrandsScreen;
		
		private var _isOpened : Boolean;
		private var _inputW : Number;
		
		public function SearchView(obj:*)
		{
			_owner = obj;
			super();
		}
		
		public function get isOpened():Boolean{
			return _isOpened;
		} 
		public function set isOpened(value:Boolean):void{
			_isOpened = value;
		}
		
		public function get inputW():Number{
			return _inputW;
		} 
		public function set inputW(value:Number):void{
			_inputW = value;
		}
		
		public function get owner():BrandsScreen{
			return _owner;
		} 
		public function set owner(value:BrandsScreen):void{
			_owner = value;
		}
		
		public function get stateMaximized():Boolean{
			return _stateMaximized;
		} 
		public function set stateMaximized(value:Boolean):void{
			_stateMaximized = value;
		}
		
		public function get text():String {
			return textInput ? textInput.text : _text; 
		}
		public function set text(value:String):void{
			_text = value;
			invalidate("text");
		}
				
		public function updateWithLanguage(lng:Language):void
		{
			_language = lng;	
			if (minimizedState){
				minimizedState.label = _language.translate("mob_home","com_search");
			}
			
			if (maximizedState){
				maximizedState.label = _language.translate("mob_home","com_search");
			}
		}
		
		protected override function initialize() : void
		{
			super.initialize();
			
			isOpened = false;
			stateMaximized = false;
			
			image = new Quad(1,1,0xffffff);
			
			minimizedState = ButtonFactory.searchButton("model_search_open","Search make and model") as ExtentedButton;
			minimizedState.addEventListener(Event.TRIGGERED, onSearchToggle);
			minimizedState.width = actualWidth;
			addChild(minimizedState);
			
			maximizedState = ButtonFactory.searchButton("model_search_open_bg","Search make and model") as ExtentedButton;
			maximizedState.addEventListener(Event.TRIGGERED, onSearchToggle);
			addChild(maximizedState);
			
			searchIcon = ButtonFactory.searchButton("src");
			searchIcon.alpha = .85;
			searchIcon.addEventListener(Event.TRIGGERED, onSearchToggle);
			addChild(searchIcon);
			
			textInput = new TextInputExtended(false, true, this);
			textInput.visible = false;
			textInput.owner = this;
			textInput.addEventListener(Event.CHANGE, onTextChange);
			addChild(textInput); 
			
			updateStates();
		}
		
		protected function onTextChange(event:Event) : void {
			dispatchEventWith(Event.CHANGE);
		}
		
		//var tt : int = 0; 
		public function externalCallbeck() : void 
		{
			//tt++;
			image.visible = false;
			textInput.visible  = false;
			textInput.text = "";

            if (!Constants.isIOS){
                textInput.textInput.clearFocus();
			}
			updateStates();
		}
		
		public function updateStates(clear:*=-1) : void
		{
			searchIcon.scaleX = searchIcon.scaleY = .8;
			if (stateMaximized)
			{
				maximizedState.width = actualWidth;
				searchIcon.x = maximizedState.x + 10;
				
				inputW = maximizedState.width - searchIcon.x - searchIcon.width - 5;
				if (!textInput.isIOS && textInput.clearButton){
					inputW = maximizedState.width - searchIcon.x - searchIcon.width - 5 - 1.5 * textInput.clearButton.width;
				}
				
				minimizedState.visible = false;
				maximizedState.visible = true;
			}
			else
			{
				minimizedState.width = actualWidth;
				searchIcon.x = minimizedState.x + 20;
				
				inputW = minimizedState.width - searchIcon.x - searchIcon.width - 5;
				if (!textInput.isIOS && textInput.clearButton){
					inputW = minimizedState.width - searchIcon.x - searchIcon.width - 5 - 1.5 * textInput.clearButton.width;
				}
				
				minimizedState.visible = true;
				maximizedState.visible = false;
			}
			
			searchIcon.y = (actualHeight - searchIcon.height) / 2;
			
			if (clear != -1) {
				textInput.visible = false;
				updateWithLanguage(AlertView.language);
			}
			
			if (textInput.visible)
			{
				textInput.x = searchIcon.x + searchIcon.width + 3;
				
				textInput.width = inputW;
				if (textInput.clearButton){
					textInput.clearButton.x =  inputW + 3; 
					//textInput.y =  searchIcon.y + 1.5 + 4;
				}
				
				textInput.dispatchEventWith(Event.CHANGE);
			}
		}
		
		public function onSearchToggle(event:Event=null):void
		{
			textInput.visible = !textInput.visible;
			
			if (textInput.visible){
				minimizedState.label = "";
				maximizedState.label = "";
				textInput.textInput.setFocus();
			}
			else{
				updateWithLanguage(_language);
				textInput.text = "";
				textInput.textInput.clearFocus();
			}
			
			updateStates();
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			//invalidate();
			
			if (isInvalid(INVALIDATION_FLAG_SIZE))
			{
				maximizedState.height = actualHeight;
				minimizedState.height = actualHeight;
				
				searchIcon.scaleX = searchIcon.scaleY = .8;
				
				if (stateMaximized)
				{
					maximizedState.width = actualWidth;
					
					searchIcon.x = maximizedState.x + 10;
					textInput.x = searchIcon.x + searchIcon.width + 3;
					
					textInput.width = maximizedState.width - searchIcon.x - searchIcon.width - 5;
					if (!textInput.isIOS && textInput.clearButton){
						textInput.width = maximizedState.width - searchIcon.x - searchIcon.width - 5 - 1.5 * textInput.clearButton.width;
					}
					
					minimizedState.visible = false;
					maximizedState.visible = true;
				}
				else
				{
					minimizedState.width = actualWidth;
					searchIcon.x = minimizedState.x + 20;
					textInput.x = searchIcon.x + searchIcon.width + 3;
					
					textInput.width = minimizedState.width - searchIcon.x - searchIcon.width - 5;
					if (!textInput.isIOS && textInput.clearButton){
						textInput.width = minimizedState.width - searchIcon.x - searchIcon.width - 5 - 1.5 * textInput.clearButton.width;
					}
					
					minimizedState.visible = true;
					maximizedState.visible = false;
				}
				
				
				
				searchIcon.y = (actualHeight - searchIcon.height) / 2;
				
				textInput.height = actualHeight;
				textInput.y = 0;
				
				/*if (textInput.clearButton){
					textInput.clearButton.x = inputW + 3;
					textInput.y = searchIcon.y + 1.5 + 4;
				}*/
				
				/*if (Constants.isIOS){
					textInput.y = (searchIcon.y + 3 + textInput.height / 2) - 8; 
				}*/
				
				//textInput.y = Constants.isIOS ? textInput.y - 8 : textInput.y + 4;
				
			}
			
			//trace("searchView.scale", this.scale)
		}
	}
}