package com.despark.tuning3d.view.components.core {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.ChallengeUser;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesLoadBar;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.distriqt.AdsManager;
    import com.utils.NumberUtils;
    import com.utils.NumberUtils;
    import com.utils.Tracker;

    import feathers.controls.LayoutGroup;
    import feathers.core.FeathersControl;
    import feathers.layout.AnchorLayout;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.RelativePosition;

    import flash.geom.Point;
    import flash.utils.setTimeout;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;

    public class StackedBannerScreen extends StackedScreen implements IMainMenuHolder {
        public var _mainMenuView:MainMenuView;
        private var _languages:Array;
        private var _selectedLanguage:Language;
        protected var _topMenuContainer:LayoutGroup;

        private var _loadBar:ChallengesLoadBar;
        //
        private var _diamondsLabel:ExtendedLabel;
        private var _keysLabel:ExtendedLabel;
        private var _ratingLabel:ExtendedLabel;
        private var _borders:Vector.<LayoutQuad> = new Vector.<LayoutQuad>();
        //
        private var _user:ChallengeUser;
        private static var _userRating:Number = 0;
        private static var _userKeys:int = 0;
        private static var _userDiamonds:int = 0;
        private var _rewardNotificationImage:Image;


        //
        protected static const HEADER_BG_COLOR:uint = 0x111111;
        //
        public static const CHALLENGES_LEADERBOARD_SELECTED:String = "CHALLENGES_LEADERBOARD_SELECTED";
        public static const CHALLENGES_HOME_SELECTED:String = "CHALLENGES_HOME_SELECTED";
        public static const CHALLENGES_RATE_SELECTED:String = "CHALLENGES_RATE_SELECTED";
        public static const CHALLENGES_GARAGE_SELECTED:String = "CHALLENGES_GARAGE_SELECTED";
        public static const CHALLENGES_NOTIFICATIONS_SELECTED:String = "CHALLENGES_NOTIFICATIONS_SELECTED";
        public static const CHALLENGES_HELP_SELECTED:String = "CHALLENGES_HELP_SELECTED";
        public static const UPDATE_USER_DATA:String = "UPDATE_USER_DATA";
        public function StackedBannerScreen(name : String = 'ChallengesScreen') {
            super(name);
            _loadBar = new ChallengesLoadBar();
            _loadBar.name = '_loadBar';
        }

        public function set mainMenuView(value:MainMenuView):void {
            _mainMenuView = value;
            _mainMenuView.owner = this;
        }

        public function get mainMenuView():MainMenuView {
            return _mainMenuView;
        }

        public function set languages(value:Array):void {
            _languages = value;
        }

        public function get languages():Array {
            return _languages;
        }

        public function get selectedLanguage():Language {
            return this._selectedLanguage;
        }

        public function set selectedLanguage(value:Language):void {
            this._selectedLanguage = value;
        }

        override protected function initialize():void {
            super.initialize();

            setSize(stage.stageWidth, stage.stageHeight);
            initTopMenuContainer();

            _rewardNotificationImage = new Image(assetManager.getTexture('notification'));
            _rewardNotificationImage.name = 'notificationReward';
            _rewardNotificationImage.visible = false;
            _rewardNotificationImage.touchable = false;
            _rewardNotificationImage.width = _rewardNotificationImage.height = 4.5;
            addChild(_rewardNotificationImage);
            var stackedScreen:StackedBannerScreen = this;
            AdsManager.$.rewardLimitCallback = function():void{
                var rewardsLimitDialogue:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.DAILY_REWARDS_LIMIT_REACHED, null, stackedScreen);
                rewardsLimitDialogue.addEventListener(ChallengeAlertEvent.CHALLENGES_TRIGGERED, statsHomeSelectHandler);
                rewardsLimitDialogue.open();
            };

            AdsManager.$.rewardUpdateCallback = function():void{
                var challengeProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
                challengeProxy.updateRewardData(UsersProxy.loggedInUser().id, AdsManager.$.rewardAdsStats);
            };

            AdsManager.$.checkRewardAds(function(status:Boolean):void{
                updateRewardsAdsStatus();
            });
        }

        public function updateRewardsAdsStatus():void{
            if(_rewardNotificationImage){
                _rewardNotificationImage.visible = AdsManager.$.haveRewardAds;
            }
        }

        public function showLoadBar():void{
            if(_loadBar && !_loadBar.stage){
                addChild(_loadBar);
                updateLoadBar();
            }
        }

        public function hideLoadBar():void{
            if(_loadBar && _loadBar.stage){
                _loadBar.removeFromParent();
            }
        }

        override public function transitionComplete():void{
            super .transitionComplete();
            BannerStackedScreen.showAdBanner();
        }


        override protected function feathersControl_addedToStageHandler(event:Event):void{
            super .feathersControl_addedToStageHandler(event);
            Starling.juggler.delayCall(updateLoadBar, 0.25);
        }

        private function updateLoadBar():void{
            if(_loadBar && _loadBar.stage){
                _loadBar.setSize(contentWidth, contentHeight);
                _loadBar.x = 0;
                _loadBar.y = marginTop;
                _loadBar.parent.setChildIndex(_loadBar, _loadBar.parent.numChildren);
                _loadBar.visible = contentWidth;
                _loadBar.validate();
            }
        }

        protected function initTopMenuContainer():void {
            var activeZone:Point = new Point(7,15);

            _topMenuContainer = new LayoutGroup();
            _topMenuContainer.height = marginTop;
            _topMenuContainer.layout = new AnchorLayout();

            _topMenuContainer.backgroundSkin = new Quad(1, 1, HEADER_BG_COLOR);
            addChild(_topMenuContainer);

            _diamondsLabel = new ExtendedLabel();
            _diamondsLabel.textRendererFactory = LabelFactory.challengeRatingLabel;
            _diamondsLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_rewards_icon"));
            _diamondsLabel.defaultIcon.scale = 0.85 * ButtonFactory.assetManager.scaleFactor / 2;
            if(Constants.isWideScreen){
                _diamondsLabel.labelYOffset = 2;
                _diamondsLabel.paddingLeft = 0;
                _diamondsLabel.paddingRight = 0;
            }else{
                _diamondsLabel.labelOffsetY = -2;
                _diamondsLabel.labelOffsetX = 4;
            }
            _diamondsLabel.paddingTop = 2;
            _diamondsLabel.text = _userDiamonds.toString();
            _diamondsLabel.addEventListener(Event.TRIGGERED, diamondTriggeredHandler);
            _diamondsLabel.hitAreaPadding = activeZone;
            _diamondsLabel.horizontalAlign = HorizontalAlign.CENTER;
            addChild(_diamondsLabel);

            var border:LayoutQuad = new LayoutQuad(1, 1, 0x525252);
            border.width = 0.5;
            if(Constants.isWideScreen){
                border.height = 2 * BannerStackedScreen.BANNER_SIZE / 3;
            }else{
                border.height = BannerStackedScreen.BANNER_SIZE / 3;
            }
            addChild(border);
            _borders.push(border);


            _keysLabel = new ExtendedLabel();
            _keysLabel.textRendererFactory = LabelFactory.challengeRatingLabel;
            _keysLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_keys_icon"));
            _keysLabel.defaultIcon.scale = 0.85 * ButtonFactory.assetManager.scaleFactor / 2;
            if(Constants.isWideScreen){
                _keysLabel.labelYOffset = 4;
                _keysLabel.paddingLeft = _keysLabel.paddingRight = 0;
            }else{
                _keysLabel.labelOffsetX = 4;
            }
            _keysLabel.text = _userKeys.toString();
            _keysLabel.addEventListener(Event.TRIGGERED, keysTriggeredHandler);
            _keysLabel.hitAreaPadding = activeZone;
            _keysLabel.horizontalAlign = HorizontalAlign.CENTER;
            addChild(_keysLabel);

            border = new LayoutQuad(1, 1, 0x525252);
            border.width = 0.5;
            if(Constants.isWideScreen){
                border.height = 2 * BannerStackedScreen.BANNER_SIZE / 3;
            }else{
                border.height = BannerStackedScreen.BANNER_SIZE / 3;
            }
            addChild(border);
            _borders.push(border);


            _ratingLabel = new ExtendedLabel();
            _ratingLabel.textRendererFactory = LabelFactory.challengeRatingLabel;
            _ratingLabel.defaultIcon = new LayoutImage(ButtonFactory.assetManager.getTexture("challenge_stars_icon"));
            _ratingLabel.defaultIcon.scale = 0.85 * ButtonFactory.assetManager.scaleFactor / 2;
            if(Constants.isWideScreen){
                _ratingLabel.paddingLeft = 0;
                _ratingLabel.paddingRight = 10;
                _ratingLabel.labelYOffset = 4;
            }else{
                _ratingLabel.labelOffsetX = 3;
            }
            _ratingLabel.text = NumberUtils.toFixed(_userRating,2);
            _ratingLabel.addEventListener(Event.TRIGGERED, ratingTriggeredHandler);
            _ratingLabel.hitAreaPadding = activeZone;
            _ratingLabel.horizontalAlign = HorizontalAlign.CENTER;
            addChild(_ratingLabel);

            if(Constants.isWideScreen){
                _diamondsLabel.iconPosition = RelativePosition.TOP;
                _keysLabel.iconPosition = RelativePosition.TOP;
                _ratingLabel.iconPosition = RelativePosition.TOP;
            }
        }

        private function ratingTriggeredHandler(event:Event):void {
            var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.STARS, null, this);
            statsDialog.open();
            statsDialog.addEventListener(ChallengeAlertEvent.CHALLENGES_TRIGGERED, statsHomeSelectHandler);
        }

        private function keysTriggeredHandler(event:Event):void {
            var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.KEYS, null, this);
            statsDialog.open();
            statsDialog.addEventListener(ChallengeAlertEvent.VOTING_TRIGGERED, statsRateSelectHandler);
        }

        private function diamondTriggeredHandler(event:Event):void {
            var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.DIAMONDS, null, this);
            statsDialog.open();
            statsDialog.addEventListener(ChallengeAlertEvent.CHALLENGES_TRIGGERED, statsHomeSelectHandler);
            statsDialog.addEventListener(ChallengeAlertEvent.REWARDS_TRIGGERED, rewardsButtonTriggeredHandler);
        }

        private function rewardsButtonTriggeredHandler(event: Event): void {
            SoundManager.playSound(SoundManager.PART_MENU);
            dispatchEventWith(StackedScreen.DISPLAY_REWARD_INFO_POPUP);
        }

        private function statsRateSelectHandler(event:Event):void {
            dispatchEvent(new Event(CHALLENGES_RATE_SELECTED));
        }

        private function statsHomeSelectHandler(event:Event):void {
            dispatchEvent(new Event(CHALLENGES_HOME_SELECTED));
        }

        override protected function draw():void {
            super.draw();

            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);

            if (!mainMenuView) {

            }

            if (sizeInvalid) {
                if (mainMenuView) {
                    mainMenuView.width = MainMenuView.MENU_WIDTH;
                    mainMenuView.parentActualWidth = actualWidth;
                }

                _topMenuContainer.width = actualWidth;
                _topMenuContainer.validate();
                statsBarLayout();
            }
        }

        private function statsBarLayout():void{
            var gap:Number = 8;
            if(Constants.isWideScreen){
                gap = 7;
            }
            if(_diamondsLabel && _keysLabel && _ratingLabel) {
                _diamondsLabel.validate();
                _keysLabel.validate();
                _ratingLabel.validate();
                if(Constants.isWideScreen){
                    _ratingLabel.x = actualWidth - _ratingLabel.width;
                }else{
                    _ratingLabel.x = actualWidth - (_ratingLabel.width + _sideMargin);
                }
                _borders[0].x = _ratingLabel.x - gap;
                _keysLabel.x = _ratingLabel.x - (_keysLabel.width + 2 * gap);
                _borders[1].x = _keysLabel.x - gap;
                _diamondsLabel.x = _keysLabel.x - (_diamondsLabel.width + 2 * gap);

                _diamondsLabel.y = _keysLabel.y = _ratingLabel.y = (BannerStackedScreen.BANNER_SIZE - _diamondsLabel.height) / 2;
                for (var i:int = 0; i < _borders.length; i++) {
                    _borders[i].y = (BannerStackedScreen.BANNER_SIZE - _borders[i].height) / 2;
                }

                _rewardNotificationImage.x = _diamondsLabel.x + _diamondsLabel.defaultIcon.width;
                _rewardNotificationImage.y = _diamondsLabel.y - _rewardNotificationImage.height;
            }
        }

        public function get contentWidth():Number{
            if(stage){
                return stage.stageWidth;
            }
            return 0;
        }

        public function get contentHeight():Number{
            if(stage){
                return stage.stageHeight - (marginTop + marginBottom);
            }
            return 0;
        }

        public static function get marginTop():Number{
            return 49;
        }

        public static function get marginBottom():Number{
            return 32.5;
        }

        override public function ShowDone():void {
            BannerStackedScreen.showAdBanner();

            Tracker.trackEvent("Challenges Screen", "open");
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        public function brandsScreenButtonTriggeredHandler(event:Event = null):void {
            dispatchEventWith(StackedScreen.BRANDS_SELECTED);
        }

        public function dispatchExternalEvent(msg:String):void {
            if (mainMenuView) {
                mainMenuView.hide();
            }
            dispatchEventWith(msg);
        }

        public function enterRowsState():void {
            Tracer.print('Challenge list doesn\'t have row state');
        }

        public function homeButtonTriggeredHandler(event:Event = null):void {
            dispatchEventWith(StackedScreen.HOME_SELECTED);
        }

        public function updateWithLanguage(language:Language):void {
            if (mainMenuView) mainMenuView.updateWithLanguage(language);
        }

        override public function dispose():void {
            super.dispose();
            if (_mainMenuView) {
                _mainMenuView.dispose();
                _mainMenuView = null;
            }
            if (_loadBar) {
                _loadBar.removeFromParent(true);
                _loadBar = null;
            }

            if (_topMenuContainer) {
                _topMenuContainer.removeFromParent(true);
                _topMenuContainer = null;
            }
            if (_diamondsLabel) {
                _diamondsLabel.removeFromParent(true);
                _diamondsLabel = null;
            }
            if (_keysLabel) {
                _keysLabel.removeFromParent(true);
                _keysLabel = null;
            }
            if (_ratingLabel) {
                _ratingLabel.removeFromParent(true);
                _ratingLabel = null;
            }
            if (_rewardNotificationImage) {
                _rewardNotificationImage.removeFromParent(true);
                _rewardNotificationImage = null;
            }
            if(_borders){
                while(_borders.length){
                    var border:LayoutQuad = _borders.pop() as LayoutQuad;
                    border.removeFromParent(true);
                }
                _borders = null;
            }

            _languages = null;
            _selectedLanguage = null;
            _user = null;
        }

        public function updateUserData(result:Object):void {
            if(result){
                if(result is String){
                    result = JSON.parse(result as String);
                }
            }
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.saveUserJSONData(result);

            if(_user){
                _user.updateData(result);
            }else{
                _user = new ChallengeUser(result);
                _user.onUpdate = function ():void {
                    _userRating = _user.rating;
                    _userKeys = _user.keys;
                    _userDiamonds = _user.diamonds;

                    if(_ratingLabel){
                        _ratingLabel.text = NumberUtils.toFixed(_user.rating, 2);
                    }
                    if(_keysLabel){
                        _keysLabel.text = _user.keys.toString();
                    }
                    if(_diamondsLabel){
                        _diamondsLabel.text = _user.diamonds.toString();
                    }
                    setTimeout(statsBarLayout, 50);
                }
                _user.onUpdate.call();
            }
            dispatchEventWith(UPDATE_USER_DATA);
        }

        public function get user():ChallengeUser {
            return _user;
        }
    }
}
