package com.despark.tuning3d.view.components.downloads
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.FontSelector;
	
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.Button;
	import feathers.controls.Label;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	
	import starling.events.Event;

	public class DownloadsProgressView extends FeathersControl
	{
		public static const CANCEL_SELECTED : String = "CancelSelected";
		
		public var language : Language;
		
		private var progressLabel : Label;
		private var infoLabel : MultilingualLabel;
		private var cancelButton : MultilingualButton;
		private var _progress : Number;
		
		public function DownloadsProgressView()
		{
			super();
		}
		
		public function get progress():Number
		{
			return _progress;
		}

		public function set progress(value : Number) : void
		{
			_progress = value;
			invalidate("progress");
		}

		protected override function initialize() : void
		{
			super.initialize();
			
			progressLabel = new Label();
			progressLabel.height = 25;
			addChild(progressLabel);
			
			progressLabel.textRendererFactory = function() : Object
			{
				var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(true), 20, 0xFFFFFF);
				textRenderer.textFormat.bold=true;
				textRenderer.textFormat.align = TextFormatAlign.CENTER;
				textRenderer.embedFonts = FontSelector.embedFonts();
				
				return textRenderer;
			}
				
			infoLabel = new MultilingualLabel();
			infoLabel.updateWithLanguage(language);
			infoLabel.text = language.translate("mob_downloads", "downloading");
			infoLabel.height = 20;
			addChild(infoLabel);
			
			infoLabel.textRendererFactory = function() : Object
			{
				var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 8, 0xFFFFFF);
				textRenderer.textFormat.align = TextFormatAlign.CENTER;
				textRenderer.embedFonts = Language.supportsEmbeddedFont;
				
				return textRenderer;
			}
				
			cancelButton = ButtonFactory.defaultButton(language.translate("mob_downloads", "stopDownload"), 0xB2BDC1);
			cancelButton.updateWithLanguage(language);
			cancelButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
			addChild(cancelButton);
		}
		
		protected override function draw() : void
		{
			super.draw();
			
			if(isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE))
			{
				progressLabel.width = actualWidth;
				progressLabel.y = -20 + int((actualHeight - progressLabel.height) / 2);
				
				infoLabel.width = actualWidth;
				infoLabel.y = progressLabel.y + progressLabel.height + 10;
				
				cancelButton.width = 200;
				cancelButton.y = actualHeight - cancelButton.height - 15;
				cancelButton.x = int((actualWidth - cancelButton.width) / 2);
			}
			
			if(isInvalid("progress"))
			{
				progressLabel.text = (progress * 100).toFixed(2) + "%";
			}
		}
		
		private function cancelButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(CANCEL_SELECTED);
		}
	}
}