package com.despark.tuning3d.view.components.gallery
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.users.User;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
	import com.utils.DisplayObjectUtil;

	import feathers.layout.HorizontalAlign;
	import feathers.layout.VerticalAlign;

	import flash.filesystem.File;

	import starling.core.Starling;
	import starling.events.TouchEvent;
	import starling.textures.TextureSmoothing;

	public class GalleryTuningCSItem extends GalleryTuningItem {
		private var backgroundImage:ImageLoaderWithStates=null;
		private var carTransform:Object;
		public function GalleryTuningCSItem(){

		}

		protected override function initialize() : void {
			super.initialize();
			backgroundImage = new ImageLoaderWithStates();
			backgroundImage.overrideDispose = true;
			backgroundImage.visible=false;
			addChildAt(backgroundImage, 0);
		}

        override public function dispose():void {
            DisplayObjectUtil.disposeMesh(clipRectQuad);
			clipRectQuad = null;
            if(backgroundImage) {
                if(backgroundImage.currentTexture){
                    backgroundImage.currentTexture.dispose();
				}
                backgroundImage.removeFromParent(true);
                backgroundImage = null;
            }

			if(userAvatar){
                userAvatar.removeFromParent(true);
                userAvatar = null;
			}
			super.dispose();
        }

        protected override function draw() : void {
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(!backgroundImage.mask){
				if(count == 1){
                    clipRectQuad.width = actualWidth;
                    clipRectQuad.height = actualHeight;
                    clipRectQuad.x = 0;
                    clipRectQuad.y = 0;
				}else{
                    if(horizontalAlign == HorizontalAlign.CENTER){
                        clipRectQuad.width = actualWidth - 2;
                        clipRectQuad.x = 1;
                    }else{
                        clipRectQuad.width = actualWidth - 1;
                        if(horizontalAlign == HorizontalAlign.RIGHT){
                            clipRectQuad.x = 1;
                        }
                    }

                    if(verticalAlign == VerticalAlign.MIDDLE){
                        clipRectQuad.height = actualHeight - 2;
                        clipRectQuad.y = 1;
                    }else{
                        clipRectQuad.height = actualHeight - 1;
                        if(verticalAlign == VerticalAlign.BOTTOM){
                            clipRectQuad.y = 1;
                        }
                    }
				}
                backgroundImage.mask = clipRectQuad;
			}

			if(sizeInvalid){
                backgroundImage.x = (actualWidth - backgroundImage.width) / 2;
                backgroundImage.y = (actualHeight - backgroundImage.height) / 2;
			}

			if(imageLoader.image){
                imgX = imageLoader.image.x;
                imgY = imageLoader.image.y;
                imgW = imageLoader.image.width;
                imgH = imageLoader.image.height;
			}
		}

		override protected function layout():void{
			super .layout();
			var bgScale:Number = 1;
			var imgScale:Number;
			if(backgroundImage && backgroundImage.image){
				bgScale = backgroundImage.image.scale;
			}
			if(imageLoader){
				imageLoader.visible = false;
				var isVisible:Boolean;
				if(bgScale == 1){
					bgScale = actualScale;
					isVisible = false;
					Starling.juggler.delayCall(layout, 0.1);
				}else{
					isVisible = true;
				}
				if(_imageWidth && _imageHeight){
					imgScale = bgScale * carTransform.zoomLevel;
					imageLoader.width = _imageWidth * imgScale;
					imageLoader.height = _imageHeight * imgScale;
					imageLoader.x = (actualWidth - imageLoader.width)/2 + carTransform.pan.x * imgScale + tuningDx;
					imageLoader.y = (actualHeight - imageLoader.height)/2 + carTransform.pan.y * imgScale + tuningDy;
					imageLoader.alpha = 0;
					imageLoader.visible = Boolean(imageLoader.image && backgroundImage.image) && isVisible;
					if(imageLoader.visible){
						imageLoader.image.color = 0xFF0000;
						Starling.juggler.tween(imageLoader, 0.15, {alpha: 1});
					}
				}
			}
		}

		protected function backgroundCompleteLoadImage(imageWidth:Number, imageHeight:Number):void{
			var imgRatio:Number = imageWidth / imageHeight;
			var itemRatio:Number = actualWidth / actualHeight;
            if(imgRatio < itemRatio) {
                backgroundImage.width = actualWidth;
                backgroundImage.height = actualWidth / imgRatio;
            }else{
                backgroundImage.height = actualHeight;
                backgroundImage.width = actualHeight * imgRatio;
            }
            backgroundImage.x = (actualWidth - backgroundImage.width) / 2;
            backgroundImage.y = (actualHeight - backgroundImage.height) / 2;
		}
		
		protected override function commitData() : void{
			super.commitData();
			var carBackground:CarBackground;
            var cameraIndex:int = 0;
			if(tuning && tuning.carTransform && tuning.carTransform.length){
				carTransform = tuning.carTransform[0];
			}
			carTransform ||= {zoomLevel: 1, pan: {x: 0, y: 0}};
			
			if(tuning && tuning.backgroundId && tuning.backgroundId != CarBackground.DEFAULT_ID && tuning.backgroundsMap && tuning.backgroundsMap[tuning.backgroundId]){
				carBackground= tuning.backgroundsMap[tuning.backgroundId];
                cameraIndex = 0;
				if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0 && carBackground.imagePathSecondCam && carBackground.imagePathSecondCam.length>0){
					cameraIndex = tuning.customSnapshotsCamera[0];
				}
				carBackground.showThumbnailUrl=false;
				carBackground.selectedCamera = cameraIndex;
				backgroundImage.smoothing = TextureSmoothing.BILINEAR;
				backgroundImage.visible = true;
			}else if(tuning){
				var randomSplashUrl:String;
				if(tuning && tuning.customSnapshotsBackgrounds && tuning.customSnapshotsBackgrounds[0]){
					carBackground = new CarBackground();
					carBackground.mainUrl = 'http:' + tuning.customSnapshotsBackgrounds[0].replace(/https?:/i, '');
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
				}else if(tuning && tuning.carChooserBgName) {
                    randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/" + tuning.carChooserBgName.replace('.jpg', '') + ".jpg").url;
                    if (tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length > 0) {
                        cameraIndex = tuning.customSnapshotsCamera[0];
                        if (tuning.carObj.defaultBkgMobile[cameraIndex]) {
                            randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
                        }
                    }
                    carBackground = new CarBackground();
                    carBackground.mainUrl = randomSplashUrl;
                    carBackground.isLocalFile = true;
                    carBackground.showThumbnailUrl = false;
                }else if(tuning && tuning.backgroundId && tuning.backgroundId.indexOf('data.3dtuning.com') != -1){
					if(tuning.backgroundId.indexOf('http:') == -1){
                        tuning.backgroundId = 'http:' + tuning.backgroundId;
					}
                    carBackground = new CarBackground();
                    carBackground.mainUrl = tuning.backgroundId;
                    carBackground.isLocalFile = true;
                    carBackground.showThumbnailUrl = false;
				}else{
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/car_chooser.jpg").url;
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
				}
				backgroundImage.persistentImageProvider = carBackground;
				backgroundImage.visible=true;
				backgroundImage.recieveImageSize = backgroundCompleteLoadImage;
			}
			if(tuning){
				var car:Car = new Car();
				car.imageURL = tuning.imageURL;
				imageLoader.recieveImageSize = completeLoadImage;
				if(tuning.userObj){
					_user = new User(tuning.userObj);
					if(count == 1){
                        userAvatar = new UserCircularAvatar(_user, 30, true, 1.5);
						imageLoader.bitmapScale = 1;
					}else if(count == 4){
                        userAvatar = new UserCircularAvatar(_user, 30, true, 1.5);
                        imageLoader.bitmapScale = 0.5;
					}else if(count == 9){
                        userAvatar = new UserCircularAvatar(_user, 25, true, 1.5);
                        imageLoader.bitmapScale = 0.3;
					}
					if(userAvatar){
						userAvatar.addEventListener(TouchEvent.TOUCH, avatarTouchHandler);
						addChild(userAvatar);
						userAvatar.x = actualWidth - userAvatar.width - 7;
                        if(count == 4 && verticalAlign == VerticalAlign.BOTTOM){
                            userAvatar.y = actualHeight - (userAvatar.height + 7);
                        }else{
                            userAvatar.y = 7;
                        }
					}
				}
                imageLoader.persistentImageProvider = car;
			}
		}

	private function get actualScale():Number{
		if(stage){
			if(actualWidth/actualHeight < stage.stageWidth / stage.stageHeight){
				return actualWidth / stage.stageWidth;
			}
			return actualHeight / stage.stageHeight;
		}
		return 0.3;
	}

	public function get tuningDx():Number{
		var dx:Number = customOffset.left;
		var ds:Number = 0.06 * (ButtonFactory.assetManager.scaleFactor - 1.5) / 2.5 + 0.24;
		if(tuning && tuning.car){
			/*if(!tuning.car.hdsize && tuning.car.carType == "moto" && false){
				ds = (carTransform.zoomLevel - 0.5) * 2 * 0.2 + 0.11;
				dx = 10 * ds;
			}else{*/
				dx *= ds;
			//}
		}
		return dx;
	}

	public function get tuningDy():Number{
		var dy:Number = customOffset.bottom;
		var ds:Number = 0.06 * (ButtonFactory.assetManager.scaleFactor - 1.5) / 2.5 + 0.24;
		return dy * ds + (0.73 * actualScale - 0.27) * dy;
	}

	private function get customOffset():Object{
		if(tuning){
			if(tuning.carObj && tuning.carObj.custom_offset){
				return tuning.carObj.custom_offset;
			}
			if(tuning.car && tuning.car.custom_offset){
				return tuning.car.custom_offset
			}
			return {bottom:0, left: 0};
		}
		return {bottom:0, left: 0};
	}

}
}
