package com.despark.tuning3d.view.components.mailbox {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.mails.MailsProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.gallery.*;
    import com.despark.tuning3d.view.components.mailbox.mails.MailBodyItemRenderer;
    import com.despark.tuning3d.view.components.mailbox.mails.MailDateItemRenderer;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentInput;
    import com.despark.tuning3d.view.components.timeline.ItemRenderers.UploadImageItemRenderer;
    import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
    import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
    import com.utils.DateParser;

    import feathers.controls.Button;
    import feathers.controls.Label;

    import feathers.controls.List;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;

    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.display.StageOrientation;

    import starling.core.Starling;
    import starling.display.DisplayObject;

    import starling.display.Image;
    import starling.events.Event;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;
    import starling.utils.MathUtil;

    public class MailBoxPopup extends PopUp implements Multilingual, IMainMenuHolder, ITuningListMenuVIew {
    private var _mainMenuView : MainMenuView;
    private var _languages:Array;
    private var _selectedLanguage:Language;
    private var _container:MainContainer;
    private var _closeHandler:Function;
    private var tuningView:TuningItemRenderer;
    private var _eventDispatcher:EventDispatcher;
    private var _bannerBackground:Image;
    private var _deleteButton:Button;

    private var _showBannerAfterExit:Boolean = false;
    private var _bannerScreen:String;
    private var _data:Object;
    private var _limit:int = 30;
    private var _skip:int = 0;
    private var _mailsProxy:MailsProxy;
    //
    private var _user:User;
    private var _currentUser:User;
    private var _userAvatar:UserCircularAvatar;
    private var _usernameLabel:FeathersControl;
    private var _noResultLabel:Label;
    private var _mailList:List;
    private var _postMailView:CommentInput;
    private var _isLoading:Boolean = false;
    //
    private var _dateList:Array;
    private var _messageIdList:Object;
    private var _imageList:List;
    private var _popupMode:Boolean = false;

    private static const NO_MESSAGE:String = "No messages in this conversation.";
    public function MailBoxPopup(data:Object = null, popupMode:Boolean = false) {
        _messageIdList = {};
        if(data.hasOwnProperty("toUser") && data.toUser is User){
            _user = data.toUser;
        }else{
            _data = data;
            _user = new User(_data);
            _user.id = _data._id;
        }
        _container = new MainContainer();
        _showBannerAfterExit = data.hasOwnProperty("showBannerAfterExit") && data.showBannerAfterExit;
        _bannerScreen = (data && data.hasOwnProperty("bannerScreen"))?data["bannerScreen"]:BannerScreen.BANNER_SCREEN;
        var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _currentUser = usersProxy.currentUser;
        FeedData.CURRENT_USER = _currentUser;
        //
        _popupMode = popupMode;
    }

    override protected function initialize():void {
        super.initialize();
        setSizeInternal(stage.stageWidth, stage.stageHeight, false);

        _dateList = [];
        if(contentBackgroundImage){
            contentBackgroundImage.removeFromParent();
        }

        background.color = 0xe9e1d3;
        background.alpha = 1;
        _container.setSize(actualWidth, actualHeight);

        _bannerBackground = new Image(Texture.fromColor(1,1, UIColors.BANNER_BACKGROUND_COLOR));
        addChild(_bannerBackground);

        var mainLayout : AnchorLayoutData = new AnchorLayoutData();
        _container.layoutData = mainLayout;
        if(background.parent && background.parent == this){
            addChildAt(_container, getChildIndex(_bannerBackground)+1);
        }

        var closeIcon:Image;
        if(_popupMode){
            closeIcon = new Image(ButtonFactory.assetManager.getTexture('garage_close'));
        }else{
            closeIcon = new Image(ButtonFactory.assetManager.getTexture('a_back_icon'));
        }

        var closeDownIcon:Image = new Image(closeIcon.texture);
        closeDownIcon.color = 0xFF0000;
        closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        closeDownIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        closeDownIcon.alpha = 0.6;
        closeButton.width = closeIcon.width;
        closeButton.height = closeIcon.height;
        closeButton.defaultIcon = closeIcon;
        closeButton.downIcon = closeDownIcon;
        closeButton.scaleWhenDown = 0.95;
        _container.addChild(closeButton);
        //
        _deleteButton = ButtonFactory.deleteButton();
        _deleteButton.visible = false;
        _deleteButton.addEventListener(Event.TRIGGERED, deleteTriggeredHandler);
        addChild(_deleteButton);
        //
        _userAvatar = new UserCircularAvatar(_user, BannerStackedScreen.BANNER_SIZE - 18, true, 1);
        _userAvatar.addEventListener(TouchEvent.TOUCH, profileTouchedHandler);
        addChild(_userAvatar);


        _usernameLabel = LabelFactory.createLabelMailPopupUsername();
        _usernameLabel.addEventListener(TouchEvent.TOUCH, profileTouchedHandler);
        LabelFactory.setLabelText(_usernameLabel, _user.nameToShow);
        addChild(_usernameLabel);
        //
        var mailListLayout : VerticalLayout = new VerticalLayout();
        mailListLayout.useVirtualLayout = false;
        mailListLayout.gap = 5;
        mailListLayout.paddingLeft = 0;
        mailListLayout.paddingRight = 0;
        mailListLayout.paddingBottom = 5;

        _mailList = new List();
        _mailList.layout = mailListLayout;
        _mailList.verticalScrollPolicy = ScrollPolicy.ON;
        _mailList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _mailList.addEventListener(Event.SCROLL, mailsListScrollHandler);
        _mailList.addEventListener(Event.CHANGE, mailsListChangeHandler);
        _mailList.dataProvider = new ListCollection();
        function regularItemFactory() : IListItemRenderer {
            var renderer : MailBodyItemRenderer = new MailBodyItemRenderer();
            renderer.currentUserId = _currentUser.id;
            renderer.width = _mailList.width;
            renderer.marginLeft = renderer.marginRight = MathUtil.clamp((actualWidth - MailsScreen.CONTENT_MAX_WIDTH) / 2, 0, MailsScreen.MARGIN_SIZE);
            renderer.eventDispatcher = this;
            return renderer;
        }
        function dateItemFactory() : IListItemRenderer {
            var dateRenderer : MailDateItemRenderer = new MailDateItemRenderer();
            dateRenderer.width = actualWidth;
            dateRenderer.height = 20;
            dateRenderer.eventDispatcher = this;
            return dateRenderer;
        }
        _mailList.setItemRendererFactoryWithID("message", regularItemFactory);
        _mailList.setItemRendererFactoryWithID("date", dateItemFactory);
        _mailList.factoryIDFunction = function(item:Object):String {
            return item.itemID;
        };
        addChild(_mailList);
        //
        _noResultLabel = new Label();
        _noResultLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
        _noResultLabel.text = NO_MESSAGE;
        _noResultLabel.visible = false;
        addChild(_noResultLabel);
        //
        _postMailView = new CommentInput(false, false);
        _postMailView.height = 45;
        _postMailView.addEventListener(FeedData.COMMENT_SEND, sendMessageHandler);
        _postMailView.addEventListener(CommentInput.UPDATE_UPLOADED_IMAGES, uploadPhotoButtonHandler);
        addChild(_postMailView);
        //
        _mailsProxy ||= ApplicationFacade.getInstance().retrieveProxy(MailsProxy.NAME) as MailsProxy;
        _isLoading = true;
        _mailsProxy.loadUserMessages(_user.id, _skip, _limit);
    }

        private function deleteTriggeredHandler(event:Event):void {
            if(_mailList.selectedItem && _mailList.itemToItemRenderer(_mailList.selectedItem) is MailBodyItemRenderer){
                CustomAlertView.show("Are you sure you want to delete?", {label:"Yes", callback: function():void{
                        _mailsProxy.deleteMessage(_mailList.selectedItem._id);
                        var selectedMail:Object = _mailList.selectedItem;
                        _mailList.dataProvider.removeItem(selectedMail);
                        _mailList.dataProvider.updateAll();
                        if(_mailList.dataProvider.length == 0){
                            _noResultLabel.visible = true;
                        }
                        if(_messageIdList.hasOwnProperty(selectedMail._id)){
                            delete _messageIdList[selectedMail._id];
                        }
                    }}, {label: "No", callback: function():void{}});
            }
        }

        private function profileTouchedHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(event.target as DisplayObject, TouchPhase.ENDED);
            if (touch){
                profileTriggeredHandler(null);
            }
        }

        private function profileTriggeredHandler(event:Event):void {
            Constants.selectedUser = _user;
            dispatchEventWith(StackedScreen.GARAGE_SELECTED);
        }

        private function sendMessageHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            _mailsProxy.sendMessageTo(_user.id, event.data.text, event.data.images);
        }

        private function mailsListScrollHandler(event:Event):void {
            if(!_isLoading){
                if (_mailList.verticalScrollPosition < 0) {
                    _isLoading = true;
                    _mailsProxy.loadUserMessages(_user.id, _skip, _limit);
                }else if (_mailList.verticalScrollPosition > _mailList.maxVerticalScrollPosition) {
                    _isLoading = true;
                    _mailsProxy.loadUserMessages(_user.id, 0, _limit, false);
                }

            }

        }

        private function mailsListChangeHandler(event:Event):void {
            if(_mailList.selectedItem){
                if (_mailList.itemToItemRenderer(_mailList.selectedItem) is MailBodyItemRenderer) {

                }else{
                    _mailList.selectedItem = null;
                }
            }
            _deleteButton.visible = _mailList.selectedItem;
        }

    override protected function closeButtonTriggeredHandler(event:Event):void {
        super.closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        if(_closeHandler != null){
            _closeHandler.call();
        }
        if(_eventDispatcher && ((_eventDispatcher is TimelineScreen && _showBannerAfterExit) || (_eventDispatcher is GalleryScreen))){
            BannerStackedScreen.showAdBanner();
        }
    }

    override protected function draw():void {
        super.draw();
        if(isInvalid(INVALIDATION_FLAG_SIZE)) {


            if (_bannerBackground) {
                _bannerBackground.width = actualWidth;
                _bannerBackground.height = BannerStackedScreen.BANNER_SIZE;
            }

            if (closeButton) {
                if (!closeButton.parent) {
                    addChild(closeButton);
                }
                closeButton.visible = true; // 9
                if (_popupMode) {
                    closeButton.x = actualWidth - (closeButton.width + 10);
                    closeButton.y = (BannerStackedScreen.BANNER_SIZE - closeButton.height) / 2;
                } else {
                    closeButton.x = closeButton.y = (BannerStackedScreen.BANNER_SIZE - closeButton.height) / 2;
                }
                if (_userAvatar) {
                    if (_popupMode) {
                        _userAvatar.x = 10;
                    } else {
                        _userAvatar.x = closeButton.x + closeButton.width + 14;
                    }
                    _userAvatar.y = (BannerStackedScreen.BANNER_SIZE - _userAvatar.height) / 2;
                }
                _usernameLabel.x = _userAvatar.x + _userAvatar.width + 4;
                _usernameLabel.y = (BannerStackedScreen.BANNER_SIZE - _usernameLabel.height) / 2;
            }

            if (_deleteButton) {
                _deleteButton.x = actualWidth - (_deleteButton.width + 14);
                _deleteButton.y = (BannerStackedScreen.BANNER_SIZE - _deleteButton.height) / 2;
            }

            if (_mailList) {
                _mailList.width = actualWidth;
                _mailList.height = actualHeight - BannerStackedScreen.BANNER_SIZE - _postMailView.height;
                _mailList.y = BannerStackedScreen.BANNER_SIZE;
            }

            if (_noResultLabel) {
                _noResultLabel.x = (actualWidth - _noResultLabel.width) / 2;
                if (_mailList) {
                    _noResultLabel.y = _mailList.y + (_mailList.height - _noResultLabel.height) / 2;
                } else {
                    _noResultLabel.y = (actualHeight - _noResultLabel.height) / 2;
                }
            }

            if (_postMailView) {
                _postMailView.width = actualWidth;
                _postMailView.y = actualHeight - _postMailView.height;
            }
        }

        if(isInvalid("noResultLabel")){
            if (_noResultLabel) {
                _noResultLabel.x = (actualWidth - _noResultLabel.width) / 2;
                if (_mailList) {
                    _noResultLabel.y = _mailList.y + (_mailList.height - _noResultLabel.height) / 2;
                } else {
                    _noResultLabel.y = (actualHeight - _noResultLabel.height) / 2;
                }
            }
        }
    }



    public function getVisibleTuningURL():String {
        return "";
    }

    public function set languages(value:Array):void{
        _languages=value;
    }

    public function get languages():Array{
        return _languages;
    }

    public function brandsScreenButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.BRANDS_SELECTED);
    }

    private function uploadPhotoButtonHandler(event:Event):void {
        if(!_imageList){

            var layData : AnchorLayoutData = new AnchorLayoutData();
            layData.horizontalCenter= 0;
            layData.top = BannerStackedScreen.BANNER_SIZE + 5;
            layData.bottom = _postMailView.height + 5;
            layData.topAnchorDisplayObject = _postMailView;

            var layout : HorizontalLayout = new HorizontalLayout();
            layout.hasVariableItemDimensions = true;
            layout.useVirtualLayout  = false;
            layout.verticalAlign = VerticalAlign.TOP;
            layout.horizontalAlign = HorizontalAlign.LEFT;

            _imageList = new List();
            _imageList.width = actualWidth - 20;
            _imageList.height = actualHeight - (BannerStackedScreen.BANNER_SIZE + _postMailView.height + 10);
            _imageList.layout = layout;
            _imageList.clipContent = true;
            _imageList.layoutData = layData;
            _imageList.isSelectable = false;
            _imageList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
            _imageList.horizontalScrollPolicy = ScrollPolicy.OFF;
            _imageList.verticalScrollPolicy = ScrollPolicy.OFF;
            _imageList.itemRendererFactory = uploadPhotoItemRenderer;
            _imageList.dataProvider= new ListCollection(event.data);
            _imageList.addEventListener(UploadImageItemRenderer.CLOSE_IMAGE, removeImageHandler);
            _imageList.y = BannerStackedScreen.BANNER_SIZE + 5;

            addChild(_imageList);
            updateImageList(false);

        }else {
            updateImageList(true);
        }
    }

    private function updateImageList(updateItems: Boolean) : void{
        if(updateItems)
            _imageList.dataProvider.updateAll();

        _imageList.invalidate();
        _imageList.validate();
    }

    private function removeImageHandler(event:Event):void {
        _imageList.dataProvider.removeItem(event.data);
        updateImageList(true);
    }

    private function uploadPhotoItemRenderer() : IListItemRenderer {
        var width : Number = _imageList.width / 5;
        var renderer : UploadImageItemRenderer = new UploadImageItemRenderer();
        renderer.itemWidth = width;
        renderer.setSize(width, NaN);
        return renderer;
    }

    public function homeButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.HOME_SELECTED);
    }

    public function enterRowsState():void {

    }

    public function set mainMenuView(value : MainMenuView) : void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
    }

    public function get mainMenuView():MainMenuView {
        return _mainMenuView;
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) {
            mainMenuView.hide();
        }
        dispatchEventWith(msg);
    }

    public function updateWithLanguage(language:Language):void {
        if (mainMenuView) {
            mainMenuView.updateWithLanguage(language);
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }

        if(tuningView){
            tuningView.removeFromParent(true);
            tuningView = null;
        }

        if(_container){
            _container.removeFromParent(true);
            _container = null;
        }
        _languages = null;
        _selectedLanguage = null;
    }

    public function set closeHandler(value:Function):void {
        _closeHandler = value;
    }

    public function set eventDispatcher(value:EventDispatcher):void {
        _eventDispatcher = value;
    }

    public function close():void{
        closeButtonTriggeredHandler(null);
    }

    public function addMessages(mails:Array, isPushing:Boolean = true):void {
        _noResultLabel.visible = false;
        if(!mails || !mails.length){
            _isLoading = false;
            if(!_mailList.dataProvider || !_mailList.dataProvider.length){
                _noResultLabel.visible = true;
                invalidate("noResultLabel");
            }
            return;
        }
        if(_mailList.dataProvider == null){
            _mailList.dataProvider = new ListCollection();
        }
        var reverse:Boolean = _mailList.dataProvider.length && isPushing;
        if(mails && mails.length){
            var i:int;
            var j:int;
            var messages:Array;
            for(i = mails.length - 1; i >= 0; i--){
                messages = mails[i].messages;
                for(j = messages.length - 1; j >= 0; j--){
                    if(_messageIdList.hasOwnProperty(messages[j]._id)){
                        messages.removeAt(j);
                    }else{
                        _messageIdList[messages[j]._id] = true;
                    }
                }
                if(messages.length == 0){
                    mails.removeAt(i);
                }
            }
            if(!mails.length){
                _isLoading = false;
                return;
            }
            //
            var firstDate:Date;
            var firstDateObj:Object;
            if(reverse){
                mails = mails.reverse();
            }
            if(_dateList.length) {
                if(isPushing){
                    firstDateObj = _dateList[_dateList.length - 1];
                }else{
                    firstDateObj = _dateList[0];
                }
                firstDate = firstDateObj.date;
            }
            for(i = 0; i < mails.length; i++){
                var dateObj:Object;
                messages = mails[i].messages;
                dateObj = {"itemID":"date"};
                dateObj.date = DateParser.parseDate(mails[i].date);
                var addDateLabel:Boolean = true;
                if(firstDate && firstDate.fullYear == dateObj.date.fullYear && firstDate.month == dateObj.date.month && firstDate.date == dateObj.date.date){
                    if(isPushing) {
                        _mailList.dataProvider.removeItem(firstDateObj);
                        if (_dateList.indexOf(firstDateObj) != -1) {
                            _dateList.removeAt(_dateList.indexOf(firstDateObj));
                        }
                        addDateLabel = true;
                    }else{
                        addDateLabel = false;
                    }
                }
                if(addDateLabel) {
                    if (reverse) {
                        _dateList.push(dateObj);
                    } else {
                        _dateList.unshift(dateObj);
                    }
                }
                for(j = 0; j < messages.length; j++){
                    messages[j].itemID = "message";
                }

                if(reverse){
                    if(addDateLabel){
                        messages.unshift(dateObj);
                    }
                    _mailList.dataProvider.addAllAt(new ListCollection(messages), 0);
                }else{
                    if(addDateLabel){
                        messages.unshift(dateObj);
                    }
                    _mailList.dataProvider.addAll(new ListCollection(messages));
                }
            }
        }
        if(reverse){
            Starling.juggler.delayCall(function():void{
                _mailList.scrollToPosition(0, 0, 0.2);
            }, 0.01);
        }else{
            Starling.juggler.delayCall(function():void{
                _mailList.scrollToPosition(0, _mailList.maxVerticalScrollPosition, 0);
            }, 0.01);
        }
        _mailList.dataProvider.updateAll();
        _skip = _mailList.dataProvider.length - _dateList.length;
        _isLoading = false;
    }

        public function addNewMessage(mail:Object):void {
            _noResultLabel.visible = false;
            mail.itemID = "message";
            if(_messageIdList.hasOwnProperty(mail._id)){
                return;
            }
            _messageIdList[mail._id] = true;
            var date:Date = DateParser.parseDate(mail.created);
            var lastDate:Date;
            if(_dateList.length) {
                lastDate = _dateList[0].date;
            }else{
                lastDate = new Date();
            }
            if(date.fullYear != lastDate.fullYear || date.month != lastDate.month || date.date != lastDate.date){
                var dateObj:Object;
                dateObj = {"itemID":"date"};
                dateObj.date = date;
                _mailList.dataProvider.addItem(dateObj);
                _dateList.push(dateObj);
            }
            _mailList.dataProvider.addItem(mail);
            _postMailView.clearAll();
            Starling.juggler.delayCall(function():void{
                _mailList.scrollToPosition(0, _mailList.maxVerticalScrollPosition, 0.3);
            }, 0.01);
            _skip = _mailList.dataProvider.length - _dateList.length;
        }

        public function cleanInputs():void {
            _postMailView.clearAll();
            if(_imageList){
                if(_imageList.dataProvider){
                    _imageList.dataProvider.removeAll();
                }
                _imageList.removeFromParent(true);
                _imageList = null;
            }
        }
    }
}

import feathers.core.FeathersControl;

class MainContainer extends FeathersControl{
    public function MainContainer(){
        super ();
    }
}