package com.despark.tuning3d.view.components.mailbox.mails
{
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.DateParser;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.core.FeathersControl;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;

    public class MailLabelItemRenderer extends ListItemRenderer{
		private var verSpacer    : Quad;
        private var _userAvatar : UserCircularAvatar;
        private var _user:User;

        private static const _avatarSize : Number = 40;
        private static const _gap : int = 9;
        public static const ITEM_HEIGHT:int = _avatarSize + 2 * _gap;

        protected var _nameLabel: FeathersControl;
        protected var _dateLabel: Label;
        protected var _messageLabel: FeathersControl;

        private var _avatarContainer : LayoutGroup;
        private var _buttonContainer : LayoutGroup;
        private var _mainContainer   : LayoutGroup;

        private var _eventDispatcher : *;
        private var _mailData:MailData;
        private var _background:Image;
        private var _marginLeft:Number;
        private var _marginRight:Number;
        public function MailLabelItemRenderer(){

        }

        public function set eventDispatcher(value:*) : void{
            _eventDispatcher = value;
        }

        public function get eventDispatcher() : *{
            return _eventDispatcher;
        }

        public function get friend() : Friend {
			return data as Friend;
		}

		protected override function initialize() : void {
			super.initialize();

            _background = new Image(Texture.fromColor(1, 1, 0xFF0000));
            addChild(_background);

            _mainContainer = new LayoutGroup();
            _mainContainer.layout = new AnchorLayout();
            _mainContainer.width = actualWidth;
            _mainContainer.height = actualHeight;
            addChild(_mainContainer);

            var layData : AnchorLayoutData;

            /**
             * Avatar container
             */
            layData = new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.left = _gap;

            _avatarContainer = new LayoutGroup();
            _avatarContainer.layoutData = layData;
            _mainContainer.addChild(_avatarContainer);
            _avatarContainer.addEventListener(TouchEvent.TOUCH, onFriendSelectHandler);

            /**
             * Name label
             */
            layData = new AnchorLayoutData();
            layData.left = _avatarSize + _gap + _marginLeft;
            layData.top = _gap - 1;
            layData.leftAnchorDisplayObject = _avatarContainer;
            _nameLabel = LabelFactory.createLabelMailUserName();
            _nameLabel.layoutData = layData;
            _mainContainer.addChild(_nameLabel);
            _nameLabel.addEventListener(TouchEvent.TOUCH, onFriendSelectHandler);

            /**
             * Message label
             */
            layData = new AnchorLayoutData();
            layData.left = _avatarSize + _gap + _marginLeft;
            layData.leftAnchorDisplayObject = _avatarContainer;
            _messageLabel = LabelFactory.createLabelMailMessage();
            _messageLabel.layoutData = layData;
            _mainContainer.addChild(_messageLabel);

            /**
             * Date label
             */
            layData = new AnchorLayoutData();
            layData.top = _gap;
            layData.right = _gap;
            _dateLabel = new Label();
            _dateLabel.textRendererFactory = LabelFactory.mailDateLabel;
            _dateLabel.layoutData = layData;
            _mainContainer.addChild(_dateLabel);

            /**
             * button Container
             */
            layData = new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.right = 15;
            _buttonContainer = new LayoutGroup();
            _buttonContainer.layout = new AnchorLayout();
            _buttonContainer.layoutData = layData;
            _mainContainer.addChild(_buttonContainer);

			verSpacer = new Quad(actualWidth,1,0xE1E3E2);
            verSpacer.height = 0.5;
			addChild(verSpacer);
		}

		protected override function draw() : void {
			super.draw();
            if(actualWidth != owner.width){
                trace();
            }
            _mainContainer.width = actualWidth;
            _mainContainer.height = actualHeight;
            verSpacer.width = actualWidth;
            verSpacer.y = actualHeight - verSpacer.height;
            //
            _background.width = actualWidth;
            _background.height = actualHeight;
            _background.color = _mailData.seen?0xFFFFFC:0xE0E0E0;
            //

            if(_messageLabel){
                _messageLabel.width = actualWidth - (_avatarSize + _gap + _marginLeft + _marginRight);
                _messageLabel.height = 12;
                _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
            }
            //

            if(_userAvatar){
                _userAvatar.x = _gap + _marginLeft;
                _userAvatar.y = (actualHeight - _userAvatar.height) / 2;
            }

            if(_dateLabel){
                var layData:AnchorLayoutData = new AnchorLayoutData();
                layData.top = _gap;
                layData.right = _gap + _marginRight;
                _dateLabel.layoutData = layData;
            }
		}

        private function onFriendSelectHandler(event:TouchEvent) : void {
            var touch : Touch = event.touches[0] as Touch;
            if (touch.phase == TouchPhase.ENDED && !this.owner.isScrolling) {
                Constants.selectedUser = _user.copy();
                eventDispatcher.dispatchEventWith(StackedScreen.GARAGE_SELECTED);
            }
        }

		public override function dispose() : void {
            if(_userAvatar) {
                _userAvatar.removeFromParent(true);
                _userAvatar = null;
            }

            if(_avatarContainer){
                _avatarContainer.removeFromParent(true);
                _avatarContainer.removeEventListener(TouchEvent.TOUCH, onFriendSelectHandler)
                _avatarContainer = null;
            }

            if (verSpacer) {
                removeChild(verSpacer);
            }

            if (_nameLabel){
                _nameLabel.removeEventListener(TouchEvent.TOUCH, onFriendSelectHandler)
                _nameLabel.removeFromParent(true);
            }

            super.dispose();
        }

		protected override function commitData() : void {
            super.commitData();
            if(_userAvatar){
                _userAvatar.removeFromParent(true);
            }
            _mailData = new MailData(data);
            _user = new User(data);
            _userAvatar = new UserCircularAvatar(_user, _avatarSize);
            addChild(_userAvatar);
            LabelFactory.setLabelText(_nameLabel, _user.nameToShow);
            LabelFactory.setLabelText(_messageLabel, shortenString(_mailData.text, 60).replace('\n', ' '));
            _dateLabel.text = DateParser.getDateAppropriateText(_mailData.created);
            //
            if(_nameLabel){
                _nameLabel.y = (actualHeight - _gap) / 2 - _nameLabel.height;
            }
            if(_messageLabel){
                _messageLabel.y = (actualHeight + _gap) / 2;
            }
		}

        private static function shortenString(text:String, maxLength:int = 100):String {
            if(text.length <= maxLength){
                return text;
            }
            return text.substr(0, maxLength) + "...";
        }

        public function get marginLeft():Number {
            return _marginLeft;
        }

        public function set marginLeft(value:Number):void {
            _marginLeft = value;
        }

        public function get marginRight():Number {
            return _marginRight;
        }

        public function set marginRight(value:Number):void {
            _marginRight = value;
        }
    }
}

import com.utils.DateParser;

class MailData{
    public var seen:Boolean = false;
    public var text:String = "";
    public var unreadCount:int = 0;
    public var created:Date;
    public function MailData(value:Object){
        seen = value.seen || value._id != value.from;
        text = value.text;
        unreadCount = value.unreadCount;
        created = DateParser.parseDate(value.created);
    }
}