/**
 * Created by armen on 30.08.2017.
 */
package com.despark.tuning3d.view.components.social
{
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.MultilingualButton;
    import com.despark.tuning3d.view.components.timeline.CommentData;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.TimelineLabelFactory;
    import com.utils.DateParser;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.controls.TextInput;
    import feathers.controls.text.StageTextTextEditor;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.ITextEditor;
    import feathers.core.ITextRenderer;

    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.display.Quad;
    import starling.events.Event;

    public class SocialItemRenderer extends ListItemRenderer
    {

        public static const COLOR_TEXT_LITE_GREY :uint = 0x999999;
        public static const COLOR_TEXT_NAME_ORANGE :uint = 0xB00000;
        public static const SIZE_SMALL:uint = 12;

        private var _nameLabel:Label;
        private var _commentLabel:Label;
        private var _dateLabel:Label;
        private var _gap:int;
        private var _bottomQuad:Quad;
        private var _editButton:MultilingualButton;
        private var _deleteButton:MultilingualButton;
        private var _textInput:TextInput;
        private var _applyButton:MultilingualButton;
        private var _cancelButton:MultilingualButton;

        public function SocialItemRenderer() {
        }

        private function get commentData():CommentData {
            return _data as CommentData;
        }

        override protected function initialize():void
        {
            super.initialize();

            _gap = 3;

            _nameLabel = new Label();
            _nameLabel.textRendererFactory = labelRenderer;
            _nameLabel.y = 0;
            _nameLabel.x = 0;
            addChild(_nameLabel);

            _commentLabel = new Label();
            _commentLabel.wordWrap = true;
            _commentLabel.textRendererFactory = labelRenderer;
            _commentLabel.y = 0;
            _commentLabel.x = 0;
            addChild(_commentLabel);

            _dateLabel = new Label();
            _dateLabel.textRendererFactory = labelRenderer;
            _dateLabel.y = 0;
            _dateLabel.x = 0;
            _dateLabel.alpha = 0.5;
            addChild(_dateLabel);

            _bottomQuad = new Quad(1, 1, TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
            _bottomQuad.alpha = 0.5;
            addChild(_bottomQuad);
        }

        public static function  labelRenderer():ITextRenderer
        {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_LITE_GREY);
            textRenderer.embedFonts = FontSelector.embedFonts();
            textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }

        public static function  nameRenderer():ITextRenderer
        {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_NAME_ORANGE);
            textRenderer.embedFonts = FontSelector.embedFonts();
            textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }

        private function applyButtonTriggeredHandler(event:Event):void
        {
            //commentData.comment = _textInput.text;

            update();
            this.owner.dispatchEventWith(FeedData.COMMENT_EDIT, false, commentData);

        }

        private function onEditSelectedHandler(event:Event):void {
            editState();
            //_textInput.text = _commentLabel.text;
        }

        private function cancelButtonTriggeredHandler(event:Event):void {
            stableState();
        }

        private function editState():void
        {
            //_applyButton.visible = true;
           // _cancelButton.visible = true;
            /*_textInput.visible = true;
            _commentLabel.visible = false;
            _editButton.visible = false;
            _deleteButton.visible = false;
            _textInput.setFocus();*/
        }

        private function stableState():void
        {
            //_applyButton.visible = false;
            //_cancelButton.visible = false;
            //_textInput.visible = false;
            //_commentLabel.visible = true;
            //_editButton.visible = true;
            //_deleteButton.visible = true;
            //_textInput.clearFocus();
        }

        private function inputChangeHandler(event:Event):void {
            //_applyButton.isEnabled = _textInput.text.length > 0;
        }

        private function onDeleteButtonTriggeredHandler(event:Event):void {

            //this.owner.dispatchEventWith(FeedData.COMMENT_DELETE, false, commentData);
        }

        override protected function commitData():void
        {
            super.commitData();
            update();

        }

        private function update():void
        {
            if (commentData)
            {
                _nameLabel.text = commentData.owner.username;
                _nameLabel.x = 0;
                _nameLabel.y = 7;
                _nameLabel.validate();

                _commentLabel.text = commentData.comment;
                _commentLabel.y = _nameLabel.height + _gap + _nameLabel.y;
                _commentLabel.x = 0;
                _commentLabel.maxWidth = actualWidth - 2 * _gap;
                _commentLabel.validate();

                _dateLabel.text = DateParser.getDateAppropriateText(commentData.availableDate);
                _dateLabel.y = _commentLabel.height + _commentLabel.y + 2 * _gap;
                _dateLabel.x = 0;
                _dateLabel.validate();

                setHeight();
            }
        }

        private function setHeight():void
        {
            var newHeight:Number = _dateLabel.y + _dateLabel.height + _gap;
            newHeight = (newHeight < 30) ? 30 : newHeight;
            if (newHeight != actualHeight) {
                this.setSize(actualWidth, newHeight);

            }
        }

        override public function dispose():void
        {
            if (_editButton) {
                _editButton.removeEventListener(Event.TRIGGERED, onEditSelectedHandler);
                _editButton.removeFromParent(true);
                _editButton = null;
            }
            if (_deleteButton) {
                _deleteButton.removeEventListener(Event.TRIGGERED, onDeleteButtonTriggeredHandler);
                _deleteButton.removeFromParent(true);
                _deleteButton = null;
            }

            if (_cancelButton) {
                _cancelButton.removeEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
                _cancelButton.removeFromParent(true);
                _cancelButton = null;
            }

            if (_textInput) {
                _textInput.removeEventListener(Event.CHANGE, inputChangeHandler);
                _textInput.removeFromParent(true);
                _textInput = null;
            }

            if (_applyButton) {
                _applyButton.removeEventListener(Event.TRIGGERED, applyButtonTriggeredHandler);
                _applyButton.removeFromParent(true);
                _applyButton = null;
            }

            super.dispose();
        }

        override protected function draw():void
        {
            super.draw();
            if (isInvalid(INVALIDATION_FLAG_SIZE)) {
                update();
                _bottomQuad.width = actualWidth;
                _bottomQuad.y = actualHeight - _bottomQuad.height;
            }
        }
    }
}