/**
 * Created by Render-2 on 8/11/2017.
 */
package com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments {
    import by.blooddy.crypto.serialization.JSONer;

    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.ImagePicker;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BlockerView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.MultilingualButton;
    import com.despark.tuning3d.view.components.timeline.*;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.FontSelector;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.TextInput;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.ITextRenderer;
    import feathers.events.FeathersEventType;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;

    import flash.events.IOErrorEvent;
    import flash.net.URLLoaderDataFormat;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;
    import flash.utils.ByteArray;

    import ru.inspirit.net.MultipartURLLoader;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;

//import com.freshplanet.ane.KeyboardSize.MeasureKeyboard;
    public class CommentInput extends LayoutGroup{

    public static const CAR_BUTTON_SELECTED : String = 'carButtonSelected';
    public static const UPDATE_UPLOADED_IMAGES : String = 'UPDATE_UPLOADED_IMAGES';
    public static const COMMENT_ADDED : String = 'commentAdded';

    private var _textInput : TextInput;
    private var _gap : Number;
    private var _sendButton:MultilingualButton;
    private var _buttonsContainer:LayoutGroup;
    private var _inputBoundImage : Image;
    private var _user:User;
    private var _avatarSize:Number;
    private var _postConatiner:LayoutGroup;
    private var _avatarContainer:LayoutGroup;
    private var _ownerFeedData : FeedData;

    private var _bgQuad:Quad;
    private var _uploadCarButton:Button;
    private var _uploadPhoto:Button;
    private var _multipartURLLoader : MultipartURLLoader;
    private var _blocker : BlockerView;
    private var _images: Array;
    private var _imagePicker : ImagePicker;
    private var _isSimple:Boolean;
    private var _tuningPost:Boolean;
    private var _placeholder:Label;


    public function CommentInput(isSimple:Boolean, tuningPost:Boolean = true) {
        _isSimple = isSimple;
        _tuningPost = tuningPost;
        _user = FeedData.CURRENT_USER;
        _images = [];
    }


    override protected function initialize():void {
        super.initialize();

        _gap = 10;
        _bgQuad = new Quad(10,10,0xffffff);

        _avatarSize = 31;

        this.layout = new AnchorLayout();
        this.backgroundSkin = _bgQuad;

        initAvatarContainer();
        initPostContainer();
        initButtonsContainer();
        initTextInput();

        this.addEventListener(Event.ENTER_FRAME, onEnterFrame);

        _imagePicker= new ImagePicker();
    }


    private function initButtonsContainer():void {
        var buttonLayoutData : AnchorLayoutData = new AnchorLayoutData();
        buttonLayoutData.right = 15 ;
        buttonLayoutData.verticalCenter = 0;
        var size : int = actualHeight - 12;

        var buttonsLayout : HorizontalLayout = new HorizontalLayout();
        buttonsLayout.gap = _gap;
        buttonsLayout.verticalAlign = VerticalAlign.MIDDLE;

        var seperator : Function = function() : Quad
        {
            var quad : Quad = new Quad(1, size , TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
            quad.alpha = .6;
            return quad;
        };



        _buttonsContainer = new LayoutGroup();
        _buttonsContainer.layout = buttonsLayout;
        _buttonsContainer.layoutData = buttonLayoutData;
        _postConatiner.addChild(_buttonsContainer );

        if (!_isSimple) {
            _buttonsContainer.addChild(seperator());

            if(_tuningPost){
                _uploadCarButton = ButtonFactory.simpleButtonByTextureName("timeline_car");
                _uploadCarButton.height = 20;
                _uploadCarButton.width = 30;
                _buttonsContainer.addChild(_uploadCarButton);
                _uploadCarButton.addEventListener(Event.TRIGGERED, carButtonHandler);
                _buttonsContainer.addChild(seperator());
            }

            _uploadPhoto = ButtonFactory.simpleButtonByTextureName("timeline_photo");
            _uploadPhoto.height = 22;
            _uploadPhoto.width = 26;
            _uploadPhoto.addEventListener(Event.TRIGGERED, uploadPhotoButtonHandler);
            _buttonsContainer.addChild(_uploadPhoto);

            _buttonsContainer.addChild(seperator());

        }


        _sendButton = ButtonFactory.timelineCommentSend();
        _sendButton.isEnabled= false;
        _sendButton.addEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
        _buttonsContainer.addChild(_sendButton);
    }



    private function initPostContainer():void {
        var postLayoutData : AnchorLayoutData = new AnchorLayoutData();
        postLayoutData.leftAnchorDisplayObject = _avatarContainer;
        postLayoutData.left = _gap-5;
        postLayoutData.right = _isSimple?0:_gap;
        postLayoutData.verticalCenter = 0;

        _postConatiner = new LayoutGroup();
        _postConatiner.layout = new AnchorLayout();
        _postConatiner.layoutData = postLayoutData;
        addChild(_postConatiner);

        _inputBoundImage = new Image(AlertView.assetManager.getTexture("postRectangle"));
//        _inputBoundImage.height = _avatarSize;
//        _inputBoundImage.scale9Grid = new Rectangle(0, _inputBoundImage.height*0.1, _inputBoundImage.width*0.8, _inputBoundImage.height*.9);

        _postConatiner.backgroundSkin = _inputBoundImage;
        _postConatiner.height = actualHeight;

    }

    private function initAvatarContainer():void {
        var avatarLayoutData : AnchorLayoutData = new AnchorLayoutData();
        avatarLayoutData.verticalCenter = 0;
        avatarLayoutData.left = 0;

        var avatarLayout : HorizontalLayout = new HorizontalLayout();
        avatarLayout.verticalAlign = VerticalAlign.MIDDLE;
        avatarLayout.gap = 0;
        _avatarContainer = new LayoutGroup();
        addChild(_avatarContainer);
        _avatarContainer.layoutData = avatarLayoutData;
        _avatarContainer.layout = avatarLayout;

        if(!_isSimple) {
            var lineQuad:Quad = new Quad(20, 1, 0);
            lineQuad.alpha = 0.2;
            _avatarContainer.addChild(lineQuad);
        }

        _avatarContainer.addChild(new UserCircularAvatar(_user,_avatarSize));

    }

    protected function onEnterFrame(event:Event):void
    {
        if (!Device.isIos() && !Constants.IS_EMULATOR){
//            MeasureKeyboard.getInstance().setKeyboardAdjustDefault();
        }
    }

    private function initTextInput () : void {
        var layoutData : AnchorLayoutData = new AnchorLayoutData();
        layoutData.left = _gap;
        layoutData.rightAnchorDisplayObject = _buttonsContainer;
        layoutData.right = _gap;
        layoutData.verticalCenter = 0;

        _textInput = new TextInput();
        _textInput.layoutData = layoutData;
        _textInput.textEditorFactory = TimelineLabelFactory.postInputTextEditorFactory;
        _textInput.addEventListener(Event.CHANGE, inputChangeHandler);
        _textInput.padding = 4;
        _textInput.height = actualHeight - _gap;
        //_textInput.prompt = "Post new Message";

        _placeholder = new Label();
        _placeholder.isEnabled = _placeholder.touchable = false;
        _placeholder.textRendererFactory = function() : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(true), 14, 0xacacac);
            textRenderer.textFormat.italic = true;
            textRenderer.textFormat.align = TextFormatAlign.RIGHT;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        _placeholder.text = "Post new Message";
        // _placeholder.skewX = Math.PI / 10;
        _postConatiner.addChild(_placeholder);
        _postConatiner.addChild(_textInput);

        _textInput.addEventListener(FeathersEventType.FOCUS_IN, focusInEventHandler);
        _textInput.addEventListener(FeathersEventType.FOCUS_OUT, focusOutEventHandler);
    }

    private function focusOutEventHandler(event:Event):void {
        _placeholder.visible = _textInput.text.length==0;
    }

    private function focusInEventHandler(event:Event):void {
        _placeholder.visible = false;
    }

    private function carButtonHandler(event:Event):void {
        Constants.selectedUser = null;
        this.dispatchEventWith(CAR_BUTTON_SELECTED, false, {enableClosing:true});
    }

    private function uploadPhotoButtonHandler(event:Event):void {
        if(_images.length >= 5){
            return;
        }
        SoundManager.playSound(SoundManager.SELECT_PART);
        if(Device.isEmulator()){
            _images.push({
                quad: new Quad(100,(_images.length + 1) * 100, Math.random() * 0xffffff)
            });
            dispatchEventWith(UPDATE_UPLOADED_IMAGES, false, _images);
        }else {
            addImagePickerListeners();
            _imagePicker.pickImage();
        }
    }

    private function addImagePickerListeners() : void{
        _imagePicker.addEventListener(ImagePicker.SUCCEED, imagePickedHandler);
        _imagePicker.addEventListener(ImagePicker.ERROR_OCCURRED, imagePickerFailed);
        _imagePicker.addEventListener(ImagePicker.NOT_SUPPORTED, imagePickerFailed);
    }

    private function removeImagePickerListeners() : void{
        _imagePicker.removeEventListener(ImagePicker.SUCCEED, imagePickedHandler);
        _imagePicker.removeEventListener(ImagePicker.ERROR_OCCURRED, imagePickerFailed);
        _imagePicker.removeEventListener(ImagePicker.NOT_SUPPORTED, imagePickerFailed);
    }

    private function imagePickedHandler(e : Event) : void{
        removeImagePickerListeners();
        _images.push(e.data);
        uploadPostImage(e.data.byteArray);
        enableSending();
    }

    private function imagePickerFailed(e : Event) : void{
        removeImagePickerListeners();
        if(e.type == ImagePicker.ERROR_OCCURRED){
            AlertView.show('Something went wrong ...');
        }
        if(e.type == ImagePicker.NOT_SUPPORTED){
            AlertView.show('Image picking not supported on your device ...', 1, false);
        }
    }

    public function uploadPostImage(bytes: ByteArray) : void{
        _blocker = BlockerView.show();
        var url : String = 'http://feed-staging.api.3dtuning.com/api/v1/feed/image';
        _multipartURLLoader = new MultipartURLLoader();
        _multipartURLLoader.dataFormat = URLLoaderDataFormat.BINARY;
        _multipartURLLoader.addVariable("files[]", "photo.jpeg");
        _multipartURLLoader.addFile(bytes, "photo.jpeg", "files[]", "image/jpeg");
        _multipartURLLoader.loader.addEventListener(Event.COMPLETE, multipartURLLoaderCompleteHandler);
        _multipartURLLoader.loader.addEventListener(IOErrorEvent.IO_ERROR, multipartURLLoaderIoErrorHandler);
        _multipartURLLoader.load(url);
    }

    private function multipartURLLoaderCompleteHandler(event : flash.events.Event) : void
    {
        _blocker.hide();
        _blocker = null;
        var json : String = (event.target.data as ByteArray).toString();
        var obj : Object =  _images[_images.length - 1];
        obj.url = JSONer.parse(json).url;


        _multipartURLLoader.loader.removeEventListener(Event.COMPLETE, multipartURLLoaderCompleteHandler);
        _multipartURLLoader.loader.removeEventListener(IOErrorEvent.IO_ERROR, multipartURLLoaderIoErrorHandler);
        _multipartURLLoader.close();
        _multipartURLLoader = null;

        dispatchEventWith(UPDATE_UPLOADED_IMAGES, false, _images);
        if(_imagePicker.isPending) {
            Starling.juggler.delayCall(function ():void {
                addImagePickerListeners();
                _imagePicker.commitImageLoading();
            }, 0);
        }
    }

    private function multipartURLLoaderIoErrorHandler(event : IOErrorEvent) : void
    {
        _blocker.hide();
        _blocker = null;
        _images.pop();

        AlertView.show('Something went wrong, please check the internet connection and retry.');

        _multipartURLLoader.loader.removeEventListener(Event.COMPLETE, multipartURLLoaderCompleteHandler);
        _multipartURLLoader.loader.removeEventListener(IOErrorEvent.IO_ERROR, multipartURLLoaderIoErrorHandler);
        _multipartURLLoader.close();
        _multipartURLLoader = null;
    }



    public function set owner (value : FeedData) : void
    {
        _ownerFeedData = value;
    }

    private function inputChangeHandler(event:Event):void {
        enableSending();
    }

    public function activate() : void{
        this.visible = true;
        _textInput.setFocus();
        if(_ownerFeedData) {
            _ownerFeedData.isCommentActive = true;
        }
        enableSending();
    }


    private function enableSending():void{
        _sendButton.isEnabled = (_textInput.text.length > 0 || _images.length > 0);
    }

    public function deactivate() : void
    {
        this.visible = false;
        _textInput.text = "";
        _textInput.clearFocus();
        if(_ownerFeedData)
        {
            _ownerFeedData.isCommentActive = false;
        }

        _sendButton.isEnabled= false;
    }

    public function clearFocus():void {
        _textInput.text = "";
        _textInput.clearFocus();
    }


    private function sendButtonTriggeredHandler(event:Event):void {
        dispatchEventWith(FeedData.COMMENT_SEND, false, {
            text: _textInput.text,
            images: _images
        });
    }




    override protected function draw():void {
        super.draw();

        if(isInvalid(INVALIDATION_FLAG_SIZE)){
            _avatarContainer.width = NaN;
            _avatarContainer.validate();

            _postConatiner.width = NaN;
            _postConatiner.validate();


            _buttonsContainer.width = NaN;
            _textInput.width = NaN;
            _textInput.validate();
        }

        if(_placeholder){
            _placeholder.y = (actualHeight - _placeholder.height) / 2;
            _placeholder.x = 25;
        }
    }


    override public function dispose():void {
        if(_textInput) {
            _textInput.removeFromParent(true);
        }

        if(_avatarContainer) {
            _avatarContainer.removeFromParent(true);
            _avatarContainer = null;
        }
        if(_uploadCarButton) {
            _uploadCarButton.removeEventListener(Event.TRIGGERED, carButtonHandler);
            _uploadCarButton.removeFromParent(true);
            _uploadCarButton = null;
        }

        if(_uploadPhoto) {
            _uploadPhoto.removeEventListener(Event.TRIGGERED, uploadPhotoButtonHandler);
            _uploadPhoto.removeFromParent(true);
            _uploadPhoto = null;
        }

        if(_imagePicker){
            _imagePicker.removeEventListener(ImagePicker.SUCCEED, imagePickedHandler);
            _imagePicker.removeEventListener(ImagePicker.ERROR_OCCURRED, imagePickerFailed);
            _imagePicker.removeEventListener(ImagePicker.NOT_SUPPORTED, imagePickerFailed);
            _imagePicker = null;
        }


        if(_textInput)
        {
            _textInput.removeEventListener(Event.CHANGE, inputChangeHandler);
            _textInput.removeFromParent(true);
            _textInput = null;
        }

        if(_sendButton)
        {
            _sendButton.removeEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
            _sendButton.removeFromParent(true);
            _sendButton = null;
        }
        this.removeEventListener(Event.ENTER_FRAME, onEnterFrame);


        super.dispose();
    }

        public function clearAll():void {
            _textInput.text = "";
        }
    }
}
