package com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments {
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;

import feathers.controls.LayoutGroup;
import feathers.controls.TextInput;
import feathers.controls.text.StageTextTextEditor;
import feathers.controls.text.TextFieldTextEditor;
import feathers.core.ITextEditor;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import flash.text.TextFormat;

import starling.display.Quad;
import starling.events.Event;

public class EditCommentInput extends LayoutGroup{

    public static const HIDE : String = 'hide';

    private var _textInput:TextInput;
    private var _mainContainer:LayoutGroup;
    private var _applyButton:MultilingualButton;
    private var _cancelButton:MultilingualButton;

    public function get text() : String
    { return _textInput.text}

    public function set text(v : String) : void
    {
        _textInput.text = v;
    }

    public function EditCommentInput() {

    }


    override protected function initialize():void {
        super.initialize();

        this.backgroundSkin = new Quad(10, 10, TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
        this.backgroundSkin.alpha = 0.6;

        this.layout = new AnchorLayout();

        _mainContainer = new LayoutGroup();
        _mainContainer.backgroundSkin = new Quad(10, 10, TimelineLabelFactory.COLOR_TEXT_WHITE);
        _mainContainer.layoutData = new AnchorLayoutData(1,1,1,1);
        _mainContainer.layout = new AnchorLayout();
        this.addChild(_mainContainer);

        var horizontalLay : HorizontalLayout = new HorizontalLayout();
        horizontalLay.gap = 20;
        horizontalLay.verticalAlign = VerticalAlign.MIDDLE;

        var buttonsLayData : AnchorLayoutData = new AnchorLayoutData();
        buttonsLayData.right = 3;
        buttonsLayData.bottom = 3;

        var buttonsContainer : LayoutGroup = new LayoutGroup();
        buttonsContainer.layout = horizontalLay;
        buttonsContainer.layoutData = buttonsLayData;
        _mainContainer.addChild(buttonsContainer);

        var textLayData : AnchorLayoutData = new AnchorLayoutData();
        textLayData.rightAnchorDisplayObject = buttonsContainer;
        textLayData.right = 0;
        textLayData.top = 0;
        textLayData.left = 0;
        textLayData.bottom = 0;

        _textInput = new TextInput();
        _textInput.layoutData = textLayData;
        _mainContainer.addChild(_textInput);
        _textInput.paddingTop = 34;
        _textInput.paddingLeft = 2;

        _textInput.textEditorFactory = function() : ITextEditor{
            var editor : StageTextTextEditor = new StageTextTextEditor();
            editor.fontFamily = "_sans";
            editor.fontSize = TimelineLabelFactory.SIZE_BIG;
            editor.multiline = true;
            editor.color = TimelineLabelFactory.COLOR_TEXT_GREY;
            /*var editor : TextFieldTextEditor = new TextFieldTextEditor();
            editor.textFormat = new TextFormat("_sans", TimelineLabelFactory.SIZE_BIG);
            editor.multiline = true;*/
            return editor;
        };
        _textInput.filter = TimelineLabelFactory.getFilter();
        _textInput.addEventListener(Event.CHANGE, inputChangeHandler);

        _applyButton = ButtonFactory.rectangleButton("update");
        _applyButton.setSize(48,20);
        _applyButton.isEnabled= false;
        _applyButton.labelFactory = TimelineLabelFactory.commentEditUpdateTextRenderer;
        _applyButton.labelOffsetY = -1;
        _applyButton.addEventListener(Event.TRIGGERED, applyButtonTriggeredHandler);

        _cancelButton = ButtonFactory.timelineCommentButtons("cancel");
        _cancelButton.setSize(38,14);
        _cancelButton.labelFactory = TimelineLabelFactory.commentCancelTextRenderer;
        _cancelButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
        buttonsContainer.addChild(_cancelButton);
        buttonsContainer.addChild(_applyButton);
    }

    private function cancelButtonTriggeredHandler(event:Event):void {
        dispatchEventWith(HIDE);
    }

    private function applyButtonTriggeredHandler(event:Event):void {
        this.dispatchEventWith(FeedData.COMMENT_EDIT);
    }

    protected function inputChangeHandler(event:Event):void {
        _applyButton.isEnabled = _textInput.text.length > 0;
    }

    public function activate():void {
        _textInput.setFocus();
    }

    public function deactivate():void {
        _textInput.text = "";
        _textInput.clearFocus();
    }
}
}
