package com.despark.tuning3d.view.components.timeline.ItemRenderers {
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeLabel;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract.ATimelineCommentLikeItemRender;
import com.despark.tuning3d.view.components.timeline.TimelineScreen;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.DateParser;

import feathers.controls.Label;
import feathers.core.FeathersControl;

import flash.geom.Rectangle;

import starling.core.Starling;
import starling.display.DisplayObject;

import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class FeedChallengeAcceptedItemRenderer extends ATimelineCommentLikeItemRender{
    private var _challengeLabel:ChallengeLabel;
    private var _opponentLabel:Label;
    private var _andLabel:FeathersControl;
    //
    private var challengeOwner:User;
    private var challengeReceiver:User;
    public function FeedChallengeAcceptedItemRenderer() {
        super();
    }

    protected override function initAllComponents():void {
        super.initAllComponents();

    }

    protected override function initLabels():void {
        super.initLabels();
    }


    protected override function update():void {
        if(!feedData) return;

        if(_challengeLabel){
            _challengeLabel.removeFromParent(true);
        }
        commitCreateChallengeLabel();

        challengeOwner ||= feedData.creator;
        _nameLabel["text"] = getUserName(challengeOwner);
        _nameLabel.visible = true;

        _descriptionLabel["text"] = feedData.descriptionLabel;
        _descriptionLabel.visible = true;

        _friendNameLabel.visible = false;
        _friendNameLabel.removeFromParent();

        _dateLabel.text = DateParser.getDateAppropriateText(feedData.availableDate);
        _dateLabel.visible = true;

        if(_likeButtonNew){
            _likeButtonNew.removeFromParent();
        }

        _bottomContainer.validate();
        invalidateLikeCommentsViews();
    }

    private function commitCreateChallengeLabel():void{

        Starling.juggler.delayCall(commitCreateChallengeLabel, 0.03);
    }

    private static function getUserName(user:User):String{
        if(!user){
            return '';
        }
        if(FeedData.CURRENT_USER && user.id == FeedData.CURRENT_USER.id){
            return "I";
        }
        return user.nameToShow;
    }

    override protected function draw():void {
        super.draw();
        if(_challengeLabel) {
            _challengeLabel.width = 3 * actualWidth / 4;
            _challengeLabel.height = 100;
        }
    }

    override protected function customAutosize():void {
        _mainContainer.invalidate(INVALIDATION_FLAG_SIZE);
        _mainContainer.validate();
        super.customAutosize();
    }

    override public function dispose():void {
        super.dispose();
        if(_challengeLabel){
            _challengeLabel.removeFromParent(true);
            _challengeLabel = null;
        }
    }

    private function challengeLabelTouchHandler(event:TouchEvent) : void {
        var touch : Touch = event.getTouch(event.currentTarget as DisplayObject, TouchPhase.ENDED);
        if (touch && !owner.isScrolling){
            var eventData:Object = {receiverTuning: _challengeLabel.receiverTuning, senderTuning: _challengeLabel.ownerTuning, challengeData: feedData.challengeData};
            owner.dispatchEvent(new Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
        }
    }

    override protected function initIcon() : void{
        _postIcon = new Image(iconsMap[FeedData.TYPE_CHALLENGE_ANSWER]);
        layoutIcon();
    }

    override protected function onCreatorGarageSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            Constants.selectedUser =  challengeOwner;
            checkForUserAndDispatch();
        }
    }
    private function onOpponentGarageSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            Constants.selectedUser =  challengeReceiver;
            checkForUserAndDispatch();
        }
    }
}
}
