package com.despark.tuning3d.view.components.timeline.ItemRenderers {
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract.ATimelineCommentLikeItemRender;
import com.despark.tuning3d.view.components.timeline.TimelineScreen;
import com.despark.tuning3d.view.components.timeline.components.TimelineFeedImageHolder;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.DateParser;
import com.utils.DisplayObjectUtil;

import feathers.controls.Label;

import starling.core.Starling;
import starling.display.DisplayObject;

import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class FeedChallengeWinItemRenderer extends ATimelineCommentLikeItemRender{

    protected var _carNameLabel :     Label;
    protected var _carImageContainer : TimelineFeedImageHolder;

    public function FeedChallengeWinItemRenderer() {
        super();
    }

    protected override function initAllComponents():void {
        super.initAllComponents();
        _carImageContainer = new TimelineFeedImageHolder();
        _mainContainer.addChildAt(_carImageContainer, 1);
        _mainContainer.addEventListener(TouchEvent.TOUCH, onTuningImageTouchHandler);
    }

    private function onTuningImageTouchHandler(event:TouchEvent):void {
        if(_mainContainer && _mainContainer.numChildren > 0) {
            var touch : Touch = event.getTouch(_carImageContainer, TouchPhase.ENDED);
            if (touch && !this.owner.isScrolling) {
                this.owner.dispatchEventWith(FeedData.TUNING_CAR_IMAGE_SELECTED,false, feedData);
            }
        }
    }


    override protected function draw():void {
        super.draw();
    }

    protected override function initLabels():void {
        super.initLabels();
        _carNameLabel = new Label();
        _carNameLabel.textRendererFactory = TimelineLabelFactory.carNameLabelRenderer;
        _carNameLabel.addEventListener(TouchEvent.TOUCH,onCarSelectHandler);
        _feedTitleGroup.addChild(_carNameLabel);
    }

    protected override function update():void {
        if(!feedData) return;
        _nameLabel["text"] = feedData.nameLabel;
        _nameLabel.visible = true;

        _descriptionLabel["text"] = feedData.descriptionLabel;
        _descriptionLabel.visible = true;

        _friendNameLabel.visible = false;
        _friendNameLabel.removeFromParent();

        _carNameLabel.text = feedData.getCarFullName();
        _carNameLabel.visible = true;

        _dateLabel.text = DateParser.getDateAppropriateText(feedData.availableDate);
        _dateLabel.visible = true;

        if(_likeButtonNew){
            _likeButtonNew.removeFromParent();
        }

        _bottomContainer.validate();
        invalidateLikeCommentsViews();
        commitLOadImage();
    }

    private function commitLOadImage():void{
        if(feedData && feedData.tuning){
            loadImage();
        }else{
            Starling.juggler.delayCall(commitLOadImage, .03);
        }
    }

    public function loadImage():void{
        _mainContainer.height = NaN;
        _mainContainer.invalidate();
        _bottomContainer.validate();
        var w : Number = contentWidth;
        var h : Number = stage.stageHeight - tuningImageY;
        feedData.setParent(_carImageContainer, w, h, customAutosize, _bottomContainer.height);
        addEventListener(Event.ENTER_FRAME, carImageContentChangeHandler);
    }

    private function carImageContentChangeHandler(event:Event):void {
        if(_carImageContainer.numChildren || !_carImageContainer){
            removeEventListener(event.type, carImageContentChangeHandler);
            if(_carImageContainer) {
                Starling.juggler.delayCall(function ():void {
                    if(_carImageContainer){
                        addEventListener(event.type, carImageContentChangeHandler);
                    }
                }, .03);

            }
        }
    }

    override protected function customAutosize():void {
        _carImageContainer.invalidate(INVALIDATION_FLAG_SIZE);
        _carImageContainer.validate();
        _mainContainer.invalidate(INVALIDATION_FLAG_SIZE);
        _mainContainer.validate();
        super.customAutosize();
    }

    override public function dispose():void {
        _mainContainer.removeEventListener(TouchEvent.TOUCH, onTuningImageTouchHandler);
        super.dispose();
        DisplayObjectUtil.dispose(_carImageContainer, true, true);
        _carImageContainer.dispose();
        _carImageContainer = null;
    }

    private function onCarSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.touches[0] as Touch;
        if (touch.phase == TouchPhase.ENDED && !TimelineScreen.SCROLL_FLAG){
            this.owner.dispatchEventWith(FeedData.TUNING_CAR_IMAGE_SELECTED,false, feedData);
        }
    }

    override protected function initIcon() : void{
        _postIcon = new Image(iconsMap[FeedData.TYPE_CHALLENGE_WIN]);
        layoutIcon();
    }
}
}
