package com.despark.tuning3d.view.components.timeline.TimelineGarage
{
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.MultilingualButton;
    import com.despark.tuning3d.view.components.timeline.*;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
    import com.utils.FontSelector;

    import feathers.controls.LayoutGroup;
    import feathers.controls.TextInput;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.ITextRenderer;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.VerticalAlign;

    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.display.Quad;
    import starling.events.Event;

    public class WriteComment extends LayoutGroup
    {
        private var _textInput : TextInput;
        private var _sendButton:MultilingualButton;
        private var _user:User;

        public function WriteComment()
        {
        }

        public function set user(value:User) : void{
            _user = value;
        }

        override protected function initialize():void
        {
            super.initialize();

            this.layout = new AnchorLayout();
            this.backgroundSkin = new Quad(1,1,TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
            this.backgroundSkin.alpha = 0.5;


            initTextInput();

            var sendLayData : AnchorLayoutData = new AnchorLayoutData();
            sendLayData.right = 8;
            sendLayData.verticalCenter = 0;

            _sendButton = ButtonFactory.timelineCommentSend();
            _sendButton.isEnabled = false;
            _sendButton.labelFactory = buttonsLabelRenderer;
            _sendButton.layoutData = sendLayData;
            _sendButton.addEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
            addChild(_sendButton);


            this.addEventListener(Event.ENTER_FRAME, onEnterFrame);
        }

        protected function onEnterFrame(event:Event):void
        {
            if (!Device.isIos() && !Constants.IS_EMULATOR){
//                MeasureKeyboard.getInstance().setKeyboardAdjustDefault();
            }
        }

        private function initTextInput () : void
        {
            var layData : AnchorLayoutData = new AnchorLayoutData(1,1,1,1);

            _textInput = new TextInput();
            _textInput.layoutData = layData;
            _textInput.textEditorFactory = TimelineLabelFactory.postInputTextEditorFactory;
            _textInput.promptFactory = TimelineLabelFactory.commentPromptTextRenderer;
            _textInput.verticalAlign = VerticalAlign.MIDDLE;

            _textInput.prompt = "Write a comment";
            _textInput.addEventListener(Event.CHANGE, inputChangeHandler);
            _textInput.padding = 4;
            _textInput.paddingLeft = 8;
            _textInput.paddingRight = 25;
            _textInput.backgroundSkin = new Quad(1,1,0xffffff);
            addChild(_textInput);
        }

        private function inputChangeHandler(event:Event):void {
            _sendButton.isEnabled = _textInput.text.length > 1;
        }

        public function activate() : void
        {
            _textInput.setFocus();
            _sendButton.isEnabled = _textInput.text.length > 1;
        }

        public function deactivate() : void
        {
            _textInput.text = "";
            _textInput.clearFocus();
            _sendButton.isEnabled = false;
        }

        private function sendButtonTriggeredHandler(event:Event):void
        {
            dispatchEventWith(FeedData.COMMENT_SEND, false, _textInput.text)
        }


        override protected function draw():void
        {
            super.draw();

            if (isInvalid(INVALIDATION_FLAG_SIZE))
            {
            }
        }

        override public function dispose():void
        {
            if (_textInput) {
                _textInput.removeFromParent(true);

            }

            if (_textInput) {
                _textInput.removeEventListener(Event.CHANGE, inputChangeHandler);
                _textInput.removeFromParent(true);
                _textInput = null;
            }

            if (_sendButton) {
                _sendButton.removeEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
                _sendButton.removeFromParent(true);
                _sendButton = null;
            }
            this.removeEventListener(Event.ENTER_FRAME, onEnterFrame);

            super.dispose();
        }

        private static function  buttonsLabelRenderer ():ITextRenderer
        {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 10, 0xfeffff);
            textRenderer.embedFonts = FontSelector.embedFonts();
            textRenderer.textFormat.letterSpacing = -.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }
    }
}
