/**
 * Created by Render-2 on 8/15/2017.
 */
package com.despark.tuning3d.view.components.timeline.components {
import com.despark.tuning3d.model.core.Constants;
import com.utils.FontSelector;

import feathers.controls.TextInput;

import feathers.controls.text.StageTextTextEditor;
import feathers.controls.text.TextBlockTextRenderer;
import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.IMultilineTextEditor;
    import feathers.core.ITextEditor;
import feathers.core.ITextRenderer;
    import feathers.layout.HorizontalAlign;
    import feathers.text.StageTextField;

    import flash.text.StyleSheet;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.text.engine.CFFHinting;
import flash.text.engine.ElementFormat;
import flash.text.engine.FontDescription;
import flash.text.engine.FontLookup;
import flash.text.engine.FontPosture;
import flash.text.engine.FontWeight;
import flash.text.engine.RenderingMode;

import starling.filters.ColorMatrixFilter;

public class TimelineLabelFactory {

    public static const COLOR_TEXT_GREY : uint = 0x58595b;
    public static const COLOR_TEXT_RED  : uint = 0xb00000;
    public static const COLOR_TEXT_WHITE : uint = 0xfeffff;
    public static const COLOR_TEXT_LITE_GREY :uint = 0xacacac;
    public static const COLOR_TEXT_MIDDLE_GREY :uint = 0x999999;
    public static const SIZE_EXTRA_LARGE : uint = 16;
    public static const SIZE_BIG_NAME : uint = 15;
    public static const SIZE_SMALL_NAME : uint = 10;
    public static const SIZE_BIG : uint = 14;
    public static const SIZE_SMALL  :uint = 12;
    public static const SIZE_NEWS_SMALL  :uint = 9;
    public static const SIZE_SOCIAL_SMALL  :uint = 8;
    public static const SIZE_DATE_SMALL  :uint = 8;
    public static function getFilter():ColorMatrixFilter {
        var matrix:Vector.<Number> = new <Number>[];
        matrix = matrix.concat(new <Number>[0, 0, 1, 0, 0]);
        matrix = matrix.concat(new <Number>[0, 1, 0, 0, 0]);
        matrix = matrix.concat(new <Number>[1, 0, 0, 0, 0]);
        matrix = matrix.concat(new <Number>[0, 0, 0, 1, 0]);
        return new ColorMatrixFilter(matrix);
    }


    public static function  likeNameLabelRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_GREY);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.35;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  likeCountLabelRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_BIG, COLOR_TEXT_MIDDLE_GREY);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.35;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  nameLabelRenderer ():ITextRenderer{
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getUnicode(), SIZE_BIG, COLOR_TEXT_GREY);
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.filter = getFilter();
        return textRenderer;
    }

    public static function  textLabelRenderer ():ITextRenderer {
        if(Constants.isIOS){
            var iosTextRenderer:TextBlockTextRenderer = new TextBlockTextRenderer();
            var fontDescription:FontDescription = new FontDescription(getEmojiFontName(), FontWeight.NORMAL, FontPosture.NORMAL, FontLookup.DEVICE, RenderingMode.CFF, CFFHinting.NONE );
            iosTextRenderer.elementFormat = new ElementFormat( fontDescription, SIZE_BIG, COLOR_TEXT_GREY);
            iosTextRenderer.textAlign = HorizontalAlign.LEFT;
            iosTextRenderer.filter = getFilter();
            return iosTextRenderer;
        }

        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(getEmojiFontName(), SIZE_BIG, COLOR_TEXT_GREY);
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.wordWrap = true;
        textRenderer.isHTML = true;

        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.filter = getFilter();
        return textRenderer;
    }

    public static function  textInputRenderer ():IMultilineTextEditor {
        var textEditor:StageTextTextEditor = new StageTextTextEditor();
        textEditor.multiline = true;
        textEditor.fontFamily = "Arial";
        textEditor.isEditable = false;
        textEditor.fontSize = SIZE_BIG;
        textEditor.color = COLOR_TEXT_GREY;
        return textEditor;
    }

    // Timeline url snippet renderer

    public static function  snippetTitleRenderer ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), SIZE_SMALL, 0x424242);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.35;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }
    public static function  snippetDescRenderer ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), SIZE_SMALL, 0x9E9E9E);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.35;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }
    //

    public static function  carNameLabelRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_BIG, COLOR_TEXT_RED);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function customLabelRenderer(fontColor : uint = COLOR_TEXT_WHITE, bold : Boolean = false , fontsize : int = SIZE_SMALL):Function {
        return function () : ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), fontsize,fontColor);
            textRenderer.embedFonts = FontSelector.getFont(bold);
            textRenderer.textFormat.letterSpacing = -.5;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = bold;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            return textRenderer;
        }
    }

    public static function  commentButtonLabelTextRenderer ():ITextRenderer{
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_BIG, COLOR_TEXT_WHITE);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        return textRenderer;
    }

    public static function  postDateLabelRenderer ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratMedium(), SIZE_DATE_SMALL, COLOR_TEXT_RED);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.leading = -3;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        return textRenderer;
    }

    public static function  commentTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(getEmojiFontName(), SIZE_BIG, COLOR_TEXT_GREY);
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.wordWrap = true;

        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.filter = getFilter();

        return textRenderer;
    }

    public static function  commentInputRenderer ():ITextRenderer{
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getUnicode(), SIZE_BIG, COLOR_TEXT_GREY);
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.wordWrap = true;
        textRenderer.filter = getFilter();
        return textRenderer;
        /*var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(getEmojiFontName(), SIZE_BIG, COLOR_TEXT_GREY);
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.wordWrap = true;

        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.filter = getFilter();
        return textRenderer;*/
    }

    public static function hex2css(color : int) : String {
        return "#" + color.toString(16);
    }

    public static function getEmojiFontName():String{
        return Constants.isIOS?"Apple Color Emoji":"_sans";
    }

    public static function  socialLabelsTextRenderer1 ():ITextRenderer
    {
        var body:Object = {};
        body.fontStyle = "normal";
        body.fontSize = SIZE_BIG;
        body.color = hex2css(COLOR_TEXT_LITE_GREY);
        body.fontFamily = FontSelector.getFont();
        var heading:Object = {};
        heading.fontWeight = "bold";
        heading.fontSize = SIZE_SMALL;
        heading.fontFamily = FontSelector.getFont(true);
        heading.color = hex2css(COLOR_TEXT_MIDDLE_GREY);

        var style : StyleSheet = new StyleSheet();
        style.setStyle(".heading",heading);
        style.setStyle("body",body);

        var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.wordWrap = false;
        textRenderer.isHTML = true;
        textRenderer.embedFonts = true;
        textRenderer.styleSheet = style;


        return textRenderer;
    }

    public static function  commentEditUpdateTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(true), SIZE_SMALL, COLOR_TEXT_WHITE);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  commentEditTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_RED);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  commentCancelTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_RED);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  commentPromptTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_LITE_GREY);
        textRenderer.embedFonts = FontSelector.getFont(false, true);
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = true;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function  commentDateTextRenderer ():ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), SIZE_SMALL, COLOR_TEXT_LITE_GREY);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function timelineTitleLabelFactory() : ITextRenderer
    {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMontserratSemiBold(), SIZE_BIG, COLOR_TEXT_WHITE);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.letterSpacing = -.5;
        textRenderer.textFormat.italic = false;
        textRenderer.textFormat.bold = true;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        return textRenderer;
    }

    public static function postInputTextEditorFactory():ITextEditor {
//        var editor : TextFieldTextEditor = new TextFieldTextEditor();
//        editor.textFormat = new TextFormat(FontSelector.getFont(),10,0x222222);
//        editor.multiline = true;
//        return editor;

        var editor : StageTextTextEditor = new StageTextTextEditor();
        editor.fontFamily = "Arial";
        editor.fontSize = SIZE_BIG;
        editor.multiline = true;
        editor.color = COLOR_TEXT_GREY;
        return editor;
    }
}
}
