package com.despark.tuning3d.view.components.timeline.components {
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.utils.FontSelector;
    import com.utils.ImageUtils;

    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.FeathersControl;
    import feathers.core.ITextRenderer;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;

    import flash.display.Bitmap;
    import flash.display.Loader;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.net.URLRequest;
    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;
    import flash.utils.Dictionary;

    import starling.display.Canvas;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.textures.Texture;
    import starling.utils.Color;

    public class UserCircularAvatar extends FeathersControl {

    private var _user : User;
    private var size : int;
    private var _loader:Loader;

    private static var avatarMap : Dictionary = new Dictionary();
    private var _loadComplete:Function;
    private var _border:Image;
    private var _borderSize:Number;
    private var _showBorder:Boolean;

    public function UserCircularAvatar(user : User, size : int , showBorder:Boolean = false, borderSize:Number = 0) {
        this._user = user;
        this.size = size;
        _showBorder = showBorder;
        _borderSize = borderSize;
        loadComplete = null;
    }


    private function apply(): void {
        if(_showBorder) {
            _border = new Image(ButtonFactory.assetManager.getTexture('avatar_background'));
            _border.width = _border.height = size;
            _border.color = 0xFEFFFFF;
            addChildAt(_border, 0);
        }
        if(_user){
            if(_user.avatarPath) {
                loadImage(_user.avatarPath);
            } else {
                var defaultAvatar:Sprite = UserCircularAvatar.generateAvatarWithName(_user.username?_user.username:_user.name,size - 2 * _borderSize,Color.GRAY);
                defaultAvatar.x = defaultAvatar.y = _borderSize;
                this.addChild(defaultAvatar);
                _loadComplete.call();
            }
        }
    }

    override protected function initialize():void {
        super.initialize();
        this.setSize(size, size);
        this.apply();
    }

    public function loadImage(url:String) : void {
        if(url.indexOf("http") == - 1) url = "http:" + url;
        if(avatarMap[_user.id] != null) {
            var _avatarImage:Image= new Image(avatarMap[_user.id]) ;
            this.addChild(_avatarImage);
            _avatarImage.width = size - 2 * _borderSize;
            _avatarImage.height = size - 2* _borderSize;
            _avatarImage.x = _avatarImage.y = _borderSize;

            this.setSize(size, size);
            return;
        }

        if(_loader) {
            _loader.unloadAndStop(false);
            _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
            _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
            try{
                _loader.close();
                _loader = null;
            }catch (e:Error) {
                _loader = null;
            }
        }

        _loader = new Loader();
        _loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
        _loader.contentLoaderInfo.addEventListener(Event.COMPLETE, onLoadComplete);
        _loader.load(new URLRequest(url));
    }

    private function onIOErrorHandler(event:IOErrorEvent):void {
        _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadComplete);
        _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
        _loader = null;
        //
        var defaultAvatar:Sprite = UserCircularAvatar.generateAvatarWithName(_user.username?_user.username:_user.name,size - 2 * _borderSize,Color.GRAY);
        defaultAvatar.x = defaultAvatar.y = _borderSize;
        this.addChild(defaultAvatar);
        _loadComplete.call();
    }

    public  function onLoadComplete(event:Event) : void{
        avatarMap[_user.id] = Texture.fromBitmap(ImageUtils.roundCorners(event.currentTarget.loader.content as Bitmap));

        _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE,onLoadComplete);
        _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
        _loader = null;


        var _avatarImage:Image = new Image(avatarMap[_user.id]);
        if(_border && _border.parent == this){
            this.addChildAt(_avatarImage, getChildIndex(_border) + 1);
        }else{
            this.addChildAt(_avatarImage, 0);
        }
        _avatarImage.width = size - 2 * _borderSize;
        _avatarImage.height = size - 2 * _borderSize;
        _avatarImage.x = _avatarImage.y = _borderSize;

        this.setSize(size, size);
        _loadComplete.call();
    }

    public function get user() : User{
        return _user;
    }

    public function set user(value:User):void {
        if(value.id == _user.id) {
            return;
        }
        this.removeChildren(0, -1, true);
        _user = value;
        apply();
    }

    private static function getColorByname(name: String): uint {
        if (!name || name.length === 0) {
            return Color.GRAY;
        }
        var hashStr: Number = 0;
        for (var i: int = 0; i< name.length; i++) {
             var charCode: Number = name.charCodeAt(i);
            hashStr = ((hashStr << 5) - hashStr) + charCode;
            hashStr |= 0;
        }

        var bg: String = '0000' + Math.abs(hashStr).toString(16);
        bg = bg.substr(bg.length - 6);

        var bint: int = parseInt(bg, 16);
        var r: int = (bint >> 16) & 255;
        var g: int = (bint >> 8) & 255;
        var b: int = bint & 255;
        return rgbToHex(r, g, b);
    }

    private static function rgbToHex(r: int, g: int, b: int): uint {
        var hex:uint = r << 16 | g << 8 | b;
        return hex;
    }

    public static function generateAvatarWithName(name : String, size : int, color : uint = 0xacacac) : Sprite
    {
        color = getColorByname(name);
        var canvas:Canvas = new Canvas();
        canvas.beginFill(color);
        canvas.drawCircle(size/2, size/2, size/2);
        canvas.endFill();


        var layData : AnchorLayoutData = new AnchorLayoutData();
        layData.verticalCenter = 0;
        layData.horizontalCenter = 0;

        var container : LayoutGroup = new LayoutGroup();
        container.layout = new AnchorLayout();
        container.addChild(new Quad(size, size, color));
        container.getChildAt(0).mask = canvas;

        var label  : Label = new Label();
        label.layoutData = layData;
        label.textRendererFactory = function () : ITextRenderer {
            var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), size*0.6, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        if(name == null){
            name = "?";
        }
        container.addChild(label);
        label.text = name.charAt(0).toLocaleUpperCase();

        return container;
    }

    public static function disposeStoredAvatarTextures(excludeUsers:Array = null):void {
        if(!excludeUsers){
            excludeUsers = [];
        }
        for (var userId:String in avatarMap) {
            if(excludeUsers.indexOf(userId) == -1){
                var t : Texture = avatarMap[userId] as Texture;
                if(t){
                    t.dispose();
                }
                avatarMap[userId] = null;
                delete avatarMap[userId];
            }
        }
    }

    override public function dispose():void {
        super.dispose();

        if(_loader)
        {
            try
            {
                _loader.unloadAndStop(false);
                _loader.close();

            }catch (e:Error) {

            }
            _loader.contentLoaderInfo.removeEventListener(Event.COMPLETE,onLoadComplete);
            _loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, onIOErrorHandler);
            _loader= null;
        }
        this.removeChildren(0, -1,true);
        if(_border){
            _border.removeFromParent(true);
            _border = null;
        }
    }

    public function set loadComplete(value:Function):void {
        if(value == null){
            _loadComplete = function (...args):void {

            };
            return;
        }
        _loadComplete = value;
    }
}
}
