package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.view.components.challenges.controls.ChallengeStatusInVoting;
	import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.DateParser;
	import com.utils.DisplayObjectUtil;
import com.utils.FontSelector;

import feathers.controls.Button;

import flash.filesystem.File;
	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import feathers.controls.List;
	import feathers.controls.PageIndicator;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.data.ListCollection;
	import feathers.layout.HorizontalLayout;

	import starling.core.Starling;

	import starling.display.DisplayObject;
	import starling.display.DisplayObjectContainer;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.textures.SubTexture;
	import starling.textures.TextureSmoothing;
	import starling.utils.ScaleMode;

	public class TuningRowsFHDItemRenderer extends ListItemRenderer {
		protected var nameBg:Quad;
		private var clipRectQuad:Quad;
		protected var nameLabel:MultilingualLabel;
		public var pageContentCount:int = 1;
		//
		private var leftMargin:Number = 0;
		private var rightMargin:Number = 0;
		private var topMargin:Number = 0;
		private var bottomMargin:Number = 0;
		private var marginSize:Number = 0.75;
		//
		private var itemIndex:int = 0;
		private var _imageWidth:Number = 0;
		private var _imageHeight:Number = 0;
		//
		private var inChallengeBadge:ChallengeStatusInVoting;
		//
		public function get tuning() : Tuning {
			return data as Tuning;
		}
		
		protected override function initialize() : void {
			autoResize = false;
			super.initialize();
			imageLoader.overrideDispose = true;
			label.visible = false;
			nameBg = new Quad(1, 1, 0x000000);
			nameBg.alpha = 0.4;
			addChild(nameBg);

			nameLabel = new MultilingualLabel();
			nameLabel.textFormat = new TextFormat(FontSelector.getFont(false), 10, 0xffffff);
			nameLabel.textFormat.align = TextFormatAlign.LEFT;
			if(Device.isIphoneXVariant()){
				nameLabel.textFormat.align = TextFormatAlign.CENTER;
			}
			nameLabel.textFormat.bold = false;
			addChild(nameLabel);
			nameLabel.updateWithLanguage(AlertView.language);
			clipRectQuad = new Quad(1,1,0x000000);
			addChild(clipRectQuad);
		}
		

        override public function dispose():void {
            DisplayObjectUtil.disposeMesh(nameBg);
			nameBg = null;
            DisplayObjectUtil.disposeMesh(clipRectQuad);
			clipRectQuad = null;
            if(nameLabel)
				nameLabel.dispose();
			nameLabel = null;
			super.dispose();
        }

        protected override function draw() : void {
			super.draw();
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			clipRectQuad.width = actualWidth - (leftMargin + rightMargin);
			clipRectQuad.height = actualHeight - (topMargin + bottomMargin);
			clipRectQuad.x = rightMargin;
			clipRectQuad.y = topMargin;
			mask = clipRectQuad;

			if(sizeInvalid && nameBg){
				nameBg.width = actualWidth;
				nameBg.height = 20;
				nameBg.x = 0;
				nameBg.y = actualHeight - nameBg.height;
			}

			if(dataInvalid && nameLabel){
				if(tuning){
					nameLabel.text=tuning.name;
				}
				nameLabel.height=NaN;
				nameLabel.width=actualWidth-10;
				nameLabel.validate();
				nameLabel.x=5;
				nameLabel.y=(actualHeight-nameBg.height)+((20-nameLabel.height)/2);
			}
		}
		
		protected function completeLoadImage(imageWidth:Number, imageHeight:Number):void{
			imageLoader.alpha = 0;
			imageLoader.color = 0xFF0000;
			_imageWidth = imageWidth;
			_imageHeight = imageHeight;
			layout();
		}

		override protected function layout():void{
			if(imageLoader) {
				imageLoader.width = _imageWidth * actualScale * 0.45;
				imageLoader.height = _imageHeight * actualScale * 0.45;

				imageLoader.x = (actualWidth - imageLoader.width) / 2;
				imageLoader.y = (actualHeight - imageLoader.height) / 2;

				if(imageLoader.alpha == 0){
					Starling.juggler.tween(imageLoader, 0.15, {alpha: 1});
				}
			}
		}

		private function get actualScale():Number{
			if(stage){
				if(actualWidth/actualHeight < stage.stageWidth / stage.stageHeight){
					return actualWidth / stage.stageWidth;
				}
				return actualHeight / stage.stageHeight;
			}
			return 0.3;
		}

		protected override function commitData() : void{
			super.commitData();
			if(tuning){
				itemIndex = owner.dataProvider.getItemIndex(data) % pageContentCount;
				if(pageContentCount == 1 || pageContentCount == 6){
					leftMargin = rightMargin = topMargin = bottomMargin = 0;
				}else if(pageContentCount == 4){
					if(itemIndex % 2 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 2 == 1){
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 2) == 0){
						topMargin = 0;
						bottomMargin = 1;
					}else if(int(itemIndex / 2) == 1){
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}else if(pageContentCount == 9){
					if(itemIndex % 3 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 3 == 1){
						leftMargin = marginSize;
						rightMargin = marginSize;
					}else{
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 3) == 0){
						topMargin = 0;
						bottomMargin = marginSize;
					}else if(int(itemIndex / 3) == 1){
						topMargin = marginSize;
						bottomMargin = marginSize;
					}else{
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}
				//
				if(tuning.inChallenge){
					inChallengeBadge ||= new ChallengeStatusInVoting();
					inChallengeBadge.text = "AVAILABLE FOR CHALLENGE IN " + DateParser.getInChallengeLabel(tuning.inChallenge.time - new Date().time);
					addChild(inChallengeBadge);
				}else if(inChallengeBadge){
					inChallengeBadge.removeFromParent(true);
					inChallengeBadge = null;
				}
			}

			if(tuning){
				if(imageLoader.isLoaded && imageLoader.persistentImageProvider && imageLoader.persistentImageProvider is Car && Car(imageLoader.persistentImageProvider).imageURL == tuning.imageURL){
					return;
				}

				if(imageLoader.isLoaded){
					imageLoader.unloadBitmapDataForDraw();
					imageLoader.sepcialDispose();
				}

				var car:Car = new Car();
				if(tuning.fullHdSnapshots && tuning.fullHdSnapshots[0]){
					car.imageURL = tuning.fullHdSnapshots[0];
					car.imageURL = car.imageURL.replace(/^\/\//gmi, "http://");
				}

				imageLoader.recieveImageSize = completeLoadImage;
				imageLoader.bitmapScale = 0.4;
				imageLoader.persistentImageProvider = car;
			}
		}
	}
}
