package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.cars.CarBackgroundsParser;
	import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.view.components.core.BackgroundContainer;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.utils.Console;
	import com.utils.DisplayObjectUtil;
	import com.utils.ImageUtils;
	import com.utils.ObjectUtils;
	import com.utils.RegExpUtil;

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Loader;
	import flash.display.LoaderInfo;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.filesystem.File;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.net.URLRequest;
	import flash.utils.Dictionary;

	import starling.display.Quad;
	import starling.textures.Texture;
	import starling.utils.MathUtil;

	public class TuningSlideItemRenderer extends ListItemRenderer
	{
		
		private var clipRectQuad:Quad;
		
		private var backgroundContainer:BackgroundContainer = null;
		private var _slide:Object;
		public var clipBackground:Boolean = false;

        private var _loaderArr: Dictionary = new Dictionary();
        private var _overrideDispose:Boolean = false;
		private var _fitContent:Boolean = false;
		private var _carOffset:Point = new Point();
		private var _carZoom:Number = 1;
		private var _topOffset:Number = -25;
		private var _leftOffset:Number = 0;
		private var _contentCentralized:Boolean = false;

		public function get slide() : Object {
			return data as Object;
		}
		
		protected override function initialize() : void {
			backgroundContainer = new BackgroundContainer();
			backgroundContainer.moderateSize = false;
            overrideDispose = _overrideDispose;
            setBackgroundSize();
			addChild(backgroundContainer);
			super.initialize();
			clipRectQuad = new Quad(1,1);
			this.label.visible=false;
		}

		private function setBackgroundSize():void{
            if(_fitContent && backgroundContainer.texture){
				updatePositions();
            }else if(clipBackground) {
				var h:Number = actualHeight + 50;
				var w:Number = actualWidth * h / actualHeight;
				backgroundContainer.setSize(w, h);
			} else {
                backgroundContainer.setSize(actualWidth, actualHeight);
            }
            backgroundContainer.x = (actualWidth - backgroundContainer.width) / 2 + _leftOffset;
            backgroundContainer.y = (actualHeight - backgroundContainer.height) / 2 + _topOffset;
			Console.log("setBackgroundSize", backgroundContainer.y);
		}
		
		protected override function draw() : void {
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(sizeInvalid){
				if(!this.mask){
					clipRectQuad.x = 0;
					clipRectQuad.width = actualWidth;
					if(clipBackground){
                        clipRectQuad.height = actualHeight + 50;
                        clipRectQuad.y = -25;
						addChild(clipRectQuad);
                    }else{
                        clipRectQuad.height = actualHeight;
                        clipRectQuad.y=0;
					}
					 this.mask = clipRectQuad;
				}else if(clipBackground){
                    if(mask.height != actualHeight + 50){
                        mask.height = actualHeight + 50;
                    }
                    if(mask.y != -25){
                        mask.y = -25;
                    }
                    if(!mask.parent){
                        addChild(mask);
                    }
				}
			}
		}
		
		protected function completeLoadImage(imageWidth:Number, imageHeight:Number):void{
			trace("imageWidth:", imageWidth, "imageHeight:", imageHeight);
			if(_fitContent && imageLoader.image){
				imageLoader.scale = 1;
				if(owner.width / owner.height < imageLoader.originalSourceWidth / imageLoader.originalSourceHeight){
					imageLoader.height = _carZoom * owner.height;
					imageLoader.width = _carZoom * owner.height * imageLoader.originalSourceWidth / imageLoader.originalSourceHeight;
				}else{
					imageLoader.width = _carZoom * owner.width;
					imageLoader.height = _carZoom * owner.width * imageLoader.originalSourceHeight / imageLoader.originalSourceWidth;
				}
			}else{
				imageLoader.width =  _carZoom * imageWidth / 2;
				imageLoader.height = imageLoader.originalSourceHeight * imageLoader.scaleX;
			}

			if(backgroundContainer.texture){
				setSize(owner.width, owner.height);
				if(imageLoader.image){
					imageLoader.validate();
					backgroundContainer.background.scale = imageLoader.image.scale / _carZoom;
					backgroundContainer.width = backgroundContainer.background.width;
					backgroundContainer.height = backgroundContainer.background.height;
					backgroundContainer.validate();
					if(_contentCentralized){
						var rect:Rectangle = ImageUtils.getCarImageRect(imageLoader.image);
						_leftOffset = (imageLoader.image.width - rect.width) / 2 - rect.x;
					}
				}
				backgroundContainer.x = (actualWidth - backgroundContainer.width) / 2 + _leftOffset;
				backgroundContainer.y = (actualHeight - backgroundContainer.height) /2 + _topOffset;
				Console.log("_leftOffset", _leftOffset);
			}
			var dx:Number = 0;
			var dy:Number = 0;
			if(slide && slide.tuning && slide.tuning.carObj && slide.tuning.carObj.custom_offset){
				dx = slide.tuning.carObj.custom_offset.left;
				dy = slide.tuning.carObj.custom_offset.bottom;
			}
			imageLoader.x = (actualWidth - imageLoader.width) / 2 + _carOffset.x * imageLoader.image.scale + dx + _leftOffset;
			imageLoader.y = (actualHeight - imageLoader.height)/ 2 + _carOffset.y * imageLoader.image.scale + dy + _topOffset;
			updatePositions();
		}

		override public function setSize(w:Number, h:Number):void{
			super .setSize(w, h);
		}

		public function updatePositions():void{
			if(_fitContent){
				if(backgroundContainer.texture){
					setSize(owner.width, owner.height);
					if(imageLoader.image){
						imageLoader.validate();
						backgroundContainer.background.scale = imageLoader.image.scale;
						backgroundContainer.width = backgroundContainer.background.width;
						backgroundContainer.height = backgroundContainer.background.height;
						backgroundContainer.validate();
					}
					backgroundContainer.x = (actualWidth - backgroundContainer.width) / 2 + _leftOffset;
					backgroundContainer.y = (actualHeight - backgroundContainer.height) / 2 + _topOffset;
					Console.log("updatePositions", backgroundContainer.y);
				}
				
				if(imageLoader.image){
					imageLoader.validate();
					var offsetObj:Object = ObjectUtils.getProperty(data, "tuning.carObj.custom_offset");
					if(offsetObj){
						imageLoader.x = (actualWidth - imageLoader.width) / 2 + 2 * (_carOffset.x - offsetObj.left) * imageLoader.image.scale + _leftOffset;
						imageLoader.y = (actualHeight - imageLoader.height) / 2 + 2 * (_carOffset.y + offsetObj.bottom) * imageLoader.image.scale + _topOffset;
					}
				}
			}
		}

		private function getBgUrl() : String {
            if(!slide) return "";
			var cameraIndex:int;
            var tuning:Tuning = slide.tuning;
            if(tuning.customSnapshotsBackgrounds && tuning.customSnapshotsBackgrounds[index] && tuning.customSnapshotsBackgrounds[index] != CarBackground.DEFAULT_ID
                    && tuning.backgroundsMap && tuning.backgroundsMap[tuning.customSnapshotsBackgrounds[index]]){

                var carBackground:CarBackground=  tuning.backgroundsMap[tuning.customSnapshotsBackgrounds[index]];//tuning.backgroundsMap[tuning.backgroundId];
                carBackground.showThumbnailUrl=false;
				cameraIndex =0;
                if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0 && carBackground.imagePathSecondCam && carBackground.imagePathSecondCam.length>0){
                    cameraIndex = tuning.customSnapshotsCamera[slide.index];
                }

                carBackground.selectedCamera =cameraIndex;
                return carBackground.remoteUrl;

            }else{

				var randomSplashUrl:String = "";
				if(tuning.customSnapshotsBackgrounds){
					var i:int;
					for(i = tuning.customSnapshotsBackgrounds.length-1; i >= 0; i--){
						if(!tuning.customSnapshotsBackgrounds[i] || tuning.customSnapshotsBackgrounds[i] == undefined){
							tuning.customSnapshotsBackgrounds[i] = CarBackground.DEFAULT_ID;
						}
					}
					for(i = tuning.customSnapshotsBackgrounds.length-1; i >= 0; i--){
						if(tuning.customSnapshotsCamera == null){
							tuning.customSnapshotsCamera = [];
						}
						if(!tuning.customSnapshotsCamera[i]){
							tuning.customSnapshotsCamera[i] = 0;
						}
						if(tuning.customSnapshotsBackgrounds[i] == CarBackground.DEFAULT_ID && tuning.carObj && tuning.carObj.defaultBkgSite && tuning.carObj.defaultBkgSite.length){
							tuning.customSnapshotsBackgrounds[i] = tuning.carObj.defaultBkgSite[tuning.customSnapshotsCamera[i]];
						}else if(!RegExpUtil.isUrl(tuning.customSnapshotsBackgrounds[i]) && tuning.backgroundsMap && tuning.backgroundsMap[tuning.customSnapshotsBackgrounds[i]]){
							var bg:CarBackground = tuning.backgroundsMap[tuning.customSnapshotsBackgrounds[i]];
							tuning.customSnapshotsBackgrounds[i] = bg.remoteUrl;
						}
					}
					if(tuning.customSnapshotsBackgrounds.length) {
                        index = MathUtil.clamp(index, 0, tuning.customSnapshotsBackgrounds.length - 1);
                        randomSplashUrl = tuning.customSnapshotsBackgrounds[index];
						if(randomSplashUrl && randomSplashUrl.indexOf(Constants.BACKGROUNDS_URL) != -1){
							return 'http:' + randomSplashUrl.replace(/https?:/i, '');
						}
						for(i = 0; i < tuning.customSnapshotsBackgrounds.length; i++){
							randomSplashUrl = tuning.customSnapshotsBackgrounds[i];
							if(randomSplashUrl && randomSplashUrl.indexOf(Constants.BACKGROUNDS_URL) != -1){
								return 'http:' + randomSplashUrl.replace(/https?:/i, '');
							}
						}
                    }
				}

                if(tuning.carChooserBgName) {
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/"+tuning.carChooserBgName.replace('.jpg', '') + ".jpg").url;
					if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0){
						cameraIndex = tuning.customSnapshotsCamera[slide.index];
                        if(tuning.carObj.defaultBkgMobile[cameraIndex]){
                            randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
                        }
                    }
                    return randomSplashUrl;
                }
				if(tuning.customSnapshotsBackgrounds && tuning.customSnapshotsBackgrounds.length){
					var lastIndex:int = MathUtil.clamp(index, 0, tuning.customSnapshotsBackgrounds.length-1);
					var bgURL:String = "";
					if(tuning.customSnapshotsBackgrounds[lastIndex] is String){
						bgURL = tuning.customSnapshotsBackgrounds[lastIndex];
					}else if(tuning.customSnapshotsBackgrounds[lastIndex] && tuning.customSnapshotsBackgrounds[lastIndex].hasOwnProperty("imagePath")){
						bgURL = tuning.customSnapshotsBackgrounds[lastIndex]["imagePath"];
					}
					if(bgURL.indexOf("data.3dtuning.com") > -1){
						index = lastIndex;
						return 'http:' + bgURL.replace(/https?:/i, '');
					}
				}
				// randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/car_chooser.jpg").url;
				return bgURL;
            }
		}

		private function getDefaultBgUrl():String{
			return File.applicationDirectory.resolvePath("assets/textures/bgs/car_chooser.jpg").url
		}


        override public function dispose():void {
            super.dispose();
			if(backgroundContainer){
                backgroundContainer.removeFromParent(true);
                backgroundContainer = null;
			}
			if(clipRectQuad){
                clipRectQuad.removeFromParent(true);
                clipRectQuad = null;
			}
			DisplayObjectUtil.dispose(this);
        }


        protected override function commitData() : void {
			super.commitData();
			if(!slide || slide == _slide) {
				return;
            }
			_slide = slide;
			if(slide && slide.tuning.carTransform && slide.tuning.carTransform[slide.index]){
				_carZoom = slide.tuning.carTransform[slide.index].zoomLevel;
				_carOffset.setTo(slide.tuning.carTransform[slide.index].pan.x, slide.tuning.carTransform[slide.index].pan.y);
			}else{
				_carZoom = 1;
				_carOffset.setTo(0, 0);
			}
            setBackgroundSize();
			var bgURL:String = getBgUrl();
			if(RegExpUtil.isUrl(bgURL)){
				loadBackground(bgURL);
			}else if(bgURL && bgURL.length){
				var carBackgroundsProxy: CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
				carBackgroundsProxy.getURLById(bgURL, function(success:Boolean, result:Object):void{
					if(success){
						var backgroundData:Object = JSON.parse(result as String);
						var parser:CarBackgroundsParser = new CarBackgroundsParser();
						var carBackground:CarBackground = parser.parseObject(backgroundData, null) as CarBackground;
						if(carBackground.mainUrl.indexOf("http:") == -1){
							carBackground.mainUrl = "http:" + carBackground.mainUrl;
						}
						loadBackground(carBackground.mainUrl);
					}else{
						loadBackground(getDefaultBgUrl());
					}
				});
			}

			var car:Car = new Car();
			car.imageURL = slide.url;
			imageLoader.recieveImageSize = completeLoadImage;
			imageLoader.overrideDispose = _overrideDispose;
			imageLoader.persistentImageProvider = car;
		}

		private function loadBackground(url:String):void{
			var loader: Loader = new Loader();
			_loaderArr[url] = loader;
			loader.contentLoaderInfo.addEventListener(Event.COMPLETE, function (e:Event):void {
				var currentTarget:LoaderInfo = e.currentTarget as LoaderInfo;
				var smallBMD:BitmapData =(currentTarget.content as Bitmap).bitmapData; //new BitmapData(currentTarget.content.width * scale, currentTarget.content.height * scale, true, 0x000000);

				if (backgroundContainer) {
					var bgUrl:String = url;
					var texture:Texture = Texture.fromBitmapData(smallBMD);
					texture.root.onRestore = function():void{
						loadBackground(bgUrl);
					};
					backgroundContainer.updateBackground(texture);
					setBackgroundSize();
				}

				smallBMD.dispose();
				delete _loaderArr[url];
			});
			loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, function (e:Event):void {
				trace('error', url);
				delete _loaderArr[url];
			});
			loader.contentLoaderInfo.addEventListener(SecurityErrorEvent.SECURITY_ERROR, function (e:Event):void {
				trace('error', url);
				delete _loaderArr[url];
			});

			var request:URLRequest = new URLRequest(url);
			loader.load(request);

		}

        public function set overrideDispose(overrideDispose:Boolean):void {
            _overrideDispose = overrideDispose;
			if(imageLoader){
				imageLoader.overrideDispose = _overrideDispose;
			}
        }

		public function set fitContent(value:Boolean):void {
			_fitContent = value;
		}

		public function set topOffset(value:Number):void {
			_topOffset = value;
		}

		public function set leftOffset(value:Number):void {
			_leftOffset = value;
		}

		public function set contentCentralized(value:Boolean):void {
			_contentCentralized = value;
		}
	}
}
