package com.despark.tuning3d.view.components.tunings.colors
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.Part;
	import com.despark.tuning3d.model.parts.PartColor;
	import com.despark.tuning3d.model.parts.PartMaterial;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.MaterialItemRenderer;
	import com.utils.ArrayUtil;

	import feathers.controls.List;
	import feathers.controls.ScrollContainer;
	import feathers.controls.renderers.IListItemRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ToggleGroup;
	import feathers.data.ListCollection;
	import feathers.layout.HorizontalLayout;
	import feathers.layout.VerticalAlign;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class TuningColorsView extends FeathersControl implements Multilingual {
		public static const COLOR_SELECTED : String = "ColorSelected";
		public static const COLOR_PICKER_SELECTED : String = "ColorPickerSelected";

		public static const PART_SLOT_CHANGED:String = "partSlotChanged";

		public static const MATERIAL_SELECTED : String = "MaterialSelected";
		public var initialHeight: int;
		public var assetManager : AssetManager;
		private var _partColors : Array;
		public var selectedColor : PartColor;
		
		private var currentLanguage : Language;
		private var colorContainer : ScrollContainer;
		private var quad : Quad;

		private var moreColorsSeparator:Image;
		private var partColorsSeparator:Image;

		private var partColorButton:PartColorButton;
		private var interiorColorButton:PartColorButton;
		private var exteriorColorButton:PartColorButton;

		private var toggleGroup:ToggleGroup;

		public var materialList:List;

		//данные
		private var _partSlot:PartSlot;
		private var _selectedPartSlot:PartSlot;
		private var _materialChangeBlocker:Boolean=false;
		public static const EXTERIOR:String = 'exteriorColorButton';
		public static const INTERIOR:String = 'interiorColorButton';



		public function TuningColorsView() {
			currentLanguage=AlertView.language;
			super();
		}

		public function set partColors(value:Array):void{
			if(value == null) {
				return;
			}
			_partColors=value;
			invalidate("partColors");
		}
		
		public function get partColors():Array{
			return _partColors;
		}
		
		public function updateWithLanguage(language : Language) : void {
			currentLanguage = language;
		}
		
		override protected function initialize() : void {
			super.initialize();
			quad = new Quad(1, 1, 0x000000);
			quad.alpha = 0.1;
			addChild(quad);
			
			var layout : HorizontalLayout = new HorizontalLayout();
			layout.verticalAlign = VerticalAlign.MIDDLE;
			layout.gap = 0;
			layout.paddingLeft = 0;
			layout.paddingRight = 0;

			colorContainer = new ScrollContainer();
			colorContainer.clipContent = true;
			colorContainer.layout = layout;
			addChild(colorContainer);

			moreColorsSeparator = createSeporator();
			addChild(moreColorsSeparator);

			exteriorColorButton = new PartColorButton();
			exteriorColorButton.paddingLeft = 3;
			exteriorColorButton.paddingRight = 3;
			addChild(exteriorColorButton);
			exteriorColorButton.label = "Exterior";

			interiorColorButton = new PartColorButton();
			interiorColorButton.paddingLeft = 3;
			interiorColorButton.paddingRight = 3;
			addChild(interiorColorButton);
			interiorColorButton.label = "Interior";

			partColorButton = new PartColorButton();
			partColorButton.paddingLeft = 3;
			partColorButton.paddingRight = 3;
			addChild(partColorButton);

			toggleGroup = new ToggleGroup();
			toggleGroup.addItem(exteriorColorButton);
			toggleGroup.addItem(interiorColorButton);
			toggleGroup.addItem(partColorButton);
			toggleGroup.isSelectionRequired = true;
			toggleGroup.addEventListener(Event.CHANGE, groupChangeHandler);

			partColorsSeparator=createSeporator();
			addChild(partColorsSeparator);
			
			var possiblePartSlotsLayout : HorizontalLayout = new HorizontalLayout();
			possiblePartSlotsLayout.useVirtualLayout = false;
			possiblePartSlotsLayout.gap = 16;
			possiblePartSlotsLayout.paddingLeft = 10;
			possiblePartSlotsLayout.paddingRight = 10;
			
			materialList = new List();
			materialList.snapScrollPositionsToPixels = true;
			materialList.hasElasticEdges=false;
			materialList.clipContent = true;
			materialList.layout = possiblePartSlotsLayout;
			materialList.addEventListener(Event.CHANGE, materialListChangeHandler);
			materialList.addEventListener("itemSelectedAgain", materialListItemSelectedAgain);
			materialList.itemRendererFactory = function() : IListItemRenderer {
				var renderer :MaterialItemRenderer = new MaterialItemRenderer();
				renderer.language = AlertView.language;
				return renderer;
			}
			addChild(materialList);
		}

		private function groupChangeHandler(event:Event):void {
			if(selectedComponentType.name == EXTERIOR){
				_selectedPartSlot = selectedComponentType.parentCar.getPossiblePartSlot("body");
				var maxCount:int = 0;
				var exteriorComponents:Array = selectedComponentType.parentCar.exteriorSelectedPartSlots;
				if(_selectedPartSlot && exteriorComponents.indexOf(_selectedPartSlot) == -1){
					exteriorComponents.push(_selectedPartSlot)
				}
				for each(var component:PartSlot in exteriorComponents){
					if(component && component.selectedPart && component.selectedPart.possibleMaterials){
						if(component.selectedPart.possibleMaterials.length > maxCount){
							_selectedPartSlot = component;
							maxCount = component.selectedPart.possibleMaterials.length;
						}
					}
				}
			}else if(selectedComponentType.name == INTERIOR){
				_selectedPartSlot = selectedComponentType.parentCar.interiorSelectedPartSlots[0];
			}else{
				_selectedPartSlot = _partSlot;
			}
			materialList.removeEventListener(Event.CHANGE, materialListChangeHandler);
			materialList.removeEventListener("itemSelectedAgain", materialListItemSelectedAgain);
			invalidate("updatePartSlot");
			materialList.addEventListener(Event.CHANGE, materialListChangeHandler);
			materialList.addEventListener("itemSelectedAgain", materialListItemSelectedAgain);

			if(_selectedPartSlot && _selectedPartSlot.selectedPart){
				isEnabled = hasPaintMaterial;
			}else{
				isEnabled = false;
			}

			dispatchEventWith(PART_SLOT_CHANGED, false, {partSlot: selectedComponentType});
		}

		public function set partSlot(value:PartSlot):void{
			_partSlot = value;
			_selectedPartSlot = _partSlot;
			partColorButton.partSlot = _partSlot;
			if(_partSlot && !exteriorColorButton.partSlot && !interiorColorButton.partSlot){
				exteriorColorButton.partSlot = new PartSlot(_partSlot.parentCar);
				interiorColorButton.partSlot = new PartSlot(_partSlot.parentCar);

				exteriorColorButton.partSlot.name = EXTERIOR;

				interiorColorButton.partSlot.name = INTERIOR;
			}
			if(toggleGroup.selectedItem != partColorButton){
				toggleGroup.selectedItem = partColorButton;
			}

			if(_selectedPartSlot && _selectedPartSlot.selectedPart){
				isEnabled = hasPaintMaterial;
			}else{
				isEnabled = false;
			}
			this.invalidate("updatePartSlot");
		}
		
		public function createSeporator():Image{
			
			var seporatorTexture:Texture=assetManager.getTexture("a_seporator_1");

			var seporatorImage:Image=new Image(seporatorTexture);
			
			seporatorImage.width=1.5;
			seporatorImage.height=34;
			
			return seporatorImage;
		}
		
		private function refresh() : void
		{
			
			if(!partColors)
				return;
			
			var i:int=0;
			var tuningSelectedColorView : TuningSelectedColorView;
			
			
			
			if(colorContainer){
				
				
				while( colorContainer.numChildren>0){
					
					
					tuningSelectedColorView=colorContainer.getChildAt(0) as TuningSelectedColorView;
					
					tuningSelectedColorView.removeEventListener(Event.SELECT, tuningSelectedColorViewSelectHandler);
						
					colorContainer.removeChild(tuningSelectedColorView);
					
					
					tuningSelectedColorView.assetManager=null;
					tuningSelectedColorView.dispose();
					tuningSelectedColorView=null;
				}
			}
			
			
			
			var itemHeight : int = initialHeight;

			var currentX : int = 0;

			for( i  = 0; i < partColors.length; i++)
				{
				tuningSelectedColorView  = new TuningSelectedColorView();


				tuningSelectedColorView.assetManager = assetManager;
				tuningSelectedColorView.x = currentX;
				tuningSelectedColorView.width = 40.5;
				tuningSelectedColorView.height = itemHeight;
				tuningSelectedColorView.showsMaterial = false;
				tuningSelectedColorView.partColor = partColors[i].copy();
				tuningSelectedColorView.addEventListener(Event.SELECT, tuningSelectedColorViewSelectHandler);
				
				colorContainer.addChild(tuningSelectedColorView);
			}
		}
		
		override protected function draw() : void {
			super.draw();
			
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			
			if(sizeInvalid) {
				quad.width = actualWidth;
				quad.height = actualHeight;

				partColorButton.width = 40;
				partColorButton.height = actualHeight;

				interiorColorButton.width = 40;
				interiorColorButton.height = actualHeight;

				exteriorColorButton.width = 40;
				exteriorColorButton.height = actualHeight;

				colorContainer.width = (actualWidth - 3 * partColorButton.width - 10 + 2 * moreColorsSeparator.width) / 2;
				materialList.width = (actualWidth - 3 * partColorButton.width - 10 + 2 * moreColorsSeparator.width) / 2;

				colorContainer.height = actualHeight;
				materialList.height = actualHeight;

				exteriorColorButton.x = 0;
				exteriorColorButton.y = (actualHeight - exteriorColorButton.height) / 2;

				interiorColorButton.x = exteriorColorButton.x + exteriorColorButton.width;
				interiorColorButton.y = (actualHeight - interiorColorButton.height) / 2;

				partColorButton.x = interiorColorButton.x + interiorColorButton.width;
				partColorButton.y = (actualHeight - partColorButton.height) / 2;

				moreColorsSeparator.x = partColorButton.x + partColorButton.width;
				moreColorsSeparator.y = (actualHeight - moreColorsSeparator.height) / 2;

				colorContainer.x = moreColorsSeparator.x + moreColorsSeparator.width;

				partColorsSeparator.x = colorContainer.x + colorContainer.width - partColorsSeparator.width;
				partColorsSeparator.y = (actualHeight - partColorsSeparator.height) / 2;

				materialList.x = partColorsSeparator.x + partColorsSeparator.width + 2.5;
				materialList.y = 0;
			}
			
			if(isInvalid("partColors")) {
				refresh();
			}
			
			if(isInvalid("updatePartSlot")){
				var i:int;
				if(_selectedPartSlot && _selectedPartSlot.selectedPart && _selectedPartSlot.selectedPart.possibleMaterials && _selectedPartSlot.selectedPart.possibleMaterials.length>0){
					var list:Array=[];
					var partMaterial:PartMaterial;
					//
					var fixed:Array = new Array(PartMaterial.FIXED_MATERIAL_TYPES.length);
					var others:Array = [];
					//
					for(i=0; i<_selectedPartSlot.selectedPart.possibleMaterials.length; i++){
						partMaterial=_selectedPartSlot.selectedPart.possibleMaterials[i];
						if(partMaterial.name=="metal" && _selectedPartSlot.dummy && !_selectedPartSlot.parentCar.isChopper)
							continue;
						if(partMaterial.name!="mask" && partMaterial.name!="antimask") {
                            if(PartMaterial.FIXED_MATERIAL_TYPES.indexOf(partMaterial.name) !=-1){
                                fixed[PartMaterial.FIXED_MATERIAL_TYPES.indexOf(partMaterial.name)] = partMaterial;
                            }else {
                                others.push(_selectedPartSlot.selectedPart.possibleMaterials[i]);
							}
						}
					}
					for(i = fixed.length - 1; i >= 0; i--){
						if(!fixed[i]){
							fixed.removeAt(i);
						}
					}
					others.sortOn(["name"], Array.CASEINSENSITIVE);
					list = fixed.concat(others);
					list = ArrayUtil.onlyUnique(list);
					//возможно нужно зафильтровать
					materialList.dataProvider = new ListCollection(list);
					if(_selectedPartSlot.selectedPart && _selectedPartSlot.selectedPart.selectedMaterial){
						_materialChangeBlocker = true;
						for(i = 0; i < list.length; i++) {
                            if(_selectedPartSlot.selectedPart.selectedMaterial.name == list[i].name) {
                                materialList.selectedItem=list[i];
                                _materialChangeBlocker=false;
								break;
							}
						}
						_materialChangeBlocker=false;
					}
				}else{
					materialList.dataProvider=new ListCollection();
				}
				
				if(_partSlot){
					if(_partSlot.name != "ALL") {
						partColorButton.label = AlertView.language.translate("componentTypes", _partSlot.name);
					} else {
						partColorButton.label = AlertView.language.translate("mob_saving", "all");
					}
				}
			}
		}

    protected function materialListItemSelectedAgain(event:Event):void{

        if(materialList.selectedIndex<0) return;

        if(!_partSlot) return;

        if(!_partSlot.selectedPart) return;
		var part : Part = (_partSlot.selectedPart.selectedState == null) ? _partSlot.selectedPart : _partSlot.selectedPart.selectedState;

        var isExistGlossyMaterial:Boolean=part.getPossibleMaterial("glossy");
        if((materialList.selectedItem as PartMaterial).name!="glossy" && isExistGlossyMaterial ){
            materialList.selectedIndex=-1;
            dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), partSlot: _partSlot, selectedColor:_partSlot.previousColor});
			this.colorContainer.touchable=true;
			this.colorContainer.alpha=1;
			

        }
    }


        override public function set isEnabled(value:Boolean):void {
            super.isEnabled = value;
            if(!value){
                this.colorContainer.touchable=false;
                this.colorContainer.alpha=0.5;
            }else{
                this.colorContainer.touchable=true;
                this.colorContainer.alpha=1;
            }
        }

        private function materialListChangeHandler(event:Event):void{
			if(_materialChangeBlocker) {
				return;
            }
			if(_selectedPartSlot && _selectedPartSlot.selectedPart && _selectedPartSlot.selectedPart.possibleMaterials && _selectedPartSlot.selectedPart.possibleMaterials.length>0 ){
				var selectedMaterial : PartMaterial;
				var possibleMaterials:Array=_selectedPartSlot.selectedPart.possibleMaterials;
				selectedMaterial = possibleMaterials[possibleMaterials.indexOf(materialList.selectedItem)];
				if(!selectedMaterial) {
					return;
                }
				if(_selectedPartSlot.selectedPart.selectedState) {
					var selectedState : Part = _selectedPartSlot.selectedPart.selectedState;
					var length : int = selectedState.possibleMaterials.length;
					var materialIsFound : Boolean = false;
					if(length <= 0) {
						return;
					}
					for(var i : int  = 0; i < length; i ++) {
						if(selectedState.possibleMaterials[i].name == selectedMaterial.name) {
							materialIsFound = true;
						}
					}
					if(!materialIsFound) {
						return;
					}
				}

				SoundManager.playSound(SoundManager.SELECT_PART);
				var selectedColor:PartColor = _selectedPartSlot.previousColor;
				if(selectedComponentType.name == EXTERIOR){
					selectedColor = null;
				}
				dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: selectedMaterial, partSlot: selectedComponentType, selectedColor: selectedColor});
				isEnabled = hasPaintMaterial;
			}
		}

		private function get selectedComponentType():PartSlot{
			return PartColorButton(toggleGroup.selectedItem).partSlot;
		}

        public function get hasPaintMaterial():Boolean{
			var materials:Array;
			if(!_selectedPartSlot){
				return false;
			}
			if(_selectedPartSlot.selectedPart && _selectedPartSlot.selectedPart.selectedState){
				materials = _selectedPartSlot.selectedPart.selectedState.possibleMaterials;
			}else if( _selectedPartSlot.selectedPart){
				materials = _selectedPartSlot.selectedPart.possibleMaterials;
			}else{
				return false;
			}
			var value:Boolean = false;
			for(var i:int = 0; i < materials.length; i++){
				if(PartMaterial(materials[i]).isPaintable && PartMaterial(materials[i]).name != "mask"){
					value = true;
					break;
				}
			}
            return value;
        }
	
		private function tuningSelectedColorViewSelectHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PAINTING);
			var tuningSelectedColorView : TuningSelectedColorView = TuningSelectedColorView(event.currentTarget); 
			selectedColor = tuningSelectedColorView.partColor;

            var part : Part = (_selectedPartSlot.selectedPart.selectedState == null) ? _selectedPartSlot.selectedPart : _selectedPartSlot.selectedPart.selectedState;
			
			var isExistWhiteMaterial:Boolean=part.getPossibleMaterial("white");
			var isExistGlossyMaterial:Boolean=part.getPossibleMaterial("glossy");
			if(!part.selectedMaterial){
				if(isExistGlossyMaterial){
					part.selectedMaterial = part.getPossibleMaterial("glossy");
				}else{
					part.selectedMaterial = part.possibleMaterials[0];
				}
			}

			if(!selectedColor.isCustom) {
				
				if(part.selectedMaterial.name=="mate" || part.selectedMaterial.name=="redblack"){
					
					dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
					
				}else if(isExistWhiteMaterial){
					if(selectedColor.value!=0xfeffff && part.selectedMaterial.name=="white" && isExistGlossyMaterial){

							this.materialList.selectedIndex=-1;
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedColor: selectedColor, selectedMaterial: part.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});
							//
							
						}else if(part.selectedMaterial.name=="metal" && isExistGlossyMaterial){
							
							this.materialList.selectedIndex=-1;
							
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: part.getPossibleMaterial("glossy"), selectedColor: selectedColor, partSlot: _partSlot, onComplete:function():void{
								
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});
							
						}else if(!part.selectedMaterial.isPaintable && isExistGlossyMaterial){
                            this.materialList.selectedIndex=-1;
                            dispatchEventWith(MATERIAL_SELECTED, true, {selectedColor: selectedColor, selectedMaterial: part.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
                                    dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
                                }});
                        }else{
							
							dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
						}
				}else{
						
						if(part.selectedMaterial.name!="glossy" && isExistGlossyMaterial){
							
							this.materialList.selectedIndex=-1;
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), selectedColor:selectedColor, partSlot: _partSlot, onComplete:function():void{
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});

						}else{
							
							dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
						}
				}
				
			}
			else
			{
				dispatchEventWith(COLOR_PICKER_SELECTED, true);
			}
		}

		public function get selectedPartSlot():PartSlot {
			return _selectedPartSlot;
		}

		public function get partSlot():PartSlot {
			return _partSlot;
		}
	}
}