package com.despark.tuning3d.view.components.tunings.editing
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.assets.AssetsProxy;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
	import com.despark.tuning3d.model.challenges.ChallengeData;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.PartColor;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.tunings.colors.*;
	import com.utils.ObjectUtils;

	import feathers.controls.Button;
	import feathers.controls.LayoutGroup;
	import feathers.core.FeathersControl;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.HorizontalLayout;
	import feathers.layout.RelativePosition;
	import feathers.layout.VerticalAlign;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class TuningSettingsPanel extends LayoutGroup implements Multilingual {
		public static const  OPEN_BACKGROUND:String = "OPEN_BACKGROUND";
		private static const BUTTON_GAP:int = 20;
		public var assetManager : AssetManager;
		private var currentLanguage : Language;
		private var quad : Quad;

		private var customBackgroundButton:Button;
		private var effectMenuButton:Button;
		private var effectMenu:EffectMenu;

		public var suspensionControl:TuningSuspensionControl;
		public var tintControl:SliderControl;

		//данные
		private var _backgroundsIsDownloaded: Boolean = false;
		private var _car:Car;
		private var _challengeData:ChallengeData;
		private var mainLayout : HorizontalLayout;
		public function TuningSettingsPanel() {
			currentLanguage = AlertView.language;
			super();
		}

		public function updateWithLanguage(language : Language) : void {
			currentLanguage = language;
		}
		
		override protected function initialize() : void {
			super.initialize();
			quad = new Quad(1, 1, 0x000000);
			quad.alpha = 0.1;
			// addChild(quad);

			mainLayout = new HorizontalLayout();
			mainLayout.verticalAlign = VerticalAlign.MIDDLE;
			mainLayout.horizontalAlign = HorizontalAlign.CENTER;
			mainLayout.paddingLeft = 20;
			mainLayout.paddingRight = 20;
			layout = mainLayout;

			suspensionControl = new TuningSuspensionControl();
			suspensionControl.assetManager = assetManager;
			suspensionControl.setSize(150, 30);
			suspensionControl.text = "Suspension";
			addChild(suspensionControl);
			addChild(createSeparator());

			tintControl = new SliderControl(false);
			tintControl.assetManager = assetManager;
			tintControl.setSize(150, 30);
			tintControl.text = "Window Tint";
			addChild(tintControl);
			addChild(createSeparator());


			customBackgroundButton = ButtonFactory.tuningSettingsButton("a_background_button", "Backgrounds");
			customBackgroundButton.addEventListener(Event.TRIGGERED, onOpenCustomBackgroundHandler);
			customBackgroundButton.width = 60;
			customBackgroundButton.height = 50;

			effectMenuButton = ButtonFactory.tuningSettingsButton("a_front_light_effect", "Lights on", false);
			(effectMenuButton.defaultIcon as Image).color = 0xfefefe;
			(effectMenuButton.downIcon as Image).color = 0xfefefe;
			effectMenuButton.width = 60;
			effectMenuButton.height = 50;
			effectMenuButton.addEventListener(Event.TRIGGERED, onOpenEffectMenu);
			addChild(effectMenuButton);
			addChild(createSeparator());

			customBackgroundButton.labelFactory = LabelFactory.tuningSettingLabel;
			customBackgroundButton.label = "Backgrounds";
			addChild(customBackgroundButton);

			effectMenu = new EffectMenu();
			effectMenu.includeInLayout = false;
			effectMenu.addEventListener(EffectMenu.TOGGLE_STATE, onToggleState);
			effectMenu.addEventListener(EffectMenu.TOGGLE_STATE_ERROR, onErrorToggleState);
			effectMenu.visible = false;
			addChild(effectMenu);
			if(_car){
				effectMenu.car=_car;
			}
			//
			var assetsProxy : AssetsProxy = ApplicationFacade.getInstance().retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
			assetsProxy.loadComponentIcon("suspension", function():void{
				loadAndSetIcon();
			});

			assetsProxy.loadComponentIcon("windows", function():void{
				loadAndSetWindowIcon();
			});
		}

		private function loadAndSetIcon():Boolean{
			var iconTexture:Texture = ButtonFactory.assetManager.getTexture("suspension");
			if(iconTexture) {
				var suspensionImage:Image = new Image(iconTexture);
				var suspensionDownImage:Image = new Image(iconTexture);
				suspensionDownImage.alpha = 0.6;
				suspensionImage.scale = /*0.8 * */ButtonFactory.assetManager.scaleFactor * 40 / 120;
				suspensionDownImage.scale = /*0.8 * */ButtonFactory.assetManager.scaleFactor * 40 / 120;

				invalidate();
				return true;
			}
			return false;
		}

		private function loadAndSetWindowIcon():Boolean{
			var iconTexture:Texture = ButtonFactory.assetManager.getTexture("windows");
			if(iconTexture) {
				var windowImage:Image = new Image(iconTexture);
				var windowDownImage:Image = new Image(iconTexture);
				windowDownImage.alpha = 0.6;
				windowImage.scale = 0.8 * ButtonFactory.assetManager.scaleFactor * 40 / 120;
				windowDownImage.scale = 0.8 * ButtonFactory.assetManager.scaleFactor * 40 / 120;

				invalidate();
				return true;
			}
			return false;
		}

		protected function onOpenCustomBackgroundHandler(event:Event):void{
			SoundManager.playSound(SoundManager.PART_MENU);
			if (!_backgroundsIsDownloaded) {
				var userProxy: UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
				var carBackgroundsProxy: CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
				var carType:String;
				if(_car.isChallenge){
					carType = "NONE";
				}else{
					carType = _car.carType;
				}
				carBackgroundsProxy.loadBackgrounds(userProxy.currentUser ? userProxy.currentUser.id : null, true, carType, function (): void {
					_backgroundsIsDownloaded = true;
				});
			}
			dispatchEventWith(OPEN_BACKGROUND);
		}

		protected function onToggleState(event:Event):void{
			this.dispatchEventWith(EffectMenu.TOGGLE_STATE, false, event.data);
		}
		
		protected function onErrorToggleState(event:Event):void{
			this.dispatchEventWith(EffectMenu.TOGGLE_STATE_ERROR, false, event.data);
		}

		protected function onOpenEffectMenu(event:Event):void{
			if(!effectMenu.isPossibleDiskNeon() && !effectMenu.isPossibleNeon()){
				this.effectMenu.doToggleState("front_light");
				if(effectMenu.frontLightEnabled){
					(effectMenuButton.defaultIcon as Image).color = 0x0f86ff;
					(effectMenuButton.downIcon as Image).color = 0x0f86ff;
				}else{
					(effectMenuButton.defaultIcon as Image).color = 0xfefefe;
					(effectMenuButton.downIcon as Image).color = 0xfefefe;
				}
			}else{
                (effectMenuButton.defaultIcon as Image).color = 0xfefefe;
                (effectMenuButton.downIcon as Image).color = 0xfefefe;
				effectMenu.visible=true;
			}
		}

		public function createSeparator():Quad {
			var separatorTexture:Texture = ButtonFactory.assetManager.getTexture("a_seporator_1");
			var separator:Image=new Image(separatorTexture);
			separator.width=1.5;
			separator.height=34;
			return separator;
		}

		public function set car(value:Car):void{
			_car = value;
			tintControl.isEnabled = !_car.tintDisabled;
			if(effectMenu){
				effectMenu.car = _car;
				if(!effectMenu.isPossibleDiskNeon() && !effectMenu.isPossibleNeon()){
					if(effectMenu.frontLightEnabled){
						(effectMenuButton.defaultIcon as Image).color = 0x0f86ff;
						(effectMenuButton.downIcon as Image).color = 0x0f86ff;
					}else{
						(effectMenuButton.defaultIcon as Image).color = 0xfefefe;
						(effectMenuButton.downIcon as Image).color = 0xfefefe;
					}
				}
			}
			if(customBackgroundButton){
				if(_car && _car.isChallenge){
					customBackgroundButton.isEnabled = ObjectUtils.getValue(_challengeData.featuresRules, "background", true);
					customBackgroundButton.alpha = customBackgroundButton.isEnabled?1: 0.6;
				}
			}
		}
		
		override protected function draw() : void {
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(sizeInvalid) {
				var g:Number = (actualWidth - mainLayout.paddingLeft - mainLayout.paddingRight - 3) / (45 + 4);
				var a:Number = g * 15;
				var b:Number = (a - 2 * g - 1.5) / 2;
				effectMenuButton.width = b;
				customBackgroundButton.width = b;
				suspensionControl.setSize(a, suspensionControl.height);
				tintControl.setSize(a, tintControl.height);
				//
				mainLayout.gap = g; //(actualWidth - (effectMenuButton.width + customBackgroundButton.width + suspensionControl.width + tintControl.width + 4.5 + mainLayout.paddingLeft + mainLayout.paddingRight)) / 6;
				//
				quad.width = actualWidth;
				quad.height = actualHeight;
				
				effectMenu.x = 0;
				effectMenu.y = 0;
				effectMenu.width = actualWidth;
				effectMenu.height = actualHeight;
			}
			if(effectMenu){
				var effectsEnabled:Boolean = (effectMenu.isPossibleDiskNeon() || effectMenu.isPossibleFrontLight() || effectMenu.isPossibleRearLight() || effectMenu.isPossibleNeon() || effectMenu.isPossibleLight());
				if(effectsEnabled){
					effectMenuButton.alpha = 1;
					effectMenuButton.isEnabled = true;
				} else {
					effectMenuButton.alpha = 0.6;
					effectMenuButton.isEnabled = false;
				}
			}
		}

		public function set challengeData(value:ChallengeData):void {
			_challengeData = value;
		}

		public function get challengeData():ChallengeData {
			return _challengeData;
		}
	}
}