package com.despark.tuning3d.view.components.tunings.editing {
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.TextInputNewView;
import com.utils.FontSelector;

import flash.geom.Rectangle;
import flash.text.TextFormat;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class TunningNameInputPopUp extends PopUp implements Multilingual {
		public static var OK_BUTTON_SELECTED : String = "OkButtonSelected";
		public static var NAME_CHANGED : String = "NameChanged";
		
		public var tuning : Tuning;
		
		private var textInputView : TextInputNewView;
		private var okButton:MultilingualButton;
		private var cancelButton:MultilingualButton;
		private var _okButtonEnabled : Boolean;
		
		public function TunningNameInputPopUp() {
			super();
			setSizeInternal(Starling.current.stage.width, Starling.current.stage.height, false);
		}
		
		public function get okButtonEnabled():Boolean {
			return _okButtonEnabled;
		}

		public function set okButtonEnabled(value:Boolean):void {
			_okButtonEnabled = value;
			invalidate("okButtonEnabled");
		}

		override protected function initialize() : void {
			super.initialize();
			
			contentBackgroundImage.visible = false;
			this.closeButton.visible=false;
			
			textInputView = new TextInputNewView(false,false);
			textInputView.addEventListener(Event.CHANGE, textInputViewChangeHandler);
			if(tuning) {
				textInputView.text = tuning.name;
			}
			addChild(textInputView);
			
			var scale:Number=ButtonFactory.assetManager.scaleFactor/2;
			
			okButton = new MultilingualButton();//ButtonFactory.circleButton("a_blue_bg_button");
			okButton.textFormat = new TextFormat(FontSelector.getFont(false), 10, 0xffffff);
			
			var buttonTexture:Texture = ButtonFactory.assetManager.getTexture("a_blue_bg_button");
			
			var blueButtonSkin:Image = new Image(buttonTexture);
			blueButtonSkin.scale9Grid = new Rectangle(25.5/scale, 0, 25.5/scale, buttonTexture.frameHeight);
			blueButtonSkin.width = 67.5;
			blueButtonSkin.height = 25.5;
			okButton.defaultSkin=blueButtonSkin;
			okButton.addEventListener(Event.TRIGGERED, okButtonTriggeredHandler);
			okButton.label="Save";
			okButton.updateWithLanguage(AlertView.language);
			addChild(okButton);
			
			
			cancelButton = new MultilingualButton();
			cancelButton.textFormat = new TextFormat(FontSelector.getFont(false), 10, 0xffffff);
			
			buttonTexture = ButtonFactory.assetManager.getTexture("a_transparent_bg_button");
			//var textureTransparentButton:Scale3Textures=new Scale3Textures(ButtonFactory.assetManager.getTexture("a_transparent_bg_button"), 25.5/scale,25.5/scale, Scale3Textures.DIRECTION_HORIZONTAL);
			var blueTransparentSkin:Image = new Image(buttonTexture);
			
			blueTransparentSkin.scale9Grid = new Rectangle(25.5/scale, 0, 25.5/scale, buttonTexture.frameHeight);
			
			blueTransparentSkin.width = 67.5;
			blueTransparentSkin.height = 25.5;
			
			cancelButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
			cancelButton.defaultSkin=blueTransparentSkin;
			
			cancelButton.label = "Cancel";
			cancelButton.updateWithLanguage(AlertView.language);
			addChild(cancelButton);
			
			textInputView.fun_enterPress = function() : void {
				textInputView.textInput.visible = false;
				textInputView.textInput.visible = true;
				okButtonTriggeredHandler(null);
			}
		}
		
		protected function cancelButtonTriggeredHandler(event:Event):void{
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			dispatchEventWith(CLOSE_BUTTON_SELECTED);
		}
		
		public function updateWithLanguage(language : Language) : void {
			textInputView.title = "";//language.translate("mob_alertsAndOverlays", "tuningName").toUpperCase();
			textInputView.updateWithLanguage(language);
			
			titleLabel.text = language.translate("mob_alertsAndOverlays", "Save on 3DTuning");
			titleLabel.updateWithLanguage(language);
		}
		
		public function get tuningName() : String {
			return textInputView.text;
		}
		
		override protected function draw() : void {
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				textInputView.y = 243/2;
				textInputView.width = 469/2;
				
				textInputView.x = (this.actualWidth-textInputView.width)/2;

				cancelButton.x = (this.actualWidth-((87.5*2)+19))/2;
				cancelButton.width = 87.5;
				cancelButton.y = 380/2;
				
				okButton.x =  cancelButton.x+cancelButton.width+19;
				okButton.width = 87.5;
				okButton.y = 380/2;
			}
			
			if(isInvalid("okButtonEnabled")) {
				okButton.alpha = okButtonEnabled ? 1.0 : 0.5;
				okButton.touchable = okButtonEnabled;
			}
		}
		
		public function setOKButtonEnabled(value : Boolean) : void {
			okButton.alpha = value ? 1.0 : 0.5;
			okButton.touchable = value;
		}
		
		private function okButtonTriggeredHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(OK_BUTTON_SELECTED);
		}
		
		private function textInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(NAME_CHANGED);
		}
	}
}