package com.despark.tuning3d.view.components.tunings.editing {
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.NumericStepper;
    import feathers.controls.TextInput;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalAlign;

    import starling.display.Image;
    import starling.display.Quad;

    public class ValueStepper extends NumericStepper {
        private var decrementIcon:String;
        private var incrementIcon:String;
        private var _prevValue:Number;
        private var _data:Array;
        private var _titleField:Label;
        private var borderImage:Image;
        public function ValueStepper() {
            super();

            textInputFactory = function():TextInput {
                var input:TextInput = new TextInput();
                input.textEditorFactory = LabelFactory.valueStepperLabel;
                input.verticalAlign = VerticalAlign.MIDDLE;
                input.isEditable = false;
                input.isSelectable = false;
                input.paddingLeft = 2;
                input.paddingTop = -1;
                input.paddingBottom = 1;
                input.touchable = false;
                return input;
            }

            decrementButtonFactory = function():Button {
                var button:Button;
                if(decrementIcon){
                    button = createButton(decrementIcon, actualWidth / 2, HorizontalAlign.LEFT);
                }else{
                    button = createButton(null, actualWidth / 2, HorizontalAlign.LEFT);
                }
                return button;
            }

            incrementButtonFactory = function():Button {
                var button:Button;
                if(incrementIcon){
                    button = createButton(incrementIcon, actualWidth / 2, HorizontalAlign.RIGHT);
                }else{
                    button = createButton(null, actualWidth / 2, HorizontalAlign.RIGHT);
                }
                return button;
            }

            valueFormatFunction = function(value:Number):String{
                if(_data && _data.length){
                    return _data[value].title;
                }
                return "EMPTY";
            }
        }

        public function set title(str:String):void{
            if(_titleField == null){
                _titleField = new Label();
                _titleField.width = actualWidth;
                _titleField.textRendererFactory = LabelFactory.valueStepperTitle;
                addChild(_titleField);
            }
            _titleField.text = str;
            _titleField.y = -_titleField.height - 10;
            _titleField.includeInLayout = false;
        }

        private static function createButton(textureId:String = null, width:Number = 0, iconAlign:String = null):Button{
            var button:Button = new Button();
            if(textureId){
                var defaultSkin:Quad = new Quad(1,1,0xFF0000);
                defaultSkin.alpha = 0;

                var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
                defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
                var downIcon:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
                downIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
                downIcon.alpha = 0.2;
                button.defaultIcon = defaultIcon;
                button.downIcon = button.disabledIcon = downIcon;
                button.defaultSkin = defaultSkin;
                button.setSize(width, 30);
                if(iconAlign == HorizontalAlign.LEFT){
                    button.iconOffsetX = -(width - 30) / 2;
                }else if(iconAlign == HorizontalAlign.RIGHT){
                    button.iconOffsetX = (width - 30) / 2;
                }
            }
            return button;
        }

        override protected function initialize():void{
            super .initialize();
            createTextInput();
            textInputGap = -10;
            createDecrementButton();
            createIncrementButton();
            minimum = 0;
            step = 1;
        }

        public function set border(val:Boolean):void{
            if(val){
                if(!borderImage){
                    borderImage = new Image(ButtonFactory.assetManager.getTexture("stepper_border"));
                    borderImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
                    addChildAt(borderImage, 0);
                }
            }else{
                if(borderImage){
                    borderImage.removeFromParent(true);
                    borderImage = null;
                }
            }
        }

        public function setIcons(decrementIcon:String, incrementIcon:String):void{
            this.decrementIcon = decrementIcon;
            this.incrementIcon = incrementIcon;
        }

        override protected function draw():void{
            super .draw();
            if(incrementButton){
                incrementButton.isEnabled = incrementButton.touchable = !(value == maximum);
            }
            if(decrementButton){
                decrementButton.isEnabled = decrementButton.touchable = !(value == minimum);
            }
            if(textInput){
                if(incrementButton && decrementButton){
                    textInput.width = actualWidth - (incrementButton.defaultIcon.width + decrementButton.defaultIcon.width) / 1.5;
                }
                textInput.x = (actualWidth - textInput.width) / 2;
                if(getChildAt(0) != textInput){
                    setChildIndex(textInput, 0);
                }
            }
            if(incrementButton && decrementButton && (incrementButton.isEnabled || decrementButton.isEnabled)){
                textInput.alpha = 1;
                textInput.touchable = false;
            }else{
                textInput.alpha = 0.2;
                textInput.touchable = false;
            }

            if(borderImage){
                borderImage.x = (actualWidth - borderImage.width) / 2;
                borderImage.y = (actualHeight - borderImage.height) / 2;
            }
        }

        override protected function increment():void{
            if(_prevValue != value){
                _prevValue = value;
            }
            super.increment();
        }

        override protected function decrement():void{
            if(_prevValue != value){
                _prevValue = value;
            }
            super.decrement();
        }

        override protected function refreshDisplayedText():void{
            try{
                super.refreshDisplayedText();
            }catch (e:Error) {

            }
        }

        public function reset():void{
            touchPointID = -1;
        }

        public function set data(data:Array):void {
            _data = data;
            if(_data && _data.length){
                maximum = _data.length - 1;
                invalidate();
            }
        }

        public function get data():Array {
            return _data;
        }

        public function getItemAt(index:Number):*{
            if(_data && _data[index]){
                return _data[index];
            }
            return null;
        }

        public function get selectedItem():*{
            if(_data && _data[value]){
                return _data[value];
            }
            return null;
        }

        public function get prevValue():Number {
            return _prevValue;
        }
    }
}
