package com.despark.tuning3d.view.core
{
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.assets.AssetsProxy;
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.BrandsScreenMediator;
    import com.despark.tuning3d.view.HomeScreenPopUpMediator;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.modals.InAppNotificationsPopup;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.StackedBannerScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.mailbox.MailsScreen;
    import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.despark.tuning3d.view.components.tunings.TuningsListScreen;
    import com.utils.ArrayUtil;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;

    public class StackedScreenExtendedMediator extends StackedScreenMediator {
        public static const AUTH_ACTION_GARAGE : String = "garage";
        public static const AUTH_ACTION_TUNING : String = "tuning";
        public static const AUTH_ACTION_TIMELINE : String = "timeline";
        public static const AUTH_ACTION_NOTIFICATIONS : String = "notifications";
        public static const AUTH_ACTION_IN_APP_NOTIFICATIONS : String = "InAppNotifications";
        public static const AUTH_ACTION_CHALLENGE_VOTE : String = "challengeVoting";
        public static const AUTH_ACTION_NO_COIN_POPUP : String = "noCoinPopup";
        public static const AUTH_REWARD_INFO_POPUP : String = "rewardsInfoPopup";
        public static const AUTH_ACTION_CHALLENGE_GARAGE : String = "challengeGarage";
        public static const AUTH_ACTION_CHALLENGE_NOTIFICATIONS : String = "challengeNotifications";
        public static const AUTH_ACTION_FRIENDS : String = "friends";
        public static const AUTH_ACTION_LOGIN : String = "login";
        public static const AUTH_ACTION_LOGOUT : String = "logout";
        public static const AUTH_ACTION_RESTORE : String = "restore";
        public static const AUTH_ACTION_BUY_PACKET : String = "by_packet";


        public static const AUTH_ACTION_MAILS : String = "mails";
        public static const AUTH_ACTION_CHALLENGE_REQUEST : String = "challenge_request";
        public static const AUTH_ACTION_MAIL_BOX : String = "mail_box";
        private static var _SCREEN_DISPLAY_DELAY:Number = 0.2;
		public function StackedScreenExtendedMediator(stackedScreen : StackedScreen) {
			super(stackedScreen);
			stackedScreen.assetManager = (facade.retrieveProxy(AssetsProxy.NAME) as AssetsProxy).assetManager;
		}
		
		override public function onRegister() : void {
			super.onRegister();
            /**
             * Screens and popups requiring authentication
             */
            stackedScreen.addEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.USER_GARAGE_SELECTED, userGarageSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.FRIENDS_SELECTED, friendsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.TIMELINE_SELECTED, timelineSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.NOTIFICATION_SELECTED, notificationsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.IN_APP_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
            /**
             * Screens and popups not requiring authentication
             */
            stackedScreen.addEventListener(StackedScreen.HOME_SELECTED, homeSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.VOTE_SELECTED, votesSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            stackedScreen.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
            stackedScreen.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
            stackedScreen.addEventListener(StackedBannerScreen.CHALLENGES_HOME_SELECTED, challengesHomeSelectHandler);

            /**
             * Login/logout
             */
            stackedScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
            stackedScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, logoutSelectedHandler);
		}

        override public function onRemove() : void {
            super.onRemove();
            /**
             * Screens and popups requiring authentication
             */
            stackedScreen.removeEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.USER_GARAGE_SELECTED, userGarageSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.FRIENDS_SELECTED, friendsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.TIMELINE_SELECTED, timelineSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.NOTIFICATION_SELECTED, notificationsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.IN_APP_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);

            /**
             * Screens and popups not requiring authentication
             */
            stackedScreen.removeEventListener(StackedScreen.HOME_SELECTED, homeSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.VOTE_SELECTED, votesSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            stackedScreen.removeEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
            stackedScreen.removeEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
            stackedScreen.removeEventListener(StackedBannerScreen.CHALLENGES_HOME_SELECTED, challengesHomeSelectHandler);

            /**
             * Login/logout
             */
            stackedScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
            stackedScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, logoutSelectedHandler);
        }

        protected function userAuthenticate(authAction : String):void {
            pendingForAuthAction = authAction;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function friendsSelectedHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_FRIENDS);
        }

        private function timelineSelectedHandler( e : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_TIMELINE);
        }

        private function userGarageSelectedHandler(event : Event): void {
            AppMode.self.setModeTuning();
            if(Constants.selectedUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
                pendingForAuthAction = null;
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
                userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_GARAGE);
            }
        }

        private function notificationsSelectedHandler(event : Event) : void{
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_NOTIFICATIONS);
        }

        private function challengeNotesSelectHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_IN_APP_NOTIFICATIONS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function mailSelectedHandler(event:Event):void {
            pendingForAuthAction = AUTH_ACTION_MAILS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        /**
         * Screens and popups not requiring authentication
         */
        protected function homeSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        private function brandsSelectedHandler(event:Event):void {
            if(HomeScreenPopUpMediator.isBrandScreenFirstOpen) {
                sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
            } else {
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
            HomeScreenPopUpMediator.isBrandScreenFirstOpen = false;
        }

        private function garageSelectedHandler(event : Event): void {
            Constants.selectedUser = null;
            _SCREEN_DISPLAY_DELAY = 0.2;
            pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_GARAGE;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function votesSelectedHandler(event:Event):void {
            AppMode.self.setModeTuning();
            sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
        }

        private function gallerySelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
        }

        private function challengeSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
        }

        private function leaderBoardSelectHandler(event : Event) : void {
            BannerStackedScreen.hideAdBanner(false);
        }

        private function challengesHomeSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN, event.data);
        }

        /**
         * Login/Logout
         */
        private function loginSelectedHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_LOGIN;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function logoutSelectedHandler(e:Event):void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.logoutCurrentUser();
            AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);
            if(stackedScreen.hasOwnProperty("updateNotifications")){
                stackedScreen["updateNotifications"]();
            }
            stackedScreen.ShowDone();
            if(stackedScreen is MailsScreen || stackedScreen is TimelineScreen){
                homeSelectedHandler(null);
            }
        }


        override public function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_AUTHENTICATED,
                Notifications.REWARD_ADS_DISMISSED,
                Notifications.INTERSTITIAL_ADS_DISMISSED,
                Notifications.REWARD_ARS_OPENED,
                Notifications.INTERSTITIAL_ADS_OPENED,
                Notifications.APPLICATION_BECAME_ACTIVE,
                Notifications.APPLICATION_WENT_BACKGROUND
            ])
        }

        override public function handleNotification(note:INotification):void {
            super.handleNotification(note);
			switch (note.getName()){
                case Notifications.USER_AUTHENTICATED:
                    handleUserAuthenticated();
                    break;
                case Notifications.REWARD_ARS_OPENED:
                case Notifications.INTERSTITIAL_ADS_OPENED:
                case Notifications.APPLICATION_WENT_BACKGROUND:
                    SoundManager.pauseChannel(SoundManager.THEME);
                    break;
                case Notifications.REWARD_ADS_DISMISSED:
                case Notifications.INTERSTITIAL_ADS_DISMISSED:
                case Notifications.APPLICATION_BECAME_ACTIVE:
                    SoundManager.resumeChannel(SoundManager.THEME);
                    break;
			}
        }

        private function handleUserAuthenticated():void{
            if(!stackedScreen || !stackedScreen.stage){
                return;
            }
            var popupData:Object = {showBannerAfterExit: true, isWideScreen:Constants.isWideScreen, isUltraWideScreen: Constants.isUltraWideScreen};
            if(stackedScreen is BrandsScreen){
                popupData.bannerScreen = BannerScreen.HOME_SCREEN;
                BrandsScreen(stackedScreen).locateAdBanner();
            }
            switch(pendingForAuthAction){
                case AUTH_ACTION_MAILS:
                    if(!(stackedScreen is MailsScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_MAIL_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                    break;
                case StackedScreenExtendedMediator.AUTH_ACTION_FRIENDS:
                    Starling.juggler.delayCall(function() : void{
                        BannerStackedScreen.hideAdBanner(false);
                        sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, popupData);
                    }, _SCREEN_DISPLAY_DELAY);
                    break;
                case StackedScreenExtendedMediator.AUTH_ACTION_GARAGE:
                    if(!(stackedScreen is TuningsListScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
                case StackedScreenExtendedMediator.AUTH_ACTION_TIMELINE:
                    if(!(stackedScreen is TimelineScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
                case StackedScreenExtendedMediator.AUTH_ACTION_IN_APP_NOTIFICATIONS:
                    if(!(stackedScreen is InAppNotificationsPopup)){
                        Starling.juggler.delayCall(function() : void{
                            BannerStackedScreen.hideAdBanner(false);
                            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP, popupData);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
            }

        }
	}
}