package com.distriqt {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.distriqt.extension.inappbilling.BillingService;
    import com.distriqt.extension.inappbilling.ErrorCodes;
    import com.distriqt.extension.inappbilling.InAppBilling;
    import com.distriqt.extension.inappbilling.InAppBillingServiceTypes;
    import com.distriqt.extension.inappbilling.Purchase;
    import com.distriqt.extension.inappbilling.PurchaseRequest;
    import com.distriqt.extension.inappbilling.events.InAppBillingEvent;
    import com.distriqt.extension.inappbilling.events.PurchaseEvent;

    public class BillingManager {
        private static var self : BillingManager;
        private var billingServiceIsSetup: Boolean = false;
        private var billingServiceIsSetupCommitted: Boolean = false;
        private var getProductsSuccessCallback: Function;
        private var getProductsFailureCallback: Function;
        private var makePurchaseSuccessCallback: Function;
        private var makePurchaseFailureCallback: Function;

        private var restorePurchaseUpdateCallback: Function;
        private var restorePurchaseSuccessCallback: Function;
        private var restorePurchaseFailedCallback: Function;

        private var productsId: Array;
        public var allPackets: Array;

        public var packetForRestores:Array;
        public static function get $() : BillingManager{
            if(!self){
                self = new BillingManager();
            }

            return self;
        }

        public function BillingManager() {
            if(self){
                throw new Error("Billing Manager is singleton");
            }
        }

        public function setup(productsId: Array, successCallback: Function, FailureCallback: Function): void {
            if(InAppBilling.isSupported) {
                this.productsId = productsId;
                getProductsSuccessCallback = successCallback;
                getProductsFailureCallback = FailureCallback;
                if (!billingServiceIsSetupCommitted) {
                    var service:BillingService;
                    if (Device.isIos()) {
                        service = new BillingService();
                    } else {
                        service = new BillingService()
                                .setServiceType(InAppBillingServiceTypes.GOOGLE_PLAY_INAPP_BILLING)
                                .setGooglePlayPublicKey(Constants.GOOGLE_BILLING_KEY);
                    }
                    InAppBilling.service.setup(service);
                    InAppBilling.service.addEventListener(InAppBillingEvent.SETUP_SUCCESS, setup_successHandler);
                    InAppBilling.service.addEventListener(InAppBillingEvent.SETUP_FAILURE, setup_failureHandler);
                    billingServiceIsSetupCommitted = true;
                } else if (billingServiceIsSetup) {
                    getProducts();
                }
            }
        }

        public function makePurchase(productId: String, successCallback: Function, failureCallback: Function): void {
            makePurchaseSuccessCallback = successCallback;
            makePurchaseFailureCallback = failureCallback;

            InAppBilling.service.addEventListener( PurchaseEvent.PURCHASES_UPDATED, purchases_updatedHandler );
            InAppBilling.service.addEventListener( PurchaseEvent.PURCHASE_FAILED, purchase_failedHandler );
            InAppBilling.service.addEventListener( InAppBillingEvent.FINISH_SUCCESS, finishPurchase_successHandler );
            InAppBilling.service.addEventListener( InAppBillingEvent.FINISH_FAILED, finishPurchase_failedHandler );

            var request:PurchaseRequest = new PurchaseRequest();
            request.productId = productId;
            request.quantity = 1;
            InAppBilling.service.makePurchase( request );
        }

        public function restorePurchases(purchaseUpdateCallback:Function, successCallback:Function, failedCallback:Function):void{
            restorePurchaseUpdateCallback = purchaseUpdateCallback;
            restorePurchaseSuccessCallback = successCallback;
            restorePurchaseFailedCallback = failedCallback;
            removeRestoreListeners();
            addRestoreListeners();
            InAppBilling.service.restorePurchases();
        }

        private function restorePurchases_successHandler( event:InAppBillingEvent ):void {
            removeRestoreListeners();
            if(restorePurchaseSuccessCallback!=null){
                restorePurchaseSuccessCallback();
            }
        }

        private function restorePurchases_failedHandler( event:InAppBillingEvent ):void {
            removeRestoreListeners();
            if(restorePurchaseFailedCallback!=null){
                restorePurchaseFailedCallback(event.message);
            }
        }

        internal function restore_purchases_updatedHandler(event:PurchaseEvent ):void {
            for each (var purchase:Purchase in event.data) {
                switch (purchase.transactionState) {
                    case Purchase.STATE_RESTORED:
                        addPurchaseToInventory( purchase.originalPurchase );
                        InAppBilling.service.finishPurchase( purchase );
                        break;
                }
            }
        }

        private function addPurchaseToInventory(purchase: Purchase):void{
            var receipt:String="";
            if(InAppBilling.service.applicationReceipt.isSupported){
                receipt = InAppBilling.service.applicationReceipt.getAppReceipt();
            }else{
                receipt = purchase.transactionReceipt;
            }
            var data:Object={
                receipt:receipt,
                transactionId:purchase.transactionId,
                productId:purchase.productId
            };
            packetForRestores.push({productId:purchase.productId, data: JSON.stringify(data), transactionId:purchase.transactionId });
            if(restorePurchaseUpdateCallback != null){
                restorePurchaseUpdateCallback(packetForRestores[packetForRestores.length - 1]);
            }
        }

        private function addRestoreListeners():void{
            InAppBilling.service.addEventListener( PurchaseEvent.PURCHASES_UPDATED,	restore_purchases_updatedHandler );
            InAppBilling.service.addEventListener( InAppBillingEvent.RESTORE_PURCHASES_SUCCESS, restorePurchases_successHandler );
            InAppBilling.service.addEventListener( InAppBillingEvent.RESTORE_PURCHASES_FAILED, restorePurchases_failedHandler );
        }

        private function removeRestoreListeners():void{
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASES_UPDATED,	restore_purchases_updatedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.RESTORE_PURCHASES_SUCCESS, restorePurchases_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.RESTORE_PURCHASES_FAILED, restorePurchases_failedHandler );
        }

        public function getProducts(): void {
            if(!InAppBilling.service.hasEventListener(InAppBillingEvent.PRODUCTS_LOADED)){
                InAppBilling.service.addEventListener(InAppBillingEvent.PRODUCTS_LOADED, products_loadedHandler );
            }
            if(!InAppBilling.service.hasEventListener(InAppBillingEvent.PRODUCTS_FAILED)){
                InAppBilling.service.addEventListener(InAppBillingEvent.PRODUCTS_FAILED, products_failedHandler );
            }
            if(!InAppBilling.service.hasEventListener(InAppBillingEvent.INVALID_PRODUCT)){
                InAppBilling.service.addEventListener(InAppBillingEvent.INVALID_PRODUCT, products_failedHandler );
            }
            InAppBilling.service.getProducts(productsId );
        }

        private function purchases_updatedHandler( event: PurchaseEvent ):void {
            for each (var purchase: Purchase in event.data) {
                switch (purchase.transactionState) {
                    case Purchase.STATE_PURCHASING:
                    case Purchase.STATE_DEFERRED:
                        break;
                    case Purchase.STATE_PURCHASED: {
                        if (makePurchaseSuccessCallback) {
                            makePurchaseSuccessCallback(purchase);
                        }
                        InAppBilling.service.finishPurchase( purchase );

                        break;
                    }
                    case Purchase.STATE_FAILED:
                    case Purchase.STATE_REFUNDED:
                    case Purchase.STATE_REMOVED:
                    case Purchase.STATE_CANCELLED:
                    case Purchase.STATE_NOTALLOWED:
                    case Purchase.STATE_RESTORED:
                        InAppBilling.service.finishPurchase( purchase );
                        if (purchase.transactionState == Purchase.STATE_RESTORED) {
                            if (makePurchaseSuccessCallback) {
                                makePurchaseSuccessCallback(purchase)
                            }
                        } else {
                            if (makePurchaseFailureCallback) {
                                makePurchaseFailureCallback(null);
                            }
                        }

                        break;
                }
            }
        }

        private function purchase_failedHandler( event:PurchaseEvent ):void {
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASES_UPDATED, purchases_updatedHandler );
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASE_FAILED, purchase_failedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_SUCCESS, finishPurchase_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_FAILED, finishPurchase_failedHandler );
            if (event.data && event.data.length > 0) {
                InAppBilling.service.finishPurchase( event.data[0] );
            }
            if (event.errorCode == ErrorCodes.ITEM_ALREADY_OWNED) {
                InAppBilling.service.getPurchases();
            }

            if (makePurchaseFailureCallback) {
                makePurchaseFailureCallback(event)
            }
        }

        private function finishPurchase_successHandler( event:InAppBillingEvent ):void {
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASES_UPDATED, purchases_updatedHandler );
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASE_FAILED, purchase_failedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_SUCCESS, finishPurchase_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_FAILED, finishPurchase_failedHandler );
            trace( "finish purchase complete" );

        }

        private function finishPurchase_failedHandler( event:InAppBillingEvent ):void {
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASES_UPDATED, purchases_updatedHandler );
            InAppBilling.service.removeEventListener( PurchaseEvent.PURCHASE_FAILED, purchase_failedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_SUCCESS, finishPurchase_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.FINISH_FAILED, finishPurchase_failedHandler );
            trace( "finish failed: ["+event.errorCode+"]" + event.message );
            if (makePurchaseFailureCallback) {
                makePurchaseFailureCallback(event)
            }
        }

        private function products_loadedHandler(e: InAppBillingEvent): void {
            InAppBilling.service.removeEventListener( InAppBillingEvent.PRODUCTS_LOADED, products_loadedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.PRODUCTS_FAILED, products_failedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.INVALID_PRODUCT, products_failedHandler );
            if (getProductsSuccessCallback) {
                getProductsSuccessCallback(e);
            }
        }

        private function products_failedHandler(e: InAppBillingEvent): void {
            InAppBilling.service.removeEventListener( InAppBillingEvent.PRODUCTS_LOADED, products_loadedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.PRODUCTS_FAILED, products_failedHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.INVALID_PRODUCT, products_failedHandler );
            if (getProductsFailureCallback) {
                getProductsFailureCallback(e);
            }
        }

        private function setup_successHandler(e: InAppBillingEvent): void {
            InAppBilling.service.removeEventListener( InAppBillingEvent.SETUP_SUCCESS, setup_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.SETUP_FAILURE, setup_failureHandler );
            billingServiceIsSetup = true;
            getProducts();
        }

        private function setup_failureHandler(e: InAppBillingEvent): void {
            InAppBilling.service.removeEventListener( InAppBillingEvent.SETUP_SUCCESS, setup_successHandler );
            InAppBilling.service.removeEventListener( InAppBillingEvent.SETUP_FAILURE, setup_failureHandler );
            billingServiceIsSetup = true;
        }
    }
}
