package com.distriqt {
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.distriqt.extension.facebook.core.Facebook;
    import com.distriqt.extension.facebook.login.AccessToken;
    import com.distriqt.extension.facebook.login.FacebookLogin;
    import com.distriqt.extension.facebook.login.Profile;
    import com.distriqt.extension.facebook.login.events.FacebookLoginErrorEvent;
    import com.distriqt.extension.facebook.login.events.FacebookLoginEvent;
    import com.distriqt.extension.facebook.login.events.FacebookProfileEvent;
    import com.distriqt.extension.facebook.share.FacebookShare;
    import com.distriqt.extension.facebook.share.events.ShareDialogEvent;
    import com.distriqt.extension.facebook.share.model.ShareLinkContentBuilder;
    import com.utils.Console;

    public class FacebookManager {

    private static var self : FacebookManager;

    private var _tracer : Tracer;
    private var _isInitialized : Boolean;
    private var _loginCallback:Function;
    private var _profileObj:Object;
    private var _shareUrl:String;

    public static function get $() : FacebookManager{
        if(!self) {
            self = new FacebookManager();
        }
        return self;
    }

    public function FacebookManager() {
        if(self) {
            throw new Error("Facebook manager is singleton, do not attempt to create instance");
        }
        _tracer = new Tracer("Facebook");

        try {
            DistriqtController.init();
            if (Facebook.isSupported) {
                _tracer.print("IS SUPPORTED"); // Constants.FACEBOOK_APP_ID
                if(Facebook.instance.initialise()) {
                    _isInitialized = true;
                    _tracer.print("Is initialized", Facebook.VERSION);
                    FacebookLogin.instance.addEventListener(FacebookLoginEvent.SUCCESS, loginRequestHandler );
                    FacebookLogin.instance.addEventListener(FacebookLoginEvent.CANCEL, loginRequestHandler );
                    FacebookLogin.instance.addEventListener(FacebookLoginErrorEvent.ERROR, loginRequestErrorHandler );
                    FacebookLogin.instance.addEventListener(FacebookProfileEvent.CHANGED, profileChangedHandler );
                } else {
                    _isInitialized = false;
                    _tracer.print("Not initialized");
                }
            } else {
                _isInitialized = false;
                _tracer.print("Not supported")
            }

        } catch (e:Error){
            _tracer.print("init error", e.getStackTrace());
            _isInitialized = false;
        }

    }

    internal function loginRequestHandler(event:FacebookLoginEvent):void {
        switch (event.type) {
            case FacebookLoginEvent.SUCCESS:
                    var profile:Profile = event.profile;
                    if(profile == null){
                        profile = FacebookLogin.instance.getProfile();
                    }
                if(_loginCallback && profile && event.accessToken) {
                    var profileObj : Object = {};
                    profileObj.typeOfSN = UsersProxy.typeOfSN_FB;
                    profileObj.user_id = profile.id;
                    profileObj.access_token = event.accessToken.token;
                    profileObj.oauth_token = "";
                    profileObj.oauth_token_secret = "";
                    profileObj.oauth_verifier = "";

                    _profileObj = profileObj;
                    _loginCallback(profileObj);
                    _loginCallback = null;
                }

                _tracer.print('Permission completed');
                _tracer.print("User ID:         " + event.accessToken.userId );
                _tracer.print("Expiration date: " + event.accessToken.expirationTimestamp );
                _tracer.print("Permissions:     " + event.accessToken.permissions.join(", "));
                _tracer.print("Access token:    " + event.accessToken.token);
                break;

            case FacebookLoginEvent.CANCEL:
                // User cancelled the request
                break;
        }
    }

    private function profileChangedHandler(event:FacebookProfileEvent):void{
        var profile:Profile = FacebookLogin.instance.getProfile();
        if(profile){
            var accessToken:AccessToken = FacebookLogin.instance.getAccessToken();
            if(_loginCallback && profile && accessToken) {
                var profileObj : Object = {};
                profileObj.typeOfSN = UsersProxy.typeOfSN_FB;
                profileObj.user_id = profile.id;
                profileObj.access_token = accessToken.token;
                profileObj.oauth_token = "";
                profileObj.oauth_token_secret = "";
                profileObj.oauth_verifier = "";

                _profileObj = profileObj;
                _loginCallback(profileObj);
                _loginCallback = null;
            }
        }
    }

    internal function loginRequestErrorHandler(event:FacebookLoginErrorEvent):void {
        Console.log("loginRequestErrorHandler", event.errorID);
        Console.log("loginRequestErrorHandler", event.text);
    }


    public function isLoggedIn() : Boolean {/*FacebookLogin.isSupported && _isInitialized && *//* && _profileObj*/
        return FacebookLogin.instance.isLoggedIn();
    }

    private function shareDialogEventHandler(event:ShareDialogEvent):void {
        FacebookShare.instance.shareDialog.removeEventListener(ShareDialogEvent.COMPLETE, shareDialogEventHandler );
        FacebookShare.instance.shareDialog.removeEventListener(ShareDialogEvent.CANCEL, shareDialogEventHandler );
        FacebookShare.instance.shareDialog.removeEventListener(ShareDialogEvent.ERROR, shareDialogEventHandler );

        _tracer.print('share: ' + event.type);
        if (event.postId != null) _tracer.print( "postId="+event.postId);
    }

    public function shareURL(url : String) : void {
        _tracer.print(url);
        _shareUrl = url;

        if (FacebookShare.instance.shareDialog.canShow(ShareLinkContentBuilder.TYPE)) {
            FacebookShare.instance.shareDialog.addEventListener( ShareDialogEvent.COMPLETE, shareDialogEventHandler );
            FacebookShare.instance.shareDialog.addEventListener( ShareDialogEvent.CANCEL, shareDialogEventHandler );
            FacebookShare.instance.shareDialog.addEventListener( ShareDialogEvent.ERROR, shareDialogEventHandler );

            var builder:ShareLinkContentBuilder = new ShareLinkContentBuilder();
           // builder.setQuote("3DTuning.com");
            builder.setQuote("Share your own customized cars");
            builder.setContentUrl(_shareUrl);

            var success:Boolean = FacebookShare.instance.shareDialog.show(builder.build());
            if (success) {
                _tracer.print("Share show succeeded")
            }
        } else {
            _tracer.print("Can not share");
        }
    }

    public function login(callback : Function) : void{
        if (FacebookLogin.isSupported && _isInitialized) {
            if(isLoggedIn() && _profileObj) {
                callback(_profileObj);
            } else {
                logOut();
            }
            _loginCallback = callback;
            var permissions:Array = ["public_profile", "email"];
            FacebookLogin.instance.logInWithReadPermissions( permissions);
            // FacebookAPI.service.createSession( permissions, readOnly );
        } else {
            _tracer.print( "Can not start session, isSupported:", Facebook.isSupported);
            AlertView.show("Facebook login is not supported on your device.");
        }
    }
    public function logOut() : void {
        if(isLoggedIn()) {
            FacebookLogin.instance.logout();
            _profileObj = null;
        }
    }
}
}
