package com.despark.tuning3d.view {
    import by.blooddy.crypto.image.JPEGEncoder;
    import by.blooddy.crypto.serialization.JSONer;

    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.assets.AssetsProxy;
    import com.despark.tuning3d.model.brands.BrandsProxy;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarBackground;
    import com.despark.tuning3d.model.cars.CarBackgroundsParser;
    import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
    import com.despark.tuning3d.model.cars.CarsProxy;
    import com.despark.tuning3d.model.challenges.ChallengeAlertEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.ConnectionProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.ErrorTracker;
    import com.despark.tuning3d.model.core.PromoVideoManager;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.core.http.HttpClient;
    import com.despark.tuning3d.model.core.images.BitmapDataUtil;
    import com.despark.tuning3d.model.core.images.ImageWatermarker;
    import com.despark.tuning3d.model.disks.DiskBrandsProxy;
    import com.despark.tuning3d.model.friend.FriendProxy;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.LanguagesProxy;
    import com.despark.tuning3d.model.packages.PackagesProxy;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.model.packages.Purchase;
    import com.despark.tuning3d.model.parts.Part;
    import com.despark.tuning3d.model.parts.PartColor;
    import com.despark.tuning3d.model.parts.PartDisk;
    import com.despark.tuning3d.model.parts.PartDisk;
    import com.despark.tuning3d.model.parts.PartMaterial;
    import com.despark.tuning3d.model.parts.PartMaterialImage;
    import com.despark.tuning3d.model.parts.PartMaterialImagePool;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.model.parts.PartSlotDisk;
    import com.despark.tuning3d.model.settings.Settings;
    import com.despark.tuning3d.model.settings.SettingsProxy;
    import com.despark.tuning3d.model.sharing.SharingInstagramProxy;
    import com.despark.tuning3d.model.sharing.SharingOkProxy;
    import com.despark.tuning3d.model.sharing.SharingPinProxy;
    import com.despark.tuning3d.model.sharing.SharingProxy;
    import com.despark.tuning3d.model.sharing.SharingVkProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.auth.LoginScreen;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.cars.CarView;
    import com.despark.tuning3d.view.components.cars.SnapShotFramesContainer;
    import com.despark.tuning3d.view.components.cars.SnapShotFramesContainer;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeAlertView;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.MainMenuItem;
    import com.despark.tuning3d.view.components.core.PackagesView;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.core.StackedScreenNavigator;
    import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
    import com.despark.tuning3d.view.components.instructions.TutorialView;
    import com.despark.tuning3d.view.components.partinfo.DetailInfoView;
    import com.despark.tuning3d.view.components.partinfo.HTMLCacher;
    import com.despark.tuning3d.view.components.parts.IPartItemRenderer;
    import com.despark.tuning3d.view.components.parts.PartDiskItemRenderer;
    import com.despark.tuning3d.view.components.parts.PartItemRenderer;
    import com.despark.tuning3d.view.components.tunings.colors.TuningColorPicker;
    import com.despark.tuning3d.view.components.tunings.colors.TuningColorsView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningDiskSettingsView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningSelectedColorView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningSelectedStateView;
    import com.despark.tuning3d.view.components.tunings.editing.TuningOffsetControl;
    import com.despark.tuning3d.view.components.tunings.editing.TuningScreen;
    import com.despark.tuning3d.view.components.tunings.editing.TuningSuspensionControl;
    import com.despark.tuning3d.view.components.tunings.editing.TuningVideoPopup;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.despark.tuning3d.view.core.StackedScreenMediator;
    import com.despark.tuning3d.view.core.StackedScreenMediator;
    import com.distriqt.AdsManager;
    import com.distriqt.BillingManager;
    import com.distriqt.GameService;
    import com.distriqt.extension.camerarollextended.Asset;
    import com.distriqt.extension.camerarollextended.AuthorisationStatus;
    import com.distriqt.extension.camerarollextended.CameraRollExtended;
    import com.distriqt.extension.camerarollextended.events.AuthorisationEvent;
    import com.distriqt.extension.camerarollextended.events.CameraRollExtendedEvent;
    import com.distriqt.extension.firebase.crashlytics.FirebaseCrashlytics;
    import com.distriqt.extension.inappbilling.events.PurchaseEvent;
    import com.flexnroses.mobile.utils.hardware.ExifUtils;
    import com.hurlant.util.Base64;
    import com.utils.ArrayUtil;
    import com.utils.Console;
    import com.utils.DateParser;
    import com.utils.DevHelper;
    import com.utils.ObjectUtils;
    import com.utils.ObjectUtils;
    import com.utils.RegExpUtil;
    import com.utils.RegExpUtil;

    import feathers.events.FeathersEventType;

    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.BitmapData;
    import flash.display.Loader;
    import flash.events.ErrorEvent;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.MediaEvent;
    import flash.events.PermissionEvent;
    import flash.events.StageOrientationEvent;
    import flash.filesystem.File;
    import flash.filesystem.FileMode;
    import flash.filesystem.FileStream;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.media.CameraRoll;
    import flash.media.MediaPromise;
    import flash.net.URLRequest;
    import flash.net.URLVariables;
    import flash.permissions.PermissionStatus;
    import flash.utils.ByteArray;
    import flash.utils.Dictionary;
    import flash.utils.getTimer;
    import flash.utils.setTimeout;

    import jp.shichiseki.exif.ExifInfo;
    import jp.shichiseki.exif.ExifLoader;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.display.DisplayObject;
    import starling.events.Event;
    import starling.events.TouchPhase;
    import starling.textures.Texture;

    import uk.soulwire.utils.display.DisplayUtils;

    public class TuningScreenMediator extends StackedScreenMediator {
    private static const AUTH_ACTION_SAVE:String = "save";
    private static const AUTH_ACTION_BUY_PACKET:String = "by_car_part_packet";
    public static const AUTH_ACTION_UPLOAD_BACKGROUND:String = "uploadBackground";
    public static const AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA:String = "AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA";
    public static var logoBitmap:Bitmap;
    private var carsProxy:CarsProxy;
    private var _car:Car;
    private var carLoaded:Boolean;
    private var previousPart:Part;
    private var firstPartSlotSelectionMade:Boolean;
    private var tuningColorPicker:TuningColorPicker;
    public var modifiedTuning:Boolean;
    private var inFullscreen:Boolean;
    private var shareToInstaragm:Boolean = false;
    private var cameraRoll:CameraRoll;
    private var mediaPromise:MediaPromise;
    private var exifLoader:ExifLoader;
    private var imageLoader:Loader;
    private var promoVideoManager:PromoVideoManager;
    private var restoreAdsBanner:Boolean = false;
    private var fastModeExitHandler:Function;

    public static var backgroundsToLoad : Array = [];

    public function TuningScreenMediator(stackedScreen:StackedScreen) {
        super(stackedScreen);
    }

    public function get tuningScreen():TuningScreen {
        return viewComponent as TuningScreen;
    }

    override public function onRegister():void {
        super.onRegister();
        BitmapDataUtil.clearWritingError();
        // *********** Main menu events **************
        tuningScreen.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
        tuningScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, tuningScreenLogoutSelectedHandler);
        tuningScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
        tuningScreen.addEventListener(StackedScreen.BRANDS_SELECTED, tuningScreenBrandsSelectedSelectedHandler);
        tuningScreen.addEventListener(StackedScreen.HOME_SELECTED, tuningScreenHomeSelectedSelectedHandler);
        tuningScreen.addEventListener(StackedScreen.VOTE_SELECTED, tuningScreenVotesSelectedHandler);
        tuningScreen.addEventListener(BrandsScreen.INSTRUCTION_SELECTED, tuningScreenInstructionSelectedHandler);
        tuningScreen.addEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);



        tuningScreen.addEventListener(CarView.FIRST_IMAGE_DOWNLOADED, tuningScreenFirstImageDownloadedHandler);
        tuningScreen.addEventListener(FeathersEventType.INITIALIZE, tuningScreenInitializeStageHandler);
        tuningScreen.addEventListener(TuningScreen.PART_SELECTED, tuningScreenPartSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.EXIT_FAST_MODE, tuningScreenExitFastModeHandler);
        tuningScreen.addEventListener(TuningScreen.ENTER_FAST_MODE, tuningScreenEnterFastModeHandler);
        tuningScreen.addEventListener(TuningScreen.PART_UNSELECTED, tuningScreenPartUnselectedHandler);
        tuningScreen.addEventListener(TuningScreen.DOWNLOADS_BUTTON_SELECTED, tuningScreenDownloadsButtonSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SAVE_TUNING_SELECTED, tuningScreenSaveTuningSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.PART_SLOT_SELECTED, tuningScreenPartSlotSelectedHandler);
        tuningScreen.addEventListener(TuningColorsView.COLOR_SELECTED, tuningScreenColorSelectedHandler);
        tuningScreen.addEventListener(TuningSelectedColorView.TOGGLE_COLOR_SELECTION, tuningScreenToggleColorSelectionHandler);
        tuningScreen.addEventListener(TuningColorsView.MATERIAL_SELECTED, tuningScreenMaterialSelected);
        tuningScreen.addEventListener(TuningSelectedStateView.STATE_SELECTED, tuningScreenStateSelected);
        tuningScreen.addEventListener(TuningDiskSettingsView.SHOW_BUY_PACKET_CAMBER, tuningScreenShowBuyPacketCamber);
        tuningScreen.addEventListener(TuningSelectedStateView.STATE_SELECTED_FOR_DISK_SETTINGS, onChangeNeonForDiskState);
        tuningScreen.addEventListener(TuningSuspensionControl.SUSPENSION_CHANGED, tuningScreenSuspensionChanged);
        tuningScreen.addEventListener(TuningOffsetControl.FRONT_OFFSET_CHANGED, tuningScreenFrontOffsetChanged);
        tuningScreen.addEventListener(TuningScreen.PHOTO_BUTTON_SELECTED, tuningScreenPhotoButtonSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SAVE_PROMO_VIDEO_SELECTED, tuningScreenSavePromoVideoSelectedHandler);
        tuningScreen.addEventListener(TuningColorsView.COLOR_PICKER_SELECTED, tuningScreenColorPickerSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SETTINGS_SELECTED, tuningScreenSettingsSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.FULLSCREEN_SELECTED, tuningScreenFullscreenSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHARE_VK_SELECTED, tuningScreenShareVkSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHARE_OK_SELECTED, tuningScreenShareOkSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHARE_PIN_SELECTED, tuningScreenSharePinSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHARE_FB_SELECTED, tuningScreenShareFbSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHARE_INSTAGRAM_SELECTED, tuningScreenSharInstagramSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.SHAVING_SELECTED, tuningScreenShavingSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.ACCEPT_CAMERA_SELECTED, tuningScreenAcceptCameraSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.CANCEL_CAMERA_SELECTED, tuningScreenCancelCameraSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.BACKGROUND_UPLOAD_COMPLETE, tuningScreenUploadCustomBackgroundHandler);
        tuningScreen.addEventListener(CarView.SPNAPSHOT_FROM_CAMERA_CREATED, handleSnapShotFromCameraCreated);
        tuningScreen.addEventListener(TuningScreen.PART_SELECTED_DATA_CHAGE, tuningScreenPartSelectedChangeDataHandler);
        tuningScreen.addEventListener(TuningScreen.PART_DISK_INFO_POP_UP, tuningScreenPartDiskInfoPopUpHandler);
        tuningScreen.addEventListener(TuningScreen.CAR_PART_LOCK, tuningScreenCarLock);
        tuningScreen.addEventListener(BrandsScreen.BUY_NOW_PACKET, onBuyNowPacket);
        tuningScreen.addEventListener(TuningScreen.CAR_PART_FULL_LOADED, onCarPartFullLoaded);
        tuningScreen.addEventListener(TuningScreen.CAR_PART_LOADING_PROGRESS, onCarPartLoadingProgress);
        tuningScreen.addEventListener(TuningScreen.PART_INFO_POP_UP, onPartInfoPopUp);
        tuningScreen.addEventListener(TuningScreen.ON_OPEN_SHARE_POP_UP, onOpenSharePopUp);
        tuningScreen.addEventListener(TuningScreen.BACKGROUND_SELECTED, tuningScreenBackgroundSelectedHandler);
        tuningScreen.addEventListener(TuningScreen.BACKGROUND_UPLOAD, tuningScreenBackgroundUploadHandler);
        tuningScreen.addEventListener(TuningScreen.SNAPSHOT_BACKGROUND, tuningScreenSnapShotBackgroundHandler);
        tuningScreen.addEventListener(TuningScreen.BACKGROUND_DELETE, tuningScreenBackgroundDeleteHandler);
        tuningScreen.addEventListener(TuningScreen.SHOW_BEST_VIEW, showBestViewHandler);
        tuningScreen.addEventListener(PackagesView.VIEW_CAR_360, viewCar360);
        tuningScreen.addEventListener(BrandsScreen.LANGUAGE_CHANGED, tuningScreenLanguageChangedHandler);

        tuningScreen.addEventListener(PackagesView.SHOW_CONFIRM_POP_UP, displayRewardConfirmPopUp);
        tuningScreen.addEventListener(PackagesView.SHOW_NO_COIN_POP_UP, displayRewardNoCoinPopUp);
        /*var packagesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
        tuningScreen.packages = packagesProxy.packages;*/
        Starling.juggler.delayCall(initialize, 1);
    }

        private function showBestViewHandler(event:starling.events.Event):void {
            rotateToBestView();
        }

    private function initialize():void{
        if (BillingManager.$.allPackets) {
            tuningScreen.packages = BillingManager.$.allPackets;
            var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(Constants.IN_APP_PURCHASE_ENABLED) {
                usersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
                usersProxy.getPurchases();
            }
        }
        //
        if(Device.isIphoneXVariant()){
            tuningScreen.stage.starling.nativeStage.addEventListener(StageOrientationEvent.ORIENTATION_CHANGE, stageOrientationChangeHandler);
            tuningScreen.updateButtonPositions();
        }
    }

    private function stageOrientationChangeHandler(event:StageOrientationEvent):void {
        if(!tuningScreen || !tuningScreen.stage){
            event.currentTarget.removeEventListener(event.type, stageOrientationChangeHandler);
            return;
        }
        tuningScreen.updateButtonPositions();
    }

    private var _garage_delay:Number = 0.5;
    private function displayRewardNoCoinPopUp(event: starling.events.Event): void {
        userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_NO_COIN_POPUP, event.data);
    }

    private function displayRewardConfirmPopUp(event: starling.events.Event): void {
        sendNotification(Notifications.DISPLAY_REWARD_CONFIRM_POPUP, event.data);
    }

    private function loginSelectedHandler(event: starling.events.Event) : void {
        tuningScreen.clearSound();
        userAuthenticate(StackedScreenExtendedMediator.AUTH_ACTION_TUNING);
    }

    private function userAuthenticate(pendingforauction : String, pendingNotificationData:Object = null):void {
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

        if (!usersProxy.currentUser) _garage_delay = 1.5;
        else _garage_delay = 0.2;

        ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

        pendingForAuthAction = pendingforauction;
        pendingForAuthData = pendingNotificationData;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function tuningScreenLanguageChangedHandler(event:starling.events.Event):void {
        var languagesProxy:LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguage(tuningScreen.selectedLanguage);
    }

    private function tuningScreenInstructionSelectedHandler(event:starling.events.Event):void {
        tuningScreen.clearSound();
        TutorialView.showCustomUrl();
    }

    private function mailSelectedHandler(event:starling.events.Event):void {
        pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_MAILS;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function tuningScreenLogoutSelectedHandler(event:starling.events.Event) : void {
        tuningScreen.clearSound();
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.logoutCurrentUser();

        AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);

        tuningScreen.ShowDone();
        pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_LOGOUT;
    }

    private function tuningScreenVotesSelectedHandler(event:starling.events.Event):void {
        tuningScreen.clearSound();
        if(modifiedTuning)
        {
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {
                    sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
                }
            });
        }
        else {
            sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
        }
    }
    private function challengeSelectHandler(event:starling.events.Event) : void
    {
        tuningScreen.clearSound();
        if(modifiedTuning)
        {
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {
                    sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN, null);
                }
            });
        }
        else {
            sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN, null);
        }

    }


    private function tuningScreenFriendsSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, null);
    }



    private function tuningScreenHomeSelectedSelectedHandler(event:starling.events.Event):void {
        AdsManager.$.destroyBanner();
        tuningScreen.clearSound();
        if(modifiedTuning) {
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {
                    openHomePage();
                }
            });
        } else {
            openHomePage();
        }
    }

    private function openHomePage():void{
        if(tuningScreen && tuningScreen.car && tuningScreen.car.isChallenge){
            sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
            return;
        }
        sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
    }

    private function tuningScreenBrandsSelectedSelectedHandler(event:starling.events.Event):void {
        tuningScreen.clearSound();
        if(modifiedTuning) {
            SoundManager.playSound(SoundManager.SELECT_PART);
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {
                    if(HomeScreenPopUpMediator.isBrandScreenFirstOpen) {
                        sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
                    } else {
                        sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
                    }
                    HomeScreenPopUpMediator.isBrandScreenFirstOpen = false;
                }
            });
        } else {
            if(HomeScreenPopUpMediator.isBrandScreenFirstOpen) {
                sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
            } else {
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
            HomeScreenPopUpMediator.isBrandScreenFirstOpen = false;
        }

    }


    protected function tuningScreenBackgroundSelectedHandler(event:starling.events.Event):void {

        var selectedFillColor:Object = tuningScreen.selectedFillColor;


        var copyOfCarBackground:CarBackground = null;
        if (tuningScreen && tuningScreen.selectedBackground) {
            copyOfCarBackground = tuningScreen.selectedBackground.copyWithShowMainUrl();
        }

        //tuningSettingsPopUp.touchable = false;
        Starling.juggler.delayCall(function() : void
        {
            try {




                sendNotification(Notifications.CAR_BACKGROUND_SELECTED, {carBackground:copyOfCarBackground, color:selectedFillColor});
                //sendNotification(Notifications.REMOVE_POP_UP, tuningSettingsPopUp);

            } catch (e:Error) {

                // Logger.e("tuningSettingsPopUpBackgroundSelectedHandler error:" + e.messages, this);

                Tracer.print(e.getStackTrace());
            }


        }, 0.2);
    }

    protected function tuningScreenBackgroundUploadHandler(event:starling.events.Event):void {
        selectImageFromCameraRoll();
    }

    private function selectImageFromCameraRoll() : void {
        pendingForAuthAction = AUTH_ACTION_UPLOAD_BACKGROUND;
        ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            //PopUpManager.
            //PopUpManager.addPopUp(tuningSettingsPopUp, false); //change by maxb showPopUp to addPopUp
        }
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    protected function onOpenSharePopUp(event:starling.events.Event):void {

        sendNotification(Notifications.DISPLAY_TUNING_START_SHARING_POP_UP, {});
    }

    protected function onCarPartFullLoaded(event:starling.events.Event):void {

        if (alertView) {

            alertView.hide();
        }

        tuningScreen.updateVisibleShaving();

        trace("onCarPartFullLoaded1");

        /*{
            Tracer.print("Loading car backgrounds", "TuningScreenMediator", "onCarPartFullLoaded");
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            var carBackgroundsProxy:CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
            carBackgroundsProxy.loadBackgrounds(usersProxy.currentUser ? usersProxy.currentUser.id : null, false, car.carType);
        }*/
    }

    protected function onCarPartLoadingProgress(event:starling.events.Event):void {

        if (alertView) {

            alertView.hide();
            return;

            var progressStr:String = (event.data.progress * 100).toFixed(0);
            alertView.text = AlertView.language.translate("mob_alertsAndOverlays", "loading") + "\n" + progressStr + "%";
        }
    }

    public function set car(value:Car):void {


        _car = value;
        if(!_car.brandImageUrl){
            var brandsProxy : BrandsProxy = facade.retrieveProxy(BrandsProxy.NAME) as BrandsProxy;
            var brandData:Object = brandsProxy.getBrandData(_car.name);
            if(brandData && brandData["logo"]){
                _car.brandImageUrl = brandData["logo"];
            }
        }

        if (_car && !_car.packet) {

            //Если нет пакета пытаемся его найти

            //var facade : ApplicationFacade = ApplicationFacade.getInstance();
            var packagesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
            var packs:Array = packagesProxy.findPackagesByCarId(_car.id);

            for (var k:int = 0; k < packs.length; k++) {

                _car.addPacket(packs[k]);

            }
        }
    }

    public function get car():Car {

        return _car;
    }





    override public function onRemove() : void
    {
        try {
            super.onRemove();
            AdsManager.$.destroyBanner();
            if(HTMLCacher.isInitialized)
            {
                HTMLCacher.instance.cache();
                HTMLCacher.instance.disposeCacher(); // dispose in order to free up memory
            }

            //sendNotification(Notifications.CANCEL_ALL_HTTP_REQUESTS);

            var sharingProxy:SharingProxy = facade.retrieveProxy(SharingProxy.NAME) as SharingProxy;
            sharingProxy.endUpload();

            var sharingVkProxy:SharingVkProxy = facade.retrieveProxy(SharingVkProxy.NAME) as SharingVkProxy;
            sharingVkProxy.endUpload();
            //
            Starling.current.nativeStage.removeEventListener(StageOrientationEvent.ORIENTATION_CHANGE, stageOrientationChangeHandler);
            //

            tuningScreen.removeEventListener(CarView.FIRST_IMAGE_DOWNLOADED, tuningScreenFirstImageDownloadedHandler);
            tuningScreen.removeEventListener(FeathersEventType.INITIALIZE, tuningScreenInitializeStageHandler);
            tuningScreen.removeEventListener(TuningScreen.PART_SELECTED, tuningScreenPartSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.EXIT_FAST_MODE, tuningScreenExitFastModeHandler);
            tuningScreen.removeEventListener(TuningScreen.ENTER_FAST_MODE, tuningScreenEnterFastModeHandler);
            tuningScreen.removeEventListener(TuningScreen.PART_UNSELECTED, tuningScreenPartUnselectedHandler);
            tuningScreen.removeEventListener(TuningScreen.DOWNLOADS_BUTTON_SELECTED, tuningScreenDownloadsButtonSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SAVE_TUNING_SELECTED, tuningScreenSaveTuningSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.PART_SLOT_SELECTED, tuningScreenPartSlotSelectedHandler);
            tuningScreen.removeEventListener(TuningColorsView.COLOR_SELECTED, tuningScreenColorSelectedHandler);
            tuningScreen.removeEventListener(TuningSelectedColorView.TOGGLE_COLOR_SELECTION, tuningScreenToggleColorSelectionHandler);
            tuningScreen.removeEventListener(TuningColorsView.MATERIAL_SELECTED, tuningScreenMaterialSelected);
            tuningScreen.removeEventListener(TuningSelectedStateView.STATE_SELECTED, tuningScreenStateSelected);
            tuningScreen.removeEventListener(TuningDiskSettingsView.SHOW_BUY_PACKET_CAMBER, tuningScreenShowBuyPacketCamber);
            tuningScreen.removeEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);


            tuningScreen.removeEventListener(TuningSelectedStateView.STATE_SELECTED_FOR_DISK_SETTINGS, onChangeNeonForDiskState);
            tuningScreen.removeEventListener(TuningSuspensionControl.SUSPENSION_CHANGED, tuningScreenSuspensionChanged);
            tuningScreen.removeEventListener(TuningScreen.PHOTO_BUTTON_SELECTED, tuningScreenPhotoButtonSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SAVE_PROMO_VIDEO_SELECTED, tuningScreenSavePromoVideoSelectedHandler);
            tuningScreen.removeEventListener(TuningColorsView.COLOR_PICKER_SELECTED, tuningScreenColorPickerSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SETTINGS_SELECTED, tuningScreenSettingsSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.FULLSCREEN_SELECTED, tuningScreenFullscreenSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SHARE_VK_SELECTED, tuningScreenShareVkSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SHARE_OK_SELECTED, tuningScreenShareVkSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SHARE_PIN_SELECTED, tuningScreenShareVkSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SHARE_FB_SELECTED, tuningScreenShareFbSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.SHARE_INSTAGRAM_SELECTED, tuningScreenSharInstagramSelectedHandler);

            tuningScreen.removeEventListener(TuningScreen.SHAVING_SELECTED, tuningScreenShavingSelectedHandler);


            tuningScreen.removeEventListener(TuningScreen.ACCEPT_CAMERA_SELECTED, tuningScreenAcceptCameraSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.CANCEL_CAMERA_SELECTED, tuningScreenCancelCameraSelectedHandler);

            tuningScreen.removeEventListener(CarView.SPNAPSHOT_FROM_CAMERA_CREATED, handleSnapShotFromCameraCreated);

            tuningScreen.removeEventListener(TuningScreen.PART_SELECTED_DATA_CHAGE, tuningScreenPartSelectedChangeDataHandler);

            tuningScreen.removeEventListener(TuningScreen.PART_DISK_INFO_POP_UP, tuningScreenPartSelectedChangeDataHandler);


            tuningScreen.removeEventListener(TuningScreen.CAR_PART_LOCK, tuningScreenCarLock);

            tuningScreen.removeEventListener(BrandsScreen.BUY_NOW_PACKET, onBuyNowPacket);


            tuningScreen.removeEventListener(PackagesView.VIEW_CAR_360, viewCar360);


            tuningScreen.removeEventListener(TuningScreen.CAR_PART_FULL_LOADED, onCarPartFullLoaded);

            tuningScreen.removeEventListener(TuningScreen.CAR_PART_LOADING_PROGRESS, onCarPartLoadingProgress);


            tuningScreen.removeEventListener(TuningScreen.PART_INFO_POP_UP, onPartInfoPopUp);


            tuningScreen.removeEventListener(TuningScreen.BACKGROUND_SELECTED, tuningScreenBackgroundSelectedHandler);
            tuningScreen.removeEventListener(TuningScreen.BACKGROUND_UPLOAD, tuningScreenBackgroundUploadHandler);
            tuningScreen.removeEventListener(TuningScreen.SNAPSHOT_BACKGROUND, tuningScreenSnapShotBackgroundHandler);
            tuningScreen.removeEventListener(TuningScreen.BACKGROUND_DELETE, tuningScreenBackgroundDeleteHandler);


            tuningScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, tuningScreenBrandsSelectedSelectedHandler);
            tuningScreen.removeEventListener(StackedScreen.HOME_SELECTED, tuningScreenHomeSelectedSelectedHandler);
            tuningScreen.removeEventListener(StackedScreen.VOTE_SELECTED, tuningScreenVotesSelectedHandler);

            tuningScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, tuningScreenLogoutSelectedHandler);
            tuningScreen.removeEventListener(BrandsScreen.INSTRUCTION_SELECTED, tuningScreenInstructionSelectedHandler);
            tuningScreen.removeEventListener(BrandsScreen.LANGUAGE_CHANGED, tuningScreenLanguageChangedHandler);

            tuningScreen.removeEventListener(PackagesView.SHOW_CONFIRM_POP_UP, displayRewardConfirmPopUp);
            tuningScreen.removeEventListener(PackagesView.SHOW_NO_COIN_POP_UP, displayRewardNoCoinPopUp);

            tuningScreen.removeEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);

            viewComponent.removeFromParent(true);

            //tuningScreen.dispose();

            viewComponent = null;

            if (_car) {

                _car.dispose();


            }

            _car = null;


            carsProxy = null;

            previousPart = null;

            if (tuningColorPicker) {
                tuningColorPicker.removeFromParent(true);
                tuningColorPicker = null;
            }

        } catch (e:Error) {
            FirebaseCrashlytics.service.recordError(e);
        }
    }

    private var lastComponent : Part;
    private function onPartInfoPopUp(event:starling.events.Event=null):void
    {
        var partItem : Part;
        if (event){
            var partItemRenderer:IPartItemRenderer = event.data.part as IPartItemRenderer;
            partItem = partItemRenderer.part as Part;
        }
        else{
            partItem = lastComponent as Part;
        }

        var activeComponent : String = String(partItem.name).indexOf("{") != -1 ?
                String(partItem.name).split("{")[1].split("}")[0] : partItem.name;

        var variables : URLVariables = new URLVariables();
        variables.activeComponent = activeComponent != null ? activeComponent : "";
        variables.type = car.type ? car.type : "";
        variables.make = car.name ? car.name : "";
        variables.model = car.model ? car.model : "";
        variables.year = car.year ? car.year : "";
        if(partItem.parentSlot)
        {
            variables.component = partItem.parentSlot.name ? partItem.parentSlot.name : "" ;
        }





        if (!tuningScreen.infoView){
            tuningScreen.infoView = new DetailInfoView(tuningScreen);
            tuningScreen.addChild(tuningScreen.infoView);
        }
        else
        {
            if (tuningScreen.mainMenuView.isOpened){
                tuningScreen.infoView.isOpened = true;
            }
        }

        if (tuningScreen.infoView) {
            var n : String = "";
            if(partItem && partItem.parentSlot && partItem.parentSlot.parentCar)
            {
                n += partItem.parentSlot.parentCar.year ? partItem.parentSlot.parentCar.year.toString() + " " : "";
                n += partItem.parentSlot.parentCar.name ? partItem.parentSlot.parentCar.name + " " : "";
                n += partItem.parentSlot.displayName ? partItem.parentSlot.displayName  + " " : "";
                n += partItem.name ? partItem.name : "";
            }
            else {
                n = "No information available";
            }
            tuningScreen.infoView.name = n;
        }

        tuningScreen.infoView.shopVariables = variables;
        tuningScreen.infoView.shopURL = DetailInfoView.SHOP_URL_BASE;

        tuningScreen.infoView.show(true);

        if(Constants.partsInfoMap.hasOwnProperty(partItem.id))
        {
            Constants.partsInfoMap[partItem.id].carType = car.carType;
            tuningScreen.infoView.properties = Constants.partsInfoMap[partItem.id];
        }
        else {
            if(partItem.name.indexOf("Custom") >= 0)
            {
                tuningScreen.infoView.show(true);
                tuningScreen.infoView.properties = null;
            }
            else {
                (facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy).loadPartInfo(partItem.id);
                tuningScreen.infoView.showLoading();

            }
        }
    }

    private function handleComponentInfoLoaded(info:Object):void
    {
        if(info){
            info.carName = car.name.toLowerCase();
        }
        tuningScreen.infoView.hideLoading();
        tuningScreen.infoView.properties = info;
    }

    private function handleDisplayTimelineScreenFeed(data:Object):void {
        if(modifiedTuning) {
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {
                    sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
                }
            });
        } else {
            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
        }
    }

    private function tuningScreenShowBuyPacketCamber(event:starling.events.Event):void {


        this.tuningScreen.showBuyPacket(car.packet);
    }

    private function viewCar360(event:starling.events.Event):void {

        var car:Car = event.data.car;

        sendNotification(Notifications.DISPLAY_TUNING_SCREEN, {car: car, backLevel: 0, carView: null, tunning: null});
    }

    public override function listNotificationInterests():Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.CAR_LOADED,
            Notifications.CAR_LOADED_STEP,
            Notifications.CAR_LOADING_FAILED,
            Notifications.CAR_TUNNING_LOADED,
            Notifications.CAR_TUNNING_LOADING_FAILED,
            Notifications.TUNING_SAVED,
            Notifications.BUY_NOW_PACKET,
            Notifications.PACKET_PURCHASE_SUCCESS,
            Notifications.TUNING_NOT_SAVED,
            Notifications.TUNING_BEGIN_ADDING,
            Notifications.SUBMIT_CHALLENGE,
            Notifications.CAR_BACKGROUND_SELECTED,
            Notifications.CAR_BACKGROUNDS_LOADED,
            Notifications.CAR_BACKGROUND_UPLOADED,
            Notifications.CAR_BACKGROUND_UPLOADING_FAILED,
            Notifications.USER_AUTHENTICATED,
            Notifications.SHARING_READY,
            Notifications.TUNING_SHARING_MESSAGE_ENTERED,
            Notifications.SHARING_PREPARING_ERROR,
            Notifications.SHARING_PHOTO_UPLOADED,
            Notifications.SHARING_PHOTO_UPLOADING_ERROR,
            Notifications.APPLICATION_WENT_BACKGROUND,
            Notifications.CAMERA_SNAPSHOT,
            Notifications.TUNING_VIDEO_SAVE,
            Notifications.PACKAGES_LOADED,
            Notifications.PACKAGES_LOADING_FAILED,
            Notifications.APPLICATION_BECAME_ACTIVE,
            Notifications.PURCHASES_LOADED,
            Notifications.INTERNET_CONNECTED,
            Notifications.INTERNET_DISCONNECTED,
            Notifications.DISK_BRANDS_LOADED,
            Notifications.DISK_BRANDS_LOADING_FAILED,
            Notifications.SELECTED_SOCIAL_FOR_SHARING,
            Notifications.CAR_BACKGROUNDS_LOADING_FAILED,
            Notifications.CAR_BACKGROUND_DELETED,
            Notifications.CAR_BACKGROUND_DELETION_FAILED,

            Notifications.LANGUAGES_LOADED,
            Notifications.LANGUAGES_LOADING_FAILED,
            Notifications.LANGUAGE_LOADED,
            Notifications.LANGUAGE_LOADING_FAILED,
            Notifications.USER_LOGGED_OUT,
            Notifications.USER_AUTHENTICATED,
            Notifications.USER_SESSION_EXTENDED,
            Notifications.USER_SESSION_EXTENDING_FAILED,
            Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE,

            Notifications.COMPONENT_INFO_LOADED,
            Notifications.COMPONENT_INFO_LOAD_FAILED,

            Notifications.PENDING_FRIENDS_LOADED,
            Notifications.TIMELINE_SCREEN_FEED_PREVIEW,

            Notifications.REMOVE_POP_UP,
            Notifications.USER_LOGIN_CANCELLED,
            Notifications.DISPLAY_TUNING_VIDEO_POPUP,
            Notifications.TUNING_REFRESH
        ]);
    }

    private function handleComponentInfoLoadFailed():void {
        alertView = AlertView.show("There is no data!");
    }

    public override function handleNotification(note:INotification):void {
        switch (note.getName()){

            case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                this.handleDisplayTimelineScreenFeed(note.getBody());
                break;

            case Notifications.COMPONENT_INFO_LOADED:
                this.handleComponentInfoLoaded(note.getBody().info);
                break;

            case Notifications.COMPONENT_INFO_LOAD_FAILED:
                this.handleComponentInfoLoadFailed();
                break;
                ////////////////////////////////////////////////

            case Notifications.INTERNET_CONNECTED: {
                this.handleInternetConnected();
            }
                break;
            case Notifications.INTERNET_DISCONNECTED: {
                this.handleInternetDisconnected();
            }
                break;
            case Notifications.PACKET_PURCHASE_SUCCESS:
                var isRestoring:Boolean = (note.getBody().hasOwnProperty("isRestoring") && note.getBody().isRestoring);
                this.handleBuyPacketSuccess(note.getBody().packet, isRestoring);
                break;

            case Notifications.APPLICATION_BECAME_ACTIVE: {

                handleApplicationBecameActive();
            }
                break;
            case Notifications.DISK_BRANDS_LOADING_FAILED: {

                handleDiskBrandsLoadingFailed();
            }
                break;

            case Notifications.DISK_BRANDS_LOADED: {

                handleDiskBrandsLoaded();
            }
                break;
            case Notifications.CAR_TUNNING_LOADED:
            case Notifications.CAR_LOADED:
                if(tuningScreen.car.isChallenge && tuningScreen.challengeData){
                    handleChallengeLoaded();
                    Car(note.getBody()).isChallenge = true;
                }

                handleCarLoaded(note.getBody() as Car);
                break;
            case Notifications.CAR_LOADED_STEP: {
                handleCarLoadedStep(note.getBody().progress);

            }
                break;



            case Notifications.SELECTED_SOCIAL_FOR_SHARING:
                handleSelectedSocialForSharing(note.getBody().socialId);
                break;


            case Notifications.BUY_NOW_PACKET:
                handleBuyNowPacket(note.getBody().packet, note.getBody().popUp);
                break;

            case Notifications.CAR_LOADING_FAILED:
            case Notifications.CAR_TUNNING_LOADING_FAILED:
                handleCarLoadingFailed(note.getBody().isChallenge);
                break;

            case Notifications.PURCHASES_LOADED:
                handlePurchasesLoaded(note.getBody().purchases, note.getBody().resetPackages);
                break;


            case Notifications.PURCHASES_LOADING_FAILED:
                handlePurchasesLoadingFailed();
                break;

            case Notifications.PACKAGES_LOADED:
                handleCarPackagesLoaded(note.getBody().packages);
                break;

            case Notifications.PACKAGES_LOADING_FAILED:
                handlePackagesLoadingFailed();
                break;

            case Notifications.TUNING_SAVED:
                handleTuningSaved(note.getBody() as Tuning);
                break;

            case Notifications.TUNING_NOT_SAVED:
                handleTuningNotSaved();
                break;

            case Notifications.TUNING_BEGIN_ADDING:
                this._countSaveAttempt = 0;
                handleTuningBeginAdding(note.getBody() as String);
                break;
            case Notifications.SUBMIT_CHALLENGE:
                this._countSaveAttempt = 0;

                handleSubmitChallenge();
                break;

            case Notifications.CAR_BACKGROUND_SELECTED:
                handleCarBackgroundSelected(note.getBody());
                break;

            case Notifications.USER_AUTHENTICATED:
            case Notifications.USER_SESSION_EXTENDED:
                handleUserAuthenticated(note.getBody().wasAlreadyAuthenticated);
                break;

            case Notifications.SHARING_READY:
                handleSharingReady(note.getBody() as String);
                break;

            case Notifications.SHARING_PREPARING_ERROR:
                handleSharingPreparingError(note.getBody() as String);
                break;

            case Notifications.TUNING_SHARING_MESSAGE_ENTERED:
                handleTuningSharingMessageEntered(note.getBody() as String);
                break;

            case Notifications.SHARING_PHOTO_UPLOADED:
                handleSharingPhotoUploaded();
                break;

            case Notifications.SHARING_PHOTO_UPLOADING_ERROR:
                handleSharingPhotoUploadingError();
                break;

            case Notifications.APPLICATION_WENT_BACKGROUND:
                handleApplicationWentBackground();
                break;
            case Notifications.CAMERA_SNAPSHOT:
                handleCreateCameraSnapshot(note.getBody());
                break;

            case Notifications.TUNING_VIDEO_SAVE:
                handlerTuningVideoSave(note.getBody());
                break;

            /*case Notifications.PACKET_PURCHASE_SUCCESS:
                handlePacketPurchaseSuccess(note.getBody().packet);
                break;*/

            case Notifications.CAR_BACKGROUNDS_LOADED:
                handleCarBackgroundsLoaded(note.getBody() as Array);
                break;

            case Notifications.CAR_BACKGROUNDS_LOADING_FAILED:
                handleCarBackgroundsLoadingFailed();
                break;

            case Notifications.CAR_BACKGROUND_UPLOADED:
                handleCarBackgroundUploaded(note.getBody() as Object);
                break;

            case Notifications.CAR_BACKGROUND_UPLOADING_FAILED:
                handleCarBackgroundUploadedFailed(note.getBody() as String);
                break;

            case Notifications.CAR_BACKGROUND_DELETED:
                handleCarBackgroundDeleted(note.getBody());
                break;

            case Notifications.CAR_BACKGROUND_DELETION_FAILED:
                handleCarBackgroundDeletionFailed();
                break;

            case Notifications.LANGUAGES_LOADED:
                handleLanguagesLoaded();
                break;

            case Notifications.LANGUAGES_LOADING_FAILED:
                handleLanguagesLoadingFailed();
                break;

            case Notifications.LANGUAGE_LOADED:
                handleLanguageLoaded(note.getBody().language);
                break;

            case Notifications.LANGUAGE_LOADING_FAILED:
                handleLanguageLoadingFailed();
                break;

            case Notifications.USER_LOGGED_OUT:
                handleUserLoggedOut();
                break;

            case Notifications.PENDING_FRIENDS_LOADED:
                handlePendingFriendLoaded(note.getBody());
                break;

            case Notifications.REMOVE_POP_UP:
            case Notifications.USER_LOGIN_CANCELLED:
                handlePopupRemoved();
                break;

            case Notifications.DISPLAY_TUNING_VIDEO_POPUP:
                handleVideoSaveComplete(note.getBody());
                break;

            case Notifications.TUNING_REFRESH:
                handleTuningRefresh();
                break;


                /*case Notifications.PACKAGES_LOADED:
                 handleCarPackagesLoaded(note.getBody().packages);
                 break;

                 case Notifications.PACKAGES_LOADING_FAILED:
                 handlePackagesLoadingFailed();
                 break;*/
        }

        super.handleNotification(note);
    }

    private function handleChallengeLoaded():void{
        var voteStartDate:Date = DateParser.parseDate(tuningScreen.challengeData.votingStartDate);
        var timeLeft:Number = voteStartDate.time - new Date().time;
        if(timeLeft > 0){
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.addKeyForChallenge(UsersProxy.loggedInUser().id, -parseInt(tuningScreen.challengeData.cost), function(result:Object):void{

            });

            var expireCallTime:Number = timeLeft / 1000;
            Starling.juggler.delayCall(challengeExpiredHandler, expireCallTime);
            if(timeLeft <= DateParser.MINUTE){
                var timeLeftDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.CHALLENGE_TIME_LEFT, "less than 1 minute", tuningScreen.stage);
                timeLeftDialog.open();
            }else{
                var warnCallTime:Number = (timeLeft - DateParser.MINUTE) / 1000;
                Starling.juggler.delayCall(oneMinuteLeftHandler, warnCallTime);
            }
        }else{
            challengeExpiredHandler(false);
        }
    }

    private function oneMinuteLeftHandler():void{
        var timeLeftDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.CHALLENGE_TIME_LEFT, "only 1 more minute", tuningScreen.stage);
        timeLeftDialog.open();
    }

    private function challengeExpiredHandler(returnKeys:Boolean = true):void{
        if(returnKeys){
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.addKeyForChallenge(UsersProxy.loggedInUser().id, parseInt(tuningScreen.challengeData.cost), function(result:Object):void{

            });
        }

        var challengeAlertView:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.CHALLENGE_EXPIRED, null, tuningScreen.stage);
        challengeAlertView.open();
        challengeAlertView.addEventListener(ChallengeAlertEvent.CLOSE_TRIGGERED, function(event:starling.events.Event):void{
            Starling.juggler.delayCall(function():void{
                sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
            }, 0.1);
        });
    }

    private function handlePopupRemoved(): void {
        if(restoreAdsBanner){
            AdsManager.$.bannerShow(AdsManager.$.currentBannerScreen);
            restoreAdsBanner = false;
        }
    }

    private function handleTuningRefresh(): void {
        if(tuningScreen.tunning && tuningScreen.tunning.challengeNewSystemCar && tuningScreen.car){
            tuningScreen.car.isChallenge = false;
        }
        var statsDialog:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.CHALLENGE_SUBMITTED, tuningScreen.challengeData);
        tuningScreen.addChild(statsDialog);
        statsDialog.open();
        statsDialog.addEventListener(ChallengeAlertEvent.CHALLENGES_TRIGGERED, function(event:starling.events.Event):void{
            Starling.juggler.delayCall(function():void{
                sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
            }, 0.1);
        });
    }

    private function handlePendingFriendLoaded(friends:Object):void{
        //if(tuningScreen.stage)
        tuningScreen.dispatchEventWith(FriendsListPopUp.PENDING_FRIENDS_READ, false, friends);
    }

    private function handleVideoSaveComplete(data:Object):void{
        if(tuningScreen && data.mode == TuningVideoPopup.COMPLETE){
            tuningScreen.removeVideoSpinner();
            tuningScreen.enableVideoButtons(true);
        }
    }


    private function handleLanguageLoadingFailed() : void
    {

    }

    private function handleUserLoggedOut() : void
    {
        this.tuningScreen.tunning = null;
        switch (pendingForAuthAction) {
            case StackedScreenExtendedMediator.AUTH_ACTION_LOGOUT:
            case StackedScreenExtendedMediator.AUTH_ACTION_TIMELINE:
            case StackedScreenExtendedMediator.AUTH_ACTION_GARAGE: {
                trace("handleUserLoggedOut DISPLAY_ROOT_SCREEN");
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
                break;
            case StackedScreenExtendedMediator.AUTH_ACTION_NO_COIN_POPUP:
                sendNotification(Notifications.DISPLAY_REWARD_NOCOIN_POPUP);
                break;
        }
        //tuningsListScreen.userSignedIn = false;
        sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
    }

    private function updateSettings() : void
    {
        var settingsProxy:SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var settings:Settings = settingsProxy.currentSettings;

        //settings.autoscrollEnabled = brandsScreen.autoscrollEnabled;

        if(tuningScreen.selectedLanguage)
        {
            settings.selectedLanguageCode = tuningScreen.selectedLanguage.code;
        }

        settingsProxy.update(settings);
    }



    private function handleLanguageLoaded(loadedLanguage : Language) : void
    {
        if(tuningScreen)
        {
            tuningScreen.updateWithLanguage(loadedLanguage);

            updateSettings();

            sendNotification(Notifications.LANGUAGE_CHANGED);
        }
    }

    private function handleLanguagesLoadingFailed() : void
    {
        tuningScreen.selectedLanguage = null;
    }

    private function handleLanguagesLoaded():void {

        Starling.juggler.delayCall(refreshSettings, 1.5);
    }

    private function handleInternetConnected():void {

        showAlertConnectionLost = false;

        if (reloadCarFlag) {

            tuningScreen.hideBlocker();

            alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loadingParts") + "\n" + "0%", -1);

            tuningScreen.reloadCar();
            reloadCarFlag = false;
        }
    }

    private var reloadCarFlag:Boolean = false;

    private var showAlertConnectionLost:Boolean = false;

    private function handleInternetDisconnected():void {

        //tuningScreen.sh
        if (alertView) {
            alertView.hide();
        }

        tuningScreen.backButton.touchable = true;
        tuningScreen.backButton.alpha = 1;

        if (!tuningScreen.carLoaded) {


            if (!reloadCarFlag) {
                reloadCarFlag = true;

                tuningScreen.showBlocker();
            }

        }

        if (!showAlertConnectionLost) {
            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "lostinternetconnection"), -1, false);
            showAlertConnectionLost = true;
        }
    }

    private function tuningScreenBackgroundDeleteHandler(event:starling.events.Event):void {


        deleteSelectedBackground();


    }

    private function deleteSelectedBackground() : void
    {
        var carBackgroundsProxy:CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
        if(tuningScreen && tuningScreen.selectedBackground){
            carBackgroundsProxy.deleteBackground(tuningScreen.selectedBackground);
        }
    }

    private function handleCarBackgroundDeleted(backgroundData:Object) : void {
        tuningScreen.backgroundMenuView.removeBackground(backgroundData.background, true);
        tuningScreen.backgroundMenuView.enterNormalState();
        refreshSettings();
    }

    private function handleCarBackgroundDeletionFailed() : void
    {
        tuningScreen.backgroundMenuView.enterNormalState();
    }


    private function handleCarBackgroundsLoaded(carBackgrounds : Array) : void
    {
        if(tuningScreen.carLoaded)
        {
            if(alertView)
            {
                alertView.hide();
                alertView = null;
            }
        }
        if(this.car && this.car.selectedBackground == null)
        {
            for(var i : int = carBackgrounds.length -1; i >= 0; i--)
            {
                var bg:CarBackground = carBackgrounds[i] as CarBackground;
                if(bg.id == CarBackground.DEFAULT_ID)
                {
                    car.selectedBackground = bg;
                    break;
                }
            }
        }
        this.tuningScreen.carBackgrounds = carBackgrounds;
    }

    private function handleCarBackgroundsLoadingFailed() : void
    {


    }

    private function tuningScreenSnapShotBackgroundHandler(event:starling.events.Event):void {
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
            pendingForAuthAction = AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA;
            sendNotification(Notifications.AUTHENTICATE_USER);
            return;
        }
        sendNotification(Notifications.CAMERA_SNAPSHOT);
    }

    private function handleCarBackgroundUploaded(backgroundData:Object = null):void {
        if (alertView)
            alertView.hide();
        alertView = AlertView.show(AlertView.language.translate("mob_saving", "uploaded"));
        if (alertView)
            alertView.hide();
        if(backgroundData){
            var parser:CarBackgroundsParser = new CarBackgroundsParser();
            var carBackground:CarBackground = parser.parseObject(backgroundData, null) as CarBackground;
            tuningScreen.backgroundMenuView.addBackground(carBackground);
            tuningScreen.selectedBackground = carBackground;
            Starling.juggler.delayCall(function ():void {
                tuningScreen.backgroundMenuView.setBackground(tuningScreen.selectedBackground);
            }, 0.1);
        }
    }

    private function handleCarBackgroundUploadedFailed(error : String) : void {
        if (alertView) alertView.hide();
        AlertView.show(AlertView.language.translate("mob_saving", "uploadingFailed"), -1, false);
    }

    private function handleCarLoadingFailed(isChallenge:Boolean = false):void {
        if (alertView)
            alertView.hide();


        if (tuningScreen) {

            //tuningScreen.carLoaded=true;
            tuningScreen.backButton.alpha = 1.0;

            tuningScreen.backButton.touchable = true;

            tuningScreen.carLoaded = false;
            var connectionProxy:ConnectionProxy = facade.retrieveProxy(ConnectionProxy.NAME) as ConnectionProxy;
            connectionProxy.check();


            if (!reloadCarFlag) {
                reloadCarFlag = true;

                tuningScreen.showBlocker();
            }

        }

        alertView = AlertView.show("Couldn't load car", 2);

        Starling.juggler.delayCall(function ():void {
            if(isChallenge){
                sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
            }else if(HomeScreenPopUpMediator.isBrandScreenFirstOpen) {
                sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
            } else {
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
            HomeScreenPopUpMediator.isBrandScreenFirstOpen = false;
        }, 2.2)

    }

    private function handlePurchasesLoadingFailed():void {

    }

    private function handleApplicationBecameActive():void {
        if (Device.isIos())
            return;


        if (tuningScreen) {


            //tuningScreen.invalidate();

            tuningScreen.updateLists();


            //tuningScreen
        }
    }

    private function handleCarPackagesLoaded(packages:Array):void {
        tuningScreen.packages = packages;
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.getPurchases();
    }

    private function handlePackagesLoadingFailed():void {
        //dataLoadFailed();
    }

    private function handlePurchasesLoaded(products:Array, resetPackages:Boolean):void {

        if (!products)
            return;

        var packagesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;

        if (!packagesProxy.packages)
            return;

        var i:int = 0;
        var j: int = 0;
        var packet:Packet

        var packagePacketMap:Object = {};

        for (i = 0; i < packagesProxy.packages.length; i++) {

            packet = packagesProxy.packages[i];

            for(j=0; j<products.length; j++ ){

                var packet_id: String =products[j].product;

                if(packet.id==packet_id || packet.productId === products[j].productId){
                    products[j].product = packet.id;
                    packagePacketMap[packet_id]=packet;
                }

            }

        }

        var purchases:Array = [];
        for each(packet in packagePacketMap) {

            purchases.push(new Purchase(packet.id, packet.title, true, packet.type, packet.productId));
            packet.lock = false;
            handleBuyPacketSuccess(packet, true);
        }


        if (purchases.length > 0) {
            AdsManager.turnOffAds();
            tuningScreen.topButtonPadding = 0;
        }


        tuningScreen.setPurchases(purchases, resetPackages);


    }

    private function tuningScreenCarLock(event:starling.events.Event):void{
        if(Device.isIos()){
            tuningScreenCarLockMoreOptions(event);
            return;
        }
        var selectedPacket:Packet;
        if (tuningScreen && tuningScreen.car && tuningScreen.car.packet && tuningScreen.car.packet.lock) {
            if (tuningScreen.packet) {
                selectedPacket = tuningScreen.packet;
            } else {
                selectedPacket = tuningScreen.car.packet;
            }
        }
        GameService.$.getRewardsData(function (rewards: int): void {
            if (rewards >= Packet.ONE_TIME_PACKET_FEE) {
                var eventData:Object = {packet: selectedPacket, coin3dFee: Packet.ONE_TIME_PACKET_FEE, oneTimeUnlocked:true, singleModel: true};
                eventData.moreOptionsHandler = tuningScreenCarLockMoreOptions;
                if(event.data){
                    eventData = ObjectUtils.merge(eventData, event.data);
                }
                tuningScreen.dispatchEventWith(PackagesView.SHOW_CONFIRM_POP_UP, false, eventData);
            } else {
                Starling.juggler.delayCall(function ():void {
                    AdsManager.$.rewardsCallback = function():void{
                        tuningScreenCarLock(event);
                    };
                    tuningScreen.dispatchEventWith(PackagesView.SHOW_NO_COIN_POP_UP, false, {moreOptions: true, moreOptionsHandler: tuningScreenCarLockMoreOptions});
                }, 0.25);
                tuningScreen.removeEventListener(TuningScreen.CAR_PART_LOCK, tuningScreenCarLock);
                Starling.juggler.delayCall(function ():void {
                    if(tuningScreen && tuningScreen.stage){
                        tuningScreen.addEventListener(TuningScreen.CAR_PART_LOCK, tuningScreenCarLock);
                    }
                }, 3);
            }
        })
    }

    protected function tuningScreenCarLockMoreOptions(event:starling.events.Event=null):void {
        if (tuningScreen.infoView && tuningScreen.infoView.isOpened) {
            Starling.juggler.delayCall(tuningScreenCarLock,.45);
            return;
        }

        if (tuningScreen.pendingSelectedPart && tuningScreen.pendingSelectedPart.lockPacket) {
            tuningScreen.showBuyPacket(tuningScreen.pendingSelectedPart.lockPacket);
            return;
        }

        if (tuningScreen && tuningScreen.car && tuningScreen.car.packet && tuningScreen.car.packet.lock) {

            if (tuningScreen.packet) {
                tuningScreen.showBuyPacket(tuningScreen.packet);
            }
            else {
                tuningScreen.showBuyPacket(tuningScreen.car.packet);
            }
        } else {

        }
    }

    private var pendingPacket:Packet = null;
    private var activePopUp:PopUp = null;
    private function handleBuyNowPacket(packet:Packet, popUp:PopUp):void {
        if (!tuningScreen){
            return;
        }

        var stackedScreen:StackedScreenNavigator = (tuningScreen.owner as StackedScreenNavigator);
        if (!stackedScreen.activeScreen){
            return;
        }

        if (stackedScreen.activeScreen.name != tuningScreen.name){
            return;
        }

        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if (packet.tempOpened) {
            if (popUp) {
                sendNotification(Notifications.REMOVE_POP_UP, popUp);
            }
        }

        if (Device.isIos()) {
            activePopUp = popUp;
            initPurchase(packet);
        } else {
            if (!usersProxy.currentUser && !packet.tempOpened) {
                if (popUp){
                    sendNotification(Notifications.REMOVE_POP_UP, popUp);
                }
                pendingForAuthAction = AUTH_ACTION_BUY_PACKET;
                pendingPacket = packet;
                ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

                AlertView.show(AlertView.language.translate("signUp", "register_for_purchase"), -1, false);
                sendNotification(Notifications.AUTHENTICATE_USER, {type: LoginScreen.TYPE_NORMAL});
            } else {
                activePopUp = popUp;
                initPurchase(packet);
            }
        }
    }

    private var currentPacket:Packet = null;

    private function initPurchase(packet:Packet):void {

        if (!Constants.IN_APP_PURCHASE_ENABLED) {
            return
        }

        if (packet.tempOpened) {
            sendNotification(Notifications.PACKET_PURCHASE_SUCCESS, {packet:packet});
        } else {
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if (usersProxy.isExistPacket(packet.productId))
                return;
            if (!alertPurchaseView) {
                alertPurchaseView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1);
            }

            BillingManager.$.makePurchase(packet.productId, function (purchase):void {
                var productInfo: Object = JSONer.parse(purchase.originalMessage);
                var receipt: Object = {
                    signature: purchase.signature,
                    signedData: purchase.originalMessage
                }
                var data:Object = {
                    receipt: Base64.encode(JSONer.stringify(receipt)),
                    transactionId: purchase.transactionId,
                    productId: purchase.productId,
                    packageName: productInfo['packageName']
                };
                getPurchaseItem(purchase.productId, JSON.stringify(data), purchase.transactionId, function ():void {
                    removePurchaseAlertView();
                });
            }, onPurchaseFailed);
        }

    }

    private function onPurchaseFailed(event: PurchaseEvent):void {
        if (activePopUp) {
            sendNotification(Notifications.REMOVE_POP_UP, activePopUp);
            activePopUp = null;
        }

        removePurchaseAlertView();
    }

    private function packetExistsInArray(purchases:Array, packet_id):Boolean {

        for (var index :int in purchases) {

            if (purchases[index].product == packet_id) {
                return true;
            }
        }

        return false;
    }

    protected function getPurchaseItem(productId:String, receipt:String, transactionId:String, callback:Function = null):void {

        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

        var packgesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;

        var packet:Packet = packgesProxy.findPacketByProductId(productId);

        if (!packet) {

            return;
        }

        var receiptData:Object = JSON.parse(receipt);

        var completeVerify:Function = function (verify:Boolean, error:String) :void {

            if (activePopUp) {
                sendNotification(Notifications.REMOVE_POP_UP, activePopUp);
                activePopUp = null;
            }

            if (verify) {
                sendNotification(Notifications.PACKET_PURCHASE_SUCCESS, {packet: packet});


                var json:String = usersProxy.readPurchases();

                var purchases:Array = JSON.parse(json) as Array;

                if (!packetExistsInArray(purchases, packet.productId))
                    purchases.push({product:packet.id, productId:packet.productId});
                usersProxy.savePurchases(JSON.stringify(purchases));

            } else {
                if (error != "IOError") {
                    AlertView.show(error, -1, false);
                } else {
                    usersProxy.iapVerify(receiptData, packet, completeVerify);
                    return;
                }
            }

            if (callback) {

                callback();
            }

        };

        usersProxy.iapVerify(receiptData, packet, completeVerify);

    }

    private function removeAlertView():void {
        if (alertView) {
            alertView.hide();
            alertView = null;
        }
    }

    private var alertPurchaseView:AlertView = null;

    private function removePurchaseAlertView():void {
        if (alertPurchaseView) {
            alertPurchaseView.hide();
            alertPurchaseView = null;
        }
    }

    private function onBuyNowPacket(event:starling.events.Event):void {
        var packet:Packet = event.data.packet;
        sendNotification(Notifications.BUY_NOW_PACKET, {packet: event.data.packet, popUp: null, isLocal:true});
    }

    private function handleBuyPacketSuccess(packet:Packet, isRestoring:Boolean = false):void {
        packet.lock = false;
        if(packet.type == Packet.TYPE_ALLPACKET){
            if(car && car.packet){
                car.packet.lock = false;
                car.packet.tempOpened = packet.tempOpened;
                car.packet.oneTimeUnlocked = packet.oneTimeUnlocked;
                if(packet.oneTimeUnlocked){
                    removeOneTimeUnlockPacket(car.packet);
                }
            }
            tuningScreen.removePartLock(packet);
        }else if(packet.cars && packet.cars.length && tuningScreen && tuningScreen.car) {
            for (var i:int = 0; i < packet.cars.length; i++) {
                if (tuningScreen.car.id == (packet.cars[i] as Car).id) {
                    if(car && car.packet && car.packet.productId == packet.productId){
                        car.packet.lock = false;
                        car.packet.tempOpened = packet.tempOpened;
                        car.packet.oneTimeUnlocked = packet.oneTimeUnlocked;
                        if(packet.oneTimeUnlocked){
                            removeOneTimeUnlockPacket(car.packet);
                        }
                    }
                    tuningScreen.removePartLock(packet);
                    break;
                }
            }
        }
        if(!isRestoring) {
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.getPurchases(null, false);
        }
    }

    private function tuningScreenFirstImageDownloadedHandler(e:starling.events.Event):void {
    }

    private var _alertProgressView:AlertView;

    private var countToLoad:int = 0;

    private var countLoaded:int = 0;

    private var isBackground:Boolean = true;

    private var _tuningName:String;
    private var _countSaveAttempt:int = 0;
    private function handleTuningBeginAdding(name:String):void {
        //Logger.d("handleTuningBeginAdding:"+name, this);

        _tuningName = name;
        var vsFlag:Boolean = false;
        if (name == "4a786c664bac8afc4f1c48f30750942e2fa31e7bd92067522cac3c83a3d7d011") {
            HttpClient.TOKEN = "4a786c664bac8afc4f1c48f30750942e2fa31e7bd92067522cac3c83a3d7d011";
            name = "Tuning " + car.name + " " + car.model + " " + car.year;

            vsFlag = true;

        }

        Starling.juggler.delayCall(function ():void {
            AdsManager.$.requestInterstitial(function (showed : Boolean) : void {
                try {

                    countLoaded = 0;
                    tuningScreen.closeMenu(true);
                    tuningScreen.enableButtons(false);

                    _alertProgressView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + "0%", -1);
                    tuningScreen.beginScreenShot(colorPickerViews, function ():void {
                        var tuning:Tuning;
                        var editing:Boolean = false;
                        if (vsFlag) {
                            tuning = new Tuning();
                        } else {
                            if (tuningScreen.tunning) {
                                tuning = tuningScreen.tunning;
                                editing = true; // the default value should be true, but we put false if we want to share
                            }
                            else {
                                tuning = new Tuning();
                            }
                        }

                        tuning.name = name;
                        tuning.car = car;
                        tuning.bitmapData = Starling.current.stage.drawToBitmapData();


                        var i:int = 0;
                        var arr:Array = [tuningScreen.carView.currentAngelIndex];

                        if(tuning.customSnapshotsCamera){
                            for(i = 0; i < tuning.customSnapshotsCamera.length; i++){
                                arr.push(tuning.customSnapshotsCamera[i]);
                            }
                        }
                        tuning.customSnapshotsCamera = arr;
                        var zoomLevel:Number = tuningScreen.carView.carScale;
                        var div:Number = tuningScreen.carView.offsetScale * zoomLevel;
                        var carTransform:Object = {
                            zoomLevel: zoomLevel,
                            pan:{
                                x: tuningScreen.carView.offset.x / div,
                                y: tuningScreen.carView.offset.y / div
                            }
                        };
                        if(tuning){
                            tuning.carTransform ||= [];
                            tuning.carTransform.unshift(carTransform);
                        }
                        trace("tuning.customSnapshotsCamera:", tuning.customSnapshotsCamera);
                        // ***** linked bug fix
                        for(i = 0; i < car.selectedPartSlots.length; i++){
                            var partSlot:PartSlot = car.selectedPartSlots[i] as PartSlot;
                            if(partSlot && partSlot.selectedPart && partSlot.selectedPart.linkedParts){
                                for(var j:int = 0; j < partSlot.selectedPart.linkedParts.length; j++){
                                    var linkedSlot:PartSlot = Part(partSlot.selectedPart.linkedParts[j]).parentSlot;
                                    if(linkedSlot.selectedPart && car.selectedPartSlots.indexOf(linkedSlot) == -1){
                                        car.addSelectedPartSlot(linkedSlot);
                                        if(linkedSlot.selectedPart.color != linkedSlot.selectedColor){
                                            linkedSlot.selectedPart.color = linkedSlot.selectedColor;
                                        }
                                    }
                                }
                            }
                        }
                        //

                        isBackground = true;
                        if(!tuning.backgroundId && tuning.car.defaultBgId){
                            tuning.backgroundId = tuning.car.defaultBgId;
                        }
                        var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
                        tuningsProxy.prepareAddTuningWeakDevice(tuning, editing, isBackground, completePrepareAddTuningWeakDevice);

                        tuningScreen.endScreenShot(colorPickerViews);


                    });

                } catch (e:Error) {

                    //Logger.e("handleTuningBeginAdding error:" + e.message, this);
                    Tracer.print(e.getStackTrace());

                }
            })

        }, 0.4);

    }
    private function handleSubmitChallenge():void {
        Starling.juggler.removeDelayedCalls(oneMinuteLeftHandler);
        Starling.juggler.removeDelayedCalls(challengeExpiredHandler);

        var name:String = tuningScreen.challengeData.name;
        _tuningName = name;
        Starling.juggler.delayCall(function ():void {
            AdsManager.$.requestInterstitial(saveTuningChanges);
        }, 0.1);
    }

    private function saveTuningChanges(showed : Boolean, editing:Boolean = false) : void {
        try {
            countLoaded = 0;
            tuningScreen.rotationEnabled = false;
            tuningScreen.closeMenu(true);
            tuningScreen.enableButtons(false);
            //
            if(tuningScreen.fastModeButton.isSelected && tuningScreen.isMenuOpen){
                fastModeExitHandler = function():void{
                    tuningScreen.fastModeButton.removeEventListeners();
                    tuningScreen.fastModeButton.isSelected = false;
                    saveTuningChanges(showed, editing);
                };
                return;
            }
            //
            if(tuningScreen.car.isChallenge){
                if(ObjectUtils.getValue(tuningScreen.challengeData.featuresRules, "customFrame", false)){

                }else{
                    tuningScreen.carView.offset.setTo(0, 0);
                    tuningScreen.carView.carScale = 1;
                    tuningScreen.carView.updatePosition();
                    tuningScreen.carView.showFrame(44,0);
                    // Challenge second camera background bug fix
                    tuningScreen.carView.currentAngelIndex = 0;
                    tuningScreen.carView.currentFrame = 44;
                }
            }

            _alertProgressView ||= AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + "0%", -1);
            tuningScreen.beginScreenShot(colorPickerViews, function ():void {
                tuningScreen.tunning ||= new Tuning();
                var tuning:Tuning = tuningScreen.tunning;
                tuning.name = _tuningName;
                tuning.car = car;
                // tuning.bitmapData = Starling.current.stage.drawToBitmapData();

                var i:int;
                var arr:Array = [tuningScreen.carView.currentAngelIndex];
                if(tuning.customSnapshotsCamera){
                    for(i = 0; i < tuning.customSnapshotsCamera.length; i++){
                        arr.push(tuning.customSnapshotsCamera[i]);
                    }
                }
                tuning.customSnapshotsCamera = arr;
                var carTransform:Object = {
                    zoomLevel: tuningScreen.carView.carScale,
                    pan:{
                        x: 0,
                        y: 0
                    }
                };
                if(tuning){
                    tuning.carTransform ||= [];
                    tuning.carTransform.unshift(carTransform);
                }
                Console.log("tuning.customSnapshotsCamera:", tuning.customSnapshotsCamera);
                // ***** linked bug fix
                for(i = 0; i < car.selectedPartSlots.length; i++){
                    var partSlot:PartSlot = car.selectedPartSlots[i] as PartSlot;
                    if(partSlot && partSlot.selectedPart && partSlot.selectedPart.linkedParts){
                        for(var j:int = 0; j < partSlot.selectedPart.linkedParts.length; j++){
                            var linkedSlot:PartSlot = Part(partSlot.selectedPart.linkedParts[j]).parentSlot;
                            if(linkedSlot.selectedPart && car.selectedPartSlots.indexOf(linkedSlot) == -1){
                                car.addSelectedPartSlot(linkedSlot);
                                if(linkedSlot.selectedPart.color != linkedSlot.selectedColor){
                                    linkedSlot.selectedPart.color = linkedSlot.selectedColor;
                                }
                            }
                        }
                    }
                }

                isBackground = true;
                if(!tuning.backgroundId && tuning.car.defaultBgId){
                    tuning.backgroundId = tuning.car.defaultBgId;
                }
                if(!car.challengeData || !car.challengeData.isFromGarage){
                    tuning.id = null;
                }
                tuning.shortId = null;
                var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
                tuningsProxy.prepareAddTuningWeakDevice(tuning, editing, isBackground, completePrepareAddTuningWeakDevice);
                tuningScreen.endScreenShot(colorPickerViews);
            });
        } catch (e:Error) {
            Tracer.print(e.getStackTrace());
        }
    }

    private function completePrepareAddTuningWeakDevice(success:Boolean, result:Object):void{

        try{
            _countSaveAttempt++;
            var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            if (success) {
                if (isBackground) {
                    countToLoad = 3;
                } else {
                    countToLoad = 3;
                }

                var resultJSON:Object = JSONer.parse(result as String);
                tuningsProxy.token = resultJSON.token;
                var customSnapshot:String = resultJSON.customSnapshot;
                var fullHdSnapshot:String = resultJSON.fullHdSnapshot;
                var fullHdSnapshot_2:String;
                if(resultJSON.hasOwnProperty("fullHdSnapshotBack")){
                    customSnapshot = null;
                    fullHdSnapshot_2 = resultJSON.fullHdSnapshotBack;
                    countToLoad = 3;
                    numberOfBitmapsDownloading = 2;
                }else{
                    numberOfBitmapsDownloading = 3;
                }

                _alertProgressView.text = AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + "3%";
                var isUploadFailed:Boolean = false;
                var uploadFailedCallback:Function = function():void{
                    if(!isUploadFailed) {
                        isUploadFailed = true;
                        handleTuningNotSaved();
                        sendNotification(Notifications.CAR_BACKGROUND_UPLOADING_FAILED, null);
                    }
                };
                _alertProgressView.visible = false;
                tuningScreen.carView.getSnapShotForFrame(function (bitmapData:BitmapData, snapshotType:String, step:int):void {
                    if (!bitmapData) {
                        return;
                    }
                    if (snapshotType == SnapShotFramesContainer.CREATE_SNAPSHOT_ARRAY) {
                        bitmapDownloaded();
                        return;
                    }
                    if (snapshotType == SnapShotFramesContainer.CREATE_CUSTOM_SNAPSHOT && customSnapshot) {
                        tuningsProxy.httpClient.sentBitmapData(bitmapDownloaded, 600, bitmapData, customSnapshot, bitmapData.height, bitmapData.width, false, true, uploadFailedCallback);
                    } else if(snapshotType == SnapShotFramesContainer.CREATE_SNAPSHOT_FHD) {
                        tuningsProxy.httpClient.sentBitmapData(bitmapDownloaded, 400, bitmapData, fullHdSnapshot, bitmapData.height, bitmapData.width, true, true, uploadFailedCallback);// тут должен быть с бекграундом
                    } else if(snapshotType == SnapShotFramesContainer.CREATE_SNAPSHOT_FHD_2 && fullHdSnapshot_2) {
                        tuningsProxy.httpClient.sentBitmapData(bitmapDownloaded, 200, bitmapData, fullHdSnapshot_2, bitmapData.height, bitmapData.width, true, true, uploadFailedCallback);// тут должен быть с бекграундом
                    } else if (snapshotType == SnapShotFramesContainer.CREATE_SNAPSHOT_WITHOUT_LIGHTS) {
                        bitmapDownloaded();
                    }
                });
                updateLabel();
            }else{
                if(_countSaveAttempt<2){
                    if(_alertProgressView){
                        _alertProgressView.hide();
                        _alertProgressView = null;
                    }
                    handleTuningBeginAdding(_tuningName);
                }else{
                    if(_alertProgressView){
                        _alertProgressView.hide();
                        _alertProgressView = null;
                    }
                    AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "Server error") + "\n", 1.5);
                }
            }
        }catch(e:Error){

            if(_alertProgressView){
                _alertProgressView.hide();
                _alertProgressView = null;
            }

            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "Error:"+e.getStackTrace()) + "\n" , 2.5);
        }

    }

    private var numberOfBitmapsDownloading:int;

    private function bitmapDownloaded():void {

        var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
        numberOfBitmapsDownloading--;

        this.countLoaded++;
        updateLabel();
        var percent:int = ((countLoaded / countToLoad) * 0.8) * 100;
        /*_alertProgressView.text = AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + percent.toString() + "%";*/

        if (numberOfBitmapsDownloading == 0) {
            tuningsProxy.continueAddingOfTuning(tuningScreen.challengeData);
            _alertProgressView.text = AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + (percent + 2).toString() + "%";
        }
    }

    private function updateLabel():void{
        var percent:int = ((countLoaded / countToLoad) * 0.8) * 100;
        var str:String = AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + percent.toString() + "%";
        if(_alertProgressView) {
            _alertProgressView.text = str;
            if(_alertProgressView.parent && _alertProgressView.parent.getChildIndex(_alertProgressView) != _alertProgressView.parent.numChildren-1){
                _alertProgressView.parent.setChildIndex(_alertProgressView, _alertProgressView.parent.numChildren-1);
            }
            _alertProgressView.visible = true;
        }
    }

    private function fileWritten(e:flash.events.Event):void {
    }

    private function get colorPickerViews():Array {
        if (tuningColorPicker) {
            return [tuningColorPicker, tuningColorPicker.overlayQuad, tuningColorPicker.overlayButton];
        }

        return null;
    }

    private function handleTuningSaved(tuning:Tuning):void {
        SoundManager.playSound(SoundManager.SAVE_AND_NOTIFICATION);
        _alertProgressView.text = AlertView.language.translate("mob_alertsAndOverlays", "uploading") + "\n" + "100%";


        _alertProgressView.hide();
        AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "saved"));

        if (!HttpClient.TOKEN){
            tuningScreen.tunning = tuning;
        }
        var userProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        modifiedTuning = false;
        if(car.isChallenge){
            if(tuningScreen.challengeData && tuningScreen.challengeData.type && tuningScreen.challengeData.type.indexOf("filter") != -1){
                var userName:String = userProxy.currentUser.username;
                var shortId:String = tuning.shortId;

                var index:int = 0;
                var urlToSend:String = car.getEditorLink(userName, shortId, index, true);
                var obj:Object = {
                    url: urlToSend,
                    tuning: tuning,
                    isChallenge: true
                };
                AdsManager.$.bannerHide();
                sendNotification(Notifications.DISPLAY_TUNING_EDITOR_POPUP, obj);
            }else{
                handleTuningRefresh();
            }

            return;
        }
        tuningScreen.enableButtons(true);



        //tuningScreen.

        HttpClient.TOKEN = null;

        SharingProxy.LAST_USER_PAGE = null;

        if(SAVE_FOR_FB_SHARE) {
            SAVE_FOR_FB_SHARE = false;
            SharingProxy.TUNING_FOR_SHARE = tuning;
            var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
            SharingProxy.generateURLForFbShare(userProxy.currentUser, settingsProxy.currentSettings.selectedLanguageCode)
        }
    }

    private function handleTuningNotSaved():void {
        _alertProgressView.hide();
        tuningScreen.enableButtons(true);

        HttpClient.TOKEN = null;

        SharingProxy.LAST_USER_PAGE = null;
    }

    private function tuningScreenShavingSelectedHandler(event:starling.events.Event):void {

        /*if (tuningScreen.shavingCheck.leftIsSelected) {
            this.car.isShaving = false;
        } else {
            this.car.isShaving = true;
        }

        var arrOfPartslots:Array = [];
        for each(var partSlot:PartSlot in car.selectedPartSlots) {
            if (partSlot.selectedPart && partSlot.selectedPart.states && partSlot.selectedPart.states.shaving) {
                if (this.car.isShaving) {
                    partSlot.selectedPart.selectedState = partSlot.selectedPart.states.shaving;
                } else {
                    partSlot.selectedPart.selectedState = null;
                    partSlot.currentPartWithState = null;
                }
                arrOfPartslots.push(partSlot);
            }

        }

        if (arrOfPartslots.length > 0) {
            tuningScreen.carView.updateWithPartSlots(arrOfPartslots);
        }*/

    }

    private function tuningScreenInitializeStageHandler(event:starling.events.Event):void {
        tuningScreen.enableButtons(false);

        refreshSettings();


        if (tuningScreen.loadedCar) {
            Starling.juggler.delayCall(function ():void {
                handleCarLoaded(tuningScreen.loadedCar);
            }, 0.5);

            return;
        }

        carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
        if(alertView)
        {
            alertView.hide();
        }


        if(carsProxy.hasCachedCar(tuningScreen.car))
        {
            Starling.juggler.delayCall(function() : void
            {
                startCarDataLoad(tuningScreen.car.id);
            }, 0.5);
        }
        else if(tuningScreen.car){
            startCarDataLoad(tuningScreen.car.id);
        }


    }

    private function handleDiskBrandsLoaded():void {


        if (tuningScreen.loadedCar) {
            Starling.juggler.delayCall(function ():void {
                handleCarLoaded(tuningScreen.loadedCar);
            }, 0.5);

            return;
        }

        if (carsProxy.hasCachedCar(tuningScreen.car)) {
            Starling.juggler.delayCall(loadCar, 0.5);
        }
        else {
            loadCar();
        }

    }

    private function handleDiskBrandsLoadingFailed():void {

        if (alertView)
            alertView.hide();


        if (tuningScreen) {

            //tuningScreen.carLoaded=true;
            tuningScreen.backButton.alpha = 1.0;

            tuningScreen.backButton.touchable = true;

            tuningScreen.carLoaded = false;
            var connectionProxy:ConnectionProxy = facade.retrieveProxy(ConnectionProxy.NAME) as ConnectionProxy;
            connectionProxy.check();


            if (!reloadCarFlag) {
                reloadCarFlag = true;

                tuningScreen.showBlocker();
            }

        }
    }

    private function startCarDataLoad(car_id:String):void {

        loadDiskBrands(car_id);
    }

    private function loadDiskBrands(car_id:String):void {

        var diskBrandsProxy:DiskBrandsProxy = facade.retrieveProxy(DiskBrandsProxy.NAME) as DiskBrandsProxy;

        diskBrandsProxy.loadDiskBrands(car_id);
    }

    private function loadCar():void {


        if(tuningScreen.tunning)
        {

            carsProxy.loadCarForTunning(tuningScreen.tunning)
        }
        else {
            carsProxy.loadCar(tuningScreen.car);
        }
    }

    /*private var zipCountToLoad:int=0;
    private var zipCountLoaded:int=0;
    private var _downloadZipComplete:Function = null;
    private var fZipArray:Array=[];

    private var totalCountImages:int = 0;

    private function downloadZip(car:Car, complete:Function):void{

        complete();
        return;


        var i:int = 0;
        var partSlot:PartSlot;
        var material:PartMaterial;
        var baseUrl:String;
        var fZip:FZip;
        fZipArray=[];

        _downloadZipComplete = complete;
        zipCountToLoad = 0;
        ImagesBytesList = {};
        for(i=0; i<car.possiblePartSlots.length; i++){

            partSlot = car.possiblePartSlots[i];

            if(!partSlot) continue;

            if(!partSlot.selectedPart) continue;

            if(!partSlot.selectedPart.selectedMaterial) continue;

            material = partSlot.selectedPart.selectedMaterial;


            baseUrl = CarsParser.CURRENT_CAR_IMAGE_PATH+material.componentPath;

            baseUrl=baseUrl.substr(0,baseUrl.length-1);




            fZip = new FZip();
            //fZip.addEventListener(FZipEvent.FILE_LOADED, onFzipFileLoaded);
            fZip.addEventListener(flash.events.Event.COMPLETE, onFZipComplete);

            //trace("baseUrl:", baseUrl);


            fZipArray.push({
                zip:fZip,
                baseUrl:baseUrl,
                time:0
            });


            zipCountToLoad++;

        }



        for(i=0; i<fZipArray.length; i++){

            fZip = fZipArray[i].zip;
            fZipArray[i].time = getTimer();

            trace("zip path:", baseUrl+".zip");
            fZip.load(new URLRequest(baseUrl+".zip"));
        }
        //complete();



    }

    private function getDataByFZip(fZip:FZip):Object{

        for(var i:int=0; i<fZipArray.length; i++){

            if(fZipArray[i].zip==fZip){
                return fZipArray[i];
            }
        }

        return null;
    }

    public static var ImagesBytesList:Object;

    private function onFZipComplete(event:flash.events.Event):void{

        var fZipFile:FZipFile;
        var baseUrl:String;
        var fZip:FZip = event.target as FZip;
        var file:File;
        var fileStream:FileStream;
        var filename:String;
        var filePath:String;
        fZip.removeEventListener(flash.events.Event.COMPLETE, onFZipComplete);


        var data:Object = getDataByFZip(fZip);

        if(!data) {
            trace("getDataByFZip Error")
            return;
        }

        baseUrl = data.baseUrl+"/";
        //baseUrl = CarsParser.CURRENT_CAR_IMAGE_PATH

        for(var i:int=0; i<fZip.getFileCount(); i++){

            fZipFile = fZip.getFileAt(i);

            if(fZipFile.filename.indexOf(".png")<0) continue;

            filename = fZipFile.filename.split("/")[1];

            filePath = baseUrl.replace(Constants.IMAGES_URL, "Cars/")+filename;

            file = File.cacheDirectory.resolvePath(filePath);
            file.preventBackup = true;

            //trace("filePath:", filePath)


            ImagesBytesList[filePath]=fZipFile.content;
            totalCountImages++;
            //if(file.exists) file.deleteFile();

            BitmapDataUtil.saveImageBytes(fZipFile.content, file);
            //replace(Constants.IMAGES_URL, "Cars/")

            //trace("fZipFile.filename:",fZipFile.filename);
        }

        fZip.close();
        zipCountLoaded++;

        var progress:Number = zipCountLoaded/zipCountToLoad;

        //trace("onFZipComplete count:",fZip.getFileCount(), progress, getTimer()-data.time);

        if(progress==1){

            _downloadZipComplete();
            _downloadZipComplete = null;
            fZipArray=null;

            trace("totalCountImages:", totalCountImages);
        }




    }

    private function onFzipFileLoaded(event:FZipEvent):void{



        //event.


    }*/


    private function handleCarLoadedStep(progress:Number) : void{
        tuningScreen.progressFlatBar.progress = progress;
        tuningScreen.updateProgressPosition();
    }





    private var blackFirstFrame:ImageLoaderWithStates = null;
    private function handleCarLoaded(loadedCar : Car) : void {
        Tracer.print("TuningScreenMediator handleCarLoaded");
        car = loadedCar;
        var i:int;

        if (!Device.isIos() && Constants.DISABLED_CAMBER_FOR_ANDROID_TRUCK_SUV) {

            PartDiskItemRenderer.camberAllowForCar = !(car.type == "Truck" || car.type == "Suv");
        } else {

            PartDiskItemRenderer.camberAllowForCar = true;
        }


        tuningScreen.lockMenu(true);
        tuningScreen.carView.locked = true;
        tuningScreen.carView.visible = false;

        var allPartSlot:PartSlot = new PartSlot(car);
        allPartSlot.name = "ALL";
        allPartSlot.displayName = "ALL";
        allPartSlot.dummy = true;
        allPartSlot.sortKey = 0;
        allPartSlot.bestViewFrameIndex = 9;
        car.editablePartSlots.unshift(allPartSlot);


        var ps:PartSlot = null;
        var part:Part = new Part(allPartSlot);

        var bodySelectedMaterial:PartMaterial = null;

        for (var k:int = 0; k < car.editablePartSlots.length; k++) {

            ps = car.editablePartSlots[k];

            if (ps && ps.name == "body" && ps.selectedPart && ps.selectedPart.selectedMaterial) {

                bodySelectedMaterial = ps.selectedPart.selectedMaterial;
                break;

            }
        }
        var selectedMaterialName:String = "glossy";
        if (bodySelectedMaterial)
            selectedMaterialName = bodySelectedMaterial.name;

        for (i = 0; i < car.editablePartSlots.length; i++) {

            ps = car.editablePartSlots[i];


            if (ps && ps.selectedPart) {
                for (var j:int = 0; j < ps.selectedPart.possibleMaterials.length; j++) {


                    var material:PartMaterial = ps.selectedPart.possibleMaterials[j];

                    if (!part.possibleMaterialsMap[material.name]) {
                        if (material.name!="mask" && material.name!="antimssk"/*material.name == "glossy" || material.name == "mate" || material.name == "white" || material.name == "redblack" || material.name == "metal"*/) {

                            var m1:PartMaterial = new PartMaterial(part);
                            m1.name = material.name;
                            //m1.isPaintable = material.isPaintable;
                            part.addPossibleMaterial(m1);

                            if (selectedMaterialName == material.name) {

                                part.selectedMaterial = m1;
                            }
                        }
                    }

                }


            }

        }

        allPartSlot.selectedPart = part;
        var windowsPartSlot:PartSlot = car.getPossiblePartSlot("windows");

        if (windowsPartSlot) {


            if(windowsPartSlot.possibleParts.length<=1){
                var windowsParts:Array = windowsPartSlot.possibleParts.slice();

                for each(var windowPart:Part in windowsParts) {

                    var percents:Array = [0.7, 0.4, 0.15, 0.0];
                    windowPart.brightness = 1.0;

                    var sortKey:int = 1;
                    windowPart.sortKey = sortKey;

                    for (i = 0; i < percents.length; i++) {
                        var newWindowPart:Part = new Part(windowsPartSlot);
                        sortKey++;

                        newWindowPart.id = windowPart.id;
                        newWindowPart.brightness = percents[i];
                        newWindowPart.sortKey = sortKey;
                        //windowPart.brightness = percents[i];
                        //newWindowPart.imageUrl = "http://www.gettyicons.com/download/?id=4235&t=png&s=256";
                        newWindowPart.name = windowPart.name + " " + int(percents[i] * 100) + "%";

                        var partMaterial:PartMaterial = new PartMaterial(newWindowPart);

                        if(!windowPart.selectedMaterial) windowPart.selectedMaterial = windowPart.getPossibleMaterial("glossy");
                        //partMaterial.countImages =  windowPart.selectedMaterial.countImages;
                        partMaterial.name =  windowPart.selectedMaterial._name;
                        //partMaterial.isPaintable =  windowPart.selectedMaterial.isPaintable;
                        partMaterial.parentPart =  newWindowPart;
                        partMaterial.componentPath =   windowPart.selectedMaterial.componentPath;
                        //partMaterial.imagesMap = windowPart.selectedMaterial.imagesMap;

                        for(var angelIndex:int in windowPart.selectedMaterial.imagesMap){

                            if(!windowPart.selectedMaterial.imagesMap[angelIndex]) continue;

                            for(var frameIndex:int in windowPart.selectedMaterial.imagesMap[angelIndex]){

                                if(!windowPart.selectedMaterial.imagesMap[angelIndex][frameIndex]) continue;

                                var partMaterialImage:PartMaterialImage = windowPart.selectedMaterial.imagesMap[angelIndex][frameIndex];

                                var newPartMaterialImage:PartMaterialImage = PartMaterialImagePool.getInstance(null, null, null, partMaterial);

                                newPartMaterialImage.x = partMaterialImage.x;
                                newPartMaterialImage.y = partMaterialImage.y;
                                newPartMaterialImage.frameIndex = partMaterialImage.frameIndex;
                                newPartMaterialImage.angleIndex = partMaterialImage.angleIndex;

                                newPartMaterialImage.parentMaterial = partMaterial;

                                if(!partMaterial.imagesMap[angelIndex]) partMaterial.imagesMap[angelIndex] = {};

                                partMaterial.imagesMap[angelIndex][frameIndex] = newPartMaterialImage;


                            }

                        }

                        newWindowPart.addPossibleMaterial(partMaterial);
                        newWindowPart.selectedMaterial =partMaterial;
                        newWindowPart.imageUrl = windowPart.imageUrl;
                        newWindowPart.folder = windowPart.folder;
                        newWindowPart.parentSlot = windowPart.parentSlot;
                        newWindowPart.subparts = windowPart.subparts;
                        newWindowPart.isSubpart = windowPart.isSubpart;

                        windowsPartSlot.addPossiblePart(newWindowPart);

                        if (windowsPartSlot.selectedPart && windowsPartSlot.selectedPart.brightness == percents[i])
                        //	if(windowsPartSlot.selectedPart.brightness == percents[i])
                        {
                            windowsPartSlot.selectedPart = newWindowPart;

                        }
                    }
                }
            }
        }


        /*var suspensionPartSlot:PartSlot = new PartSlot(car);
        suspensionPartSlot.name = "Suspension";
        suspensionPartSlot.displayName = "Suspension";
        suspensionPartSlot.dummy = false;
        suspensionPartSlot.sortKey = 999999;
        car.editablePartSlots.unshift(suspensionPartSlot);*/



        if (tuningScreen.car) {

            car = car;
            car.challengeData = tuningScreen.car.challengeData;
        }
        tuningScreen.car = car;
        tuningScreen.possiblePartSlots = car.editablePartSlots;

        tuningScreen.carView.car = car;
        var blackPartSlot:PartSlot = car.getPossiblePartSlot("black");

        if (!blackPartSlot) {

            blackPartSlot = car.getPossiblePartSlot("body");
        }

        var blackPartMaterial:PartMaterial = blackPartSlot.selectedPart.possibleMaterials[0];
        var blackPartMaterialImage:PartMaterialImage = blackPartMaterial.getImage(0);
        /* var prevPath:String = blackPartMaterialImage.parentMaterial.parentPart.parentSlot.parentCar.imagePath;
        blackPartMaterialImage.parentMaterial.parentPart.parentSlot.parentCar.imagePath += ".jpg"; */
        var blackFirstFrameAttempts:int = 0;
        blackFirstFrame = new ImageLoaderWithStates();
        blackFirstFrame.overrideDispose = false;
        blackFirstFrame.snapToPixels = false;
        blackFirstFrame.carPart = true;

        blackFirstFrame.visible = false;
        blackFirstFrame.errorHandler = function ():void {
            if(++blackFirstFrameAttempts < 4) {
                blackFirstFrame.startLoading();
            }else{
                // blackPartMaterialImage.parentMaterial.parentPart.parentSlot.parentCar.imagePath = prevPath;
                blackFirstFrame.recieveImageSize(770, 412);
            }
        };


        if(!car.partMaterialZip)
            handleCarLoadedStep(0.01);


        tuningScreen.addChild(blackFirstFrame);
        blackFirstFrame.recieveImageSize = function (image_width:Number, image_height:Number):void {

            Tracer.print("recieveImageSize");
            tuningScreen.carView.calculateHeight(image_width, image_height);

            blackFirstFrame.removeFromParent(true);

            tuningScreen.carView.frameHeight = tuningScreen.frameImage.height;

            tuningScreen.updateCarViewY();


            function orderSortKey(a:*, b:*):int {
                var a_i:int = int(a.layerIndex);
                var b_i:int = int(b.layerIndex);

                if (a_i < b_i) return -1;
                if (a_i > b_i) return 1;

                return 0;
            }

            var result:Array = car.selectedPartSlots.sort(orderSortKey);
            var newPartSlots:Array = [];
            for (var index:int in result) {
                var partSlot:PartSlot = result[index];

                newPartSlots.push(partSlot);

            }

            if (BrandsScreen.selectedMaterialName ) {

                var materialName:String = BrandsScreen.selectedMaterialName;

                BrandsScreen.selectedCar = null;
                BrandsScreen.selectedCarId = null;

                setMaterialForPartSlots(newPartSlots, materialName);


            }


            if(!car.partMaterialZip) handleCarLoadedStep(0.03);

            var secondCamLoadFinished : Function = function () : void
            {
                tuningScreen.setPartListTouchable(true);
                tuningScreen.lockMenu(false);
                tuningScreen.carView.locked = false;
                car.countAngels = 2;
                tuningScreen.carView.updateSuspension();
                tuningScreen.carView.cameraSecondCameraIsLoading = false;
            };


            var cameraAngelsToLoadFirst : Array  = [0];



            var onFullLoadComplete : Function = function () :void{

                tuningScreen.enableButtons(true);
                AdsManager.$.bannerEnabled = true;
                if ((car.packet && car.packet.type != Packet.TYPE_SUBCOMPONENT && car.packet.lock)) {
                    //saveButton.//push(saveButton);
                    /*tuningScreen.saveButton.alpha = 0.5;
                    tuningScreen.saveButton.touchable = false;*/
                    tuningScreen.setLockIcon();
                }

                ImageLoaderWithStates.disposeWhenRemovedFromStage = true;

                if(car.countAngels > cameraAngelsToLoadFirst.length) // check if there is second camera, need to download it
                {
                    Starling.juggler.delayCall(function () :void {
                        tuningScreen.setPartListTouchable(false);
                        car.countAngels = 1;
                        tuningScreen.carView.cameraSecondCameraIsLoading = true;
                        tuningScreen.carView.updateWithPartSlots(newPartSlots,false, secondCamLoadFinished, [1], false); // passing [1] we are indicating the second camera index
                    }, 0.1)
                }
                else {
                    tuningScreen.lockMenu(false);
                    tuningScreen.carView.locked = false;
                }

                if (car.packet && car.packet.tempOpened && car.packet.oneTimeUnlocked) {
                    removeOneTimeUnlockPacket(car.packet);
                }
            };

            /**
             * VERY IMPORTANT, we are passing [0] parameter, which will allow to download only the first camera, and show the car loaded
             *
             * after we will download the second camera on background
             */
            tuningScreen.carView.updateWithPartSlots(newPartSlots, false, onFullLoadComplete ,cameraAngelsToLoadFirst);


            for each(var selectedPartSlot:PartSlot in car.selectedPartSlots) {
                var isPack:Boolean = (selectedPartSlot.name == PartSlot.BODY_PACK || selectedPartSlot.name == PartSlot.FACTORY_TRIM);
                if (isPack && selectedPartSlot.selectedPart && selectedPartSlot.selectedPart.excludedPartSlotsNames && selectedPartSlot.selectedPart.excludedPartSlotsNames.indexOf("disks") >= 0) {
                    tuningScreen.setSuspensionControlEnabled(false);
                    SuspensionControlEnabledFlag = false;
                }
            }
            if(!tuningScreen.car.selectedBackground && tuningScreen.tunning && tuningScreen.tunning.backgroundId){
                var selectedBackground:CarBackground = new CarBackground();
                selectedBackground.id = tuningScreen.tunning.backgroundId;
                selectedBackground.isCustom = true;
                selectedBackground.mainUrl = tuningScreen.tunning.customSnapshotsBackgrounds[0];
                selectedBackground.showThumbnailUrl = false;
                tuningScreen.car.selectedBackground = selectedBackground;
            }

            tuningScreen.carView.updateBackground();

            if (BrandsScreen.selectedMaterialName) {
                tuningScreen.selectedPartSlotIndex = 0;
                tuningScreen.showColorMenuPanel(false);

                BrandsScreen.selectedMaterialName = null;
            } else {
                tuningScreen.selectedPartSlotIndex = 0;
                tuningScreen.showColorMenuPanel(false);
            }
            carLoaded = true;
        };


        blackFirstFrame.persistentImageProvider = blackPartMaterialImage;

    }

    public function removeOneTimeUnlockPacket(unlockedPacket:Packet):void {
        GameService.$.getSaveData(function(data: String): void {
            var saveData: Object = JSONer.parse(data);
            var userId:String = UsersProxy.loggedInUser().id;
            var packetId:String = unlockedPacket.id;
            var oneTimeUnlocked:Object = (saveData.hasOwnProperty(userId) && saveData[userId].hasOwnProperty("oneTimeUnlocked")?saveData[userId].oneTimeUnlocked:{});
            if(oneTimeUnlocked[packetId]){
                delete oneTimeUnlocked[packetId];
            }
            GameService.$.saveGameData(JSON.stringify(saveData));
        });
    }

    private function tuningScreenPartSlotSelectedHandler(event:starling.events.Event):void {
        if (tuningScreen.selectedPartSlot.selectedPart) {
            if (!tuningScreen.selectedPartSlot.selectedPart.selectedMaterial && tuningScreen.selectedPartSlot.selectedPart.linkedParts && tuningScreen.selectedPartSlot.selectedPart.linkedParts.length > 0) {
                var isPaintable:Boolean = false;
                if (tuningScreen.selectedPartSlot.selectedPart.linkedParts) {
                    for (var i:int = 0; i < tuningScreen.selectedPartSlot.selectedPart.linkedParts.length; i++) {
                        var linkedPart:Part = tuningScreen.selectedPartSlot.selectedPart.linkedParts[i];
                        if (linkedPart.possibleMaterials) {
                            for (var j:int = 0; j < linkedPart.possibleMaterials.length; j++) {
                                var linkedPartMaterial:PartMaterial = linkedPart.possibleMaterials[j];
                                if (linkedPartMaterial && linkedPartMaterial.isPaintable) {
                                    isPaintable = true;
                                    break;
                                }
                            }
                        }

                        if (isPaintable) {
                            break;
                        }
                    }
                }
                tuningScreen.colorsViewEnabled = isPaintable;
            } else if(tuningScreen.selectedPartSlot.selectedPart.selectedMaterial && tuningScreen.tuningColorsView){
                tuningScreen.allPartsColorsView.isEnabled = tuningScreen.allPartsColorsView.hasPaintMaterial; //tuningScreen.selectedPartSlot.selectedPart.selectedMaterial.isPaintable;
            }
        } else {
            tuningScreen.colorsViewEnabled = false;
        }

        if (firstPartSlotSelectionMade && !CarView.isBackgroundPart(tuningScreen.selectedPartSlot.name)) {
            rotateToBestView();
        }

        if(tuningColorPicker){
            tuningColorPicker.partSlot = tuningScreen.selectedPartSlot;
        }
        firstPartSlotSelectionMade = true;
    }

    private function rotateToBestView(resetDefault:Boolean = false):void{
        if(!tuningScreen.rotationEnabled){
            return;
        }
        Console.log("rotateToBestView", tuningScreen.selectedPartSlot.name);
        var selectedPartSlot:PartSlot = tuningScreen.selectedPartSlot;
        var nextAngle:int = (selectedPartSlot.bestViewFrameIndex > 25? 1: 0);
        if (tuningScreen.carView.previewMode && tuningScreen.isMenuOpen) {
            if(selectedPartSlot.name == TuningColorsView.EXTERIOR){
                if(tuningScreen.car.getPossiblePartSlot(PartSlot.BODY_PACK)){
                    selectedPartSlot = tuningScreen.car.getPossiblePartSlot(PartSlot.BODY_PACK);
                }else if(tuningScreen.car.getPossiblePartSlot(PartSlot.BODY)){
                    selectedPartSlot = tuningScreen.car.getPossiblePartSlot(PartSlot.BODY);
                }
            }else if(selectedPartSlot.name == TuningColorsView.INTERIOR){
                var interiorPartSlots:Array = tuningScreen.car.interiorSelectedPartSlots;
                if(interiorPartSlots.length){
                    selectedPartSlot = interiorPartSlots[0] as PartSlot;
                }
            }
            var bestViewFrameIndex:int;
            if(tuningScreen.carFastModeData && tuningScreen.carFastModeData.hasOwnProperty(selectedPartSlot.name) && tuningScreen.carFastModeData[selectedPartSlot.name].hasOwnProperty("rollToFrame")){
                bestViewFrameIndex = tuningScreen.carFastModeData[selectedPartSlot.name]["rollToFrame"];
            }else{
                bestViewFrameIndex = selectedPartSlot.bestViewFrameIndex;
            }
            nextAngle = (bestViewFrameIndex > 25? 1: 0);

            var carTransform:Object;
            if(tuningScreen.carFastModeData && tuningScreen.carFastModeData.hasOwnProperty(selectedPartSlot.name) && tuningScreen.carFastModeData[selectedPartSlot.name].hasOwnProperty("carTransform")){
                carTransform = tuningScreen.carFastModeData[selectedPartSlot.name]["carTransform"];
                carTransform["zoomLevel"] ||= 1;
                carTransform["pan"] ||= {"x": 0,"y": 0};
            }else{
                carTransform = {"zoomLevel":1,"pan":{"x": 0,"y": 0}};
            }
            var div:Number = tuningScreen.carView.offsetScale * carTransform.zoomLevel;
            var nextFrame:int = tuningScreen.carView.bestIndexToFrameIndex(bestViewFrameIndex);
            if(tuningScreen.carView.currentFrame != nextFrame || tuningScreen.carView.currentAngelIndex != nextAngle) {
                tuningScreen.carView.offset.setTo(carTransform.pan.x * div, carTransform.pan.y * div);
                tuningScreen.carView.carScale = carTransform.zoomLevel;
                tuningScreen.carView.updatePosition();
                tuningScreen.carView.updateSuspension();

                tuningScreen.carView.loadSelectedFrame = bestViewFrameIndex;
                var prevFrame:int = tuningScreen.carView.currentFrame;
                if(tuningScreen.carView.fastModeLoadedFrames && tuningScreen.carView.fastModeLoadedFrames[nextAngle] && tuningScreen.carView.fastModeLoadedFrames[nextAngle][nextFrame]){
                    tuningScreen.carView.beforeLoading(false);
                    tuningScreen.carView.showFrame(nextFrame, nextAngle);
                    var newPartSlots:Array = tuningScreen.carView.fastModeLoadedPartSlots;
                    var colArr:Array = [];
                    for(var i:int = 0; i < newPartSlots.length; i++){
                        var partSlot:PartSlot = newPartSlots[i] as PartSlot;
                        if(partSlot.name != PartSlot.BODY_PACK && partSlot.name != PartSlot.FACTORY_TRIM){
                            colArr.push(partSlot);
                        }
                    }
                    tuningScreen.carView.updateWithPartSlots(colArr, true);
                    tuningScreen.carView.afterLoading();
                    return;
                }
                var prevAngle:int = tuningScreen.carView.currentAngelIndex;
                tuningScreen.carView.currentAngelIndex = nextAngle;
                loadCarFullFrame(bestViewFrameIndex);
                tuningScreen.carView.currentFrame = prevFrame;
                tuningScreen.carView.currentAngelIndex = prevAngle;
                tuningScreen.carView.showFrame(tuningScreen.carView.loadSelectedFrame, nextAngle);
            }else{
                tuningScreen.carView.beforeLoading(false);
                tuningScreen.carView.offset.setTo(carTransform.pan.x * div, carTransform.pan.y * div);
                tuningScreen.carView.carScale = carTransform.zoomLevel;
                tuningScreen.carView.updatePosition();
                tuningScreen.carView.updateSuspension();
                tuningScreen.carView.afterLoading();
            }
        }else if(resetDefault){
            tuningScreen.carView.beforeLoading(false);
            tuningScreen.carView.offset.setTo(0, 0);
            tuningScreen.carView.carScale = 1;
            tuningScreen.carView.updatePosition();
            tuningScreen.carView.updateSuspension();

            tuningScreen.carView.showFrame(tuningScreen.carView.currentFrame, nextAngle);
            tuningScreen.carView.removeSnapshot(0.5);
            // tuningScreen.carView.rotateToFrame2(0);
            Console.log("rotateToBestView# rotateToFrame2", tuningScreen.selectedPartSlot.name);
        }else {
            tuningScreen.carView.showFrame(tuningScreen.carView.currentFrame, nextAngle);
            tuningScreen.carView.removeLoadingCar();
            tuningScreen.carView.rotateToFrame2(selectedPartSlot.bestViewFrameIndex);
            Console.log("rotateToBestView# rotateToFrame2", tuningScreen.selectedPartSlot.name);
        }
    }

    private function loadCarFullFrame(frameIndex:int = -1):void {
        tuningScreen.carView.loadSelectedFrame = frameIndex;
        tuningScreen.carView.currentFrame = tuningScreen.carView.loadSelectedFrame;
        tuningScreen.carView.setFastModeCurrentLoaded();

        function orderSortKey(a:*, b:*):int {
            var a_i:int = int(a.layerIndex);
            var b_i:int = int(b.layerIndex);

            if (a_i < b_i) return -1;
            if (a_i > b_i) return 1;

            return 0;
        }

        var newPartSlots:Array = tuningScreen.carView.fastModeLoadedPartSlots.sort(orderSortKey);
        if (BrandsScreen.selectedMaterialName ) {
            var materialName:String = BrandsScreen.selectedMaterialName;
            setMaterialForPartSlots(newPartSlots, materialName);
        }

        var cameraAngelsToLoadFirst : Array  = [tuningScreen.carView.currentAngelIndex];
        var fullFrameLoadCompleteHandler : Function = function () :void{
            ImageLoaderWithStates.disposeWhenRemovedFromStage = true;
            tuningScreen.carView.updateBackground();
            tuningScreen.carView.updateCarY();

            tuningScreen.carView.locked = false;
            tuningScreen.carView.afterLoading();
        };
        tuningScreen.carView.beforeLoading();

        var carProxy:CarsProxy =  carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
        carProxy.loadComponent(newPartSlots, function(components:Array):void{
            tuningScreen.carView.updateWithPartSlots(newPartSlots, false, fullFrameLoadCompleteHandler, cameraAngelsToLoadFirst);
        });

        for each(var selectedPartSlot:PartSlot in car.selectedPartSlots) {
            var isPack:Boolean = (selectedPartSlot.name == PartSlot.BODY_PACK || selectedPartSlot.name == PartSlot.FACTORY_TRIM);
            if (isPack && selectedPartSlot.selectedPart && selectedPartSlot.selectedPart.excludedPartSlotsNames
                    && selectedPartSlot.selectedPart.excludedPartSlotsNames.indexOf("disks") >= 0) {
                tuningScreen.setSuspensionControlEnabled(false);
                SuspensionControlEnabledFlag = false;
            }
        }
    }

    private function tuningScreenExitFastModeHandler(event:starling.events.Event = null):void{
        exitFastModeAndReload();
    }

    private function tuningScreenEnterFastModeHandler(event:starling.events.Event = null):void{
        loadFastModeData();
    }

    private function loadFastModeData():void{
        if(tuningScreen.carFastModeData){
            if(tuningScreen.rotationEnabled){
                rotateToBestView();
            }
        }else{
            var carProxy:CarsProxy =  carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
            carProxy.loadFastModeData(car.id, function(fastModeData:Object):void{
                tuningScreen.carFastModeData = ObjectUtils.getProperty(fastModeData, "fastModeSettings.byCompType")||fastModeData;
                rotateToBestView();
            });
        }
    }

    private function exitFastModeAndReload():void {
        if(tuningScreen.carView.fastModeLoadedPartSlots.length == 0){
            tuningScreen.enableButtons(true);
            tuningScreen.lockMenu(false);
            tuningScreen.carView.locked = false;
            rotateToBestView(true);
            if(fastModeExitHandler != null){
                fastModeExitHandler();
            }
            return;
        }

        function orderSortKey(a:*, b:*):int {
            var a_i:int = int(a.layerIndex);
            var b_i:int = int(b.layerIndex);

            if (a_i < b_i) return -1;
            if (a_i > b_i) return 1;

            return 0;
        }

        var newPartSlots:Array = tuningScreen.carView.fastModeLoadedPartSlots.sort(orderSortKey);
        if (BrandsScreen.selectedMaterialName ) {
            var materialName:String = BrandsScreen.selectedMaterialName;
            setMaterialForPartSlots(newPartSlots, materialName);
        }
        this.tuningScreen.carView.beforeLoading();
        if(this.tuningScreen.carView.progressHandler){
            this.tuningScreen.carView.progressHandler(0);
            this.tuningScreen.carView.progressHandler(0.01);
        }

        var carProxy:CarsProxy =  carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
        carProxy.loadComponent(newPartSlots, function(components:Array):void{
            Starling.juggler.delayCall(function():void{
                tuningScreen.carView.updateWithPartSlots(newPartSlots, false, function():void{
                    tuningScreen.carView.updateBackground();
                    tuningScreen.carView.updateCarY();

                    ImageLoaderWithStates.disposeWhenRemovedFromStage = true;
                    tuningScreen.lockMenu(false);

                    tuningScreen.enableButtons(true);
                    tuningScreen.carView.locked = false;
                    tuningScreen.carView.afterLoading();
                    rotateToBestView(true);

                    while(newPartSlots.length){
                        newPartSlots.pop();
                    }
                    newPartSlots = null;
                    //
                    while (tuningScreen.carView.fastModeLoadedPartSlots.length){
                        tuningScreen.carView.fastModeLoadedPartSlots.pop();
                    }
                    if(fastModeExitHandler != null){
                        Starling.juggler.delayCall(fastModeExitHandler, 0.25);
                    }
                });
            }, 1);

        });
    }


    private var isPartSelectedChangeData:Boolean = false;

    private function tuningScreenPartSelectedChangeDataHandler(event:starling.events.Event):void {


        if (!carLoaded) {
            Tracer.print("CAR DOESN'T LOADED", "tuningScreenPartSelectedChangeDataHandler");
            return;
        }

        isPartSelectedChangeData = true;

        // TODO need to uncomment/delete this part
        if(tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart){
            tuningScreen.carView.removeImagesOfPartSlotName("masks_"+tuningScreen.selectedPartSlot.selectedPart.id, -1);

            var antimaskPartMaterial: PartMaterial =  tuningScreen.selectedPartSlot.selectedPart.getPossibleMaterial("antimask");
            if (antimaskPartMaterial) {
                tuningScreen.carView.removeImagesOfPartSlotName("antimask_" + tuningScreen.selectedPartSlot.selectedPart.id, -1);
            }
        }


        /*if (tuningScreen.carView.partSlotMask) {
            tuningScreen.carView.removePartSlot(tuningScreen.carView.partSlotMask);
            tuningScreen.carView.partSlotMask.selectedPart = null;
        }*/


        tuningScreenPartSelectedHandler();

        isPartSelectedChangeData = false;


    }

    private function tuningScreenPartDiskInfoPopUpHandler(event:starling.events.Event):void {
        var packet:Packet = event.data.packet as Packet;
        sendNotification(Notifications.DISPLAY_INFO_POP_UP, {packet: packet});
    }

    private function findMaterialInSubpart(part:Part, materialName:String):PartMaterial {

        if (part && part.subparts && part.subparts.length > 0) {


            for (var k:int = 0; k < part.subparts.length; k++) {


                var subpart:Part = part.subparts[k];

                if (subpart.possibleMaterials) {

                    for (var l:int = 0; l < subpart.possibleMaterials.length; l++) {

                        var mat:PartMaterial = subpart.possibleMaterials[l];

                        if (mat && mat.name == materialName) {

                            return mat;
                            ;
                        }

                    }

                }


            }


        }


        return null;
    }

    private var SuspensionControlEnabledFlag:Boolean = true;

    private function tuningScreenPartSelectedHandler(event:starling.events.Event = null):void {
        ErrorTracker.setReference("TuningScreenMediator/tuningScreenPartSelectedHandler");
        if (!carLoaded) {
            Console.log("CAR DOESN'T LOADED", "tuningScreenPartSelectedHandler");
            return;
        }
        var i:int;
        var prefix:String;
        var material:PartMaterial;
        var part:Part = (tuningScreen?tuningScreen.selectedPart:null);
        ErrorTracker.setLineLabel("#1");
        var addOnRemoveComponents:Array = [];
        if(!part){
            return;
        }
        ErrorTracker.setLineLabel("#2");

        if (tuningScreen.infoView && tuningScreen.infoView.isOpened) {
            lastComponent = part;
            onPartInfoPopUp();
        }
        ErrorTracker.setLineLabel("#3");
        if(Car.DISKS_SLOTS.indexOf(tuningScreen.selectedPart.parentSlot.name) != -1){
            if(tuningScreen.selectedPart.parentSlot.name=="disks"){
                tuningScreen.frontWheelModified = tuningScreen.rearWheelModified = true;
            }else{
                if(tuningScreen.selectedPart.parentSlot.name.indexOf("front") != -1){
                    tuningScreen.frontWheelModified = true;
                }else{
                    tuningScreen.rearWheelModified = true;
                }
            }
        }
        ErrorTracker.setLineLabel("#4");
        var partSlot:PartSlot = part.parentSlot;
        if (part && part.possibleMaterials && !part.possibleMaterials.length && !part.selectedMaterial && !part.isBack && part.parentSlot.name != PartSlot.BODY_PACK && part.parentSlot.name != PartSlot.FACTORY_TRIM) {
            if (!part.linkedParts || !part.linkedParts.length){
                return;
            }
        }
        ErrorTracker.setLineLabel("#5");
        if (part.isBack) {
            return;

        }
        if(tuningScreen.carView && tuningScreen.carView.visible){
            tuningScreen.carView.beforeLoading(true, event && partSlot.name == "disks");
        }
        if(event &&  event.data && event.data.beforeRemovePartSlot){
            addOnRemoveComponents = removePartSlot(null, event.data.beforeRemovePartSlot).concat(addOnRemoveComponents);
        }
        ErrorTracker.setLineLabel("#6");
        if (car && car.selectedPartSlots.indexOf(partSlot) < 0) {
            car.addSelectedPartSlot(partSlot);
        }
        if (partSlot.selectedPart) {
            if (partSlot.selectedPart.selectedState && partSlot.selectedPart.selectedState.nameOfState && partSlot.name == "rear_lights") {
                if (part && part.states && part.states[partSlot.selectedPart.selectedState.nameOfState]){
                    part.selectedState = part.states[partSlot.selectedPart.selectedState.nameOfState];
                }
            }
            ErrorTracker.setLineLabel("#7");
            if(partSlot.name != "rear_lights" && partSlot.name != "front_lights" && part && part.states && part.states['lights']){
                var lightsSlot:PartSlot = car.getPossiblePartSlot("front_lights");
                lightsSlot ||= car.getPossiblePartSlot("rear_lights");
                if(lightsSlot && lightsSlot.selectedPart && lightsSlot.selectedPart.selectedState && lightsSlot.selectedPart.selectedState.nameOfState == "lights"){
                    part.selectedState = part.states[lightsSlot.selectedPart.selectedState.nameOfState];
                }
            }
            ErrorTracker.setLineLabel("#8");
            tuningScreen.carView.removeMaskImages(partSlot.selectedPart);
            ErrorTracker.setLineLabel("#9");
            if (tuningScreen.carView.partSlotMask) {
                if (part.id != partSlot.selectedPart.id) {
                    tuningScreen.carView.removeImagesOfPartSlotName("masks_" + partSlot.selectedPart.id, -1);
                    if (Car.CAR_DISKS_SLOTS.indexOf(partSlot.name)>=0) {
                        tuningScreen.carView.isExistCember = false;
                        tuningScreen.invalidate('car');
                    }
                }
            }
            ErrorTracker.setLineLabel("#10");
            if(partSlot.name==PartSlot.BODY_PACK){
                if(part.removeTypes.indexOf(PartSlot.BODY_PACK) == -1){
                    tuningScreen.carView.removeImagesOfPartSlotName(PartSlot.BODY_PACK, -1);
                }else if(part.removeTypes.indexOf(PartSlot.FACTORY_TRIM) == -1){
                    tuningScreen.carView.removeImagesOfPartSlotName(PartSlot.FACTORY_TRIM, -1);
                }else{
                    addOnRemoveComponents = removePartSlot(null, partSlot, true).concat(addOnRemoveComponents);
                }
            }else if(partSlot.name==PartSlot.FACTORY_TRIM){
                if(part.removeTypes.indexOf(PartSlot.FACTORY_TRIM) == -1){
                    tuningScreen.carView.removeImagesOfPartSlotName(PartSlot.FACTORY_TRIM, -1);
                }else if(part.removeTypes.indexOf(PartSlot.BODY_PACK) == -1){
                    tuningScreen.carView.removeImagesOfPartSlotName(PartSlot.BODY_PACK, -1);
                }else{
                    addOnRemoveComponents = removePartSlot(null, partSlot, true).concat(addOnRemoveComponents);
                }
            }
            ErrorTracker.setLineLabel("#11");
            if (partSlot.selectedPart) {
                var antimaskPartMaterial:PartMaterial = partSlot.selectedPart.getPossibleMaterial("antimask");
                if (antimaskPartMaterial) {
                    if (partSlot.selectedPart != part) {
                        tuningScreen.carView.removeImagesOfPartSlotName("antimask_" + partSlot.selectedPart.id, -1);
                    }
                }
            }
            ErrorTracker.setLineLabel("#12");
        }
        ErrorTracker.setLineLabel("#13");
        if ((partSlot.selectedPart != part && tuningScreen.carView.saveSelectedPart == null) || isPartSelectedChangeData) {
            if (partSlot.selectedPart) {
                var selectedMaterial:PartMaterial = partSlot.selectedPart.selectedMaterial;
                var mainPart:Part;
                if (!selectedMaterial) {
                    mainPart = partSlot.selectedPart.getMainPartFromLinked();
                    if (mainPart) {
                        selectedMaterial = mainPart.selectedMaterial;
                    }
                }
                ErrorTracker.setLineLabel("#14");
                if (selectedMaterial) {
                    if (part.possibleMaterials.length > 0) {
                        for each(material in part.possibleMaterials) {
                            if (/*partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && */ material.name == selectedMaterial.name) {
                                part.selectedMaterial = part.getPossibleMaterial(material.name);
                            }

                        }
                    } else {
                        mainPart = part.getMainPartFromLinked();
                        if (mainPart) {
                            for each(material in mainPart.possibleMaterials) {
                                if (/*partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && */ material.name == selectedMaterial.name) {
                                    part.selectedMaterial = mainPart.getPossibleMaterial(material.name);
                                }
                            }
                            if (!part.selectedMaterial && mainPart.possibleMaterials && mainPart.possibleMaterials.length > 0) {

                                part.selectedMaterial = mainPart.possibleMaterials[0];
                            }
                        }
                    }
                }try{

                } catch (error:Error) {
                    Tracer.print("Error selected material:"+ error.message+ error.getStackTrace());
                }
                ErrorTracker.setLineLabel("#15");
            }
            ErrorTracker.setLineLabel("#16");
            var excludedPartSlotsToAdd:Array = [];
            if (partSlot.selectedPart) {
                if (partSlot.selectedPart && partSlot.selectedPart.linkedParts.length > 0) {
                    var updateSlots:Array = [];
                    var linkedPart1:Part;
                    ErrorTracker.setLineLabel("#17");
                    for (i = 0; i < partSlot.selectedPart.linkedParts.length; i++) {
                        linkedPart1 = partSlot.selectedPart.linkedParts[i];
                        if (!linkedPart1)
                            continue;
                        if (!linkedPart1.parentSlot)
                            continue;
                        ErrorTracker.setLineLabel("#18");
                        if(Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0 && (Car.BRAKES_SLOTS.indexOf(linkedPart1.parentSlot.name) >= 0 || Car.TIRES_SLOTS.indexOf(linkedPart1.parentSlot.name) >= 0 || Car.TIRE_VINYL_PARTS.indexOf(linkedPart1.parentSlot.name) >= 0)) { // if parent of linked is disk, so we need to ignore tires
                            continue;
                        }
                        tuningScreen.carView.removeMaskImages(linkedPart1.parentSlot.selectedPart);
                        ErrorTracker.setLineLabel("#19");
                        if (!car.isBeetle && linkedPart1) {
                            linkedPart1.parentSlot.selectedPart = linkedPart1;
                        }
                        /*if (!car.isBeetle && linkedPart1.parentSlot.savePartLinked) {
                            linkedPart1.parentSlot.selectedPart = linkedPart1.parentSlot.savePartLinked;
                        }*/
                        //TODO DEP
                        ErrorTracker.setLineLabel("#20");
                        if(Car.DISKS_SLOTS.indexOf(linkedPart1.name) >= 0){
                            tuningScreen.carView.diskRadiusSync(linkedPart1.parentSlot, linkedPart1, excludedPartSlotsToAdd);
                        }else if(Car.BRAKES_SLOTS.indexOf(linkedPart1.parentSlot.name) >= 0){
                            linkedPart1.parentSlot.selectedPart = tuningScreen.carView.brakeDiskSync(linkedPart1.parentSlot);
                        }else if(Car.TIRES_SLOTS.indexOf(linkedPart1.parentSlot.name) >= 0){
                            linkedPart1.parentSlot.selectedPart = tuningScreen.carView.tireDiskSync(linkedPart1.parentSlot);
                        }else if(Car.TIRE_VINYL_PARTS.indexOf(linkedPart1.parentSlot.name) >= 0){
                            linkedPart1.parentSlot.selectedPart = tuningScreen.carView.vinylDiskSync(linkedPart1.parentSlot);
                        }
                        ErrorTracker.setLineLabel("#21");
                        linkedPart1.parentSlot.isExcluded = false;
                        excludedPartSlotsToAdd.push(linkedPart1.parentSlot);
                        if (!car.isBeetle) {
                            linkedPart1.parentSlot.savePartLinked = null;
                        }
                        ErrorTracker.setLineLabel("#22");
                    }
                }
            }
            ErrorTracker.setLineLabel("#23");
            //Перебераем что нужно восстановить
            if (partSlot.selectedPart && partSlot.selectedPart.excludedPartSlotsNames) {
                for each(var excludedPartSlotName:String in partSlot.selectedPart.excludedPartSlotsNames) {
                    var excludedPartSlot:PartSlot = car.getPossiblePartSlot(excludedPartSlotName);
                    if (excludedPartSlot && excludedPartSlot.isExcluded) {
                        excludedPartSlot.isExcluded = false;
                        if (!excludedPartSlot.selectedPart) {
                            excludedPartSlot.selectedPart = excludedPartSlot.oldBeforeExludeSelectedPart;
                            excludedPartSlot.oldBeforeExludeSelectedPart = null;
                        }
                        excludedPartSlotsToAdd.push(excludedPartSlot);
                    }
                }
            }
            ErrorTracker.setLineLabel("#24");
            var arrOfPartSlotForUpdate:Array = excludedPartSlotsToAdd;
            var tirePair : PartSlot;
            var diskPair : PartSlotDisk;
            var oppositePairs : Array;
            var tires : Array = [];
            var vinyls : Array = [];
            var brakePair : PartSlot;
            var radius:int;
            if(car.suvTires){
                switch (partSlot.name){
                    case "disk_front_left":
                        diskPair = car.getPossiblePartSlot("disk_front_right") as PartSlotDisk;
                        oppositePairs = [car.getPossiblePartSlot("disk_rear_right"),car.getPossiblePartSlot("disk_rear_left")];
                        ErrorTracker.setLineLabel("#25");
                        break;
                    case "disk_front_right":
                        diskPair = car.getPossiblePartSlot("disk_front_left") as PartSlotDisk;
                        oppositePairs = [car.getPossiblePartSlot("disk_rear_right"),car.getPossiblePartSlot("disk_rear_left")];
                        ErrorTracker.setLineLabel("#26");
                        break;
                    case "disk_rear_left":
                        diskPair = car.getPossiblePartSlot("disk_rear_right") as PartSlotDisk;
                        oppositePairs = [car.getPossiblePartSlot("disk_front_left"),car.getPossiblePartSlot("disk_front_right")];
                        ErrorTracker.setLineLabel("#27");
                        break;
                    case "disk_rear_right":
                        diskPair = car.getPossiblePartSlot("disk_rear_left") as PartSlotDisk;
                        oppositePairs = [car.getPossiblePartSlot("disk_front_left"),car.getPossiblePartSlot("disk_front_right")];
                        ErrorTracker.setLineLabel("#28");
                        break;
                    case "tire_front_left":
                    case "tire_front_right":
                    case "tire_rear_right":
                    case "tire_rear_left":
                        tires = car.possiblePartSlotsTires;
                        ErrorTracker.setLineLabel("#29");
                        break;
                    case "tire_vinyl_front_left":
                    case "tire_vinyl_front_right":
                    case "tire_vinyl_rear_left":
                    case "tire_vinyl_rear_right":
                        vinyls = car.possiblePartSlotsVinyls;
                        ErrorTracker.setLineLabel("#30");
                        break;
                    case "brake_front_left":
                        brakePair = car.getPossiblePartSlot("brake_front_right");
                        ErrorTracker.setLineLabel("#31");
                        break;
                    case "brake_front_right":
                        brakePair = car.getPossiblePartSlot("brake_front_left");
                        ErrorTracker.setLineLabel("#32");
                        break;
                    case "brake_rear_right":
                        brakePair = car.getPossiblePartSlot("brake_rear_left");
                        ErrorTracker.setLineLabel("#33");
                        break;
                    case "brake_rear_left":
                        brakePair = car.getPossiblePartSlot("brake_rear_right");
                        ErrorTracker.setLineLabel("#34");
                        break;
                    case "tires_rear":
                        tirePair = car.getPossiblePartSlot("tires_front");
                        diskPair = car.getPossiblePartSlot("disks_front")as PartSlotDisk;
                        ErrorTracker.setLineLabel("#35");
                        break;
                    case "tires_front":
                        tirePair = car.getPossiblePartSlot("tires_rear");
                        diskPair = car.getPossiblePartSlot("disks_rear") as PartSlotDisk;
                        ErrorTracker.setLineLabel("#36");
                        break;
                }
                ErrorTracker.setLineLabel("#37");
                if(brakePair) {
                    if (brakePair.selectedPart) {
                        tuningScreen.carView.removeMaskImages(brakePair.selectedPart);
                    }
                    ErrorTracker.setLineLabel("#38");
                    diskPair = car.getPossiblePartSlot(partSlot.name.replace("brake", "disk") ) as PartSlotDisk;
                    if (brakePair.selectedPart && brakePair.selectedPart.properties && brakePair.selectedPart.properties.dependence && brakePair.selectedPart.properties.dependence.length >= 1) {
                        brakePair.selectedPart = brakePair.getPossiblePartForAGivenNameAndRadius(part.name, (diskPair.selectedPart as PartDisk).selectedDiskSize, brakePair.selectedPart.properties.dependence);
                    } else if(diskPair.selectedPart){
                        brakePair.selectedPart = brakePair.getPossiblePartForAGivenNameAndRadius(part.name, (diskPair.selectedPart as PartDisk).selectedDiskSize);
                    }
                    arrOfPartSlotForUpdate.push(brakePair);
                    ErrorTracker.setLineLabel("#39");
                }
                var rad : int = 0;
                if(tires.length) {
                    for each (var tr : PartSlot in tires) {
                        if(tr.name == partSlot.name)
                            continue;
                        ErrorTracker.setLineLabel("#40");
                        if(tr) // means that the partSlot is either the rear or front , and the tirePair is the opposite
                        {
                            if (tr.selectedPart) {
                                tuningScreen.carView.removeMaskImages(tr.selectedPart);
                            }
                            rad = 0;
                            var possibleDisk:PartSlotDisk;
                            if(car.possiblePartSlotsDisks.length >2) {
                                possibleDisk = car.getPossiblePartSlot("disk" + tr.name.replace("tire", "")) as PartSlotDisk;
                            }else {
                                possibleDisk = car.getPossiblePartSlot("disks" + tr.name.replace("tires", "")) as PartSlotDisk;
                            }
                            if(possibleDisk && possibleDisk.selectedPart){
                                rad = (possibleDisk.selectedPart as PartDisk).selectedDiskSize;
                            }
                            tr.selectedPart = tr.getPossiblePartForAGivenNameAndRadius(part.name, rad);
                            arrOfPartSlotForUpdate.push(tr);
                        }
                        ErrorTracker.setLineLabel("#41");
                    }
                }
                ErrorTracker.setLineLabel("#42");
                if(vinyls.length) {
                    for each (var vl : PartSlot in vinyls) {
                        if(vl.name == partSlot.name)
                            continue;

                        if(vl) // means that the partSlot is either the rear or front , and the tirePair is the opposite
                        {
                            tuningScreen.carView.removeMaskImages(vl.selectedPart);
                            rad = 0;
                            if(car.possiblePartSlotsVinyls.length >2) {
                                rad = (car.getPossiblePartSlot("disk" + vl.name.replace("tire_vinyl", "")).selectedPart as PartDisk).selectedDiskSize;
                            }else {
                                rad = (car.getPossiblePartSlot("disks" + vl.name.replace("tire_vinyls", "")).selectedPart as PartDisk).selectedDiskSize;
                            }
                            vl.selectedPart = vl.getPossiblePartForAGivenNameAndRadius(part.name, rad);
                            arrOfPartSlotForUpdate.push(vl);
                        }
                    }
                }
                ErrorTracker.setLineLabel("#43");
                if(tirePair) // means that the partSlot is either the rear or front , and the tirePair is the opposite
                {
                    if (tirePair.selectedPart) {
                        tuningScreen.carView.removeMaskImages(tirePair.selectedPart);
                    }
                    tirePair.selectedPart = tirePair.getPossiblePartForAGivenNameAndRadius(part.name,(diskPair.selectedPart as PartDisk).selectedDiskSize );
                    arrOfPartSlotForUpdate.push(tirePair);
                }
                ErrorTracker.setLineLabel("#44");
                if(oppositePairs) {
                    if (diskPair.selectedPart) { // removing mask for disk pair
                        tuningScreen.carView.removeMaskImages(diskPair.selectedPart);
                    }
                    ErrorTracker.setLineLabel("#45");
                    var currentDiskRadius:int = 0;
                    var diskPart:PartDisk;
                    var diskProfile:String;
                    var partName:String;
                    if(part){
                        partName = part.name;
                    }
                    ErrorTracker.setLineLabel("#45 - 1");
                    if(part && part is PartDisk){
                        diskPart = PartDisk(part);
                        ErrorTracker.setLineLabel("#45 - 2");
                        currentDiskRadius = diskPart.selectedDiskSize;
                        ErrorTracker.setLineLabel("#45 - 3");
                        diskProfile = diskPart.profile;
                        if(PartDisk(partSlot.selectedPart).profile == "default" && diskProfile != "default"){
                            diskProfile = diskPart.getDefaultProfile();
                        }
                        if(diskProfile != diskPart.profile){
                            diskPart = diskPart.getProfilePart(diskProfile);
                        }
                        ErrorTracker.setLineLabel("#45 - 4");
                    }
                    ErrorTracker.setLineLabel("#45 - 5");
                    var foundPartDiskSlot:PartDisk;
                    if(diskPair){
                        ErrorTracker.setLineLabel("#45 - 6");
                        var diskData:Object = diskPair.findPartDiskForRadius(partName, currentDiskRadius, diskProfile);
                        if(diskData && diskData.hasOwnProperty("part")){
                            foundPartDiskSlot = diskData.part;
                        }
                        if (!foundPartDiskSlot) {
                            ErrorTracker.setLineLabel("#45 - 7");
                            prefix = PartSlotDisk.getPrefixDep(part);
                            ErrorTracker.setLineLabel("#45 - 8");
                            foundPartDiskSlot = diskPair.findMainPartDiskByName(partName + prefix);
                            ErrorTracker.setLineLabel("#45 - 9");
                        }

                        ErrorTracker.setLineLabel("#46");
                        if(!diskPair.selectedPart){
                            diskPair.selectedPart = foundPartDiskSlot;
                        }

                        if(foundPartDiskSlot) {
                            if (part.selectedState) {
                                foundPartDiskSlot.selectedState = foundPartDiskSlot.states[part.selectedState.nameOfState];
                            } else {
                                foundPartDiskSlot.selectedState = null;
                            }
                            if (diskPair && diskPair.selectedPart && diskPair.selectedPart.selectedMaterial) {
                                foundPartDiskSlot.selectedMaterial = foundPartDiskSlot.getPossibleMaterial(diskPair.selectedPart.selectedMaterial.name);
                            } else {
                                foundPartDiskSlot.selectedMaterial = null;
                            }
                            foundPartDiskSlot.parentSlotDisk.selectedColor = part.parentSlot.selectedColor;
                        }
                        ErrorTracker.setLineLabel("#47");
                        diskPair.selectedPart = foundPartDiskSlot;
                        if (diskPair.selectedPart) {
                            (diskPair.selectedPart as PartDisk).selectedDiskSize = currentDiskRadius;
                            diskPair.selectedPart.selectedMaterial = diskPair.selectedPart.getPossibleMaterial(part.selectedMaterial.name);
                            if(diskPair.selectedPart.selectedMaterial.isPaintable) {
                                diskPair.selectedColor= partSlot.selectedColor;
                            }
                            arrOfPartSlotForUpdate.push(diskPair);
                        }
                    }

                    ErrorTracker.setLineLabel("#48");
                    // check the opposite diskSlots, if the radius has changed, than we should update those disks radius too

                    for each (var odp : PartSlotDisk in oppositePairs)// odb = opposite disk pair
                    {
                        if(!odp.selectedPart){
                            odp.selectedPart = odp.findPartDiskForRadius(part.name, currentDiskRadius, diskProfile).part;
                        }
                        var odpsp : PartDisk = odp.selectedPart as PartDisk; // odpsp = opposite disk pair selected part
                        if(odpsp.selectedDiskSize != currentDiskRadius) {
                            var mainPartName : String = odpsp.name;
                            var newOdpsp : PartDisk = odp.getPossiblePartForAGivenNameAndRadius(odpsp.name, currentDiskRadius) as PartDisk; // if there is the same disk for this radius
                            if(newOdpsp == null) // if there is not, than we should put the selected one
                            {
                                newOdpsp = odp.getPossiblePartForAGivenNameAndRadius(part.name, currentDiskRadius) as PartDisk;
                                mainPartName = part.name;
                            }

                            if (odp.selectedPart) { // removing mask/antimask
                                tuningScreen.carView.removeMaskImages(odp.selectedPart);
                            }
                            ErrorTracker.setLineLabel("#49");
                            var odpFindedPartDiskSlot: PartDisk = odp.findMainPartDiskByName(mainPartName);
                            if (!odpFindedPartDiskSlot) {
                                prefix = PartSlotDisk.getPrefixDep(odpsp);
                                odpFindedPartDiskSlot = odp.findMainPartDiskByName(mainPartName + prefix);
                            }
                            if(odpFindedPartDiskSlot){
                                odp.selectedPart = newOdpsp = odpFindedPartDiskSlot;
                                newOdpsp.selectedDiskSize = currentDiskRadius;
                                newOdpsp.selectedMaterial = newOdpsp.getPossibleMaterial(odp.selectedMaterialName);
                                arrOfPartSlotForUpdate.push(odp);
                            }
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#50");
            if(Car.CAR_DISKS_SLOTS.indexOf(partSlot.name) >= 0) {
                tuningScreen.carView.diskRadiusSync(partSlot, part, arrOfPartSlotForUpdate);
            }else if(Car.BRAKES_SLOTS.indexOf(partSlot.name) >= 0) {
                part = tuningScreen.selectedPart = tuningScreen.carView.brakeDiskSync(partSlot, part);
            }else if(Car.TIRES_SLOTS.indexOf(partSlot.name) >= 0) {
                part = tuningScreen.selectedPart = tuningScreen.carView.tireDiskSync(partSlot, part);
            }else if(Car.TIRE_VINYL_PARTS.indexOf(partSlot.name) >= 0) {
                part = tuningScreen.selectedPart = tuningScreen.carView.vinylDiskSync(partSlot, part);
            }
            ErrorTracker.setLineLabel("#51");
            tuningScreen.carView.removeMaskImages(partSlot.selectedPart);
            var pv:Part = partSlot.selectedPart;
            partSlot.selectedPart = part;

            if (partSlot.selectedPart && !partSlot.selectedPart.selectedMaterial) {
                var findMaterialName:String;
                if (tuningScreen.bodyPartSlot && tuningScreen.bodyPartSlot.selectedPart && tuningScreen.bodyPartSlot.selectedPart.selectedMaterial) {
                    findMaterialName = tuningScreen.bodyPartSlot.selectedPart.selectedMaterial.name;
                    for each(material in part.possibleMaterials) {
                        if (material.name == findMaterialName) {
                            part.selectedMaterial = part.getPossibleMaterial(findMaterialName);
                        }
                    }
                }
                ErrorTracker.setLineLabel("#52");
                if (!part.selectedMaterial) {
                    if (partSlot.selectedPart.possibleMaterials.length > 0) {
                        for each(material in partSlot.selectedPart.possibleMaterials){
                            if(material.name == "metal"){
                                partSlot.selectedPart.selectedMaterial = material;
                                break;
                            }
                        }
                        partSlot.selectedPart.selectedMaterial ||= partSlot.selectedPart.getPossibleMaterial("glossy");
                        partSlot.selectedPart.selectedMaterial ||= partSlot.selectedPart.getPossibleMaterial("mate");
                        partSlot.selectedPart.selectedMaterial ||= partSlot.selectedPart.possibleMaterials[0];
                    } else {
                        mainPart = part.getMainPartFromLinked();
                        if (mainPart) {
                            partSlot.selectedPart.selectedMaterial = mainPart.getPossibleMaterial(findMaterialName);
                            if (!partSlot.selectedPart.selectedMaterial)
                                partSlot.selectedPart.selectedMaterial = mainPart.selectedMaterial;
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#53");
            if (partSlot.selectedPart && partSlot.selectedPart.selectedMaterial) {
                if (partSlot.selectedColor && partSlot.selectedColor.value == 0xffffff && partSlot.selectedPart.selectedMaterial.isPaintable) {
                    partSlot.selectedColor.value = 0xfeffff;
                }
                if(Car.TIRES_SLOTS.indexOf(partSlot.name) != -1){
                    for(i = 0; i < arrOfPartSlotForUpdate.length; i++){
                        if(Car.TIRES_SLOTS.indexOf(arrOfPartSlotForUpdate[i].name) != -1){
                            PartSlot(arrOfPartSlotForUpdate[i]).selectedColor = partSlot.selectedColor.copy();
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#54");
            modifiedTuning = true;

            var isPack:Boolean = (partSlot.name == PartSlot.BODY_PACK || partSlot.name == PartSlot.FACTORY_TRIM);
            if (isPack && partSlot.selectedPart.excludedPartSlotsNames && partSlot.selectedPart.excludedPartSlotsNames.indexOf("disks") >= 0) {
                tuningScreen.setSuspensionControlEnabled(false);
                SuspensionControlEnabledFlag = false;
            } else if (isPack) {
                SuspensionControlEnabledFlag = true;
                tuningScreen.setSuspensionControlEnabled(SuspensionControlEnabledFlag);
            }
            ErrorTracker.setLineLabel("#55");
            var linkedPartSlots:Array = [];
            arrOfPartSlotForUpdate.push(partSlot);
            for(var iii : int = 0; iii < arrOfPartSlotForUpdate.length; iii ++) {
                var ppp : PartSlot = arrOfPartSlotForUpdate[iii];
                ErrorTracker.setLineLabel("#56");
                if (ppp && ppp.selectedPart && ppp.selectedPart.linkedParts && ppp.selectedPart.linkedParts.length > 0 && ppp.selectedPart.selectedMaterial) {

                    var defaultMaterial:String = ppp.selectedPart.selectedMaterial.name;
                    for (i = 0; i < ppp.selectedPart.linkedParts.length; i++) {
                        var linkedPart2:Part = ppp.selectedPart.linkedParts[i];
                        if(!linkedPart2) {
                            continue;
                        }
                        if(!linkedPart2.parentSlot) {
                            continue;
                        }
                        ErrorTracker.setLineLabel("#57");
                        if(Car.DISKS_SLOTS.indexOf(ppp.name) >= 0 && (Car.BRAKES_SLOTS.indexOf(linkedPart2.parentSlot.name) >= 0 || Car.TIRES_SLOTS.indexOf(linkedPart2.parentSlot.name) >= 0 || Car.TIRE_VINYL_PARTS.indexOf(linkedPart2.parentSlot.name) >= 0))
                        { // if parent of linked is disk, so we need to ignore tires
                            continue;
                        }
                        if (linkedPart2.parentSlot.selectedPart) {
                            //осбый алгоритм для выставление дисков
                            if (Car.DISKS_SLOTS.indexOf(linkedPart2.parentSlot.name )>=0) {
                                var partSlotDisk:PartSlotDisk = linkedPart2.parentSlot as PartSlotDisk;
                                var partDisk:PartDisk = linkedPart2 as PartDisk;
                                var selectedRadiusSize:int = partDisk.radius;
                                var mainPartDisk:PartDisk = partSlotDisk.findMainPartDiskByName(linkedPart2.name);
                                if (!mainPartDisk) {
                                    prefix =  PartSlotDisk.getPrefixDep(linkedPart2);
                                    mainPartDisk = partSlotDisk.findMainPartDiskByName(linkedPart2.name + prefix);
                                }
                                if(mainPartDisk) {
                                    mainPartDisk.selectedDiskSize = selectedRadiusSize;
                                    linkedPart2 = mainPartDisk;
                                }
                                tuningScreen.carView.diskRadiusSync(linkedPart2.parentSlot,linkedPart2,arrOfPartSlotForUpdate);
                            }
                        }else{
                            Tracer.print("не было");
                        }
                        ErrorTracker.setLineLabel("#58");
                        if (linkedPart2.parentSlot.selectedPart){
                            Tracer.print("стало ", linkedPart2.name);
                        }else{
                            Tracer.print("не стало");
                        }
                        if (linkedPart2.selectedMaterial) {
                            if (Car.DISKS_SLOTS.indexOf(linkedPart2.parentSlot.name )>=0) {
                                linkedPart2.selectedMaterial = linkedPart2.getPossibleMaterial("metal");
                            } else {
                                var fl:Boolean = false;
                                var m:PartMaterial;
                                if (linkedPart2.parentSlot.selectedPart && linkedPart2.parentSlot.selectedPart.selectedMaterial) {
                                    m = linkedPart2.getPossibleMaterial(linkedPart2.parentSlot.selectedPart.selectedMaterial.name);
                                    if (m) {
                                        fl = true;
                                        linkedPart2.selectedMaterial = m;
                                    }
                                }

                                if (!fl && linkedPart2.selectedMaterial.name != defaultMaterial) {

                                    m = linkedPart2.getPossibleMaterial(defaultMaterial);

                                    if (m) linkedPart2.selectedMaterial = m;
                                }
                            }
                            ErrorTracker.setLineLabel("#59");
                        } else {
                            if (linkedPart2.possibleMaterials.length > 0) {
                                material = linkedPart2.getPossibleMaterial(defaultMaterial);
                                if (material) {
                                    linkedPart2.selectedMaterial = material
                                } else if(linkedPart2.getPossibleMaterial("glossy")){
                                    linkedPart2.selectedMaterial = linkedPart2.getPossibleMaterial("glossy");
                                    if(tuningScreen.car.getPossiblePartSlot("body").selectedColor){
                                        linkedPart2.parentSlot.selectedColor = tuningScreen.car.getPossiblePartSlot("body").selectedColor;
                                    }
                                } else if(linkedPart2.getPossibleMaterial("metal")){
                                    linkedPart2.selectedMaterial = linkedPart2.getPossibleMaterial("metal");
                                    if(tuningScreen.car.getPossiblePartSlot("body").selectedColor){
                                        linkedPart2.parentSlot.selectedColor = tuningScreen.car.getPossiblePartSlot("body").selectedColor;
                                    }
                                }else{
                                    linkedPart2.selectedMaterial = linkedPart2.possibleMaterials[0];
                                }
                            }
                        }
                        ErrorTracker.setLineLabel("#60");
                        if (tuningScreen.carView.partSlotMask && linkedPart2.parentSlot.selectedPart && linkedPart2) {
                            if (linkedPart2.parentSlot.selectedPart.id != linkedPart2.id) {
                                tuningScreen.carView.removeMaskImages(linkedPart2.parentSlot.selectedPart);

                                if (Car.DISKS_SLOTS.indexOf(linkedPart2.parentSlot.name )>=0) {
                                    tuningScreen.carView.isExistCember = false;
                                    tuningScreen.invalidate('car');
                                }
                            }
                        }
                        ErrorTracker.setLineLabel("#61");
                        linkedPart2.parentSlot.isExcluded = false;
                        if(linkedPart2 != linkedPart2.parentSlot.selectedPart && PartSlot.TWO_TONE_VINYLS_LIST.indexOf(linkedPart2.parentSlot.name) == -1 && PartSlot.VINYL_PACK_LINKED.indexOf(linkedPart2.parentSlot.name) == -1){
                            linkedPart2.parentSlot.savePartLinked = linkedPart2.parentSlot.selectedPart;
                        }

                        if(linkedPart2.parentSlot != partSlot && (!linkedPart2.parentSlot.selectedPart || partSlot.selectedPart.linkedParts.indexOf(linkedPart2.parentSlot.selectedPart) == -1)){
                            tuningScreen.carView.removeMaskImages(linkedPart2.parentSlot.selectedPart);
                            linkedPart2.parentSlot.selectedPart = linkedPart2;
                        }
                        if(linkedPartSlots.indexOf(linkedPart2.parentSlot) == -1){
                            linkedPartSlots.push(linkedPart2.parentSlot);
                        }
                        arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(car.getDependence(linkedPart2));
                        if (linkedPart2.parentSlot.selectedPart && linkedPart2.parentSlot.selectedPart.selectedMaterial) {
                            if (linkedPart2.parentSlot.selectedColor && linkedPart2.parentSlot.selectedColor.value == 0xffffff && linkedPart2.parentSlot.selectedPart.selectedMaterial.isPaintable) {
                                linkedPart2.parentSlot.selectedColor.value = 0xFEFFFF;
                            }
                        }
                        ErrorTracker.setLineLabel("#62");
                        if (arrOfPartSlotForUpdate.indexOf(linkedPart2.parentSlot) < 0){
                            if( linkedPart2.parentSlot.savePartLinked!=linkedPart2.parentSlot.selectedPart){
                                arrOfPartSlotForUpdate.push(linkedPart2.parentSlot);
                            }
                        }
                        ErrorTracker.setLineLabel("#63");
                    }
                }
            }
            ErrorTracker.setLineLabel("#64");
            var dependenceArray: Array = car.getDependence(part);
            for each (var dependenceArrayItem: PartSlot in arrOfPartSlotForUpdate) {
                if (dependenceArrayItem) {
                    if (Car.TIRES_SLOTS.indexOf(dependenceArrayItem.name) > -1 || Car.TIRE_VINYL_PARTS.indexOf(dependenceArrayItem.name) > -1) {
                        if (dependenceArrayItem.selectedPart) {
                            var tireDependence:Array = car.getDependence(dependenceArrayItem.selectedPart);
                            arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(tireDependence);
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#65");

            arrOfPartSlotForUpdate = arrOfPartSlotForUpdate.concat(dependenceArray);

            var k : int;
            if (!car.isChopper) {
                if(pv) {
                    var arr:Array = getDependenceOfPartFromPartSlots(pv);
                    var isDuplicate:Boolean;
                    for (k = 0; k < arr.length; k++) {
                        isDuplicate = false;
                        for(var g : int = 0; g < arrOfPartSlotForUpdate.length; g++) {
                            if(arrOfPartSlotForUpdate[g].name == arr[k].name) {
                                isDuplicate = true;
                                break;
                            }
                        }

                        if(!isDuplicate) {
                            addOnRemoveComponents = removePartSlot(null, arr[k], false).concat(addOnRemoveComponents);
                            //tuningScreen.carView.removePartSlot(arr[k], false);
                            arr[k].selectedPart = null;
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#66");
            if (car.componentsDependenciesWheel) {
                for (i = 0; i < car.componentsDependenciesWheel.length; i++) {
                    var dependency:Object = car.componentsDependenciesWheel[i];
                    for (var slotName:String in dependency) {
                        for each(var inSlotName:String in dependency[slotName]) {
                            for (var j:int = 0; j < arrOfPartSlotForUpdate.length; j++) {
                                var updatedSlotName:String = arrOfPartSlotForUpdate[j].name;
                                var slot:PartSlot = null;
                                var partSel:Part = null;
                                if (slotName == updatedSlotName) {
                                    for (var index:int in dependency[slotName]) {
                                        slot = car.getPossiblePartSlot(dependency[slotName][index]);
                                        if (slot) {
                                            if (!slot.selectedPart) {
                                                partSel = slot.possibleParts[0];
                                                if (partSel) {
                                                    slot.selectedPart = partSel;
                                                    arrOfPartSlotForUpdate.push(slot);
                                                }
                                            }
                                        }
                                    }
                                }

                                //Обратная зависимость не понятно нужна ли она но делаем
                                if (updatedSlotName == inSlotName) {
                                    //Нужно обновить слот с именем slotName
                                    slot = car.getPossiblePartSlot(slotName);
                                    if (slot) {
                                        if (!slot.selectedPart) {
                                            partSel = slot.possibleParts[0];
                                            if (partSel) {
                                                slot.selectedPart = partSel;
                                                arrOfPartSlotForUpdate.push(slot);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#67");
            var filterArrOfPartSlotForUpdate:Array = [];
            var r:int;
            for (r = 0; r < arrOfPartSlotForUpdate.length; r++) {
                var slot1:PartSlot = arrOfPartSlotForUpdate[r];

                if (slot1.selectedPart) {
                    if (filterArrOfPartSlotForUpdate.indexOf(slot1) < 0) {
                        filterArrOfPartSlotForUpdate.push(slot1);
                    }
                } else if (!slot1.selectedPart) {
                    addOnRemoveComponents = removePartSlot(null, slot1, true).concat(addOnRemoveComponents);
                    //tuningScreen.carView.removePartSlot(slot1, true);
                }
            }
            ErrorTracker.setLineLabel("#68");
            // ********************************************************
            var depPart:Part;
            var linkedPartArchive:Dictionary = new Dictionary();
            var selectedLinkedSlots:Array = [];
            if(partSlot && (partSlot.name == PartSlot.VINYL_PACK || partSlot.name == PartSlot.TWO_TONE_VINYL_PACK)){
                if(partSlot.selectedPart && partSlot.selectedPart.linkedParts){
                    for(i = partSlot.selectedPart.linkedParts.length-1; i >= 0; i--){
                        var pt:Part = partSlot.selectedPart.linkedParts[i];
                        selectedLinkedSlots.push(pt.parentSlot);
                        depPart = car.getDependentPart(pt);
                        if(depPart && depPart!=depPart.parentSlot.selectedPart){
                            partSlot.selectedPart.linkedParts[i] = depPart;
                            depPart.parentSlot.selectedPart = depPart;
                        }else if(!depPart){
                            depPart = partSlot.selectedPart.linkedParts[i];
                            if(depPart.parentSlot.selectedPart){
                                depPart.parentSlot.selectedPart = null;
                            }
                            var removedPart:Part = partSlot.selectedPart.linkedParts.removeAt(i) as Part;
                            linkedPartArchive[partSlot.selectedPart] ||= [];
                            linkedPartArchive[partSlot.selectedPart].push(removedPart);
                        }
                    }
                }
                if(pv && pv.linkedParts){
                    for(i = 0; i < pv.linkedParts.length; i++){
                        if(selectedLinkedSlots.indexOf(Part(pv.linkedParts[i]).parentSlot) == -1 && Part(pv.linkedParts[i]).parentSlot.selectedPart){
                            removePartSlot(null, Part(pv.linkedParts[i]).parentSlot, false);
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#69");
            for(i = filterArrOfPartSlotForUpdate.length-1; i >= 0; i--){
                depPart = PartSlot(filterArrOfPartSlotForUpdate[i]).selectedPart;
                if(depPart && (PartSlot.VINYLS.indexOf(depPart.parentSlot.name) != -1 || PartSlot.TWO_TONE_VINYLS_LIST.indexOf(depPart.parentSlot.name) != -1) && depPart.dependence && depPart.dependence.length){
                    depPart = car.getDependentPart(depPart);
                    if(!depPart){
                        filterArrOfPartSlotForUpdate.removeAt(i);
                    }
                }
                if(depPart && Car.TIRES_SLOTS.indexOf(partSlot.name)!=-1 && !depPart.selectedMaterial && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial){
                    depPart.selectedMaterial = depPart.getPossibleMaterial(partSlot.selectedPart.selectedMaterial.name);
                }
            }
            ErrorTracker.setLineLabel("#70");
            // ***********************************************************
            if(partSlot && Car.WINDOW_SLOTS.indexOf(partSlot.name) != -1){
                var execResult:Array = part?RegExpUtil.getWindowTint(part.name):null;
                var selectedTint:int = -1;
                if(execResult && execResult.length){
                    selectedTint = parseInt(execResult[0]);
                }

                for(i = 0; i < Car.WINDOW_SLOTS.length; i++){
                    var window:PartSlot = car.getPossiblePartSlot(Car.WINDOW_SLOTS[i]);
                    if(!window || !window.selectedPart){
                        continue;
                    }
                    var folderName:String = "";
                    var result2:Array = RegExpUtil.getWindowTintState(window.selectedPart.folder);
                    if(result2 && result2.length){
                        folderName = result2[0];
                    }
                    for(var i2:int = 0; i2 < window.possibleParts.length; i2++){
                        var result:Array = RegExpUtil.getWindowTint(window.possibleParts[i2].name);
                        var tint:int = -1;
                        if(result){
                            tint = parseInt(result[result.length - 1]);
                        }
                        if(tint == selectedTint && Part(window.possibleParts[i2]).folder.indexOf(folderName) != -1){
                            window.selectedPart = window.possibleParts[i2];
                            if(filterArrOfPartSlotForUpdate.indexOf(window) == -1){
                                filterArrOfPartSlotForUpdate.push(window);
                            }
                            if(window.selectedPart.selectedMaterial && ObjectUtils.isEmpty(window.selectedPart.selectedMaterial.imagesMap)){
                                for(var i3:int = 0; i3 < window.possibleParts.length; i3++){
                                    if(window.possibleParts[i3].selectedMaterial && !ObjectUtils.isEmpty(window.possibleParts[i3].selectedMaterial.imagesMap)){
                                        window.selectedPart.selectedMaterial.imagesMap = window.possibleParts[i3].selectedMaterial.imagesMap;
                                        break;
                                    }
                                }
                            }
                            break;
                        }
                    }
                }
            }
            ErrorTracker.setLineLabel("#71");
            // ***********************************************************
            //
            for(i = 0; i < filterArrOfPartSlotForUpdate.length; i++){
                part = PartSlot(filterArrOfPartSlotForUpdate[i]).selectedPart;
                if(part && part.removeTypes && part.removeTypes.length){
                    for(j = 0; j < part.removeTypes.length; j++){
                        var removingPartSlot:PartSlot = car.getPossiblePartSlot(part.removeTypes[j]);
                        // linkedPartSlots.indexOf(removingPartSlot) == -1 && TODO: check
                        // partSlot.selectedPart.linkedParts.indexOf(removingPartSlot.selectedPart) == -1 &&
                        if(removingPartSlot && partSlot.selectedPart.linkedParts.indexOf(removingPartSlot.selectedPart) == -1 && removingPartSlot != partSlot) {
                            addOnRemoveComponents = removePartSlot(filterArrOfPartSlotForUpdate, removingPartSlot, false, false, tuningScreen.selectedPart.parentSlotName).concat(addOnRemoveComponents);
                        }
                    }
                }
                tuningScreen.carView.removeMaskImages(PartSlot(filterArrOfPartSlotForUpdate[i]).prevSelectedPart);
            }
            ErrorTracker.setLineLabel("#72");
            //
            filterArrOfPartSlotForUpdate = ArrayUtil.uniqueConcat(filterArrOfPartSlotForUpdate, addOnRemoveComponents);
            //
            if(brakePair && brakePair.selectedPart && part && part.selectedMaterial){
                brakePair.selectedPart.selectedMaterial = brakePair.selectedPart.getPossibleMaterial(part.selectedMaterial.name);
                if(part.parentSlot.selectedColor){
                    brakePair.selectedColor = part.parentSlot.selectedColor.copy();
                }
            }
            ErrorTracker.setLineLabel("#73");
            // ***********************************************************
            tuningScreen.carView.resetFastModeCurrentLoaded();
            // ***********************************************************
            var carProxy:CarsProxy =  carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
            carProxy.loadComponent(filterArrOfPartSlotForUpdate, function(components:Array):void{
                if(event &&  event.data && event.data.loadCompleteHandler){
                    if(event.data.hasOwnProperty("updateWithPartSlots")){
                        if(event.data.updateWithPartSlots){
                            tuningScreen.carView.updateWithPartSlots(filterArrOfPartSlotForUpdate, false, event.data.loadCompleteHandler);
                        }else{
                            event.data.loadCompleteHandler();
                        }
                    }else{
                        tuningScreen.carView.updateWithPartSlots(filterArrOfPartSlotForUpdate, false, event.data.loadCompleteHandler);
                    }
                }else{
                    tuningScreen.carView.updateWithPartSlots(filterArrOfPartSlotForUpdate, false);
                }
                var timeDelay:Number = tuningScreen.carView.previewMode? 0: 100;
                setTimeout(function():void{
                    for(var item:Part in linkedPartArchive){
                        var linkeds:Array = linkedPartArchive[item] as Array;
                        for(var i:int = 0; i < linkeds.length; i++){
                            if(item.linkedParts.indexOf(linkeds[i])==-1){
                                item.linkedParts.push(linkeds[i]);
                            }
                        }
                    }
                }, timeDelay);
            });
        }
        ErrorTracker.setLineLabel("#74");
        if (tuningScreen.carView.saveSelectedPart) {

            partSlot.selectedPart = tuningScreen.carView.saveSelectedPart;
            partSlot.selectedColor = tuningScreen.carView.saveSelectedColor;
            tuningScreen.selectedPart = partSlot.selectedPart;

            tuningScreen.carView.saveSelectedPart = null;
            tuningScreen.carView.saveSelectedColor = null;
        }
        ErrorTracker.setLineLabel("#75");
        updateColorViewEnabled();
        tuningScreen.invalidate('car');
        ErrorTracker.setLineLabel(null);
        ErrorTracker.setReference(null);
    }

    /**
     *  returns the array of partSlot from which current part is dependent of
     * @param findPart
     * @return
     */
    private function getDependenceOfPartFromPartSlots(findPart:Part):Array //Created By Vardan
    {
        var result:Array = [];
        for each( var slotName : String in findPart.depsMap)
        {
            var partSlot:PartSlot = car.getPossiblePartSlot(slotName);
            if (partSlot.selectedPart)
                result.push(partSlot);
        }
        return result;
    }



    private function _isSelectedName(name:String, type:String):Boolean {
        var checkPartSlot:PartSlot = car.getPossiblePartSlot(type);
        var activePart :Part = checkPartSlot.selectedPart;
        return (activePart.name == name);
    }

    private function getTypeById(id:String):String {
        var allSlots:Array = car.possiblePartSlots;
        for (var i:String in allSlots) {
            for (var j:String in allSlots[i].possibleParts) {
                if (allSlots[i].possibleParts[j].id == id) {
                    return allSlots[i].name;
                }
            }
        }
        return null;
    }

    private function updateFrameBackIfNeeded():PartSlot {

        var updateFlag:Boolean = false;
        //if(partSlot.name=="frame" && partSlot.selectedPart){
        var framePartSlot:PartSlot = car.getPossiblePartSlot("frame");

        if (!framePartSlot) return null;

        var frameBackPartSlot:PartSlot = car.getPossiblePartSlot("frame_back");

        if (!frameBackPartSlot) return null;

        if (framePartSlot && framePartSlot.selectedPart) {


            if (frameBackPartSlot.selectedPart && framePartSlot.selectedPart.selectedMaterial) {

                var materialName:String = framePartSlot.selectedPart.selectedMaterial.name;


                var material:PartMaterial = frameBackPartSlot.selectedPart.getPossibleMaterial(materialName);

                if (!material && frameBackPartSlot.selectedPart.possibleMaterials.length > 0) {


                    material = frameBackPartSlot.selectedPart.possibleMaterials[0];
                }

                if (material != frameBackPartSlot.selectedPart.selectedMaterial) {
                    frameBackPartSlot.selectedPart.selectedMaterial = material;


                    updateFlag = true;
                }


                if (frameBackPartSlot.selectedColor && framePartSlot.selectedColor) {

                    if (frameBackPartSlot.selectedColor.value != framePartSlot.selectedColor.value) {

                        frameBackPartSlot.selectedColor = framePartSlot.selectedColor.copy();
                        updateFlag = true;
                    }
                }
            }

        } else if (!framePartSlot.selectedPart && frameBackPartSlot.selectedPart) {

            //удаляем фрейм бэк

            //tuningScreen.carView.removePartSlot(frameBackPartSlot);
            removePartSlot(null, frameBackPartSlot);
            frameBackPartSlot.selectedPart = null;
            //updateFlag=true;

        }


        if (updateFlag) {

            return frameBackPartSlot;
        } else {

            return null;
        }


        //}
    }

//    private function findDependency(partSlot:PartSlot, arrOfPartSlotForUpdate:Array, currentRadius:int = -1):Array {
//        var pSlot:PartSlot;
//        var p:Part;
//        var antimaskPartMaterial:PartMaterial;
//        var i:int;
//        for each(partSlot in car.possiblePartSlots) {
//
//            if (!partSlot.selectedPart) continue;
//
//
//            possibleParts = tuningScreen.wideFilter(possibleParts, partSlot);
//            var possibleParts:Array = tuningScreen.dependendFilter(partSlot.possibleParts, partSlot);
//
//
//            for (i = 0; i < possibleParts.length; i++) {
//
//                p = possibleParts[i];
//
//
//                if (!p.wide && p.dependence && p.name == partSlot.selectedPart.name) {
//
//                    partSlot.selectedPart = p;
//                    arrOfPartSlotForUpdate.push(partSlot);
//                } else if (p.wide && p.name == partSlot.selectedPart.name && p.wide + "" == currentRadius + "") {
//
//                    partSlot.selectedPart = p;
//                    arrOfPartSlotForUpdate.push(partSlot);
//                }
//            }
//
//        }
//
//
//        return arrOfPartSlotForUpdate;
//    }

    private function updateColorViewEnabled():void {
        if (tuningScreen.selectedPartSlot.selectedPart) {
            if (tuningScreen.selectedPartSlot.selectedPart.selectedState) {
                if (tuningScreen.selectedPartSlot.selectedPart.selectedState.selectedMaterial) {
                    tuningScreen.colorsViewEnabled = tuningScreen.selectedPartSlot.selectedPart.selectedState.selectedMaterial.isPaintable;
                }

            } else {
                if (tuningScreen.selectedPartSlot.selectedPart.selectedMaterial && tuningScreen.allPartsColorsView) {
                    //tuningScreen.colorsViewEnabled = tuningScreen.selectedPartSlot.selectedPart.selectedMaterial.isPaintable;
                    tuningScreen.allPartsColorsView.isEnabled = tuningScreen.allPartsColorsView.hasPaintMaterial;
                } else {


                    var mainPart:Part = tuningScreen.selectedPartSlot.selectedPart.getMainPartFromLinked();

                    if (mainPart && mainPart.parentSlot && mainPart.parentSlot.selectedPart && mainPart.parentSlot.selectedPart && mainPart.parentSlot.selectedPart.selectedMaterial) {

                        tuningScreen.colorsViewEnabled = mainPart.parentSlot.selectedPart.selectedMaterial.isPaintable;
                    } else {

                        tuningScreen.colorsViewEnabled = false;
                    }
                }

            }

        }
    }

    private function tuningScreenToggleColorSelectionHandler(event:starling.events.Event):void {
        if (tuningScreen.colorSelectionOpened) {
            tuningScreen.closeColorSelection();
        }
        else {
            tuningScreen.openColorSelection();
        }
    }

    private function tuningScreenColorSelectedHandler(event:starling.events.Event):void {

        trace("tuningScreenColorSelectedHandler");
        if (tuningScreen && tuningScreen.progressBar && tuningScreen.progressBar.visible) return;
        var partSlots:Array;
        var i:int;
        if(tuningScreen.selectedPartSlot.name == TuningColorsView.INTERIOR){
            partSlots = car.interiorSelectedPartSlots;
        }else if(tuningScreen.selectedPartSlot.name == TuningColorsView.EXTERIOR){
            partSlots = car.exteriorSelectedPartSlots;
        }else  if (!tuningScreen.selectedPartSlot.dummy) {
            partSlots = [tuningScreen.selectedPartSlot];

            if(tuningScreen.selectedPartSlot.name.indexOf(PartSlot.AEROGRAPHY)>=0 || tuningScreen.selectedPartSlot.name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE)>=0){
                if(this.tuningScreen.carView.saveSelectedPart){
                    tuningScreen.selectedPartSlot.selectedPart = this.tuningScreen.carView.saveSelectedPart;
                }
            }

            if (tuningScreen.selectedPartSlot.selectedPart && tuningScreen.selectedPartSlot.selectedPart.linkedParts && tuningScreen.selectedPartSlot.selectedPart.linkedParts.length > 0) {
                if(Car.CAR_DISKS_SLOTS.indexOf(tuningScreen.selectedPartSlot.name)>=0 ){
                    for (i = 0; i < tuningScreen.selectedPartSlot.selectedPart.linkedParts.length; i++) {
                        var linkedPart:Part = tuningScreen.selectedPartSlot.selectedPart.linkedParts[i];
                        partSlots.push(linkedPart.parentSlot);
                    }
                }
            }


            //Если изменили цвет у переднего правого колеса то меняем левое
            if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == "front_wheel_right") {

                var frontWheelLeftSlot:PartSlot = car.getPossiblePartSlot("front_wheel_left");

                if (frontWheelLeftSlot) {

                    partSlots.push(frontWheelLeftSlot);
                }

            }

            // If the part slot is a tire
            if (tuningScreen.selectedPartSlot && Car.TIRES_SLOTS.indexOf(tuningScreen.selectedPartSlot.name) != -1) {
                for(i = 0; i < Car.TIRES_SLOTS.length; i++){
                    if(Car.TIRES_SLOTS[i] != tuningScreen.selectedPartSlot.name) {
                        var partSlotTire:PartSlot = car.getPossiblePartSlot(Car.TIRES_SLOTS[i]);
                        if (partSlotTire) {
                            partSlots.push(partSlotTire);
                        }
                    }
                }
            }




            //Если изменили цвет у переднего левого колеса то меняем правое
            if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == "front_wheel_left") {

                var frontWheelRightSlot:PartSlot = car.getPossiblePartSlot("front_wheel_right");

                if (frontWheelRightSlot) {

                    partSlots.push(frontWheelRightSlot);
                }
            }


            //Если изменили цвет у заднего правого колеса то меняем левое
            if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == "rear_wheel_right") {

                var rearWheelLeftSlot:PartSlot = car.getPossiblePartSlot("rear_wheel_left");

                if (rearWheelLeftSlot) {

                    partSlots.push(rearWheelLeftSlot);
                }

            }


            //Если изменили цвет у заднего левого колеса то меняем правое
            if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == "rear_wheel_left") {

                var rearWheelRightSlot:PartSlot = car.getPossiblePartSlot("rear_wheel_right");

                if (rearWheelRightSlot) {

                    partSlots.push(rearWheelRightSlot);
                }
            }


            if (tuningScreen.selectedPartSlot && Car.TIRE_VINYL_PARTS.indexOf(tuningScreen.selectedPartSlot.name) != -1) {
                var tireVinylPair:PartSlot;
                for(var j:int = 0; j < Car.TIRE_VINYL_PARTS.length; j++){
                    if(tuningScreen.selectedPartSlot.name != Car.TIRE_VINYL_PARTS[j]){
                        tireVinylPair = car.getPossiblePartSlot(Car.TIRE_VINYL_PARTS[j]);
                        if (tireVinylPair) {
                            partSlots.push(tireVinylPair);
                        }
                    }
                }
            }
        } else {
            partSlots = car.exteriorSelectedPartSlots;
        }

        modifiedTuning = true;
        var selectedColor:PartColor = PartColor(event.data.selectedColor).copy();
        if((event.data && selectedColor && !selectedColor.isCustom) || tuningColorPicker.touchPhase == TouchPhase.ENDED){
            var transitionMethod:Function = function():void{
                tuningScreen.carView.removeLoadingCar();
                tuningScreen.carView.beforeLoading(false);
                updatePartSlotsWithColor(partSlots, selectedColor, tuningScreen.selectedPartSlot.dummy);
                var selectedPartSlotName:String = tuningScreen.selectedPartSlot.name;
                tuningScreen.carView.colorChangeEffect(selectedPartSlotName);
                partSlots = null;
            };

            if(tuningScreen.carView.loadingCarImage){
                Starling.juggler.tween(tuningScreen.carView.loadingCarImage, 0.15, {alpha: 0, onComplete: transitionMethod});
            }else{
                transitionMethod();
            }
        }else{
            tuningScreen.carView.removeLoadingCar();
            updatePartSlotsWithColor(partSlots, selectedColor, tuningScreen.selectedPartSlot.dummy);
            partSlots = null;
        }
    }

    private function updatePartSlotsWithColor(partSlots:Array, partColor:PartColor, allSlots:Boolean = false):void {

        if(car.suvTires && !allSlots )
        {
            for each (var p : PartSlot in partSlots)
            {
                var pairPartSlot : PartSlot;

                if(p.name == "left_windows")
                {
                    pairPartSlot = car.getPossiblePartSlot("right_windows");
                }

                if(p.name == "right_windows")
                {
                    pairPartSlot = car.getPossiblePartSlot("left_windows");
                }

                if(p.name == "left_doors")
                {
                    pairPartSlot = car.getPossiblePartSlot("right_doors");
                }
                if(p.name == "right_doors")
                {
                    pairPartSlot = car.getPossiblePartSlot("left_doors")
                }

                if(p.name == "mirror_left")
                {
                    pairPartSlot = car.getPossiblePartSlot("mirror_right");
                }
                if(p.name == "mirror_right")
                {
                    pairPartSlot = car.getPossiblePartSlot("mirror_left")
                }

                if(pairPartSlot && partSlots.indexOf(pairPartSlot) < 0 )
                {
                    pairPartSlot.selectedColor = partColor;
                    partSlots.push(pairPartSlot);
                }

                if(Car.CAR_DISKS_SLOTS.indexOf(p.name ) >= 0)
                {
                    var diskPair : PartSlotDisk;

                    switch (p.name)
                    {
                        case "disk_front_left":
                            diskPair = car.getPossiblePartSlot("disk_front_right") as PartSlotDisk;
                            break;
                        case "disk_front_right":
                            diskPair = car.getPossiblePartSlot("disk_front_left") as PartSlotDisk;
                            break;
                        case "disk_rear_left":
                            diskPair = car.getPossiblePartSlot("disk_rear_right") as PartSlotDisk;
                            break;
                        case "disk_rear_right":
                            diskPair = car.getPossiblePartSlot("disk_rear_left") as PartSlotDisk;
                            break;

                    }
                    if(diskPair && partSlots.indexOf(diskPair) < 0 )
                    {
                        diskPair.selectedColor = partColor;
                        partSlots.push(diskPair);
                    }
                }else if(Car.BRAKES_SLOTS.indexOf(p.name ) >= 0)
                {
                    var brakePair : PartSlot;

                    switch (p.name)
                    {
                        case "brake_front_left":
                            brakePair = car.getPossiblePartSlot("brake_front_right");
                            break;
                        case "brake_front_right":
                            brakePair = car.getPossiblePartSlot("brake_front_left") ;
                            break;
                        case "brake_rear_left":
                            brakePair = car.getPossiblePartSlot("brake_rear_right") ;
                            break;
                        case "brake_rear_right":
                            brakePair = car.getPossiblePartSlot("brake_rear_left");
                            break;

                    }
                    if(brakePair && partSlots.indexOf(brakePair) < 0 )
                    {
                        brakePair.selectedColor = partColor;
                        partSlots.push(brakePair)
                    }
                }
            }
        }

        if(allSlots){
            for(var i:int = partSlots.length-1; i > -1; i--){
                if(PartSlot.VINYLS.indexOf(partSlots[i].name)!=-1 || PartSlot.TWO_TONE_VINYLS_LIST.indexOf(partSlots[i].name)!=-1 || partSlots[i].name==PartSlot.VINYL_PACK || partSlots[i].name==PartSlot.TWO_TONE_VINYL_PACK || partSlots[i].name.indexOf(PartSlot.TWO_TONE_VINYL_TYPE) != -1){
                    partSlots.removeAt(i);
                }
            }
        }else{
            for each (var p2 : PartSlot in partSlots) {
                var isPack:Boolean = (p2.name == PartSlot.VINYL_PACK || p2.name == PartSlot.TWO_TONE_VINYL_PACK || p2.name == PartSlot.BODY_PACK || p2.name == PartSlot.FACTORY_TRIM);
                if (isPack && p2.selectedPart && p2.selectedPart.linkedParts && p2.selectedPart.linkedParts.length) {
                    for (var j:int = 0; j < p2.selectedPart.linkedParts.length; j++) {
                        if (partSlots.indexOf(Part(p2.selectedPart.linkedParts[j]).parentSlot) == -1) {
                            partSlots.push(Part(p2.selectedPart.linkedParts[j]).parentSlot);
                        }
                    }
                }
            }
        }


        for each(var partSlot:PartSlot in partSlots) {
            if (partSlot.selectedPart && partSlot.selectedPart.selectedMaterial &&  (partSlot.selectedPart.selectedMaterial.isPaintable || partSlot.selectedPart.selectedMaterial.name == "metal")) {
                if (partSlots.length != car.selectedPartSlots.length || (partSlots.length == car.selectedPartSlots.length && partSlot.name != "windows")) {
                    partSlot.selectedColor = partColor.copy();
                    if(partSlot.selectedPart && partSlot.selectedPart.color){
                        partSlot.selectedPart.color = partSlot.selectedColor;
                    }
                }

                if (partSlot.name == "body") {
                    car.bodyPartColor.value = partColor.value;
                }
            } else if (!partSlot.selectedPart && partSlot.possibleParts.length > 0) {


                var part:Part = partSlot.possibleParts[0];

                if (part && part.selectedMaterial && part.selectedMaterial.isPaintable) {
                    partSlot.selectedColor = partColor;
                }
            }
        }

        if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == "frame") {

            var ps:PartSlot = updateFrameBackIfNeeded();
            if (ps) partSlots.push(ps);
        }

        if (allSlots) {
            tuningScreen.carView.allSlotUpdate = true;
            tuningScreen.lastAllColor = partColor.value;
            tuningScreen.allColorIsChange = true;

        }




        tuningScreen.carView.updateWithPartSlots(partSlots, true);
        //if(partSlots.length>10)
        tuningScreen.carView.allSlotUpdate = false;
    }

    private function onChangeNeonForDiskState(event:starling.events.Event):void {
        var selectedState:String = event.data.selectedState;

        if (!tuningScreen.selectedPartSlot)
            return;
        var part:Part = tuningScreen.selectedPartSlot.selectedPart;//event.data.part;

        if (this.tuningScreen.partDiskSettingView) {

            if (selectedState != "neon") {

                part.selectedState = null;
            }

            this.tuningScreen.partDiskSettingView.doChangeData(selectedState == "neon")
        }


    }

    private var fender_car_ids:Array = ["55c60df93d170ef40ce753d9",
        "55bf8946f9aa53f848e7bbb2",
        "5503e77137d1ba00398323c1",
        "5506ff905e4453ac192e3839",
        "5571bc788647f6380c22dc14",
        "5512a1d7726223643418e31e",
        "55729bc88647f6380c22dc9c",
        "557985dc699fadd43ffb2f6a",
        "557940a0699fadd43ffb2e79",
        "5512a23b726223643418e351",
        "557ac8090b916b10071f9c60",
        "55c0b3a87cb8f49412d59aa5",
        "55c242247cb8f49412d59bf1",
        "55c0d27a7cb8f49412d59baf",
        "550bc11b6889354c4c445e42",
        "551284e3726223643418e0c1"];

    private function tuningScreenStateSelected(event:starling.events.Event):void {
        if(this.tuningScreen.carView.progressHandler){
            this.tuningScreen.carView.progressHandler(0);
            this.tuningScreen.carView.progressHandler(0.01);
        }

        var selectedState:String = event.data.selectedState;
        trace("selectedState: " + selectedState);

        var stateType:String = event.data.stateType;  //disk_neon
        trace("stateType: " + stateType);

        var updateWithPartSlots:Boolean = !(event && event.data && event.data.hasOwnProperty("updateWithPartSlots") && !event.data.updateWithPartSlots);
        // TODO : optimize for disk changes
        setTimeout(applyState, 200, stateType, selectedState, true);
    }

    private function applyState(stateType:String, selectedState:String, updateWithPartSlots:Boolean = true):void {
        var additionalInfo:String = stateType?stateType+", ":"" + selectedState?selectedState:"";
        ErrorTracker.setReference("TuningScreenMediator/applyState: " + additionalInfo);
        ErrorTracker.setLineLabel("#1");
        var fenderFlaresPartSlot:PartSlot;
        if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart) {
            tuningScreen.carView.removeMaskImages(tuningScreen.selectedPartSlot.selectedPart);
        }
        ErrorTracker.setLineLabel("#2");
        var part:Part = tuningScreen.selectedPartSlot.selectedPart;
        var slotsForUpdate:Array = [tuningScreen.selectedPartSlot];
        var selectedStateName:String;
        var i:int;
        ErrorTracker.setLineLabel("#3");
        if(car.suvTires) {
            var diskPair : PartSlot;
            switch (tuningScreen.selectedPartSlot.name) {
                case "disk_front_left":
                    diskPair = car.getPossiblePartSlot("disk_front_right");
                    break;
                case "disk_front_right":
                    diskPair = car.getPossiblePartSlot("disk_front_left");
                    break;
                case "disk_rear_left":
                    diskPair = car.getPossiblePartSlot("disk_rear_right");
                    break;
                case "disk_rear_right":
                    diskPair = car.getPossiblePartSlot("disk_rear_left");
                    break;
            }
            ErrorTracker.setLineLabel("#4");
            if(diskPair) {
                if (diskPair.selectedPart) {
                    tuningScreen.carView.removeMaskImages(diskPair.selectedPart);
                }
                ErrorTracker.setLineLabel("#5");
                if(part && part is PartDisk) {
                    var pairDiskSlot:PartSlotDisk = diskPair as PartSlotDisk;
                    var diskPart:PartDisk = part as PartDisk;
                    var pairPart:PartDisk;
                    var obj:Object = pairDiskSlot.findPartDiskForRadius(diskPart.name, diskPart.radius, diskPart.profile);
                    if (stateType == 'disk_neon' && part.selectedState) {
                        selectedStateName = part.selectedState.nameOfState;
                    }
                    if((!obj || !obj.hasOwnProperty("part")) && diskPart.manufacturer == "factory"){
                        obj = pairDiskSlot.findPartDiskForRadius("{factory}", diskPart.radius, diskPart.profile);
                    }
                    if(obj && obj.hasOwnProperty("part")){
                        pairPart = obj.part as PartDisk;
                    }
                    if(pairPart) {
                        if (selectedStateName) {
                            pairPart.selectedState = pairPart.states[selectedStateName];
                        } else if (selectedState) {
                            pairPart.selectedState = pairPart.states[selectedState];
                        } else {
                            pairPart.selectedState = null;
                        }
                        diskPair.selectedPart = pairPart;
                        slotsForUpdate.push(diskPair);
                    }
                }
                ErrorTracker.setLineLabel("#6");
            }
        }
        ErrorTracker.setLineLabel("#7");
        var isLight:Boolean = (stateType && stateType.indexOf("light") != -1);
        if (tuningScreen.selectedPartSlot && (selectedState == "lights" || isLight)) {
            for(var k:int = 0; k < car.possiblePartSlots.length; k++){
                var lightPartSlot:PartSlot = car.possiblePartSlots[k] as PartSlot;
                if(lightPartSlot && lightPartSlot.selectedPart && lightPartSlot.selectedPart.states && lightPartSlot.selectedPart.states["lights"]){
                    if(slotsForUpdate.indexOf(lightPartSlot)==-1){
                        slotsForUpdate.push(lightPartSlot);
                    }else{
                        continue;
                    }
                    if (selectedState) {
                        lightPartSlot.selectedPart.selectedState = lightPartSlot.selectedPart.states["lights"];
                    } else {
                        lightPartSlot.selectedPart.selectedState = null;
                    }
                    if(slotsForUpdate.indexOf(lightPartSlot)==-1){
                        slotsForUpdate.push(lightPartSlot);
                    }
                }
            }
        }
        ErrorTracker.setLineLabel("#8");
        if(stateType=="disk_neon"){
            if(Car.CAR_DISKS_SLOTS.indexOf(part.parentSlot.name ) >= 0 ){
                var disksSlots:Array =[].concat(car.possiblePartSlotsDisks);// [disksPartSlot, tiresPartSlot];
                var p : PartSlot;
                for each(var ts : String in Car.TIRES_SLOTS) {
                    p = car.getPossiblePartSlot(ts);
                    if(p){
                        disksSlots.push(p);
                    }
                }
                ErrorTracker.setLineLabel("#9");
                // tire vinyls slots
                for each(var vs : String in Car.TIRE_VINYL_PARTS) {
                    p = car.getPossiblePartSlot(vs);
                    if(p){
                        disksSlots.push(p);
                    }
                }
                ErrorTracker.setLineLabel("#10");
                for(i = 0; i<disksSlots.length; i++){

                    var disksPartSlot:PartSlot = disksSlots[i];

                    if(disksPartSlot && disksPartSlot.selectedPart){

                        trace("disksPartSlot:", disksPartSlot.name);
                        tuningScreen.carView.removeMaskImages(disksPartSlot.selectedPart);
                        ErrorTracker.setLineLabel("#11");
                        var currentStateName:String = "";
                        if(disksPartSlot.selectedPart.selectedState){
                            currentStateName = disksPartSlot.selectedPart.selectedState.nameOfState;
                        }

                        if(selectedState){ //ставим
                            if(currentStateName.length>0) {
                                if(Car.TIRES_SLOTS.indexOf(disksPartSlot.name) < 0){
                                    currentStateName=Car.statesMapInvert[currentStateName];
                                }else if(currentStateName=="neon") {
                                    currentStateName = "";
                                }
                            }else{
                                if(Car.TIRES_SLOTS.indexOf(disksPartSlot.name) < 0){
                                    currentStateName="neon";
                                }
                            }
                            ErrorTracker.setLineLabel("#12");
                            //if(currentStateName.indexOf("neon")>=0 && disksPartSlot.name!="tires_front" && disksPartSlot.name!="tires_rear")
                            if(disksPartSlot.selectedPart.states[currentStateName]){
                                disksPartSlot.selectedPart.selectedState = disksPartSlot.selectedPart.states[currentStateName];
                                slotsForUpdate.push(disksPartSlot);
                            }
                        }else{ //снимаем
                            ErrorTracker.setLineLabel("#13");
                            if(currentStateName.indexOf("-neon")>=0 || currentStateName.indexOf("neon-")>=0 || currentStateName.indexOf("-neon-")>=0){
                                //currentStateName = currentStateName.replace("-neon-", "");
                                //currentStateName = currentStateName.replace("-neon", "");
                                //currentStateName = currentStateName.replace("neon-", "");
                                if(Car.TIRES_SLOTS.indexOf(disksPartSlot.name) < 0)
                                    currentStateName = Car.statesMap[currentStateName];
                            }

                            if(currentStateName=="neon"){

                                disksPartSlot.selectedPart.selectedState = null;
                            }else if(disksPartSlot.selectedPart.states[currentStateName]){

                                disksPartSlot.selectedPart.selectedState = disksPartSlot.selectedPart.states[currentStateName];
                            }else{
                                disksPartSlot.selectedPart.selectedState = null;
                            }
                            ErrorTracker.setLineLabel("#14");
                            if(disksPartSlot.selectedPart.getPossibleMaterial("metal")){

                                disksPartSlot.selectedPart.selectedMaterial = disksPartSlot.selectedPart.getPossibleMaterial("metal");
                            }

                            slotsForUpdate.push(disksPartSlot);
                        }

                    }

                }


            }
        }else {
            ErrorTracker.setLineLabel("#15");
            if (selectedState) {

                if(stateType !="disk_neon")
                {
                    part.selectedState = part.states[selectedState];
                }

                if (selectedState && selectedState.indexOf("neon")<0 && selectedState != "lights") { //тут может быть любой стейт кроме неона и фар

                    ErrorTracker.setLineLabel("#16");
                    if (Car.CAR_DISKS_SLOTS.indexOf(part.parentSlot.name )>=0) {

                        if (selectedState && (selectedState.indexOf("of1") >= 0 || selectedState.indexOf("of2") >= 0)) {
                            fenderFlaresPartSlot = car.getPossiblePartSlot("fender_flares");

                            if (fender_car_ids.indexOf(car.id) >= 0) {
                                if (fenderFlaresPartSlot) {

                                    if (fenderFlaresPartSlot.possibleParts[0]) {
                                        fenderFlaresPartSlot.selectedPart = fenderFlaresPartSlot.possibleParts[0];
                                        fenderFlaresPartSlot.isMandatory = true;
                                        slotsForUpdate.push(fenderFlaresPartSlot);
                                        car.selectedPartSlots.push(fenderFlaresPartSlot);
                                    }
                                }
                            }
                            ErrorTracker.setLineLabel("#17");
                        } else {
                            ErrorTracker.setLineLabel("#18");
                            fenderFlaresPartSlot = car.getPossiblePartSlot("fender_flares");
                            if (fender_car_ids.indexOf(car.id) >= 0) {
                                if (fenderFlaresPartSlot) {
                                    fenderFlaresPartSlot.isMandatory = false;
                                    slotsForUpdate.push(fenderFlaresPartSlot);
                                }
                            }
                        }
                        ErrorTracker.setLineLabel("#19");
                        if (part.camberLock) {
                            tuningScreen.showBuyPacket(car.packet);
                            return;
                        }

                        tuningScreen.carView.isExistCember = true;
                        ErrorTracker.setLineLabel("#20");
                    }

                } else {
                    ErrorTracker.setLineLabel("#21");
                    if (Car.CAR_DISKS_SLOTS.indexOf(part.parentSlot.name )>=0) {
                        if (fender_car_ids.indexOf(car.id) >= 0) {
                            fenderFlaresPartSlot = car.getPossiblePartSlot("fender_flares");//tuningScreen.possiblePartSlots[slotIndex];
                            if (fenderFlaresPartSlot) {
                                fenderFlaresPartSlot.isMandatory = false;
                                slotsForUpdate.push(fenderFlaresPartSlot);
                            }
                        }

                        tuningScreen.carView.isExistCember = false;
                        tuningScreen.invalidate('car');
                    }
                    ErrorTracker.setLineLabel("#22");
                }


            } else {
                ErrorTracker.setLineLabel("#23");
                var oldState:String="";

                if(part.selectedState)
                    oldState = part.selectedState.nameOfState;

                part.selectedState = null;

                if (oldState != "lights"){
                    if(!part.selectedMaterial) {
                        part.selectedMaterial = part.oldSelectedMaterial;
                    }
                }
                ErrorTracker.setLineLabel("#24");

                tuningScreen.selectedPartSlot.currentPartWithState = null;
                if (Car.CAR_DISKS_SLOTS.indexOf(part.parentSlot.name )>=0) {
                    fenderFlaresPartSlot = car.getPossiblePartSlot("fender_flares");//tuningScreen.possiblePartSlots[slotIndex];
                    if (fender_car_ids.indexOf(car.id) >= 0) {
                        if (fenderFlaresPartSlot) {
                            fenderFlaresPartSlot.isMandatory = false;
                            slotsForUpdate.push(fenderFlaresPartSlot);
                        }
                    }
                    tuningScreen.carView.isExistCember = false;
                    tuningScreen.invalidate('car');
                }
                ErrorTracker.setLineLabel("#25");
            }
        }
        ErrorTracker.setLineLabel("#26");
        if ((selectedState && selectedState.indexOf("neon") >= 0) || selectedState == "lights") {
            tuningScreen.carView.neonOn = true;
            tuningScreen.carView.neonOn = true
        }
        ErrorTracker.setLineLabel("#27");
        var selectedStateForTairsAndBrake:String;
        var state:Part;
        if(car.possiblePartSlotsDisks.length>0) {
            for each (var disk : PartSlotDisk in car.possiblePartSlotsDisks) {
                if(disk && disk.selectedPart && disk.name.indexOf("disk")>= 0) {
                    var prefix : String = disk.name.indexOf("disks") >= 0 ? disk.name.replace("disks", "") : disk.name.replace("disk", "");

                    selectedStateForTairsAndBrake = disk.selectedPart.selectedState &&
                    disk.selectedPart.selectedState.nameOfState.indexOf("neon")>=0 ? Car.statesMapForTairsAndBrake[disk.selectedPart.selectedState.nameOfState]:
                            disk.selectedPart.selectedState ? disk.selectedPart.selectedState.nameOfState: null;
                    ErrorTracker.setLineLabel("#28");
                    var tirePart:PartSlot = car.getPossiblePartSlot('tires' + prefix);
                    if (!tirePart) {
                        tirePart = car.getPossiblePartSlot('tire' + prefix);
                    }
                    var brakePartSlot:PartSlot = car.getPossiblePartSlot('brake' + prefix);
                    if(tirePart) {
                        state = tirePart.selectedPart.states[selectedStateForTairsAndBrake];
                        tirePart.selectedPart.selectedState = state ? state : null;
                        if(slotsForUpdate.indexOf(tirePart) == -1) {
                            slotsForUpdate.push(tirePart);
                        }
                    }
                    ErrorTracker.setLineLabel("#29");
                    // *************************** vinyls ************************
                    var vinylPart:PartSlot = car.getPossiblePartSlot('tire_vinyls' + prefix);
                    if (!vinylPart) {
                        vinylPart = car.getPossiblePartSlot('tire_vinyl' + prefix);
                    }
                    ErrorTracker.setLineLabel("#30");
                    if(vinylPart && vinylPart.selectedPart) {
                        state = vinylPart.selectedPart.states[selectedStateForTairsAndBrake];
                        vinylPart.selectedPart.selectedState = state ? state : null;
                        if(slotsForUpdate.indexOf(vinylPart)<0)
                            slotsForUpdate.push(vinylPart);
                    }
                    // ************************* end vinyls **********************
                    ErrorTracker.setLineLabel("#31");
                    if(brakePartSlot && brakePartSlot.selectedPart) {
                        state = brakePartSlot.selectedPart.states[selectedStateForTairsAndBrake];
                        brakePartSlot.selectedPart.selectedState = state ? state : null;

                        if(slotsForUpdate.indexOf(brakePartSlot)<0)
                            slotsForUpdate.push(brakePartSlot);
                    }
                    ErrorTracker.setLineLabel("#32");
                    var shadowPart: PartSlot = car.getPossiblePartSlot('shadow' + prefix);
                    if (shadowPart && shadowPart.selectedPart) {
                        state = shadowPart.selectedPart.states[selectedStateForTairsAndBrake];
                        shadowPart.selectedPart.selectedState = state ? state : null;
                        if(slotsForUpdate.indexOf(shadowPart)<0) {
                            slotsForUpdate.push(shadowPart);
                        }
                    }
                    ErrorTracker.setLineLabel("#33");
                }
            }

        }

        ErrorTracker.setLineLabel("#34");
        if (car.componentsDependenciesWheel) {
            for (i = 0; i < car.componentsDependenciesWheel.length; i++) {
                var dependency:Object = car.componentsDependenciesWheel[i];
                for (var slotName:String in dependency) {
                    for each (var inSlotName:String in dependency[slotName]) {
                        for (var j:int = 0; j < slotsForUpdate.length; j++) {
                            var updatedSlotName:String = slotsForUpdate[j].name;
                            var slot:PartSlot = null;
                            //В  dependency[slotName] все слоты с которым связан updatedSlotName
                            if (slotName == updatedSlotName) {
                                for (var index:int in dependency[slotName]) {
                                    slot = car.getPossiblePartSlot(dependency[slotName][index]);
                                    if (slot && slot.selectedPart) {
                                        slot.selectedPart.selectedState = selectedState ? slot.selectedPart.states[selectedState] : null;
                                        slotsForUpdate.push(slot);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        ErrorTracker.setLineLabel("#35");
        var skipBeforeLoading:Boolean = false;
        for(i = 0; i < slotsForUpdate.length; i++){
            if(PartSlot(slotsForUpdate[i]).name == "disks"){
                skipBeforeLoading = true;
                break;
            }
        }
        ErrorTracker.setLineLabel("#36");
        tuningScreen.carView.beforeLoading(true, skipBeforeLoading);
        var newSlotsForUpdate:Array = [];
        for each(var partSlot:PartSlot in slotsForUpdate){
            if(newSlotsForUpdate.indexOf(partSlot) == -1) {
                newSlotsForUpdate.push(partSlot);
            }
        }
        ErrorTracker.setLineLabel("#37");
        // ***********************************************************
        tuningScreen.carView.resetFastModeCurrentLoaded();
        // ***********************************************************

        if(updateWithPartSlots){
            tuningScreen.carView.updateWithPartSlots(newSlotsForUpdate);
        }
        tuningScreen.carView.neonOn = false;
        modifiedTuning = true;
        updateColorViewEnabled();
        if(stateType=="disk_neon"){
            tuningScreen.selectedPartSlot = tuningScreen.oldSelectedPartSlot;
        }

        tuningScreen.carView.filteringSuspension();

        ErrorTracker.setLineLabel(null);
        ErrorTracker.setReference(null);
    }

    private var materialData:Object;
    private var neededData:Array = [];
    private var materialSelectSharedIndex:int = 0;
    private var delay:int = 10;

    private function tuningScreenMaterialSelected(event:starling.events.Event) : void
    {
        if (this.tuningScreen && this.tuningScreen.progressBar && this.tuningScreen.progressBar.visible) {
            return;
        }


        tuningScreen.setPartListTouchable(false);
        tuningScreen.allPartsColorsView.materialList.touchable = false;


        var onComplete:Function = null;
        if (event.data && event.data.onComplete) {
            onComplete = event.data.onComplete;
        }

        materialSelectSharedIndex = -1;
        neededData.length = 0;

        materialData = event.data;
        if(tuningScreen.allPartsColorsView) {
            //tuningScreen.allPartsColorsView.invalidate("updatePartSlot");
        }
        if (tuningScreen.carView.progressHandler){// calling this for user, because the delay is about 1-2 seconds
            tuningScreen.carView.progressHandler(0);
            tuningScreen.carView.progressHandler(0.01);
        }
        var timeDelay:Number = tuningScreen.carView.previewMode?0:200;
        var updateWithPartSlots:Boolean = !(event && event.data && event.data.hasOwnProperty("updateWithPartSlots") && !event.data.updateWithPartSlots);
        setTimeout(function():void{

            var partSlot:PartSlot;
            if (neededData.length == 0) {
                for each(partSlot in car.possiblePartSlots) {
                    if(Car.EXTERIOR_EXCLUDE.indexOf(partSlot.name) == -1){
                        neededData.push(partSlot);
                    }
                }
            }

            tuningScreenMaterialParse(updateWithPartSlots);
            if(tuningScreen.allPartsColorsView) {
                //tuningScreen.allPartsColorsView.invalidate("updatePartSlot");
            }
        },timeDelay);

    }

    private function tuningScreenMaterialParse(updateWithPartSlots:Boolean = true):void {
        if(car == null) {
            return;
        }
        ErrorTracker.setReference("TuningScreenMediator/tuningScreenMaterialParse");
        ErrorTracker.setLineLabel("#1");
        var selectedColor:PartColor = (materialData?materialData.selectedColor:null);
        var partMaterial2:PartMaterial;
        var selectedMaterial:PartMaterial;
        var partMaterial:PartMaterial;
        var part:Part;
        var i:int;
        // ***********************************************************
        if(!tuningScreen || !tuningScreen.carView){
            return;
        }
        ErrorTracker.setLineLabel("#2");
        tuningScreen.carView.resetFastModeCurrentLoaded();
        // ***********************************************************
        if (materialData && materialData.partSlot && !materialData.partSlot.dummy) {
            ErrorTracker.setLineLabel("#3");
            if (this.tuningScreen.carView.visible){
                this.tuningScreen.carView.beforeLoading();
            }

            if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart) {
                tuningScreen.carView.removeMaskImages(tuningScreen.selectedPartSlot.selectedPart);
            }
            ErrorTracker.setLineLabel("#4");
            var eventSelectedMaterial:PartMaterial;
            if(tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart && materialData.selectedMaterial){
                eventSelectedMaterial = tuningScreen.selectedPartSlot.selectedPart.getPossibleMaterial(materialData.selectedMaterial.name);
            }
            var paintableRestored:Boolean = false;
            ErrorTracker.setLineLabel("#5");
            if (eventSelectedMaterial && tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart) {
                if(tuningScreen.selectedPartSlot.selectedPart.selectedMaterial){
                    paintableRestored = (!tuningScreen.selectedPartSlot.selectedPart.selectedMaterial.isPaintable && eventSelectedMaterial.isPaintable);
                }else{
                    paintableRestored = eventSelectedMaterial.isPaintable;
                }
                tuningScreen.selectedPartSlot.selectedPart.selectedMaterial = eventSelectedMaterial;
                ErrorTracker.setLineLabel("#6");
                if (!eventSelectedMaterial.isPaintable) {
                    tuningScreen.selectedPartSlot.previousColor ||= tuningScreen.selectedPartSlot.selectedColor;
                    tuningScreen.selectedPartSlot.selectedColor = PartColor.noColor();
                } else if (paintableRestored && tuningScreen.selectedPartSlot.previousColor) {
                    tuningScreen.selectedPartSlot.selectedColor = tuningScreen.selectedPartSlot.previousColor;
                } else {
                    tuningScreen.selectedPartSlot.selectedColor ||= car.bodyPartColor.copy();
                }
                ErrorTracker.setLineLabel("#7");
                if (Car.CAR_DISKS_SLOTS.indexOf(tuningScreen.selectedPartSlot.selectedPart.name) >= 0 &&
                        tuningScreen.selectedPartSlot.selectedPart.selectedMaterial.name != "metal" &&
                        tuningScreen.selectedPartSlot.selectedColor.value == 0xFFFFFF) {
                    tuningScreen.selectedPartSlot.selectedColor.value = 0xEEEEEE;
                }
                ErrorTracker.setLineLabel("#8");
            }

            ErrorTracker.setLineLabel("#9");
            var partSlots:Array;
            selectedMaterial = materialData.selectedMaterial;
            if(tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart && tuningScreen.selectedPartSlot.selectedPart.linkedParts && (tuningScreen.selectedPartSlot.name == PartSlot.BODY_PACK || tuningScreen.selectedPartSlot.name == PartSlot.FACTORY_TRIM)){
                partSlots = [tuningScreen.selectedPartSlot];
                ErrorTracker.setLineLabel("#10");
                for (var k:int = 0; k < tuningScreen.selectedPartSlot.selectedPart.linkedParts.length; k++) {
                    var partSlot:PartSlot = (tuningScreen.selectedPartSlot.selectedPart.linkedParts[k] as Part).parentSlot;
                    if (partSlot.selectedPart) {
                        if(partSlots.indexOf(partSlot) == -1){
                            partSlots.push(partSlot);
                        }
                        ErrorTracker.setLineLabel("#11");
                        partMaterial = partSlot.selectedPart.getPossibleMaterial(selectedMaterial.name);
                        if (partMaterial) {
                            partSlot.selectedPart.selectedMaterial = partMaterial;
                            part = partSlot.selectedPart;
                            if (part.selectedMaterial.isPaintable) {
                                if (part.color) {
                                    if (part.color && part.color.value == 0xffffff) {
                                        part.color = car.bodyPartColor.copy();
                                        part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                                    } else {
                                        part.color ||= car.bodyPartColor.copy();
                                        part.parentSlot.selectedColor.value = part.color.value;
                                    }
                                } else {
                                    part.color = car.bodyPartColor.copy();
                                    part.parentSlot.selectedColor = car.bodyPartColor.copy();
                                }
                            } else {
                                ErrorTracker.setLineLabel("#12");
                                part.color = PartColor.noColor();
                                part.parentSlot.selectedColor = PartColor.noColor();
                            }
                        }
                    } else {
                        /*
                         Если детель не выбрана то установим для всех возможных деталей выбранный материал если он есть для детали
                         */
                        ErrorTracker.setLineLabel("#13");
                        for each(part in partSlot.possibleParts){
                            partMaterial2 = part.getPossibleMaterial(selectedMaterial.name);
                            if (partMaterial2) {
                                part.selectedMaterial = partMaterial2;
                                if (part.selectedMaterial.name != "metal" && part.selectedMaterial.name != "white") {
                                    if (part.parentSlot.selectedColor) {
                                        if (part.parentSlot.selectedColor.value == 0xffffff) {
                                            part.color = car.bodyPartColor.copy();
                                            part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                                        }
                                    } else {
                                        part.color ||= car.bodyPartColor.copy();
                                        part.parentSlot.selectedColor ||= car.bodyPartColor.copy();
                                    }
                                } else {
                                    part.color = PartColor.noColor();
                                    part.parentSlot.selectedColor = PartColor.noColor();
                                }
                            }
                        }
                        ErrorTracker.setLineLabel("#14");
                    }
                    ErrorTracker.setLineLabel("#15");
                    if (!selectedMaterial.isPaintable) {
                        partSlot.previousColor = partSlot.selectedColor;
                        partSlot.selectedColor = PartColor.noColor();

                        if (partSlot.name == "rear_lights") {
                            partSlot.selectedColor.value = 0xffffff;
                        }
                    } else if (partSlot.name == "rear_lights" && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && partSlot.selectedPart.selectedMaterial.name == "metal") {
                        ErrorTracker.setLineLabel("#16");
                        partSlot.previousColor = partSlot.selectedColor;
                        partSlot.selectedColor = PartColor.noColor();

                        if (partSlot.name == "rear_lights") {
                            partSlot.selectedColor.value = 0xffffff;
                        }
                    } else if (partSlot.previousColor) {
                        partSlot.selectedColor = partSlot.previousColor;
                    }
                    ErrorTracker.setLineLabel("#17");
                    if (partSlot.name == "disks" &&
                            partSlot.selectedPart.selectedMaterial.name != "metal" &&
                            partSlot.selectedColor.value == 0xFFFFFF) {
                        partSlot.selectedColor.value = 0xEEEEEE;
                    }
                    ErrorTracker.setLineLabel("#18");
                }
            }else if(tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == TuningColorsView.INTERIOR){
                partSlots = car.interiorSelectedPartSlots;
            }else if(tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.name == TuningColorsView.EXTERIOR){
                partSlots = car.exteriorSelectedPartSlots;
            }else{
                partSlots = [tuningScreen.selectedPartSlot];
            }
            ErrorTracker.setLineLabel("#19");
            /**
             * DOORS  and WINDOWS SYNC
             */
            /*var pairPartSlot : PartSlot;
            if(pairPartSlot){
                pairPartSlot.selectedPart.selectedMaterial = pairPartSlot.selectedPart.getPossibleMaterial(materialData.selectedMaterial.name);
                if(materialData.selectedMaterial.isPaintable)
                {
                    pairPartSlot.selectedColor = tuningScreen.selectedPartSlot.selectedColor;
                }
                partSlots.push(pairPartSlot);
            }*/

            /**
             * DISKS SYNC
             */
            if(car.suvTires && tuningScreen.selectedPartSlot) {
                if(Car.CAR_DISKS_SLOTS.indexOf(tuningScreen.selectedPartSlot.name ) >= 0) {
                    var diskPair:PartSlotDisk;

                    switch (tuningScreen.selectedPartSlot.name) {
                        case "disk_front_left":
                            diskPair = car.getPossiblePartSlot("disk_front_right") as PartSlotDisk;
                            break;
                        case "disk_front_right":
                            diskPair = car.getPossiblePartSlot("disk_front_left") as PartSlotDisk;
                            break;
                        case "disk_rear_left":
                            diskPair = car.getPossiblePartSlot("disk_rear_right") as PartSlotDisk;
                            break;
                        case "disk_rear_right":
                            diskPair = car.getPossiblePartSlot("disk_rear_left") as PartSlotDisk;
                            break;
                    }
                    if (diskPair && partSlots.indexOf(diskPair) < 0) {
                        diskPair.selectedPart.selectedMaterial = diskPair.selectedPart.getPossibleMaterial(materialData.selectedMaterial.name);
                        if(materialData.selectedMaterial.isPaintable) {
                            diskPair.selectedColor = tuningScreen.selectedPartSlot.selectedColor;
                        }
                        partSlots.push(diskPair);
                    }
                }
            }
            ErrorTracker.setLineLabel("#20");
            var pairPart:PartSlot;
            var pairPartName:String;
            if(tuningScreen.selectedPartSlot){
                if(tuningScreen.selectedPartSlot.name.indexOf("left") != -1){
                    pairPartName = tuningScreen.selectedPartSlot.name.replace("left", "right");
                }else if(tuningScreen.selectedPartSlot.name.indexOf("right") != -1){
                    pairPartName = tuningScreen.selectedPartSlot.name.replace("right", "left");
                }
            }

            ErrorTracker.setLineLabel("#21");
            if(pairPartName){
                pairPart = car.getPossiblePartSlot(pairPartName);
                if (pairPart && partSlots.indexOf(pairPart) < 0) {
                    pairPart.selectedPart.selectedMaterial = pairPart.selectedPart.getPossibleMaterial(materialData.selectedMaterial.name);
                    if(materialData.selectedMaterial.isPaintable) {
                        pairPart.selectedColor = tuningScreen.selectedPartSlot.selectedColor;
                    }
                    partSlots.push(pairPart);
                }
            }

            ErrorTracker.setLineLabel("#22");
            /*if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart && tuningScreen.selectedPartSlot.selectedPart.linkedParts && tuningScreen.selectedPartSlot.selectedPart.linkedParts.length > 0) {
                for (i = 0; i < tuningScreen.selectedPartSlot.selectedPart.linkedParts.length; i++) {
                    var linkedPart:Part = tuningScreen.selectedPartSlot.selectedPart.linkedParts[i];
                    partSlots.push(linkedPart.parentSlot);
                }
            }*/

            ErrorTracker.setLineLabel("#23");
            var frameBackPartSlot:PartSlot = updateFrameBackIfNeeded();
            if (frameBackPartSlot) {
                partSlots.push(frameBackPartSlot);
            }
            if(!selectedColor && materialData.selectedMaterial.isPaintable){
                selectedColor = car.getPossiblePartSlot('body').selectedColor;
            }

            ErrorTracker.setLineLabel("#24");
            if(!paintableRestored || (tuningScreen.selectedPartSlot.name == TuningColorsView.EXTERIOR && !materialData.selectedColor)){
                selectedColor = null;
            }

            if(selectedColor) {
                updatePartSlotsWithColor(partSlots, selectedColor, false);
            }

            ErrorTracker.setLineLabel("#25");
            var updateColor:Boolean = false;
            for(i = partSlots.length - 1; i >= 0; i--){
                if(partSlots.indexOf(partSlots[i]) != i || !PartSlot(partSlots[i]).selectedPart){
                    partSlots.removeAt(i);
                    continue;
                }
                if(PartSlot(partSlots[i]).selectedPart.getPossibleMaterial(materialData.selectedMaterial.name)){
                    var currentPartColor:Boolean = false;
                    if((!PartSlot(partSlots[i]).selectedPart.selectedMaterial || !PartSlot(partSlots[i]).selectedPart.selectedMaterial.isPaintable) && materialData.selectedMaterial.isPaintable){
                        updateColor = true;
                        currentPartColor = true;
                    }
                    PartSlot(partSlots[i]).selectedPart.selectedMaterial = PartSlot(partSlots[i]).selectedPart.getPossibleMaterial(materialData.selectedMaterial.name);
                    if(materialData.selectedMaterial.isPaintable && materialData.selectedColor){
                        PartSlot(partSlots[i]).selectedPart.color = PartSlot(partSlots[i]).selectedColor = materialData.selectedColor;
                    }else if(currentPartColor){
                        PartSlot(partSlots[i]).selectedPart.color = PartSlot(partSlots[i]).selectedColor = PartSlot(partSlots[i]).previousColor;
                    }
                }
            }

            ErrorTracker.setLineLabel("#26");
            if(updateColor){
                tuningScreen.carView.updateWithPartSlots(partSlots, true);
            }

            if(updateWithPartSlots){
                tuningScreen.carView.updateWithPartSlots(partSlots, false, function ():void {
                    if (tuningScreen.carView.progressHandler) {
                        tuningScreen.carView.progressHandler(1);
                    }
                });
            }

            ErrorTracker.setLineLabel("#27");
            tuningScreen.colorsViewEnabled = materialData.selectedMaterial.isPaintable;
        } else {
            ErrorTracker.setLineLabel("#28");
            //Смена всех слотов
            materialSelectSharedIndex++;
            if(materialSelectSharedIndex == neededData.length) {
                return;
            }
            selectedMaterial = materialData.selectedMaterial;
            if (car.pizdec) {
                ErrorTracker.setLineLabel("#29");
                Starling.juggler.delayCall(function () :void{
                    AlertView.show("Все изменения будут стерты, продолжить?", -1, false, function (yesSelected:Boolean):void {
                        if (yesSelected) {
                            BrandsScreen.selectedMaterialName = selectedMaterial.name;
                            BrandsScreen.selectedCarId = car.id;
                            //Делаем скрин экрана чтоб прикрыть пиздец
                            var bitmapData:BitmapData = tuningScreen.copyToBitmap();
                            var ts:Number = 1;
                            if (bitmapData.width > 1024) {
                                ts = 1024 / bitmapData.width;
                                var newBitmapData:BitmapData = SnapShotFramesContainer.scaleBitmapData(bitmapData, ts);
                                bitmapData.dispose();
                                bitmapData = newBitmapData;
                            }

                            if (BrandsScreen.screenShotImage) {
                                BrandsScreen.screenShotImage.removeFromParent(true);
                                BrandsScreen.screenShotImage = null;
                            }

                            if (BrandsScreen.screenShotTexture) {
                                BrandsScreen.screenShotTexture.dispose();
                                BrandsScreen.screenShotTexture = null;
                            }
                            BrandsScreen.screenShotTexture = Texture.fromBitmapData(bitmapData, false, false, 1);
                            //имеем все данные чтоб сделать пиздец
                            sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN, {backLevel: stackedScreen.backLevel});
                        } else {
                            //tuningScreen.allPartSlotsColorView.cancelChangeMaterial();
                        }
                    });
                }, 0.5);
                return;
            }
            ErrorTracker.setLineLabel("#30");
            if(tuningScreen.selectedPartSlot.selectedPart){
                tuningScreen.selectedPartSlot.selectedPart.selectedMaterial = selectedMaterial;
            }
            ErrorTracker.setLineLabel("#31");
            if (materialSelectSharedIndex < neededData.length - 1) {
                var timeDelay:Number = tuningScreen.carView.previewMode?0:delay;
                setTimeout(tuningScreenMaterialParse, timeDelay);
            }
            ErrorTracker.setLineLabel("#32");
            partSlot = neededData[materialSelectSharedIndex];
            if (Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0) {
                if (materialSelectSharedIndex == neededData.length - 1) {
                    if(this.tuningScreen.carView.visible) {
                        this.tuningScreen.carView.beforeLoading();
                    }
                    if (selectedColor) {
                        updatePartSlotsWithColor(car.possiblePartSlots, selectedColor, true);
                    }
                    if(updateWithPartSlots){
                        tuningScreen.carView.updateWithPartSlots(car.possiblePartSlots, false);
                    }
                    modifiedTuning = true;
                }
                return;
            }
            ErrorTracker.setLineLabel("#33");
            if (partSlot.selectedPart ) {
                partMaterial = partSlot.selectedPart.getPossibleMaterial(selectedMaterial.name);
                if (partMaterial) {
                    partSlot.selectedPart.selectedMaterial = partMaterial;

                    part = partSlot.selectedPart;

                    if (part.selectedMaterial.name != "metal" && part.selectedMaterial.name != "white") {
                        if (part.color) {
                            if (part.color && part.color.value == 0xffffff) {
                                part.color = car.bodyPartColor.copy();
                                part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                            } else {
                                part.color ||= car.bodyPartColor.copy();
                                //part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                            }
                        } else {
                            part.color = car.bodyPartColor.copy();
                            part.parentSlot.selectedColor = car.bodyPartColor.copy();
                        }
                    } else {
                        part.color = PartColor.noColor();
                        part.parentSlot.selectedColor = PartColor.noColor();
                    }
                    ErrorTracker.setLineLabel("#34");
                    part = part.parentSlot.savePartLinked;

                    if (part) {
                        partMaterial = part.getPossibleMaterial(partMaterial.name);
                        if (!partMaterial) {
                            partMaterial = part.possibleMaterials[0];
                        }
                        ErrorTracker.setLineLabel("#35");
                        part.selectedMaterial = partMaterial;
                        if (part.selectedMaterial.name != "metal" && part.selectedMaterial.name != "white") {

                            //if(partSlot.name=="mirors"){

                            if (part.color) {
                                if (part.color && part.color.value == 0xffffff) {

                                    part.color = car.bodyPartColor.copy();
                                    part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                                } else {
                                    part.color ||= car.bodyPartColor.copy();
                                    part.parentSlot.selectedColor ||= part.color;
                                    part.parentSlot.selectedColor.value = part.color.value;
                                }

                                ErrorTracker.setLineLabel("#36");
                            } else {
                                part.color = car.bodyPartColor.copy();
                                part.parentSlot.selectedColor = car.bodyPartColor.copy();
                            }
                        } else {
                            ErrorTracker.setLineLabel("#37");
                            part.color = PartColor.noColor();
                            part.parentSlot.selectedColor = PartColor.noColor();
                        }
                    }
                }
            } else {
                ErrorTracker.setLineLabel("#38");
                /*
                 Если детель не выбрана то установим для всех возможных деталей выбранный материал если он есть для детали
                 */
                if(partSlot && partSlot.possibleParts && selectedMaterial) {
                    for each(part in partSlot.possibleParts) {
                        partMaterial2 = part.getPossibleMaterial(selectedMaterial.name);
                        if (partMaterial2) {
                            part.selectedMaterial = partMaterial2;
                            if (part.selectedMaterial.name != "metal" && part.selectedMaterial.name != "white") {
                                if (part.parentSlot && part.parentSlot.selectedColor) {
                                    if (part.parentSlot.selectedColor.value == 0xffffff && car.bodyPartColor) {
                                        part.color = car.bodyPartColor.copy();
                                        part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                                    }
                                } else {
                                    part.color ||= car.bodyPartColor.copy();
                                    part.parentSlot.selectedColor ||= car.bodyPartColor.copy();
                                }
                            } else {
                                part.color = PartColor.noColor();
                                part.parentSlot.selectedColor = PartColor.noColor();
                            }
                        }
                    }
                }
                ErrorTracker.setLineLabel("#39");
            }

            ErrorTracker.setLineLabel("#40");
            if (!selectedMaterial.isPaintable) {
                ErrorTracker.setLineLabel("#41");
                partSlot.previousColor ||= partSlot.selectedColor;
                partSlot.selectedColor = PartColor.noColor();

                if (partSlot.name == "rear_lights") {
                    partSlot.selectedColor.value = 0xffffff;
                }
            } else if (partSlot.name == "rear_lights" && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && partSlot.selectedPart.selectedMaterial.name == "metal") {
                ErrorTracker.setLineLabel("#42");
                partSlot.previousColor = partSlot.selectedColor;
                partSlot.selectedColor = PartColor.noColor();

                if (partSlot.name == "rear_lights") {
                    partSlot.selectedColor.value = 0xffffff;
                }
            } else if (partSlot.previousColor) {
                ErrorTracker.setLineLabel("#43");
                partSlot.selectedColor = partSlot.previousColor;
            }
            ErrorTracker.setLineLabel("#44");
            if (Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0 && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && partSlot.selectedPart.selectedMaterial.name != "metal" && partSlot.selectedColor.value == 0xFFFFFF) {
                partSlot.selectedColor.value = 0xEEEEEE;
            }

            ErrorTracker.setLineLabel("#45");
            if (materialSelectSharedIndex == neededData.length - 1) {
                if(this.tuningScreen.carView.visible) {
                    this.tuningScreen.carView.beforeLoading();
                }
                if (selectedColor) {
                    updatePartSlotsWithColor(car.possiblePartSlots, selectedColor, true);
                }
                ErrorTracker.setLineLabel("#46");
                tuningScreen.carView.resetFastModeCurrentLoaded();
                if(updateWithPartSlots){
                    tuningScreen.carView.updateWithPartSlots(car.possiblePartSlots, false);
                }
                modifiedTuning = true;
            }
        }
        ErrorTracker.setLineLabel(null);
        ErrorTracker.setReference(null);
    }

    private function setMaterialForPartSlots(slots:Array, materialName:String):void {

        //Смена всех слотов

        //var selectedMaterial : PartMaterial = event.data.selectedMaterial;
        //var car.getPossiblePartSlot("ALL");
        var allPartSlot:PartSlot = car.editablePartSlots[0];

        var selectedMaterial:PartMaterial = allPartSlot.selectedPart.getPossibleMaterial(materialName);

        allPartSlot.selectedPart.selectedMaterial = selectedMaterial;

        tuningScreen.selectedPartSlot = allPartSlot;

        //tuningScreen.selectedPartSlot.selectedPart.selectedMaterial = selectedMaterial;
        var part:Part;
        for each(var partSlot:PartSlot in car.possiblePartSlots)//car.selectedPartSlots
        {
            if (partSlot.selectedPart)
            {
                var partMaterial:PartMaterial = partSlot.selectedPart.getPossibleMaterial(selectedMaterial.name);

                if (partMaterial)
                {
                    partSlot.selectedPart.selectedMaterial = partMaterial;

                    part = partSlot.selectedPart;

                    if (part.selectedMaterial.isPaintable)
                    {
                        if (part.color)
                        {
                            if (part.color && part.color.value == 0xffffff) {
                                part.color = car.bodyPartColor.copy();
                                part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                            } else {
                                part.color = car.bodyPartColor.copy();
                                part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                            }
                        } else {
                            part.color = car.bodyPartColor.copy();
                            part.parentSlot.selectedColor = car.bodyPartColor.copy();
                        }
                    } else {
                        part.color = PartColor.noColor();
                        part.parentSlot.selectedColor = PartColor.noColor();
                    }
                }
            }
            else
            {
                //Если детель не выбрана то установим для всех возможных деталей выбранный материал если он есть для детали
                for each(part in partSlot.possibleParts)//car.selectedPartSlots
                {
                    var partMaterial2:PartMaterial = part.getPossibleMaterial(selectedMaterial.name);

                    if (partMaterial2)
                    {
                        part.selectedMaterial = partMaterial2;
                        if (part.selectedMaterial.isPaintable)
                        {
                            if (part.parentSlot.selectedColor)
                            {
                                if (part.parentSlot.selectedColor.value == 0xffffff) {
                                    part.color = car.bodyPartColor.copy();
                                    part.parentSlot.selectedColor.value = car.bodyPartColor.value;
                                }
                            } else {
                                part.color ||= car.bodyPartColor.copy();
                                part.parentSlot.selectedColor = car.bodyPartColor.copy();
                            }
                        } else {
                            part.color = PartColor.noColor();
                            part.parentSlot.selectedColor = PartColor.noColor();
                        }
                    }
                }
            }

            if (!selectedMaterial.isPaintable)
            {
                partSlot.previousColor = partSlot.selectedColor;
                partSlot.selectedColor = PartColor.noColor();

                if (partSlot.name == "rear_lights") {
                    partSlot.selectedColor.value = 0xffffff;
                }
            } else if (partSlot.name == "rear_lights" && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial && partSlot.selectedPart.selectedMaterial.name == "metal") {

                partSlot.previousColor = partSlot.selectedColor;
                partSlot.selectedColor = PartColor.noColor();

                if (partSlot.name == "rear_lights") {
                    partSlot.selectedColor.value = 0xffffff;
                }
            }
            else if (partSlot.previousColor) {
                partSlot.selectedColor = partSlot.previousColor;
            }

            if (partSlot.name == "disks" &&
                    partSlot.selectedPart.selectedMaterial.name != "metal" &&
                    partSlot.selectedColor.value == 0xFFFFFF) {
                partSlot.selectedColor.value = 0xEEEEEE;
            }
        }
    }

    private function tuningScreenFrontOffsetChanged(event:starling.events.Event):void {


        var offset:Number = event.data.offset;

        car.frontOffset = offset;

        modifiedTuning = true;

    }

    private function tuningScreenSuspensionChanged(event:starling.events.Event):void {

        car.suspension = event.data.suspension;

        tuningScreen.carView.updateSuspension();

        modifiedTuning = true;
    }

    private function tuningScreenPhotoButtonSelectedHandler(event:starling.events.Event):void {
        if ((Device.isIos() && CameraRoll.supportsAddBitmapData) || (!Device.isIos() && CameraRollExtended.isSupported)) {
            var began:Boolean = true;
            var closed:Boolean = false;

            var check:Function = function ():void {
                if (began && closed) {
                    tuningScreen.touchable = false;
                    takeScreenShotAndSaveToGallery();
                }
            }

            tuningScreen.closeMenu(true, function ():void {
                closed = true;
                check();
            });
        }else{
            AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                tuningScreen.stopCamera();
                tuningScreen.enableButtons(true);
            }, true);
        }
    }

    private function takeScreenShotAndSaveToGallery():void {
        if(!Device.isIos()){
            takeScreenShotAndSaveToGalleryAndroid();
            return;
        }
        var cameraRoll:CameraRoll = new CameraRoll();
        if (CameraRoll.supportsAddBitmapData) {
            //
            var bitmapData:BitmapData = createCarViewScreenshot();
            tuningScreen.touchable = false;
            cameraRoll.addEventListener(flash.events.Event.COMPLETE, function (event:flash.events.Event):void {
                tuningScreen.touchable = true;
                Starling.juggler.delayCall(function ():void {
                    AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", (/*Device.isIos() ? "savedToCameraRoll" : */"savedToGallery")));
                }, 0.7);
            });

            cameraRoll.addEventListener(ErrorEvent.ERROR, function (event:ErrorEvent):void {
                tuningScreen.touchable = true;
                tuningScreenCancelCameraSelectedHandler(null);
                AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                    tuningScreen.enableButtons(true);
                }, true);
            });
            var imageWatermarker:ImageWatermarker = new ImageWatermarker();
            imageWatermarker.addWatermark(bitmapData, logoBitmap, tuningScreen.assetManager.scaleFactor / 8);

            if(CameraRoll.permissionStatus == PermissionStatus.GRANTED){
                cameraRoll.addBitmapData(bitmapData);
            }else{
                cameraRoll.addEventListener(PermissionEvent.PERMISSION_STATUS, function(permissionEvent:PermissionEvent):void{
                    if(CameraRoll.permissionStatus == PermissionStatus.GRANTED){
                        cameraRoll.addBitmapData(bitmapData);
                    }else{
                        AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                            tuningScreen.stopCamera();
                            tuningScreen.enableButtons(true);
                        }, true);
                    }
                });
                cameraRoll.requestPermission();
            }
        } else {
            AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                tuningScreen.stopCamera();
                tuningScreen.enableButtons(true);
            }, true);
        }
    }

    private function takeScreenShotAndSaveToGalleryAndroid():void {
        if (CameraRollExtended.isSupported) {
            switch (CameraRollExtended.service.authorisationStatus()){
                case AuthorisationStatus.SHOULD_EXPLAIN:
                case AuthorisationStatus.NOT_DETERMINED:
                    CameraRollExtended.service.addEventListener(AuthorisationEvent.CHANGED, authorisationChangedHandler);
                    CameraRollExtended.service.requestAccess();
                    return;
                case AuthorisationStatus.DENIED:
                case AuthorisationStatus.UNKNOWN:
                case AuthorisationStatus.RESTRICTED:
                    if (CameraRollExtended.service.canOpenDeviceSettings) {
                        CameraRollExtended.service.openDeviceSettings();
                    }else{
                        AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                            tuningScreen.enableButtons(true);
                        }, true);
                    }
                    return;
                case AuthorisationStatus.AUTHORISED:
                    saveScreenshot();
                    break;
            }
        } else {
            AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                tuningScreen.enableButtons(true);
            }, true);
        }
    }

    private function createCarViewScreenshot():BitmapData{
        var visibleObjects:Array = [];
        var obj:DisplayObject;
        var i:int;
        for(i = 0; i < tuningScreen.numChildren; i++){
            obj = tuningScreen.getChildAt(i);
            if(obj.visible && !(obj is CarView)){
                obj.visible = false;
                visibleObjects.push(obj);
            }
        }
        //
        var bitmapData:BitmapData = Starling.current.stage.drawToBitmapData();
        for(i = 0; i < visibleObjects.length; i++){
            obj = visibleObjects[i];
            obj.visible = true;
            if(obj.hasOwnProperty("isEnabled")){
                obj["isEnabled"] = true;
            }
        }
        return bitmapData;
    }

    private function saveScreenshot():void{
        var bitmapData:BitmapData = createCarViewScreenshot();
        var imageWatermark:ImageWatermarker = new ImageWatermarker();
        imageWatermark.addWatermark(bitmapData, logoBitmap, tuningScreen.assetManager.scaleFactor / 8);
        tuningScreen.touchable = false;
        //
        var byteArray:ByteArray = JPEGEncoder.encode(bitmapData, 80);
        var fileName:String = "tuning3d_screenshot_" + new Date().getTime().toString()+".jpg";
        var tempFile:File = File.applicationStorageDirectory.resolvePath(fileName);
        var imageFile:File = new File(tempFile.nativePath);
        var fileStream:FileStream = new FileStream();
        fileStream.open(imageFile, FileMode.WRITE);
        fileStream.writeBytes(byteArray, 0, byteArray.length);
        fileStream.close();
        if(!CameraRollExtended.service.hasEventListener(CameraRollExtendedEvent.ADD_FILE_COMPLETE)){
            CameraRollExtended.service.addEventListener(CameraRollExtendedEvent.ADD_FILE_COMPLETE, function (event:CameraRollExtendedEvent):void {
                tuningScreen.touchable = true;
                Starling.juggler.delayCall(function ():void {
                    AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", (Device.isIos() ? "savedToCameraRoll" : "savedToGallery")));
                }, 0.7);
            });
        }
        if(!CameraRollExtended.service.hasEventListener(CameraRollExtendedEvent.ADD_FILE_ERROR)) {
            CameraRollExtended.service.addEventListener(CameraRollExtendedEvent.ADD_FILE_ERROR, function (event:CameraRollExtendedEvent):void {
                tuningScreen.touchable = true;
                tuningScreenCancelCameraSelectedHandler(null);
                AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                    tuningScreen.enableButtons(true);
                }, true);
            });
        }
        var success:Boolean = CameraRollExtended.service.addFile(imageFile, Asset.IMAGE);
        if(success){
            tuningScreen.touchable = true;
            trace(imageFile.url);
        }
    }



    private function authorisationChangedHandler( event:AuthorisationEvent ):void {
        CameraRollExtended.service.removeEventListener(AuthorisationEvent.CHANGED, authorisationChangedHandler);
        switch (event.status) {
            case AuthorisationStatus.SHOULD_EXPLAIN:
                AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                    tuningScreen.enableButtons(true);
                }, true);
                break;
            case AuthorisationStatus.AUTHORISED:
                saveScreenshot();
                break;
            case AuthorisationStatus.RESTRICTED:
            case AuthorisationStatus.DENIED:
                AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                    tuningScreen.enableButtons(true);
                }, true);
                break;
        }
    }

    private function tuningScreenColorPickerSelectedHandler(event:starling.events.Event):void {
        var assetsProxy :AssetsProxy = facade.retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
        assetsProxy.loadCustomColorAssets(function():void{

            tuningColorPicker = new TuningColorPicker();
            tuningColorPicker.addEventListener(FeathersEventType.INITIALIZE, function (event:starling.events.Event):void {
                tuningColorPicker.updateWithLanguage(currentLanguage);
            });

            tuningColorPicker.addEventListener(TuningColorPicker.DISMISSED, tuningColorPickerDismissedHandler);
            if(tuningScreen.allPartsColorsView){
                tuningColorPicker.partSlot = tuningScreen.allPartsColorsView.selectedPartSlot;
            }
            tuningColorPicker.assetManager = tuningScreen.assetManager;
            tuningColorPicker.showIn(tuningScreen);
            tuningScreen.handleTouchable = false;
        }, true);
    }

    private function tuningColorPickerColorConfiguredHandler(event:starling.events.Event):void {
        if (!tuningScreen.selectedPartSlot.dummy) {
            updatePartSlotsWithColor([tuningScreen.selectedPartSlot], event.data.selectedColor);
            tuningScreen.updateSelectedColorForPartSlot(tuningScreen.selectedPartSlot);
        }
        else {
            updatePartSlotsWithColor(car.selectedPartSlots, event.data.selectedColor);
        }

        tuningColorPicker = null;
    }

    private function tuningColorPickerDismissedHandler(event:starling.events.Event):void {
        tuningScreen.handleTouchable = true;
        tuningColorPicker = null;
    }

    private function tuningScreenSaveTuningSelectedHandler(event:starling.events.Event):void {
        var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.checkUserData(userDataSuccessHandler);
    }

    private function userDataSuccessHandler(success:Boolean):void{
        pendingForAuthAction = AUTH_ACTION_SAVE;
        ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
        restoreAdsBanner = AdsManager.$.bannerHide();
        if(success){
            sendNotification(Notifications.AUTHENTICATE_USER);
        }else{
            var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.logoutCurrentUser();
            sendNotification(Notifications.AUTHENTICATE_USER);
        }
    }

    private function tuningScreenDownloadsButtonSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.DISPLAY_DOWNLOADS_POP_UP, {car: tuningScreen.car});
    }

    private function tuningScreenSettingsSelectedHandler(event:starling.events.Event):void {
        sendNotification(Notifications.DISPLAY_TUNING_SETTINGS_POP_UP, {car: car});
    }

    private function handleSnapShotFromCameraCreated(event:starling.events.Event):void {
        tuningScreen.bitmapdataFromCamera = event.data as BitmapData;
        upload_bitmapdataFromCameraIfNeed();
    }

    private function handleCarBackgroundSelected(background : Object) : void
    {
        var carBackground:CarBackground = background.carBackground;

        var color:Object = background.color;


        car.selectedBackground = carBackground;
        car.selectedFillColor = color;
        tuningScreen.carView.updateBackground();
    }

    private function tuningScreenPartUnselectedHandler(event:starling.events.Event):void {
        var partSlot : PartSlot = tuningScreen.selectedPartSlot;
        var arrOfPartSlotForUpdate:Array = [];

        if (!partSlot.isMandatory) {


            this.tuningScreen.carView.beforeLoading();

            if (partSlot.selectedPart) {

                var antimaskPartMaterial:PartMaterial = partSlot.selectedPart.getPossibleMaterial("antimask");

                if (antimaskPartMaterial) {

                    tuningScreen.carView.removeImagesOfPartSlotName("antimask_" + partSlot.selectedPart.id, -1);

                }
            }

            var linkedPartToUnselect : Part;

            if(partSlot.selectedPart && partSlot.selectedPart.linkedParts && partSlot.selectedPart.linkedParts.length && ((partSlot.selectedPart.properties && partSlot.selectedPart.properties['autoRemoveLinked']) || partSlot.name == PartSlot.VINYL_PACK || partSlot.name == PartSlot.TWO_TONE_VINYL_PACK))
            {
                for each (linkedPartToUnselect in partSlot.selectedPart.linkedParts)
                {
                    if(Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0 && (Car.BRAKES_SLOTS.indexOf(linkedPartToUnselect.parentSlot.name) >= 0 || Car.TIRES_SLOTS.indexOf(linkedPartToUnselect.parentSlot.name) >= 0 || Car.TIRE_VINYL_PARTS.indexOf(linkedPartToUnselect.parentSlot.name) >= 0))
                    { // if parent of linked is disk, so we need to ignore tires
                        continue;
                    }

                    if(linkedPartToUnselect.parentSlot.selectedPart &&
                            linkedPartToUnselect.parentSlot.selectedPart.id == linkedPartToUnselect.id) {
                        tuningScreen.carView.removeMaskImages(linkedPartToUnselect);

                        linkedPartToUnselect.parentSlot.selectedPart = null;
                        linkedPartToUnselect.parentSlot.savePartLinked = null;
                        arrOfPartSlotForUpdate = removePartSlot(null, linkedPartToUnselect.parentSlot).concat(arrOfPartSlotForUpdate);
                        //tuningScreen.carView.removePartSlot(linkedPartToUnselect.parentSlot);
                    }
                }


            }

            arrOfPartSlotForUpdate = removePartSlot(null, tuningScreen.selectedPartSlot).concat(arrOfPartSlotForUpdate);
            //tuningScreen.carView.removePartSlot(tuningScreen.selectedPartSlot);


            var isPack:Boolean = (tuningScreen.selectedPartSlot.name == PartSlot.BODY_PACK || tuningScreen.selectedPartSlot.name == PartSlot.FACTORY_TRIM);
            if (isPack && tuningScreen.selectedPartSlot.selectedPart &&
                    tuningScreen.selectedPartSlot.selectedPart.excludedPartSlotsNames &&
                    tuningScreen.selectedPartSlot.selectedPart.excludedPartSlotsNames.indexOf("disks") >= 0) {
                this.SuspensionControlEnabledFlag = true;
                tuningScreen.setSuspensionControlEnabled(true);
            }

            if (tuningScreen.carView.partSlotMask) {
                if (tuningScreen.selectedPartSlot.selectedPart) {
                    if (tuningScreen.selectedPartSlot.selectedPart.affectedSlots && tuningScreen.selectedPartSlot.selectedPart.affectedSlots.length > 0) {
                        tuningScreen.carView.removeImagesOfPartSlotName("masks_" + tuningScreen.selectedPartSlot.selectedPart.id, -1);
                    }
                }
            }
            var allCurrentPartSlots: Array = car.possiblePartSlots;
            for (var i: int = 0; i < allCurrentPartSlots.length; i++) {
                var partSlot2: PartSlot = allCurrentPartSlots[i] as PartSlot;
                if (tuningScreen.selectedPartSlot && tuningScreen.selectedPartSlot.selectedPart && partSlot2.selectedPart && partSlot2.selectedPart.properties && partSlot2.selectedPart.properties.dependence) {
                    if (partSlot2.selectedPart && partSlot2.selectedPart.properties && partSlot2.selectedPart.properties.dependence) {
                        if (partSlot2.selectedPart.properties.dependence.indexOf(tuningScreen.selectedPartSlot.selectedPart.id) >= 0) {
                            arrOfPartSlotForUpdate = removePartSlot(null, partSlot2, false).concat(arrOfPartSlotForUpdate);
                            //tuningScreen.carView.removePartSlot(partSlot2, false);
                            partSlot2.selectedPart = null;
                        }
                    }
                }

            }


            tuningScreen.selectedPartSlot.selectedPart = null;
            tuningScreen.carView.filteringSuspension();


            if (tuningScreen.selectedPartSlot.name == "frame") {
                var frameBackPartSlot:PartSlot = updateFrameBackIfNeeded();

            }

            modifiedTuning = true;
            //
            // ***********************************************************
            // var isPack:Boolean = (tuningScreen.selectedPartSlot.name == PartSlot.BODY_PACK || tuningScreen.selectedPartSlot.name == PartSlot.FACTORY_TRIM || tuningScreen.selectedPartSlot.name == PartSlot.TWO_TONE_VINYL_PACK || tuningScreen.selectedPartSlot.name == PartSlot.VINYL_PACK);
            if(!isPack || arrOfPartSlotForUpdate.length){
                var carProxy:CarsProxy =  carsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;
                carProxy.loadComponent(arrOfPartSlotForUpdate, function(components:Array):void{
                    if(event &&  event.data && event.data.loadCompleteHandler){
                        if(event.data.hasOwnProperty("updateWithPartSlots")){
                            if(event.data.updateWithPartSlots){
                                tuningScreen.carView.updateWithPartSlots(arrOfPartSlotForUpdate, false, event.data.loadCompleteHandler);
                            }else{
                                event.data.loadCompleteHandler();
                            }
                        }else{
                            tuningScreen.carView.updateWithPartSlots(arrOfPartSlotForUpdate, false, event.data.loadCompleteHandler);
                        }
                    }else{
                        tuningScreen.carView.updateWithPartSlots(arrOfPartSlotForUpdate, false);
                    }
                });
            }
        }
    }

    protected override function stackedScreenBackButtonSelectedHandler(event:starling.events.Event):void {
        //var superFunc:Function = super.stackedScreenBackButtonSelectedHandler;

        var defaultImpl:Function = function ():void {
            var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            tuningsProxy.deletePendingTuning();

            //AdsButton.activeRemove();

            sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            //superFunc(event);
        }

        if (inFullscreen) {
            tuningScreen.endScreenShot(colorPickerViews, 0, false);
            tuningScreen.carView.canTouchParts = true;
            //tuningScreen.carView.curate();
            inFullscreen = false;
            if (tuningScreen.selectedPartSlot.name != PartSlot.BODY_PACK && tuningScreen.selectedPartSlot.name != PartSlot.FACTORY_TRIM) {
                tuningScreen.setSuspensionControlEnabled(true);
            }
        }
        else if (modifiedTuning) {

            AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "closeWithoutSaving"), -1, false, function (yesSelected:Boolean):void {
                if (yesSelected) {

                    defaultImpl();
                }
            });

        }
        else {
            defaultImpl();
        }
    }

    private function tuningScreenFullscreenSelectedHandler(event:starling.events.Event):void {
        tuningScreen.beginScreenShot(colorPickerViews, null, true, false);
        tuningScreen.carView.canTouchParts = false;
        tuningScreen.closeMenu(true);
        inFullscreen = true;
    }

    private function handleUserAuthenticated(wasAlreadyAuthenticated:Boolean):void {
        trace("TuningScreenMediator handleUserAuthenticated 1");
        if (!stackedScreen.stage) {
            return;
        }

        var userProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(tuningScreen.tunning && !tuningScreen.tunning.tuningUserID && tuningScreen.tunning.userObj){
            tuningScreen.tunning.tuningUserID = tuningScreen.tunning.userObj._id;
        }
        if (tuningScreen.tunning && tuningScreen.tunning.tuningUserID != userProxy.currentUser.id) {
            tuningScreen.tunning = null;
        }
        trace("TuningScreenMediator handleUserAuthenticated 2");
        ImageLoaderWithStates.disposeWhenRemovedFromStage = true;
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if (pendingForAuthAction == AUTH_ACTION_SAVE) {
            if(car.isChallenge){
                if(usersProxy.isUserDataValid()){
                    var submitAlert:ChallengeAlertView = ChallengeAlertView.show(ChallengeAlertView.SUBMIT_CHALLENGE, tuningScreen.challengeData.type, tuningScreen.stage);
                    submitAlert.addEventListener(ChallengeAlertEvent.SUBMIT_TRIGGERED, function(event:ChallengeAlertEvent):void{
                        sendNotification(Notifications.SUBMIT_CHALLENGE);
                    });
                }else{
                    usersProxy.extendSession();
                }
            }else{
                sendNotification(Notifications.DISPLAY_TUNNING_NAME_INPUT_POP_UP, tuningScreen.tunning);
                this.tuningScreen.mainMenuView.updateStates();
                refreshSettings();
            }
        }else if (pendingForAuthAction == AUTH_ACTION_BUY_PACKET) {
            if (tuningScreen){
                tuningScreen.showBuyPacket(pendingPacket);
            }
            initPurchase(pendingPacket);
            pendingPacket = null;
        }else if(pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_IN_APP_NOTIFICATIONS) {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        } else if(pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_MAILS){
            sendNotification(Notifications.DISPLAY_MAIL_SCREEN);
        }else if (pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_FRIENDS){
            sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN);
        }else if (pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_TIMELINE){
            Starling.juggler.delayCall(function ():void {
                sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
            }, 2);
        }else if (pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_GARAGE){
            Starling.juggler.delayCall(function ():void {
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
            }, _garage_delay);
        }else if (pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_TUNING){
            Starling.juggler.delayCall(function ():void {
                tuningScreen.mainMenuView.updateStates();
                refreshSettings();
            }, 0);
        }else if(pendingForAuthAction == AUTH_ACTION_UPLOAD_BACKGROUND) {
            if(!wasAlreadyAuthenticated){
                refreshSettings();
            }
            browseForImageUpload();
        }else if(pendingForAuthAction == AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA) {
            tuningSettingsSnapshotCameraSelectedHandler(null);
        }else if(pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_NO_COIN_POPUP){
            sendNotification(Notifications.DISPLAY_REWARD_NOCOIN_POPUP, pendingForAuthData);
        }

        usersProxy.getPurchases();
        pendingForAuthAction = null;
    }

    private function browseForImageUpload():void {
        tuningScreen.backgroundMenuView.browseForUpload();
    }

    private function tuningSettingsSnapshotCameraSelectedHandler(event : starling.events.Event) : void {
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

        if(!usersProxy.currentUser)
        {
            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

            pendingForAuthAction = AUTH_ACTION_UPLOAD_PHOTO_FROM_CAMERA;

            sendNotification(Notifications.AUTHENTICATE_USER);

            return;
        }

        sendNotification(Notifications.CAMERA_SNAPSHOT);
    }

    private function scaleRectangle(rectangle : Rectangle, factor : Number) :void
    {
        rectangle.x *= factor;
        rectangle.y *= factor;
        rectangle.width *= factor;
        rectangle.height *= factor;
    }


    //private var mediaSource:CameraRoll;
    private function browseForImage_iOS() : void {

    }

    private var imageLoader2:Loader;
    private function imageSelected( event:MediaEvent ):void {
        log("Image selected...");

        var imagePromise:MediaPromise = event.data;

        imageLoader2 = new Loader();
        if( imagePromise.isAsync ) {
            log("Asynchronous media promise.");
            imageLoader2.contentLoaderInfo.addEventListener(flash.events.Event.COMPLETE, imageLoaded);
            imageLoader2.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, imageLoadFailed);
            imageLoader2.loadFilePromise(imagePromise);
        } else {
            log("Synchronous media promise.");
            imageLoader.loadFilePromise(imagePromise);
            //this.addChild( imageLoader2 );
        }
    }

    private function browseCanceled( event:flash.events.Event ):void {
        log("Image browse canceled.");
    }

    private function imageLoaded( event:flash.events.Event ):void {
        log("Image loaded asynchronously.");
        //this.addChild( imageLoader );
    }

    private function imageLoadFailed( event:IOErrorEvent ):void {
        log("Image load failed.");
    }

    private function log( text:String ):void {

    }


    private function browseForImage_Android() : void {
        if(CameraRoll.supportsBrowseForImage ) {
            cameraRoll = new CameraRoll();
            cameraRoll.addEventListener(MediaEvent.SELECT, cameraRollSelectHandler);
            cameraRoll.browseForImage();
        } else {
            AlertView.show("Browsing in camera roll is not supported.", -1, false);
        }
    }

    private function cameraRollSelectHandler(event : MediaEvent) : void {
        mediaPromise = event.data;

        exifLoader = new ExifLoader();
        exifLoader.addEventListener(flash.events.Event.COMPLETE, exifLoaderCompleteHandler);
        exifLoader.load(new URLRequest(mediaPromise.file.url));
    }

    private function exifLoaderCompleteHandler(event : flash.events.Event) : void {
        if(imageLoader) {
            imageLoader.unload();
        }
        imageLoader = new Loader();
        imageLoader.contentLoaderInfo.addEventListener(flash.events.Event.COMPLETE, imageLoaderCompleteHandler);
        imageLoader.loadFilePromise(mediaPromise);
    }


    private function imageLoaderCompleteHandler(event : flash.events.Event) : void {
        var exif:ExifInfo = this.exifLoader.exif;
        var rotation:int = ExifUtils.getEyeOrientedAngle(exif.ifds);
        var bitmap:Bitmap = ExifUtils.getEyeOrientedBitmap(Bitmap(event.currentTarget.content), exif.ifds);
        processAndUploadBitmapData(bitmap.bitmapData);
    }

    private function processAndUploadBitmapData(bitmapData : BitmapData) : void {
        if (bitmapData.width >= Constants.MIN_UPLOAD_IMAGE_WIDTH && bitmapData.height >= Constants.MIN_UPLOAD_IMAGE_HEIGHT) {
            uploadBitmapData(bitmapData);
        } else {
            AlertView.show(AlertView.language.translate("mob_saving", "imageSizeValidation") + " " + Constants.MIN_UPLOAD_IMAGE_WIDTH + "x" + Constants.MIN_UPLOAD_IMAGE_HEIGHT + ".", -1, false);
        }
    }

    private function uploadBitmapData(bitmapData : BitmapData) : void {
        if (alertView) {
            alertView.hide();
        }
        alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "uploading"), 0);
        var processedBitmapData:BitmapData = processedBitmapDataForUpload(bitmapData);
        bitmapData.dispose();
        tuningScreen.backgroundMenuView.enterUploadingState();
        var carBackgroundsProxy:CarBackgroundsProxy = facade.retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
        carBackgroundsProxy.upload(processedBitmapData);
    }

    private function processedBitmapDataForUpload(bitmapData : BitmapData) : BitmapData {
        HttpClient.saveTestFile("bitmap1.png", bitmapData);
        var bitmap:Bitmap = DisplayUtils.createThumb(bitmapData, Constants.MIN_UPLOAD_IMAGE_WIDTH, Constants.MIN_UPLOAD_IMAGE_HEIGHT);
        HttpClient.saveTestFile("bitmap2.png", bitmap.bitmapData);

        return bitmap.bitmapData;
    }

    private function refreshSettings() : void {
        if(tuningScreen == null) {
            return;
        }
        if (alertView) {
            alertView.hide();
        }
        var settingsProxy:SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var languagesProxy:LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        var settings:Settings = settingsProxy.currentSettings;

        if(!settings.selectedLanguageCode){
            settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
        }

        tuningScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        if (!tuningScreen.selectedLanguage) {
            settings.selectedLanguageCode = "en-US";
            tuningScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        }

        tuningScreen.languages = languagesProxy.languages;

        if (tuningScreen.languages != null && tuningScreen.languages.length <= 0) {
            languagesProxy.loadLanguages();
        } else if (tuningScreen.languages == null) {
            languagesProxy.loadLanguages();
        }
    }

    private function upload_bitmapdataFromCameraIfNeed() : void {
        if (tuningScreen.bitmapdataFromCamera) {
            //увеличем размер картинки до минимальных размеров если необходимо
            uploadBitmapData(tuningScreen.bitmapdataFromCamera);
            tuningScreen.bitmapdataFromCamera.dispose();
            tuningScreen.bitmapdataFromCamera = null;

        }
    }

    /*private function upload_bitmapdataFromCameraIfNeed() : void
     {

     if(tuningScreen.bitmapdataFromCamera){

     //увеличем размер картинки до минимальных размеров если необходимо

     uploadBitmapData(tuningSettingsPopUp.bitmapdataFromCamera);
     tuningSettingsPopUp.bitmapdataFromCamera.dispose();
     tuningSettingsPopUp.bitmapdataFromCamera = null;

     }
     }*/

    private function handleSelectedSocialForSharing(socialId:String):void {


        if (socialId == "fb") {

            tuningScreenShareFbSelectedHandler(null);
        } else if (socialId == "tw") {

            sendNotification(Notifications.DISPLAY_TUNING_SHARING_POP_UP, {title: "Sharing", challenge: false});

        } else if (socialId == "vk") {

            tuningScreenShareVkSelectedHandler(null);

        } else if (socialId == "instagram") {

            tuningScreenSharInstagramSelectedHandler(null);
        }

    }

    private function tuningScreenSharePinSelectedHandler(event : starling.events.Event=null) : void
    {
        shareToInstaragm = false;

        var sharingPinProxy:SharingPinProxy = facade.retrieveProxy(SharingPinProxy.NAME) as SharingPinProxy;
        sharingPinProxy.prepare();
    }


    private function tuningScreenShareOkSelectedHandler(event:starling.events.Event):void {
        shareToInstaragm = false;

        var sharingOkProxy:SharingOkProxy = facade.retrieveProxy(SharingOkProxy.NAME) as SharingOkProxy;
        sharingOkProxy.prepare();
    }

    private function tuningScreenShareVkSelectedHandler(event:starling.events.Event):void {
        shareToInstaragm = false;

        var sharingVkProxy:SharingVkProxy = facade.retrieveProxy(SharingVkProxy.NAME) as SharingVkProxy;
        sharingVkProxy.prepare();
    }

    private var _isFbChallenge:Boolean = false;

    public static var SAVE_FOR_FB_SHARE : Boolean;

    private function tuningScreenShareFbSelectedHandler(event:starling.events.Event):void {

        /*  // OLD SHARING METHOD
        SharingProxy.LAST_USER_PAGE = null;

        if (event.data && event.data.hasOwnProperty("challenge")) {

            _isFbChallenge = event.data.challenge;
        }

        shareToFB = true;

        var sharingProxy:SharingProxy = facade.retrieveProxy(SharingProxy.NAME) as SharingProxy;
        sharingProxy.prepare();

        */

        /**
         * New sharing method
         * First saves tuning
         * after generates url for this tuning sharing
         * navigates to that url (via mobile browser)
         */

        SAVE_FOR_FB_SHARE = true;
        handleTuningBeginAdding("Facebook share " + car.name + " " + car.model + " " + car.year);
    }





    private function tuningScreenSharInstagramSelectedHandler(event:starling.events.Event):void {
        if (event.data && event.data.hasOwnProperty("challenge")) {

            _isFbChallenge = event.data.challenge;
        }
        shareToInstaragm = true;

        var sharingInstagramProxy:SharingInstagramProxy = facade.retrieveProxy(SharingInstagramProxy.NAME) as SharingInstagramProxy;
        sharingInstagramProxy.prepare();
    }

    private function handleSharingReady(__title:String):void {
        //tuningScreen.closeMenu(true);

        if (_isFbChallenge) {


            sendNotification(Notifications.DISPLAY_TUNING_SHARING_POP_UP, {
                title: __title,
                challenge: true,
                siteLink:car.getSiteLinkTuning(),
                hashTags: tuningScreen.car.hashTags
            });

        } else {

            sendNotification(Notifications.DISPLAY_TUNING_SHARING_POP_UP, {title: __title, siteLink:car.getSiteLinkTuning()});

        }
    }

    private function handleSharingPreparingError(error:String):void {
        AlertView.show(error, -1, false);

        _isFbChallenge = false;
    }

    private function handleTuningSharingMessageEntered(message:String):void {
        Starling.juggler.delayCall(function ():void {


            tuningScreen.beginScreenShot(colorPickerViews, function ():void {
                //tuningScreen.enableButtons(false);

                var bitmapData:BitmapData = Starling.current.stage.drawToBitmapData();

                alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "uploading"), -1);


                Starling.juggler.delayCall(function ():void {

                    if (shareToInstaragm) {
                        var sharingInstagramProxy:SharingInstagramProxy = facade.retrieveProxy(SharingInstagramProxy.NAME) as SharingInstagramProxy;

                        sharingInstagramProxy.sharePhoto(message, bitmapData, function ():void {

                            bitmapData.dispose();
                            tuningScreen.endScreenShot(colorPickerViews);
                        }, message);

                    } else {
                        var sharingVkProxy:SharingVkProxy = facade.retrieveProxy(SharingVkProxy.NAME) as SharingVkProxy;
                        sharingVkProxy.sharePhoto(message, bitmapData, function ():void {

                            bitmapData.dispose();
                            tuningScreen.endScreenShot(colorPickerViews);

                        });
                    }


                }, 0.4);
            });
        }, 0.4);
    }

    private function handleSharingPhotoUploaded():void {


        if (_isFbChallenge) {
            Starling.juggler.delayCall(function ():void {
                alertView.hide();

                tuningScreen.enableButtons(true);


                sendNotification(Notifications.TUNING_BEGIN_ADDING, "4a786c664bac8afc4f1c48f30750942e2fa31e7bd92067522cac3c83a3d7d011");

                _isFbChallenge = false;

            }, 1);


        } else {

            alertView.hide();
            tuningScreen.enableButtons(true);

            AlertView.show(AlertView.language.translate("mob_saving", "done"));

        }
    }

    private function handleSharingPhotoUploadingError():void {
        alertView.hide();
        tuningScreen.enableButtons(true);

        _isFbChallenge = false;

        AlertView.show(AlertView.language.translate("mob_saving", "uploadingFailed"));
    }

    private function handleApplicationWentBackground():void {
        if (car && !tuningScreen.tunning) {
            var tuning:Tuning = new Tuning();
            tuning.car = car;

            var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            tuningsProxy.savePendingTuning(tuning);
        }
    }

    private function handleCreateCameraSnapshot(paramObject:Object):void {
        var began:Boolean = false;
        var closed:Boolean = false;
        var check:Function = function ():void {
            if (began && closed) {
                var allowed:Boolean = tuningScreen.startCamera();
                if(!allowed){
                    AlertView.show(currentLanguage.translate("mob_alertsAndOverlays", "no_access_photo"), -1, false, function(val:Boolean = true):void{
                        tuningScreen.stopCamera();
                        tuningScreen.enableButtons(true);
                        tuningScreen.endScreenShot();
                    }, true);
                }
            }
        }

        tuningScreen.beginScreenShot(colorPickerViews, function ():void {
            began = true;
            check();
        }, false,true,true);

        tuningScreen.closeMenu(true, function ():void {
            closed = true;
            check();
        });

    }

    private function tuningScreenAcceptCameraSelectedHandler(event:starling.events.Event):void {
        var bitmapData:BitmapData = tuningScreen.stopCamera(true);
        if(bitmapData){
            tuningScreen.bitmapdataFromCamera = bitmapData;
            if(tuningScreen.isChallengeARMode()){
                tuningScreen.carView.isARBackgroundSet = true;
                var resultBitmapData:BitmapData = tuningScreen.setCustomBackground(tuningScreen.bitmapdataFromCamera);
                tuningScreen.carView.backgroundContainer.ARBackgroundData = resultBitmapData.clone();
                tuningScreen.bitmapdataFromCamera.dispose();
                if(resultBitmapData != tuningScreen.bitmapdataFromCamera){
                    resultBitmapData.dispose();
                    resultBitmapData = null;
                }
                tuningScreen.bitmapdataFromCamera = null;
                tuningScreen.updateARButtons();
            }else{
                upload_bitmapdataFromCameraIfNeed();
            }
        }
        tuningScreen.endScreenShot(colorPickerViews);
    }

    private function tuningScreenUploadCustomBackgroundHandler(event:starling.events.Event):void {
        var bitmapData:BitmapData = tuningScreen.setCustomBackground(event.data.bitmapData);
        if(bitmapData){
            tuningScreen.bitmapdataFromCamera = bitmapData;
            upload_bitmapdataFromCameraIfNeed();
        }
    }

    private function tuningScreenCancelCameraSelectedHandler(event:starling.events.Event):void {
        tuningScreen.stopCamera();
        tuningScreen.endScreenShot(colorPickerViews, 0.5, true, true);
    }

    private function tuningScreenSavePromoVideoSelectedHandler(event:starling.events.Event):void {
        if(!PromoVideoManager.inProcess){
            sendNotification(Notifications.DISPLAY_TUNING_VIDEO_POPUP, {mode: TuningVideoPopup.START, videoData: event.data});
        }
    }

    // private var timingObject:Object; // TEMPORARY VAR
    private function handlerTuningVideoSave(data:Object):void{
        // timingObject = {frames:getTimer(), send:0, check: 0, download: 0, total: getTimer()};
        if(alertView){
            alertView.hide();
        }
        alertView = AlertView.show("Preparing video...", 0);
        Starling.juggler.delayCall(function():void{
            tuningScreen.getVideoData(data.button, sendVideoFrames);
        }, 0.36);
    }

        private function sendVideoFrames(videoData:Object):void{
            if(alertView){
                alertView.hide();
                alertView = null;
            }
            var tuningsProxy:TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            tuningsProxy.savePromoVideoFrames(videoData.snapshots, videoData.theme, function(success:Boolean, result:Object = null):void{
                if(success){
                    if(result is String){
                        result = JSON.parse(result as String);
                    }
                    var fileName:String = result.outputFileName;
                    var checkFileMethod:Function = function():void{
                        tuningsProxy.checkPromoVideo(fileName, function(success:Boolean, result:Object = null):void{
                            if(success){
                                Console.log("https://tuningvideomakerfiles.s3.amazonaws.com/" + fileName, result);
                                promoVideoManager = PromoVideoManager.instance;
                                promoVideoManager.addEventListener(PromoVideoManager.SAVE_COMPLETE, videoSaveCompleteHandler);
                                promoVideoManager.addEventListener(PromoVideoManager.SAVE_FAILED, videoSaveCompleteHandler);
                                promoVideoManager.downloadAndSaveVideo(fileName);
                            }else{
                                Starling.juggler.delayCall(checkFileMethod, 2);
                            }
                        });
                    };
                    Starling.juggler.delayCall(checkFileMethod, 2);
                    Console.log("Video saved successfully", "https://tuningvideomakerfiles.s3.amazonaws.com/" + result.outputFileName,JSON.stringify(result));
                }else{
                    Console.log("Failed to save video");
                    tuningScreen.removeVideoSpinner();
                    tuningScreen.enableVideoButtons(true);
                }
            });
            tuningScreen.enableVideoButtons(false);
            tuningScreen.addVideoSpinner();
        }

    private function videoSaveCompleteHandler(event:starling.events.Event):void {
        SoundManager.playSound(SoundManager.SAVE_AND_NOTIFICATION);
        promoVideoManager.removeEventListener(PromoVideoManager.SAVE_COMPLETE, videoSaveCompleteHandler);
        promoVideoManager.removeEventListener(PromoVideoManager.SAVE_FAILED, videoSaveCompleteHandler);
        if(tuningScreen){
            tuningScreen.removeVideoSpinner();
            tuningScreen.enableVideoButtons(true);
        }
        if(!tuningScreen){
            promoVideoManager = null;
        }
        sendNotification(Notifications.DISPLAY_TUNING_VIDEO_POPUP, {mode: TuningVideoPopup.COMPLETE});
    }

    private function removePartSlot(excludePartSlots:Array, partSlot:PartSlot, byExcluding:Boolean = false, callAfterLoad:Boolean = true, skipRestore:String = null):Array{
        var j:int;
        var part:Part = partSlot.selectedPart;
        var result:Array = [];
        result = tuningScreen.carView.removePartSlot(partSlot, false, byExcluding, callAfterLoad, skipRestore, excludePartSlots);
        if(part){
            if(part.removeTypesOnRemove && part.removeTypesOnRemove.length){
                for(j = 0; j < part.removeTypesOnRemove.length; j++){
                    if(car.getPossiblePartSlot(part.removeTypesOnRemove[j])) {
                        var removingPartSlot:PartSlot = car.getPossiblePartSlot(part.removeTypesOnRemove[j]);
                        result = result.concat(removePartSlot(null, removingPartSlot, false, false, null));
                    }
                }
            }

            if(part.addTypesOnRemove && part.addTypesOnRemove.length){
                for(j = 0; j < part.addTypesOnRemove.length; j++){
                    var addPart:Part;
                    if(part.addTypesOnRemove[j] is String){
                        addPart = car.findPart(part.addTypesOnRemove[j]);
                        if(addPart){
                            part.addTypesOnRemove[j] = addPart;
                        }
                    }
                    if(!(part.addTypesOnRemove[j] is Part)){
                        continue;
                    }
                    addPart = part.addTypesOnRemove[j];
                    if(!excludePartSlots || excludePartSlots.indexOf(addPart.parentSlot) == -1){
                        addPart.parentSlot.selectedPart = addPart;
                        result.push(addPart.parentSlot);
                    }
                }
            }
        }
        return result;
    }
}
}